///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      ABCACHE.H      
//
//  Description
//      Implementation of object property cache
//
//  Author
//      Les Thaler
//
//  Revision: 1.00
//
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//
#ifndef _ABCACHE_
#define _ABCACHE_
#include "abpab.h"
#include "cdbfile.h"
#include "template.h"

class CFlatPropList
{
    public:
        CFlatPropList();        
        ~CFlatPropList();

        STDMETHODIMP    Insert(const LPSPropValue);
        STDMETHODIMP    Insert(const PFlatPropValue pfpvProp);

        STDMETHODIMP    Delete(const ULONG ulPropTag,PFlatPropNode * ret);

        PFlatPropNode   WINAPI Find(const ULONG ulPropTag);
        PFlatPropNode   WINAPI Scan(BOOL bWalking = 1);
        PFlatPropNode   WINAPI Walk(BOOL bWalking = 1);
        inline ULONG    WINAPI Size() { return m_ulCnt;};
        PFlatPropNode   WINAPI Dequeue();
        VOID            WINAPI Destroy();

    private:
        PFlatPropNode   m_pHead,
                        m_pTemp;      // used by Walk
        ULONG           m_ulCnt;
        void            WINAPI InsertNode(PFlatPropNode pfpnInsertee);
};


class  CPropCache
{
    friend class CABContainer;
    public:

        CPropCache(PDATABASE pDB);
        ~CPropCache();
        STDMETHODIMP    Init();
        STDMETHODIMP    Load(RECORD_NUMBER & RecNum);
        STDMETHODIMP    Flush(RECORD_NUMBER & RecNum);
        
        PCFlatPropList  m_pProps;

    private:       
        CList<ULONG>    m_MyChildren,
                        m_MyParents,
                        m_DeletedProps;    
        PABPAB_EID      m_pMyEID;        
        PDATABASE       m_pDB;
        BOOL            m_bDirty;       
};

BOOL operator == (const SPropValue & lhs, const SPropValue &rhs);
STDMETHODIMP Flatten(const LPSPropValue pspv, PFlatPropNode * ppfsp);
STDMETHODIMP UnFlatten(PFlatPropValue pfpv, LPVOID pParent,SPropValue & spv);

STDMETHODIMP UnFlatten(const PFlatPropNode    pFlatPropNode, 
                       LPVOID                 pAllocParent,
                       SPropValue           & Prop);
#endif

// End of file for ABCACHE.H
