///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      ABPAB.H
//
//  Description
//       Declarations of classes, data structures, function prototypes, defines, etc.
//       for ABPAB
//  Author
//      Les Thaler
//
//  Revision: 1.00
//
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//

#ifndef _ABPAB_H_
#define _ABPAB_H_


#ifdef _DEBUG
    #ifndef ENABLE_DEBUG_OUTPUT
        #define ENABLE_DEBUG_OUTPUT
    #endif
#endif //  _DEBUG

#define STRICT
#include <WINDOWS.H>
#include <MAPITAGS.H>
#include <MAPIDEFS.H>
#include <MAPICODE.H>
#include <MAPISPI.H>
#include <MAPIUTIL.H>
#include <MAPIWIN.H>
#include <MAPIVAL.H>
#include <STDDEF.H>
#include <string.h>
#include "traces.h"
#include "util.h"
#include "template.h"
#include "resource.h"

#define SP      ' '
#define TAB     '\t'
#define CR      '\r'

#define ALIGNMENT           sizeof DWORDLONG
#define MAX_SEARCH_KEY      2048

// returns cnt "rounded up" to a multiple of align
#define CEIL(cnt,align)     ((cnt) % (align)                ? \
                            ((cnt) / (align) + 1) * (align) : \
                            (cnt))
// writes Tag2's property ID to Tag1
#define SWAP_IDS(Tag1,Tag2) ((Tag1) = ((Tag1) & 0xFFFF) | ((Tag2) & 0xFFFF0000))

typedef struct
{
    ULONG       Num,                // DB entry number
                Type;               // PR_OBJECT_TYPE | FOREIGN_ENTRY,e.g.
} ABPAB_ID, *PABPAB_ID;

typedef struct _ABPAB_EID
{
    BYTE        bFlags[4];
    MAPIUID     muid;
    ULONG       version;            // put struct on 8-byte boundary
    ABPAB_ID    ID;                 
} ABPAB_EID, *PABPAB_EID;

typedef struct
{
    ULONG       ulID,
                ulPad;
    MAPIUID     muid;
} *PSMAPKEY, SMapKey;

typedef enum _FIELD_ {fldPROPS,
                      fldCHILDREN,
                      fldPARENTS,
                      fldTABLEROWS,
                      fldCAPABILITIES = fldTABLEROWS,    // they both live in the same stream
                      NUM_REC_FLDS} FIELD;

typedef struct
{
    LPSTREAM    pCols[NUM_REC_FLDS];    // columns (i.e. fields) in record
    ABPAB_ID    ID;                     // this record's ID
} RECORD_NUMBER;

#define DECLARE_REC_NUM(_name_)  RECORD_NUMBER _name_ = {NULL,NULL,NULL,NULL,0}
 
typedef struct
{
    ULONG       IDNum;
    LPMAPIPROP  pObj;
} ORPHAN;

typedef struct _FlatPropHdr_
{
     ULONG       cbTotal,   // size of header + data + padding
                 cbData;    // size of data only (no padding)
     SPropValue  spv;
} FlatPropHdr, *PFlatPropHdr;

#define FPN_BYTES           8

typedef struct _FlatPropValue
{
    ULONG       cbTotal,
                cbData;
    SPropValue  spv;
    BYTE        Data[FPN_BYTES];
} FlatPropValue, *PFlatPropValue;

typedef struct _FlatPropNode_
{
    struct _FlatPropNode_ * pNext;
    ULONG                   ulPad;     // for MIPS
    FlatPropValue           fpv;
} FlatPropNode, *PFlatPropNode;

#define BASE_PROVIDER_ID    0x6600  // Look at the comments in MAPITAGS.H
#define BASE_NAMED_ID       0x8000


// Columns in our name-to-ID mapping table
#define PR_MAP_UID          PROP_TAG(PT_BINARY,  (BASE_PROVIDER_ID + 0x0001))
#define PR_MAP_ID           PROP_TAG(PT_LONG,    (BASE_PROVIDER_ID + 0x0002))
#define PR_MAP_SZNAME       PROP_TAG(PT_UNICODE, (BASE_PROVIDER_ID + 0x0003))
#define PR_MAP_ULNAME       PROP_TAG(PT_LONG,    (BASE_NAMED_ID    + 0x0004))
#define PR_MAP_KIND         PROP_TAG(PT_LONG,    (BASE_NAMED_ID    + 0x0005))
#define PR_MAP_NEXTID       PR_MAP_KIND

#define PR_EDIT_BUTTON      PROP_TAG(PT_OBJECT,  (BASE_PROVIDER_ID + 0x0007))

#define URL_NAME            (BASE_NAMED_ID + 1)
#define PR_SMP_AB_PATH      PROP_TAG(PT_STRING8, (BASE_PROVIDER_ID + 0x0008))
#define PR_ABPAB_ROWID      PROP_TAG(PT_LONG,    (BASE_PROVIDER_ID + 0x0009))
#define PR_BROWSE_BUTTON    PROP_TAG(PT_OBJECT,  (BASE_PROVIDER_ID + 0x000A))
#define PR_URL              PROP_TAG(PT_STRING8, URL_NAME)


#define SZ_BLANK            "                    "
#define SZ_PSS_SAMPLE_AB    "Sample Personal Address Book"
#define SZ_DLL_NAME         "ABPAB.DLL"
#define SZ_AB_DEF_EXT       "AB"
#define SZ_OFN_FILTER       "AB  Files  (*.AB)\0*.AB\0All Files  (*.*)\0*.*\0\0"                             
#define SZ_MAPIPDL          "MAPIPDL"
#define ONE_OFF             0x100
#define DONT_CARE           0xffffffff
#define ISMV_TAG(ulTag)     !!(MV_FLAG & PROP_TYPE((ulTag))) 
#define ISMV_TYPE(ulType)   !!(MV_FLAG & (ulType))          

// our provider UID {3B812993-895B-11cf-91A0-00AA00C14870}
DEFINE_GUID(ABPAB_PROVIDER_UID, 
0x3b812993, 0x895b, 0x11cf, 0x91, 0xa0, 0x0, 0xaa, 0x0, 0xc1, 0x48, 0x70);

// our named properties {0A0F1DC6-9035-11cf-B3CB-00AA0060D63E}
DEFINE_GUID(PS_ABPAB, 
0xa0f1dc6, 0x9035, 0x11cf, 0xb3, 0xcb, 0x0, 0xaa, 0x0, 0x60, 0xd6, 0x3e);

enum {MAP_ID,MAP_UID,MAP_SZNAME,MAP_ULNAME,MAP_KIND,MAP_NEXTID = MAP_KIND,NUM_MAP_PROPS};
static const SizedSPropTagArray(NUM_MAP_PROPS, g_sptMAPTags) =
    {
        NUM_MAP_PROPS,
        {   
            PR_MAP_ID,
            PR_MAP_UID,
            PR_MAP_SZNAME,
            PR_MAP_ULNAME,
            PR_MAP_KIND
        }
    };


#define NULL_STRING         ""
#define DEF_PATH            ""
#define DEF_COMMENT         ""
#define DT_EDIT_FLAGS       (DT_EDITABLE | DT_REQUIRED)

// These have the same meaning as the MAPI constants but frees
// us from depending on the bit values if MAPI redefines them.
// Also, we stuff a little more info into the entry type: whether
// the entry has the PR_TEMPLATEID property and if its from another
// address book (it has a stored details table). We pick values for 
// FOREIGN_ENTRY,HAS_TEMPLATEID that don't collide with MAPI object types.
#define MY_DISTLIST         0x00000008
#define MY_MAILUSER         0x00000006
#define MY_ABCONT           0x00000004
#define FOREIGN_ENTRY       0x1000
#define HAS_TEMPLATEID      0x2000
#define ONEOFF_ENTRY        0x4000
#define ONEOFF_USER_ID      0xfffffffd
#define ONEOFF_DL_ID        0xfffffffc
#define NAME_MAP_ID         0xfffffffb
#define MY_ENTRY_MASK       (HAS_TEMPLATEID | FOREIGN_ENTRY)
#define FOREIGN_DL          (MY_DISTLIST    | FOREIGN_ENTRY)
#define FOREIGN_USER        (MY_MAILUSER    | FOREIGN_ENTRY)
#define FOREIGN_ABCONT      (MY_ABCONT      | FOREIGN_ENTRY)
#define MY_ONEOFF_USER      (MY_MAILUSER    | ONEOFF_ENTRY)
#define MY_ONEOFF_DL        (MY_DISTLIST    | ONEOFF_ENTRY)
#define TEMPLATE_USER       (FOREIGN_USER   | HAS_TEMPLATEID)
#define TEMPLATE_DL         (FOREIGN_DL     | HAS_TEMPLATEID)


// maps a MAPI object type into a ABPAB entry type | ulFlags

#define ENTRY_TYPE(ulObjType)\
            (MAPI_DISTLIST == (ulObjType) ? MY_DISTLIST :\
            (MAPI_MAILUSER == (ulObjType) ? MY_MAILUSER :\
            (MAPI_ABCONT   == (ulObjType) ? MY_ABCONT   : 0)))

#define OBJECT_TYPE(ulEntryType)\
            (MY_DISTLIST == ((ulEntryType) & ~MY_ENTRY_MASK) ? MAPI_DISTLIST :\
            (MY_MAILUSER == ((ulEntryType) & ~MY_ENTRY_MASK) ? MAPI_MAILUSER :\
            (MY_ABCONT   == ((ulEntryType) & ~MY_ENTRY_MASK) ? MAPI_ABCONT   : 0)))
extern DTPAGE  GDLDetailsPage; 
extern DTPAGE  GConfigDetailsPage;
extern DTPAGE  GUserDetailsPage;

enum {DT_CTL_PAGE,
	  DT_CTL_PATH,
	  DT_CTL_NAME,
	  DT_CTL_COMMENT,
      DT_CTL_BROWSE,
	  DT_CTL_LBL1,
	  DT_CTL_LBL2,
	  DT_CTL_LBL3, 
	  DT_NUM_CTLS};

enum _IF_ERR_ {IE_BASEPROP  = 10000000,
               IE_IABCONT   = 20000000,
               IE_MAILUSER  = 30000000,
               IE_IABLOGON  = 40000000,
               IE_MEMBERS   = 50000000,
               IE_BROWSE    = 60000000};
                
// use like FAILED macro, except it calls the IFs SetLastError
#define M_FAIL(hr)  FAILED(SetLastError((hr),__LINE__,__FILE__))

class ConfigObj;
class CABLogon;
class CABProvider;
class CABContainer;
class CMailUser;
class CDataBase;
class CPropCache;
class CUserDetails;
class CDLDetails;
class CFlatPropList;
class CMembersButton;       // a whole IMAPIProp class for one button
class CBrowseButton;
class CTemplateUser;
class CTemplateDL;

typedef CDataBase * PDATABASE;
typedef CFlatPropList * PCFlatPropList;
typedef class CABContainer      * LPCABCont;
typedef class ConfigObj         * LPConfigObj;
typedef class CABProvider       * LPCABProv;
typedef class CMailUser         * PMailUser;
typedef class CPropCache        * LPPropCache;
typedef class SessionListNode   * LPSessionNode;
typedef class CABLogon          * LPCABLogon;
typedef class CContAdviseSink   * PContAdviseSink;
typedef class CUserAdviseSink   * PUserAdviseSink;

typedef struct
{
    ULONG   ulCnx,
            ulListenID,
            ulBCastID;
} NotifCnx;

class ConfigObj : public IMAPIProp
{   
    friend class CBrowseButton;
    public:

        ConfigObj(HINSTANCE         hInst,
                  HWND              hWnd,
                  LPPROVIDERADMIN   pAdminProv,
                  LPMAPISUP         pMapiSup,
                  LPDTPAGE          pDTPage);
        ~ConfigObj();
        STDMETHODIMP        OpenProperty(ULONG                       ulPropTag,                      
                                         LPCIID                      lpiid,                          
                                         ULONG                       ulInterfaceOptions,             
                                         ULONG                       ulFlags,                        
                                         LPUNKNOWN FAR *             lppUnk);                         
        STDMETHODIMP        DoLogonDlg();
        STDMETHODIMP        Init(LPSPropTagArray pLogonTags);
        STDMETHODIMP        DefaultLogonProps(LPSPropTagArray pLogonTags);
        STDMETHODIMP        DefaultLogonProps(LPSPropTagArray pLogonTags,
                                              LPSPropValue    pWizardProps,
                                              ULONG           ulWizardPropCount);
        STDMETHODIMP        UpdateProfile();
        LPSPropValue        m_pProps;           // Property array pointer       

    private: 
        HINSTANCE           m_hInst;            // Instance of DLL
        HWND                m_hWnd;             // Handle of parent window  
        LPMAPISUP           m_pSupObj;          // IMAPISupport object  
        LPPROFSECT          m_pProfSecn;
        LPPROVIDERADMIN     m_pAdminProv;
        LPPROPDATA 		    m_pPropData;        // for display table
        LPMAPITABLE 	    m_pDispTbl;
        LPTABLEDATA         m_pDispTblData;
        LPDTPAGE            m_pDTPage;


        STDMETHODIMP  QueryInterface(REFIID riid, LPVOID * ppvObj) 
        { 
            return m_pPropData -> QueryInterface(riid,ppvObj);
        }

        STDMETHODIMP_ (ULONG) AddRef()
        {
            return m_pPropData -> AddRef();
        }
                   
        STDMETHODIMP_ (ULONG) Release()
        {
            return m_pPropData -> Release();
        }

        STDMETHODIMP GetLastError(HRESULT       hResult,                        
                                          ULONG         ulFlags,                        
                                          LPMAPIERROR * lppMAPIError)                   
        {                                                                                     
            return m_pPropData -> GetLastError(hResult,              
                                            ulFlags,              
                                            lppMAPIError);         
        }                                                                                      
                                                                                               
        STDMETHODIMP SaveChanges(ULONG  ulFlags)                        
        {                                                                                     
            return m_pPropData -> SaveChanges(ulFlags);             
        }                                                                                     
                                                                                              
        STDMETHODIMP GetProps(LPSPropTagArray  lpPropTagArray,                     
                              ULONG            ulFlags,                            
                              ULONG  *         lpcValues,                          
                              LPSPropValue  *  lppPropArray)                       
        {                                                                                     
            return m_pPropData -> GetProps(lpPropTagArray,           
                                        ulFlags,                  
                                        lpcValues,                
                                        lppPropArray);             
        }                                                                                     
                                                                                              
        STDMETHODIMP GetPropList(ULONG             ulFlags,                         
                                 LPSPropTagArray * lppPropTagArray)                 
        {                                                                                     
            return m_pPropData -> GetPropList(ulFlags,               
                                           lppPropTagArray);       
        }                                                                                     
                                                                                              
                                                                                              
        STDMETHODIMP SetProps(ULONG                 cValues,                            
                              LPSPropValue          lpPropArray,                        
                              LPSPropProblemArray * lppProblems)                        
        {                                                                                     
            return m_pPropData -> SetProps(cValues,                  
                                        lpPropArray,              
                                        lppProblems);              
        }                                                                                     
                                                                                              
        STDMETHODIMP DeleteProps(LPSPropTagArray       lpPropTagArray,                  
                                 LPSPropProblemArray * lppProblems)                     
        {                                                                                     
            return m_pPropData -> DeleteProps(lpPropTagArray,        
                                           lppProblems);
        }                                                                                     
                                                                                              
        STDMETHODIMP CopyTo(ULONG                 ciidExclude,                          
                            LPCIID                rgiidExclude,                         
                            LPSPropTagArray       lpExcludeProps,                       
                            ULONG                 ulUIParam,                            
                            LPMAPIPROGRESS        lpProgress,                           
                            LPCIID                lpInterface,                          
                            LPVOID                lpDestObj,                            
                            ULONG                 ulFlags,                              
                            LPSPropProblemArray * lppProblems)                          
        {                                                                                     
            return m_pPropData -> CopyTo(ciidExclude,                
                                         rgiidExclude,               
                                         lpExcludeProps,             
                                         ulUIParam,                  
                                         lpProgress,                 
                                         lpInterface,                
                                         lpDestObj,                  
                                         ulFlags,                    
                                         lppProblems);                
        }                                                                                     
                                                                                              
        STDMETHODIMP CopyProps(LPSPropTagArray       lpIncludeProps,                    
                               ULONG                 ulUIParam,                         
                               LPMAPIPROGRESS        lpProgress,                        
                               LPCIID                lpInterface,                       
                               LPVOID                lpDestObj,                         
                               ULONG                 ulFlags,                           
                               LPSPropProblemArray * lppProblems)                       
        {                                                                                     
            return m_pPropData -> CopyProps(lpIncludeProps,          
                                            ulUIParam,               
                                            lpProgress,              
                                            lpInterface,             
                                            lpDestObj,               
                                            ulFlags,                 
                                            lppProblems);             
        }                                                                                        
                                                                                                 
        STDMETHODIMP GetNamesFromIDs(LPSPropTagArray * lppPropTags,                 
                                     LPGUID            lpPropSetGuid,               
                                     ULONG             ulFlags,                     
                                     ULONG  *          lpcPropNames,                
                                     LPMAPINAMEID **   lpppPropNames)               
        {                                                                                     
            return m_pPropData -> GetNamesFromIDs(lppPropTags,       
                                                  lpPropSetGuid,     
                                                  ulFlags,           
                                                  lpcPropNames,      
                                                  lpppPropNames);     
        }                                                                                     
                                                                                              
        STDMETHODIMP GetIDsFromNames(ULONG             cPropNames,                  
                                     LPMAPINAMEID *    lppPropNames,                
                                     ULONG             ulFlags,                     
                                     LPSPropTagArray * lppPropTags)                 
        {                                                                                     
            return m_pPropData -> GetIDsFromNames(cPropNames,        
                                                  lppPropNames,      
                                                  ulFlags,           
                                                  lppPropTags);       
        }                                                                                        
                                                                                                              
        STDMETHODIMP    OpenProfileSection();
        BOOL    WINAPI  GetValidFileName(LPTSTR * pszFile);
        STDMETHODIMP    UseCannedInProp(LPSPropValue pProp,ULONG LogonTag);
};

extern "C"
{

    STDINITMETHODIMP ABProviderInit
                    (HINSTANCE                  hInstance, 
                     LPMALLOC                   pMallocObj,
                     LPALLOCATEBUFFER           pfnAllocateBuffer,
                     LPALLOCATEMORE             pfnAllocateMore,
                     LPFREEBUFFER               pfnFreeBuffer,
                     ULONG                      ulFlags,
                     ULONG                      ulMAPIVer,
                     ULONG *                    pulProviderVer,
                     LPABPROVIDER *             ppABProviderObj);

    HRESULT STDAPICALLTYPE ServiceEntry(HINSTANCE          hInstance,
                                        LPMALLOC           pMallocObj,
                                        LPMAPISUP          pMAPISupObj,
                                        ULONG              ulUIParam,
                                        ULONG              ulFlags,
                                        ULONG              ulContext,
                                        ULONG              ulCfgPropCount,
                                        LPSPropValue       pCfgProps,
                                        LPPROVIDERADMIN    pAdminProvObj,
                                        LPMAPIERROR *      ppMAPIError);

    STDMETHODIMP GetValidFileName (LPTSTR * pszFile,
                                   HWND     hWnd,
                                   LPVOID   pParent,
                                   ULONG    ulFlags);

    STDMETHODIMP Register3DCtls(HINSTANCE hInst,HINSTANCE * pLibHandle);
    STDMETHODIMP UnRegister3DCtls(HINSTANCE hInstMod,HINSTANCE hInst3DLib);
    STDMETHODIMP BrowseFile(HWND hWnd,LPTSTR *pszFile, LPVOID pParent);
}   // C decls

class CABProvider : public IABProvider
{
    public :
        MAPI_IUNKNOWN_METHODS(IMPL);
        MAPI_IABPROVIDER_METHODS(IMPL);

        CABProvider(HINSTANCE         hInst);
        ~CABProvider();

    private :

        ULONG            m_cRef;
        HINSTANCE        m_hInst;
        CRITICAL_SECTION m_cs;

        STDMETHODIMP     OpenDatabase(ULONG          ulFlags, 
                                      HWND           hWnd,
                                      MAPIUID      & DBUID,
                                      LPMAPISUP      pSupObj,
                                      CDataBase   ** ppDB);
};


class CUserDetails
{
    public:
        CUserDetails(HINSTANCE hInst, LPDTPAGE pUserPage);
        ~CUserDetails();
        STDMETHODIMP BuildDisplayTable(LPMAPITABLE *ppTbl);      
        enum {PAGE,
              NAME,             
              ADDRTYPE,
              EMAIL,
              LBL1,
              LBL2,
              LBL3,
              RTF,
              NUM_CTLS,
              NUM_PAGES = 1};
    private:
        HINSTANCE     m_hInst;
        LPDTPAGE      m_pDTPage;
        LPPROPDATA 	  m_pPropData;        // for display table
        LPMAPITABLE   m_pDispTbl;         // for display table
};

class CDetails
{
    public:
        CDetails(HINSTANCE           hInst,
                LPDTPAGE            pDetailsPage);
        ~CDetails(){};                       
    protected:
        HINSTANCE           m_hInst;
        LPDTPAGE            m_pDetailsPage;
        LPPROPDATA 		    m_pPropData;        // for display table
        LPMAPITABLE 	    m_pDispTbl;         // for display table

};

class CContainerDetails: public CDetails
{
    public:
        CContainerDetails(HINSTANCE hInst,
                          LPDTPAGE  pDetailsPage) : CDetails(hInst,pDetailsPage){};
    
        STDMETHODIMP BuildDisplayTable(LPMAPITABLE *ppTbl);      
};

class CDLDetails: public CDetails
{
    public:
        CDLDetails(HINSTANCE           hInst,
                   LPDTPAGE            pDetailsPage): CDetails(hInst,pDetailsPage){};
          
        STDMETHODIMP BuildDisplayTable(LPMAPITABLE *ppTbl);      
};

class CABLogon : public IABLogon
{
    friend class CABContainer;
    friend class CMailUser;
    friend class CBaseProp;
    friend class CMembersButton;
    friend class CTemplateDL;
    friend class CTemplateUser;

    public :
        MAPI_IUNKNOWN_METHODS(IMPL);
        MAPI_IABLOGON_METHODS(IMPL);
        CABLogon(ULONG             ulFlags,
                 HINSTANCE         hInst,
                 LPMAPISUP         pMAPISup,
                 MAPIUID         & UID,
                 PDATABASE         pDB);
                                         
        ~CABLogon();
        STDMETHODIMP        Init(LPDTPAGE          pDLDetailsPage,
                                 LPDTPAGE          pConfigPage,
                                 LPDTPAGE          pUserPage);

        STDMETHODIMP        Notify(ULONG           ulEvent, 
                                   PABPAB_EID      pParentEID,
                                   PABPAB_EID      pObjectEID,
                                   LPSPropTagArray pPropTags);
        STDMETHODIMP        SetStatusRow(BOOL bAvail);

    private:             
        ULONG               m_cRef,
                            m_ulOneOffID,
                            m_ulVersion,
                            m_ulContext;
        TCHAR               m_szError[MAX_PATH];
        HINSTANCE           m_hInst;
        MAPIUID             m_UID;
        LPMAPISUP           m_pSupObj;
        CUserDetails      * m_pUserDetails;
        CDLDetails        * m_pDLDetails;
        CContainerDetails * m_pRootDetails;
        CABContainer      * m_pRootCont,
                          * m_pDirCont;        
        CRITICAL_SECTION    m_cs,
                            m_MapCS;
        PDATABASE           m_pDB;
                            
        CList<ULONG>      * m_pAdviseList;
        CList<ABPAB_ID>     m_OrphanRecs;                         
        RECORD_NUMBER       m_NameIDRec;
        LPTABLEDATA         m_pMapTable;
        LPMAPITABLE         m_pMapView;
                            
        STDMETHODIMP        GetCTblRow(PCFlatPropList   pFlatPropList,
                                       LPSPropValue *   ppColProps);
                            
        STDMETHODIMP        OpenTemplates(ULONG         ulFlags,                    
                                          LPMAPITABLE * lppTable);
        STDMETHODIMP        OpenForeignUser(PABPAB_EID pEID, CTemplateUser ** ppUser);
        STDMETHODIMP        OpenForeignDL(PABPAB_EID pEID, CTemplateDL ** ppDL);
        STDMETHODIMP        OpenContainer(PABPAB_EID   pEID,
                                          ULONG     *  pulObjType,                
                                          LPUNKNOWN *  ppUnk);
        STDMETHODIMP        OpenRoot(ULONG     *  pulObjType,                
                                     LPUNKNOWN *  ppUnk);
        ULONG WINAPI        CheckEID(ULONG cbEntryID, LPENTRYID pEntryID);
        STDMETHODIMP        CreateEID(ULONG & ulEntryType, PABPAB_EID * ppEID);
                    
        STDMETHODIMP        NewEID(ULONG           ulFlags,
                                   ULONG           ulID,
                                   ULONG           ulEntryType,
                                   LPVOID          pParent,    //NULL if not linked w/ MAPIAllocateMore 
                                   PABPAB_EID    * ppNewEID);
                            
        STDMETHODIMP        AdrToEntryList(LPADRLIST pAdrList, LPENTRYLIST pEntryList);
        STDMETHODIMP        GetMyOneOffTable(ULONG         ulFlags,                    
                                             LPMAPITABLE * lppTable);
        STDMETHODIMP        GetMapTable();
        STDMETHODIMP        SetLastError(HRESULT hRes,ULONG ulLine,LPSTR szFile);
};

class CContAdviseSink : public IMAPIAdviseSink
{
    public:
        CContAdviseSink(LPCABCont pCont);
        ~CContAdviseSink()
        {
            DeleteCriticalSection(&m_cs);
        };
        MAPI_IUNKNOWN_METHODS(IMPL);
        STDMETHODIMP_ (ULONG)   OnNotify(ULONG cNotif, LPNOTIFICATION pNotifs);
    private:
        ULONG       m_cRef;
        LPCABCont   m_pCont;
        CRITICAL_SECTION    m_cs;
};

class CBaseProp
{
    public:
        CBaseProp(LPCABLogon  pLogon, 
                  PABPAB_EID  pCreateContainerEID,
                  PABPAB_EID  pEID,
                  ULONG       ulCreateFlags,
                  LPCABCont   pParentCont,
                  BOOL        bNewEntry);
        virtual ~CBaseProp();

        MAPI_IMAPIPROP_METHODS(IMPL);
        STDMETHODIMP      Init();

    protected:
        CPropCache          * m_pPropCache;
        LPCABLogon            m_pLogon;
        ULONG                 m_fDirtyFlags,
                              m_ulCreateFlags,
                              m_cRef,
                              m_ulVersion,
                              m_ulContext;
        TCHAR                 m_szError[MAX_PATH];
                            
        BOOL                  m_bNewEntry;
        LPCABCont             m_pParentCont;
        ABPAB_EID             m_MyEID,
                              m_CreateContainerEID;   // EID of container this created in
        CRITICAL_SECTION      m_cs;
        STDMETHODIMP          HrSetOneProp(LPSPropValue pProp);
        STDMETHODIMP          UpdateComputedProps(LPSPropValue pProp);
        STDMETHODIMP          PropSettable(ULONG ulPropTag);
        STDMETHODIMP          HrGetOneProp(ULONG ulTag, LPSPropValue * ppProp);
                             
        STDMETHODIMP          DeleteDups(LPCABCont pParentCont);
        STDMETHODIMP          GetAllProps(ULONG        * pcValues,
                                          LPSPropValue * ppProps);
        STDMETHODIMP          NotifyParents(RECORD_NUMBER & MyRecNum,
                                            ULONG           ulTableEvent,
                                            PCFlatPropList   pFlatPropList);
        STDMETHODIMP          SetLastError(HRESULT hRes,ULONG ulLine,LPSTR szFile);
        STDMETHODIMP          NewMapping(LPMAPINAMEID pPropName, ULONG * pulID);
        STDMETHODIMP          GetAllNames(LPSRestriction pRes,LPSPropTagArray *ppPropTags, LPMAPINAMEID ** pppNIDs);
        STDMETHODIMP          GetAllIDs(LPSPropTagArray *ppPropTags);

};

class CABContainer : public CBaseProp,public IABContainer
{
    friend class CMembersButton;
    friend class CContAdviseSink;
    friend class CABLogon;
    friend class CTemplateDL;

    public :
        CABContainer(LPCABLogon pLogon, 
                     PABPAB_EID pCreateContainerEID,
                     PABPAB_EID pMyEID,
                     ULONG      ulCreateFlags,
                     LPCABCont  pParentCont = NULL,
                     BOOL       bNewEntry = FALSE);

        ~CABContainer();
        STDMETHODIMP Init();

        MAPI_IUNKNOWN_METHODS(IMPL);
        MAPI_IMAPIPROP_METHODS(IMPL);
        MAPI_IMAPICONTAINER_METHODS(IMPL);
        MAPI_IABCONTAINER_METHODS(IMPL);


    private:
        ULONG             m_ulCnx,        // cnx no. I get notifs on
                          m_ulVersion,
                          m_ulContext;
        TCHAR             m_szError[MAX_PATH];

        CList<ABPAB_ID>   m_UnsavedChildren;
        PContAdviseSink   m_pAdviseSink;

        STDMETHODIMP      DeleteEntries(CList<ABPAB_ID> & Entries);
        STDMETHODIMP      CopyEntries(CList<ABPAB_ID> & Entries);
        STDMETHODIMP      CreateForeignEntry(LPSRow       pProps, 
                                             ULONG      * pulObjType,
                                             PMailUser  * ppNewEntry);

        STDMETHODIMP      CreateTemplateEntry(ULONG        cbEntryID,              
                                              LPENTRYID    lpEntryID,              
                                              ULONG        ulCreateFlags,          
                                              LPMAPIPROP * lppMAPIPropEntry);

        STDMETHODIMP      SetDefaultDLProps(PABPAB_EID pEID);
        STDMETHODIMP      SetDefaultContProps(PABPAB_EID pEID);
        STDMETHODIMP      PropSettable(ULONG ulPropTag);
        STDMETHODIMP      DeleteChildren(ABPAB_ID & MyID, CList<ABPAB_ID> * pDeletions);
        STDMETHODIMP      InitTemplateUser(LPMAPIPROP      pForeignObj, 
                                           CTemplateUser * pHostUser);
        STDMETHODIMP      InitTemplateDL(LPMAPIPROP     pForeignObj, 
                                         CTemplateDL  * pHostDL);
        STDMETHODIMP      InitContainer();

        STDMETHODIMP      CreateCTbl(LPMAPITABLE   * ppTable,
                                     RECORD_NUMBER & RecNum);

        STDMETHODIMP      CreateRootHTbl(LPMAPITABLE * ppHTbl);
        STDMETHODIMP      CreateBogusCTbl(LPMAPITABLE * ppTable);
        STDMETHODIMP      CopyAdrEntry(LPSPropTagArray pTags, 
                                       LPSRow pRow,
                                       LPADRENTRY  pAdrEntry);
        VOID WINAPI       FreeProps(ULONG cValues,LPSPropValue pProps);
        STDMETHODIMP      HrGetView(LPTABLEDATA pTblData,
                                    LPSSortOrderSet lpSSortOrderSet, 
                                    CALLERRELEASE FAR * lpfCallerRelease, 
                                    ULONG ulCallerData, 
                                    LPMAPITABLE FAR * lppMAPITable);
        STDMETHODIMP      ReplaceDups();
        STDMETHODIMP      SetLastError(HRESULT hRes,ULONG ulLine,LPSTR szFile);

    protected:    
        LPTABLEDATA       m_pCntsTblData,
                          m_pHierTblData;
        BOOL              m_bCTblDirty;
        STDMETHODIMP      UpdateCTbl(RECORD_NUMBER   NewEntryRecNum,
                                     PCFlatPropList  pEntryTblProps);
        STDMETHODIMP      SetDefaultProps(PABPAB_EID pEID);
        STDMETHODIMP      Subscribe();
};

class CMailUser : public CBaseProp, public IMailUser
{
    friend class CABLogon;
    friend class CUserAdviseSink;
    friend class CABContainer;
    friend class CMAPIControl;

    public :
        CMailUser(LPCABLogon pLogon, 
                  PABPAB_EID pCreateContainerEID,
                  PABPAB_EID pEID,
                  ULONG      ulCreateFlags = 0,
                  LPCABCont  pParentCont = NULL,
                  BOOL       bNewEntry   = FALSE);
        ~CMailUser();
        STDMETHODIMP         Init();

        MAPI_IUNKNOWN_METHODS(IMPL);
        MAPI_IMAPIPROP_METHODS(IMPL);

    protected:
        STDMETHODIMP SetLastError(HRESULT hRes,ULONG ulLine,LPSTR szFile);

    private:
        ULONG        m_ulVersion,
                     m_ulContext;
        TCHAR        m_szError[MAX_PATH];

        STDMETHODIMP PropSettable(ULONG ulPropTag);
        STDMETHODIMP SetDefaultProps(PABPAB_EID pEID);
};

class CTemplateUser : public CMailUser
{
    public:
        CTemplateUser(LPCABLogon pLogon, 
                      PABPAB_EID pCreateContainerEID,
                      PABPAB_EID pEID,
                      ULONG      ulCreateFlags = 0,
                      LPCABCont  pCreateCont = NULL,
                      BOOL       bNewEntry = FALSE);
        ~CTemplateUser();

        LPMAPIPROP   m_pRemoteIProp;
        LPMAPITABLE  m_pDetails;
    
        STDMETHODIMP OpenProperty(ULONG       ulPropTag,  
                                  LPCIID      lpiid,      
                                  ULONG       ulInterfaceOptions, 
                                  ULONG       ulFlags,    
                                  LPUNKNOWN * lppUnk);
        STDMETHODIMP SetForeignDetails(LPMAPIPROP pForeignObj);
        STDMETHODIMP GetForeignDetails(LPMAPITABLE * ppTbl);
};

class CTemplateDL : public CABContainer
{
    public :
        CTemplateDL(LPCABLogon pLogon, 
                    PABPAB_EID pCreateContainerEID,
                    PABPAB_EID pMyEID,
                    ULONG      ulCreateFlags,
                    LPCABCont  pCreateCont = NULL,
                    BOOL       bNewEntry = FALSE);

        virtual ~CTemplateDL();

        LPMAPIPROP    m_pRemoteIProp;
        LPMAPITABLE   m_pDetails;

        STDMETHODIMP  OpenProperty(ULONG       ulPropTag,  
                                   LPCIID      lpiid,      
                                   ULONG       ulInterfaceOptions, 
                                   ULONG       ulFlags,    
                                   LPUNKNOWN * lppUnk);
        STDMETHODIMP  SetForeignDetails(LPMAPIPROP pForeignObj);
        STDMETHODIMP  GetForeignDetails(LPMAPITABLE * ppTbl);
};

class CMembersButton : public IMAPIControl
{
    public :
        MAPI_IUNKNOWN_METHODS(IMPL);
        MAPI_IMAPICONTROL_METHODS(IMPL);

        CMembersButton(LPCABCont pCont); 
        ~CMembersButton();

    private :
        LPCABCont        m_pCont;
        ULONG            m_cRef,
                         m_ulVersion,
                         m_ulContext;
        TCHAR            m_szError[MAX_PATH];

        HINSTANCE        m_hInst;
        CRITICAL_SECTION m_cs;
        STDMETHODIMP     SetLastError(HRESULT hRes,ULONG ulLine,LPSTR szFile);
};

class CHomePageButton : public IMAPIControl
{
    public :
        MAPI_IUNKNOWN_METHODS(IMPL);
        MAPI_IMAPICONTROL_METHODS(IMPL);

        CHomePageButton(PMailUser pUser); 
        ~CHomePageButton();

    private :
        PMailUser        m_pUser;
        ULONG            m_cRef,
                         m_ulVersion,
                         m_ulContext;
        TCHAR            m_szError[MAX_PATH];

        HINSTANCE        m_hInst;
        CRITICAL_SECTION m_cs;
        STDMETHODIMP     SetLastError(HRESULT hRes,ULONG ulLine,LPSTR szFile);
};


class CBrowseButton : public IMAPIControl
{
    public :
        MAPI_IUNKNOWN_METHODS(IMPL);
        MAPI_IMAPICONTROL_METHODS(IMPL);

        CBrowseButton(ConfigObj * pConfig); 
        ~CBrowseButton();

    private :
        ULONG       m_cRef,
                    m_ulVersion,
                    m_ulContext;
        TCHAR       m_szError[MAX_PATH];
        HINSTANCE   m_hInst;
        ConfigObj * m_pConfig;
        CRITICAL_SECTION m_cs;
        STDMETHODIMP        SetLastError(HRESULT hRes,ULONG ulLine,LPSTR szFile);
};

STDMETHODIMP CreateDataBase(LPTSTR szDBPath, LPSTORAGE * ppStgDB);
ULONG WINAPI CheckEID(ULONG cbEntryID, LPENTRYID pEntryID);
STDMETHODIMP SetDefContProps(PABPAB_EID pEID, CFlatPropList * pFlatPropList);
LPTSTR       GetNextToken(LPTSTR * pszInput);
void         Normalize(LPTSTR szIn, ULONG  * ulLen);
STDMETHODIMP GetLastError(LPSTR szErr, LPSTR szComp,ULONG ulCtx,LPMAPIERROR * ppErr);
STDMETHODIMP SaveForeignDetails(PDATABASE        pDB,
                                const ABPAB_ID & ID,
                                LPMAPIPROP       pRemoteObj,
                                LPMAPIPROP       pHostObj,
                                LPMAPITABLE    * ppTbl);


enum {INST_KEY,
      DEPTH,
      ACCESS,
      EID,
      STATUS,
      OBJ_TYPE,
      DISP_TYPE,
      NAME,
      COMMENT,
      CONT_FLAGS,
      PROV_UID,
      NUM_HTBL_PROPS};

typedef enum {LOGON_DB_PATH,
              LOGON_DISP_NAME,     
              LOGON_COMMENT,
              LOGON_NUM_PROPS} LOGON_PROP_IDX;

const SizedSPropTagArray(LOGON_NUM_PROPS,sptLogonTags) = 
         {LOGON_NUM_PROPS,{PR_SMP_AB_PATH,
                           PR_DISPLAY_NAME,
                           PR_COMMENT}};
enum 
{
    CT_NAME,
    CT_EID,
    CT_ADDRTYPE,
    CT_ADDR,
    CT_DISP_TYPE,
    CT_SEARCH_KEY,
    CT_OBJTYPE,
    CT_TRANS_NAME,
    CT_INST_KEY,
    CT_SEND_RTF,
    CT_7BIT_NAME,
    NUM_CTBL_COLS
};

// contents table columns
const SizedSPropTagArray(NUM_CTBL_COLS, g_sptCntTbl) =
{
    NUM_CTBL_COLS,
    {
        PR_DISPLAY_NAME,
        PR_ENTRYID,
        PR_ADDRTYPE,
        PR_EMAIL_ADDRESS,
        PR_DISPLAY_TYPE,
        PR_SEARCH_KEY,
        PR_OBJECT_TYPE,
        PR_TRANSMITABLE_DISPLAY_NAME,
        PR_INSTANCE_KEY,
        PR_SEND_RICH_INFO,
        PR_7BIT_DISPLAY_NAME
    }
};

// Hierarchy table columns
const SizedSPropTagArray(NUM_HTBL_PROPS, sptHierTbl) =
{
    NUM_HTBL_PROPS,
    {
        PR_INSTANCE_KEY,
        PR_DEPTH,
        PR_ACCESS,
        PR_ENTRYID,
        PR_STATUS,
        PR_OBJECT_TYPE,
        PR_DISPLAY_TYPE,
        PR_DISPLAY_NAME,
        PR_COMMENT,
        PR_CONTAINER_FLAGS,
        PR_AB_PROVIDER_ID
    }
};


// CTL3D32 stuff
extern HRESULT gHR_To_IDS[];
typedef BOOL (WINAPI *CTL3DFCN)(HINSTANCE);
extern CTL3DFCN         pfnCtl3dAutoSubClass;
extern CTL3DFCN         pfnCtl3dRegister;
extern CTL3DFCN         pfnCtl3dUnregister;

// MAPI allocation functions
extern LPALLOCATEBUFFER gAllocBuff;  
extern LPALLOCATEMORE   gAllocMore;     
extern LPFREEBUFFER     gFreeBuff;       

// some handy overloads
BOOL operator ==(const ABPAB_ID & lhs, const ABPAB_ID & rhs);
BOOL operator ==(const ABPAB_EID & lhs, const ABPAB_EID & rhs);
BOOL operator !=(const MAPIUID & lhs, const MAPIUID & rhs);
void *operator new(size_t size);
void operator delete(void * pObj);

// utility function for setting prop problems
void HrSetPropProblem(LPSPropProblemArray pProbs,
                      ULONG ulIdx, 
                      ULONG ulTag,
                      HRESULT hRes);
#endif

// End of file for ABPAB.H
