///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      IABLOGON.CPP  
//
//  Description
//      IABLogon implementation. Note: this implementation doesn't support 
//      notifications across process boundaries.
//
//  Author
//      Les Thaler
//
//  Revision: 1.00
//
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//
//
#include "abpab.h"
#include "cdbfile.h"
#include "abdb.h"
#include "abcache.h"
#include "traces.h"
#include "list.inl"
#include "idh.h"


#define SUPPORTED_EVENTS    (fnevObjectModified | \
                             fnevObjectCopied   | \
                             fnevObjectDeleted)
#define ABPAB_HLP "ABPAB Help"

// Display Table controls for DistList details page
DTBLLABEL   DLLabel      = {sizeof(DTBLLABEL), 0};

DTBLPAGE    DLPage       = {sizeof DTBLPAGE   , 0      , 0, IDH_CREATE_DL};
DTBLLBX     DLMembers    = {0                 , PR_NULL, PR_CONTAINER_CONTENTS};
DTBLEDIT    DLName       = {sizeof DTBLEDIT   , 0      , MAX_PATH , PR_DISPLAY_NAME};
DTBLBUTTON  DLEditButton = {sizeof DTBLBUTTON , 0      , PR_EDIT_BUTTON};


DTCTL  GDLControls[] = 
       {
          {DTCT_PAGE  ,0             ,NULL ,0 ,NULL ,0                ,&DLPage      },
          {DTCT_EDIT  ,DT_EDIT_FLAGS ,NULL ,0 ,"*"  ,IDC_EDIT_NAME    ,&DLName      },
          {DTCT_LBX   ,DT_EDIT_FLAGS ,NULL ,0 ,"*"  ,IDC_LIST_MEMBERS ,&DLMembers   },
          {DTCT_LABEL ,0             ,NULL ,0 ,NULL ,IDC_STATIC1      ,&DLLabel     },
          {DTCT_LABEL ,0             ,NULL ,0 ,NULL ,IDC_STATIC2      ,&DLLabel     },
          {DTCT_BUTTON,0             ,NULL ,0 ,NULL ,IDC_BUTTON_EDIT  ,&DLEditButton},
       };

DTPAGE       GDLDetailsPage = {6,MAKEINTRESOURCE(IDD_DL_DETAILS),ABPAB_HLP, GDLControls};

DTBLPAGE     UserPage      = {sizeof DTBLPAGE     , 0, 0        , IDH_CREATE_RECIP};
DTBLEDIT     UserName      = {sizeof(DTBLEDIT)    , 0, MAX_PATH , PR_DISPLAY_NAME};
DTBLEDIT     UserAddrType  = {sizeof(DTBLEDIT)    , 0, MAX_PATH , PR_ADDRTYPE};
DTBLEDIT     UserEmailAddr = {sizeof(DTBLEDIT)    , 0, MAX_PATH , PR_EMAIL_ADDRESS};
DTBLCHECKBOX UserRTF       = {sizeof DTBLCHECKBOX , 0, PR_SEND_RICH_INFO};

DTCTL  GUserControls[] = 
       {// ulCtlType    ulCtlFlags    lpb/cbNotif Filter  ItemID           ctl.?
          {DTCT_PAGE    ,0             ,NULL ,0 ,NULL   ,0                ,&UserPage     },
          {DTCT_EDIT    ,DT_EDIT_FLAGS ,NULL ,0 ,"*"    ,IDC_EDIT_NAME    ,&DLName       },
          {DTCT_EDIT    ,DT_EDIT_FLAGS ,NULL ,0 ,"*"    ,IDC_EDIT_ADDRTYPE,&UserAddrType },
          {DTCT_EDIT    ,DT_EDIT_FLAGS ,NULL ,0 ,"*"    ,IDC_EDIT_EMAIL   ,&UserEmailAddr},
          {DTCT_LABEL   ,0             ,NULL ,0 ,NULL   ,IDC_STATIC1      ,&DLLabel      },
          {DTCT_LABEL   ,0             ,NULL ,0 ,NULL   ,IDC_STATIC2      ,&DLLabel      },
          {DTCT_LABEL   ,0             ,NULL ,0 ,NULL   ,IDC_STATIC3      ,&DLLabel      },
          {DTCT_CHECKBOX,DT_EDITABLE   ,NULL ,0 ,NULL   ,IDC_RTF_CHECK    ,&UserRTF      }
       };


DTPAGE GUserDetailsPage = {8,MAKEINTRESOURCE(IDD_USER_DETAILS),ABPAB_HLP, GUserControls};


DTBLPAGE    ConfigPage    = {sizeof DTBLPAGE   ,0 , sizeof DTBLPAGE, IDH_CONFIG};
DTBLEDIT    ConfigName    = {sizeof(DTBLEDIT)  ,0 , MAX_PATH, PR_DISPLAY_NAME};
DTBLEDIT    ConfigPath    = {sizeof(DTBLEDIT)  ,0 , MAX_PATH, PR_SMP_AB_PATH};
DTBLEDIT    ConfigComment = {sizeof(DTBLEDIT)  ,0 , MAX_PATH, PR_COMMENT};
DTBLBUTTON  ConfigBrowse  = {sizeof(DTBLBUTTON),0 , PR_BROWSE_BUTTON};
DTBLLABEL   ConfigLabel   = {sizeof(DTBLLABEL) ,0};

#define PATH_KEY ((LPBYTE)"BROWSE_BUTTON")
#define CB_KEY   sizeof PATH_KEY

DTCTL  GConfigPageControls[] = 
       {
         {DTCT_PAGE  ,0             ,NULL     ,0      ,NULL ,0             ,&ConfigPage   },
         {DTCT_EDIT  ,DT_REQUIRED   ,PATH_KEY ,CB_KEY ,"*"  ,IDC_ABPATH    ,&ConfigPath   },
         {DTCT_EDIT  ,DT_EDIT_FLAGS ,NULL     ,0      ,"*"  ,IDC_ABNAME    ,&ConfigName   },
         {DTCT_EDIT  ,DT_EDITABLE   ,NULL     ,0      ,"*"  ,IDC_ABCOMMENT ,&ConfigComment},
         {DTCT_BUTTON,0             ,NULL     ,0      ,NULL ,IDC_BROWSE    ,&ConfigBrowse },
         {DTCT_LABEL ,0             ,NULL     ,0      ,NULL ,IDC_STATIC1   ,&ConfigLabel  },
         {DTCT_LABEL ,0             ,NULL     ,0      ,NULL ,IDC_STATIC2   ,&ConfigLabel  },
         {DTCT_LABEL ,0             ,NULL     ,0      ,NULL ,IDC_STATIC3   ,&ConfigLabel  }
       };

DTPAGE GConfigDetailsPage = {8,MAKEINTRESOURCE(IDD_CONFIGPAGE),ABPAB_HLP, GConfigPageControls};


///////////////////////////////////////////////////////////////////////////////
//
//    QueryInterface()
//
//    Parameters                        
//          See the OLE documentation for details.
//
//    Purpose
//          Returns an interface if supported.
//      
//      
//    Return Value
//          NOERROR on success, E_NOINTERFACE if the interface
//          can't be returned.
//
STDMETHODIMP CABLogon::QueryInterface(REFIID   riid, 
                                      LPVOID * ppvObj)
{
    TraceMessage("CABLogon :: QueryInterface");

    if (IsBadWritePtr(ppvObj,sizeof LPUNKNOWN))
        return MAPI_E_INVALID_PARAMETER;

    *ppvObj = NULL;

    // If this is one of the two IID return an interface pointer to it
    if (riid == IID_IUnknown || riid == IID_IABLogon)
    {
        EnterCriticalSection(&m_cs);
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        m_cRef++;
        LeaveCriticalSection(&m_cs);
        return NOERROR;
    }

    return E_NOINTERFACE;
}

///////////////////////////////////////////////////////////////////////////////
//    CABLogon::AddRef()
//
//    Parameters
//          Refer to OLE Documentation on this method
//
//    Purpose
//          To increase the reference count of this object,
//          and of any contained (or aggregated) object within
//
//    Return Value
//      Usage count for this object
//
STDMETHODIMP_ (ULONG) CABLogon::AddRef (void)
{   
    ULONG ulRef;

    EnterCriticalSection(&m_cs);
    ulRef = ++m_cRef;
    TraceString1("CABLogon::AddRef, Ref: %d", m_cRef);
    LeaveCriticalSection(&m_cs);

    return ulRef;
}

///////////////////////////////////////////////////////////////////////////////
//    CABLogon::Release()
//
//    Parameters
//          Refer to OLE Documentation on this method
//
//    Purpose
//          Decreases the usage count of this object. If the 
//          count is down to zero, object deletes itself.
//
//    Return Value
//      Usage of this object
//
STDMETHODIMP_ (ULONG) CABLogon::Release (void)
{
    ULONG ulRefCount;

    EnterCriticalSection(&m_cs);
    ulRefCount = --m_cRef;
    LeaveCriticalSection(&m_cs); 

    TraceString1("CABLogon::Release, Ref: %d", ulRefCount);

    // If nobody is using this object, delete it
    if (0 == ulRefCount)
    {
        // Destroy the object
        delete this;        // don't need to leave the critical section
        return ulRefCount;  // its deleted when the object is destroyed
    }
        
    return ulRefCount;
}


///////////////////////////////////////////////////////////////////////////////
//
//    CABLogon()
//
//    Parameters                        
//          hWnd            [in] handle to main application window
//          ulFlags         [in] creation flags  
//          hInst           [in] module instance handle
//          pMAPISup        [in] pointer to the support object
//          pDLDetailsPage  [in] pointer to the DL details DTPAGE
//          pConfigPage     [in] pointer to the DIR container DTPAGE
//          UID             [in] reference to this provider's UID
//          pDB             [in] pointer to a database object
//    Purpose
//          Class constructor for CABLogon object
//      
//    Return Value
//          none.
//
CABLogon :: CABLogon(ULONG             ulFlags,
                     HINSTANCE         hInst,
                     LPMAPISUP         pMAPISup,
                     MAPIUID         & UID,
                     PDATABASE         pDB)
{
    TraceMessage("CABLogon :: CABLogon");

    m_cRef         = 0;
    m_hInst        = hInst;
    m_pDB          = pDB;
    m_pSupObj      = pMAPISup;
    m_pRootCont    = m_pDirCont = NULL;
    m_ulOneOffID   = 0;
    m_ulVersion    = 0;
    m_ulContext    = 0;
    m_pUserDetails = NULL;
    m_pRootDetails = NULL;
    m_pAdviseList  = NULL;
    m_pMapTable    = NULL;
    m_pMapView     = NULL;
    ZeroMemory((LPVOID) & m_NameIDRec, sizeof RECORD_NUMBER);
    m_szError[0] = '\0';
    m_pDB -> AddRef();
    m_pSupObj -> AddRef();

    memcpy((LPVOID) &m_UID, (LPVOID) &UID, sizeof UID);
    InitializeCriticalSection(&m_cs);
    InitializeCriticalSection(&m_MapCS);
}

///////////////////////////////////////////////////////////////////////////////
//
//    ~CABLogon()
//
//    Parameters                        
//          none.
//
//    Purpose
//          Class destructor.
//
//    Return Value
//          none.
//
CABLogon :: ~CABLogon()
{   
    // most of the destruction is done in Logoff 
    DeleteCriticalSection(&m_cs);
    DeleteCriticalSection(&m_MapCS);
}

///////////////////////////////////////////////////////////////////////////////
//
//    Init()
//
//    Parameters                        
//          pDLDetailsPage  [in] pointer to the DL details DTPAGE
//          pConfigPage     [in] pointer to the DIR container DTPAGE
//          pUserPage       [in] pointer to the user DTPAGE
//    Purpose
//          Class constructor for CABLogon object
//      
//    Return Value
//          none.
//
STDMETHODIMP CABLogon :: Init(LPDTPAGE          pDLDetailsPage,
                              LPDTPAGE          pConfigPage,
                              LPDTPAGE          pUserPage)
{

    HRESULT             hRes = S_OK;
    ABPAB_ID            NameMapID = {NAME_MAP_ID,0};
    LPSRowSet           pSRows = NULL;

    m_pUserDetails = new CUserDetails(m_hInst,pUserPage);

    if (!m_pUserDetails)
    {
        hRes = MAPI_E_NOT_ENOUGH_MEMORY;
        goto Quit;
    }

    m_pDLDetails = new CDLDetails(m_hInst,
                                  pDLDetailsPage);
    if (!m_pDLDetails)
    {
        hRes = MAPI_E_NOT_ENOUGH_MEMORY;
        goto Quit;
    }

    m_pRootDetails = new CContainerDetails(m_hInst,
                                           pConfigPage);
    if (!m_pRootDetails)
    {
        hRes = MAPI_E_NOT_ENOUGH_MEMORY;
        goto Quit;
    }

   m_pAdviseList = new CList<ULONG>;

   if (!m_pAdviseList)
   {
        hRes = MAPI_E_NOT_ENOUGH_MEMORY;
        goto Quit;
   } 

   // set up our name-to_ID mapping table
   if (M_FAIL(hRes = Open(m_pDB,CREATE_IF_NOT_EXIST,NameMapID, m_NameIDRec)))
       goto Quit;

   if (M_FAIL(hRes = CreateTable((LPIID) &IID_IMAPITableData,
                                  gAllocBuff,
                                  gAllocMore,
                                  gFreeBuff,
                                  NULL,
                                  TBLTYPE_DYNAMIC,
                                  PR_MAP_ID,
                                  (LPSPropTagArray) &g_sptMAPTags,
                                  &m_pMapTable)))
        goto Quit;
    
    if (M_FAIL(hRes = Fetch(m_NameIDRec, fldTABLEROWS, &pSRows)))
        goto Quit;
    
    hRes = m_pMapTable -> HrModifyRows(0,pSRows);

Quit:
    Close(m_NameIDRec);
    FreeProws(pSRows);

    if (FAILED(hRes))
    {
        

        if (m_pUserDetails)
            delete m_pUserDetails;

        if (m_pDLDetails)
            delete m_pDLDetails;

        if (m_pRootDetails)
            delete m_pRootDetails;

        if (m_pAdviseList)
            delete m_pAdviseList;

        m_pUserDetails = NULL;
        m_pDLDetails   = NULL;
        m_pRootDetails = NULL;
        m_pAdviseList  = NULL;
    }
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  SetStatusRow
//
//    Parameters                        
//          none.
//
//    Purpose
//          Sets the row in the status table for this provider. PR_STATUS_CODE,
//          PR_RESORUCE_METHODS, and PR_PROVIDER_DISPLAY are required columns.
//    Return Value
//          none.
//

STDMETHODIMP CABLogon :: SetStatusRow(BOOL bAvail)
{
    #define NUM_STATUS_ROW_PROPS    9
    SPropValue spvStatusRow[NUM_STATUS_ROW_PROPS] = { 0 };
    
    ULONG i = 0;
    spvStatusRow[i].ulPropTag = PR_PROVIDER_DISPLAY;
    spvStatusRow[i++].Value.lpszA = SZ_PSS_SAMPLE_AB;
    
    spvStatusRow[i].ulPropTag = PR_STATUS_CODE;
    spvStatusRow[i++].Value.l = bAvail ? STATUS_AVAILABLE : STATUS_FAILURE;
    
    spvStatusRow[i].ulPropTag = PR_STATUS_STRING;
    spvStatusRow[i++].Value.lpszA = bAvail ? "Available" : "Not Available";
    
    spvStatusRow[i].ulPropTag = PR_DISPLAY_NAME;
    spvStatusRow[i++].Value.lpszA = SZ_PSS_SAMPLE_AB;
    
    spvStatusRow[i].ulPropTag = PR_RESOURCE_FLAGS;
    spvStatusRow[i++].Value.l = STATUS_NO_PRIMARY_IDENTITY;
    
    spvStatusRow[i].ulPropTag = PR_RESOURCE_METHODS;
    spvStatusRow[i++].Value.l = 0;

    spvStatusRow[i].ulPropTag = PR_RESOURCE_TYPE;
    spvStatusRow[i++].Value.l = MAPI_AB_PROVIDER;

    spvStatusRow[i].ulPropTag = PR_OBJECT_TYPE;
    spvStatusRow[i++].Value.l = MAPI_STATUS;

    spvStatusRow[i].ulPropTag = PR_PROVIDER_DLL_NAME;
    spvStatusRow[i++].Value.lpszA = SZ_DLL_NAME;

    return m_pSupObj->ModifyStatusRow (i, spvStatusRow, 0);
}

///////////////////////////////////////////////////////////////////////////////
//
//    SetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Sets detailed error information based on hRes.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//
STDMETHODIMP CABLogon :: SetLastError(HRESULT       hRes,    
                                      ULONG         ulLine,    
                                      LPSTR         szFile)
{
    if (FAILED(hRes))
    {
        EnterCriticalSection(&m_cs);
        m_ulContext = ulLine + IE_IABLOGON;
        lstrcpyn(m_szError,szFile,MAX_PATH);
        m_szError[MAX_PATH - 1] = '\0';
        LeaveCriticalSection(&m_cs);
    }
    return hRes;
}


///////////////////////////////////////////////////////////////////////////////
//
//    GetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Gets detailed error information based on hResult.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//
STDMETHODIMP CABLogon :: GetLastError(HRESULT         hResult,                    
                                      ULONG           ulFlags,                    
                                      LPMAPIERROR  *  lppMAPIError)
{
    HRESULT hRes;
    TraceMessage("CABLogon :: GetLastError");
    CheckParameters_IABLogon_GetLastError(this,
                                          hResult,     
                                          ulFlags,     
                                          lppMAPIError);
    EnterCriticalSection(&m_cs);
    hRes =::GetLastError(m_szError,SZ_PSS_SAMPLE_AB,m_ulContext,lppMAPIError);
    LeaveCriticalSection(&m_cs);
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Logoff()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
STDMETHODIMP CABLogon :: Logoff(ULONG ulFlags)
{
    TraceMessage("CABLogon :: Logoff"); 
    CheckParameters_IABLogon_Logoff(this, ulFlags );

    EnterCriticalSection(&m_cs);
    EnterCriticalSection(&m_MapCS);

    if (m_pMapView)
        m_pMapView -> Release();

    if (m_pMapTable)
        m_pMapTable -> Release();

    LeaveCriticalSection(&m_MapCS);

    Close(m_NameIDRec);

    // delete any orphan records
    for (PABPAB_ID   pID = m_OrphanRecs.Dequeue();
         pID;
         pID = m_OrphanRecs.Dequeue())
    {
        DECLARE_REC_NUM(RecNum);

        Open(m_pDB, OVERWRITE, *pID,RecNum);   // ignore errors
        DeleteRec(m_pDB,RecNum);
        Close(RecNum);

        delete pID;
    }

    ULONG * pulCnx;
    
    if (m_pAdviseList)
    {
        pulCnx = m_pAdviseList -> Dequeue();

        while (pulCnx)
        {
            m_pSupObj -> Unsubscribe(*pulCnx);
            delete pulCnx;
            pulCnx = m_pAdviseList -> Dequeue();
        }

        delete m_pAdviseList;
    }

    if (m_pUserDetails)
        delete m_pUserDetails;

    if (m_pDLDetails)
        delete m_pDLDetails;

    if (m_pRootDetails)
        delete m_pRootDetails;

    if (m_pRootCont)
        m_pRootCont -> Release();

    if (m_pDirCont)
        m_pDirCont -> Release();

    if (m_pDB)
        m_pDB -> Release();

    if (m_pSupObj)
        m_pSupObj -> Release();

    LeaveCriticalSection(&m_cs);

    return S_OK;
}
           
///////////////////////////////////////////////////////////////////////////////
//
//    CheckEID()
//
//    Parameters                        
//          cbEntryID   [in] size of entry ID
//          pEntryID    [in] pointer to an entry ID
//    Purpose
//          Check if the EID is valid, if it's own of ours and if so,
//          what type of object is its owner, container or mailuser,
//          foreign entry?
//      
//    Return Value
//          Code indicating the ID type, or MAPI_E_INVALID_ENTRYID is 
//          type can't be determined.
//
ULONG WINAPI CABLogon :: CheckEID(ULONG cbEntryID, LPENTRYID pEntryID)
{

    PABPAB_EID pEID = (PABPAB_EID) pEntryID;

    if (cbEntryID && IsBadReadPtr((LPVOID) pEntryID, cbEntryID))
        return (ULONG) MAPI_E_INVALID_ENTRYID;

    // root container is NULL
    if (0 == cbEntryID || NULL == pEID)
        return MY_ABCONT;

    if (pEID -> muid != m_UID)
        return FOREIGN_ENTRY;

    return pEID -> ID.Type;
}

///////////////////////////////////////////////////////////////////////////////
//
//    OpenForeignDL()
//
//    Parameters                        
//          pEID    [in]     pointer to the DL's entryID
//          ppDL    [in/out] pointer to place to write the new DL object
//    Purpose
//          Create a new MY_DISTLIST object that was copied from another
//          AB. The only difference is that foreign entries will have a 
//          different details page (we got it from the foreign AB when we
//          CopyEntries'd it over or CreateEntry'd it from a foreign one-off 
//          template. We return a ForeignUser object because OpenProperty on 
//          PR_DETAILS_TABLE needs to retrieve the stored details table. Also, 
//          if the original object had a template ID, we can let the foreign 
//          AB delegate OpenProperty calls. To let the foreign AB delegate, 
//          we return call OpenTemplateID and return the remote IProp nested 
//          in this object. If the original object doesn't have a template ID,
//          we return the outer object, no delagtion occurs. 
//          We check the ID.Type flags to see if there's a template ID.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: OpenForeignDL(PABPAB_EID pEID, CTemplateDL ** ppDL)
{

    HRESULT         hRes    = S_OK;
    CTemplateDL   * pHostDL = NULL;
    LPSPropValue    pTID    = NULL;
    BOOL            bUseRemoteData = FALSE;
    DECLARE_REC_NUM(RecNum);

    TraceString1("CABLogon::OpenForeignDL %x",pEID -> ID.Num);

    *ppDL   = NULL;
    pHostDL = new CTemplateDL(this,NULL,pEID,0);

    if (!pHostDL)
        return MAPI_E_NOT_ENOUGH_MEMORY;
   
    if (FAILED(hRes = pHostDL -> Init()))
        goto Quit;   

    if (FAILED(hRes = Open(m_pDB,
                           OPEN_ONLY, 
                           pEID -> ID,
                           RecNum)))
        goto Quit;
    
    hRes = pHostDL -> m_pPropCache -> Load(RecNum);
    Close(RecNum);

    if (FAILED(hRes))
        goto Quit;

    if (TEMPLATE_DL == pEID -> ID.Type) // i.e. it has PR_TEMPLATEID
    {
        if (FAILED(hRes = pHostDL -> HrGetOneProp(PR_TEMPLATEID,
                                                  &pTID)))
            goto Quit;                                                        

        hRes = m_pSupObj -> OpenTemplateID(pTID -> Value.bin.cb,
                                           (LPENTRYID) pTID ->Value.bin.lpb,
                                           0,
                                           pHostDL,
                                           NULL,
                                           (LPMAPIPROP*) &(pHostDL -> m_pRemoteIProp),
                                           NULL);
        // A failure on OpenTemplateID isn't fatal. Most likely the server not available, 
        // use our local data instead. Fields dependent on server will be blank in the details.
        bUseRemoteData = SUCCEEDED(hRes);
        hRes = S_OK;
    }

    if (bUseRemoteData)       
    {        
        // if OpenTemplateID provides remote IProp, it'll AddRefs us
        if (pHostDL != pHostDL -> m_pRemoteIProp)
            pHostDL -> Release();

       *ppDL = (CTemplateDL *) pHostDL -> m_pRemoteIProp;
    }
    else
    {
        // server won't delegate, all OpenProperty calls go
        // first to CTemplateUser::, then CMAPIProp::
        *ppDL = pHostDL;
    }
    
    // create a cnx so this object can hear it's parents
    // calling it
    if (FAILED(hRes = pHostDL -> Subscribe()))
        goto Quit;

    // if we pass remote IProp, client Releases, server delegates to 
    // to us. if we pass host, client Releases directly

Quit:
    if (FAILED(hRes))
    {
        if (pHostDL)
            pHostDL -> Release();
    }

    gFreeBuff(pTID);    
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    OpenForeignUser()
//
//    Parameters                        
//          pEID    [in]     pointer to the DL's entryID
//          ppDL    [in/out] pointer to place to write the new DL object
//    Purpose
//          Create a new MY_MAILUSER object that was copied from another
//          AB. The only difference is that foreign entries will have a 
//          different details page (we got it from the foreign AB when we
//          CopyEntries'd it over or CreateEntry'd it from a foreign one-off 
//          template. We return a ForeignUser object because OpenProperty on 
//          PR_DETAILS_TABLE needs to retrieve the stored details table. Also, 
//          if the original object had a template ID, we can let the foreign 
//          AB delegate OpenProperty calls. To let the foreign AB delegate, 
//          we call OpenTemplateID and return the remote IProp nested 
//          in this object. If the original object doesn't have a template ID,
//          we return the outer object, no delagtion occurs. 
//          We check the ID.Type flags to see if there's a template ID.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: OpenForeignUser(PABPAB_EID pEID, CTemplateUser ** ppUser)                                      
{
    HRESULT         hRes           = S_OK;
    CTemplateUser * pHostUser      = NULL;
    LPSPropValue    pTID           = NULL;
    BOOL            bUseRemoteData = FALSE;

    DECLARE_REC_NUM(RecNum);


    TraceString1("CABLogon::OpenForeignUser %x",pEID -> ID.Num);

    *ppUser = NULL;
    pHostUser = new CTemplateUser(this,NULL,pEID);

    if (!pHostUser)
        return MAPI_E_NOT_ENOUGH_MEMORY;

    if (FAILED(hRes = pHostUser -> Init()))
        goto Quit;
   
    if (FAILED(hRes = Open(m_pDB,
                           OPEN_ONLY, 
                           pEID -> ID,
                           RecNum)))
        goto Quit;


    hRes = pHostUser -> m_pPropCache -> Load(RecNum);
    Close(RecNum);

    if (FAILED(hRes))
        goto Quit;
     
    if (TEMPLATE_USER == pEID -> ID.Type) // i.e. it has PR_TEMPLATEID
    {
        if (FAILED(hRes = pHostUser -> HrGetOneProp(PR_TEMPLATEID,&pTID)))
            goto Quit;                                                        

        hRes = m_pSupObj -> OpenTemplateID(pTID -> Value.bin.cb,
                                           (LPENTRYID) pTID ->Value.bin.lpb,
                                           0,
                                           (LPMAPIPROP)pHostUser,
                                           NULL,
                                           (LPMAPIPROP*) &(pHostUser -> m_pRemoteIProp),
                                           NULL);

        // A failure on OpenTemplateID isn't fatal. Most likely the server isnt available, 
        // use our local data instead. Fields dependent on server will be blank in the details.
        bUseRemoteData = SUCCEEDED(hRes);
        hRes = S_OK;
    }

    if (bUseRemoteData)       
    {        
        // if OpenTemplateID provides remote IProp, it'll AddRefs us
        if (pHostUser != pHostUser -> m_pRemoteIProp)
            pHostUser -> Release();

       *ppUser = (CTemplateUser *) pHostUser -> m_pRemoteIProp;
    }
    else
    {
        // server won't delegate, all OpenProperty calls go
        // first to CTemplateUser::, then CMAPIProp::
        *ppUser = pHostUser;
        // if we pass remote IProp, client Releases, server delegates to 
        // to us. if we pass host, client Releases directly
    }

Quit:
    if (FAILED(hRes))
    {
        if (pHostUser)
            pHostUser -> Release();
    }

    gFreeBuff(pTID);    
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    OpenContainer()
//
//    Parameters                        
//          pEID        [in]  pointer to entry ID of object to open     
//          pulObjType  [out] pointer where we write the object's type 
//          ppUnk       [out] pointer to pointer where we write the object
//    Purpose
//          Opens a container or DL object. Sets default properties and
//          any already existing in the database. Creates a contents
//          table and subscribes to listen for notifications from its
//          children.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: OpenContainer(PABPAB_EID   pEID,
                                       ULONG     *  pulObjType,                
                                       LPUNKNOWN *  ppUnk)
{   
    HRESULT         hRes  = S_OK;
    LPCABCont       pCont = NULL;
    DECLARE_REC_NUM(RecNum);
    
    *pulObjType = 0;
    *ppUnk      = NULL;
    pCont = new CABContainer(this,NULL,pEID,0);
    
    if (!pCont)
        return MAPI_E_NOT_ENOUGH_MEMORY;

    if (FAILED(hRes = pCont -> Init()))
        goto Quit;    

    if (ONEOFF_ENTRY & pEID -> ID.Type)
    {
        hRes = pCont -> SetDefaultProps(pEID);
        *ppUnk =  pCont;
        *pulObjType = OBJECT_TYPE(pEID -> ID.Type);

        return hRes;
    }

    if (FAILED(hRes = Open(m_pDB,
                           OPEN_ONLY,
                           pEID -> ID, 
                           RecNum)))
            goto Quit;

    // need to Notify any listening parents and children
    // when this object changes                                
    hRes = pCont -> m_pPropCache -> Load(RecNum);
    
    if (FAILED(hRes))
    {
        // This is a special case, for a new DL that you 
        // create a new entry 'just in this DL' before SaveChanges. 
        // It's props haven't been saved yet, but its an orphan rec.
        // IAddrBook OpenEntry's the unsaved DL to add the new entry.
        if (MAPI_E_NOT_FOUND == hRes)
        {
            if (FAILED(hRes = pCont -> SetDefaultProps(pEID)))
            {
                Close(RecNum);
                goto Quit;
            }    
        }
        else
        {
            Close(RecNum);
            goto Quit;
        }
    }
        
    // create a cnx so this object can listen 
    // for changes to its children
    if (FAILED(hRes = pCont -> Subscribe()))
    {
        Close(RecNum);
        goto Quit;
    }
    
    if (DIR_ID_NUM == pEID -> ID.Num)
    {
        // quick sanity check
        ASSERT(!m_pDirCont);

        // cache DIR container,AddRef for us
        m_pDirCont = pCont;
        pCont -> AddRef();
    }

    Close(RecNum);

Quit:
    if (FAILED(hRes))
    {
        if (pCont)
            delete pCont;
        return hRes;
    }

    *ppUnk =  pCont;
    *pulObjType = OBJECT_TYPE(pEID -> ID.Type);

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    OpenRoot()
//
//    Parameters                        
//          pulObjType  [in]  pointer to place to return new object's type
//          ppUnk       [out] pointer to pointer to where we write new object
//    Purpose
//          Creates a new root container. Sets default properties or any
//          existing in the database. Creates the hierarchy and contents table.
//          Listens for notifications from its children.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: OpenRoot(ULONG     *  pulObjType,                
                                  LPUNKNOWN *  ppUnk)

{   
    HRESULT         hRes  = S_OK;
    PABPAB_EID      pEID  = NULL;
    LPCABCont       pCont = NULL;
    DECLARE_REC_NUM(RecNum);

    if (M_FAIL(hRes = NewEID(0,
                             ROOT_ID_NUM,  
                             MAPI_ABCONT,
                             NULL, 
                             &pEID)))
        goto Quit;

    pCont = new CABContainer(this,NULL,pEID,0);
    
    if (!pCont)
    {
        hRes = MAPI_E_NOT_ENOUGH_MEMORY;
        goto Quit;
    }
    
    if (FAILED(hRes = pCont -> Init()))
        goto Quit;
        
    if (FAILED(hRes = Open(m_pDB,
                           OPEN_ONLY,
                           pEID -> ID, 
                           RecNum)))
        goto Quit;
    
    if (FAILED(hRes = pCont -> m_pPropCache -> Load(RecNum)))
        goto Quit;
    
Quit:
    Close(RecNum);
    gFreeBuff(pEID);

    if (FAILED(hRes))
    {
        delete pCont;
        return hRes;
    }

    // once for us
    m_pRootCont = pCont;
    m_pRootCont -> AddRef();
    
    *pulObjType = MAPI_ABCONT;
    *ppUnk = (LPUNKNOWN) pCont;
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    OpenEntry()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: OpenEntry(ULONG        cbEntryID,                  
                                   LPENTRYID    pEntryID,                  
                                   LPCIID       pInterface,                
                                   ULONG        ulFlags,                    
                                   ULONG  *     pulObjType,                
                                   LPUNKNOWN *  ppUnk)
{
    HRESULT         hRes      = S_OK;
    LPCABCont       pCont     = NULL;
    PMailUser       pMailUser = NULL;
    PABPAB_EID      pEID      = (PABPAB_EID) pEntryID,
                    pOOEID    = NULL;
    LPMAPITABLE     pTbl      = NULL;
    REFIID          rIID      = *pInterface;
    ULONG           ulEntryType;
    DECLARE_REC_NUM(RecNum);

    TraceString1("CABLogon :: OpenEntry %x",pEntryID ? pEID -> ID.Num : 0xfffffffe);
    CheckParameters_IABLogon_OpenEntry(this,
                                       cbEntryID, 
                                       pEntryID,  
                                       pInterface,
                                       ulFlags,   
                                       pulObjType,
                                       ppUnk);     
    
    if (NULL               != pInterface          &&
        IID_IMAPIContainer != (REFIID)*pInterface &&
        IID_IABContainer   != (REFIID)*pInterface &&
        IID_IDistList      != (REFIID)*pInterface &&
        IID_IMailUser      != (REFIID)*pInterface &&
        IID_IMAPIProp      != (REFIID)*pInterface)
        return MAPI_E_INTERFACE_NOT_SUPPORTED;
    
    // Validate the EID is one of ours
    // if it's NULL *pulObjType <- MAPI_ABCONT
    // else *pulObjType <- MAPI_DISTLIST or MAPI_MAILUSER
    ulEntryType = CheckEID(cbEntryID,pEntryID);
    EnterCriticalSection(&m_cs);

    switch (ulEntryType)
    {
        case MY_ABCONT:
        case MY_DISTLIST:
        case MY_ONEOFF_DL:
            if (!pEID || ROOT_ID_NUM == pEID -> ID.Num) 
            {
                if (m_pRootCont)
                {
                    m_pRootCont -> AddRef();                
                    *ppUnk = (LPUNKNOWN)(LPCABCont) m_pRootCont; 
                    *pulObjType = MAPI_ABCONT;            
                }
                else
                    M_FAIL(hRes = OpenRoot(pulObjType,ppUnk));
                break;
            } 
            
            if (DIR_ID_NUM == pEID -> ID.Num)
                if (m_pDirCont)
                {
                    m_pDirCont -> AddRef();                
                    *ppUnk = (LPUNKNOWN)(LPCABCont) m_pDirCont; 
                    *pulObjType = MAPI_ABCONT;    
                    break;
                }
            M_FAIL(hRes = OpenContainer(pEID,pulObjType,ppUnk));
            break;
                             
        case FOREIGN_DL:
        case TEMPLATE_DL:
        {
            CTemplateDL * pTemplDL;

            if (M_FAIL(hRes = OpenForeignDL(pEID, &pTemplDL)))
                break;
            else
            {
                *pulObjType = MAPI_DISTLIST;
                *ppUnk = pTemplDL;
            }
            break;
        }
        case FOREIGN_USER:
        case TEMPLATE_USER:
        {
            CTemplateUser * pTemplUser;

            if (M_FAIL(hRes = OpenForeignUser(pEID, &pTemplUser)))
                break;
            else
            {
                *pulObjType = MAPI_MAILUSER;
                *ppUnk    = pTemplUser;
            }
            break;                  
        }    

        case MY_ONEOFF_USER:
        case MY_MAILUSER:
            pMailUser = new CMailUser(this,NULL,(PABPAB_EID) pEID);
            
            if (!pMailUser)
               return MAPI_E_NOT_ENOUGH_MEMORY;

            if (M_FAIL(hRes = pMailUser -> Init()))
            {
               delete pMailUser;
               return hRes;
            }
           
            if (MY_MAILUSER == ulEntryType)
            {
                if (M_FAIL(hRes = Open(m_pDB,
                                       OPEN_ONLY, 
                                       pEID -> ID,
                                       RecNum)))
                {
                    delete pMailUser;
                    return hRes;
                }

                hRes = pMailUser -> m_pPropCache -> Load(RecNum);                
                Close(RecNum);

                if (M_FAIL(hRes))
                {
                    delete pMailUser;
                    return hRes;
                }
            }            
            *ppUnk = (LPUNKNOWN)pMailUser;
            *pulObjType = MAPI_MAILUSER;            

            return S_OK;

        default:            
            M_FAIL(hRes = MAPI_E_INVALID_ENTRYID);        
    }

    LeaveCriticalSection(&m_cs);

    if (pOOEID)
        gFreeBuff(pOOEID);

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    CompareEntryIDs()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: CompareEntryIDs(ULONG       cbEntryID1,                 
                                         LPENTRYID   lpEntryID1,                 
                                         ULONG       cbEntryID2,                 
                                         LPENTRYID   lpEntryID2,                 
                                         ULONG       ulFlags,                    
                                         ULONG  *    lpulResult)
{
    PABPAB_EID  lhs = (PABPAB_EID) lpEntryID1,
                rhs = (PABPAB_EID) lpEntryID2;

    TraceMessage("CABLogon :: CompareEntryIDs");
    CheckParameters_IABLogon_CompareEntryIDs(this,
                                             cbEntryID1,
                                             lpEntryID1,
                                             cbEntryID2,
                                             lpEntryID2,
                                             ulFlags,   
                                             lpulResult);
    
    *lpulResult = (*lhs ==*rhs);        
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Advise()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: Advise(ULONG             cbEntryID,                  
                                LPENTRYID         lpEntryID,                  
                                ULONG             ulEventMask,                
                                LPMAPIADVISESINK  lpAdviseSink,               
                                ULONG  *          lpulConnection)
{
    HRESULT     hRes = S_OK;
    PABPAB_EID  pEID = (PABPAB_EID) lpEntryID;
    ULONG       ulCnx;
    SizedNOTIFKEY(sizeof ABPAB_EID, Key);
    
    TraceString1("CABLogon::Advise on object %x",pEID->ID.Num);
    CheckParameters_IABLogon_Advise(this,
                                    cbEntryID,     
                                    lpEntryID,     
                                    ulEventMask,   
                                    lpAdviseSink,  
                                    lpulConnection);

    EnterCriticalSection(&m_cs);

    // Make sure entryID is one of mine
    hRes = CheckEID(cbEntryID, lpEntryID);
    if (FOREIGN_ENTRY == hRes || MAPI_E_INVALID_ENTRYID == hRes)
    {
        M_FAIL(hRes = MAPI_E_INVALID_ENTRYID);
        goto Quit;
    }
       
    // Make sure it's an event we can support
    if (ulEventMask & ~SUPPORTED_EVENTS)
    {
        M_FAIL(hRes = MAPI_E_NO_SUPPORT);
        goto Quit;
    }

    Key.cb = cbEntryID;
    memcpy((LPVOID)Key.ab,(LPVOID)lpEntryID,Key.cb);

    // let MAPI do the work
    if (M_FAIL(hRes = m_pSupObj -> Subscribe((LPNOTIFKEY)& Key,
                                             ulEventMask,
                                             0,
                                             lpAdviseSink,  
                                             & ulCnx)))
        goto Quit;

    // save the cnx number
    if (! m_pAdviseList -> Queue(ulCnx))
    {
        M_FAIL(hRes = MAPI_E_NOT_ENOUGH_MEMORY);
        goto Quit;
    }

    *lpulConnection = ulCnx;    
Quit:
    LeaveCriticalSection(&m_cs);
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Unadvise()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: Unadvise(ULONG ulConnection)
{
    BOOL         bRes;
    ULONG      * pulCnx;

    TraceString1("CABLogon::Unadvise on cnx %x",ulConnection);
    CheckParameters_IABLogon_Unadvise(this,ulConnection);

    EnterCriticalSection(&m_cs);    
    bRes = m_pAdviseList -> Delete(ulConnection,&pulCnx);
    LeaveCriticalSection(&m_cs);

    if (bRes && pulCnx)
    {
        m_pSupObj -> Unsubscribe(*pulCnx);
        delete pulCnx;
        return S_OK;
    }

    M_FAIL(MAPI_E_CALL_FAILED);
    return MAPI_E_CALL_FAILED;
}

///////////////////////////////////////////////////////////////////////////////
//
//    OpenStatusEntry()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: OpenStatusEntry(LPCIID         lpInterface,                
                                         ULONG          ulFlags,                    
                                         ULONG  *       lpulObjType,                
                                         LPMAPISTATUS * lppEntry)
{
    TraceMessage("CABLogon :: OpenStatusEntry");
    CheckParameters_IABLogon_OpenStatusEntry(this,
                                             lpInterface,
                                             ulFlags,    
                                             lpulObjType,
                                             lppEntry);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//
//    OpenTemplateID()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: OpenTemplateID(ULONG         cbTemplateID,               
                                        LPENTRYID     lpTemplateID,               
                                        ULONG         ulTemplateFlags,            
                                        LPMAPIPROP    lpMAPIPropData,             
                                        LPCIID        lpInterface,                
                                        LPMAPIPROP *  lppMAPIPropNew,             
                                        LPMAPIPROP    lpMAPIPropSibling)
{
    TraceMessage("CABLogon :: OpenTemplateID");
    CheckParameters_IABLogon_OpenTemplateID(this,
                                            cbTemplateID,     
                                            lpTemplateID,     
                                            ulTemplateFlags,  
                                            lpMAPIPropData,   
                                            lpInterface,      
                                            lppMAPIPropNew,   
                                            lpMAPIPropSibling);

    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//
//    CreateEID()
//
//    Parameters                        
//          ulObjType   [in/out]  oneoff object's type
//          ppEID       [out]     ptr to ptr to newly created entryID
//
//    Purpose
//          Creates a new entry ID for a oneoff that's about to be added
//          to our database. The oneoff's entry ID is hardcoded. Here
//          we create a new entry ID that identifies a new record in the
//          database.
//
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//

STDMETHODIMP CABLogon :: CreateEID(ULONG & ulEntryType, PABPAB_EID * ppEID)
{
    HRESULT  hRes = S_OK;
    ULONG    ulID;

    *ppEID = NULL;

    if (FAILED(hRes = NextID(m_pDB,ulID)))
        return hRes;
    
    ulEntryType &= ~ONEOFF_ENTRY;

    return NewEID(0,ulID,ulEntryType,NULL,ppEID);
}


///////////////////////////////////////////////////////////////////////////////
//
//    NewEID()
//
//    Parameters                        
//          ulFlags     [in]  entry ID abflags 
//          ulID        [in]  object's database ID number
//          ulEntryType [in]  object's type
//          pParent     [in]  block to link allocation to, if non-
//                            NULL, EID is allocated MAPIAllocMore
//          ppNewEID    [out] pointer to pointer where we write new EID
//
//    Purpose
//          Creates a new entry ID. The allocation can be linked to another
//          block of memory if pParent is non-NULL (a la MAPIAllocateMore)
//
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: NewEID(ULONG           ulFlags,
                                ULONG           ulID,
                                ULONG           ulEntryType,
                                LPVOID          pParent,
                                PABPAB_EID    * ppNewEID)
{
    HRESULT      hRes;
    PABPAB_EID   pEID = NULL;

    if (pParent)
        hRes = gAllocMore(sizeof ABPAB_EID, pParent, (LPVOID *) &pEID);
    else
        hRes = gAllocBuff(sizeof ABPAB_EID,(LPVOID *) &pEID);

    if (SUCCEEDED(hRes))
    {
        ZeroMemory((LPVOID) pEID, sizeof ABPAB_EID);
        memcpy((LPVOID) &(pEID -> muid), (LPVOID)& m_UID, sizeof MAPIUID);
        memcpy((LPVOID) pEID -> bFlags, (LPVOID) &ulFlags, sizeof ULONG);            
        pEID -> ID.Num = ulID;
        pEID -> ID.Type = ulEntryType;
        *ppNewEID = pEID;        
    }
    return hRes;    
}

///////////////////////////////////////////////////////////////////////////////
//
//    GetOneOffTable()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          Returns our two templates: a user and DL template   
//          IMAPISuport::GetOneOffTable calls each provider's   
//          IABLogon::GetOneOffTable to create the merged oneoff
//          table. We contribute our two rows here.             
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CABLogon :: GetOneOffTable(ULONG         ulFlags,                    
                                        LPMAPITABLE * lppTable)
{
    TraceMessage("CABLogon :: GetOneOffTable");
    CheckParameters_IABLogon_GetOneOffTable(this,ulFlags,lppTable);

    HRESULT      hRes;
    SRow         sRow;
    ULONG        i;
    LPTABLEDATA  pMyTbl = NULL;
    PABPAB_EID   pEID   = NULL;


    enum {NAME,EID,DEPTH,SELECT,ADDR_TYPE,DISP_TYPE,INST_KEY,NUM_ONEOFF_PROPS};
    static const SizedSPropTagArray(NUM_ONEOFF_PROPS, OneOffTags) =
    {
        NUM_ONEOFF_PROPS,
        {
            PR_DISPLAY_NAME,
            PR_ENTRYID,
            PR_DEPTH,
            PR_SELECTABLE,
            PR_ADDRTYPE,
            PR_DISPLAY_TYPE,
            PR_INSTANCE_KEY
        }
    };

    SPropValue rgsPropValue[NUM_ONEOFF_PROPS];
    
    if (ulFlags & MAPI_UNICODE)
        return MAPI_E_BAD_CHARWIDTH;


    EnterCriticalSection(&m_cs);

    if (M_FAIL(hRes = CreateTable((LPIID) &IID_IMAPITableData,
                                  gAllocBuff,
                                  gAllocMore,
                                  gFreeBuff,
                                  NULL,
                                  TBLTYPE_DYNAMIC,
                                  PR_INSTANCE_KEY,
                                  (LPSPropTagArray) &OneOffTags,
                                  &pMyTbl)))
        goto Quit;

    sRow.cValues = NUM_ONEOFF_PROPS;
    sRow.lpProps = rgsPropValue;
    
    for (i = NAME; i < NUM_ONEOFF_PROPS; i++)
        rgsPropValue[i].ulPropTag = OneOffTags.aulPropTag[i];
    
    if (M_FAIL(hRes = NewEID(0,
                             ONEOFF_USER_ID,
                             MY_MAILUSER | ONEOFF_ENTRY,
                             NULL,
                             &pEID)))
        goto Quit;                                

    rgsPropValue[NAME].Value.LPSZ        = "ABPAB Address";
    rgsPropValue[EID].Value.bin.cb       = sizeof(ABPAB_EID);
    rgsPropValue[EID].Value.bin.lpb      = (LPBYTE) pEID;
    rgsPropValue[DEPTH].Value.l          = 0;
    rgsPropValue[SELECT].Value.b         = TRUE;
    rgsPropValue[ADDR_TYPE].Value.LPSZ   = "";
    rgsPropValue[DISP_TYPE].Value.l      = DT_MAILUSER;
    rgsPropValue[INST_KEY].Value.bin.cb  = sizeof(ULONG);
    rgsPropValue[INST_KEY].Value.bin.lpb = (LPBYTE) &pEID -> ID.Num; 

    if (M_FAIL(hRes =  pMyTbl->HrModifyRow(&sRow)))
        goto Quit;

    gFreeBuff(pEID);
    pEID = NULL;

    if (M_FAIL(hRes = NewEID(0,
                             ONEOFF_DL_ID,
                             MY_DISTLIST | ONEOFF_ENTRY,
                             NULL,
                             &pEID)))
        goto Quit;

    rgsPropValue[EID].Value.bin.lpb      = (LPBYTE) pEID;
    rgsPropValue[NAME].Value.LPSZ        = "ABPAB Distribution List";
    rgsPropValue[ADDR_TYPE].Value.LPSZ   = "MAPIPDL";
    rgsPropValue[DISP_TYPE].Value.l      = DT_DISTLIST;
    rgsPropValue[INST_KEY].Value.bin.lpb = (LPBYTE) &pEID -> ID.Num;
    
    if (M_FAIL(hRes =  pMyTbl->HrModifyRow(&sRow)))
        goto Quit;

    hRes = pMyTbl->HrGetView(NULL,
                             NULL,
                             0,
                             (LPMAPITABLE *) lppTable);
Quit:
    LeaveCriticalSection(&m_cs);

    gFreeBuff(pEID);

    if (pMyTbl)
        pMyTbl -> Release();

    if (FAILED(hRes))
        if (*lppTable)
            (*lppTable) -> Release();

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    OpenTemplates()
//
//    Parameters                        
//          ulFlags     [in]  table access flags
//          lppTable    [out] pointer to pointer to hold new oneoff table
//    Purpose
//          Creates a merged oneoff table containing our native 
//          templates plus all foreign templates in the session.
//
//          We let anyone put entries in us, so our oneoff table is a 
//          merge of the session table (with the other AB's templates) 
//          plus our own. This is a little tricky. Other ABs call 
//          IMAPISupport::GetOneOffTable to get our templates so they 
//          can put entries of our type in their databases.
//
//          Our IABLogon::GetOneOffTable gets called --once-- to 
//          create the merged table.
//
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: OpenTemplates(ULONG         ulFlags,                    
                                       LPMAPITABLE * lppTable)
{
    TraceMessage("CABLogon :: OpenTemplates");

    HRESULT      hRes     = S_OK;
    LPTABLEDATA  pMyTbl   = NULL;
    LPMAPITABLE  pSessTbl = NULL;
    PABPAB_EID   pEID     = NULL;
    LPSRowSet    pRows    = NULL;
    ULONG        ulRow    = 0,
                 ulRecipID,
                 ulDLID,
                 i;
    SRow         sRow;

    enum {NAME,EID,DEPTH,SELECT,ADDR_TYPE,DISP_TYPE,INST_KEY,ROWID,NUM_ONEOFF_PROPS};
    static const SizedSPropTagArray(NUM_ONEOFF_PROPS, OneOffTags) =
    {
        NUM_ONEOFF_PROPS,
        {
            PR_DISPLAY_NAME,
            PR_ENTRYID,
            PR_DEPTH,
            PR_SELECTABLE,
            PR_ADDRTYPE,
            PR_DISPLAY_TYPE,
            PR_INSTANCE_KEY,
            PR_ROWID
        }
    };

    SPropValue rgsPropValue[NUM_ONEOFF_PROPS];

    if (FAILED(hRes = CreateTable((LPIID) &IID_IMAPITableData,
                                   gAllocBuff,
                                   gAllocMore,
                                   gFreeBuff,
                                   NULL,
                                   TBLTYPE_DYNAMIC,
                                   PR_ROWID,
                                   (LPSPropTagArray) &OneOffTags,
                                   &pMyTbl)))
            goto Quit;

    if (FAILED(hRes = m_pSupObj -> GetOneOffTable(ulFlags, &pSessTbl)))
        goto Quit;

    if (FAILED(hRes = HrQueryAllRows(pSessTbl,
                                     (LPSPropTagArray) &OneOffTags,
                                     NULL,
                                     NULL,
                                     0,
                                     &pRows)))
        goto Quit;

    // copy all rows from other providers, delete any from me
    for (i = 0; i < pRows -> cRows; i++)
    {
        PABPAB_EID      pEID;
        pEID  = (PABPAB_EID) pRows-> aRow[i].lpProps[EID].Value.bin.lpb;

        // add all rows that aren't mine untouched
        if (pEID -> muid !=  m_UID)
        {
            pRows -> aRow[i].lpProps[ROWID].Value.l = ulRow++;

            if (FAILED(hRes =  pMyTbl->HrModifyRow(&pRows->aRow[i])))
                goto Quit;
        }
    }

    sRow.cValues = NUM_ONEOFF_PROPS;
    sRow.lpProps = rgsPropValue;

    rgsPropValue[NAME].ulPropTag = PR_DISPLAY_NAME;
    rgsPropValue[NAME].Value.LPSZ = "ABPAB Address";
   
    if (FAILED(hRes = NewEID(0,
                             ONEOFF_USER_ID,
                             MY_MAILUSER | ONEOFF_ENTRY,
                             NULL,
                             &pEID)))
        goto Quit;                                
  
    rgsPropValue[EID].ulPropTag          = PR_ENTRYID;
    rgsPropValue[EID].Value.bin.cb       = sizeof(ABPAB_EID);
    rgsPropValue[EID].Value.bin.lpb      = (LPBYTE) pEID;
                                         
    rgsPropValue[DEPTH].ulPropTag        = PR_DEPTH;
    rgsPropValue[DEPTH].Value.l          = 0;
                                      
    rgsPropValue[SELECT].ulPropTag       = PR_SELECTABLE;
    rgsPropValue[SELECT].Value.b         = TRUE;

    rgsPropValue[ADDR_TYPE].ulPropTag    = PR_ADDRTYPE;
    rgsPropValue[ADDR_TYPE].Value.LPSZ   = "";

    rgsPropValue[DISP_TYPE].ulPropTag    = PR_DISPLAY_TYPE;
    rgsPropValue[DISP_TYPE].Value.l      = DT_MAILUSER;

    rgsPropValue[INST_KEY].ulPropTag     = PR_INSTANCE_KEY;
    rgsPropValue[INST_KEY].Value.bin.cb  = sizeof(ULONG);
    rgsPropValue[INST_KEY].Value.bin.lpb = (LPBYTE) &ulRecipID;

    rgsPropValue[ROWID].ulPropTag        = PR_ROWID;
    rgsPropValue[ROWID].Value.l          = ulRow++; 

    if (FAILED(hRes =  pMyTbl->HrModifyRow(&sRow)))
        goto Quit;

    gFreeBuff(pEID);
    pEID = NULL;

    if (FAILED(hRes = NewEID(0,
                             ONEOFF_DL_ID,
                             MY_DISTLIST | ONEOFF_ENTRY,
                             NULL,
                             &pEID)))
        goto Quit;

    rgsPropValue[EID].Value.bin.lpb         = (LPBYTE) pEID;
    rgsPropValue[NAME].Value.LPSZ           = "ABPAB Distribution List";

    rgsPropValue[ADDR_TYPE].Value.LPSZ      = "MAPIPDL";
    rgsPropValue[DISP_TYPE].Value.l         = DT_DISTLIST;
    rgsPropValue[INST_KEY].Value.bin.lpb    = (LPBYTE) &ulDLID;
    rgsPropValue[ROWID].Value.l             = ulRow; 
    
    if (FAILED(hRes =  pMyTbl->HrModifyRow(&sRow)))
        goto Quit;

    hRes = pMyTbl->HrGetView(NULL,
                             NULL,
                             0,
                             (LPMAPITABLE *) lppTable);
Quit:
    gFreeBuff(pEID);
    FreeProws(pRows);

    if (pMyTbl)
        pMyTbl -> Release();

    if (pSessTbl)
        pSessTbl -> Release();

    if (FAILED(hRes))
        if (*lppTable)
            (*lppTable) -> Release();

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    AE_Delete()
//
//    Parameters                        
//          ulTag       [in]     property tag of entry to delete   
//          pAdrEntry   [in]     ADRENTRY to delete
//          pParent     [in]     parent block to link copied property's
//                               new allocation to
//          pspNew      [in/out] pointer to an SPropValue we copy the new
//                               property to.
//    Purpose
//          Scans an ADRENTRY's properties for ulTag and if found  copies the
//          found SPropValue into pspNew (allocated by the caller). If pParent 
//          is non-null, any additional memory (e.g. PT_BINARY) is linked to 
//          the pParent. The found property in pAdrEntry is marked as deleted.
//
//          If the tag isn't found, the ADRENTRY is left alone and the pspNew
//          if marked as PT_ERROR 
//      
//    Return Value
//          none.
//
void AE_Delete(ULONG        ulTag, 
               LPADRENTRY   pAdrEntry,
               LPVOID       pParent,
               LPSPropValue pspNew)
{
    LPSPropValue pspFound = PpropFindProp(pAdrEntry -> rgPropVals,
                                          pAdrEntry -> cValues,
                                          ulTag);

    if (pspFound)
    {
        if (!pspNew)
        {
            pspFound -> ulPropTag = (ULONG) MAPI_E_OBJECT_DELETED;      
            return;
        }

        if (SUCCEEDED(CopyProp(pspNew, pspFound, (LPSPropValue) pParent)))
            pspFound -> ulPropTag = (ULONG) MAPI_E_OBJECT_DELETED;      
        else
            pspNew -> ulPropTag = CHANGE_PROP_TYPE(ulTag,PT_ERROR);
    }
    else
        if (pspNew)
            pspNew -> ulPropTag = CHANGE_PROP_TYPE(ulTag,PT_ERROR);
}

///////////////////////////////////////////////////////////////////////////////
//
//    PrepareRecips()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: PrepareRecips(ULONG           ulFlags,                    
                                       LPSPropTagArray lpPropTagArray,             
                                       LPADRLIST       lpRecipList)
{
    HRESULT         hRes = S_OK;
    CFlatPropList   fplDBProps;
    ULONG           i;
                                                    
    TraceMessage("CABLogon :: PrepareRecips");
    CheckParameters_IABLogon_PrepareRecips(this,
                                           ulFlags,       
                                           lpPropTagArray,
                                           lpRecipList);   
    // no work to do
    if (!lpPropTagArray)
        return hRes;            

    for (i = 0; i < lpRecipList -> cEntries; i++)
    {
                        
        PABPAB_EID      pEID    = NULL;
        LPSPropValue    pProps  = NULL,
                        pspvTmp = NULL;
        ULONG           cAEVals = lpRecipList -> aEntries[i].cValues,
                        cPTVals = lpPropTagArray -> cValues,
                        ulCnt   = 0,
                        j,
                        k;

        DECLARE_REC_NUM(RecNum);

        // Exchange always passes us an adrlist of 5 
        // (1 valid, 4 invalid) entries for a single recipient
        if (!cAEVals)
            goto NextEntry;
                    
        pspvTmp = PpropFindProp(lpRecipList -> aEntries[i].rgPropVals,
                                cAEVals,
                                PR_ENTRYID);
        if (pspvTmp)
            pEID = (PABPAB_EID) pspvTmp -> Value.bin.lpb;

        // no entry ID, bail
        if (!pEID)
        {
            hRes = MAPI_E_INVALID_ENTRYID;
            goto NextEntry;
        }
         
        // make sure it's ours and a valid recipient
        switch(CheckEID(pspvTmp -> Value.bin.cb,
                        (LPENTRYID)pspvTmp ->Value.bin.lpb))
        {
           case MY_MAILUSER:
           case MY_DISTLIST:
           case FOREIGN_DL   :
           case TEMPLATE_DL  :
           case FOREIGN_USER :
           case TEMPLATE_USER:
               break;

           default:         // not mine, skip it
               continue;
        }

        // open this entry's DB record
        if (FAILED(hRes = Open(m_pDB,OPEN_ONLY,pEID -> ID,RecNum)))
            goto NextEntry;
        
        // get stored properties
        if (FAILED(hRes = Fetch(RecNum, fldPROPS, & fplDBProps)))
            goto NextEntry;
        
        // allocate a SPV array big enough for worst case
        if (FAILED(hRes = gAllocBuff((cAEVals + cPTVals) * sizeof SPropValue,
                                     (LPVOID *) &pProps)))
            goto NextEntry;

        // get requested props from database and put at front of list.
        // If it's not in the database, see if it's in the ADRENTRY
        for (j = 0; j < lpPropTagArray -> cValues; j++)
        {
            PFlatPropNode pFlatPropNode = NULL;
            
            fplDBProps.Delete(lpPropTagArray -> aulPropTag[j], &pFlatPropNode);
            
            if (pFlatPropNode)              // we found it in the database
            {
                if (FAILED(UnFlatten(pFlatPropNode,
                                     pProps,
                                     pProps[j])))
                    pProps[j].ulPropTag = CHANGE_PROP_TYPE(pProps[j].ulPropTag,PT_ERROR);
                
                AE_Delete(lpPropTagArray -> aulPropTag[j],  // updates pProps to PT_ERROR if not found
                          &lpRecipList -> aEntries[i], 
                          NULL, 
                          NULL);

                delete pFlatPropNode;
            }
            else
                 AE_Delete(lpPropTagArray -> aulPropTag[j],  // updates pProps to PT_ERROR if not found
                           &lpRecipList -> aEntries[i], 
                           pProps, 
                           &pProps[j]);
            ulCnt++;
        }

        // add original props from ADRENTRY that weren't requested
        for (k = 0; k < cAEVals; k++)
            if (MAPI_E_OBJECT_DELETED != lpRecipList -> aEntries[i].rgPropVals[k].ulPropTag)
                AE_Delete(lpRecipList -> aEntries[i].rgPropVals[k].ulPropTag, 
                          &lpRecipList -> aEntries[i], 
                          pProps,
                          &pProps[ulCnt++]);

        gFreeBuff((LPVOID)lpRecipList -> aEntries[i].rgPropVals);
        lpRecipList -> aEntries[i].rgPropVals = pProps;
        lpRecipList -> aEntries[i].cValues    = ulCnt;

NextEntry:

        Close(RecNum);
        fplDBProps.Destroy();
        
        // invalidate this recipient's entry
        if (FAILED(hRes))
        {
            for (j = 0; j < lpRecipList -> aEntries[i].cValues; j++)
                CHANGE_PROP_TYPE(lpRecipList -> aEntries[i].rgPropVals[j].ulPropTag,
                                 PT_ERROR);
            gFreeBuff(pProps);
        }
    }
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Notify()
//
//    Parameters                        
//          ulEvent         [in] notification type
//          pParentEID      [in] pointer to object's parent entry ID
//          pObjectEID      [in] pointer to object's entry ID
//          pPropTags       [in] array of prop tags for changed properties
//
//    Purpose
//          This function creates a notification key (the entry ID of the
//          object for whom notifications were Advise()'d) then creates
//          a single notifcation struct and calls the support object's
//          Notify to broadcast the notification.
//      
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: Notify(ULONG           ulEvent, 
                                PABPAB_EID      pParentEID,
                                PABPAB_EID      pObjectEID,
                                LPSPropTagArray pPropTags)
{
    HRESULT       hRes    = S_OK;
    ULONG         ulFlags = 0;
    NOTIFICATION  Notif;
    SizedNOTIFKEY(sizeof ABPAB_EID, Key);

    // make a key
    Key.cb = sizeof ABPAB_EID;
    memcpy(Key.ab, (LPBYTE) pObjectEID, Key.cb);
    ZeroMemory(&Notif,sizeof Notif);

    Notif.ulEventType             = ulEvent;
    Notif.info.obj.cbEntryID      = sizeof *pObjectEID;
    Notif.info.obj.lpEntryID      = (LPENTRYID) pObjectEID;
    Notif.info.obj.ulObjType      = OBJECT_TYPE(pObjectEID -> ID.Type);
    Notif.info.obj.cbParentID     = pParentEID ? sizeof *pParentEID : 0;
    Notif.info.obj.lpParentID     = (LPENTRYID) pParentEID;
    Notif.info.obj.lpPropTagArray = pPropTags;

    // let MAPI do the notifications
    return m_pSupObj -> Notify((LPNOTIFKEY)&Key,
                               1,
                               &Notif,
                               &ulFlags);
}

///////////////////////////////////////////////////////////////////////////////
//
//    ~CUserDetails()
//
//    Parameters                        
//          none.
//
//    Purpose
//          Class destructor.
//      
//      
//    Return Value
//          none.
CUserDetails :: ~CUserDetails()
{
}

///////////////////////////////////////////////////////////////////////////////
//
//    CUserDetails()
//
//    Parameters                        
//          hInst   [in] module instance handle
//
//    Purpose
//          Class constructor. This object wraps display table
//          data structures and has a single method for returning
//          a details display table view.
//      
//    Return Value
//          none.

CUserDetails :: CUserDetails(HINSTANCE hInst, LPDTPAGE pUserPage)
{
    m_hInst     = hInst;
    m_pPropData = NULL;
    m_pDispTbl  = NULL;
    m_pDTPage   = pUserPage;
}

///////////////////////////////////////////////////////////////////////////////
//
//    BuildDisplayTable()
//
//    Parameters                        
//          ppTbl   [out]  pointer to pointer where we write the
//                         details display table.
//
//    Purpose
//          Returns a display table for MAILUSER details page.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CUserDetails :: BuildDisplayTable(LPMAPITABLE * ppTbl)
{
    *ppTbl = NULL;    
    return :: BuildDisplayTable(gAllocBuff,
                                gAllocMore,
                                gFreeBuff,
                                NULL,
                                m_hInst,
                                NUM_PAGES,
                                m_pDTPage,
                                0,
                                ppTbl,       
                                NULL);
}

///////////////////////////////////////////////////////////////////////////////
//
//    Details()
//
//    Parameters                        
//          hInst        [in] module instance handle
//          pDetailsPage [in] pointer to a display table
//                            page struct describing details page
//                             
//    Purpose
//          Class constructor. Wraps a DTPAGE struct and BuildDisplayTable
//          with a base class. Users and DLs derive from this to build
//          their own details display tables.      
//      
//    Return Value
//          none.
//
CDetails :: CDetails(HINSTANCE  hInst,
                     LPDTPAGE   pDetailsPage)
{
    m_hInst        = hInst;
    m_pPropData    = NULL;
    m_pDispTbl     = NULL;
    m_pDetailsPage = pDetailsPage;
}

///////////////////////////////////////////////////////////////////////////////
//
//    BuildDisplayTable()
//
//    Parameters                        
//          ppTbl   [out]  pointer to pointer where we write the
//                         details display table.
//
//    Purpose
//          Returns a display table for DISTLIST details page.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CDLDetails :: BuildDisplayTable(LPMAPITABLE * ppTbl)
{
    
    *ppTbl = NULL;    
    return :: BuildDisplayTable(gAllocBuff,
                                gAllocMore,
                                gFreeBuff,
                                NULL,
                                m_hInst,
                                1,
                                m_pDetailsPage,
                                0,
                                ppTbl,       
                                NULL);
}

///////////////////////////////////////////////////////////////////////////////
//
//    BuildDisplayTable()
//
//    Parameters                        
//          ppTbl   [out]  pointer to pointer where we write the
//                         details display table.
//
//    Purpose
//          Returns a display table for a container's details page.
//          (root container)
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CContainerDetails :: BuildDisplayTable(LPMAPITABLE * ppTbl)
{
    *ppTbl = NULL;
       
    for (int i = DT_CTL_PATH; i <= DT_CTL_COMMENT; i++)
        // Disable the controls (make them read only)
        m_pDetailsPage -> lpctl[i].ulCtlFlags &= ~DT_EDITABLE;

    return :: BuildDisplayTable(gAllocBuff,
                                gAllocMore,
                                gFreeBuff,
                                NULL,
                                m_hInst,
                                1,
                                m_pDetailsPage,
                                0,
                                ppTbl,       
                                NULL);
}

///////////////////////////////////////////////////////////////////////////////
//
//    GetCTblRow()
//
//    Parameters                        
//          pFlatPropList   [in]  pointer to a FlatPropList with
//                                the properties for one row
//          ppColProps      [out] pointer to pointer where we writ
//                                an array of properties (columns) for
//                                the row.
//    Purpose
//          Utility function that gets an object's properties (DL, 
//          cont, or Mailuser) from its flat prop list and converts
//          them into an array of SPropValues with all the default
//          contents table columns. Plug the props into an 
//          SRow.rgPropVals for updating the contents table
//
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: GetCTblRow(PCFlatPropList   pFlatPropList,
                                    LPSPropValue *   ppColProps)
{

    HRESULT             hRes      = S_OK,
                        hr        = S_OK;
    PFlatPropNode       pFlatNode = NULL;
    LPSPropValue        pColProps = NULL;
    PABPAB_EID          pEID;
    int                 i;

    if (FAILED(hRes = gAllocBuff(NUM_CTBL_COLS * sizeof SPropValue, 
                                 (LPVOID *)  & pColProps)))
        goto Quit;
   
    ZeroMemory((LPVOID) pColProps,NUM_CTBL_COLS * sizeof SPropValue);

    // relies on order of CT_XX enums
    for (i = 0; i < NUM_CTBL_COLS; i++)
    {
        pFlatNode = pFlatPropList -> Find(g_sptCntTbl.aulPropTag[i]);

        if (pFlatNode)
            hRes = UnFlatten(pFlatNode, 
                             (LPVOID) pColProps, 
                             pColProps[i]);

        if (!pFlatNode || FAILED(hRes)) 
            hr = HrPropNotFound(pColProps[i],g_sptCntTbl.aulPropTag[i]);
    }                        

    if (PR_ENTRYID != pColProps[CT_EID].ulPropTag)
        hr = HrPropNotFound(pColProps[CT_INST_KEY],PR_INSTANCE_KEY);
    else
    {   
        // set the computed properties
        pEID = (PABPAB_EID) pColProps[CT_EID].Value.bin.lpb;
        pColProps[CT_INST_KEY].ulPropTag    = PR_INSTANCE_KEY;
        pColProps[CT_INST_KEY].Value.bin.cb = sizeof ULONG;

        // copy the ID from the entry ID to PR_INSTANCE_KEY
        if (SUCCEEDED(hRes = gAllocMore(sizeof ULONG, 
                                        (LPVOID) pColProps, 
                                        (LPVOID *) &pColProps[CT_INST_KEY].Value.bin.lpb)))
            *(ULONG *)pColProps[CT_INST_KEY].Value.bin.lpb = pEID -> ID.Num;
    }
Quit:
    if (FAILED(hRes))
        if (pColProps)
        {
            gFreeBuff((LPVOID) pColProps);
            pColProps = NULL;
        }

    *ppColProps = pColProps;

    // if no errors, return any warning, else return errors
    return (hRes != S_OK ? hRes : hr);
}                    

///////////////////////////////////////////////////////////////////////////////
//
//    GetMapTable
//
//    Parameters                        
//          ppTbl   pointer to the table view
//
//    Purpose
//          Creates a view of the name-to-ID mapping table.
//      
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CABLogon :: GetMapTable()
{
    HRESULT  hRes;
    SizedSSortOrderSet(1,SortOrder) = {1,0,0,{PR_MAP_ID,TABLE_SORT_ASCEND}};

    EnterCriticalSection(&m_MapCS);
    hRes = m_pMapTable -> HrGetView((LPSSortOrderSet) &SortOrder,
                                    NULL,
                                    NULL,
                                    &m_pMapView);
    LeaveCriticalSection(&m_MapCS);

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    operator !=()
//
//    Parameters                        
//     
//
//    Purpose
//      
//      
//      
//    Return Value
//          TRUE if lhs != rhs, FALSE otherwise
//

 inline BOOL operator !=(const MAPIUID & lhs, const MAPIUID & rhs)
 {
    return memcmp((LPVOID) & lhs, (LPVOID) & rhs, sizeof MAPIUID);

 }

 // End of file for IABLOGON.CPP




