///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      ITABLE.CPP      
//
//  Description
//      Wraps MAPI's IMAPITable implementation to allow Restrict on PR_ANR.
//
//  Author
//      Les Thaler
//
//  Revision: 1.00
//
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//
#include "abpab.h"
#include "list.inl"
#include "itable.h"

///////////////////////////////////////////////////////////////////////////////
//
//    CMAPITable()
//
//    Parameters                        
//          pTbl        [in] pointer to table we wrap
//          ulParent    [in] ID of object that owns this table
//    Purpose
//          Class constructor. This class wraps an IMAPITable so
//          we can intercept Restrict on PR_ANR
//      
//    Return Value
//          none.
//
CMAPITable :: CMAPITable(LPMAPITABLE pTbl, ULONG ulParent)
{
    TraceMessage("CMAPITable::CMAPITable");

    m_pTbl = pTbl;
    m_cRef = 0;
    m_ParentID = ulParent;
    InitializeCriticalSection(&m_cs);
}

///////////////////////////////////////////////////////////////////////////////
//
//    ~CMAPITable()
//
//    Parameters                        
//          none.
//
//    Purpose
//          Class destructor.      
//      
//    Return Value
//          none.
//
CMAPITable :: ~CMAPITable()
{
    TraceMessage("CMAPITable::~CMAPITable");

    m_pTbl -> Release(); 
    DeleteCriticalSection(&m_cs);
  
}

///////////////////////////////////////////////////////////////////////////////
//
//    GetLastError()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CMAPITable :: GetLastError(HRESULT             hResult,        
                                        ULONG               ulFlags,        
                                        LPMAPIERROR FAR *   lppMAPIError) 
{
    return m_pTbl -> GetLastError(hResult,        
                                  ulFlags,        
                                  lppMAPIError);   
}

///////////////////////////////////////////////////////////////////////////////
//
//    Advise()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: Advise(ULONG             ulEventMask,    
                                  LPMAPIADVISESINK  lpAdviseSink,   
                                  ULONG FAR *       lpulConnection) 
{
    TraceString1("Container %x CMAPITable::Advise",m_ParentID);

    return m_pTbl -> Advise(ulEventMask,    
                            lpAdviseSink,   
                            lpulConnection);
}

///////////////////////////////////////////////////////////////////////////////
//
//   Unadvise()
//
//    Parameters                        
//     
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: Unadvise(ULONG   ulConnection) 
{
    return m_pTbl -> Unadvise(ulConnection);
}

///////////////////////////////////////////////////////////////////////////////
//
//    GetStatus()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: GetStatus(ULONG FAR * lpulTableStatus,
                                     ULONG FAR * lpulTableType) 
{
    return m_pTbl -> GetStatus(lpulTableStatus,
                               lpulTableType); 

}

///////////////////////////////////////////////////////////////////////////////
//
//    SetColumns()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: SetColumns(LPSPropTagArray lpPropTagArray, 
                                      ULONG           ulFlags) 
{
    return m_pTbl -> SetColumns(lpPropTagArray, 
                                ulFlags); 
}

///////////////////////////////////////////////////////////////////////////////
//
//    QueryColumns()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: QueryColumns(ULONG                 ulFlags,        
                                        LPSPropTagArray FAR * lpPropTagArray) 
{
    return m_pTbl -> QueryColumns(ulFlags,        
                                  lpPropTagArray);

}

///////////////////////////////////////////////////////////////////////////////
//
//    GetRowCount()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: GetRowCount(ULONG       ulFlags,        
                                       ULONG FAR * lpulCount) 
{
    return m_pTbl -> GetRowCount(ulFlags,        
                                 lpulCount); 

}

///////////////////////////////////////////////////////////////////////////////
//
//    SeekRow()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: SeekRow(BOOKMARK   bkOrigin,       
                                   LONG       lRowCount,      
                                   LONG FAR * lplRowsSought) 
{
    return m_pTbl -> SeekRow(bkOrigin,       
                             lRowCount,      
                             lplRowsSought); 
}

///////////////////////////////////////////////////////////////////////////////
//
//    SeekRowApprox()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: SeekRowApprox(ULONG  ulNumerator,    
                                         ULONG  ulDenominator) 
{
    return m_pTbl -> SeekRowApprox(ulNumerator,    
                                   ulDenominator);
}

///////////////////////////////////////////////////////////////////////////////
//
//    QueryPosition()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: QueryPosition(ULONG FAR *  lpulRow,        
                                         ULONG FAR *  lpulNumerator,  
                                         ULONG FAR *  lpulDenominator) 
{
    return m_pTbl -> QueryPosition(lpulRow,        
                                   lpulNumerator,  
                                   lpulDenominator);
}

///////////////////////////////////////////////////////////////////////////////
//
//    FindRow()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: FindRow(LPSRestriction  lpRestriction,  
                                   BOOKMARK        bkOrigin,       
                                   ULONG           ulFlags) 
{
    return m_pTbl -> FindRow(lpRestriction,  
                             bkOrigin,       
                             ulFlags);
}

///////////////////////////////////////////////////////////////////////////////
//
//    Restrict()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          This gets called if we return MAPI_E_AMBIGUOUS for entries in 
//          ResolveName, and for the finder. We use the same initial search
//          algorithm as for ResolveName: we find the first token in the target
//          and do a content restriction for all substrings. This yields lots of
//          hits. We don't do the added step of parsing to filter out the spurious
//          hits because the user can select them from the UI. IAddrBook calls this
//          immediately after ResolveNames if we mark a name as ambiguous, but if
//          we return a single row here as the ambiguous match, it treats it as if
//          it were resolved. 
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CMAPITable :: Restrict(LPSRestriction  lpRestriction,  
                                    ULONG           ulFlags) 
{
    HRESULT         hRes = S_OK;
    LPSRestriction  pRes = lpRestriction;
    SRestriction    SRes;
    SPropValue      spTemp;

    TraceString1("CMAPITable :: Restrict parent %x",m_ParentID);
    Validate_IMAPITable_Restrict(this,lpRestriction,ulFlags);

    if (lpRestriction && (lpRestriction -> rt == RES_PROPERTY))
        if (PROP_ID(PR_ANR) == PROP_ID(lpRestriction -> 
                                       res.resProperty.ulPropTag))
        {

            TCHAR szBuf[MAX_PATH];
            LPSTR szTemp = szBuf;

            if (IsBadReadPtr((LPBYTE) lpRestriction -> res.resProperty.lpProp -> Value.lpszA, 
                             Cbtszsize(lpRestriction -> res.resProperty.lpProp -> Value.lpszA)))
                return MAPI_E_INVALID_PARAMETER;

            lstrcpyn(szTemp,
                     lpRestriction -> res.resProperty.lpProp -> Value.lpszA,
                     MAX_PATH);
            szTemp[MAX_PATH - 1] = '\0';

            spTemp.ulPropTag = PR_DISPLAY_NAME;
            spTemp.Value.lpszA = GetNextToken(&szTemp);

            SRes.rt = RES_CONTENT;
  	        SRes.res.resContent.ulPropTag    = PR_DISPLAY_NAME;
	        SRes.res.resContent.lpProp       = &spTemp;
            SRes.res.resContent.ulFuzzyLevel = FL_SUBSTRING | FL_IGNORECASE;
            pRes = &SRes;
        }

    hRes = m_pTbl -> Restrict(pRes, ulFlags);
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    CreateBookmark()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: CreateBookmark(BOOKMARK FAR *  lpbkPosition) 
{
    return m_pTbl -> CreateBookmark(lpbkPosition);
}

///////////////////////////////////////////////////////////////////////////////
//
//    FreeBookmark()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: FreeBookmark(BOOKMARK  bkPosition) 
{
    return m_pTbl -> FreeBookmark(bkPosition);
}

///////////////////////////////////////////////////////////////////////////////
//
//    SortTable()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: SortTable(LPSSortOrderSet  lpSortCriteria, 
                                     ULONG            ulFlags) 
{
    return m_pTbl -> SortTable(lpSortCriteria, 
                               ulFlags);
}

///////////////////////////////////////////////////////////////////////////////
//
//    QuerySortOrder()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: QuerySortOrder(LPSSortOrderSet FAR * lppSortCriteria) 
{
    return m_pTbl -> QuerySortOrder(lppSortCriteria); 
}

///////////////////////////////////////////////////////////////////////////////
//
//    QueryRows()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: QueryRows(LONG             lRowCount,      
                                     ULONG            ulFlags,        
                                     LPSRowSet FAR *  lppRows) 
{
    return m_pTbl -> QueryRows(lRowCount,      
                               ulFlags,        
                               lppRows);
}

///////////////////////////////////////////////////////////////////////////////
//
//    Abort()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: Abort()
{
    return m_pTbl -> Abort();
}

///////////////////////////////////////////////////////////////////////////////
//
//    ExpandRow()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: ExpandRow(ULONG            cbInstanceKey,  
                                     LPBYTE           pbInstanceKey,  
                                     ULONG            ulRowCount,     
                                     ULONG            ulFlags,        
                                     LPSRowSet FAR *  lppRows,       
                                     ULONG FAR *      lpulMoreRows) 
{
    return m_pTbl -> ExpandRow(cbInstanceKey,  
                               pbInstanceKey,  
                               ulRowCount,     
                               ulFlags,        
                               lppRows,       
                               lpulMoreRows); 
}

///////////////////////////////////////////////////////////////////////////////
//
//    CollapseRow()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: CollapseRow(ULONG       cbInstanceKey,  
                                       LPBYTE      pbInstanceKey,  
                                       ULONG       ulFlags,        
                                       ULONG FAR * lpulRowCount) 
{
    return m_pTbl -> CollapseRow(cbInstanceKey,  
                                 pbInstanceKey,  
                                 ulFlags,        
                                 lpulRowCount); 
}

///////////////////////////////////////////////////////////////////////////////
//
//    WaitForCompletion()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: WaitForCompletion(ULONG       ulFlags,                    
                                             ULONG       ulTimeout,      
                                             ULONG FAR * lpulTableStatus)
{
    return m_pTbl -> WaitForCompletion(ulFlags,                    
                                       ulTimeout,      
                                       lpulTableStatus); 
}

///////////////////////////////////////////////////////////////////////////////
//
//    GetCollapseState()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: GetCollapseState(ULONG        ulFlags,        
                                            ULONG        cbInstanceKey,  
                                            LPBYTE       lpbInstanceKey, 
                                            ULONG FAR *  lpcbCollapseState,  
                                            LPBYTE FAR * lppbCollapseState) 
{
    return m_pTbl -> GetCollapseState(ulFlags,        
                                      cbInstanceKey,  
                                      lpbInstanceKey, 
                                      lpcbCollapseState,  
                                      lppbCollapseState); 
}

///////////////////////////////////////////////////////////////////////////////
//
//    SetCollapseState()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//

STDMETHODIMP CMAPITable :: SetCollapseState(ULONG          ulFlags,        
                                            ULONG          cbCollapseState,
                                            LPBYTE         pbCollapseState,
                                            BOOKMARK FAR * lpbkLocation) 
{
    return m_pTbl -> SetCollapseState(ulFlags,        
                                      cbCollapseState,
                                      pbCollapseState,
                                      lpbkLocation);
}

///////////////////////////////////////////////////////////////////////////////
//
//    QueryInterface()
//
//    Parameters                        
//          See the OLE documentation for details.
//
//    Purpose
//          Returns an interface if supported.
//      
//      
//    Return Value
//          NOERROR on success, E_NOINTERFACE if the interface can't be
//          returned.
//

STDMETHODIMP CMAPITable::QueryInterface(REFIID   riid, 
                                        LPVOID * ppvObj)
{
    TraceMessage("CMAPITable :: QueryInterface");

    if (IsBadWritePtr(ppvObj,sizeof LPUNKNOWN))
        return MAPI_E_INVALID_PARAMETER;

    *ppvObj = NULL;

    // If this is one of the two IID return an interface pointer to it
    if (riid == IID_IUnknown || riid == IID_IMAPITable)
    {
        EnterCriticalSection(&m_cs);
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        m_cRef++;
        LeaveCriticalSection(&m_cs);
        return NOERROR;
    }

    return E_NOINTERFACE;
}

///////////////////////////////////////////////////////////////////////////////
//    AddRef()
//
//    Parameters
//          Refer to OLE Documentation on this method
//
//    Purpose
//          To increase the reference count of this object,
//          and of any contained (or aggregated) object within
//
//    Return Value
//      Usage count for this object
//
STDMETHODIMP_ (ULONG) CMAPITable::AddRef (void)
{   

    // Increment object's internal counter
    EnterCriticalSection(&m_cs);
    ULONG ulRefCount = ++m_cRef;
    TraceString2("CMAPITable :: AddRef parent %x ref %x",m_ParentID,m_cRef);
    LeaveCriticalSection(&m_cs);

    return ulRefCount;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPITable::Release()
//
//    Parameters
//      { Refer to OLE Documentation on this method }
//
//    Purpose
//      Decreases the usage count of this object and deletes
//      the object when the count reaches 0.
//
//    Return Value
//      Usage of this object
//
STDMETHODIMP_ (ULONG) CMAPITable::Release(void)
{
    ULONG ulRefCount;

    EnterCriticalSection(&m_cs);
    ulRefCount = --m_cRef;
    TraceString2("CMAPITable :: Release parent %x ref %x",m_ParentID,m_cRef);
    LeaveCriticalSection(&m_cs); 

    // If nobody is using this object, delete it
    if (0 == ulRefCount)
    {
        // Destroy the object
        delete this;        // don't need to leave the critical section
        return ulRefCount;  // its deleted when the object is destroyed
    }
        
    return ulRefCount;
}

// End of file for ITABLE.CPP
