///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//     UI.CPP 
//
//  Description
//      Processing for Details, Address and configuration dialog
//      button controls.
//
//  Author
//      Les Thaler
//
//  Revision: 1.00
//
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//
#include "abpab.h"
#include "abdb.h"
#include "list.inl"

///////////////////////////////////////////////////////////////////////////////
//
//    CMembersButton()
//
//    Parameters                        
//          pCont    [in] pointer to the parent container
//
//    Purpose
//          Class constructor
//      
//    Return Value
//          none.

CMembersButton :: CMembersButton(LPCABCont pCont)
{
    TraceMessage("CMembersButton::CMembersButton");
    m_pCont = pCont;
    m_pCont -> AddRef();
    m_cRef = 0;
    m_ulContext = 0;
    m_ulVersion = 0;   
    m_szError[0] = '\0';
    m_hInst = pCont -> m_pLogon -> m_hInst;
    InitializeCriticalSection(&m_cs);
}

///////////////////////////////////////////////////////////////////////////////
//
//    ~CMembersButton()
//
//    Parameters                        
//          none.
//
//    Purpose
//          Class destructor
//      
//    Return Value
//          none.
//
CMembersButton :: ~CMembersButton()
{
    TraceMessage("CMembersButton::~CMembersButton");

    m_pCont -> Release();
    DeleteCriticalSection(&m_cs);
}

///////////////////////////////////////////////////////////////////////////////
//
//    CMembersButton::AddRef()
//
//    Parameters
//          Refer to OLE Documentation on this method.
//
//    Purpose
//          To increase the reference count of this object, and of 
//          any contained (or aggregated) object within
//
//    Return Value
//          Usage count for this object
//
STDMETHODIMP_ (ULONG) CMembersButton::AddRef (void)
{   
    ULONG   ulRef;

    // Increment object's internal counter
    EnterCriticalSection(&m_cs);
    ulRef = ++m_cRef;
    TraceMessage("CMembersButton :: AddRef");
    LeaveCriticalSection(&m_cs);
       
    return ulRef;
}

///////////////////////////////////////////////////////////////////////////////
//
//    CMembersButton::Release()
//
//    Parameters
//          Refer to OLE Documentation on this method.
//
//    Purpose
//          Decrements usage count of this object. If count is zero,
//          object deletes itself.
//
//    Return Value
//          Usage of this object
//
STDMETHODIMP_ (ULONG) CMembersButton :: Release (void)
{
    ULONG ulRefCount;

    EnterCriticalSection(&m_cs);
    ulRefCount = --m_cRef;
    LeaveCriticalSection(&m_cs); 

    TraceString1("CMembersButton :: Release ref %x",ulRefCount);

    // If nobody is using this object, delete it
    if (0 == ulRefCount)
        // Destroy the object
        delete this;        
        
    return ulRefCount;
}        

///////////////////////////////////////////////////////////////////////////////
//
//    QueryInterface()
//
//    Parameters                        
//          See the OLE documentation for details.
//
//    Purpose
//          Returns an interface if supported.
//      
//      
//    Return Value
//          NOERROR on success, E_NOINTERFACE if the interface can't be
//          returned.
//

STDMETHODIMP CMembersButton :: QueryInterface(REFIID   riid, 
                                              LPVOID * ppvObj)
{
    TraceMessage("CMembersButton :: QueryInterface");

    if (IsBadWritePtr(ppvObj,sizeof LPUNKNOWN))
        return MAPI_E_INVALID_PARAMETER;

    *ppvObj = NULL;

    // If this is one of the two IID return an interface pointer to it
    if (riid == IID_IUnknown || riid == IID_IMAPIControl)
    {
        EnterCriticalSection(&m_cs);
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        m_cRef++;
        LeaveCriticalSection(&m_cs);
        return NOERROR;
    }

    return E_NOINTERFACE;
} 

///////////////////////////////////////////////////////////////////////////////
//
//    GetState()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK;
//
STDMETHODIMP CMembersButton :: GetState(ULONG ulFlags, ULONG * pulState)
{
    TraceMessage("CMembersButton :: GetState");
    *pulState = MAPI_ENABLED;

    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Activate()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          This method is called when the user hits the 
//          Add/Delete members button on a DL details or
//          create dialog. We use the IMAPISupport::Address
//          method to do most of the work. We pass Address an
//          ADRLIST of current members of the DL. When Address
//          returns we have an ADRLIST of members the user 
//          selected for the DL. We delete from the DL any entries
//          that are in the first list, but not the second, and
//          copy to the DL any entries in the second that aren't
//          in the first. We use DeleteEntries and CopyEntries 
//          to do the work.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CMembersButton :: Activate(ULONG ulFlags, ULONG ulUIParam)
{
    HRESULT         hRes        = S_OK;
    LPADRLIST       pAdrList    = NULL,
                    pAdrListNew = NULL;
    LPMAPITABLE     pTbl        = NULL;
    PABPAB_EID      pEID        = NULL;
    PABPAB_ID       pID         = NULL;
    LPSRow          pRow        = NULL;
    PMailUser       pNewEntry   = NULL;
    LPSBinary       pNewMembers = NULL;
    ULONG           k           = 0,
                    i;
    LPTSTR          szDestTitles[1] = {"&Members"};
    SBinaryArray    NewMemberList;
    ADRPARM         AdrParm;
    SPropValue      spInstKey;

    CList<ABPAB_ID> DeleteList;

    enum 
    {
        NAME,
        EID,
        ADDRTYPE,
        ADDR,
        DISP_TYPE,
        SEARCH_KEY,
        OBJTYPE,
        INST_KEY,
        RECIP_TYPE,
        NUM_COLS
    };

    static const SizedSPropTagArray(NUM_COLS, sptAdrList) =
    {
        NUM_COLS,
        {
            PR_DISPLAY_NAME,
            PR_ENTRYID,
            PR_ADDRTYPE,
            PR_EMAIL_ADDRESS,
            PR_DISPLAY_TYPE,
            PR_SEARCH_KEY,
            PR_OBJECT_TYPE,
            PR_INSTANCE_KEY,
            PR_NULL
        }
    };

    TraceMessage("CMembersButton :: Activate");

    AdrParm.cbABContEntryID    = sizeof ABPAB_EID;
    AdrParm.lpABContEntryID    = (LPENTRYID) &(m_pCont -> m_MyEID);
    AdrParm.ulFlags            = AB_RESOLVE | DIALOG_MODAL ;
    AdrParm.lpReserved         = 0;
    AdrParm.ulHelpContext      = 0;
    AdrParm.lpszHelpFileName   = "";
    AdrParm.lpfnABSDI          = NULL;
    AdrParm.lpfnDismiss        = NULL;
    AdrParm.lpvDismissContext  = NULL;
    AdrParm.lpszCaption        = "Edit New Personal Distribution List Members";
    AdrParm.lpszNewEntryTitle  = "&Just in this Personal Distribution List";
    AdrParm.lpszDestWellsTitle = "Personal Distribution List";
    AdrParm.cDestFields        = 1;
    AdrParm.nDestFieldFocus    = 0;
    AdrParm.lppszDestTitles    = szDestTitles;
    AdrParm.lpulDestComps      = NULL;
    AdrParm.lpContRestriction  = NULL;
    AdrParm.lpHierRestriction  = NULL;

    // get all the current members of this DL
    if (M_FAIL(hRes = m_pCont -> GetContentsTable(0,&pTbl)))
        goto Quit;

    if (M_FAIL(hRes = HrQueryAllRows(pTbl,
                                     (LPSPropTagArray) &sptAdrList,
                                     NULL,
                                     NULL,
                                     0,
                                     (LPSRowSet *) &pAdrList)))
        goto Quit;

    
    // set the PR_RECIPIENT_TYPE so Address is happy
    // save this DL's current children's IDs
    for (i = 0; i < pAdrList -> cEntries; i++)
    {
        pEID = (PABPAB_EID) pAdrList -> aEntries[i].rgPropVals[EID].Value.bin.lpb;

        // this is so scuzzy, but Address complains if it's not there        
        pAdrList -> aEntries[i].rgPropVals[RECIP_TYPE].ulPropTag = PR_RECIPIENT_TYPE;        
        pAdrList -> aEntries[i].rgPropVals[RECIP_TYPE].Value.l = MAPI_TO;

        if (! DeleteList.Queue(pEID -> ID))
        {
            hRes = MAPI_E_NOT_ENOUGH_MEMORY;
            goto Quit;
        }

    }
    
    // Address will let us add/delete members. pass the existing members
    // so it can display them in the right pane.
    if (M_FAIL(hRes = m_pCont   -> 
                      m_pLogon  -> 
                      m_pSupObj -> Address(&ulUIParam,&AdrParm,&pAdrList)))
        goto Quit;

    if (M_FAIL(hRes) || ! pAdrList)
        goto Quit;


    if (M_FAIL(hRes = gAllocBuff((sizeof SBinary) * (pAdrList -> cEntries), 
                                 (LPVOID *) & pNewMembers)))
        goto Quit; 
    
    NewMemberList.lpbin   = pNewMembers;    
    NewMemberList.cValues = 0;

    for (i = 0; i < pAdrList -> cEntries; i++)
    {   
        if (!pAdrList -> aEntries[i].cValues)
            continue;

        LPSPropValue pPropEID = PpropFindProp(pAdrList -> aEntries[i].rgPropVals,
                                              pAdrList -> aEntries[i].cValues,
                                              PR_ENTRYID);

        if (!pPropEID)
            continue;

        // get this entry's row from root contents, add to this container's table
        pEID = (PABPAB_EID) pPropEID -> Value.bin.lpb;                    
        pID  = NULL;

        // If the entry is still in the original list, it 
        // hasn't been deleted, so delete it from the 'to
        // delete' list.

        //!! THIS IS RISKY, a foreign entry could have the same ID
        //!! as one of mine, BETTER, to make the list a list of EIDs
        if (! DeleteList.Delete(pEID -> ID, &pID))
        {
            M_FAIL(hRes = MAPI_E_NOT_ENOUGH_MEMORY);
            goto Quit;
        }
        
        // this entry was in the original list, since its not new,
        // we don't need to add it a second time
        if (pID)
            delete pID;                
        else
        {
            // new entry, add to new member list
            NewMemberList.cValues++;                                        
            NewMemberList.lpbin[k++] =  pPropEID -> Value.bin;
        }
    }
    
    // update the database with my new relations
    if (M_FAIL(hRes = m_pCont -> CopyEntries(&NewMemberList,0,NULL,0)))
        goto Quit;

    spInstKey.ulPropTag = PR_INSTANCE_KEY;
    spInstKey.Value.bin.cb = sizeof ULONG;

    // update the table's data        
    for (pID = DeleteList.Scan(0);
         pID;
         pID = DeleteList.Scan())
    {
        spInstKey.Value.bin.lpb = (LPBYTE) &(pID -> Num);

        if (M_FAIL(hRes = m_pCont        -> 
                          m_pCntsTblData -> HrDeleteRow(&spInstKey)))
            goto Quit;    
    }
    
    hRes = m_pCont -> DeleteChildren(m_pCont -> m_MyEID.ID, &DeleteList);
    M_FAIL(hRes);

Quit:
    FreePadrlist(pAdrList);
    gFreeBuff((LPVOID) pNewMembers);

    if (pTbl)
        pTbl -> Release();
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    SetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Sets detailed error information based on hRes.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//

STDMETHODIMP CMembersButton :: SetLastError(HRESULT       hRes,    
                                            ULONG         ulLine,    
                                            LPSTR         szFile)
{
    if (FAILED(hRes))
    {
        EnterCriticalSection(&m_cs);
        m_ulContext = ulLine + IE_IABLOGON;
        lstrcpyn(m_szError,szFile,MAX_PATH);
        m_szError[MAX_PATH - 1] = '\0';        
        LeaveCriticalSection(&m_cs);
    }
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    GetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Gets detailed error information based on hResult.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//
STDMETHODIMP GetLastError(LPSTR szErr, LPSTR szComp,ULONG ulCtx,LPMAPIERROR * ppErr)
{
    HRESULT     hRes = S_OK;
    LPMAPIERROR pErr;
 
    if (FAILED(hRes = gAllocBuff(sizeof MAPIERROR, (LPVOID *) &pErr)))
        goto Quit;

    ZeroMemory((LPVOID) pErr,sizeof MAPIERROR);

    if (FAILED(hRes = gAllocMore(Cbtszsize(szErr),
                                 (LPVOID) pErr, 
                                 (LPVOID *)&pErr -> lpszError)))
        goto Quit;

    if (FAILED(hRes = gAllocMore(Cbtszsize(szComp),
                                 (LPVOID) pErr, 
                                 (LPVOID *)&pErr -> lpszComponent)))
        goto Quit;

    lstrcpy(pErr -> lpszError,szErr);
    lstrcpy(pErr -> lpszComponent,szComp);
    pErr -> ulContext = ulCtx;
Quit:
    if (FAILED(hRes))
        gFreeBuff(pErr);
    else
        *ppErr = pErr;
            
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    GetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Gets detailed error information based on hResult.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//
STDMETHODIMP CMembersButton :: GetLastError(HRESULT         hResult,                    
                                            ULONG           ulFlags,                    
                                            LPMAPIERROR  *  lppMAPIError)
{
    HRESULT hRes;
    TraceMessage("CMembersButton :: GetLastError");
    Validate_IMAPIControl_GetLastError(this,
                                       hResult,     
                                       ulFlags,     
                                       lppMAPIError);
 
    EnterCriticalSection(&m_cs);
    hRes =::GetLastError(m_szError,SZ_PSS_SAMPLE_AB,m_ulContext,lppMAPIError);
    LeaveCriticalSection(&m_cs);
    return hRes;
}






///////////////////////////////////////////////////////////////////////////////
//
//    CHomePageButton()
//
//    Parameters                        
//          pCont    [in] pointer to the parent container
//
//    Purpose
//          Class constructor
//      
//    Return Value
//          none.

CHomePageButton :: CHomePageButton(PMailUser pUser)
{
    TraceMessage("CHomePageButton::CHomePageButton");
    m_pUser = pUser;
    m_pUser -> AddRef();
    m_cRef = 0;
    m_ulContext = 0;
    m_ulVersion = 0;   
    m_szError[0] = '\0';
  //!! for now  m_hInst = pUser -> m_pLogon -> m_hInst;
    InitializeCriticalSection(&m_cs);
}

///////////////////////////////////////////////////////////////////////////////
//
//    ~CHomePageButton()
//
//    Parameters                        
//          none.
//
//    Purpose
//          Class destructor
//      
//    Return Value
//          none.
//
CHomePageButton :: ~CHomePageButton()
{
    TraceMessage("CHomePageButton::~CHomePageButton");

    m_pUser -> Release();
    DeleteCriticalSection(&m_cs);
}

///////////////////////////////////////////////////////////////////////////////
//
//    CHomePageButton::AddRef()
//
//    Parameters
//          Refer to OLE Documentation on this method.
//
//    Purpose
//          To increase the reference count of this object, and of 
//          any contained (or aggregated) object within
//
//    Return Value
//          Usage count for this object
//
STDMETHODIMP_ (ULONG) CHomePageButton::AddRef (void)
{   
    ULONG   ulRef;

    // Increment object's internal counter
    EnterCriticalSection(&m_cs);
    ulRef = ++m_cRef;
    TraceMessage("CHomePageButton :: AddRef");
    LeaveCriticalSection(&m_cs);
       
    return ulRef;
}

///////////////////////////////////////////////////////////////////////////////
//
//    CHomePageButton::Release()
//
//    Parameters
//          Refer to OLE Documentation on this method.
//
//    Purpose
//          Decrements usage count of this object. If count is zero,
//          object deletes itself.
//
//    Return Value
//          Usage of this object
//
STDMETHODIMP_ (ULONG) CHomePageButton :: Release (void)
{
    ULONG ulRefCount;

    EnterCriticalSection(&m_cs);
    ulRefCount = --m_cRef;
    LeaveCriticalSection(&m_cs); 

    TraceString1("CHomePageButton :: Release ref %x",ulRefCount);

    // If nobody is using this object, delete it
    if (0 == ulRefCount)
        // Destroy the object
        delete this;        
        
    return ulRefCount;
}        

///////////////////////////////////////////////////////////////////////////////
//
//    QueryInterface()
//
//    Parameters                        
//          See the OLE documentation for details.
//
//    Purpose
//          Returns an interface if supported.
//      
//      
//    Return Value
//          NOERROR on success, E_NOINTERFACE if the interface can't be
//          returned.
//

STDMETHODIMP CHomePageButton :: QueryInterface(REFIID   riid, 
                                              LPVOID * ppvObj)
{
    TraceMessage("CHomePageButton :: QueryInterface");

    if (IsBadWritePtr(ppvObj,sizeof LPUNKNOWN))
        return MAPI_E_INVALID_PARAMETER;

    *ppvObj = NULL;

    // If this is one of the two IID return an interface pointer to it
    if (riid == IID_IUnknown || riid == IID_IMAPIControl)
    {
        EnterCriticalSection(&m_cs);
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        m_cRef++;
        LeaveCriticalSection(&m_cs);
        return NOERROR;
    }

    return E_NOINTERFACE;
} 

///////////////////////////////////////////////////////////////////////////////
//
//    GetState()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK;
//
STDMETHODIMP CHomePageButton :: GetState(ULONG ulFlags, ULONG * pulState)
{
    TraceMessage("CHomePageButton :: GetState");
    *pulState = MAPI_ENABLED;

    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Activate()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          This method is called when the user hits the 
//          Add/Delete members button on a DL details or
//          create dialog. We use the IMAPISupport::Address
//          method to do most of the work. We pass Address an
//          ADRLIST of current members of the DL. When Address
//          returns we have an ADRLIST of members the user 
//          selected for the DL. We delete from the DL any entries
//          that are in the first list, but not the second, and
//          copy to the DL any entries in the second that aren't
//          in the first. We use DeleteEntries and CopyEntries 
//          to do the work.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CHomePageButton :: Activate(ULONG ulFlags, ULONG ulUIParam)
{
    HRESULT hRes        = S_OK;

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    SetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Sets detailed error information based on hRes.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//

STDMETHODIMP CHomePageButton :: SetLastError(HRESULT       hRes,    
                                            ULONG         ulLine,    
                                            LPSTR         szFile)
{
    if (FAILED(hRes))
    {
        EnterCriticalSection(&m_cs);
        m_ulContext = ulLine + IE_IABLOGON;
        lstrcpyn(m_szError,szFile,MAX_PATH);
        m_szError[MAX_PATH - 1] = '\0';        
        LeaveCriticalSection(&m_cs);
    }
    return hRes;
}


///////////////////////////////////////////////////////////////////////////////
//
//    GetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Gets detailed error information based on hResult.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//
STDMETHODIMP CHomePageButton :: GetLastError(HRESULT         hResult,                    
                                            ULONG           ulFlags,                    
                                            LPMAPIERROR  *  lppMAPIError)
{
    HRESULT hRes;
    TraceMessage("CHomePageButton :: GetLastError");
    Validate_IMAPIControl_GetLastError(this,
                                       hResult,     
                                       ulFlags,     
                                       lppMAPIError);
 
    EnterCriticalSection(&m_cs);
    hRes =::GetLastError(m_szError,SZ_PSS_SAMPLE_AB,m_ulContext,lppMAPIError);
    LeaveCriticalSection(&m_cs);
    return hRes;
}


///////////////////////////////////////////////////////////////////////////////
//
//    CBrowseButton()
//
//    Parameters                        
//          pConfig    [in] pointer to the parent configuration object
//
//    Purpose
//          Class constructor
//      
//    Return Value
//          none.

CBrowseButton :: CBrowseButton(ConfigObj * pConfig)
{
    TraceMessage("CBrowseButton::CBrowseButton");
    m_pConfig = pConfig;
    m_hInst   = pConfig -> m_hInst;
    m_cRef = 0;
    m_ulContext = 0;
    m_ulVersion = 0;
    m_szError[0] = '\0';
    InitializeCriticalSection(&m_cs);
}

///////////////////////////////////////////////////////////////////////////////
//
//    ~CBrowseButton()
//
//    Parameters                        
//          none.
//
//    Purpose
//          Class destructor
//      
//    Return Value
//          none.
//
CBrowseButton :: ~CBrowseButton()
{
    TraceMessage("CBrowseButton::~CBrowseButton");
    DeleteCriticalSection(&m_cs);
}

///////////////////////////////////////////////////////////////////////////////
//
//    CBrowseButton::AddRef()
//
//    Parameters
//          Refer to OLE Documentation on this method.
//
//    Purpose
//          To increase the reference count of this object, and of 
//          any contained (or aggregated) object within
//
//    Return Value
//          Usage count for this object
//
STDMETHODIMP_ (ULONG) CBrowseButton::AddRef (void)
{   
    ULONG   ulRef;

    // Increment object's internal counter
    EnterCriticalSection(&m_cs);
    ulRef = ++m_cRef;
    TraceMessage("CBrowseButton :: AddRef");
    LeaveCriticalSection(&m_cs);
       
    return ulRef;
}

///////////////////////////////////////////////////////////////////////////////
//
//    CBrowseButton::Release()
//
//    Parameters
//          Refer to OLE Documentation on this method.
//
//    Purpose
//          Decrements usage count of this object. If count is zero,
//          object deletes itself.
//
//    Return Value
//          Usage of this object
//
STDMETHODIMP_ (ULONG) CBrowseButton :: Release (void)
{
    ULONG ulRefCount;

    EnterCriticalSection(&m_cs);
    ulRefCount = --m_cRef;
    LeaveCriticalSection(&m_cs); 

    TraceString1("CBrowseButton :: Release  ref %x",ulRefCount);

    // If nobody is using this object, delete it
    if (0 == ulRefCount)
        // Destroy the object
        delete this;
        
    return ulRefCount;
}        

///////////////////////////////////////////////////////////////////////////////
//
//    QueryInterface()
//
//    Parameters                        
//          See the OLE documentation for details.
//
//    Purpose
//          Returns an interface if supported.
//      
//      
//    Return Value
//          NOERROR on success, E_NOINTERFACE if the interface can't be
//          returned.
//

STDMETHODIMP CBrowseButton :: QueryInterface(REFIID   riid, 
                                             LPVOID * ppvObj)
{
    TraceMessage("CBrowseButton :: QueryInterface");

    if (IsBadWritePtr(ppvObj,sizeof LPUNKNOWN))
        return MAPI_E_INVALID_PARAMETER;

    *ppvObj = NULL;

    // If this is one of the two IID return an interface pointer to it
    if (riid == IID_IUnknown || riid == IID_IMAPIControl)
    {
        EnterCriticalSection(&m_cs);
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        m_cRef++;
        LeaveCriticalSection(&m_cs);
        return NOERROR;
    }

    return E_NOINTERFACE;
} 

///////////////////////////////////////////////////////////////////////////////
//
//    GetState()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          See the MAPI documentation for details.
//      
//    Return Value
//          S_OK;
//
STDMETHODIMP CBrowseButton :: GetState(ULONG ulFlags, ULONG * pulState)
{
    TraceMessage("CBrowseButton :: GetState");
    *pulState = MAPI_ENABLED;

    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//
//    Activate()
//
//    Parameters                        
//          See the MAPI documentation for details.
//
//    Purpose
//          This method is called when the user hits the 
//          Browse button on the configuration dialog. We
//          display the open file dialog.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise.
//
STDMETHODIMP CBrowseButton :: Activate(ULONG ulFlags, ULONG ulUIParam)
{
    HRESULT hRes     = S_OK;
    ULONG   iRow;

    TraceMessage("CBrowseButton :: Activate");

    // bring OpenFile dialog
    hRes = BrowseFile((HWND)ulUIParam, 
                      &m_pConfig -> m_pProps[LOGON_DB_PATH].Value.LPSZ,
                      (LPVOID)m_pConfig -> m_pProps);

    // either a real error or user cancelled
    if (M_FAIL(hRes))
        return MAPI_E_CANCEL == hRes ? S_OK : hRes;

    // new file selected, update props
    if (M_FAIL(hRes = HrSetOneProp(m_pConfig,&m_pConfig -> m_pProps[LOGON_DB_PATH])))
        goto Quit;

    // walk rows in display table's data
    for (iRow = 0; ; iRow++)
    {
        LPSRow       pRow = NULL;
        LPSPropValue pControlID;

        hRes = m_pConfig -> m_pDispTblData->HrEnumRow(iRow, &pRow);

        if (!pRow || M_FAIL(hRes))
            goto Quit;
        
        // get cotnrol ID
        pControlID = PpropFindProp(pRow->lpProps, pRow->cValues, PR_CONTROL_ID);

        // if control has notification key (control ID non-NULL), notify
        if (pControlID->Value.bin.cb)
    	    m_pConfig -> m_pDispTblData -> HrNotify(0, 1, pControlID);

        gFreeBuff(pRow);
    }

Quit:    
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    SetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Sets detailed error information based on hRes.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//

STDMETHODIMP CBrowseButton :: SetLastError(HRESULT       hRes,    
                                            ULONG         ulLine,    
                                            LPSTR         szFile)
{
    if (FAILED(hRes))
    {
        EnterCriticalSection(&m_cs);
        m_ulContext = ulLine + IE_IABLOGON;
        lstrcpyn(m_szError,szFile,MAX_PATH);
        m_szError[MAX_PATH - 1] = '\0';
        LeaveCriticalSection(&m_cs);
    }
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//    GetLastError()
//
//    Parameters                        
//          See the MAPI Documentation for details.
//
//    Purpose
//          Gets detailed error information based on hResult.
//      
//    Return Value
//          S_OK on success, a MAPI error code otherwise
//

STDMETHODIMP CBrowseButton :: GetLastError(HRESULT         hResult,                    
                                           ULONG           ulFlags,                    
                                           LPMAPIERROR  *  lppMAPIError)
{
    HRESULT     hRes          = S_OK;
    TraceMessage("CBrowseButton:: GetLastError");
    Validate_IMAPIControl_GetLastError(this,
                                       hResult,     
                                       ulFlags,     
                                       lppMAPIError);
    return hRes;
}

// End of file for UI.CPP
