///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      ABCONT.CPP
//
//  Description
//      This file implements the IABContainer and IDistList interfaces with
//      the methods specified in the MAPI SPI 1.0.
//      Note that IDistList and IABContainer are implemented in a single
//      class CABContainer. When methods need to behave differently, between
//      IDistList and IABContainer, the results are controlled by the internal
//      flags of object.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#include "ABWDS.H"
#include "TABLE.H"

// Remark this line to turn verbose tracing OFF
//#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)        TraceInfoMessage(a)
#define InfoTraceRaw(a)     TraceRaw(a)
#define InfoTraceProp(a)    TraceProp(a)
#else
#define InfoTrace(a)        0
#define InfoTraceRaw(a)     0
#define InfoTraceProp(a)    0
#endif // DO_INFO_TRACES

#define SEARCH_CONTAINER_ID      0xFFFFFFF5

#define NUM_AB_CONTAINER_PROPS        13
// Properties available in our implementation of IABContainer
const static SizedSPropTagArray(NUM_AB_CONTAINER_PROPS, sptABContObj) =
{
    NUM_AB_CONTAINER_PROPS,
    {
        PR_ACCESS,
        PR_ACCESS_LEVEL,
        PR_RECORD_KEY,
        PR_ENTRYID,
        PR_CONTENT_COUNT,
        PR_CONTAINER_HIERARCHY,
        PR_CONTAINER_CONTENTS,
        PR_OBJECT_TYPE,
        PR_CREATION_VERSION,
        PR_CURRENT_VERSION,
        PR_DISPLAY_TYPE,
        PR_DISPLAY_NAME,
        PR_INITIAL_DETAILS_PANE
    }
};

#define NUM_DIST_LIST_PROPS        20
// Properties available in our implementation of IDistList
const static SizedSPropTagArray(NUM_DIST_LIST_PROPS, sptDistListObj) =
{
    NUM_DIST_LIST_PROPS,
    {
        PR_ACCESS,
        PR_ACCESS_LEVEL,
        PR_RECORD_KEY,
        PR_ENTRYID,
        PR_SEARCH_KEY,
        PR_MAPPING_SIGNATURE,
        PR_CONTENT_COUNT,
        PR_CONTAINER_CONTENTS,
        PR_OBJECT_TYPE,
        PR_CREATION_VERSION,
        PR_CURRENT_VERSION,
        PR_DISPLAY_TYPE,
        PR_DISPLAY_NAME,
        PR_TRANSMITABLE_DISPLAY_NAME,
        PR_ACCOUNT,
        PR_ADDRTYPE,
        PR_EMAIL_ADDRESS,
        PR_SEND_RICH_INFO,
        PR_INITIAL_DETAILS_PANE,
        PR_TEMPLATEID
    }
};

#define NUM_AB_FINDER_PROPS        13
// Properties available in our implementation of IDistList
const static SizedSPropTagArray(NUM_AB_FINDER_PROPS, sptABFinderObj) =
{
    NUM_AB_FINDER_PROPS,
    {
        PR_ACCESS,
        PR_ACCESS_LEVEL,
        PR_RECORD_KEY,
        PR_ENTRYID,
        PR_SEARCH_KEY,
        PR_MAPPING_SIGNATURE,
        PR_CONTAINER_CONTENTS,
        PR_OBJECT_TYPE,
        PR_CREATION_VERSION,
        PR_CURRENT_VERSION,
        PR_DISPLAY_TYPE,
        PR_INITIAL_DETAILS_PANE,
        PR_CONTENT_COUNT
    }
};

#define RESOLVE_NAMES_COLUMNS   3
const static SizedSPropTagArray (RESOLVE_NAMES_COLUMNS, sptResolveNamesCols) =
{
    RESOLVE_NAMES_COLUMNS,
    {
        PR_DISPLAY_NAME,
        PR_ACCOUNT,
        PR_INSTANCE_KEY
    }
};

SSortOrderSet sosDefaultSort = 
{
    1,
    0,
    0,
    {
        PR_DISPLAY_NAME,
        TABLE_SORT_ASCEND
    }
};



// Control description structures
DTBLLBX  DtDLMembers     = { 0, PR_NULL, PR_CONTAINER_CONTENTS };
DTBLEDIT DtLULUpdate     = { sizeof(DTBLEDIT), fMapiUnicode, 64, PR_SMP_LUL_LAST_UPDATE };
DTBLEDIT DtOwnerAlias    = { sizeof(DTBLEDIT), fMapiUnicode, 64, PR_WINDS_AB_DL_OWNER_ACCT };
DTBLEDIT DtOwnerFullName = { sizeof(DTBLEDIT), fMapiUnicode, 64, PR_WINDS_AB_DL_OWNER_NAME };
DTBLEDIT DtOwnerComment  = { sizeof(DTBLEDIT), fMapiUnicode, 512, PR_COMMENT };

// Description table for the controls in the property pages
DTCTL DLPropPage[] =
{
    { DTCT_PAGE,     0, NULL, 0, NULL,     0,                 &DtPage        },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_ALIAS,    &DtObjAlias    },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_FULLNAME, &DtObjFullName },
    { DTCT_LBX,      0, NULL, 0, szFilter, IDC_DL_MEMBERS,    &DtDLMembers   },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC1,       &DtLabel       },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC2,       &DtLabel       },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC3,       &DtGroupBox    },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC4,       &DtGroupBox    }
};

DTCTL DLOwnerPage[] =
{
    { DTCT_PAGE,     0, NULL, 0, NULL,     0,                 &DtPage          },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_ALIAS,    &DtOwnerAlias    },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_FULLNAME, &DtOwnerFullName },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_COMMENTS,      &DtOwnerComment  },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC1,       &DtLabel         },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC2,       &DtLabel         },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC3,       &DtLabel         },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC4,       &DtGroupBox      }
};

DTCTL ABContPropPage[] =
{
    { DTCT_PAGE,     0, NULL, 0, NULL,     0,                     &DtPage        },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_AB_CONTAINER_NAME, &DtObjFullName },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_SERVERNAME,        &DtServer      },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_ALIAS,        &DtUserName    },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_LUL_UPDATE_DATE,   &DtLULUpdate   },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC1,           &DtLabel       },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC2,           &DtLabel       },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC3,           &DtGroupBox    },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC4,           &DtGroupBox    },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC5,           &DtGroupBox    },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC6,           &DtGroupBox    }
};

extern DTBLEDIT DtMBUserPhone;
extern DTBLEDIT DtMBUserOffice;

DTCTL ABFinderPropPage[] =
{
    { DTCT_PAGE,     0,           NULL, 0, NULL,     0,                 &DtPage         },
    { DTCT_EDIT,     DT_EDITABLE, NULL, 0, szFilter, IDC_USER_ALIAS,    &DtObjAlias     },
    { DTCT_EDIT,     DT_EDITABLE, NULL, 0, szFilter, IDC_USER_FULLNAME, &DtObjFullName  },
    { DTCT_EDIT,     DT_EDITABLE, NULL, 0, szPhoneFilter, IDC_USER_PHONE,    &DtMBUserPhone  },
    { DTCT_EDIT,     DT_EDITABLE, NULL, 0, szFilter, IDC_USER_OFFICE,   &DtMBUserOffice },
    { DTCT_LABEL,    0,           NULL, 0, NULL,     IDC_STATIC1,       &DtLabel        },
    { DTCT_LABEL,    0,           NULL, 0, NULL,     IDC_STATIC2,       &DtLabel        },
    { DTCT_LABEL,    0,           NULL, 0, NULL,     IDC_STATIC3,       &DtLabel        },
    { DTCT_LABEL,    0,           NULL, 0, NULL,     IDC_STATIC4,       &DtLabel        },
    { DTCT_LABEL,    0,           NULL, 0, NULL,     IDC_STATIC5,       &DtLabel        },
    { DTCT_LABEL,    0,           NULL, 0, NULL,     IDC_STATIC6,       &DtLabel        },
    { DTCT_LABEL,    0,           NULL, 0, NULL,     IDC_STATIC7,       &DtLabel        },
    { DTCT_GROUPBOX, 0,           NULL, 0, NULL,     IDC_STATIC8,       &DtGroupBox     }
};


///////////////////////////////////////////////////////////////////////////////
//    CABContainer::CABContainer()
//
//    Parameters
//      pLogon              Logon session to which this object is linked to
//      pParentEID          Entry ID of the parent of this object
//      pContainerEID       Entry ID of the container object itself
//      lContainerType      Type of container this is (AB container or
//                          Distribution list)
//      pList               Property cache with the back end stored
//                          properties for this object
//      pNPList             Pointer to the named property map list
//      lAccess             Access level to this object
//      
//    Purpose
//      Constructor of the object. Parameters are passed to initialize the
//      data members with the appropiate values.
//
//    Return Value
//      None
//
CABContainer::CABContainer (CABLogon *        pLogon,
                            PRIVATE_ENTRYID * pParentEID,
                            PRIVATE_ENTRYID * pContainerEID,
                            long              lContainerType,
                            CPropList *       pList,
                            CNamePropMgr *    pNPList,
                            long              lAccess) : CGenericProp (pLogon->m_pBEobj,
                                                                       pParentEID,
                                                                       pContainerEID,
                                                                       pList,
                                                                       FALSE,
                                                                       lAccess)
{
    InfoTrace ("CABContainer: Constructor called");
    m_pNPList               = pNPList;
    m_pHierTable            = NULL;
    m_pContTable            = NULL;
    m_lContainerType        = lContainerType;
    m_pLogon                = pLogon;
    m_pLogon->AddRef();
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::~CABContainer()
//
//    Parameters
//      None
//
//    Purpose
//      Destructor of CABContainer.
//
//    Return Value
//      None
//
CABContainer::~CABContainer()
{
    InfoTrace ("CABContainer: Destructor called");
    if (m_pHierTable)
    {
        m_pHierTable->Release();
        m_pHierTable = NULL;
    }
    if (m_pContTable)
    {
        m_pContTable->Release();
        m_pContTable = NULL;
    }
    m_pLogon->Release();
    m_pLogon = NULL;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::QueryInterface()
//
//    Parameters
//      { Refer to the OLE documentation for parameter description }
//
//    Purpose
//      Returns a pointer to a interface requested if the interface is 
//      supported and implemented by this object. If it is not supported, it
//      returns NULL
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::QueryInterface (REFIID riid, LPVOID * ppvObj)
{
    Validate_IUnknown_QueryInterface (this, riid, ppvObj);
    // OLE requires NULLing parameter
    *ppvObj = NULL;
    // If the interface requested is supported by this object, return a pointer
    // to ourselves, with the reference count incremented by one.
    if (riid == IID_IABContainer      ||
        riid == IID_IDistList         ||
        riid == IID_IMAPIContainer    ||
        riid == IID_IMAPIProp         ||
        riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    // This object does not support the interface requested
    return E_NOINTERFACE;
}

///////////////////////////////////////////////////////////////////////////////
// IMAPIProp virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetLastError()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Return a MAPIERROR structure filled up with meaningful descriptions
//      about an error that occurred in our provider. The caller is responsible
//      for freeing the memory returned.
//      In our implementation I delegate the call up to the base class which
//      implements this call.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetLastError (HRESULT       hError,
                                         ULONG         ulFlags,
                                         LPMAPIERROR * ppMAPIError)
{
    InfoTrace ("CABContainer::GetLastError method called");
    Validate_IMAPIProp_GetLastError (this, hError, ulFlags, ppMAPIError);
    return CGeneric::GetLastError (hError,
                                   ulFlags,
                                   ppMAPIError,
                                   m_pLogon->m_hInstance,
                                   PROVIDER_DISPLAY_NAME_STRING);
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::SaveChanges()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. Our implementation is read-only so this is a no-op call.
//
//    Return Value
//      S_OK always.
//
STDMETHODIMP CABContainer::SaveChanges (ULONG ulFlags)
{
    InfoTrace ("CABContainer::SaveChanges method called");
    Validate_IMAPIProp_SaveChanges (this, ulFlags);
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetProps()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Return the value of the properties requested in the list of tags
//      passed in. The returned property array is in the same order as
//      the tag array passed.
//      The memory for returned properties is allocated using the MAPI
//      allocation functions. The caller is responsible for free it with
//      MAPIFreeBuffer() or the equivalent.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetProps (LPSPropTagArray pPropTagArray,
                                     ULONG           ulFlags,
                                     ULONG *         pcValues,
                                     LPSPropValue *  ppPropArray)
{
    InfoTrace ("CABContainer::GetProps method called");
    Validate_IMAPIProp_GetProps (this, pPropTagArray, ulFlags, pcValues, ppPropArray);
    #ifdef DO_INFO_TRACES
    if (!pPropTagArray)
    {
        TraceInfoMessage ("CABContainer::GetProps: All properties requested");
    }
    else
    {
        TraceMessage ("CABContainer::GetProps Properties requested...");
        for (ULONG j=0; j<pPropTagArray->cValues; j++) { TraceProp (pPropTagArray->aulPropTag[j]); }
    }
    #endif // DO_INFO_TRACES

    EnterCriticalSection (&m_csObj);
    BOOL fGotErrors = FALSE;
    LPSPropTagArray pObjTags;
    // Depending on the object type and flags, see which object
    if (MAPI_ABCONT == m_lContainerType)
    {
        // The container is a FINDER container
        if (OBJECT_IS_SEARCH_CONTAINER & m_ulObjFlags)
        {
            pObjTags = (LPSPropTagArray)&sptABFinderObj;
        }
        else
        {
            // A regular AB container
            pObjTags = (LPSPropTagArray)&sptABContObj;
        }
    }
    else
    {
        // A distribution list container
        pObjTags = (LPSPropTagArray)&sptDistListObj;
    }
    PRIVATE_ENTRYID TemplateID = { 0 };

    // Ask the base class to do partial work for this call:
    //      1. Collect all the property tags of the object including the ones in the property cache
    //      2. Allocate a new SPropValue array for the returned properties
    //      3. Replace the tags of PT_UNSPECIFIED type in the tag passed in
    //         with the actual type of the store property (if it exists in this object)
    HRESULT hResult = CGenericProp::GetProps (pPropTagArray,
                                              ulFlags,
                                              pcValues,
                                              ppPropArray,
                                              pObjTags);
    if (hResult)
    {
        goto ErrorExit;
    }

    LPVOID pSrc;
    ULONG cbSize, i;
    // Get the values for all the properties requested
    for (cbSize=0, i=0; i<*pcValues; i++)
    {
        pSrc = NULL;
        hResult = S_OK;
        switch ((*ppPropArray)[i].ulPropTag)
        {
            // Get the PT_OBJECT/PR_NULL properties
            case PR_NULL :
            case PR_CONTAINER_CONTENTS :
                (*ppPropArray)[i].Value.err = S_OK;
                break;
            case PR_CONTAINER_HIERARCHY :
                // A hierachy only exist in AB containers that logically have them
                if (MAPI_ABCONT == m_lContainerType && OBJECT_MAY_HAVE_CONTAINERS & m_ulObjFlags)
                {
                    (*ppPropArray)[i].Value.err = S_OK;
                }
                else
                {
                    hResult = MAPI_E_NOT_FOUND;
                }
                break;
            // Get the PT_LONG properties
            case PR_ACCESS :
                (*ppPropArray)[i].Value.l = m_lAccess;
                break;
            case PR_ACCESS_LEVEL :
                (*ppPropArray)[i].Value.l = 0;
                break;
            case PR_OBJECT_TYPE :
                (*ppPropArray)[i].Value.l = m_lContainerType;
                break;
            case PR_INITIAL_DETAILS_PANE :
                (*ppPropArray)[i].Value.l = 0;
                break;

            // Get the PT_BOOLEAN properties
            case PR_SEND_RICH_INFO :
                if (MAPI_DISTLIST == m_lContainerType)
                {
                    (*ppPropArray)[i].Value.b = TRUE;
                }
                else
                {
                    hResult = MAPI_E_NOT_FOUND;
                }
                break;
            // Get the PT_TSTRING properties
            case PR_SMP_USER_NAME :
                // This property is for the details table of AB containers
                if ((MAPI_DISTLIST == m_lContainerType) || (OBJECT_IS_SEARCH_CONTAINER & m_ulObjFlags))
                {
                    hResult = MAPI_E_NOT_FOUND;
                }
                else
                {
                    pSrc = (LPVOID)m_pLogon->GetUserName();
                }
                break;
            case PR_SMP_REMOTE_SERVER :
                // For the details table
                pSrc = (LPVOID)m_pLogon->GetServerName();
                break;
            case PR_SMP_LUL_LAST_UPDATE :
                // For the details table
                pSrc = (LPVOID)m_pLogon->GetLocalABDLastUpdate();
                break;
            case PR_ADDRTYPE :
                // Only for distribution list objects
                if (MAPI_DISTLIST == m_lContainerType)
                {
                    pSrc = (LPVOID)WINDS_ADDRESS_TYPE;
                }
                else
                {
                    hResult = MAPI_E_NOT_FOUND;
                }
                break;
            // Get the PT_I8 properties
            case PR_CREATION_VERSION :
                (*ppPropArray)[i].Value.li.LowPart = ADR_BOOK_VERSION_MAJOR;
                (*ppPropArray)[i].Value.li.HighPart = ADR_BOOK_VERSION_MINOR;
                break;
            case PR_CURRENT_VERSION :
                (*ppPropArray)[i].Value.li.LowPart = ADR_BOOK_VERSION_MAJOR;
                (*ppPropArray)[i].Value.li.HighPart = ADR_BOOK_VERSION_MINOR;
                break;
            // Get the PT_BINARY properties
            case PR_MAPPING_SIGNATURE :
                // Even though I don't support named properties, for completeness,
                // I have this property available.
                cbSize = sizeof(MAPIUID);
                pSrc = (LPVOID)&m_pBEobj->GetBackEndUID();
                break;
            case PR_TEMPLATEID :
                // I only support template ID on distribution list of this address
                // book where they are copied to foreign address books
                if (MAPI_DISTLIST == m_lContainerType)
                {
                    TemplateID = m_ObjectEID;
                    TemplateID.bObject = EID_OBJTYPE_DL_TEMPLATE;
                    cbSize = CB_PRIVATE_EID;
                    pSrc = (LPVOID)&TemplateID;
                }
                else
                {
                    hResult = MAPI_E_NOT_FOUND;
                }
                break;
            // The search property, in this implementation, is just the entry ID of the object, and so is the record key.
            case PR_SEARCH_KEY :
                if (MAPI_DISTLIST == m_ObjectEID.bObject)
                {   
                    // Distribution lists have a different kind of search key.
                    hResult = CGenericProp::GetProps (&(*ppPropArray)[i], *ppPropArray);
                    break;
                }
                // else fall through
            case PR_RECORD_KEY :
            case PR_ENTRYID :
                cbSize = CB_PRIVATE_EID;
                pSrc = (LPVOID)&m_ObjectEID;
                break;
            // If the property was not one of the above, try the property cache.
            default :
                // If the property was not one of the above, ask the base class implementation
                // to search for it in the property cache of the object. If the property was in the
                // property cache, the base class copies it to the LPSPropValue passed in, linking
                // its memory to the parent memory block passed in.
                hResult = CGenericProp::GetProps (&(*ppPropArray)[i], *ppPropArray);
                #ifdef DO_INFO_TRACES
                if (MAPI_E_NOT_FOUND == hResult)
                {
                    TraceRaw ("CABContainer::GetProps: Not Found - "); TraceProp ((*ppPropArray)[i].ulPropTag);
                }
                #endif // #DO_INFO_TRACES
                break;
        }
        // Now that I have the value of the property in pSrc or the error in hResult,
        // copy the property or set the property tag and error if not found.
        if (S_OK != CGenericProp::GetProps (hResult,
                                            cbSize,
                                            pSrc,
                                            &(*ppPropArray)[i],
                                            *ppPropArray))
        {
            fGotErrors = TRUE;
        }
    }
ErrorExit :
    LeaveCriticalSection (&m_csObj);
    if (fGotErrors)
	{
        // I got at least one error. Return partial success.
		hResult = MAPI_W_ERRORS_RETURNED;
	}
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetPropList()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Duplicate the tag array with ALL the available properties
//      on this object.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetPropList (ULONG             ulFlags,
                                        LPSPropTagArray * ppAllTags)
{
    InfoTrace ("CABContainer::GetPropList method called");
    Validate_IMAPIProp_GetPropList (this, ulFlags, ppAllTags);
    LPSPropTagArray pObjTags;
    if (MAPI_ABCONT == m_lContainerType)
    {
        if (OBJECT_IS_SEARCH_CONTAINER & m_ulObjFlags)
        {
            pObjTags = (LPSPropTagArray)&sptABFinderObj;
        }
        else
        {
            pObjTags = (LPSPropTagArray)&sptABContObj;
        }
    }
    else
    {
        pObjTags = (LPSPropTagArray)&sptDistListObj;
    }
    // Ask the base class to do the bulk work
    return CGenericProp::GetPropList (ulFlags, pObjTags, ppAllTags);
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::OpenProperty()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      This function offers limited capabilities to open 4 special
//      properties supported by address book containers in this implementation:
//      1. PR_CONTAINER_CONTENTS: Returns an IMAPITable interface with a list
//         of mailable objects of this container.
//      2. PR_CONTAINER_HIERARCHY: Returns an IMAPITable interface with
//         the list of subcontainers of this container.
//      3. PR_DETAILS_TABLE: Returns and IMAPITable interface with the display
//         table for the display property sheet of the properties of
//         this container.
//      4. PR_SEARCH: Return and IMAPIContainer interface that the call can
//         use to build a more complex query for a search of the contents of
//         this container.
//      
//      Note that each property supported, returns a specific interface. If the
//      interface requested is not the one supported for that particular
//      property I return E_NOINTERFACE. If the property to open is no the
//      supported set, I return MAPI_E_NO_SUPPORT.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::OpenProperty (ULONG       ulPropTag,
                                         LPCIID      piid,
                                         ULONG       ulInterfaceOptions,
                                         ULONG       ulFlags,
                                         LPUNKNOWN * ppUnk)
{
    InfoTrace ("CABContainer::OpenProperty method called");
    Validate_IMAPIProp_OpenProperty (this,
                                     ulPropTag, 
                                     piid,
                                     ulInterfaceOptions,
                                     ulFlags,
                                     ppUnk);
    InfoTraceRaw ("Property to open: "); InfoTraceProp (ulPropTag);
    if (PR_DETAILS_TABLE != ulPropTag && 
        PR_CONTAINER_CONTENTS != ulPropTag &&
        PR_CONTAINER_HIERARCHY != ulPropTag &&
        PR_SEARCH != ulPropTag)
    {
        return MAPI_E_NO_SUPPORT;
    }

    if (PR_DETAILS_TABLE == ulPropTag ||
        PR_CONTAINER_CONTENTS == ulPropTag ||
        PR_CONTAINER_HIERARCHY == ulPropTag)
    {
        if (IID_IMAPITable != *piid)
        {
            TraceMessage ("CABContainer::OpenProperty: I only support the IMAPITable interface on these properties");
            return E_NOINTERFACE;
        }
    }
    if (PR_SEARCH == ulPropTag && IID_IMAPIContainer != *piid)
    {
        TraceMessage ("CABContainer::OpenProperty: I only support the IMAPIContainer interface on search containers");
        return E_NOINTERFACE;
    }

    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);

    if (PR_DETAILS_TABLE == ulPropTag)
    {
        // Build the display page structure with the static information at the top of this file.
        // Each container type has a different property page so we must select the right one
        DTPAGE DtProperties[2] = { 0 };
        ULONG ulPages;
        DtProperties[0].lpszComponent = szBlank;
        if (MAPI_DISTLIST == m_lContainerType)
        {
            DtProperties[0].cctl = sizeof(DLPropPage)/sizeof(DTCTL);
            DtProperties[0].lpszResourceName = MAKEINTRESOURCE (IDD_DETAILS_SERVERDL);
            DtProperties[0].lpctl = DLPropPage;
            
            DtProperties[1].cctl = sizeof(DLOwnerPage)/sizeof(DTCTL);
            DtProperties[1].lpszResourceName = MAKEINTRESOURCE (IDD_DETAILS_DL_OWNER);
            DtProperties[1].lpctl = DLOwnerPage;
            
            DtProperties[1].lpszComponent = szBlank;
            ulPages = 2;
        }
        else
        {
            ulPages = 1;
            if (OBJECT_IS_SEARCH_CONTAINER & m_ulObjFlags)
            {
                DtProperties[0].cctl = sizeof(ABFinderPropPage)/sizeof(DTCTL);
                DtProperties[0].lpszResourceName = MAKEINTRESOURCE (IDD_DETAILS_SEARCH_FINDER);
                DtProperties[0].lpctl = ABFinderPropPage;
            }
            else
            {
                DtProperties[0].cctl = sizeof(ABContPropPage)/sizeof(DTCTL);
                DtProperties[0].lpszResourceName = MAKEINTRESOURCE (IDD_DETAILS_ABCONTAINER);
                DtProperties[0].lpctl = ABContPropPage;
            }
        }
        hResult = BuildDisplayTable (GetMAPIAllocBuffer(),
                                     GetMAPIAllocMore(),
                                     GetMAPIFreeBuffer(),
                                     NULL,
                                     m_pLogon->m_hInstance,
                                     ulPages,
                                     DtProperties,
                                     fMapiUnicode,
                                     (LPMAPITABLE *)ppUnk,
                                     NULL);
    }
    if (PR_CONTAINER_CONTENTS == ulPropTag)
    {
        hResult = GetContentsTable (ulFlags, (LPMAPITABLE *)ppUnk);
        if (!hResult)
        {
            // We set the columns of the container so that the MAPI address book dialog
            // display the columns in our default order.
            ((LPMAPITABLE)(*ppUnk))->SetColumns ((LPSPropTagArray)&sptDLMembersLbx, 0);
        }
    }
    if (PR_CONTAINER_HIERARCHY == ulPropTag)
    {
        hResult = GetHierarchyTable (ulFlags, (LPMAPITABLE *)ppUnk);
    }
    if (PR_SEARCH == ulPropTag)
    {
        // The search container is associated with the container where it was opened.
        // So the parent entry ID of the search container is this container's entry ID.
        PRIVATE_ENTRYID eidSearchContainer = { 0 };
        eidSearchContainer.uidGlobal = m_pBEobj->GetBackEndUID();
        eidSearchContainer.bVersion = ENTRYID_VERSION;
        eidSearchContainer.bObject = MAPI_ABCONT;
        eidSearchContainer.dwObjID = SEARCH_CONTAINER_ID;

        CABContainer * pContObj;
        SPropValue spvProps = { 0 };
        CPropList * pList = new CPropList;
        if (pList)
        {
            spvProps.ulPropTag = PR_DISPLAY_TYPE;
            spvProps.Value.l = DT_NOT_SPECIFIC;
            pList->Insert (&spvProps);
            spvProps.ulPropTag = PR_CONTENT_COUNT;
            spvProps.Value.ul = 0;
            pList->Insert (&spvProps);
        }
        else
        {
            TraceMessage ("CABContainer::OpenProperty: Failed to allocate property cache for search container");
            hResult = E_OUTOFMEMORY;
        }
        if (!hResult)
        {
            pContObj = new CABContainer (m_pLogon,
                                         &m_ObjectEID,
                                         &eidSearchContainer,
                                         MAPI_ABCONT,
                                         pList,
                                         m_pNPList,
                                         MAPI_ACCESS_READ | MAPI_ACCESS_MODIFY);
            if (pContObj)
            {
                // Mark the flags of this container
                pContObj->m_ulObjFlags |= OBJECT_IS_SEARCH_CONTAINER;
                *ppUnk = (LPUNKNOWN)pContObj;
            }
            else
            {
                hResult = E_OUTOFMEMORY;
                delete pList;
            }
        }
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CABContainer::OpenProperty", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::SetProps()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      For distribution lists and normal containers, the call is a stub that
//      always returns E_ACCESSDENIED because those object are ALWAYS
//      read-only in our implementation.
//      For the advaced search container (aka. AB Finder), the property sets
//      the values collected by the UI in the details table. With those
//      values I build a restriction to do the more detailed search.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::SetProps (ULONG                 cValues,
                                     LPSPropValue          pPropArray,
                                     LPSPropProblemArray * ppProblems)
{
    InfoTrace ("CABContainer::SetProps method called");
    Validate_IMAPIProp_SetProps (this, cValues, pPropArray, ppProblems);
    // For non-search containers, the call is a stub, so return an error.
    if (!(OBJECT_IS_SEARCH_CONTAINER & m_ulObjFlags))
    {
        // If the object is not a finder container, then the object is read-only
        return E_ACCESSDENIED;
    }

    #ifdef DO_INFO_TRACES
    TraceMessage ("CABContainer::SetProps: Properties to set...");
    for (ULONG j=0; j<cValues; j++) { TraceProp (pPropArray[j].ulPropTag); }
    #endif DO_INFO_TRACES

    HRESULT hTagError, hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    for (ULONG i=0; i<cValues; i++)
    {
        switch (PROP_TYPE(pPropArray[i].ulPropTag))
        {
            case PT_NULL :
            case PT_UNSPECIFIED :
            case PT_ERROR :
                TraceMessage ("CABContainer::SetProps: Ignoring property of PT_NULL, PT_UNSPEFICIED, or PT_ERROR type");
                continue;
        }
        hTagError = AddPropToList (&pPropArray[i]);
        if (hTagError && ppProblems)
        {
            hResult = AddTagToProbArray (cValues,
                                         ppProblems,
                                         i,
                                         pPropArray[i].ulPropTag,
                                         hTagError);
            if (hResult)
            {
                break; // Out of the FOR() loop
            }
        }
    }
    LeaveCriticalSection (&m_csObj);
    // If an error occurred while adding errors to the problem array, the user
    // won't attempt to delete it so I should.
    if (hResult && ppProblems)
    {
        PrivMAPIFree ((LPVOID *)ppProblems);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::DeleteProps()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. Our IMAPIProp implementation is read-only.
//
//    Return Value
//      E_ACCESSDENIED always
//
STDMETHODIMP CABContainer::DeleteProps (LPSPropTagArray          pPropTagArray,
                                        LPSPropProblemArray *    ppProblems)
{
    InfoTrace ("CABContainer::DeleteProps method called");
    Validate_IMAPIProp_DeleteProps (this, pPropTagArray, ppProblems);
    return E_ACCESSDENIED;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::CopyTo()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Copy an object onto another. In our implementation I delegate the
//      call to MAPI to let it do the work for us.
//      There is no real advantage for this implementation to do it manually
//      because the object is foreign. We would never be doing a copy onto an
//      object of our own implementation because we are read only.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::CopyTo (ULONG                 ciidExclude,
                                   LPCIID                rgiidExclude,
                                   LPSPropTagArray       pExcludeProps,
                                   ULONG                 ulUIParam,
                                   LPMAPIPROGRESS        pProgress,
                                   LPCIID                pInterface,
                                   LPVOID                pDestObj,
                                   ULONG                 ulFlags,
                                   LPSPropProblemArray * ppProblems)
{
    InfoTrace ("CABContainer::CopyTo method called");
    Validate_IMAPIProp_CopyTo (this,
                               ciidExclude,
                               rgiidExclude,
                               pExcludeProps,
                               ulUIParam,
                               pProgress,
                               pInterface,
                               pDestObj,
                               ulFlags,
                               ppProblems);
    return m_pLogon->m_pSupObj->DoCopyTo (pInterface,
                                          (LPVOID)this,
                                          ciidExclude,
                                          rgiidExclude,
                                          pExcludeProps,
                                          ulUIParam,
                                          pProgress,
                                          pInterface,
                                          pDestObj,
                                          ulFlags,
                                          ppProblems);
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::CopyProps()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Copy object properties onto another. In our implementation I
//      delegate the call to MAPI to let it do the work for me.
//      There is no real advantage for this implementation to do it manually
//      because the object is foreign. We would never be doing a copy onto an
//      object of our own implementation because we are read only.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::CopyProps (LPSPropTagArray       pIncludeProps,
                                      ULONG                 ulUIParam,
                                      LPMAPIPROGRESS        pProgress,
                                      LPCIID                pInterface,
                                      LPVOID                pDestObj,
                                      ULONG                 ulFlags,
                                      LPSPropProblemArray * ppProblems)
{
    InfoTrace ("CABContainer::CopyProps method called");
    Validate_IMAPIProp_CopyProps (this,
                                  pIncludeProps,
                                  ulUIParam,
                                  pProgress,
                                  pInterface,
                                  pDestObj,
                                  ulFlags,
                                  ppProblems);
    return m_pLogon->m_pSupObj->DoCopyProps (pInterface,
                                             (LPVOID)this,
                                             pIncludeProps,
                                             ulUIParam,
                                             pProgress,
                                             pInterface,
                                             pDestObj,
                                             ulFlags,
                                             ppProblems);
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetNamesFromIDs()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Returns the names of the named properties of this object. We defer
//      down to the common implementation to retrieve the names for the
//      caller.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetNamesFromIDs (LPSPropTagArray * ppPropTags,
                                            LPGUID            pPropSetGuid,
                                            ULONG             ulFlags,
                                            ULONG *           pcPropNames,
                                            LPMAPINAMEID **   pppPropNames)
{
    InfoTrace ("CABContainer::GetNamesFromIDs method called");
    Validate_IMAPIProp_GetNamesFromIDs (this,
                                        ppPropTags,
                                        pPropSetGuid,
                                        ulFlags,
                                        pcPropNames,
                                        pppPropNames);
    LPSPropTagArray pTags;
    HRESULT hResult = CGenericProp::GetPropList (0, (LPSPropTagArray)&sptABContObj, &pTags);
    if (!hResult)
    {
        hResult = CGenericProp::GetNamesFromIDs (ppPropTags,
                                                 pPropSetGuid,
                                                 ulFlags,
                                                 pcPropNames,
                                                 pppPropNames,
                                                 pTags);
        PrivMAPIFree ((LPVOID *)&pTags);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetIDsFromNames()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Return the IDs from our named properties mapping for a list of names.
//      We defer down to the common implementation to retrieve the IDs for the
//      names the caller requested.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetIDsFromNames (ULONG             cPropNames,
                                            LPMAPINAMEID *    ppPropNames,
                                            ULONG             ulFlags,
                                            LPSPropTagArray * ppPropTags)
{
    InfoTrace ("CABContainer::GetIDsFromNames method called");
    Validate_IMAPIProp_GetIDsFromNames (this,
                                        cPropNames,
                                        ppPropNames,
                                        ulFlags,
                                        ppPropTags);
    LPSPropTagArray pTags;
    HRESULT hResult = CGenericProp::GetPropList (0, (LPSPropTagArray)&sptABContObj, &pTags);
    if (!hResult)
    {
        hResult = CGenericProp::GetIDsFromNames (cPropNames,
                                                 ppPropNames,
                                                 ulFlags,
                                                 ppPropTags,
                                                 pTags,
                                                 NULL);
        PrivMAPIFree ((LPVOID *)&pTags);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
// IMAPIContainer virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetContentsTable()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Returns a READ-only table for the contained mail users, distribution
//      lists, and other subcontainers in this container.
//      If the container does not have any mail user or distribution list
//      because logically, there shouldn't be any (i.e. the root container),
//      I return an empty table with zero rows in it.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetContentsTable (ULONG         ulFlags,
                                             LPMAPITABLE * ppTable)
{
    InfoTrace ("CABContainer::GetContentsTable method called");
    Validate_IMAPIContainer_GetContentsTable (this, ulFlags, ppTable);
    
    if (MAPI_ASSOCIATED & ulFlags)
    {
        TraceMessage ("CABContainer::GetContentsTable: Associated contents tables are not supported in AB's");
        return MAPI_E_NO_SUPPORT;
    }
    if (MAPI_UNICODE & ulFlags)
    {
        TraceMessage ("CABContainer::GetContentsTable: This AB does not support UNICODE yet");
        return MAPI_E_BAD_CHARWIDTH;
    }

    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    hResult = GetGALTable();
    if (!hResult)
    {
        hResult = m_pContTable->HrGetView (NULL, NULL, 0, ppTable);
        if (!hResult && (m_ulObjFlags & OBJECT_MAY_HAVE_RECIPIENTS))
        {
            CTable * pPrivTable = new CTable (*ppTable);
            (*ppTable)->Release(); // The constructor did and AddRef() in this object
            if (pPrivTable)
            {
                // The contents table returned is used by the MAPI UI to display the table rows.
                // The UI will take the table in the returned stated and display ALL the columns
                // that is can, in the UI.
                // This means that MAPI will NOT sort the table or rearrange the colums before
                // diplaying it. We must do that if we want specific columns to show up and
                // in what order and sort.
                pPrivTable->SetColumns ((LPSPropTagArray)&sptDefaultContTableCols, TBL_BATCH);
                pPrivTable->SortTable (&sosDefaultSort, TBL_BATCH);
                *ppTable = (LPMAPITABLE)pPrivTable;
            }
            else
            {
                hResult = E_OUTOFMEMORY;
                TraceMessage ("CABContainer::GetContentsTable: Failed to create a new private table object");
            }
            
        }
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CABContainer::GetContentsTable", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetHierarchyTable()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Returns a READ-only table for the subcontainers in this container.
//      If the container does not have any subcontainers because logically,
//      there shouldn't be any (i.e. the GAL container), I return an empty
//      table with zero rows in it.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetHierarchyTable (ULONG         ulFlags,
                                              LPMAPITABLE * ppTable)
{
    InfoTrace ("CABContainer::GetHierarchyTable method called");
    Validate_IMAPIContainer_GetHierarchyTable (this, ulFlags, ppTable);

    if (MAPI_UNICODE & ulFlags)
    {
        TraceMessage ("CABContainer::GetHierarchyTable: This AB does not support UNICODE yet");
        return MAPI_E_BAD_CHARWIDTH;
    }

    BOOL fReturnEmptyTable = (m_ulObjFlags & OBJECT_MAY_HAVE_CONTAINERS ? FALSE : TRUE);
    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    if (!m_pHierTable)
    {
        hResult = m_pBEobj->GetTable (TABLE_HIERARCHY,
                                      m_ObjectEID,
                                      0,
                                      (LPVOID)ulFlags,
                                      (LPVOID)fReturnEmptyTable,
                                      &m_pHierTable);
    }
    if (!hResult)
    {
        hResult = m_pHierTable->HrGetView (NULL, NULL, 0, ppTable);
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CABContainer::GetHierarchyTable", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::OpenEntry()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Open an entry in this container. In our implementation I delegate
//      the call to the linked and contained IABLogon object so that all the
//      open entry code happens in one place. Note however that this
//      implementation has a side effect: Any entry can be open in any
//      container without enforcing a check that object is indeed a child
//      object of this container. This is not a issue in most implementation
//      but if it is for your, think about sharing as much code as possible.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::OpenEntry (ULONG       cbEntryID,
                                      LPENTRYID   pEntryID,
                                      LPCIID      pInterface,
                                      ULONG       ulFlags,
                                      ULONG *     pulObjType,
                                      LPUNKNOWN * ppUnk)
{
    InfoTrace ("CABContainer::OpenEntry method called");
    Validate_IMAPIContainer_OpenEntry (this,
                                       cbEntryID,
                                       pEntryID,
                                       pInterface,
                                       ulFlags,
                                       pulObjType,
                                       ppUnk);
    // Delegate the call to the IABLogon object
    return m_pLogon->OpenEntry (cbEntryID, pEntryID, pInterface, ulFlags, pulObjType, ppUnk);
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::SetSearchCriteria()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. AB container do implement this method. For AB Finder
//      containers, the search criteria is set by setting the properties
//      in the details page of the container, getting the SRestriction using
//      IMAPIContainer::GetSearchCriteria, and then restricting the contents
//      table of the container where PR_SEARCH was opened.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always
//
STDMETHODIMP CABContainer::SetSearchCriteria (LPSRestriction     pRestriction,
                                              LPENTRYLIST        pContainerList,
                                              ULONG              ulSearchFlags)
{
    InfoTrace ("CABContainer::SetSearchCriteria method called");
    Validate_IMAPIContainer_SetSearchCriteria (this, pRestriction, pContainerList, ulSearchFlags);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetSearchCriteria()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      For non-AB Finder containers, the call is just a stub method that
//      always returns MAPI_E_NO_SUPPORT.
//      For AB Finders, we return an SRestricion built with the set properties
//      (IMAPIContainer::SetProps) that the client collected from the details
//      page of the container.
//      If non of these properties have been set, then we return
//      MAPI_E_NOT_INITIALIZED meaning that the container does not have the
//      needed properties to build a valid search criteria restriction.
//      
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetSearchCriteria (ULONG            ulFlags,
                                              LPSRestriction * ppRestriction,
                                              LPENTRYLIST *    ppContainerList,
                                              ULONG *          pulSearchState)
{
    InfoTrace ("CABContainer::GetSearchCriteria method called");
    Validate_IMAPIContainer_GetSearchCriteria (this,
                                               ulFlags,
                                               pRestriction,
                                               pContainerList,
                                               ulSearchFlags);
    if (!(OBJECT_IS_SEARCH_CONTAINER & m_ulObjFlags))
    {
        TraceMessage ("CABContainer::GetSearchCriteria: Cannot get the search criteria of a regular container");
        return MAPI_E_NO_SUPPORT;
    }
    SBinaryArray * pEIDList = NULL;
    LPSBinary pEID;
    HRESULT hResult = S_OK;
    // If the caller asked for a list of container where the list will happens, then
    // return the entry ID of the parent of this search container.
    if (ppContainerList)
    {
        hResult = PrivMAPIAllocate (sizeof(SBinaryArray), (LPVOID *)&pEIDList);
        if (!hResult)
        {
            hResult = PrivMAPIAllocate (sizeof(SBinary), (LPVOID *)&pEID, pEIDList); // MAPI allocate more
            if (!hResult)
            {
                hResult = PrivMAPIAllocate (CB_PRIVATE_EID, (LPVOID *)&(pEID->lpb), pEIDList); // MAPI allocate more
                if (!hResult)
                {
                    pEID->cb = CB_PRIVATE_EID;
                    CopyMemory (pEID->lpb, &m_ParentEID, CB_PRIVATE_EID);
                    pEIDList->cValues = 1;
                    pEIDList->lpbin = pEID;
                }
            }
        }
        if (hResult)
        {
            PrivMAPIFree ((LPVOID *)&pEIDList);
            TraceResult ("CABContainer::GetSearchCriteria: Allocating EID list", hResult);
        }
        else
        {
            *ppContainerList = (LPENTRYLIST)pEIDList;
        }
    }
    // The search state is ALWAYS zero in this implementation
    if (pulSearchState)
    {
        *pulSearchState = 0;
    }
    // If no restriction was asked for, return
    if (NULL == ppRestriction)
    {
        return hResult;
    }

    EnterCriticalSection (&m_csObj);

    LPSRestriction pRest = NULL, pSubRes;
    LPSPropValue pProp;
    
    ULONG ulResCount = 0;
    SPropValue spvAccount = { 0 }, spvName = { 0 }, spvPhone = { 0 }, spvOffice = { 0 };
    // To build the restriction we need at least ONE of the following properties set on the container
    if (S_OK == m_pList->Find (PR_ACCOUNT, &spvAccount, TRUE))
    {
        if (lstrlen (spvAccount.Value.lpszA))
        {
            ulResCount++;
        }
    }
    if (S_OK == m_pList->Find (PR_DISPLAY_NAME, &spvName, TRUE))
    {
        if (lstrlen (spvName.Value.lpszA))
        {
            ulResCount++;
        }
    }
    if (S_OK == m_pList->Find (PR_OFFICE_TELEPHONE_NUMBER, &spvPhone, TRUE))
    {
        if (lstrlen (spvPhone.Value.lpszA))
        {
            ulResCount++;
        }
    }
    if (S_OK == m_pList->Find (PR_OFFICE_LOCATION, &spvOffice, TRUE))
    {
        if (lstrlen (spvOffice.Value.lpszA))
        {
            ulResCount++;
        }
    }
    // If we got at least ONE properties set AND retrieved, build the SRestriction for the search
    if (ulResCount)
    {
        // The memory for the restriction and the subrestriction and properties is linked
        // all to a single block that the client should release
        hResult = PrivMAPIAllocate (sizeof(SRestriction), (LPVOID *)&pRest);
        if (!hResult)
        {
            hResult = PrivMAPIAllocate (sizeof(SRestriction)*ulResCount, (LPVOID *)&pSubRes, pRest); // MAPI Allocate more
            if (!hResult)
            {
                // If the number of restriction is more that one, the AND the properties
                if (ulResCount > 1)
                {
                    pRest->rt = RES_AND;
                    pRest->res.resAnd.cRes = ulResCount;
                    pRest->res.resAnd.lpRes = pSubRes;
                }
                else
                {
                    // ... otherwise, OR the one property we have
                    pRest->rt = RES_OR;
                    pRest->res.resOr.cRes = ulResCount;
                    pRest->res.resOr.lpRes = pSubRes;
                }
                if (PR_ACCOUNT == spvAccount.ulPropTag)
                {
                    ulResCount--;
                    hResult = PrivMAPIAllocate (sizeof(SPropValue), (LPVOID *)&pProp, pRest); // MAPI Allocate more
                    if (!hResult)
                    {
                        hResult = PropCopyMore (pProp, &spvAccount, GetMAPIAllocMore(), pRest);
                        if (!hResult)
                        {
                            pSubRes[ulResCount].rt = RES_CONTENT;
                            pSubRes[ulResCount].res.resContent.ulFuzzyLevel = FL_FULLSTRING | FL_IGNORECASE;
                            pSubRes[ulResCount].res.resContent.ulPropTag = pProp->ulPropTag;
                            pSubRes[ulResCount].res.resContent.lpProp = pProp;
                        }
                    }
                }
                if (!hResult && PR_DISPLAY_NAME == spvName.ulPropTag)
                {
                    ulResCount--;
                    hResult = PrivMAPIAllocate (sizeof(SPropValue), (LPVOID *)&pProp, pRest); // MAPI Allocate more
                    if (!hResult)
                    {
                        hResult = PropCopyMore (pProp, &spvName, GetMAPIAllocMore(), pRest);
                        if (!hResult)
                        {
                            pSubRes[ulResCount].rt = RES_CONTENT;
                            pSubRes[ulResCount].res.resContent.ulFuzzyLevel = FL_SUBSTRING | FL_IGNORECASE;
                            pSubRes[ulResCount].res.resContent.ulPropTag = pProp->ulPropTag;
                            pSubRes[ulResCount].res.resContent.lpProp = pProp;
                        }
                    }
                }
                if (!hResult && PR_OFFICE_TELEPHONE_NUMBER == spvPhone.ulPropTag)
                {
                    ulResCount--;
                    hResult = PrivMAPIAllocate (sizeof(SPropValue), (LPVOID *)&pProp, pRest); // MAPI Allocate more
                    if (!hResult)
                    {
                        hResult = PropCopyMore (pProp, &spvPhone, GetMAPIAllocMore(), pRest);
                        if (!hResult)
                        {
                            pSubRes[ulResCount].rt = RES_CONTENT;
                            pSubRes[ulResCount].res.resContent.ulFuzzyLevel = FL_SUBSTRING | FL_IGNORECASE;
                            pSubRes[ulResCount].res.resContent.ulPropTag = pProp->ulPropTag;
                            pSubRes[ulResCount].res.resContent.lpProp = pProp;
                        }
                    }
                }
                if (!hResult && PR_OFFICE_LOCATION == spvOffice.ulPropTag)
                {
                    ulResCount--;
                    hResult = PrivMAPIAllocate (sizeof(SPropValue), (LPVOID *)&pProp, pRest); // MAPI Allocate more
                    if (!hResult)
                    {
                        hResult = PropCopyMore (pProp, &spvOffice, GetMAPIAllocMore(), pRest);
                        if (!hResult)
                        {
                            pSubRes[ulResCount].rt = RES_CONTENT;
                            pSubRes[ulResCount].res.resContent.ulFuzzyLevel = FL_SUBSTRING | FL_IGNORECASE;
                            pSubRes[ulResCount].res.resContent.ulPropTag = pProp->ulPropTag;
                            pSubRes[ulResCount].res.resContent.lpProp = pProp;
                        }
                    }
                }
            }
        }
        if (hResult)
        {
            // And error occurred, bail out of here after cleaning up
            PrivMAPIFree ((LPVOID *)&pRest);
            TraceResult ("CABContainer::GetSearchCriteria: Allocating search restriction", hResult);
        }
        else
        {
            *ppRestriction = pRest;
        }
    }
    else
    {
        hResult = MAPI_E_NOT_INITIALIZED;
    }
    if (hResult)
    {
        PrivMAPIFree ((LPVOID *)&pEIDList);
    }

    LeaveCriticalSection (&m_csObj);

    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
// IABContainer virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::CreateEntry()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. Our implementation does not support creating entries
//      because we are read-only and server based. The creation of entries
//      occurs on the server.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always
//
STDMETHODIMP CABContainer::CreateEntry (ULONG        cbEntryID,
                                        LPENTRYID    pEntryID,
                                        ULONG        ulCreateFlags,
                                        LPMAPIPROP * ppMAPIPropEntry)
{
    InfoTrace ("CABContainer::CreateEntry method called");
    Validate_IABContainer_CreateEntry (this,
                                       cbEntryID,
                                       pEntryID,
                                       ulCreateFlags,
                                       ppMAPIPropEntry);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::CopyEntries()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. Our implementation does not support copying entries
//      because we are read-only and server based.
//
//    Return Value
//      E_ACCESSDENIED always
//
STDMETHODIMP CABContainer::CopyEntries (LPENTRYLIST    pEntries,
                                        ULONG          ulUIParam,
                                        LPMAPIPROGRESS pProgress,
                                        ULONG          ulFlags)
{
    InfoTrace ("CABContainer::CopyEntries method called");
    Validate_IABContainer_CopyEntries (this,
                                       pEntries,
                                       ulUIParam,
                                       pProgress,
                                       ulFlags);
    return E_ACCESSDENIED;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::DeleteEntries()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. Our implementation does not support deleting entries
//      because we are read-only and server based. The deletion of entries
//      occurs on the server.
//
//    Return Value
//      E_ACCESSDENIED always
//
STDMETHODIMP CABContainer::DeleteEntries (LPENTRYLIST pEntries,
                                          ULONG       ulFlags)
{
    InfoTrace ("CABContainer::DeleteEntries method called");
    Validate_IABContainer_DeleteEntries (this, pEntries, ulFlags);
    return E_ACCESSDENIED;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::ResolveNames()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Given a user name, resolve the entry in our address book by search a
//      matching recipient and returning all the properties requested.
//      First our implementation tries to do a match on PR_ACCOUNT and if
//      there is no hit, I do a more detail search on PR_DISPLAY_NAME with
//      partial name resolution.
//      If more that one address book entry start with the pattern supplied by
//      the caller, I return the entry as an ambiguous recipient in which
//      case MAPI will do a PR_ANR restriction on the contents table of this
//      container.
//      If the object is a distribution list, I return MAPI_E_NO_SUPPORT.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::ResolveNames (LPSPropTagArray pPropTagArray,
                                         ULONG           ulFlags,
                                         LPADRLIST       pAdrList,
                                         LPFlagList      pFlagList)
{
    InfoTrace ("CABContainer::ResolveNames method called");
    Validate_IABContainer_ResolveNames (this,
                                        pPropTagArray,
                                        ulFlags,
                                        pAdrList,
                                        pFlagList);
    if (MAPI_ABCONT != m_ObjectEID.bObject)
    {
        TraceMessage ("CABContainer::ResolveNames: I don't resolve names in non-ab container objects");
        return MAPI_E_NO_SUPPORT;
    }
    LPSPropValue pProps, pOneProp;
    LPSRowSet pRows;
    LPSRow pOneRow;
    LPTSTR pszName;
    ULONG k, j, ulReqPropTag, ulPropCount = 0;
    BOOL fNameResolved;
    DWORD dwInstID;
    HRESULT hPropError, hOpError, hSearchResult, hResult = S_OK;
    SPropValue spvString = { 0 }, spvInstKey = { 0 };

    spvInstKey.ulPropTag = PR_INSTANCE_KEY;
    spvInstKey.Value.bin.cb = sizeof(DWORD);
    spvInstKey.Value.bin.lpb = (LPBYTE)&dwInstID;
    
    SRestriction SRes = { 0 };
    SRes.rt = RES_CONTENT;
    SRes.res.resContent.lpProp = &spvString;
    EnterCriticalSection (&m_csObj);
    // Get a view on the contents table data of this container
    LPMAPITABLE pTable;
    hResult = GetGALTable();
    if (!hResult)
    {
        hResult = m_pContTable->HrGetView (NULL, NULL, 0, &pTable);
        if (!hResult)
        {
            hResult = pTable->SetColumns ((LPSPropTagArray)&sptResolveNamesCols, TBL_BATCH);
            if (!hResult)
            {
                // Sort the table so that we can use it for resolving the name using FindRow()
                hResult = pTable->SortTable (&sosDefaultSort, TBL_BATCH);
            }
        }
    }
    if (!hResult)
    {
        ASSERTMSG(pTable, "Where is the view of the contents table?");
        for (k=0; k<pAdrList->cEntries; k++)
        {
            // Skip the entires with 0 properties OR entries that have been resolved by other address books
            if (0 == pAdrList->aEntries[k].cValues || MAPI_UNRESOLVED != pFlagList->ulFlag[k])
            {
                continue;
            }
            fNameResolved = FALSE;
            dwInstID = 0;
            // Get the display name we are resolving
            pOneProp = PpropFindProp (pAdrList->aEntries[k].rgPropVals, pAdrList->aEntries[k].cValues, PR_DISPLAY_NAME);
            ASSERTMSG(pOneProp, "Missing PR_DISPLAY_NAME in one of the passed ADRLIST entries");
            if (!pOneProp)
            {
                continue;
            }
            SRes.res.resContent.ulFuzzyLevel = FL_FULLSTRING | FL_IGNORECASE;
            SRes.res.resContent.ulPropTag = spvString.ulPropTag = PR_ACCOUNT;
            pszName = pOneProp->Value.lpszA;
            // Skip any beginning blanks or tabs in the name to resolve
            while (*pszName == ' ' || *pszName == '\t')
            {
                pszName++;
            }
            spvString.Value.lpszA = pszName;
            // Try a full hit or PR_ACCOUNT
            hSearchResult = pTable->FindRow (&SRes, BOOKMARK_BEGINNING, 0);
            if (S_OK == hSearchResult)
            {
                // Hit on PR_ACCOUNT I don't need to do anything else here.
                fNameResolved = TRUE;
            }
            else
            {
                // The display name does not match any account name
                if (MAPI_E_NOT_FOUND == hSearchResult)
                {
                    // Search on ALL the names in the table that meet this criteria. Use sub-string search.
                    SRes.res.resContent.ulFuzzyLevel = FL_SUBSTRING | FL_IGNORECASE;
                    SRes.res.resContent.ulPropTag = spvString.ulPropTag = PR_DISPLAY_NAME;
                    hSearchResult = pTable->FindRow (&SRes, BOOKMARK_BEGINNING, 0);
                    // If we found at least ONE name that matched the sub restriction, process the rows in the table
                    if (S_OK == hSearchResult)
                    {
                        // Use the helper function
                        ResolveOneName (pszName, pTable, pFlagList->ulFlag[k], dwInstID);
                        if (dwInstID)
                        {
                            fNameResolved = TRUE;
                        }
                    }
                }
                // Trace out any error other than NOT FOUND
                if (hSearchResult && MAPI_E_NOT_FOUND != hSearchResult)
                {
                    TraceResult ("CABContainer::ResolveNames: Failed to find rows in the table", hSearchResult);
                }
            }
            // Get got a full hit and the name can be resolved in this address book
            if (fNameResolved)
            {
                hOpError = S_OK;
                // If the instance is 0 is because the hit was on the ACCOUNT name and we need to get the current row in the table
                if (0 == dwInstID)
                {
                    hOpError = pTable->QueryRows (1, 0, &pRows);
                    if (!hOpError)
                    {
                        ASSERTMSG (pRows->cRows == 1, "But I asked for 1 row in the RN View?!");
                        ASSERTMSG (pRows->aRow[0].cValues == 3, "But I asked for 3 properties from the RN View?!");
                        ASSERTMSG (pRows->aRow[0].lpProps[2].ulPropTag == PR_INSTANCE_KEY, "Where is PR_INSTANCE_KEY in the RN view?!");
                        CopyMemory (&dwInstID, pRows->aRow[0].lpProps[2].Value.bin.lpb, sizeof(DWORD));
                    }
                }
                else
                {
                    pRows = NULL;
                }
                if (!hOpError)
                {
                    // the svpInstKey property has the instance key of the row where the hit occured. With it,
                    // get all the properties of the row on the ITableData object
                    hOpError = m_pContTable->HrQueryRow (&spvInstKey, &pOneRow, NULL);
                    if (!hOpError)
                    {
                        // See if this property was passed in the entry of the ADRLIST we are processing
                        ulPropCount = pPropTagArray->cValues;
                        if (PpropFindProp (pAdrList->aEntries[k].rgPropVals, pAdrList->aEntries[k].cValues, PR_RECIPIENT_TYPE))
                        {
                            ulPropCount++;
                        }
                        // Allocate the array for the new entry properties
                        hOpError = PrivMAPIAllocate (ulPropCount*sizeof(SPropValue), (LPVOID *)&pProps);
                        if (!hOpError)
                        {
                            // Returned the properties in the same order as we got asked for in the original tag array
                            for (j=0; j<pPropTagArray->cValues; j++)
                            {
                                ulReqPropTag = pPropTagArray->aulPropTag[j];
                                // The transmittable display name and the 7-bit display name, in our implementation,
                                // is the same as the display name
                                if (PR_TRANSMITABLE_DISPLAY_NAME == ulReqPropTag ||
                                    PR_7BIT_DISPLAY_NAME == ulReqPropTag)
                                {
                                    ulReqPropTag = PR_DISPLAY_NAME;
                                }
                                pOneProp = PpropFindProp (pOneRow->lpProps, pOneRow->cValues, ulReqPropTag);
                                if (pOneProp)
                                {
                                    hPropError = PropCopyMore (&pProps[j], pOneProp, GetMAPIAllocMore(), pProps);
                                }
                                else
                                {
                                    TraceRaw ("CABContainer::ResolveNames: Properties not found: ");
                                    TraceProp (pPropTagArray->aulPropTag[j]);
                                    hPropError = MAPI_E_NOT_FOUND;
                                }
                                if (hPropError)
                                {
                                    pProps[j].ulPropTag = PROP_TAG(PT_ERROR, PROP_ID(pPropTagArray->aulPropTag[j]));
                                    pProps[j].Value.err = hPropError;
                                }
                                else
                                {
                                    pProps[j].ulPropTag = pPropTagArray->aulPropTag[j];
                                }
                            }
                        }
                        PrivMAPIFree ((LPVOID *)&pOneRow);
                    }
                    if (pRows)
                    {
                        FreeProws (pRows);
                    }
                }
                if (!hOpError)
                {
                    pOneProp = PpropFindProp (pAdrList->aEntries[k].rgPropVals, pAdrList->aEntries[k].cValues, PR_RECIPIENT_TYPE);
                    if (pOneProp)
                    {
                        pProps[ulPropCount-1].ulPropTag = pOneProp->ulPropTag;
                        pProps[ulPropCount-1].Value = pOneProp->Value;
                    }
                    // Now replace the entry with the new property array and set the flag to fully resolve (MAPI_RESOLVED)
                    PrivMAPIFree ((LPVOID *)&(pAdrList->aEntries[k].rgPropVals));
                    pAdrList->aEntries[k].rgPropVals = pProps;
                    pAdrList->aEntries[k].cValues = ulPropCount;
                    pFlagList->ulFlag[k] = MAPI_RESOLVED;
                }
                TraceResult ("CABContainer::ResolveNames: Failed to copy the properties of a resolved name", hOpError);
            }
        }
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CABContainer::ResolveNames", hResult);
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::ResolveOneName()
//
//    Parameters
//      pszName         Pointer to a string with the name we are resolving
//      pTable          IMAPITable restricted and at the position where the
//                      FindRow() found a partial hit for name we are resolving
//      ulResolution    Where we return the resolve flag: Weather is ambiguous,
//                      fully resolved or, not resolved at all.
//      dwInstID        If the name was fully resolved we return the
//                      dwInstanceKey of the row where the name was found.
//
//    Purpose
//      Once FindRow() found hits on a particular entry we are resolving,
//      the table is placed at the first row of the hit.
//      To resolve a name I only have to walk TWO rows to determine
//      it resolution.
//      This function walks those row trying to determine if the row is
//      fully resolved or an ambiguity exists.
//      i.e.
//          If we are resolving "Irving" and the table has "Irving De la Cruz"
//          and "Irving Thomas", the function find and ambiguity.
//          If we are resolving "Irving D" and the table has
//          "Irving De la Cruz" and "Irving Thomas", the function find a full
//          hit on "Irving De la Cruz"
//
//    Return Value
//      None.
//
void WINAPI CABContainer::ResolveOneName (LPTSTR      pszName,
                                          LPMAPITABLE pTable,
                                          ULONG &     ulResolution,
                                          DWORD &     dwInstID)
{
    dwInstID = 0;
    BOOL fExactMatch = FALSE;
    HRESULT hFindResult = S_OK;
    LPSRowSet pRows;
    DWORD dwHitInstID;
    // If the length of the name is greater that this size, the entry does not exist in our address book
    if (lstrlen (pszName) > MAX_STRING_SIZE)
    {
        return;
    }
    // Use this temporary buffer to manipulate the string without touching the original sources.
    TCHAR szUserName[MAX_STRING_SIZE+1], szABName[MAX_STRING_SIZE+1];
    // Copy the incoming name ot a local buffer and change it to upper case
    lstrcpy (szUserName, pszName);
    CharUpper (szUserName);
    while (S_OK == hFindResult)
    {
        hFindResult = pTable->QueryRows (1, 0, &pRows);
        if (!hFindResult)
        {
            if (pRows->cRows)
            {
                ASSERTMSG (pRows->cRows == 1, "But I only asked for ONE row?!?!");
                ASSERTMSG (pRows->aRow[0].lpProps[0].ulPropTag == PR_DISPLAY_NAME, "Where is PR_DISPLAY_NAME?! I can't resolve the a name without this");
                // Copy the row name ot a local buffer and change it to upper case
                lstrcpy (szABName, pRows->aRow[0].lpProps[0].Value.lpszA);
                CharUpper (szABName);
                // If the first letter are the same, compare the name
                if (szABName[0] == szUserName[0])
                {
                    // Compare the string UP TO THE LENGTH of the incoming name
                    if (0 == memcmp (szABName, szUserName, lstrlen (szUserName)))
                    {
                        // if the name has been matched in a previous pass of the loop, the name is ambiguous
                        if (fExactMatch)
                        {
                            hFindResult = S_FALSE;
                            ulResolution = MAPI_AMBIGUOUS;
                        }
                        else
                        {
                            // Mark the entry as fully resolved
                            ulResolution = MAPI_RESOLVED;
                            fExactMatch = TRUE;
                            CopyMemory (&dwHitInstID, pRows->aRow[0].lpProps[2].Value.bin.lpb, sizeof(DWORD));
                        }
                    }
                }
                else
                {
                    // Not need to continue
                    hFindResult = S_FALSE;
                }
            }
            else
            {
                // No more rows in the table
                hFindResult = S_FALSE;
            }
            FreeProws (pRows);
        }
    }
    // If the QueryRow() call failed, at least tell the system that a name could exist in our address book but we failed internally.
    if (FAILED(hFindResult))
    {
        ulResolution = MAPI_AMBIGUOUS;
        TraceResult ("CABContainer::ResolveOneName: QueryRows() failed defalting to MAPI_AMBIGUOUS", hFindResult);
    }
    // If the name was resolved, return the instance key of the row where the hit occurred.
    if (MAPI_RESOLVED == ulResolution)
    {
        dwInstID = dwHitInstID;
    }
}

///////////////////////////////////////////////////////////////////////////////
//    CABContainer::GetGALTable()
//
//    Parameters
//      None
//
//    Purpose
//      For the GAL (Global Address List) AB container, we return the contents
//      table which is cached in the logon session object. This saves us from
//      the requesting the server for a new one everytime we need to resolve
//      a name. If the object is not the GAL container or the table has not
//      been cached yet, we request the back end object to go to the server
//      and get the data for one.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CABContainer::GetGALTable()
{
    if (!m_pContTable && (GAL_CONTAINER_ID == m_ObjectEID.dwObjID))
    {
        m_pLogon->GetCachedGAL (&m_pContTable);
    }
    if (m_pContTable)
    {
        return S_OK;
    }
    // If the container is not supposed to have recipients, we return an
    // empty table and the back end does not have to go the server to request anything.
    BOOL fReturnEmptyTable = (m_ulObjFlags & OBJECT_MAY_HAVE_RECIPIENTS ? FALSE : TRUE);
    HRESULT hResult = m_pBEobj->GetTable (TABLE_CONTENTS,
                                          m_ObjectEID,
                                          0,
                                          NULL,
                                          (LPVOID)fReturnEmptyTable,
                                          &m_pContTable);
    if (!hResult && (GAL_CONTAINER_ID == m_ObjectEID.dwObjID))
    {
        m_pLogon->SetCachedGAL (m_pContTable);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//
// Implementation of the IDistList object for template ID
//

///////////////////////////////////////////////////////////////////////////////
//    CDLTemplate::OpenProperty()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      This is one method we implement to wrap the IMAPIProp interface
//      passed to us in the IABLogon::OpenTemplateID. We only support
//      template ID on distribution lists and only ONE property to be opened
//      in templates: PR_CONTAINER_CONTENTS.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CDLTemplate::OpenProperty (ULONG       ulPropTag,
                                        LPCIID      piid,
                                        ULONG       ulInterfaceOptions,
                                        ULONG       ulFlags,
                                        LPUNKNOWN * ppUnk)
{
    InfoTrace ("CDLTemplate::OpenProperty method called");
    Validate_IMAPIProp_OpenProperty (this,
                                     ulPropTag, 
                                     piid,
                                     ulInterfaceOptions,
                                     ulFlags,
                                     ppUnk);
    HRESULT hResult = S_OK;
    InfoTraceRaw ("CDLTemplate::OpenProperty: Property to open: "); InfoTraceProp (ulPropTag);
    EnterCriticalSection (&m_csObj);
    if (PR_CONTAINER_CONTENTS == ulPropTag)
    {
        if (PR_CONTAINER_CONTENTS == ulPropTag ||
            PR_CONTAINER_HIERARCHY == ulPropTag)
        {
            if (IID_IMAPITable != *piid)
            {
                TraceMessage ("CDLTemplate::OpenProperty: I only support the IMAPITable interface on these properties");
                hResult = E_NOINTERFACE;
                goto ErrorExit;
            }
        }
        if (PR_CONTAINER_HIERARCHY == ulPropTag)
        {
            hResult = GetHierarchyTable (ulFlags, (LPMAPITABLE *)ppUnk);
        }
        if (PR_CONTAINER_CONTENTS == ulPropTag)
        {
            // Return a contents table of our distribution list on the
            // foreign address book
            hResult = GetContentsTable (ulFlags, (LPMAPITABLE *)ppUnk);
            if (!hResult)
            {
                // We set the columns of the container so that the MAPI address book dialog
                // display the columns in our default order.
                ((LPMAPITABLE)(*ppUnk))->SetColumns ((LPSPropTagArray)&sptDLMembersLbx, 0);
            }
        }
ErrorExit:
        TraceResult ("CDLTemplate::OpenProperty", hResult);
    }
    else
    {
        hResult = m_pImpObj->OpenProperty (ulPropTag, piid, ulInterfaceOptions, ulFlags, ppUnk);
    }
    LeaveCriticalSection (&m_csObj);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDLTemplate::GetContentsTable()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Returns a READ-only table for the contained mail users in this DL.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CDLTemplate::GetContentsTable (ULONG         ulFlags,
                                            LPMAPITABLE * ppTable)
{
    InfoTrace ("CDLTemplate::GetContentsTable method called");
    Validate_IMAPIContainer_GetContentsTable (this, ulFlags, ppTable);

    if (MAPI_ASSOCIATED & ulFlags)
    {
        TraceMessage ("CDLTemplate::GetContentsTable: Associated contents tables are not supported in AB's");
        return MAPI_E_NO_SUPPORT;
    }
    if (MAPI_UNICODE & ulFlags)
    {
        TraceMessage ("CDLTemplate::GetContentsTable: This AB does not support UNICODE yet");
        return MAPI_E_BAD_CHARWIDTH;
    }

    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    if (!m_pContTable)
    {
        hResult = m_pBEobj->GetTable (TABLE_CONTENTS,
                                      m_ObjectEID,
                                      0,
                                      NULL,
                                      (LPVOID)FALSE,
                                      &m_pContTable);
    }
    if (!hResult)
    {
        hResult = m_pContTable->HrGetView (NULL, NULL, 0, ppTable);
        if (!hResult)
        {
            // Arrange the contents for the UI
            (*ppTable)->SetColumns ((LPSPropTagArray)&sptDLMembersLbx, 0);
            (*ppTable)->SortTable (&sosDefaultSort, TBL_BATCH);
        }
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CDLTemplate::GetContentsTable", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDLTemplate::GetHierarchyTable()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Returns a an empty table with zero rows in it.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CDLTemplate::GetHierarchyTable (ULONG         ulFlags,
                                             LPMAPITABLE * ppTable)
{
    InfoTrace ("CDLTemplate::GetHierarchyTable method called");
    Validate_IMAPIContainer_GetHierarchyTable (this, ulFlags, ppTable);
    
    if (MAPI_UNICODE & ulFlags)
    {
        TraceMessage ("CDLTemplate::GetHierarchyTable: This AB does not support UNICODE yet");
        return MAPI_E_BAD_CHARWIDTH;
    }

    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    if (!m_pHierTable)
    {
        hResult = m_pBEobj->GetTable (TABLE_HIERARCHY,
                                      m_ObjectEID,
                                      0,
                                      (LPVOID)ulFlags,
                                      (LPVOID)TRUE,
                                      &m_pHierTable);
    }
    if (!hResult)
    {
        hResult = m_pHierTable->HrGetView (NULL, NULL, 0, ppTable);
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CDLTemplate::GetHierarchyTable", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDLTemplate::OpenEntry()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Open an entry in this container. In our implementation I delegate
//      the call to the linked and contained IABLogon object so that all the
//      open entry code happens in one place. Note however that this
//      implementation has a side effect: Any entry can be open in any
//      container without enforcing a check that object is indeed a child
//      object of this container. This is not a issue in most implementation
//      but if it is for your, think about sharing as much code as possible.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CDLTemplate::OpenEntry (ULONG       cbEntryID,
                                     LPENTRYID   pEntryID,
                                     LPCIID      pInterface,
                                     ULONG       ulFlags,
                                     ULONG *     pulObjType,
                                     LPUNKNOWN * ppUnk)
{
    InfoTrace ("CDLTemplate::OpenEntry method called");
    Validate_IMAPIContainer_OpenEntry (this,
                                       cbEntryID,
                                       pEntryID,
                                       pInterface,
                                       ulFlags,
                                       pulObjType,
                                       ppUnk);
    // Delegate the call to the IABLogon object
    return m_pLogon->OpenEntry (cbEntryID,
                                pEntryID,
                                pInterface,
                                ulFlags,
                                pulObjType,
                                ppUnk);
}

// End of file for ABCONT.CPP
