///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      ABSRVBE.H
//              
//  Description
//      This file implements a class derived from the abstract class CBackEnd
//      used by the MAPI sample providers to access the provider's data storage.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _ABSRVBE_H
#define _ABSRVBE_H

#include "PROPLIST.H"
#include "GENPROP.H"

#ifdef UNICODE
#error MAPI Samples: We don't support UNICODE yet... IrvingD
#endif // UNICODE

extern "C"
{
    HRESULT WINAPI OpenRemoteDataBase
                    (LPTSTR                 pszServer,
                     MAPIUID *              pUID,
                     BOOL                   fWorkOffLine,
                     LPTSTR                 pszABDLocalFile,
                     CBackEnd **            ppBackEnd);
};

class CABServerBackEnd : public CBackEnd
{
public:
///////////////////////////////////////////////////////////////////////////////
// CBackEnd members implemented here
//
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    STDMETHODIMP CloseResources
                    () { return S_OK; };
    STDMETHODIMP CreateObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID *          pNewObjEID,
                     CPropList **               ppList,
                     ULONG                      ulObjType,
                     LPVOID                     pData1,
                     LPVOID                     pData2)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP OpenObject
                    (PRIVATE_ENTRYID            ObjectEID,
                     CPropList **               ppList);
    STDMETHODIMP SaveObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID            ObjectEID,
                     CPropList *                pList,
                     COpenPropList *            pOpenList,
                     LPTABLEDATA                pTable)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP OpenPropertyInMemory
                    (PRIVATE_ENTRYID            ObjectEID,
                     ULONG                      ulMaxBytes,
                     LPVOID                     pMemBlock,
                     LPSPropValue               pProp)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP OpenProperty
                    (PRIVATE_ENTRYID            ObjectEID,
                     LPSPropValue               pProp,
                     LPUNKNOWN *                ppUnk)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP CopyObjects
                    (PRIVATE_ENTRYID            SrcParentContainerEID,
                     PRIVATE_ENTRYID            SrcContainerEID,
                     PRIVATE_ENTRYID            DstParentContainerEID,
                     PRIVATE_ENTRYID            DstContainerEID,
                     DWORD                      dwCopyFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP DeleteOneObject
                    (PRIVATE_ENTRYID            ObjectEID,
                     PRIVATE_ENTRYID            ParentEID)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP DeleteObjects
                    (PRIVATE_ENTRYID            ContainerEID,
                     ULONG                      ulDeleteFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP GetTable
                    (const TABLE_TYPE           tblType,
                     PRIVATE_ENTRYID            ContainerEID,
                     long                       lAccess,
                     LPVOID                     pData1,
                     LPVOID                     pData2,
                     LPTABLEDATA *              ppTable);
    STDMETHODIMP GetTableProps
                    (const TABLE_TYPE           tblType,
                     long                       lAccess,
                     PRIVATE_ENTRYID *          ParentEID,
                     PRIVATE_ENTRYID *          ObjectEID,
                     CPropList *                pList,
                     LPSPropValue *             ppProps,
                     DWORD &                    dwLastAllocated,
                     LPTABLEDATA                pTable,
                     LPVOID                     pData1,
                     LPVOID                     pData2)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP UpdateContentCount
                    (PRIVATE_ENTRYID            ContainerEID,
                     PRIVATE_ENTRYID            ObjectEID,
                     BOOL                       fAssociated,
                     BOOL                       fUnread)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP SetObjNameInContainer
                    (LPTSTR                     pszNewFolderName,
                     PRIVATE_ENTRYID            ParentFolderEID,
                     PRIVATE_ENTRYID            FolderEID)
                    { return MAPI_E_NO_SUPPORT; }

    STDMETHODIMP STDMETHODCALLTYPE SetReadFlags
                    (PRIVATE_ENTRYID            FolderEID,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP SetRFSEntryID
                    (LPTSTR                     pszMsgClass,
                     PRIVATE_ENTRYID            FolderEID,
                     LPTABLEDATA                pTable)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP GetRFSEntryID
                    (LPTSTR                     pszMsgClass,
                     PRIVATE_ENTRYID *          pFolderEID,
                     LPTSTR                     pszExplicitClass)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP GetNamedPropertiesStream
                    (LPSTREAM *                 ppStream)
                    { return MAPI_E_NO_SUPPORT; }
	STDMETHODIMP SetBackEndStrings
					(LPTSTR                     szName,
                     LPTSTR                     szComment)
					{ return MAPI_E_NO_SUPPORT; }

///////////////////////////////////////////////////////////////////////////////
// CABServerBackEnd members
//
public:
    inline void WINAPI GoOffLine
                    () { EnterCriticalSection (&m_csObj);
                         m_fUseLocalABD = TRUE;
                         LeaveCriticalSection (&m_csObj); }
    inline void WINAPI GoOnLine
                    () { EnterCriticalSection (&m_csObj);
                         m_fUseLocalABD = FALSE;
                         LeaveCriticalSection (&m_csObj); }
private:
    inline CPropList * WINAPI AllocatePropList
                    () { CPropList * pList = new CPropList; return pList; }
    STDMETHODIMP FillHierarchyTable
                    (PRIVATE_ENTRYID            ContainerEID,
                     DWORD                      dwFlags,
                     LPTABLEDATA                pTable);
    STDMETHODIMP FillContentsTable
                    (PRIVATE_ENTRYID            ContainerEID,
                     LPTABLEDATA                pTable);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CABServerBackEnd 
                    (MAPIUID *                  pUIDinFile,
                     LPTSTR                     pszRemoteServer,
                     BOOL                       fWorkOffLine,
                     LPTSTR                     pszABDLocalFile);
    ~CABServerBackEnd
                    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    ULONG               m_cRef;
    TCHAR               m_szServer[32];
    BOOL                m_fUseLocalABD;
    TCHAR               m_szABDLocalFile[MAX_PATH];
    LPTABLEDATA         m_pCachedLocalGAL;
};

enum
{
    CONT_INST_KEY,
    CONT_ADDRTYPE,
    CONT_OBJTYPE,
    CONT_ENTRYID,
    CONT_SKEY,
    CONT_DISPTYPE,
    CONT_ACCOUNT,
    CONT_EMAILADDR,
    CONT_DISPNAME,
    CONT_TITLE,
    CONT_OFFICE_LOC,
    CONT_OFFICE_PHONE,
    CONT_OTHER_PHONE,
    CONT_FAX,
    CONT_COMMENT,
    CONT_RTF,
    CONT_COMPANY,
    CONT_DEPT,
    CONT_MANAGER,
    CONTENTS_TABLE_PROPS
};

const static SizedSPropTagArray (CONTENTS_TABLE_PROPS, sptContainerContProps) =
{
    CONTENTS_TABLE_PROPS,
    {
        PR_INSTANCE_KEY,
        PR_ADDRTYPE,
        PR_OBJECT_TYPE,
        PR_ENTRYID,
        PR_SEARCH_KEY,
        PR_DISPLAY_TYPE,
        PR_ACCOUNT,
        PR_EMAIL_ADDRESS,
        PR_DISPLAY_NAME,
        PR_TITLE,
        PR_OFFICE_LOCATION,
        PR_OFFICE_TELEPHONE_NUMBER,
        PR_OTHER_TELEPHONE_NUMBER,
        PR_PRIMARY_FAX_NUMBER,
        PR_COMMENT,
        PR_SEND_RICH_INFO,
        PR_COMPANY_NAME,
        PR_DEPARTMENT_NAME,
        PR_WINDS_AB_MANAGER_NAME
    }
};

#endif // _ABSRVBE_H

// End of file for ABSRVBE.H
