///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      ABSRVCE.CPP
//
//  Description
//      This file implements some of the entry point (ABProviderInit and 
//      ServiceEntry) for this provider DLL.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#include "ABWDS.H"

// Remark this line to turn verbose tracing OFF
//#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)                TraceInfoMessage(a)
#define InfoTraceString1(a, b)      TraceString1(a, b)
#else
#define InfoTrace(a)
#define InfoTraceString1(a, b)
#endif // DO_INFO_TRACES

extern "C"
{
    void STDAPICALLTYPE DownloadABDirectory
                    (CUIMAPIProp *              pObj,
                     HWND                       hOwnerWnd,
                     HINSTANCE                  hInstance);
    LPTSTR WINAPI GetTimeString
                    (LPTSTR                     pszTime,
                     SYSTEMTIME                 SysTime);
    MSGSERVICEENTRY ServiceEntry;
    STDINITMETHODIMP ABProviderInit
                    (HINSTANCE                  hInstance, 
                     LPMALLOC                   pMallocObj,
                     LPALLOCATEBUFFER           pfnAllocateBuffer,
                     LPALLOCATEMORE             pfnAllocateMore,
                     LPFREEBUFFER               pfnFreeBuffer,
                     ULONG                      ulFlags,
                     ULONG                      ulMAPIVer,
                     ULONG *                    pulProviderVer,
                     LPABPROVIDER *             ppABProviderObj);

    MAPIUID guidABEntries = AB_UID_PROVIDER;
    MAPIUID guidXPEntries = XP_WINDS_AB_ENTRIES;
    TCHAR gszProviderName[] = "WINDS Address Book";
};
// Control description structures
DTBLEDIT DtComment       = { sizeof(DTBLEDIT),   fMapiUnicode, 64, PR_COMMENT };
DTBLEDIT DtLULLastUpdate = { sizeof(DTBLEDIT),   fMapiUnicode, 32, PR_SMP_LUL_LAST_UPDATE };
DTBLBUTTON DtDownLoadLUL = { sizeof(DTBLBUTTON), fMapiUnicode, PR_SMP_LUL_DOWNLOAD };

#define LAST_UPDATE_VALUE       "UI_LOCAL_ABD_LAST_UPDATE"
#define LAST_UPDATE_KEY         ((LPBYTE)LAST_UPDATE_VALUE)
#define CB_LAST_UPDATE_KEY      sizeof(LAST_UPDATE_VALUE)

// Description table for the controls in the second page
DTCTL ABConfigPage[] =
{
    {DTCT_PAGE,     0, NULL,            0,                  NULL,     0,                   &DtPage          },
    {DTCT_EDIT,     0, LAST_UPDATE_KEY, CB_LAST_UPDATE_KEY, szFilter, IDC_LUL_UPDATE_DATE, &DtLULLastUpdate },
    {DTCT_BUTTON,   0, NULL,            0,                  NULL,     IDC_LUL_DOWNLOAD,    &DtDownLoadLUL   },
    {DTCT_LABEL,    0, NULL,            0,                  NULL,     IDC_STATIC1,         &DtLabel         },
    {DTCT_GROUPBOX, 0, NULL,            0,                  NULL,     IDC_STATIC2,         &DtGroupBox      }
};

///////////////////////////////////////////////////////////////////////////////
//    ServiceEntry()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by the profile setup adminitration tool to display the provider
//      configuration properties for this message provider
//
//    Return Value
//      An HRESULT
//
HRESULT STDAPICALLTYPE ServiceEntry (HINSTANCE          hInstance,
                                     LPMALLOC           pMallocObj,
                                     LPMAPISUP          pSupObj,
                                     ULONG              ulUIParam,
                                     ULONG              ulFlags,
                                     ULONG              ulContext,
                                     ULONG              ulCfgPropCount,
                                     LPSPropValue       pCfgProps,
                                     LPPROVIDERADMIN    pAdminProvObj,
                                     LPMAPIERROR *      ppMAPIError)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    // Send output only to the COM1 port
    //InitTraces (0);
    // Send output to a console window BUT NOT to the COM1
    InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);
    // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);

    InfoTrace ("ServiceEntry function called");
    CGeneric obj;
    HRESULT hResult = S_OK;
    LPPROFSECT pProfileObj = NULL;
    ULONG ulPropCount;
    LPSPropValue pProps = NULL;
    HANDLE hUIMutex = NULL;
    // This is for our internal memory heap management
    BOOL fDeInitMemoryHeap = FALSE;
    
    // What context where were we called in?
    if (MSG_SERVICE_INSTALL         == ulContext ||
        MSG_SERVICE_UNINSTALL       == ulContext ||
        MSG_SERVICE_PROVIDER_CREATE == ulContext ||
        MSG_SERVICE_PROVIDER_DELETE == ulContext)
    {
        // We don't handle any of these contexts
        InfoTraceString1 ("ServiceEntry: Immediate return. Context: %d", ulContext);
        goto ErrorExit;
    }
    ghInstance = hInstance;
    InfoTraceString1 ("ServiceEntry: Context for call: %d", ulContext);
    hResult = InitMemoryHeap (pSupObj);
    if (FAILED(hResult)) // It may return a warning
    {
        return hResult;
    }
    if (S_OK == hResult)
    {
        fDeInitMemoryHeap = TRUE;
    }
        
    // Open the Profile Section for this service
    hResult = pAdminProvObj->OpenProfileSection (NULL,
                                                 NULL,
                                                 MAPI_MODIFY,
                                                 &pProfileObj);
    if (hResult)
    {
        goto ErrorExit;
    }

    if (MSG_SERVICE_CREATE == ulContext)
    {
        // If ServiceEntry is called because it the service is being created, we create a temporary
        // file where we will same the user address book directory if the user does a download of
        // the server address book database.
        TCHAR szTmpDir[512], szFile[512];
        GetTempPath (_MAX_PATH, szTmpDir);
        lstrcat (szTmpDir, WINDS_DATA_DIRECTORY);
        CreateDirectory (szTmpDir, NULL);
        GetTempFileName (szTmpDir,      // Create the file in the system temp directory
                         "ABD",   // Our address book fixed prefix for temp files
                         0,             // Use the time to create a pseudo-random number
                         szFile);       // Destination buffer
        CharUpper (szFile);
        ULONG i = 0;
        SPropValue spvProps[4] = { 0 };
        spvProps[i].ulPropTag = PR_SMP_LUL_FILENAME;
        spvProps[i++].Value.lpszA = szFile;
        spvProps[i].ulPropTag = PR_SMP_LUL_LAST_UPDATE;
        spvProps[i++].Value.lpszA = szBlank;
        spvProps[i].ulPropTag = PR_COMMENT;
        spvProps[i++].Value.lpszA = szBlank;
        spvProps[i].ulPropTag = PR_SMP_CONNECTION_TYPE;
        spvProps[i++].Value.l = LAN_CONNECT;
        hResult = pProfileObj->SetProps (i, spvProps, NULL);
        TraceResult ("ServiceEntry: Failed to save the LUL file name/time back into the profile", hResult);
    }

    // Profile is open, get the properties out of it
    hResult = pProfileObj->GetProps ((LPSPropTagArray)&sptLogonProps,
                                     fMapiUnicode,
                                     &ulPropCount,
                                     &pProps);
    if (FAILED(hResult))
    {
        goto ErrorExit;
    }
    
    if (MSG_SERVICE_DELETE == ulContext)
    {
        // If the service is being removed from a profile, we must cleanup
        // any temporary file (i.e. the local address book directory text file)
        if (PR_SMP_LUL_FILENAME == pProps[LUL_FILENAME].ulPropTag)
        {
            DeleteFile (pProps[LUL_FILENAME].Value.lpszA);
        }
        TCHAR szTmpDir[512];
        GetTempPath (_MAX_PATH, szTmpDir);
        lstrcat (szTmpDir, WINDS_DATA_DIRECTORY);
        RemoveDirectory (szTmpDir);
        hResult = S_OK;
        // We are done
        goto ErrorExit;
    }

    // After the setup wizard has been called the ServiceEntry functions gets called
    // with a set of properties obtained from the user for configuring this provider.
    // Merge or replace those properties onto the one we obtained from the profile section.
    if (pCfgProps && ulCfgPropCount)
    {
        MergeConfigProps (ulCfgPropCount, pCfgProps, pProps);
    }

    // If we can, display the UI...
    if (SERVICE_UI_ALWAYS & ulFlags || SERVICE_UI_ALLOWED & ulFlags)
    {
        CFGDLG CfgDialog = { 0 };
        // Get the mutex to gain exclusive access to the UI dialog
        // This prevent multiple processes and/or threads from displaying
        // the configuration dialog for this service
        hUIMutex = CreateMutex (NULL, FALSE, CONFIG_UI_MUTEX);
        if (NULL == hUIMutex)
        {
            TraceResult ("ServiceEntry: Failed to create UI mutext", HRESULT_FROM_WIN32(GetLastError()));
        }
        PrivInitialize3DCtl (hInstance);
        if (WAIT_TIMEOUT == WaitForSingleObject (hUIMutex, 250)) // Wait for the MUTEX for 1/4 second
        {
            PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE, (HWND)ulUIParam);
            hResult = MAPI_E_BUSY;
            goto ErrorExit;
        }

        // Fill in the logon UI structure
        CfgDialog.hInst    = hInstance;
        CfgDialog.hWnd     = (HWND)ulUIParam;
        CfgDialog.ppProps  = &pProps;
        CfgDialog.pSupObj  = pSupObj;
        CfgDialog.ulFlags  = MSG_SERVICE_UI_READ_ONLY & ulFlags ? UI_READONLY : 0;
        CfgDialog.hUIMutex = NULL; // So that the DoLogonDlg() won't have to wait.
ShowPropsAgain:
        hResult = DoLogonDlg (&CfgDialog);
        if (hResult)
        {
            goto ErrorExit;
        }
        if (!PingRemoteServer ((HWND)ulUIParam, pProps))
        {
            goto ShowPropsAgain;
        }
    }

    TCHAR szTime[32];
    // The first time the provider is added to the profile, if we have not downloaded
    // at least once the ABD, do so in case the server goes off line when the user logs in.
    if (PR_SMP_REMOTE_SERVER == pProps[SERVER_NAME].ulPropTag &&
        PR_SMP_LUL_FILENAME == pProps[LUL_FILENAME].ulPropTag &&
        PR_SMP_LUL_LAST_UPDATE == pProps[LUL_LASTUPDATE].ulPropTag &&
        0 == lstrlen (pProps[LUL_LASTUPDATE].Value.lpszA))
    {
        if (S_OK == GetServerABD (pProps[SERVER_NAME].Value.lpszA,
                                  pProps[LUL_FILENAME].Value.lpszA,
                                  NULL))
        {
            // Construct a string with the curent time/date for the download that we just did
            SYSTEMTIME st;
            GetLocalTime (&st);
            pProps[LUL_LASTUPDATE].Value.lpszA = GetTimeString (szTime, st);
        }
    }
    
    hResult = pProfileObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);
    TraceResult ("ServiceEntry: Failed to save the properties back into the profile", hResult);

ErrorExit:
    if (pProfileObj)
    {
        pProfileObj->Release();
    }
    if (NULL != GetMAPIFreeBuffer())
    {
        PrivMAPIFree ((LPVOID *)&pProps);
    }
    if (hUIMutex)
    {
        ReleaseMutex (hUIMutex);
        CloseHandle (hUIMutex);
    }
    // In case the DLL was loaded, unload it.
    // This DLL gets loaded and initialized in the call to DoLogonDlg()
    PrivUninitialize3DCtl (hInstance);

    // If we have remote connections going on, close them now
    TerminateRemoteConnections();

    // Close down the traces
    UnInitTraces();
    if (hResult)
    {
        TraceResult ("ServiceEntry", hResult);
        obj.GetLastError (hResult, fMapiUnicode, ppMAPIError, hInstance, PROVIDER_DISPLAY_NAME_STRING);
    }
    if (fDeInitMemoryHeap)
    {
        CloseMemoryHeap();
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    GetTimeString()
//
//    Parameters
//      pszTime     Pointer to a buffer where the function copies the
//                  formatted time string.
//      sysTime     A SYSTEMTIME structure with the time for the string
//
//    Purpose
//      This function returns a string formatted as "MM/DD/YY at xx:xx AM|PM"
//      The formatted string is returned in a buffer supplied by the caller
//      of the function.
//      
//    Return Value
//      A pointer to the pszTime string formatted with the time from FileTime
//
LPTSTR WINAPI GetTimeString (LPTSTR pszTime, SYSTEMTIME sysTime)
{
    // Initialize the flag for the AM/PM suffix
    BOOL fIsPM = FALSE;
    // Discard the century (This code must be revised in the year 2000) for the year number
    sysTime.wYear -= 1900;
    // If the hour is greater that 12, it was in the afternoon
    if (sysTime.wHour > 12)
    {
        sysTime.wHour -= 12;
        fIsPM = TRUE;
    }
    else
    {
        // If the hour is 0, means midnight, for common format the 0:00 is
        // 12:00 AM, so make the conversion
        if (0 == sysTime.wHour)
        {
            sysTime.wHour = 12;
        }
        else
        {
            // If the hour is 12, then it is 12:xx PM
            if (12 == sysTime.wHour)
            {
                fIsPM = TRUE;
            }
        }
    }
    // Write the formatted string. No leading zero for the day, leading
    // zero for the month and the year. No leading zero for the hour,
    // leading zero for the minutes. And no secods please.
    wsprintf (pszTime,
              "%d/%02d/%2d at %d:%02d %s",
              sysTime.wMonth,
              sysTime.wDay,
              sysTime.wYear,
              sysTime.wHour,
              sysTime.wMinute,
              fIsPM ? "PM" : "AM");
    return pszTime;
}

///////////////////////////////////////////////////////////////////////////////
//    DownloadMsgDlgProc()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      Dialog procedure for the dialog box with the message during a
//      address book directory download.
//
//    Return Value
//      FALSE always
//
BOOL CALLBACK DownloadMsgDlgProc (HWND    hDlg, 
                                  UINT    message, 
    							  WPARAM  wParam, 
    							  LPARAM  lParam)
{
    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//    DownloadABDirectory()
//
//    Parameters
//      pObj            Pointer to our IMAPIProp implementation (CUIMAPIProp)
//                      passed in to the DoConfidPropSheet() function
//      hOwnerWnd       The owner of any window we display
//      hInstance       The instance of this DLL
//
//    Purpose
//      This function get called by our implementation of
//      IMAPIControl::Active() by a user hitting a "Download Address Book"
//      on the provider configuration property sheet.
//      Here we download the server address book to a local address book
//      directory file and update the string of the last update on the file.
//
//    Return Value
//      None
//
void STDAPICALLTYPE DownloadABDirectory (CUIMAPIProp *  pObj,
                                         HWND           hOwnerWnd,
                                         HINSTANCE      hInstance)
{
    TCHAR szServer[MAX_STRING_SIZE+1];
    LPSPropValue pProp = NULL;
    SPropTagArray sptServer;
    sptServer.cValues = 1;
    sptServer.aulPropTag[0] = PR_SMP_REMOTE_SERVER;
    ULONG cValues;
    HWND hMsgWnd = CreateDialog (hInstance, MAKEINTRESOURCE(IDD_AB_DOWNLOAD_WAIT), hOwnerWnd, DownloadMsgDlgProc);
    ShowWindow (hMsgWnd, SW_SHOW);

    // Get the server we need to connect to
    HRESULT hResult = pObj->GetProps (&sptServer, fMapiUnicode, &cValues, &pProp);
    if (FAILED(hResult) || MAPI_W_ERRORS_RETURNED == hResult)
    {
        TraceResult ("DownloadABDirectory: Getting Server Name", hResult);
        PrivMAPIFree ((LPVOID *)&pProp);
        PrivateMessageBox (IDS_MSG_NEED_SERVER_NAME, hMsgWnd);
        DestroyWindow (hMsgWnd);
    	return;
    }
    lstrcpy (szServer, pProp->Value.lpszA);
    PrivMAPIFree ((LPVOID *)&pProp);
    if (!IsValidServerName (szServer))
    {
        MessageBeep (MB_ICONEXCLAMATION);
        PrivateMessageBox (IDS_MSG_INVALID_SERVER_NAME, hMsgWnd);
        DestroyWindow (hMsgWnd);
    	return;
    }
    if (!IsWINDSServerAvailable (szServer))
    {
        PrivateMessageBox (IDS_MSG_SERVER_NOT_FOUND, hMsgWnd);
        DestroyWindow (hMsgWnd);
        return;
    }
    // Get the file we a going to download it to
    sptServer.aulPropTag[0] = PR_SMP_LUL_FILENAME;
    hResult = pObj->GetProps (&sptServer, fMapiUnicode, &cValues, &pProp);
    if (FAILED(hResult) || MAPI_W_ERRORS_RETURNED == hResult)
    {
        TraceResult ("DownloadABDirectory: Getting Local ABD Filename", hResult);
        PrivMAPIFree ((LPVOID *)&pProp);
        PrivateMessageBox (IDS_MSG_SERVICE_NOT_PROPERLY_CFG, hMsgWnd);
        DestroyWindow (hMsgWnd);
        return;
    }
    SetCursor (LoadCursor (NULL, IDC_WAIT));
    hResult = GetServerABD (szServer, pProp->Value.lpszA, hMsgWnd);
    PrivMAPIFree ((LPVOID *)&pProp);
    if (S_OK == hResult)
    {
        // Construct a string with the curent time/date for the download that we just did
        SYSTEMTIME st;
        TCHAR szTime[32];
        GetLocalTime (&st);
        SPropValue spvLastUpdate = { 0 };
        spvLastUpdate.ulPropTag = PR_SMP_LUL_LAST_UPDATE;
        spvLastUpdate.Value.lpszA = GetTimeString (szTime, st);
        pObj->SetProps (1, &spvLastUpdate, NULL);

        // Notify the display table that it need to reload the data for the edit control of PR_SMP_LUL_LAST_UPDATE
        for (ULONG iRow=0; ; iRow++)
        {
            LPSRow pRow;
            LPSPropValue pControlID;
            hResult = pObj->m_pTableData->HrEnumRow (iRow, &pRow);
            if (hResult || !pRow)
        	{
        		TraceResult ("DownloadABDirectory: Traversing the display table", hResult);
                break; // Out of the FOR() loop
            }
            pControlID = PpropFindProp (pRow->lpProps, pRow->cValues, PR_CONTROL_ID);
            if (pControlID->Value.bin.cb)
            {
        	    pObj->m_pTableData->HrNotify (0, 1, pControlID);
            }
            PrivMAPIFree ((LPVOID *)&pRow);
        }
    }
    SetCursor (LoadCursor (NULL, IDC_ARROW));
    DestroyWindow (hMsgWnd);
}

///////////////////////////////////////////////////////////////////////////////
//    GetServerABD()
//
//    Parameters
//      pszServer       Name of the server to get the directory from.
//      pszABDFile      Name of the local address book directory data file
//      hOwnerWnd       Handle to an owner window we use to display any UI.
//                      If this parameter is NULL, the function will not
//                      display any UI.
//
//    Purpose
//      This function makes a remote procedure call to the remote server and
//      request that all the directory information for the GAL container be
//      sent through a pipe. This data is saved to a binary data file that we
//      can use so fill the contents table of the GAL AB container and work
//      offline when the address book is configured for offline use or if the
//      network connection is broken.
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI GetServerABD (LPTSTR pszServer, LPTSTR pszABDFile, HWND hOwnerWnd)
{
    // Open the file, always overwritting what is there
    HRESULT hResult;
    HANDLE hFile = CreateFile (pszABDFile, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if (INVALID_HANDLE_VALUE == hFile)
    {
        hResult = HRESULT_FROM_WIN32(GetLastError());
        TraceResult ("GetServerABD: Failed to create local ABD file", hResult);
        if (hOwnerWnd)
        {
            PrivateMessageBox (IDS_MSG_SYSTEM_CALL_FAILED, hOwnerWnd);
        }
        return hResult;
    }

    HANDLE hPipe;
    long lPipeNum;
    AB_ENTRY_INFO abEntry = { 0 };
    DWORD dwBytesWritten, dwBytesRead;
    TCHAR szPipeName[64];

    RpcTryExcept
    {
        // Ask the server to get ALL the recipient of a particular container. In this case,
        // the GAL (Global Address List) container.
        hResult = RemoteGetContainerRecipients (0, GAL_CONTAINER_ID, &lPipeNum);
        if (!hResult)
        {
            // Construct the download pipe name
            wsprintf (szPipeName, PIPE_NAME_FORMAT, pszServer, lPipeNum);
            // Create our endpoint and connect    
            hPipe = CreateFile (szPipeName, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, NULL);
            if (INVALID_HANDLE_VALUE == hPipe)
            {
                hResult = HRESULT_FROM_WIN32(GetLastError());
            }
            else
            {
                do
                {
                    // Read the pipe handle.
                    if (!ReadFile (hPipe, &abEntry, sizeof(AB_ENTRY_INFO), &dwBytesRead, NULL))
                    {
                        hResult = HRESULT_FROM_WIN32(GetLastError());
                        if (HRESULT_FROM_WIN32(ERROR_BROKEN_PIPE)   != hResult &&   // For Windows NT
                            HRESULT_FROM_WIN32(ERROR_ACCESS_DENIED) != hResult)     // For Windows 95
                        {
                            // There was an error and we can't continue
                            TraceResult ("GetServerABD: Failed to read from the pipe", hResult);
                        }
                        else
                        {
                            // If the pipe was broken, it means the server finished writing
                            // to the it, so we are finished reading from it.
                            hResult = 0;
                        }
                    }
                    else
                    {
                        if (SERVER_USER_MAILBOX == abEntry.Type ||
                            SERVER_DISTRIBUTION_LIST == abEntry.Type ||
                            GATEWAY_RECIPIENT == abEntry.Type)
                        {
                            if (SERVER_DISTRIBUTION_LIST == abEntry.Type)
                            {
                                if (HIDE_IN_DIR & abEntry.Info.DL.dwFlags)
                                {
                                    continue;
                                }
                            }
                            else
                            {
                                if (HIDE_IN_DIR & abEntry.Info.MB.dwFlags)
                                {
                                    continue;
                                }
                            }
                            WriteFile (hFile, &abEntry, sizeof(AB_ENTRY_INFO), &dwBytesWritten, NULL);
                        }
                    }
                } while (dwBytesRead && !hResult);
                CloseHandle (hPipe);
            }
        }
    }
    RpcExcept(1)
    {        
        // If we got here is because there was an error while call was made
        // or when it was about to be made.
        hResult = RpcExceptionCode();
        hResult = MAKE_HRESULT(1, FACILITY_RPC, hResult);
    }
    RpcEndExcept
    // Terminate connections with the remote server
    BindToServer (NULL);
    
    CloseHandle (hFile);
    if (hResult)
    {   
        if (FACILITY_NULL == HRESULT_FACILITY(hResult))
        {
            hResult = HRESULT_FROM_WIN32 (hResult);
        }
    }
    TraceResult ("GetServerABD", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    DoLogonDlg()
//
//    Parameters
//      pDialog   Pointer to a CFGDLG structure.
//
//    Purpose
//      This functions gets called to display the configuration properties for
//      this provider. When invoked, it will parse the message store logon 
//      property array and initialize the fields in the property sheet with it.
//      The function could also get invoked to display the settings in 
//      READ-ONLY mode
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI DoLogonDlg (PCFGDLG pCfgDialog)
{
    DWORD dwWait = WaitForSingleObject (pCfgDialog->hUIMutex, 250); // Wait for the MUTEX for 1/4 second
    PrivInitialize3DCtl (pCfgDialog->hInst);
    if (WAIT_TIMEOUT == dwWait)
    {
        PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE, pCfgDialog->hWnd);
        return MAPI_E_BUSY;
    }

    // Take ownership of the property value array, so the calling
    // code will not attempt to free this block
    LPSPropValue pProps = *pCfgDialog->ppProps;
    *pCfgDialog->ppProps = NULL;
    
    // Retrieve the dialog title string
    TCHAR szDialogTitle[64];
    LoadString (pCfgDialog->hInst, IDS_SERVICE_PROPERTY_SHEETS_TITLE, szDialogTitle, 64);
    LPMAPITABLE pTableObj = NULL;
    CUIMAPIProp * pPropObj = NULL;
    HRESULT hResult = S_OK;
    try
    {
        pPropObj = new CUIMAPIProp (pCfgDialog->hInst,
                                    GetMAPIAllocBuffer(),
                                    GetMAPIAllocMore(),
                                    GetMAPIFreeBuffer(),
                                    pCfgDialog->ulFlags & UI_READONLY);
        if (!pPropObj)
        {
            hResult = E_OUTOFMEMORY;
        }
    }
    catch (CException & Exception)
    {
        hResult = Exception.GetError();
    }
    if (!hResult)
    {
        // Set the properties in the property array. The properties MUST exist in
        // the array before invoking the property page with MAPI methods, 
        // otherwise the call fails
        hResult = pPropObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);
    }
    if (hResult)
    {
        goto DoLogonDlgFinished;
    }
    DTPAGE DtPropPages[2];
    DtPropPages[0].cctl = NUM_CFG_USERINFO_PAGE_CTLS;   // Count of controls in the ConfigPage array
    DtPropPages[0].lpszComponent = "";
    DtPropPages[0].lpszResourceName = MAKEINTRESOURCE (IDD_CFG_USERINFO);
    DtPropPages[0].lpctl = UserConfigPage;

    DtPropPages[1].cctl = sizeof(ABConfigPage)/sizeof(DTCTL);   // Count of controls in the ConfigPage array
    DtPropPages[1].lpszComponent = "";
    DtPropPages[1].lpszResourceName = MAKEINTRESOURCE (IDD_CFG_ABCONNECT);
    DtPropPages[1].lpctl = ABConfigPage;

    // If the UI_READONLY flag is set, then we are being called from the 
    // SettingDialog method in Read Only mode. Disable all of the controls
    // on the property sheet, else Enable them
    if (pCfgDialog->ulFlags & UI_READONLY)
    {
        // Disable the controls (make them read-only)
        DtPropPages[0].lpctl[1].ulCtlFlags &= ~(DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[0].lpctl[6].ulCtlFlags &= ~(DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[0].lpctl[7].ulCtlFlags &= ~(DT_EDITABLE | DT_SET_IMMEDIATE);
    }
    else
    {
        // Enable the controls (make them read/write)
        DtPropPages[0].lpctl[1].ulCtlFlags |= (DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[0].lpctl[6].ulCtlFlags |= (DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[0].lpctl[7].ulCtlFlags |= (DT_EDITABLE | DT_SET_IMMEDIATE);
    }

    // Create the display table for the logon dialog. It is based on dialog
    // resources plus the static information at the head of this module.
    LPTABLEDATA pTableData;
    hResult = BuildDisplayTable (GetMAPIAllocBuffer(),
                                 GetMAPIAllocMore(),
                                 GetMAPIFreeBuffer(),
                                 NULL,
                                 pCfgDialog->hInst,
                                 sizeof(DtPropPages)/sizeof(DTPAGE),
                                 DtPropPages,
                                 fMapiUnicode,
                                 &pTableObj,
                                 &pTableData);
    if (!hResult)
    {
        pPropObj->SetTableData (pTableData);
        pTableData->Release();
        pPropObj->SetPropCallBack (PR_SMP_LUL_DOWNLOAD, DownloadABDirectory);

        // Display the dialog/property sheets
DisplayPropsAgain:
        hResult = pCfgDialog->pSupObj->DoConfigPropsheet ((ULONG)pCfgDialog->hWnd,
                                                          0,
                                                          szDialogTitle,
                                                          pTableObj,
                                                          pPropObj,
                                                          0);     // Default index for the top property sheet
        if (!hResult)
        {
            // Retrieve the altered data
            PrivMAPIFree ((LPVOID *)&pProps); // The pointer comes back NULL
            ULONG ulPropCount;
            hResult = pPropObj->GetProps ((LPSPropTagArray)&sptLogonProps, fMapiUnicode, &ulPropCount, &pProps);
            if (FAILED(hResult))
            {
                goto DoLogonDlgFinished;
            }
            hResult = S_OK;
            // Check the Server name
            if (!IsValidServerName(pProps[SERVER_NAME].Value.lpszA))
            {
                PrivateMessageBox (IDS_MSG_INVALID_SERVER_NAME, pCfgDialog->hWnd);
                PrivMAPIFree ((LPVOID *)&pProps);
                goto DisplayPropsAgain;
            }
            // Return the new property array
            *(pCfgDialog->ppProps) = pProps;
            pProps = NULL;  // Do not free this in cleanup code below
        }
    }
DoLogonDlgFinished:
    // Release objects
    if (pTableObj)
    {
        pTableObj->Release();
    }
    if (pPropObj)
    {
        pPropObj->Release();
    }
    PrivMAPIFree ((LPVOID *)&pProps);
    if (MAPI_E_USER_CANCEL != hResult)
    {
        TraceResult ("DoLogonDlg", hResult);
    }
    PrivUninitialize3DCtl (pCfgDialog->hInst);
    ReleaseMutex (pCfgDialog->hUIMutex);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    ABProviderInit()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Entry pointe called by MAPI when a profile uses this 
//      provider. The MAPI calls this method and expects a pointer to an
//      implementation of the IABProvider interface. MAPI uses the returned
//      IABProvider interface pointer to logon on the address book provider
//
//    Return Value
//      An HRESULT
//
STDINITMETHODIMP ABProviderInit (HINSTANCE          hInstance, 
                                 LPMALLOC           pMallocObj,
                                 LPALLOCATEBUFFER   pfnAllocateBuffer,
                                 LPALLOCATEMORE     pfnAllocateMore,
                                 LPFREEBUFFER       pfnFreeBuffer,
                                 ULONG              ulFlags,
                                 ULONG              ulMAPIVer,
                                 ULONG *            pulProviderVer,
                                 LPABPROVIDER *     ppABProviderObj)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    // Send output only to the COM1 port
    //InitTraces (0);
    // Send output to a console window BUT NOT to the COM1
    InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);
    // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);

    InfoTrace ("ABProviderInit function called");
    *pulProviderVer = CURRENT_SPI_VERSION;
    if (ulMAPIVer < CURRENT_SPI_VERSION)
    {
        TraceMessage ("XPProviderInit: The version of the subsystem cannot handle this version of the provider");
        return MAPI_E_VERSION;
    }

    HRESULT hResult = InitMemoryHeap (pfnAllocateBuffer, pfnAllocateMore, pfnFreeBuffer);
    if (FAILED(hResult))
    {
        return hResult;
    }
    ASSERTMSG (S_OK == hResult, "Contact irvingd@microsoft.com");

    ghInstance = hInstance;
    // Allocate space for the ABProvider object, the constructor will be called to initialize member variables
    CABProvider * pABProvider = new CABProvider (hInstance);
    if (!pABProvider)
    {
        TraceMessage ("ABProviderInit: Failed to allocate new CABProvider object");
        hResult = E_OUTOFMEMORY;
    }
    // Copy pointer to the allocated object back into the return IMSProvider object pointer
    *ppABProviderObj = (LPABPROVIDER)pABProvider;
    if (hResult)
    {
        CloseMemoryHeap();
    }
    return hResult;
}

// End of file for ABSRVCE.CPP
