//////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      ABWDS.H
//
//  Description
//      Interface declaraction file for CABLogon, CABProvider,
//      CMailUser, CABContainer,  and CDLTemplate
//      Property tag definitions.
//      Constant definitions
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _ABWDS_H
#define _ABWDS_H

#pragma warning (disable : 4100)
#pragma warning (disable : 4201)
#pragma warning (disable : 4514)
#pragma warning (disable : 4702)
#pragma warning (disable : 4710)

#define STRICT
#include <WINDOWS.H>
#include <CTL3D.H>
#include <MAPIWIN.H>
#include <MAPISPI.H>
#include <MAPIUTIL.H>
#include <MAPIVAL.H>

#ifdef _DEBUG
#define ENABLE_DEBUG_OUTPUT     1
#endif // _DEBUG

// Header file for the trace and logging functions.
#include "TRACES.H"

// Include our base implementation of IMAPIProp
#include "GENPROP.H"

// Common defs used by providers to communicate to the WINDS Sample Messaging Host
#include "COMWINDS.H"

// The order for this enumarations must match the order of the properties
// in the tag arrays. These arrays are used to access the 
// LPSPropValue arrays returned from MAPI interfaces. The use of enumerations
// hides differences when the number of properties in the tag array increases
// or decreases. It also helps maintain error-free indexing in the use of 
// the prop arrays
enum
{
    //SERVER_NAME           = 0,        // Array Indexes
    //MAILBOX_NAME          = 1,
    //MAILBOX_ID            = 2,
    //USER_NAME             = 3,
    //PASSWORD              = 4,
    //NET_CON               = 5,
    COMMENT                 = 6,
    LUL_FILENAME            = 7,
    LUL_LASTUPDATE          = 8,
    NUM_LOGON_PROPERTIES    = 9         // Array Size
};

// ID array of property tags for properties stored in the profile section.
// These properties are retrieved at logon time to get the necessary
// information to connect to the server.
const static SizedSPropTagArray(NUM_LOGON_PROPERTIES, sptLogonProps) =
{
    NUM_LOGON_PROPERTIES,
    {
        PR_SMP_REMOTE_SERVER,           // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 0)
        PR_SMP_MAILBOX_NAME,            // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 1)
        PR_SMP_MAILBOX_ID,              // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 2)
        PR_SMP_USER_NAME,               // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 3)
        PR_SMP_MAILBOX_PASSWORD,        // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 4)
        PR_SMP_CONNECTION_TYPE,         // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 5)
        PR_COMMENT,
        PR_SMP_LUL_FILENAME,
        PR_SMP_LUL_LAST_UPDATE
    }
};

// Version of this adresss book provider returned in PR_CREATION_VERSION and PR_CURRENT_VERSION
#define ADR_BOOK_VERSION_MAJOR              1
#define ADR_BOOK_VERSION_MINOR              8

// Display name for the provider used in PR_PROVIDER_NAME
#define PROVIDER_DISPLAY_NAME_STRING    TEXT("WINDS Address Book")

// The target OS should not be included in the DLL name. MAPI will handle that.
// For Win32 platforms (Windows NT and Windows 95), the full name will be ABWDS32.DLL
#define AB_DLL_NAME_STRING              TEXT("ABWDS.DLL")

extern "C"
{
    HRESULT WINAPI DoLogonDlg
                    (PCFGDLG                    pCfgDialog);
    HRESULT WINAPI GetServerABD
                    (LPTSTR                     pszServer,
                     LPTSTR                     pszABDFile,
                     HWND                       hOwnerWnd);
    extern MAPIUID guidABEntries;
    extern MAPIUID guidXPEntries;
    extern DTBLEDIT DtObjAlias;
    extern DTBLEDIT DtObjFullName;
    extern DTBLEDIT DtServer;
    extern DTBLEDIT DtComment;
};

// Forward class declarations
class CABProvider;
class CABLogon;
class CABMAPIStatus;

DWORD WINAPI ABMailslotListenThreadProc (CABLogon * pLogon);

class CABLogon : public CGeneric, public IABLogon
{
friend CABMAPIStatus;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IABLOGON_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
public:
    STDMETHODIMP InitializeStatusRow
                    (ULONG                      ulFlag = 0);
    STDMETHODIMP GetOneOffProperties
                    (CPropList **               ppList);
    inline void WINAPI SetCachedGAL
                    (LPTABLEDATA                pGALTable)
                    { EnterCriticalSection (&m_csGALCache);
                      if (NULL == m_pGALTable)
                      {
                          m_pGALTable = pGALTable;
                          m_pGALTable->AddRef();
                      }
                      LeaveCriticalSection (&m_csGALCache); }
    inline void WINAPI GetCachedGAL
                    (LPTABLEDATA *              ppGALTable)
                    { EnterCriticalSection (&m_csGALCache);
                      if (m_pGALTable)
                      {
                          *ppGALTable = m_pGALTable;
                          (*ppGALTable)->AddRef();
                      }
                      else
                      {
                        *ppGALTable = NULL;
                      }
                      LeaveCriticalSection (&m_csGALCache); }
    inline MAPIUID WINAPI GetBackEndUID
                    () { return m_pBEobj->GetBackEndUID(); }
    inline HANDLE WINAPI GetListenMailslot
                    () { return m_hMailslot; }
    inline void WINAPI SetListenMailslot
                    (HANDLE hNewMailSlot)
                    { CloseHandle (m_hMailslot);
                      m_hMailslot = hNewMailSlot; }
    inline DWORD WINAPI GetConnectionID
                    () { return m_dwServerConnectionID; }
    inline LPTSTR WINAPI GetServerName
                    () { return m_szServer; }
    inline LPTSTR WINAPI GetUserName
                    () { return m_UserInfo.szFullName; }
    inline PRIVATE_ENTRYID * WINAPI GetUserEID
                    () { return &m_UserEID; }
    inline LPTSTR WINAPI GetLocalABD
                    () { return m_szLocalABD; }
    inline LPTSTR WINAPI GetLocalABDLastUpdate
                    () { return m_szLocalABDLastUpdate; }
    inline CONNECTION_TYPE WINAPI GetConnectionType
                    () { EnterCriticalSection (&m_csObj);
                         CONNECTION_TYPE Type = m_ConnectionType;
                         LeaveCriticalSection (&m_csObj);
                         return Type; }
    inline void WINAPI SetConnectionType
                    (CONNECTION_TYPE            NewState)
                    { EnterCriticalSection (&m_csObj);
                      m_ConnectionType = NewState;
                      LeaveCriticalSection (&m_csObj); }
    inline void WINAPI ResetServerNotifications
                    () { EnterCriticalSection (&m_csObj);
                         ::ResetServerNotifications (m_szServer,
                                                     m_UserInfo.szMailboxName,
                                                     WINDS_NOTIF_ON_AB | WINDS_NOTIF_ON_USER,
                                                     &m_dwServerConnectionID);
                         LeaveCriticalSection (&m_csObj); }

private :
    BOOL WINAPI IsValidXPEntryID
                    (PRIVATE_XP_ENTRYID *       pXPEID);
    STDMETHODIMP GetABEntryIDFromXPEntryID
                    (PRIVATE_XP_ENTRYID         XPEID, 
                     PRIVATE_ENTRYID *          pEID);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CABLogon        (HINSTANCE                  hInstance,
                     CBackEnd *                 pBEobj,
                     CNamePropMgr *             pNPList,
                     LPMAPISUP                  pSupObj,
                     DWORD                      dwConnectID,
                     MAILBOX_INFO *             pUserMailboxInfo,
                     DWORD                      dwMailboxID,
                     LPTSTR                     pszRemoteServer,
                     CONNECTION_TYPE            ConnectionType,
                     LPTSTR                     pszLocalABD,
                     LPTSTR                     pszLocalABDLastUpdate,
                     HANDLE                     hUIMutex);
    ~CABLogon       ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    LPTABLEDATA         m_pOneOffTable;
    CRITICAL_SECTION    m_csGALCache;
    LPTABLEDATA         m_pGALTable;
    
    HANDLE              m_hMailslot;

    MAILBOX_INFO        m_UserInfo;
    PRIVATE_ENTRYID     m_UserEID;

    TCHAR               m_szServer[MAX_STRING_SIZE+1];
    CONNECTION_TYPE     m_ConnectionType;
    DWORD               m_dwServerConnectionID;

    TCHAR               m_szLocalABD[MAX_PATH];
    TCHAR               m_szLocalABDLastUpdate[32];
    
    CABLogon *          m_pNext;
    CABMAPIStatus *     m_pStatusObj;
    CNamePropMgr *      m_pNPList;

public :
    CBackEnd *          m_pBEobj;
    HINSTANCE           m_hInstance;
    HANDLE              m_hUIMutex;
    LPMAPISUP           m_pSupObj;
};

class CABProvider : public IABProvider
{
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IABPROVIDER_METHODS(IMPL);
    
///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
private :
    STDMETHODIMP AddWindsABNamedPropsToList
                    (CNamePropMgr *             pNPList);
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CABProvider     (HINSTANCE                  hInst);
    ~CABProvider    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG               m_cRef; 
    CABLogon *          m_pABLogonList;
    CRITICAL_SECTION    m_csObj;
    HINSTANCE           m_hInstance;
};

class CABContainer : public CGenericProp, public IABContainer
{
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMAPICONTAINER_METHODS(IMPL);
    MAPI_IABCONTAINER_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
public:
    
private:
    STDMETHODIMP GetGALTable
                    ();
    void WINAPI ResolveOneName
                    (LPTSTR                     pszName,
                     LPMAPITABLE                pTable,
                     ULONG &                    ulResolution,
                     DWORD &                    dwInstID);
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CABContainer    (CABLogon *                 pLogon,
                     PRIVATE_ENTRYID *          pParentEID,
                     PRIVATE_ENTRYID *          pContainerEID,
                     long                       lContainerType,
                     CPropList *                pList,
                     CNamePropMgr *             pNPList,
                     long                       lAccess);
    ~CABContainer   ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private:
    long                m_lContainerType;
    CABLogon *          m_pLogon;
    LPTABLEDATA         m_pHierTable;
    LPTABLEDATA         m_pContTable;
};

class CMailUser : public CGenericProp, public IMailUser
{
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMAILUSER_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
public:
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMailUser       (CABLogon *                 pLogon,
                     PRIVATE_ENTRYID *          pObjectEID,
                     CPropList *                pList,
                     CNamePropMgr *             pNPList,
                     long                       lAccess);
    ~CMailUser      ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private:
    CABLogon *          m_pLogon;
};

class CABMAPIStatus : public CGenericProp, public IMAPIStatus
{
friend CABLogon;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMAPISTATUS_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this transport
//
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CABMAPIStatus   (CABLogon *                 pLogon,
                     LPPROFSECT                 pProfSectObj);
    ~CABMAPIStatus  ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG           m_cRef;
    CABLogon *      m_pLogon;
    LPPROFSECT      m_pProfSectObj;
};

class CDLTemplate : public IABContainer
{
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
public:
    inline STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj)
                    { *ppvObj = NULL;
                      HRESULT hResult = S_OK;
                      EnterCriticalSection (&m_csObj);
                      if (riid == IID_IMAPIProp || riid == IID_IUnknown)
                      {
                          *ppvObj = (LPVOID)this;
                          AddRef();
                      }
                      else
                      {
                          hResult = E_NOINTERFACE;
                      }
                      LeaveCriticalSection (&m_csObj);
                      return hResult; }
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    inline STDMETHODIMP GetLastError
                    (HRESULT                    hResult,
                     ULONG                      ulFlags,
                     LPMAPIERROR *              ppMAPIError)
                    { return m_pImpObj->GetLastError (hResult, ulFlags, ppMAPIError); }
    inline STDMETHODIMP SaveChanges
                    (ULONG                      ulFlags)
                    { return m_pImpObj->SaveChanges (ulFlags); }
    inline STDMETHODIMP GetProps
                    (LPSPropTagArray            pPropTagArray,
                     ULONG                      ulFlags,
                     ULONG *                    pcValues,
                     LPSPropValue *             ppPropArray)
                    { return m_pImpObj->GetProps (pPropTagArray, ulFlags, pcValues, ppPropArray); }
    inline STDMETHODIMP GetPropList
                    (ULONG                      ulFlags,
                     LPSPropTagArray *          ppAllTags)
                    { return m_pImpObj->GetPropList (ulFlags, ppAllTags); }
    STDMETHODIMP OpenProperty
                    (ULONG                      ulPropTag,
                     LPCIID                     piid,
                     ULONG                      ulInterfaceOptions,
                     ULONG                      ulFlags,
                     LPUNKNOWN *                ppUnk);
    inline STDMETHODIMP SetProps
                    (ULONG                      cValues,
                     LPSPropValue               pPropArray,
                     LPSPropProblemArray *      ppProblems)
                    { return m_pImpObj->SetProps (cValues, pPropArray, ppProblems); }
    inline STDMETHODIMP DeleteProps
                    (LPSPropTagArray            pPropTagArray,
                     LPSPropProblemArray *      ppProblems)
                    { return m_pImpObj->DeleteProps (pPropTagArray, ppProblems); }
    inline STDMETHODIMP CopyTo
                    (ULONG                      ciidExclude,
                     LPCIID                     rgiidExclude,
                     LPSPropTagArray            pExcludeProps,
                     ULONG                      ulUIParam,
                     LPMAPIPROGRESS             pProgress,
                     LPCIID                     pInterface,
                     LPVOID                     pDestObj,
                     ULONG                      ulFlags,
                     LPSPropProblemArray *      ppProblems)
                    { return m_pImpObj->CopyTo (ciidExclude,
                                                rgiidExclude,
                                                pExcludeProps,
                                                ulUIParam,
                                                pProgress,
                                                pInterface,
                                                pDestObj,
                                                ulFlags,
                                                ppProblems); }
    inline STDMETHODIMP CopyProps
                    (LPSPropTagArray            pIncludeProps,
                     ULONG                      ulUIParam,
                     LPMAPIPROGRESS             pProgress,
                     LPCIID                     pInterface,
                     LPVOID                     pDestObj,
                     ULONG                      ulFlags,
                     LPSPropProblemArray *      ppProblems)
                    { return m_pImpObj->CopyProps (pIncludeProps,
                                                   ulUIParam,
                                                   pProgress,
                                                   pInterface,
                                                   pDestObj,
                                                   ulFlags,
                                                   ppProblems); }
    inline STDMETHODIMP GetNamesFromIDs
                    (LPSPropTagArray *          ppPropTags,
                     LPGUID                     pPropSetGuid,
                     ULONG                      ulFlags,
                     ULONG *                    pcPropNames,
                     LPMAPINAMEID **            pppPropNames)
                    { return m_pImpObj->GetNamesFromIDs (ppPropTags,
                                                         pPropSetGuid,
                                                         ulFlags,
                                                         pcPropNames,
                                                         pppPropNames); }
    inline STDMETHODIMP GetIDsFromNames
                    (ULONG                      cPropNames,
                     LPMAPINAMEID *             ppPropNames,
                     ULONG                      ulFlags,
                     LPSPropTagArray *          ppPropTags)
                    { return m_pImpObj->GetIDsFromNames (cPropNames,
                                                         ppPropNames,
                                                         ulFlags,
                                                         ppPropTags); }
    STDMETHODIMP GetContentsTable
                    (ULONG                      ulFlags,
                     LPMAPITABLE *              ppTable);
    STDMETHODIMP GetHierarchyTable
                    (ULONG                      ulFlags,
                     LPMAPITABLE *              ppTable);
    STDMETHODIMP OpenEntry
                    (ULONG                      cbEntryID,
                     LPENTRYID                  pEntryID,
                     LPCIID                     pInterface,
                     ULONG                      ulFlags,
                     ULONG *                    pulObjType,
                     LPUNKNOWN *                ppUnk);
    inline STDMETHODIMP SetSearchCriteria
                    (LPSRestriction             pRestriction,
                     LPENTRYLIST                pContainerList,
                     ULONG                      ulSearchFlags)
                    { Validate_IMAPIContainer_SetSearchCriteria (this,
                                                                 pRestriction,
                                                                 pContainerList,
                                                                 ulSearchFlags);
                      return MAPI_E_NO_SUPPORT; }
    inline STDMETHODIMP GetSearchCriteria
                    (ULONG                      ulFlags,
                     LPSRestriction *           ppRestriction,
                     LPENTRYLIST *              ppContainerList,
                     ULONG *                    pulSearchState)
                    { Validate_IMAPIContainer_GetSearchCriteria (this,
                                                                 ulFlags,
                                                                 pRestriction,
                                                                 pContainerList,
                                                                 ulSearchFlags);
                      return MAPI_E_NO_SUPPORT; }
    inline STDMETHODIMP CreateEntry
                    (ULONG                      cbEntryID,
                     LPENTRYID                  pEntryID,
                     ULONG                      ulCreateFlags,
                     LPMAPIPROP *               ppMAPIPropEntry)
                    { Validate_IABContainer_CreateEntry (this,
                                                         cbEntryID,
                                                         pEntryID,
                                                         ulCreateFlags,
                                                         ppMAPIPropEntry);
                      return MAPI_E_NO_SUPPORT; }
    inline STDMETHODIMP CopyEntries
                    (LPENTRYLIST                pEntries,
                     ULONG                      ulUIParam,
                     LPMAPIPROGRESS             pProgress,
                     ULONG                      ulFlags)
                    { Validate_IABContainer_CopyEntries (this,
                                                         pEntries,
                                                         ulUIParam,
                                                         pProgress,
                                                         ulFlags);
                     return E_ACCESSDENIED; }
    inline STDMETHODIMP DeleteEntries
                    (LPENTRYLIST                pEntries,
                     ULONG                      ulFlags)
                    { Validate_IABContainer_DeleteEntries (this,
                                                           pEntries,
                                                           ulFlags);
                      return E_ACCESSDENIED; }
    inline STDMETHODIMP ResolveNames
                    (LPSPropTagArray            pPropTagArray,
                     ULONG                      ulFlags,
                     LPADRLIST                  pAdrList,
                     LPFlagList                 pFlagList)
                    { Validate_IABContainer_ResolveNames (this,
                                                          pPropTagArray,
                                                          ulFlags,
                                                          pAdrList,
                                                          pFlagList);
                      return MAPI_E_NO_SUPPORT; }
///////////////////////////////////////////////////////////////////////////////
// Other methods private to this class
//

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CDLTemplate    (LPMAPIPROP                 pDataObj,
                    CABLogon *                 pLogon,
                    CBackEnd *                 pBEobj,
                    PRIVATE_ENTRYID            ObjectEID)
                    { m_cRef = 1;
                      m_ObjectEID = ObjectEID;
                      m_pContTable = NULL;
                      m_pHierTable = NULL;
                      m_pBEobj = pBEobj;
                      m_pBEobj->AddRef();
                      m_pImpObj = pDataObj;
                      m_pImpObj->AddRef();
                      m_pLogon = pLogon;
                      m_pLogon->AddRef();
                      InitializeCriticalSection (&m_csObj); }
    ~CDLTemplate   ()
                    { m_pBEobj->Release();
                      m_pLogon->Release();
                      m_pImpObj->Release();
                      if (m_pContTable) { m_pContTable->Release(); }
                      if (m_pHierTable) { m_pHierTable->Release(); }
                      DeleteCriticalSection (&m_csObj); }

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG               m_cRef;
    LPMAPIPROP          m_pImpObj;
    CABLogon *          m_pLogon;
    CBackEnd *          m_pBEobj;
    PRIVATE_ENTRYID     m_ObjectEID;
    LPTABLEDATA         m_pContTable;
    LPTABLEDATA         m_pHierTable;
    CRITICAL_SECTION    m_csObj;
};

enum
{
    LBX_INSTKEY,
    LBX_EID,
    LBX_OBJTYPE,
    LBX_DISPTYPE,
    LBX_DISPNAME,
    LBX_PHONE,
    LBX_OFFICE,
    LBX_TITLE,
    LBX_DEPT,
    LBX_COMPANY,
    LBX_ACCT,
    NUM_DIST_LIST_MEMBERS_DTLBX
};

const static SizedSPropTagArray(NUM_DIST_LIST_MEMBERS_DTLBX, sptDLMembersLbx) =
{
    NUM_DIST_LIST_MEMBERS_DTLBX,
    {
        PR_INSTANCE_KEY,
        PR_ENTRYID,
        PR_OBJECT_TYPE,
        PR_DISPLAY_TYPE,
        PR_DISPLAY_NAME,
        PR_OFFICE_TELEPHONE_NUMBER,
        PR_OFFICE_LOCATION,
        PR_TITLE,
        PR_DEPARTMENT_NAME,
        PR_COMPANY_NAME,
        PR_ACCOUNT
    }
};

#define DEFAULT_CONTENTS_COLUMNS        11
const static SizedSPropTagArray (DEFAULT_CONTENTS_COLUMNS, sptDefaultContTableCols) =
{
    DEFAULT_CONTENTS_COLUMNS,
    {
        PR_INSTANCE_KEY,
        PR_ENTRYID,
        PR_OBJECT_TYPE,
        PR_DISPLAY_TYPE,
        PR_DISPLAY_NAME,
        PR_OFFICE_TELEPHONE_NUMBER,
        PR_OFFICE_LOCATION,
        PR_TITLE,
        PR_DEPARTMENT_NAME,
        PR_COMPANY_NAME,
        PR_ACCOUNT
    }
};

#endif // _ABWDS_H

// End of file for ABWDS.H
