//////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      NAMEPROP.H
//
//  Description
//      This file defines the CNamePropMgr class used to implement a named
//      property manager to access and manage the named properties in
//      IMAPIProp-derived objects.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _NAMEPROP_H
#define _NAMEPROP_H

#pragma warning (disable : 4100)
#pragma warning (disable : 4201)
#pragma warning (disable : 4514)

#define STRICT
#include <WINDOWS.H>
#include <MAPISPI.H>
#include <MAPIUTIL.H>
#include <MAPIVAL.H>

#ifdef _DEBUG
#define     ENABLE_DEBUG_OUTPUT     1
#endif // _DEBUG

// Header file for the trace and logging functions.
#include "TRACES.H"

#define PR_NAMED_PROP_GUID      PROP_TAG (PT_CLSID,   6700)
#define PR_NAMED_PROP_ID        PROP_TAG (PT_LONG,    6701)
#define PR_NAMED_PROP_KIND      PROP_TAG (PT_LONG,    6702)
#define PR_NAMED_PROP_NAME      PROP_TAG (PT_UNICODE, 6703)
#define PR_NAMED_PROP_IDNAME    PROP_TAG (PT_LONG,    6704)

enum
{
    NP_GUID,
    NP_ID,
    NP_KIND,
    NP_NAME,
    NP_IDNAME,
    NAMED_PROPERTIES_TABLE_PROPS
};

const static SizedSPropTagArray(NAMED_PROPERTIES_TABLE_PROPS, sptNPProps) =
{
    NAMED_PROPERTIES_TABLE_PROPS,
    {
        PR_NAMED_PROP_GUID,
        PR_NAMED_PROP_ID,
        PR_NAMED_PROP_KIND,
        PR_NAMED_PROP_NAME,
        PR_NAMED_PROP_IDNAME
    }
};

typedef struct _STORED_NAMED_PROP
{
    MAPIUID     uidPropSet;
    ULONG       ulPropID;
    ULONG       ulKind;
    long        lIDNameOrSize;
} STORED_NAMED_PROP, *PSTORED_NAMED_PROP;

///////////////////////////////////////////////////////
// Named Properties Manager

class CNamePropMgr
{
public:
    STDMETHODIMP InitializeForInMemoryUse
                    ();
    STDMETHODIMP LoadMappingFromStream
                    (LPSTREAM                   pStream);
    STDMETHODIMP SaveMappingToStream
                    (LPSTREAM                   pStream);

    STDMETHODIMP GetIDForName
                    (LPGUID                     pPropSetGUID,
                     LPWSTR                     wstrName,
                     long		                lIDName,
                     ULONG						ulKind,
                     ULONG *                    pulNewID,
                     BOOL                       fCreateIfNotFound);
    STDMETHODIMP GetNameOfID
                    (ULONG                      ulPropID,
                     LPVOID                     pParentMemBlock,
                     LPMAPINAMEID  *            ppNamedProp);
    
    STDMETHODIMP GetAllNamesAndTags
                    (LPSPropTagArray *          ppPropTags,
                     LPMAPINAMEID **            pppPropNames,
                     ULONG                      ulFlags,
                     ULONG *                    pcPropNames);
    STDMETHODIMP GetMAPINamedPropsAndTags
                    (LPSPropTagArray            pObjPropTags,
                     LPSPropTagArray *          ppPropTags,
                     LPMAPINAMEID **            pppPropNames,
                     ULONG *                    pcPropNames);

    inline BOOL WINAPI IsDirty
                    () { return m_bMappingChanged; }

private:
    STDMETHODIMP CopyNamedProp
                    (LPVOID                     pParentMem,
                     LPMAPINAMEID  *            ppNamedProp,
                     LPSPropValue               pProps);
    inline STDMETHODIMP GetTableView
                    ();


///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CNamePropMgr
                    (LPALLOCATEBUFFER           pfnAllocateBuffer,
                     LPALLOCATEMORE             pfnAllocateMore,
                     LPFREEBUFFER               pfnFreeBuffer);
    ~CNamePropMgr();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    LPALLOCATEBUFFER    m_pfnAllocateBuffer;
    LPALLOCATEMORE      m_pfnAllocateMore;
    LPFREEBUFFER        m_pfnFreeBuffer;

    CRITICAL_SECTION    m_csObj;
    BOOL                m_bMappingChanged;
    ULONG               m_ulNextID;
    LPMAPITABLE         m_pNameTable;
    LPTABLEDATA         m_pMapData;
};

#endif // _NAMEPROP_H

// End of file for NAMEPROP.H
