
///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      FindCtlr.CPP
//
//  Author(s)
//      Nishad Mulye
//
//  Revision: 1.00
//

#pragma warning (disable : 4201 4514)

#include <windows.h>
#include <windowsx.h>
#include <mapix.h>
#include <mapiutil.h>
#include <commctrl.h>
#include <stdio.h>

#include "findctlr.h"
#include "findmdl.h"
#include "Advise.h"

void                    DestroyRestriction        ( LPSRestriction        pRes);
LPSRestriction          ConstructRestriction      ( const char* const     pszSender,
                                                    const char* const     pszRecip,
                                                    const char* const     pszSubject,
                                                    const char* const     pszBody);
void                    DestroySubRestriction     ( LPSRestriction        pRes);
LPSRestriction          ConstructSubRestriction   ( const char* const     pzNames, 
                                                    unsigned long         ulPropTag,
                                                    unsigned long*        pulRes);
unsigned long           Tokenize                  ( const char* const     pszNames,
                                                    char***               pppszTokens);

// file scope variables
unsigned long f_ulStoreIndex    = (unsigned long)-1;
LPMAPISESSION f_pSession        = 0;
LPMAPIFOLDER  f_pFolderSearch   = 0;
LPMDB         f_pMDBCurrent     = 0;
CAdviseSink*  f_pAdviseSink     = 0;
unsigned long f_ulConnection    = 0;

///////////////////////////////////////////////////////////////////////////////
//    CtlrInitialize()
//
//    Parameters
//		hWnd,     window handle of the main dialog box
//    hWndTree, window handle of the TreeView that displays folders Folders
//    hWndList, window handle of the ListView that displays message headers
//    hWndLB,   window handle of the List box that displays currently selected 
//              store
//
//    Purpose
//		initialization
//      
//    Return Value
//    TRUE on success, FALSE otherwise
BOOL                    CtlrInitialize            ( HWND                  hWnd,
                                                    HWND                  hWndTree,
                                                    HWND                  hWndList,
                                                    HWND                  hWndLB)
{
  BOOL    fRet  = FALSE;

  f_pAdviseSink = new CAdviseSink(hWnd);

  if (f_pAdviseSink)
  {
    f_pAdviseSink->AddRef();
    HRESULT hr    = MAPIInitialize(0);
    if (SUCCEEDED(hr))
    {
      hr = MAPILogonEx( 0,
                        0,
                        0,
                        MAPI_EXTENDED | MAPI_LOGON_UI,
                        &f_pSession);
      if (SUCCEEDED(hr))
      {
        char**        ppszStores = 0;
        unsigned long ulStores = 0;

        hr = ConstructMsgStoreList( f_pSession,
                                    &ppszStores,
                                    &ulStores);
        if (SUCCEEDED(hr))
        {
          for (unsigned long i = 0;i < ulStores;i++)
          {
            if (ppszStores[i])
            {
              SendMessage(hWndLB, 
                          CB_ADDSTRING, 
                          0, 
                          (LPARAM)ppszStores[i]);
              delete []ppszStores[i];                      
            }
          }
          delete []ppszStores;
          fRet = CtlrChangeStore( hWndTree,
                                  hWndList,
                                  0);
          if (fRet)
          {
            f_ulStoreIndex = 0;
            SendMessage(hWndLB,
                        CB_SETCURSEL,
                        (WPARAM)f_ulStoreIndex,
                        0);
          }
        }
      }
    }
  }
  return fRet;
}

///////////////////////////////////////////////////////////////////////////////
//    CtlrUninitialize()
//
//    Parameters
//    hWndTree, window handle of the TreeView that displays folders Folders
//    hWndList, window handle of the ListView that displays message headers
//
//    Purpose
//		de-initialization
//      
//    Return Value
//    TRUE on success, FALSE otherwise
BOOL                    CtlrUninitialize          ( HWND hWndTree,
                                                    HWND hWndList)
{
  BOOL  fRet = FALSE;

  if (f_pSession)
  {
    CtlrDestroyMsgList( hWndList);
    DestroyFolderTree(hWndTree);
    if (f_pFolderSearch)
    {
      DestroySearchFolder(f_pFolderSearch);
    }
    if (f_pMDBCurrent)
    {
      f_pMDBCurrent->Unadvise(f_ulConnection);
      f_ulConnection = 0;
      f_pMDBCurrent->Release();
    }
    DestroyMsgStoreList();
    f_pSession->Logoff( 0,
                        0,
                        0);
    f_pSession->Release();
    MAPIUninitialize();
  }
  if (f_pAdviseSink)
  {
    f_pAdviseSink->Release();
  }
  return fRet;
}

///////////////////////////////////////////////////////////////////////////////
//    CtlrFind()
//
//    Parameters
//    hWndTree,   window handle of the TreeView that displays folders Folders
//    hWndList,   window handle of the ListView that displays message headers
//    pszSender,  sender name to search for
//    pszRecip,   recipient name to search for
//    pszSubject, message subject to search for
//    pszBody,    message body to search for
//    fRecurse,   recursive search or shallow search
//
//    Purpose
//		setup and initiate the search
//      
//    Return Value
//    TRUE on success, FALSE otherwise
BOOL                    CtlrFind                  ( HWND      hWndTree,
                                                    HWND      hWndList,
                                                    char*     pszSender,
                                                    char*     pszRecip,
                                                    char*     pszSubject,
                                                    char*     pszBody,
                                                    BOOL      fRecurse)
{
  BOOL  fRet = FALSE;
  TV_ITEM tvI;

  tvI.hItem = TreeView_GetSelection(hWndTree);
  tvI.mask  = TVIF_HANDLE | TVIF_PARAM;
  TreeView_GetItem(hWndTree, &tvI);

  SBinary* pSB = (SBinary*)tvI.lParam;
  if (pSB)
  {
    ENTRYLIST entryList;

    entryList.cValues = 1;
    entryList.lpbin   = pSB;

    LPSRestriction pRes = ConstructRestriction( pszSender,
                                                pszRecip,
                                                pszSubject,
                                                pszBody);
    if (pRes)
    {
      unsigned long ulFlags = RESTART_SEARCH | 
                              FOREGROUND_SEARCH |
                              (fRecurse ? RECURSIVE_SEARCH : 0);

      CtlrDestroyMsgList(hWndList);
      HRESULT hr = f_pFolderSearch->SetSearchCriteria(pRes,
                                                      &entryList,
                                                      ulFlags);
      if (SUCCEEDED(hr))
      {
        fRet = TRUE;
      }
      DestroyRestriction(pRes);
    }
  }
  return fRet;
}

///////////////////////////////////////////////////////////////////////////////
//    CtlrStopFind()
//
//    Parameters
//
//    Purpose
//		abort the search
//      
//    Return Value
//    TRUE on success, FALSE otherwise
BOOL                    CtlrStopFind              ()
{
  BOOL  fRet = FALSE;

  HRESULT hr = f_pFolderSearch->SetSearchCriteria(0,
                                                  0,
                                                  STOP_SEARCH);
  if (SUCCEEDED(hr))
  {
    fRet = TRUE;
  }
  return fRet;
}

///////////////////////////////////////////////////////////////////////////////
//    CtlrChangeStore()
//
//    Parameters
//    hWndTree,   window handle of the TreeView that displays folders Folders
//    hWndList,   window handle of the ListView that displays message headers
//    ulIndex,    index of the new store selection
//
//    Purpose
//		change the current store selection and set up data
//      
//    Return Value
//    TRUE on success, FALSE otherwise
BOOL                    CtlrChangeStore           ( HWND          hWndTree,
                                                    HWND          hWndList,
                                                    unsigned long ulIndex)
{
  BOOL  fRet = FALSE;

  if (ulIndex != f_ulStoreIndex)
  {
    LPMDB pMDB = 0;
    HRESULT hr = SelectMsgStore(f_pSession,
                                ulIndex,
                                &pMDB);
    if (SUCCEEDED(hr))
    {
      CtlrDestroyMsgList( hWndList);
      DestroyFolderTree(  hWndTree);
      hr = ConstructFolderTree( pMDB,
                                hWndTree);
      if (SUCCEEDED(hr))
      {
        if (f_pFolderSearch)
        {
          DestroySearchFolder(f_pFolderSearch);
        }
        if (f_pMDBCurrent)
        {
          f_pMDBCurrent->Unadvise(f_ulConnection);
          f_ulConnection = 0;
          f_pMDBCurrent->Release();
        }
        f_pMDBCurrent = pMDB;
        f_ulStoreIndex = ulIndex;
        hr = f_pMDBCurrent->Advise( 0,
                                    0,
                                    fnevSearchComplete,
                                    f_pAdviseSink,
                                    &f_ulConnection);
        if (SUCCEEDED(hr))
        {
          hr = ConstructSearchFolder( f_pMDBCurrent,
                                      &f_pFolderSearch);
          if (SUCCEEDED(hr))
          {
            fRet = TRUE;
          }
        }
      }
    }
  }
  return fRet;
}

///////////////////////////////////////////////////////////////////////////////
//    CtlrDisplayMessage()
//
//    Parameters
//    hWnd,       handle of the parent window
//
//    Purpose
//		display the selected message
//      
//    Return Value
//    TRUE on success, FALSE otherwise
BOOL                    CtlrDisplayMessage        ( HWND hWnd)
{
  BOOL            fRet      = FALSE;
  DWORD           dwMsgPos  = GetMessagePos();
  LV_HITTESTINFO  lvHit;
  POINT           ptScreen,
                  ptClient;

  ZeroMemory( &lvHit,
              sizeof(lvHit));
  ptScreen.x  = LOWORD(dwMsgPos);
  ptScreen.y  = HIWORD(dwMsgPos);
  ptClient    = ptScreen;
  
  ScreenToClient( hWnd,
                  &ptClient);

  ptClient.x = 10;

  lvHit.pt    = ptClient;
  lvHit.flags = LVHT_ONITEM;
  lvHit.iItem = 0;

  ListView_HitTest( hWnd,
                    &lvHit);

  if (lvHit.iItem != -1)
  {
    LV_ITEM lvI;

    lvI.mask = LVIF_PARAM;
    lvI.iItem = lvHit.iItem;
    lvI.iSubItem = 0;
    ListView_GetItem( hWnd,
                      &lvI);

    SBinary* pSB = (SBinary*)lvI.lParam;
    if (pSB)
    {
      HRESULT       hr            = 0;
      LPMESSAGE     pMessage      = 0;
      unsigned long ulObjectType  = 0;
      
      hr = f_pFolderSearch->OpenEntry(pSB->cb,
                                      (LPENTRYID)pSB->lpb,
                                      0,
                                      0,
                                      &ulObjectType,
                                      (LPUNKNOWN*)&pMessage);
      if (SUCCEEDED(hr))
      {
        unsigned long ulToken = 0;
        hr = f_pSession->PrepareForm( 0,
                                      pMessage,
                                      &ulToken);
        if (SUCCEEDED(hr))
        {
          const SizedSPropTagArray(4, atProps) =
          {
            4,
            PR_MSG_STATUS,
            PR_MESSAGE_FLAGS,
            PR_ACCESS_LEVEL,
            PR_MESSAGE_CLASS
          };
          unsigned long ulTmp = 0;
          SPropValue*   pVal  = 0;
          hr = pMessage->GetProps((LPSPropTagArray)&atProps,
                                  0,
                                  &ulTmp,
                                  &pVal);
          pMessage->Release();
          if (SUCCEEDED(hr))
          {
            hr = f_pSession->ShowForm(0,
                                      f_pMDBCurrent, 
                                      f_pFolderSearch,
                                      0,
                                      ulToken,
                                      0,
                                      MAPI_POST_MESSAGE,
                                      pVal[0].Value.ul, 
                                      pVal[1].Value.ul,
                                      pVal[2].Value.ul,
                                      pVal[3].Value.lpszA);
            MAPIFreeBuffer(pVal);
            fRet = TRUE;
          }
        }
        else
        {
          pMessage->Release();
        }
      }
    }
  }
  return fRet;
}

///////////////////////////////////////////////////////////////////////////////
//    ConstructRestriction()
//
//    Parameters
//		hWnd,	window handle of the main dialog
//
//    Purpose
//		build a restriction using the specified keys
//      
//    Return Value
//		pointer to a SRestriction structure
// 
LPSRestriction          ConstructRestriction      ( const char* const pszSender,
                                                    const char* const pszRecip,
                                                    const char* const pszSubject,
                                                    const char* const pszBody)
{
  LPSRestriction pRes = 0;
  const unsigned nMaxRes = 4;
  unsigned       nActualRes = 0;
  unsigned       i = 0;
  BOOL           fSuccess = FALSE;

  if (pszSender)
  {
    if (*pszSender)
    {
      nActualRes++;
    }
  }
  if (pszRecip)
  {
    if (*pszRecip)
    {
      nActualRes++;
    }
  }
  if (pszSubject)
  {
    if (*pszSubject)
    {
      nActualRes++;
    }
  }
  if (pszBody)
  {
    if (*pszBody)
    {
      nActualRes++;
    }
  }
  if (nActualRes)
  {
    pRes = new SRestriction;
    if (pRes)
    {
      ZeroMemory( pRes,
                  sizeof(SRestriction));
      LPSRestriction resArray = new SRestriction[nActualRes];

      if (resArray)
      {
        ZeroMemory( resArray,
                    sizeof(SRestriction) * nActualRes);
        if (nActualRes > 1)
        {
          pRes->rt = RES_AND;
          pRes->res.resAnd.cRes = nActualRes;
          pRes->res.resAnd.lpRes = resArray;
        }
        else
        {
          pRes->rt = RES_OR;
          pRes->res.resOr.cRes = nActualRes;
          pRes->res.resOr.lpRes = resArray;
        }
        LPSPropValue   pSPV = 0;
        i = nMaxRes-1;
        unsigned long j = nActualRes - 1;
        // Body
        if (*pszBody && j >= 0)
        {
          fSuccess = FALSE;
          resArray[j].rt =  RES_CONTENT;
          resArray[j].res.resContent.ulFuzzyLevel = 
                            FL_SUBSTRING | FL_IGNORECASE;
          pSPV = new SPropValue;
          if (pSPV)
          {
            resArray[j].res.resContent.lpProp = pSPV;
            pSPV->ulPropTag = PR_BODY;
            resArray[j].res.resContent.ulPropTag = PR_BODY;
            pSPV->Value.lpszA = new char[strlen(pszBody)+1];
            if (pSPV->Value.lpszA)
            {
              strcpy( pSPV->Value.lpszA,
                      pszBody);
              fSuccess = TRUE;
            }
          }
          j--;
        }
        // Subject
        i--;
        if (*pszSubject && j >= 0)
        {
          fSuccess = FALSE;
          resArray[j].rt =  RES_CONTENT;
          resArray[j].res.resContent.ulFuzzyLevel = 
                            FL_SUBSTRING | FL_IGNORECASE;
          pSPV = new SPropValue;
          if (pSPV)
          {
            ZeroMemory(pSPV, sizeof(SPropValue));
            resArray[j].res.resContent.lpProp = pSPV;
            pSPV->ulPropTag = PR_SUBJECT;
            resArray[j].res.resContent.ulPropTag = PR_SUBJECT;
            unsigned uSize = strlen(pszSubject)+1;
            pSPV->Value.LPSZ = new char[uSize];
            if (pSPV->Value.lpszA)
            {
              strcpy( pSPV->Value.lpszA,
                      pszSubject);
              fSuccess = TRUE;
            }
          }
          j--;
        }
        // Recipient
        i--;
        if (*pszRecip && j >= 0)
        {
          resArray[j].rt = RES_SUBRESTRICTION;
          resArray[j].res.resSub.ulSubObject = PR_MESSAGE_RECIPIENTS;
          resArray[j].res.resSub.lpRes = ConstructSubRestriction( pszRecip,
                                                                  PR_DISPLAY_NAME,
                                                                  0);
          fSuccess = resArray[j].res.resSub.lpRes ? TRUE : FALSE;
          j--;
        }
        // Sender
        i--;
        if (*pszSender && j >= 0)
        {
          unsigned long ulRes = 0;
          resArray[j].rt = RES_OR;
          resArray[j].res.resOr.lpRes = ConstructSubRestriction(pszSender,
                                                                PR_SENDER_NAME,
                                                                &ulRes);
          resArray[j].res.resOr.cRes = ulRes;
          fSuccess = resArray[j].res.resOr.lpRes ? TRUE : FALSE;
          j--;
        }
      }
    }
  }
  if (fSuccess == FALSE)
  {
    DestroyRestriction(pRes);
    pRes = 0;
  }
  return pRes;
}

///////////////////////////////////////////////////////////////////////////////
//    DestroyRestriction()
//
//    Parameters
//		pRes,	pointer to a SRestriction structure
//
//    Purpose
//		garbage collection
//      
//    Return Value
// 
void                    DestroyRestriction        ( LPSRestriction pRes)
{
  if (pRes)
  {
    LPSRestriction resArray = 0;
    unsigned long  ulRes = 0;
    if (pRes->rt == RES_AND)
    {
      resArray = pRes->res.resAnd.lpRes;
      ulRes = pRes->res.resAnd.cRes;
    }
    else if (pRes->rt == RES_OR)
    {
      resArray = pRes->res.resOr.lpRes;
      ulRes = pRes->res.resOr.cRes;
    }
    if (resArray)
    {
      for (unsigned i = 0;i < ulRes;i++)
      {
        if (resArray[i].rt == RES_CONTENT)
        {
          delete resArray[i].res.resContent.lpProp->Value.LPSZ;
          delete resArray[i].res.resContent.lpProp;
        }
        else if (resArray[i].rt == RES_OR)
        {
          DestroySubRestriction(resArray[i].res.resOr.lpRes);
        }
        else if (resArray[i].rt == RES_SUBRESTRICTION)
        {
          DestroySubRestriction(resArray[i].res.resSub.lpRes);
        }
      }
      delete []resArray;
    }
    delete pRes;
  }
}

///////////////////////////////////////////////////////////////////////////////
//    ConstructSubRestriction()
//
//    Parameters
//		pzNames,	string of tokens
//		ulPropTag,	property of the key, PR_DISPLAY_NAME etc
//		pulRes,		number of SRestriction's allocated
//
//    Purpose
//      
//    Return Value
//		pointer to a SRestriction
// 
LPSRestriction          ConstructSubRestriction   ( const char* const pzNames, 
                                                    unsigned long ulPropTag,
                                                    unsigned long *pulRes)
{
  BOOL          fSuccess  = FALSE;
  unsigned long ulTokens  = 0;
  LPSRestriction pRes     = new SRestriction;

  if (pRes)
  {
    char**        ppszTokens = 0;

    ZeroMemory( pRes,
                sizeof(SRestriction));
    ulTokens = Tokenize(pzNames,
                        &ppszTokens);
    if (ulTokens != 0)
    {
      pRes->rt = RES_OR;
      pRes->res.resOr.lpRes = new SRestriction[ulTokens];
      LPSPropValue pProp = new SPropValue[ulTokens];
      if (pRes->res.resOr.lpRes && pProp)
      {
        pRes->res.resOr.cRes = ulTokens;
        ZeroMemory(pRes->res.resOr.lpRes, ulTokens * sizeof(SRestriction));
        ZeroMemory(pProp, ulTokens * sizeof(SPropValue));
        unsigned long i;
        unsigned long ulSeeed = 0;
        for (i = 0;i < ulTokens;i++)
        {
          pRes->res.resOr.lpRes[i].rt = RES_CONTENT;
          pRes->res.resOr.lpRes[i].res.resContent.ulFuzzyLevel = 
                                              FL_SUBSTRING | FL_IGNORECASE;
          pRes->res.resOr.lpRes[i].res.resContent.ulPropTag = ulPropTag;
          pRes->res.resOr.lpRes[i].res.resContent.lpProp = &pProp[i];
          pProp[i].ulPropTag = ulPropTag;
          pProp[i].Value.lpszA = ppszTokens[i];
        }
        fSuccess = TRUE;
      }
    }
    delete []ppszTokens;
  }
  if (!fSuccess)
  {
    DestroySubRestriction(pRes);
    pRes = 0;
  }
  else
  {
    if (pulRes)
    {
      *pulRes = ulTokens;
    }
  }
  return pRes;
}

///////////////////////////////////////////////////////////////////////////////
//    DestroySubRestriction()
//
//    Parameters
//		pRes,	pointer to a SRestriction structure
//
//    Purpose
//		garbage collection
//      
//    Return Value
// 
void                    DestroySubRestriction     ( LPSRestriction pRes)
{
  if (pRes)
  {
    if (pRes->res.resOr.lpRes)
    {
      if (pRes->res.resOr.lpRes[0].res.resContent.lpProp)
      {
        for (unsigned long i = 0;i < pRes->res.resOr.cRes;i++)
        {
          if (pRes->res.resOr.lpRes[i].res.resContent.lpProp)
          {
            delete [](pRes->res.resOr.lpRes[i].res.resContent.lpProp->Value.lpszA);
          }
        }
        delete [](pRes->res.resOr.lpRes[0].res.resContent.lpProp);
      }
      delete [](pRes->res.resOr.lpRes);
    }
    delete pRes;
  }
}

///////////////////////////////////////////////////////////////////////////////
//    Tokenize()
//
//    Parameters
//    pszStr,        source string
//    pppszTokens,   array of of tokens
//
//    Purpose
//		break the string into tokens delimited by ';'
//      
//    Return Value
//    TRUE on success, FALSE otherwise
unsigned long           Tokenize                  ( const char* const pszStr,
                                                    char***           pppszTokens)
{
  unsigned long ulCount = 0;

  for (char* pc = (char*) pszStr;*pc;pc++)
  {
    if (*pc == ';')
    {
      ulCount++;
    }
  }
  ulCount += 2;

  char**  ppszTokens  = new char*[ulCount];
  char*   pszToken    = 0;
  
  ZeroMemory( ppszTokens,
              sizeof(char*) * ulCount);
  pszToken = strtok((char*)pszStr,
                    ";");
  ulCount = 0;
  while(pszToken != 0)
  {
    ppszTokens[ulCount] = new char[strlen(pszToken)+1];
    strcpy( ppszTokens[ulCount],
            pszToken);
    ulCount++;
    pszToken = strtok(0,
                      ";");
  }
  *pppszTokens = ppszTokens;
  return ulCount;
}

///////////////////////////////////////////////////////////////////////////////
//    CtlrConstructMsgList()
//
//    Parameters
//    hWndList,     ListView control to display message headers
//
//    Purpose
//		populate the ListView control with message headers that satisfied the 
//    search criteria
//      
//    Return Value
//    TRUE on success, FALSE otherwise
HRESULT                 CtlrConstructMsgList      ( HWND         hWndList)
{
  LPMAPITABLE pTable = 0;
  HRESULT hr = f_pFolderSearch->GetContentsTable( fMapiUnicode,
                                                  &pTable);
  if (SUCCEEDED(hr))
  {
    const SizedSPropTagArray(4, atProp) = 
    {
      4,
      PR_SENDER_NAME,
      PR_SUBJECT,
      PR_MESSAGE_DELIVERY_TIME,
      PR_ENTRYID,
    };
    LPSRowSet pRows = 0;

    hr = HrQueryAllRows(pTable,
                        (SPropTagArray*)&atProp, 
                        0,
                        0,
                        0,
                        &pRows);
    if (SUCCEEDED(hr))
    {
      LPSPropValue pVal = 0;
      SBinary *pSB = 0;

      LV_ITEM lvI;
      lvI.mask = LVIF_PARAM;
      lvI.iSubItem = 0;
      for (unsigned long i = 0;i < pRows->cRows;i++)
      {
        pVal = pRows->aRow[i].lpProps;
        lvI.iItem = i;
        if (pVal[3].ulPropTag == PR_ENTRYID)
        {
          pSB = CopySBinary(&(pVal[3].Value.bin));

          if (pSB)
          {
            lvI.lParam = (LPARAM)pSB;
            ListView_InsertItem(hWndList,
                                &lvI);

            unsigned j  = 0;
            if (pVal[j].ulPropTag == PR_SENDER_NAME)
            {
              ListView_SetItemText( hWndList,
                                    i,
                                    j, 
                                    pVal[j].Value.LPSZ);
            }
            j++;
            if (pVal[j].ulPropTag == PR_SUBJECT)
            {
              ListView_SetItemText( hWndList,
                                    i,
                                    j,
                                    pVal[j].Value.LPSZ);
            }
            j++;
            if (pVal[j].ulPropTag == PR_MESSAGE_DELIVERY_TIME)
            {
              char        pTmp[128];
              SYSTEMTIME  st;

              FileTimeToSystemTime( &pVal[j].Value.ft,
                                    &st);
              sprintf(pTmp,
                      "%u/%u/%u %u:%02u",
                      st.wMonth,
                      st.wDay,
                      st.wYear,
                      st.wHour,
                      st.wMinute);
              ListView_SetItemText( hWndList,
                                    i,
                                    j,
                                    pTmp);
            }
          }
        }
      }
      FreeProws(pRows);
    }
    pTable->Release();
  }
  return 0;
}

///////////////////////////////////////////////////////////////////////////////
//    CtlrDestroyMsgList()
//
//    Parameters
//    hWndList,     ListView control to display message headers
//
//    Purpose
//		cleanup the message list
//      
//    Return Value
//    TRUE on success, FALSE otherwise
HRESULT                 CtlrDestroyMsgList        ( HWND hWndList)
{
  unsigned nItems = ListView_GetItemCount(hWndList);
  
  LV_ITEM lvI;
  lvI.mask = LVIF_PARAM;
  lvI.iSubItem = 0;
  for (unsigned i = 0;i < nItems;i++)
  {
    lvI.iItem = i;
    if (ListView_GetItem( hWndList,
                          &lvI))
    {
      SBinary* pSB = (SBinary*) lvI.lParam;
      DestroySBinary(pSB);
    }
  }
  ListView_DeleteAllItems(hWndList);
  return 0;
}
