
///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      FindMdl.CPP
//
//  Author(s)
//      Nishad Mulye
//
//  Revision: 1.00
//

#pragma warning (disable : 4201 4514)

#include <windows.h>
#include "FindMdl.h"
#include <mapiutil.h>
#include <commctrl.h>

// file scope variables
SBinary**     f_apSBStoreEID = 0;
unsigned long f_ulStoreListSize = 0;

///////////////////////////////////////////////////////////////////////////////
//    CopySBinary()
//
//    Parameters
//    pSB,      source to copy
//
//    Purpose
//    make a copy of the source SBinary
//      
//    Return Value
//    new SBinary structure
//      
SBinary*                CopySBinary               ( const SBinary* pSB)
{
  SBinary*  pSBRet = new SBinary();

  if (pSBRet)
  {
    pSBRet->lpb = new BYTE[pSB->cb];
    if (pSBRet->lpb)
    {
      MoveMemory( pSBRet->lpb,
                  pSB->lpb,
                  sizeof(BYTE) * pSB->cb);
      pSBRet->cb = pSB->cb;
    }
    else
    {
      delete pSBRet;
      pSBRet = 0;
    }
  }
  return pSBRet;
}

///////////////////////////////////////////////////////////////////////////////
//    DestroySBinary()
//
//    Parameters
//    pSB,      source to copy
//
//    Purpose
//    destroy the SBinary
//      
//    Return Value
//      
void                      DestroySBinary          ( SBinary* pSB)
{
  if (pSB)
  {
    delete pSB->lpb;
  }
  delete pSB;
}

///////////////////////////////////////////////////////////////////////////////
//    ConstructSearchFolder()
//
//    Parameters
//    pMDB,       selected store
//    ppFolder,   newly created search folder
//
//    Purpose
//    create a search folder in the selected store's FINDER subtree
//      
//    Return Value
//    HRESULT
HRESULT                 ConstructSearchFolder     ( LPMDB         pMDB,
                                                    LPMAPIFOLDER* ppFolder)
{
  HRESULT hr = S_OK;
  const SizedSPropTagArray(1, atProps) =
  {
    1,
    PR_FINDER_ENTRYID
  };
  LPSPropValue pVal = 0;
  unsigned long ulVal = 0;
  // get the entry ID for finder subtree
  hr = pMDB->GetProps((LPSPropTagArray)&atProps,
                      fMapiUnicode, 
                      &ulVal,
                      &pVal);
  if (SUCCEEDED(hr))
  {
    unsigned long ulObjectType  = 0;
    LPMAPIFOLDER  pFolder       = 0;

    // open the finder subtree
    hr = pMDB->OpenEntry( pVal[0].Value.bin.cb, 
                          (LPENTRYID)pVal[0].Value.bin.lpb,
                          0,
                          MAPI_MODIFY,
                          &ulObjectType, 
                          (LPUNKNOWN*)&pFolder);
    if (SUCCEEDED(hr))
    {
      // create our search folder 
      hr = pFolder->CreateFolder( FOLDER_SEARCH, 
                                  "Search Folder", 
                                  "Search Folder",
                                  0,
                                  fMapiUnicode | OPEN_IF_EXISTS,
                                  ppFolder);
      pFolder->Release();
    }
    MAPIFreeBuffer(pVal);
  }
  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    DestroySBinary()
//
//    Parameters
//    pFolder,    search folder
//
//    Purpose
//    delete a search folder in the selected store's FINDER subtree
//      
//    Return Value
//    HRESULT
HRESULT                 DestroySearchFolder       ( LPMAPIFOLDER  pFolder)
{
  HRESULT       hr            = S_OK;
  LPMAPIFOLDER  pFolderParent = 0;

  hr = pFolder->OpenProperty( PR_PARENT_ENTRYID,
                              0,
                              0,
                              MAPI_MODIFY,
                              (LPUNKNOWN*)&pFolderParent);
  if (SUCCEEDED(hr))
  {
    const SizedSPropTagArray(1, atProps) =
    {
      1,
      PR_ENTRYID
    };
    LPSPropValue  pVal    = 0;
    unsigned long ulCount = 0;

    hr = pFolder->GetProps( (LPSPropTagArray)&atProps,
                            0,
                            &ulCount,
                            &pVal);
    pFolder->Release();
    if (SUCCEEDED(hr))
    {
      hr = pFolderParent->DeleteFolder( pVal->Value.bin.cb,
                                        (LPENTRYID)pVal->Value.bin.lpb,
                                        0,
                                        0,
                                        DEL_FOLDERS | DEL_MESSAGES);
      MAPIFreeBuffer(pVal);
    }
    pFolderParent->Release();
  }
  else
  {
    pFolder->Release();
  }
  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    SelectMsgStore()
//
//    Parameters
//    pSession,     handle of the MAPI session
//    ulIndex,      index of the store to select
//    ppMDB,        handle of the new store
//
//    Purpose
//    open and initialize a new message store
//      
//    Return Value
//    HRESULT
HRESULT                 SelectMsgStore            ( LPMAPISESSION pSession,
                                                    unsigned long ulIndex,
                                                    LPMDB*        ppMDB)
{
  HRESULT hr = S_OK;

  if (ulIndex >= 0 && ulIndex < f_ulStoreListSize)
  {
    hr = pSession->OpenMsgStore(0,
                                f_apSBStoreEID[ulIndex]->cb,
                                (LPENTRYID)f_apSBStoreEID[ulIndex]->lpb,
                                0,
                                MDB_WRITE,
                                ppMDB);
  }
  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    ConstructMsgStoreList()
//
//    Parameters
//    pSession,     handle of the MAPI session
//    pppszStores,  display names of stores
//    pulStores,    number of store
//
//    Purpose
//    construct a list of message stores
//      
//    Return Value
//    HRESULT
HRESULT                 ConstructMsgStoreList     ( LPMAPISESSION   pSession,
                                                    char***         pppszStores,
                                                    unsigned long*  pulStores)
{
  LPMAPITABLE pTable  = 0;
  HRESULT     hr      = pSession->GetMsgStoresTable(0,
                                                    &pTable);
  if (SUCCEEDED(hr))
  {
    const SizedSPropTagArray (2, taProps) = 
    {
      2,
      PR_DISPLAY_NAME,
      PR_ENTRYID
    };
    LPSRowSet pRows = 0;

    hr = HrQueryAllRows(pTable,
                        (LPSPropTagArray)&taProps,
                        0,
                        0,
                        0,
                        &pRows);
    if (SUCCEEDED(hr))
    {
      *pppszStores    = new char*[pRows->cRows];
      f_apSBStoreEID  = new SBinary*[pRows->cRows];
      if (pppszStores && f_apSBStoreEID)
      {
        LPSPropValue  pVal = 0;
        LPMDB         pMDB = 0;
        ZeroMemory( *pppszStores,
                    sizeof(char*) * pRows->cRows);
        ZeroMemory( f_apSBStoreEID,
                    sizeof(SBinary*) * pRows->cRows);
        f_ulStoreListSize = 0;
        for (unsigned long i = 0;i < pRows->cRows;i++)
        {
          pVal = pRows->aRow[i].lpProps;
          hr = pSession->OpenMsgStore(0,
                                      pVal[1].Value.bin.cb,
                                      (LPENTRYID)pVal[1].Value.bin.lpb,
                                      0, 
                                      MDB_WRITE,
                                      &pMDB);
          if (SUCCEEDED(hr))
          {
            const SizedSPropTagArray(1, taPropStore) =
            {
              1,
              PR_STORE_SUPPORT_MASK
            };
            LPSPropValue val = 0;
            unsigned long ulCnt = 0;
            hr = pMDB->GetProps((LPSPropTagArray)&taPropStore,
                                0,
                                &ulCnt,
                                &val);
            if (SUCCEEDED(hr))
            {
              ulCnt = val->Value.ul;
              if ((ulCnt & STORE_SEARCH_OK)       &&
                  (ulCnt & STORE_RESTRICTION_OK)  &&
                  (ulCnt & STORE_MODIFY_OK)       &&
                  (ulCnt & STORE_NOTIFY_OK))
              {
                (*pppszStores)[f_ulStoreListSize] = new char[strlen(pVal->Value.lpszA)+1];
                f_apSBStoreEID[f_ulStoreListSize] = CopySBinary(&(pVal[1].Value.bin));
                if ((*pppszStores)[f_ulStoreListSize] && 
                    f_apSBStoreEID[f_ulStoreListSize])
                {
                  strcpy( (*pppszStores)[f_ulStoreListSize],
                          pVal->Value.lpszA);
                  f_ulStoreListSize++;
                }
              }
            }
            pMDB->Release();
          }
        }
        *pulStores = f_ulStoreListSize;
        if (f_ulStoreListSize == 0)
        {
          hr = MAPI_E_NOT_FOUND;
        }
      }
      else
      {
        delete []*pppszStores;
        *pppszStores = 0;
        delete []f_apSBStoreEID;
      }
      FreeProws(pRows);
    }
    pTable->Release();
  }
  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    DestroyMsgStoreList()
//
//    Parameters
//
//    Purpose
//    destroy and cleanup message stores list
//      
//    Return Value
//    HRESULT
HRESULT                 DestroyMsgStoreList       ()
{
  HRESULT hr = S_OK;

  for (unsigned long i = 0;i < f_ulStoreListSize;i++)
  {
    DestroySBinary(f_apSBStoreEID[i]);
  }
  delete []f_apSBStoreEID;
  
  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    InsertNodes()
//
//    Parameters
//    pMDB      message store
//    hWnd      handle of the tree view control
//    hTreeItem positon in the tree view
//
//    Purpose
//    insert folders in the TreeView control
//      
//    Return Value
//    
void                    InsertNodes               ( LPMDB     pMDB,
                                                    HWND      hWnd,
                                                    HTREEITEM hTreeItem)
{
  TV_ITEM tvI;
  ZeroMemory( &tvI,
              sizeof(tvI));

  tvI.mask          = TVIF_HANDLE | TVIF_PARAM;
  tvI.hItem         = hTreeItem;
  TreeView_GetItem( hWnd,
                    &tvI);

  SBinary*  pSB = (SBinary*)tvI.lParam;

  if (pSB)
  {
    unsigned long ulObjType = 0;
    LPMAPIFOLDER  pFolder   = 0;
    HRESULT hr = pMDB->OpenEntry( pSB->cb,
                                  (LPENTRYID)pSB->lpb,
                                  0,
                                  MAPI_BEST_ACCESS,
                                  &ulObjType,
                                  (LPUNKNOWN*)&pFolder);
    if (SUCCEEDED(hr))
    {
      LPMAPITABLE pTable = 0;
      hr = pFolder->GetHierarchyTable(0,
                                      &pTable);
      if (SUCCEEDED(hr))
      {
        const SizedSPropTagArray(2, atProps) = 
        {
          2,
          PR_DISPLAY_NAME,
          PR_ENTRYID
        };
        unsigned long ulCount = 0;
        LPSRowSet     pRows    = 0;

        hr = HrQueryAllRows(pTable,
                            (LPSPropTagArray)&atProps,
                            0,
                            0,
                            0,
                            &pRows);
        if (SUCCEEDED(hr))
        {
          TV_INSERTSTRUCT  tvIS;
          ZeroMemory( &tvIS,
                      sizeof(tvIS));

          tvI.mask          = TVIF_TEXT | TVIF_PARAM;
          tvIS.hInsertAfter = TVI_SORT;
          tvIS.hParent      = hTreeItem;
          LPSPropValue pVal = 0;

          for (unsigned long i = 0;i < pRows->cRows;i++)
          {
            pVal = pRows->aRow[i].lpProps;
            tvI.cchTextMax  = strlen(pVal[0].Value.lpszA);
            tvI.pszText     = pVal[0].Value.lpszA;
            tvI.lParam      = (LPARAM)CopySBinary(&(pVal[1].Value.bin));
            tvIS.item       = tvI;
            TreeView_InsertItem(hWnd,
                                &tvIS);
          }
          FreeProws(pRows);
        }
        pTable->Release();
      }
      pFolder->Release();
    }
  }
}

///////////////////////////////////////////////////////////////////////////////
//    ConstructFolderTree()
//
//    Parameters
//    pMDB      message store
//    hWnd      handle of the tree view control
//
//    Purpose
//    insert folders in the TreeView control
//      
//    Return Value
//    HRESULT
HRESULT                 ConstructFolderTree       ( LPMDB pMDB,
                                                    HWND  hWnd)
{
  HRESULT hr = S_OK;
  const SizedSPropTagArray(2, atProps) = 
  {
    2,
    PR_DISPLAY_NAME,
    PR_IPM_SUBTREE_ENTRYID
  };
  unsigned long ulCount = 0;
  LPSPropValue  pVal    = 0;

  hr = pMDB->GetProps((LPSPropTagArray)&atProps,
                      0,
                      &ulCount,
                      &pVal);
  if (SUCCEEDED(hr))
  {
    // insert
    TV_ITEM          tvI;
    TV_INSERTSTRUCT  tvIS;

    ZeroMemory( &tvI,
                sizeof(tvI));
    ZeroMemory( &tvIS,
                sizeof(tvIS));

    tvI.mask        = TVIF_TEXT | TVIF_PARAM;
    tvI.cchTextMax  = strlen(pVal[0].Value.lpszA);
    tvI.pszText     = pVal[0].Value.lpszA;
    tvI.lParam      = (LPARAM)CopySBinary(&(pVal[1].Value.bin));

    tvIS.item         = tvI;
    tvIS.hInsertAfter = TVI_SORT;

    HTREEITEM hTreeP = 0,
              hTreeQ = 0;
    hTreeP = TreeView_InsertItem( hWnd,
                                  &tvIS);
    while (hTreeP)
    {
      InsertNodes(pMDB,
                  hWnd,
                  hTreeP);
      hTreeQ = hTreeP;
      hTreeP = TreeView_GetChild( hWnd,
                                  hTreeP);
      if (hTreeP == 0)
      {
        hTreeP = TreeView_GetNextSibling( hWnd,
                                          hTreeQ);
        if (hTreeP == 0)
        {
          hTreeP = TreeView_GetParent(hWnd,
                                      hTreeQ);
          if (hTreeP != 0)
          {
            hTreeQ = hTreeP;
            hTreeP = TreeView_GetNextSibling( hWnd,
                                              hTreeQ);
          }
        }
      }
    }
    MAPIFreeBuffer(pVal);
  }
  return hr;
}

///////////////////////////////////////////////////////////////////////////////
//    DestroyFolderTree()
//
//    Parameters
//    hWnd      handle of the tree view control
//
//    Purpose
//    destroy the TreeView control
//      
//    Return Value
//    
HRESULT                 DestroyFolderTree         ( HWND hWnd)
{
  HRESULT   hr      = S_OK;
  HTREEITEM hTreeP  = 0,
            hTreeQ  = 0;
  TV_ITEM   tvI;

  ZeroMemory( &tvI,
              sizeof(tvI));

  tvI.mask          = TVIF_HANDLE | TVIF_PARAM;

  hTreeP = TreeView_GetRoot(hWnd);
  while (hTreeP != 0)
  {
    tvI.hItem         = hTreeP;
    TreeView_GetItem( hWnd,
                      &tvI);
    DestroySBinary((SBinary*)tvI.lParam);

    hTreeQ = hTreeP;
    hTreeP = TreeView_GetChild( hWnd,
                                hTreeP);
    if (hTreeP == 0)
    {
      hTreeP = TreeView_GetNextSibling( hWnd,
                                        hTreeQ);
      if (hTreeP == 0)
      {
        hTreeP = TreeView_GetParent(hWnd,
                                    hTreeQ);
        if (hTreeP != 0)
        {
          hTreeP = TreeView_GetNextSibling( hWnd,
                                            hTreeP);
        }
      }
    }
  }

  TreeView_DeleteAllItems(hWnd);
  return hr;
}
