
///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      MsgFinder.CPP
//
//  Author(s)
//      Nishad Mulye
//
//  Revision: 1.00
//

#pragma warning (disable : 4201 4514)

#define INITGUID
#define USES_IID_IMAPIAdviseSink

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <ctl3d.h>
#include <initguid.h>
#include <mapiguid.h>

#include "resource.h"
#include "FindCtlr.h"

// file scope typedefs
typedef BOOL (WINAPI *t_pfnCtl3dColorChange)    (void);
typedef BOOL (WINAPI *t_pfnCtl3dAutoSubclass)   (HINSTANCE);
typedef BOOL (WINAPI *t_pfnCtl3dUnAutoSubclass) (void);
typedef BOOL (WINAPI *t_pfnCtl3dRegister)       (HINSTANCE);
typedef BOOL (WINAPI *t_pfnCtl3dUnregister)     (HINSTANCE);

// file scope variables
t_pfnCtl3dColorChange f_pfnCtl3dColorChange = 0;
unsigned long         f_ulIndex             = 0;
BOOL                  f_fFinding            = FALSE;

///////////////////////////////////////////////////////////////////////////////
//    CenterWindow()
//
//    Parameters
//      handle of the window to center
//
//    Purpose
//      to center a window relative to the screen dimensions or its parent
//      
//    Return Value
//      None
//      
void                    CenterWindow              ( HWND    hWnd,
                                                    HWND    hWndParent)
{
	RECT rect;
	unsigned uXExt;
	unsigned uYExt;
	if (hWndParent != 0)
	{
		GetWindowRect(hWndParent, &rect);
	  uXExt = rect.left + (rect.right  - rect.left) / 2;
	  uYExt = rect.top  + (rect.bottom - rect.top ) / 2;
	}
	else
	{
		uXExt = GetSystemMetrics(SM_CXSCREEN) / 2;
		uYExt = GetSystemMetrics(SM_CYSCREEN) / 2;
	}
	GetWindowRect(hWnd, &rect);
	uXExt -= (rect.right  - rect.left) / 2;
	uYExt -= (rect.bottom - rect.top ) / 2;
	SetWindowPos( hWnd,
                0,
                uXExt,
                uYExt,
                0,
                0,
                SWP_NOSIZE);
}

///////////////////////////////////////////////////////////////////////////////
//    EnableControls()
//
//    Parameters
//      hWnd,     handle of the window to center
//      fEnable,  enable or disable
//
//    Purpose
//      to center a windowrelative to the screen dimensions
//      
//    Return Value
//      None
//      
void                    EnableControls            ( HWND    hWnd,
                                                    BOOL    fEnable)
{
  EnableWindow( GetDlgItem( hWnd,
                            IDC_STORE_CB),
                fEnable);
  EnableWindow( GetDlgItem( hWnd,
                            IDC_MESSAGE_LV),
                fEnable);
  EnableWindow( GetDlgItem( hWnd,
                            IDC_FOLDERS_TV),
                fEnable);
  EnableWindow( GetDlgItem( hWnd,
                            IDC_RECURSE_XB),
                fEnable);
  EnableWindow( GetDlgItem( hWnd,
                            IDC_SENDER_ED),
                fEnable);
  EnableWindow( GetDlgItem( hWnd,
                            IDC_RECIP_ED),
                fEnable);
  EnableWindow( GetDlgItem( hWnd,
                            IDC_SUBJECT_ED),
                fEnable);
  EnableWindow( GetDlgItem( hWnd,
                            IDC_BODY_ED),
                fEnable);
  EnableWindow( GetDlgItem( hWnd,
                            IDCLOSE),
                fEnable);
}

///////////////////////////////////////////////////////////////////////////////
//    FindDlgCmdHandler()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      WM_COMMAND messages handler for main dialog
//      
//    Return Value
// 
BOOL                    FindDlgCmdHandler         ( HWND      hWnd,
                                                    UINT      ,
                                                    WPARAM    wParam,
                                                    LPARAM    lParam)
{
  BOOL fRet = FALSE;
  switch (LOWORD(wParam))
  {
    case IDC_SENDER_ED:
    case IDC_SUBJECT_ED:
    case IDC_RECIP_ED:
    case IDC_BODY_ED:
      // every time one of the above edit controls loses focus, or changes
      // perform checks and set the state of the Find button
      if (HIWORD(wParam) == EN_KILLFOCUS || HIWORD(wParam) == EN_CHANGE)
      {
        if (GetWindowTextLength(GetDlgItem(hWnd, IDC_SENDER_ED )) > 0 ||
            GetWindowTextLength(GetDlgItem(hWnd, IDC_SUBJECT_ED)) > 0 ||
            GetWindowTextLength(GetDlgItem(hWnd, IDC_RECIP_ED  )) > 0 ||
            GetWindowTextLength(GetDlgItem(hWnd, IDC_BODY_ED   )) > 0)
        {
          EnableWindow( GetDlgItem( hWnd,
                                    IDC_FIND_PB),
                        TRUE);
        }
        else
        {
          EnableWindow( GetDlgItem( hWnd,
                                    IDC_FIND_PB),
                        FALSE);
        }
      }
      break;
    break;
    case IDCLOSE:
      if (HIWORD(wParam) == BN_CLICKED)
      {
        EndDialog(hWnd,
                  IDOK);
      }
      break;
    // Find push button
    case IDC_FIND_PB:
      if (HIWORD(wParam) == BN_CLICKED && !f_fFinding)
      {
        const int nStrLen     = 256;
        BOOL      fRecurse    = FALSE;
        char      pszSender[nStrLen];
        char      pszRecip[nStrLen];
        char      pszSubject[nStrLen];
        char      pszBody[nStrLen];

        GetWindowText(GetDlgItem( hWnd,
                                  IDC_SENDER_ED),
                      pszSender,
                      nStrLen);
        GetWindowText(GetDlgItem( hWnd,
                                  IDC_RECIP_ED),
                      pszRecip,
                      nStrLen);
        GetWindowText(GetDlgItem( hWnd,
                                  IDC_SUBJECT_ED),
                      pszSubject,
                      nStrLen);
        GetWindowText(GetDlgItem( hWnd,
                                  IDC_BODY_ED),
                      pszBody,
                      nStrLen);
        fRecurse = SendDlgItemMessage(hWnd, 
                                      IDC_RECURSE_XB,
                                      BM_GETSTATE,
                                      0,
                                      0) & 0x0003;
        if (CtlrFind( GetDlgItem( hWnd,
                                  IDC_FOLDER_TV),
                      GetDlgItem( hWnd,
                                  IDC_MESSAGE_LV),
                      pszSender,
                      pszRecip,
                      pszSubject,
                      pszBody,
                      fRecurse))
        {
          EnableControls( hWnd,
                          FALSE);
          f_fFinding = TRUE;
        }
      }
      else
      {
        // CtlrStopFind();
        f_fFinding = FALSE;
        EnableControls( hWnd,
                        TRUE);
      }
    break;
    // Find Complete
    case IDC_FIND_COMPLETE:
      f_fFinding = FALSE;
      EnableControls( hWnd,
                      TRUE);
      CtlrConstructMsgList( GetDlgItem( hWnd,
                                        IDC_MESSAGE_LV));
      break;
    // Store combo box
    case IDC_STORE_CB:
      if (HIWORD(wParam) == CBN_SELCHANGE)
      {
        unsigned long ulIndex = SendMessage((HWND)lParam,
                                            CB_GETCURSEL,
                                            0,
                                            0);
        if (ulIndex != f_ulIndex)
        {
          HWND hWndTree = GetDlgItem( hWnd,
                                      IDC_FOLDER_TV);
          if (CtlrChangeStore(hWndTree,
                              GetDlgItem( hWnd,
                                          IDC_MESSAGE_LV),
                              ulIndex))
          {
            HTREEITEM hTreeRoot = TreeView_GetRoot(hWndTree);
            TreeView_Expand(hWndTree,
                            hTreeRoot,
                            TVE_EXPAND);
            TreeView_SelectItem(hWndTree,
                                hTreeRoot);
            f_ulIndex = ulIndex;
          }
          else
          {
            SendMessage((HWND)lParam,
                        CB_SETCURSEL,
                        f_ulIndex,
                        0);
          }
        }

      }
      break;
    default:
      break;
  }
  return fRet;
}

///////////////////////////////////////////////////////////////////////////////
//    FindDlgProc()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
// 
BOOL CALLBACK           FindDlgProc               ( HWND      hWnd,
                                                    UINT      uMsgID, 
                                                    WPARAM    wParam,
                                                    LPARAM    lParam)
{
  //Return: FALSE allows default handling
  BOOL fRet = FALSE;  

  switch (uMsgID)
  {
  case WM_INITDIALOG:
    SetClassLong( hWnd,
                  GCL_HICON,
                  (LONG) LoadIcon(GetWindowInstance(hWnd),
                                  MAKEINTRESOURCE(IDI_FIND)));
    EnableWindow( GetDlgItem( hWnd,
                              IDC_FIND_PB),
                  FALSE);
    CenterWindow( hWnd,
                  0);
    if (CtlrInitialize( hWnd,
                        GetDlgItem( hWnd,
                                    IDC_FOLDER_TV),
                        GetDlgItem( hWnd,
                                    IDC_MESSAGE_LV),
                        GetDlgItem( hWnd,
                                    IDC_STORE_CB)) == FALSE)
    {
      EndDialog(hWnd, -1);
    }
    else
    {
      LV_COLUMN lvC;
      lvC.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
      lvC.fmt = LVCFMT_LEFT;
      char* pszHeading[] =
      {
        "Sender",
        "Subject",
        "Date",
      };
      unsigned pColSize[] =
      {
        30,
        50,
        15
      };
    
      RECT rect;
      unsigned uWidth = 0;
      HWND hWndMsgList = GetDlgItem(hWnd,
                                    IDC_MESSAGE_LV);
      GetWindowRect(hWndMsgList,
                    &rect);
      uWidth = rect.right - rect.left;
      // Add the columns.
      for (unsigned i = 0;i < sizeof(pszHeading) / sizeof(char*);i++)
      {
          lvC.cx = pColSize[i] * uWidth / 100;
          lvC.iSubItem = i;
          lvC.pszText = pszHeading[i];
          ListView_InsertColumn(hWndMsgList,
                                i,
                                &lvC);
      }

      f_ulIndex = SendMessage(GetDlgItem( hWnd,
                                          IDC_STORE_CB),
                              CB_GETCURSEL,
                              0,
                              0);
      HWND hWndTree = GetDlgItem( hWnd,
                                  IDC_FOLDER_TV);
      HTREEITEM hTreeRoot = TreeView_GetRoot(hWndTree);
      TreeView_Expand(hWndTree,
                      hTreeRoot,
                      TVE_EXPAND);
      TreeView_SelectItem(hWndTree,
                          hTreeRoot);
      SendDlgItemMessage( hWnd, 
                          IDC_RECURSE_XB,
                          BM_SETSTATE,
                          0x0003,
                          0);
    }
    break;
  // handle notication etc.
  case WM_COMMAND:
    fRet = FindDlgCmdHandler( hWnd,
                              uMsgID,
                              wParam,
                              lParam);
  break;
  // for 3dControls
  case WM_SYSCOLORCHANGE:
    if (f_pfnCtl3dColorChange)
    {
      f_pfnCtl3dColorChange();
      fRet = TRUE;
    }
  break;
  // perform uninitialization
  case WM_DESTROY:
    CtlrUninitialize( GetDlgItem( hWnd,
                                  IDC_FOLDER_TV),
                      GetDlgItem( hWnd,
                                  IDC_MESSAGE_LV));
  break;
  // notifications from child controls
  case WM_NOTIFY:
  if (wParam == IDC_MESSAGE_LV)
  {
    NM_LISTVIEW *lpNM = (NM_LISTVIEW *)lParam;

    if (lpNM->hdr.code == NM_DBLCLK || lpNM->hdr.code == NM_RETURN)
    {
      // double click on a ListView item, display the message
      CtlrDisplayMessage( GetDlgItem( hWnd,
                                      IDC_MESSAGE_LV));
    }
  }
  break;
  default:
    break;
  }
  return fRet;
}

///////////////////////////////////////////////////////////////////////////////
//    WinMain()
//
//    Parameters
//      { Refer to Win32 API documentation }
//
//    Purpose
//      { Refer to Win32 API documentation }
//      
//    Return Value
//      { Refer to Win32 API documentation }
//      
int WINAPI              WinMain                   ( HINSTANCE hInstance,
                                                    HINSTANCE ,
                                                    LPSTR     ,
                                                    int       )
{
  int                       nErrorCode              = 0;
  t_pfnCtl3dRegister        pfnCtl3dRegister        = 0;
  t_pfnCtl3dUnregister      pfnCtl3dUnregister      = 0;
  t_pfnCtl3dAutoSubclass    pfnCtl3dAutoSubclass    = 0;
  t_pfnCtl3dUnAutoSubclass  pfnCtl3dUnAutoSubclass  = 0;

#ifdef WIN32
  HINSTANCE hInstanceCtl3d  = LoadLibrary("ctl3d32.dll");
#else
  HINSTANCE hInstanceCtl3d  = LoadLibrary("ctl3d.dll");
#endif

  if (hInstanceCtl3d)
  {
    pfnCtl3dRegister        = (t_pfnCtl3dRegister)      GetProcAddress( hInstanceCtl3d,
                                                                        "Ctl3dRegister");
    pfnCtl3dUnregister      = (t_pfnCtl3dUnregister)    GetProcAddress( hInstanceCtl3d,
                                                                        "Ctl3dUnregister");
    pfnCtl3dAutoSubclass    = (t_pfnCtl3dAutoSubclass)  GetProcAddress( hInstanceCtl3d,
                                                                        "Ctl3dAutoSubclass");
    pfnCtl3dUnAutoSubclass  = (t_pfnCtl3dUnAutoSubclass)GetProcAddress( hInstanceCtl3d,
                                                                        "Ctl3dUnAutoSubclass");
    if (pfnCtl3dRegister(hInstance))
    {
      if (pfnCtl3dAutoSubclass)
      {
        pfnCtl3dAutoSubclass(hInstance);
        f_pfnCtl3dColorChange = (t_pfnCtl3dColorChange)GetProcAddress(hInstanceCtl3d, 
                                                                      "Ctl3dColorChange");
      }
    }
  }
  InitCommonControls();

  nErrorCode = DialogBox( hInstance,
                          MAKEINTRESOURCE(IDD_FIND),
                          0,
                          FindDlgProc);
  if (nErrorCode == -1)
  {
    MessageBox( 0,
                "Unable to initialize!",
                "Find",
                MB_APPLMODAL | MB_OK | MB_ICONEXCLAMATION);
  }

  if (hInstanceCtl3d)
  {
    if (pfnCtl3dUnAutoSubclass)
    {
      pfnCtl3dUnAutoSubclass();
    }
    if (pfnCtl3dUnregister)
    {
      pfnCtl3dUnregister(hInstance);
    }
    FreeLibrary(hInstanceCtl3d);
  }

  return nErrorCode;
}
