///////////////////////////////////////////////////////////////////////////////
//
//  HelloMAPI.CPP
//  This module contains all the MAPI code for a minimal extended MAPI mail client
//
//  Author :    Les Thaler
//  Copyright (c) 1996 Irving De La Cruz and Les Thaler
//  
#include <windows.h>

#define USES_IID_IMAPITable
#define USES_IID_IMessage
#define USES_IID_IMAPIStatus
#define INITGUID

#include <initguid.h>
#include <mapiguid.h>
#include <mapix.h>
#include <mapiutil.h>
#include <mapitags.h>
#include "hellomapi.h"   

SizedSPropTagArray(NUM_MSG_PROPS,g_sptMsgProps) = {NUM_MSG_PROPS, 
                                                   PR_SUBJECT,
                                                   PR_BODY,
                                                   PR_MESSAGE_FLAGS,
                                                   PR_SENT_REPRESENTING_NAME,
                                                   PR_HASATTACH};

SizedSPropTagArray(NUM_RECIP_PROPS,g_sptRecips) = {NUM_RECIP_PROPS,
                                                   PR_DISPLAY_NAME,
                                                   PR_ENTRYID,
                                                   PR_ADDRTYPE,
                                                   PR_RECIPIENT_TYPE,
                                                   PR_EMAIL_ADDRESS,
                                                   PR_SEARCH_KEY};

SizedSPropTagArray(NUM_RECIP_PROPS,g_sptSender) = {NUM_RECIP_PROPS,
                                                   PR_SENT_REPRESENTING_NAME,         
                                                   PR_SENT_REPRESENTING_ENTRYID,      
                                                   PR_SENT_REPRESENTING_ADDRTYPE,     
                                                   PR_SENT_REPRESENTING_RECIP_TYPE,   
                                                   PR_SENT_REPRESENTING_EMAIL_ADDRESS,
                                                   PR_SENT_REPRESENTING_SEARCH_KEY};  

//////////////////////////////////////////////////////////////////////////////
//
//  Main Window procedure.
//  
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    static  PCSession pCSess;
            HINSTANCE hInst   = (HINSTANCE)GetWindowLong(hWnd,GWL_HINSTANCE);
    	    WORD      wmId    = LOWORD(wParam), 
	                  wmEvent = HIWORD(wParam);
 
    if (!hInst)
        return 0;

	switch (message) 
	{ 
        case WM_CREATE:
            OnError(hWnd,InitSession(hWnd,hInst,&pCSess));
            break;

		case WM_COMMAND:
			switch (wmId) 
			{
                case IDM_INBOX:
                    OnError(hWnd, pCSess -> ShowDialog(hWnd,
                                                       CTX_INBOX, 
                                                       pCSess -> m_psbFolderEID[IPM_ROOT].cb,
                                                       (LPENTRYID)pCSess -> m_psbFolderEID[IPM_ROOT].lpb,
                                                       "Inbox"));
                    break;
                case IDM_SEND:
                    OnError(hWnd, pCSess -> ShowDialog(hWnd, CTX_COMPOSE, 0, NULL, "New Message"));
                    break;
			    case IDM_ABOUT:
					DialogBox(hInst, "AboutBox", hWnd, (DLGPROC)About);
					break;
                case ID_ERROR_EXIT:
				case IDM_EXIT:
                    if (pCSess)
                        delete pCSess;
                    
					DestroyWindow(hWnd);
					break;
            	default:
					return (DefWindowProc(hWnd, message, wParam, lParam));
			}
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
        case WM_SYSCOMMAND:
            if (SC_CLOSE == wParam)
                if (pCSess)
                    delete pCSess;
            // fall through
		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
	}
	return (0);
}

//////////////////////////////////////////////////////////////////////////////
//
//  Initializes MAPI,logs on, creates and initializes the CSession object.
//  
STDMETHODIMP InitSession(HWND hWnd,HINSTANCE hInst,PCSession * ppCSess)
{
    LPMAPISESSION pSession = NULL;
    PCSession     pCSess   = NULL;
    HRESULT       hRes;
        
    if (FAILED(hRes = MAPIInitialize(NULL)))
        return hRes;

    if (FAILED(hRes = MAPILogonEx((ULONG)hWnd,
                                  NULL,
                                  NULL,
                                  LOGON_FLAGS,
                                  &pSession)))
        goto Quit;
    
    pCSess = new CSession(pSession,hInst);

    if (!pCSess)
    {
        hRes = MAPI_E_NOT_ENOUGH_MEMORY;
        goto Quit;
    }

    // initialize CSession object    
    if (FAILED(hRes = pCSess -> Init()))
        goto Quit;

    // open default msg store
    if (FAILED(hRes = pCSess -> OpenDefStore(hWnd)))
        goto Quit;

    // open top-level address book
    if (FAILED(hRes = pSession -> OpenAddressBook((ULONG)hWnd,
			                                      NULL,
							                      0,
									              &pCSess -> m_pAddrBook)))
    {
        ShowLastError(hRes,hWnd,pSession);
        goto Quit;
    }

    // get IPM folder entry IDs
    if (FAILED(hRes = pCSess -> OpenIPMSubtree(hWnd)))
        goto Quit;
   
    hRes = pCSess -> GetSpoolerStatus();
Quit:
    if (FAILED(hRes))
    {
        // destructor releases session and uninitializes
        if (pCSess)
            delete pCSess;
        else
        {
            if (pSession)
                pSession -> Release();        

            MAPIUninitialize();
        }
        return hRes;
    }

    *ppCSess = pCSess;
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Create an array of entry IDs for the IPM subtree folders
//  
STDMETHODIMP CSession::Init()
{
    HRESULT hRes;
    
    if (FAILED(hRes = MAPIAllocateBuffer(NUM_FOLDERS * sizeof SBinary, 
                                            (LPVOID *) & m_psbFolderEID)))
        goto Quit;

    ZeroMemory((LPVOID) m_psbFolderEID, NUM_FOLDERS * sizeof SBinary);
Quit:        
    return hRes;    
}

///////////////////////////////////////////////////////////////////////////////
//
//  Open the default store for this session
//  
STDMETHODIMP CSession::OpenDefStore(HWND hWnd)
{
    LPMAPITABLE         pStoresTbl = NULL;
    LPSRowSet           pRow       = NULL;
    SBinary             sbEID      = {0,NULL};
    static SRestriction sres;
    SPropValue          spv;
    HRESULT             hRes;

    static SizedSPropTagArray(2,sptCols) = {2,PR_ENTRYID,PR_DEFAULT_STORE};
    
    if (FAILED(hRes = m_pSess -> GetMsgStoresTable(0, &pStoresTbl)))
    {
        ShowLastError(hRes,hWnd,m_pSess);
        goto Quit;
    }

    sres.rt = RES_PROPERTY;
    sres.res.resProperty.relop = RELOP_EQ;
    sres.res.resProperty.ulPropTag = PR_DEFAULT_STORE;
    sres.res.resProperty.lpProp = &spv;

    spv.ulPropTag = PR_DEFAULT_STORE;
    spv.Value.b   = TRUE;

    if (FAILED(hRes = HrQueryAllRows(pStoresTbl,
                                     (LPSPropTagArray) &sptCols,
                                     &sres,
                                     NULL,
                                     0,
                                     &pRow)))
        goto Quit;

    if (pRow                    &&
        pRow -> cRows           && 
        pRow -> aRow[0].cValues &&
        PR_ENTRYID == pRow -> aRow[0].lpProps[0].ulPropTag)
    {
        sbEID  = pRow -> aRow[0].lpProps[0].Value.bin;
    }
    else
    {
        hRes = MAPI_E_NOT_FOUND;
        goto Quit;
    }        

    if (FAILED(hRes = m_pSess -> OpenMsgStore((ULONG)hWnd,
                                              sbEID.cb,
                                              (LPENTRYID)sbEID.lpb,
                                              NULL,
                                              MDB_WRITE,
                                              &m_pMDB)))
    {
        ShowLastError(hRes,hWnd,m_pSess);
        goto Quit;
    }
    
Quit:
    FreeProws(pRow);
    
    if (pStoresTbl)
        pStoresTbl -> Release();
                
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  
//  Open each folder and save a pointer to open interfaces,
//
STDMETHODIMP CSession::OpenIPMSubtree(HWND hWnd)
{
    LPSPropValue  pProps = NULL;
    HRESULT       hRes;
    ULONG         cProps,
                  ulObjType;
    SBinary       sbInboxEID;    
    int           i;
    static SizedSPropTagArray(NUM_FOLDERS,sptFldrs) = {NUM_FOLDERS,PR_NULL,
                                                                   PR_IPM_OUTBOX_ENTRYID,
                                                                   PR_IPM_WASTEBASKET_ENTRYID,
                                                                   PR_IPM_SENTMAIL_ENTRYID,
                                                                   PR_IPM_SUBTREE_ENTRYID};

    if (FAILED(hRes = m_pMDB -> GetProps((LPSPropTagArray)&sptFldrs,
                                          0,
                                          &cProps,
                                          &pProps)))
        goto Quit;

    // get Inbox folder's entry ID
    if (FAILED(hRes = m_pMDB -> GetReceiveFolder("IPM.Note",
                                                 0,
                                                 &sbInboxEID.cb,
                                                 (LPENTRYID *) &sbInboxEID.lpb,
                                                 NULL)))
    {
        ShowLastError(hRes, hWnd, m_pMDB);
        goto Quit;
    }

    pProps[INBOX].Value.bin = sbInboxEID;

    for (i = INBOX; i < NUM_FOLDERS; i++)
    {
        if (pProps[i].ulPropTag != sptFldrs.aulPropTag[i])
        {
            hRes = MAPI_E_NOT_FOUND;
            goto Quit;
        }

        // copy the folder's entry ID to class member
        if (FAILED(hRes = CopySBinary(& m_psbFolderEID[i], 
                                      & pProps[i].Value.bin,
                                      (LPVOID) m_psbFolderEID)))
                goto Quit;
    }

    for (i = INBOX; i <= WASTE; i++)
    {
        if (FAILED(hRes = m_pSess -> OpenEntry(m_psbFolderEID[i].cb,
                                               (LPENTRYID) m_psbFolderEID[i].lpb,
                                               NULL,
                                               MAPI_MODIFY,
                                               &ulObjType,
                                               (LPUNKNOWN *) &m_pFolders[i])))
        {
            ShowLastError(hRes,hWnd,m_pSess);
            goto Quit;
        }

        if (MAPI_FOLDER != ulObjType)
        {
            hRes = MAPI_E_INVALID_OBJECT;
            goto Quit;
        }
    }                         
    
Quit:
    MAPIFreeBuffer((LPVOID) pProps);
    MAPIFreeBuffer(sbInboxEID.lpb);
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Gets the spooler's status object from the session status table
//  
STDMETHODIMP CSession :: GetSpoolerStatus()
{
    LPMAPITABLE     pTbl  = NULL;
    LPMAPISTATUS    pStat = NULL;
    LPSRowSet       pRow  = NULL;
    HRESULT         hRes;
    SRestriction    sres;
    SPropValue      spv;
    ULONG           ulObjType;

    const static SizedSPropTagArray(2,sptCols) = {2,PR_RESOURCE_TYPE,PR_ENTRYID};
    
    if (FAILED(hRes = m_pSess -> GetStatusTable(0,&pTbl)))
        goto Quit;

    sres.rt = RES_PROPERTY;
    sres.res.resProperty.relop     = RELOP_EQ;
    sres.res.resProperty.ulPropTag = PR_RESOURCE_TYPE;
    sres.res.resProperty.lpProp    = &spv;

    spv.ulPropTag = PR_RESOURCE_TYPE;
    spv.Value.l   = MAPI_SPOOLER;

    if (FAILED(hRes = HrQueryAllRows(pTbl,
                                     (LPSPropTagArray) &sptCols,
                                     &sres,
                                     NULL,
                                     0,
                                     &pRow)))
        goto Quit;

    if (!pRow -> cRows || PR_ENTRYID != pRow -> aRow[0].lpProps[1].ulPropTag)
    {
        hRes = MAPI_E_NOT_FOUND;
        goto Quit;
    }

    hRes = m_pSess -> OpenEntry(pRow -> aRow[0].lpProps[1].Value.bin.cb,
                                (LPENTRYID)pRow -> aRow[0].lpProps[1].Value.bin.lpb,
                                &IID_IMAPIStatus,
                                MAPI_BEST_ACCESS,
                                &ulObjType,
                                (LPUNKNOWN *)&pStat);

    if (FAILED(hRes) || MAPI_STATUS != ulObjType)
    {
        hRes = hRes ? hRes : MAPI_E_INVALID_OBJECT;
        goto Quit;
    }

    m_pSpoolerStat = pStat;
Quit:
    if (pTbl)
        pTbl -> Release();

    FreeProws(pRow);
    return hRes;    
}

///////////////////////////////////////////////////////////////////////////////
//
//  Brings up the correct dialog for the context
//  
STDMETHODIMP CSession :: ShowDialog(HWND      hWnd,
                                    ULONG     ulCtx, 
                                    ULONG     cbEID, 
                                    LPENTRYID pEID,
                                    LPSTR     szTitle,
                                    LPMESSAGE pMsg)
{
    PContext pCtx = new CContext(ulCtx, cbEID, pEID, this, szTitle,pMsg);

    if (!pCtx)
        return MAPI_E_NOT_ENOUGH_MEMORY;
    
    DialogBoxParam(m_hInst,
                   MAKEINTRESOURCE(CTX_INBOX == ulCtx ? IDD_INBOX
                                                      : CTX_READNOTE == ulCtx ? IDD_READNOTE 
                                                                              : IDD_COMPOSE),
                   hWnd,
                   (DLGPROC) (CTX_INBOX == ulCtx ? ViewInbox : ComposeNote),
                   (LPARAM) pCtx);
    delete pCtx;
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Inbox viewer's dialog proc.
//  
LRESULT CALLBACK ViewInbox(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    static PContext    pCtx;
    static PCSession   pCSess;
           HRESULT     hRes = S_OK;

	switch (message) 
	{
        case WM_INITDIALOG:
            InitDialog(hDlg,lParam,pCtx);
            pCSess = pCtx -> m_pSess; 
            hRes = LoadListBox(hDlg,lParam,IDC_CTBL_LIST);  
            OnError(hDlg,hRes);
        	ShowWindow (hDlg, SW_HIDE);
			return (TRUE);

        case WM_COMMAND:
            WORD    wNotif = HIWORD(wParam),
                    wCtlID = LOWORD(wParam);   	
            
            switch(wCtlID)
            {
                case IDC_FLUSH:
                    OnError(hDlg,pCSess -> FlushQs(hDlg));
                    break;
                case IDC_DELETE:
                    OnError(hDlg,pCSess -> DeleteMsg(hDlg));
                    return TRUE;
                case IDC_CTBL_LIST:
                    switch(wNotif)
                    {
                        case LBN_SELCHANGE:         // an item is hilighted
                            EnableWindow(GetDlgItem(hDlg,IDC_DELETE),TRUE);
                            return TRUE;
                            
                        case LBN_DBLCLK:            // open selected msg
                        {
                            PSMsgID  pMsgID = NULL;
                            ULONG    ulCtx;
                            
                            if (LB_ERR == GetLBSelection(hDlg,
                                                         wCtlID,
                                                         (LPVOID *) &pMsgID))
                                OnError(hDlg,MAPI_E_CALL_FAILED);

                            ulCtx = ((!pMsgID -> ulFlags) || 
                                     (pMsgID -> ulFlags & MSGFLAG_UNSENT) ? CTX_SAVED 
                                                                          : CTX_READNOTE);
                            OnError(hDlg, pCSess -> ShowDialog(hDlg,
                                                               ulCtx, 
                                                               pMsgID -> sbEID.cb,
                                                               (LPENTRYID)pMsgID -> sbEID.lpb));
                            return TRUE;
                        }
                        default:
                            return FALSE;
                    }
                    break;
                
                case IDOK:
                case IDCANCEL:
                case ID_ERROR_EXIT:
                    EmptyListBox(hDlg,IDC_CTBL_LIST);
	    			EndDialog(hDlg, TRUE);
    				return (TRUE);
		        default:
                    return FALSE;
			}
			break;
	}
    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Loads the ViewInbox listbox from the Inbox contents table
//  
STDMETHODIMP LoadListBox(HWND hDlgParent,LPARAM lParam,ULONG ulCtlID)
{
    LPMAPITABLE pCTbl = NULL;
    LPSRowSet   pRows = NULL;
    TCHAR       szSndr[MAX_SNDR],
                szSubj[MAX_SUBJ],
                szBuf[MAX_SUBJ + MAX_SNDR + 4];
    HRESULT     hRes;
    PCSession   pCSess;
    PContext    pCtx;
    ULONG       i;
    enum {NAME,SUBJ,EID,FLAGS,NUM_COLS};
    static SizedSPropTagArray(NUM_COLS,sptCols) = {NUM_COLS,PR_SENT_REPRESENTING_NAME,
                                                            PR_SUBJECT,
                                                            PR_ENTRYID,
                                                            PR_MESSAGE_FLAGS};
    pCtx = (PContext) lParam;    
    pCSess = pCtx -> m_pSess;

    if (FAILED(hRes = pCSess -> m_pFolders[INBOX] -> GetContentsTable(0,&pCTbl)))
        goto Quit;

    // YOU should use QueryRows,SeekRowApprox since ContentsTable can be huge!
    if (FAILED(hRes = HrQueryAllRows(pCTbl,
                                     (LPSPropTagArray) &sptCols,
                                     NULL,
                                     NULL,
                                     0,
                                     &pRows)))
        goto Quit;
    
    for (i = 0; i < pRows -> cRows; i++)
    {
        HRESULT hr;
        ULONG   ulIdx;
        PSMsgID pMsgID = NULL;

        if (PR_ENTRYID == pRows -> aRow[i].lpProps[EID].ulPropTag)
        {
            LPSBinary pMsgEID = NULL;

            if (SUCCEEDED(hr = MAPIAllocateBuffer(sizeof SMsgID,
                                                  (LPVOID *) &pMsgID)))
            {
                if (SUCCEEDED(hr = CopySBinary(&pMsgID -> sbEID,
                                               &pRows -> aRow[i].lpProps[EID].Value.bin,
                                               pMsgID)))
                {
                    if (PR_MESSAGE_FLAGS == pRows -> aRow[i].lpProps[FLAGS].ulPropTag)
                        pMsgID -> ulFlags = pRows -> aRow[i].lpProps[FLAGS].Value.l;
                    else
                        pMsgID -> ulFlags = 0;
                                                                                                                
                    if (PR_SENT_REPRESENTING_NAME == pRows -> aRow[i].lpProps[NAME].ulPropTag)
                        lstrcpyn(szSndr, 
                                pRows -> aRow[i].lpProps[NAME].Value.lpszA,
                                MAX_SNDR);
                    else
                        lstrcpy(szSndr,"<no sender>");

                    if (PR_SUBJECT == pRows -> aRow[i].lpProps[SUBJ].ulPropTag)
                        lstrcpyn(szSubj, 
                                 pRows -> aRow[i].lpProps[SUBJ].Value.lpszA,
                                 MAX_SUBJ);
                    else
                        lstrcpy(szSubj,"<no subject>");
                    
                    // lstrcpyn doesn't terminate n length strings
                    szSubj[MAX_SUBJ - 1] = szSndr[MAX_SNDR - 1] = '\0';
                    wsprintf(szBuf,SZ_FRMT,szSndr,szSubj);
                    ulIdx = SendDlgItemMessage(hDlgParent,
                                               ulCtlID,
                                               LB_ADDSTRING,
                                               0,
                                               (LPARAM)szBuf);
                    if (LB_ERR != ulIdx)
                        ulIdx = SendDlgItemMessage(hDlgParent,
                                                   ulCtlID,
                                                   LB_SETITEMDATA,
                                                   ulIdx,
                                                   (LPARAM)pMsgID);
                }   
                                         
                if (S_OK != hr || LB_ERR == ulIdx)
                {
                    MAPIFreeBuffer((LPVOID)pMsgEID);
                    hRes = MAPI_W_ERRORS_RETURNED;
                }
            }
        } 
    }

Quit:
    FreeProws(pRows);

    if (pCTbl)
        pCTbl -> Release();
    return hRes;
}        

///////////////////////////////////////////////////////////////////////////////
//
//  Forces spooler to flush inbound and outbound queues (ALL transports)
//  
STDMETHODIMP CSession :: FlushQs(HWND hWnd)
{
    return m_pSpoolerStat -> FlushQueues((ULONG) hWnd,0,NULL,FLUSH_FLAGS);
}

///////////////////////////////////////////////////////////////////////////////
//
//  Gets the current selection's itemdata from the listbox. extracts the
//  msg's entry ID and uses it to move msg to deleted items folder.
//  
STDMETHODIMP CSession :: DeleteMsg(HWND hDlg)
{
    HRESULT      hRes   = S_OK;
    PSMsgID      pMsgID = NULL;
    LONG         lIdx   = GetLBSelection(hDlg,IDC_CTBL_LIST,(LPVOID*)&pMsgID),
                 lCnt;
    SBinaryArray sbaEID;

    if (LB_ERR == lIdx)
        return MAPI_E_CALL_FAILED;

    sbaEID.cValues = 1;
    sbaEID.lpbin = &(pMsgID -> sbEID);

    // move the selected msg to the delete folder
    hRes = m_pFolders[INBOX] -> CopyMessages(&sbaEID,
                                             NULL,
                                             m_pFolders[WASTE],
                                             (ULONG)hDlg,
                                             NULL,
                                             MESSAGE_MOVE | MESSAGE_DIALOG);
    if (FAILED(hRes))
        goto Quit;

    MAPIFreeBuffer((LPVOID) pMsgID);

   lCnt = SendDlgItemMessage(hDlg,
                             IDC_CTBL_LIST,
                             LB_DELETESTRING,
                             lIdx,
                             0);
    if (LB_ERR == lCnt)
    {
        hRes = MAPI_E_CALL_FAILED;        
        goto Quit;
    }

    if (lCnt)               // if more,move hilite to next entry
        SendDlgItemMessage(hDlg,
                           IDC_CTBL_LIST,
                           LB_SETCURSEL,
                           (lIdx >= lCnt ? lCnt - 1: lIdx),
                           0);
    else                    // no more, disable delete button
        EnableWindow(GetDlgItem(hDlg,IDC_DELETE),FALSE);
Quit:
    return hRes;                                                         
}

///////////////////////////////////////////////////////////////////////////////
//
//  ComposeNote and ReadNote dialog proc.
//  
LRESULT CALLBACK ComposeNote(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	static  LPMESSAGE pNewMsg;
	static  LPMESSAGE pOrigMsg;
    static  PContext  pCtx;  
    static  PCSession pCSess;
            HRESULT   hRes    = S_OK;

	switch (message) 
	{
        case WM_INITDIALOG:
            InitDialog(hDlg,lParam,pCtx);
            pCSess = pCtx -> m_pSess;   
            pOrigMsg = pCtx -> m_pMsg;  // NULL except in CTX_REPLY_XX/FORWARD
            pNewMsg = NULL;
            
            switch(pCtx -> m_lContext)
            {
                case CTX_COMPOSE:       // new msg, from 'New Message' menu item
                    OnError(hDlg,pCSess ->m_pFolders[INBOX]->CreateMessage(NULL,
                                                                           0,
                                                                           &pNewMsg));
                    EnableWindow(GetDlgItem(hDlg,IDC_SEND),FALSE);
                    break;

                case CTX_SAVED:         // opening existing msg in the inbox
                case CTX_READNOTE:
                    OnError(hDlg,pCSess -> PopulateReadNote(hDlg,pCtx,&pNewMsg)); 
                    break;

                case CTX_REPLY:         // we already have an inbox msg open,
                case CTX_REPLY_ALL:     // we want to forward it or reply               
                case CTX_FORWARD:       
                    OnError(hDlg,pCSess -> CreateReplyNote(hDlg,pCtx,&pNewMsg));
                    break;
            }
     	    ShowWindow (hDlg, SW_HIDE);
			return (TRUE);

		case WM_COMMAND:
            WORD    wNotif = HIWORD(wParam),
                    wCtlID = LOWORD(wParam);   	
            
            switch(wCtlID)
            {
                case IDC_TO:            // only enabled for new msg or forward
                    // Browse address book
                    hRes = pCtx -> m_pSess -> AddRecips(hDlg,
                                                        pCtx -> m_szTitle,
                                                        pCSess,
                                                        pNewMsg);
                    if (MAPI_E_USER_CANCEL == hRes)
                        return FALSE;

                    OnError(hDlg,hRes);
                    EnableWindow(GetDlgItem(hDlg,IDC_SEND),TRUE);
                    break;
                case IDC_FROM:
                    // view sender details
                    OnError(hDlg,pCSess -> SenderDetails(&hDlg,pNewMsg));
                    break;
                case IDC_SAVE:
                    OnError(hDlg,pCSess -> Save(hDlg,pNewMsg));
                    break;
                case IDC_ATTACH_FILE:
                    OnError(hDlg,pCSess -> AttachFile(hDlg,pNewMsg));
                    break;
                case IDC_SEND:
                    // submit message
                    OnError(hDlg,pCSess -> SendNote(hDlg,pNewMsg));
                    
                    // SendNote releases pNewMsg
                    pNewMsg = pOrigMsg; // restore msg we "replied to"
	    			EndDialog(hDlg, TRUE);
    				return (TRUE);
                case IDC_REPLY:         // reply to/forward a msg in our inbox
                case IDC_REPLY_ALL:
                case IDC_FORWARD:
                {
                    ULONG ulCtx = (IDC_REPLY     == wCtlID ? CTX_REPLY : 
                                   IDC_REPLY_ALL == wCtlID ? CTX_REPLY_ALL :
                                   IDC_FORWARD   == wCtlID ? CTX_FORWARD :
                                   (hRes = MAPI_E_CALL_FAILED));

                    OnError(hDlg, hRes);

                    hRes = pCSess -> ShowDialog(hDlg,
                                                ulCtx, 
                                                pCtx -> m_cbEID,
                                                pCtx -> m_pParentEID,
                                                "",
                                                pNewMsg);

                    if (pNewMsg)
                        pNewMsg -> Release();
                    
                    pNewMsg = pOrigMsg;     // restore any replied-to msg
                    OnError(hDlg, hRes);
                    EndDialog(hDlg,TRUE);
                    return TRUE;
                }

                case IDC_BODY:              // move caret to beginning
                    if (EN_SETFOCUS == wNotif)
                        SendMessage((HWND) lParam,EM_SETSEL,0,0);
                    break;
                case IDOK:
                case IDCANCEL:
                case ID_ERROR_EXIT:
                    // release msg object
                    if (pNewMsg)
                       pNewMsg -> Release();
                    
                    pNewMsg = pOrigMsg;     // restore any replied-to msg
	    			EndDialog(hDlg, TRUE);
    				return (TRUE);
		        default:
                    return FALSE;
			}
			break;
	}
    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Populates read note dialog box controls from message properties.
//  
STDMETHODIMP CSession :: PopulateReadNote(HWND hDlg, PContext pCtx,LPMESSAGE * ppMsg)
{
    HRESULT         hRes     = S_OK;
    LPMESSAGE       pMsg     = NULL; 
    LPMAPITABLE     pTbl     = NULL;
    LPSRowSet       pRows    = NULL;
    LPSTR           szToLine = NULL;
    ULONG           ulObjType;

    *ppMsg = NULL;

    if (FAILED(hRes = m_pMDB -> OpenEntry(pCtx -> m_cbEID,
                                          pCtx -> m_pParentEID,
                                          NULL,
                                          MAPI_BEST_ACCESS,
                                          &ulObjType,
                                          (LPUNKNOWN *) &pMsg)))
        return hRes;

    if (MAPI_MESSAGE != ulObjType) 
    {
        hRes = MAPI_E_INVALID_OBJECT;
        goto Quit;
    }

    if (FAILED(hRes = PopulateNote(hDlg,pMsg)))
        goto Quit;

    if (FAILED(hRes = pMsg -> GetRecipientTable(0, &pTbl)))
        return hRes;

    if (FAILED(hRes = HrQueryAllRows(pTbl,
                                     (LPSPropTagArray) &g_sptRecips,
                                     NULL,
                                     NULL,
                                     0,
                                     &pRows)))
        goto Quit;
    
    if (pRows -> cRows)
    {
        szToLine = ConcatRecips((LPADRLIST)pRows);
        SetDlgItemText(hDlg,IDC_RECIP,szToLine);
        EnableWindow(GetDlgItem(hDlg,IDC_SEND),TRUE);
    }
    else
        EnableWindow(GetDlgItem(hDlg,IDC_SEND),FALSE);

Quit:
    if (FAILED(hRes))
        if (pMsg)
            pMsg -> Release();

    if (pTbl)
        pTbl -> Release();

    MAPIFreeBuffer((LPVOID) szToLine);
    FreeProws(pRows);
    *ppMsg = pMsg;

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Populates dialog controls with properties from the message.
//  
STDMETHODIMP PopulateNote(HWND hDlg,LPMESSAGE pMsg)
{
    LPSPropValue    pProps = NULL;
    HRESULT         hRes;
    ULONG           cVals,
                    i;

    if (FAILED(hRes = pMsg -> GetProps((LPSPropTagArray) &g_sptMsgProps,
                                       0,
                                       &cVals,
                                       &pProps)))
        goto Quit;
    else
        hRes = S_OK;

    if (PR_SUBJECT == pProps[SUBJ].ulPropTag)
    {
        SetDlgItemText(hDlg,IDC_SUBJECT,pProps[SUBJ].Value.LPSZ);
        SetWindowText(hDlg,pProps[SUBJ].Value.LPSZ);
    }
    
    if (PR_BODY == pProps[BODY].ulPropTag)
        SetDlgItemText(hDlg,IDC_BODY,pProps[BODY].Value.LPSZ);

    if (PR_SENT_REPRESENTING_NAME == pProps[SNDR].ulPropTag)
        SetDlgItemText(hDlg,IDC_SENDER,pProps[SNDR].Value.LPSZ);

    if (PR_HASATTACH == pProps[ATT].ulPropTag && pProps[ATT].Value.b)
    {
        LPMAPITABLE pAttTbl = NULL;
        LPSRowSet   pRows   = NULL;
        static SizedSPropTagArray(2,sptCols) = {2,PR_ATTACH_FILENAME,
                                                  PR_ATTACH_NUM};

        if (SUCCEEDED(hRes = pMsg -> OpenProperty(PR_MESSAGE_ATTACHMENTS,
                                                  &IID_IMAPITable,
                                                  0,
                                                  0,
                                                  (LPUNKNOWN *) &pAttTbl)))

            if (SUCCEEDED(hRes = pAttTbl -> SetColumns((LPSPropTagArray) &sptCols,
                                                       TBL_BATCH)))

                if (SUCCEEDED(hRes = HrQueryAllRows(pAttTbl,
                                                   (LPSPropTagArray) &sptCols,
                                                   NULL,
                                                   NULL,
                                                   0,
                                                   &pRows)))
    
                    for (i = 0; i < pRows -> cRows; i++)
                    {
                        ULONG ulIdx;

                        if (PR_ATTACH_FILENAME == pRows -> aRow[i].lpProps[0].ulPropTag)
                           ulIdx =  SendDlgItemMessage(hDlg,
                                                       IDC_ATTACHMENTS,
                                                       LB_ADDSTRING,
                                                       0,
                                                       (LPARAM)pRows -> aRow[i].lpProps[0].Value.LPSZ);

                        if (PR_ATTACH_NUM == pRows -> aRow[i].lpProps[1].ulPropTag)
                            SendDlgItemMessage(hDlg,
                                               IDC_ATTACHMENTS,
                                               LB_SETITEMDATA,
                                               ulIdx,
                                               (LPARAM)pRows -> aRow[i].lpProps[1].Value.l);
                    }

        FreeProws(pRows); 

        if (pAttTbl)
            pAttTbl -> Release();                   
    }
Quit:
    MAPIFreeBuffer((LPVOID) pProps);
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Calls IAddrBook::Address to add recipients to msg and populate TO control
//  
//
STDMETHODIMP CSession :: AddRecips(HWND      hDlg,
                                   LPSTR     szTitle,
                                   PCSession pSess,
                                   LPMESSAGE pMsg)
{
    LPADRLIST   pAdrList = NULL;
    LPSTR       szRecips = NULL;
    HRESULT     hRes;
    ADRPARM     AdrParm;

    // Only let user select recipients from address book
    ZeroMemory((LPVOID) & AdrParm, sizeof ADRPARM);

    // set up the adrparm struct for Address
	AdrParm.ulFlags	    	   = DIALOG_MODAL | AB_RESOLVE;
   	AdrParm.cDestFields		   = 1;
	AdrParm.nDestFieldFocus	   = 0;
	AdrParm.lpszNewEntryTitle  = szTitle;
    AdrParm.lpszDestWellsTitle = "";

	AdrParm.lpszCaption = "Browse Address Book";

    if (FAILED(hRes = m_pAddrBook -> Address((ULONG *) &hDlg,
                                             &AdrParm,
                                             &pAdrList)))
        goto Quit;

    // e.g. user cancelled
    if (!pAdrList || !pAdrList -> cEntries)
    {
        hRes = MAPI_E_USER_CANCEL;
        goto Quit;
    }

    // passing 0 replaces the entire table w/ pAdrList
    if (FAILED(hRes = pMsg -> ModifyRecipients(0, pAdrList)))
        goto Quit;

    szRecips = ConcatRecips(pAdrList);

    if (!szRecips)
    {
        hRes = MAPI_E_CALL_FAILED;
        goto Quit;
    }        
    
    SetDlgItemText(hDlg,IDC_RECIP,szRecips);
    MAPIFreeBuffer((LPVOID) szRecips);

Quit:    
    FreePadrlist(pAdrList);
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Displays the sender's details page, e.g. when the From button is clicked.
//  
STDMETHODIMP CSession :: SenderDetails(HWND * phDlg,LPMESSAGE pMsg)
{
    LPSPropValue    pspv = NULL;
    HRESULT         hRes;

    if (FAILED(hRes = HrGetOneProp(pMsg,PR_SENT_REPRESENTING_ENTRYID,&pspv)))
        return hRes;

    hRes = m_pAddrBook -> Details((ULONG *) phDlg,
                                  NULL,
                                  NULL,
                                  pspv -> Value.bin.cb,
                                  (LPENTRYID)pspv -> Value.bin.lpb,
                                  NULL,
                                  NULL,
                                  NULL,
                                  DIALOG_MODAL);
    MAPIFreeBuffer((LPVOID) pspv);
    return MAPI_E_USER_CANCEL == hRes ? S_OK : hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Commits changes on the message object,e.g. when user clicks 'Save'
//  
STDMETHODIMP CSession :: Save(HWND hDlg, LPMESSAGE pMsg)
{
    LPSPropValue  pspvSaved = NULL,
                  pspvEID   = NULL;
    HRESULT       hRes;

    if (FAILED(hRes = MAPIAllocateBuffer(sizeof SPropValue * 3,
                                         (LPVOID *) &pspvSaved)))
        return hRes;

    pspvSaved[SUBJ].ulPropTag = PR_SUBJECT;
    pspvSaved[BODY].ulPropTag = PR_BODY;
    pspvSaved[FLAG].ulPropTag = PR_MESSAGE_FLAGS;

    if (FAILED(hRes = GetUIData(hDlg,pspvSaved)))
        goto Quit;

    pspvSaved[FLAG].Value.l = MSGFLAG_UNSENT | MSGFLAG_READ;

    if (FAILED(hRes = pMsg -> SetProps(3,pspvSaved,NULL)))
        goto Quit;

    pMsg -> SaveChanges(KEEP_OPEN_READWRITE);

Quit:
    MAPIFreeBuffer((LPVOID)pspvSaved);        
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Display dialog for choosing a file, then attach it to message.
//  
STDMETHODIMP CSession :: AttachFile(HWND hWnd,LPMESSAGE pMsg)
{
    LPSTREAM      pStrmSrc  = NULL,
                  pStrmDest = NULL;
    LPATTACH      pAtt      = NULL;
    ULONG         ulAttNum,
                  ulIdx;
    HRESULT       hRes;
    OPENFILENAME  ofn;
    TCHAR         szFile[MAX_PATH];
    STATSTG       StatInfo;

    enum {FILENAME,METHOD,RENDERING,NUM_ATT_PROPS};
    SPropValue    spvAttach[NUM_ATT_PROPS];

    ZeroMemory((LPVOID) &ofn, sizeof ofn);

    lstrcpy(szFile,"*.*");
    ofn.lStructSize = sizeof(ofn);
    ofn.hwndOwner   = hWnd;
    ofn.lpstrFilter = "All files\0*.*\0";
    ofn.lpstrFile   = szFile;
    ofn.nMaxFile    = MAX_PATH;
    ofn.lpstrTitle  = "Attach File";
    ofn.Flags       = OFN_NONETWORKBUTTON | OFN_FILEMUSTEXIST |
					  OFN_NOCHANGEDIR	  | OFN_PATHMUSTEXIST;
        
    if (GetOpenFileName(&ofn))
    {
        if (FAILED(hRes = OpenStreamOnFile(MAPIAllocateBuffer,
                                           MAPIFreeBuffer,
                                           STGM_READ,
                                           ofn.lpstrFile,
                                           NULL,
                                           &pStrmSrc)))
            goto Quit;

        if (FAILED(hRes = pMsg -> CreateAttach(NULL,0, &ulAttNum,&pAtt)))
            goto Quit;

        if (FAILED(hRes = pAtt -> OpenProperty(PR_ATTACH_DATA_BIN,
                                               (LPIID)&IID_IStream,
                                               0,
                                               MAPI_MODIFY | MAPI_CREATE,
                                               (LPUNKNOWN *)&pStrmDest)))
            goto Quit;

        pStrmSrc -> Stat(&StatInfo, STATFLAG_NONAME);

        if (FAILED(hRes = pStrmSrc -> CopyTo(pStrmDest, 
                                             StatInfo.cbSize,
                                             NULL, 
                                             NULL)))
            goto Quit;
    
        spvAttach[FILENAME].ulPropTag = PR_ATTACH_FILENAME;
        spvAttach[FILENAME].Value.lpszA = &ofn.lpstrFile[ofn.nFileOffset];

        spvAttach[METHOD].ulPropTag = PR_ATTACH_METHOD;
        spvAttach[METHOD].Value.l = ATTACH_BY_VALUE;
  
        spvAttach[RENDERING].ulPropTag = PR_RENDERING_POSITION;
        spvAttach[RENDERING].Value.l = -1;      

        if (FAILED(hRes = pAtt -> SetProps(NUM_ATT_PROPS,
                                           (LPSPropValue)&spvAttach,
                                           NULL)))
            goto Quit;
        
        pAtt -> SaveChanges(0);
        ulIdx = SendDlgItemMessage(hWnd,
                                   IDC_ATTACHMENTS,
                                   LB_ADDSTRING,
                                   0,
                                   (LPARAM)&ofn.lpstrFile[ofn.nFileOffset]);
        if (LB_ERR != ulIdx)
            ulIdx = SendDlgItemMessage(hWnd,
                                       IDC_ATTACHMENTS,
                                       LB_SETITEMDATA,
                                       ulIdx,
                                       (LPARAM)ulAttNum);
    }

Quit:
    if (pStrmDest)
        pStrmDest -> Release();

    if (pStrmSrc)
        pStrmSrc -> Release();

    if (pAtt)
        pAtt -> Release();

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Gets properties from dialog, sets in msg,moves to the outbox, and submits.
//  
STDMETHODIMP CSession :: SendNote(HWND hDlg,LPMESSAGE pMsg)
{
    LPSPropValue  pspvEID    = NULL,
                  pspvOut    = NULL;
    LPMESSAGE     pSubmitMsg = NULL;
    ENTRYLIST     eidMsg;
    HRESULT       hRes;

    if (FAILED(hRes = MAPIAllocateBuffer(sizeof SPropValue * NUM_OUTBOUND_PROPS,
                                         (LPVOID *) &pspvOut)))
        return hRes;

    if (FAILED(hRes = GetUIData(hDlg,pspvOut)))
        goto Quit;

    if (FAILED(hRes = SetOutgoingProps(pMsg,pspvOut)))
        goto Quit;

    if (FAILED(hRes = m_pFolders[OUTBOX] -> CreateMessage(NULL,
                                                          0,
                                                          &pSubmitMsg)))
        goto Quit;

    if (FAILED(hRes = pMsg -> CopyTo(0,
                                     NULL,
                                     NULL,
                                     (ULONG) hDlg,
                                     NULL,
                                     &IID_IMessage,
                                     pSubmitMsg,
                                     0,
                                     NULL)))
            goto Quit;
  
    if (FAILED(hRes = pSubmitMsg -> SubmitMessage(0)))
        goto Quit;

    if (FAILED(hRes = HrGetOneProp(pMsg,PR_ENTRYID, &pspvEID)))
        goto Quit; 
    
    eidMsg.cValues = 1;
    eidMsg.lpbin   = &pspvEID[0].Value.bin;

    pMsg -> Release();  // can't delete an open message

    if (FAILED(hRes = m_pFolders[INBOX] -> DeleteMessages(&eidMsg,
                                                          (ULONG)hDlg,
                                                          NULL,
                                                          MESSAGE_DIALOG)))
        goto Quit;

Quit:
    if (pSubmitMsg)
        pSubmitMsg -> Release();

    MAPIFreeBuffer(pspvEID);
    MAPIFreeBuffer(pspvOut);

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Sets mandatory outgoing message properties.
//  
STDMETHODIMP    CSession :: SetOutgoingProps(LPMESSAGE pMsg, LPSPropValue pspvOut)
{
    LPSPropValue  pspv = NULL;
    HRESULT       hRes;

    pspvOut[OBFLAG].ulPropTag = PR_MESSAGE_FLAGS;
    pspvOut[OBSUBJ].ulPropTag = PR_SUBJECT;
    pspvOut[OBBODY].ulPropTag = PR_BODY;
    pspvOut[OBSENT].ulPropTag = PR_SENTMAIL_ENTRYID;

    if (FAILED(hRes = CopySBinary(& pspvOut[OBSENT].Value.bin, 
                                  & m_psbFolderEID[SENT],
                                  pspvOut)))
        goto Quit;                                    

    pspvOut[OBFLAG].Value.l = MSGFLAG_SUBMIT | MSGFLAG_READ;
    
    if (SUCCEEDED(hRes = HrGetOneProp(pMsg,PR_HASATTACH,&pspv)))
        pspvOut[OBFLAG].Value.l |= (pspv -> Value.b ? MSGFLAG_HASATTACH : 0);

    if (FAILED(hRes = pMsg -> SetProps(NUM_OUTBOUND_PROPS,pspvOut,NULL)))
        goto Quit;

    hRes = pMsg -> SaveChanges(KEEP_OPEN_READWRITE);

Quit:
    MAPIFreeBuffer(pspv);
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Creates a reply message, copies/sets appropriate properties,enables 
//  dialog box controls.
//
STDMETHODIMP CSession :: CreateReplyNote(HWND        hDlg, 
                                         PContext    pCtx, 
                                         LPMESSAGE * ppReplyMsg)
{
    LPSTR        szToLine = NULL;
    LPMESSAGE    pNewMsg  = NULL;
    SPropValue   spvNew[3];
    HRESULT      hRes;

    static const SizedSPropTagArray(13,sptExclude) = 
                                   {13, PR_SENDER_NAME,
                                        PR_SENDER_ADDRTYPE,
                                        PR_SENDER_EMAIL_ADDRESS,
                                        PR_SENDER_ENTRYID,
                                        PR_SENDER_SEARCH_KEY,
                                        PR_SENT_REPRESENTING_NAME,         
                                        PR_SENT_REPRESENTING_ADDRTYPE,     
                                        PR_SENT_REPRESENTING_EMAIL_ADDRESS,
                                        PR_SENT_REPRESENTING_ENTRYID,      
                                        PR_SENT_REPRESENTING_SEARCH_KEY,   
                                        PR_MESSAGE_RECIPIENTS,
                                        PR_BODY,
                                        PR_SUBJECT};

    spvNew[FLAG].ulPropTag = PR_MESSAGE_FLAGS;
    spvNew[SUBJ].ulPropTag = PR_SUBJECT;
    spvNew[BODY].ulPropTag = PR_BODY;
    
    spvNew[SUBJ].Value.lpszA = NULL;
    spvNew[BODY].Value.lpszA = NULL;
    spvNew[FLAG].Value.l = MSGFLAG_SUBMIT | MSGFLAG_UNSENT | MSGFLAG_READ;

    if (FAILED(hRes = m_pFolders[INBOX] -> CreateMessage(NULL,
                                                         0,
                                                         &pNewMsg)))
        goto Quit;

    if (CTX_FORWARD == pCtx -> m_lContext)
    {   // copy everything (e.g. attachments), except recips, subject,body
        hRes = (pCtx -> m_pMsg) -> CopyTo(0,
                                          NULL,
                                          (LPSPropTagArray) &sptExclude,
                                          (ULONG) hDlg,
                                          NULL,
                                          &IID_IMessage,
                                          pNewMsg,
                                          0,
                                          NULL);
        if (FAILED(hRes))
            goto Quit;
    }
    else    // replies only have what we set explicitly
        if (FAILED(hRes = SetReplyRecips(pNewMsg,
                                         pCtx -> m_pMsg,
                                         pCtx -> m_lContext,
                                         &szToLine)))
            goto Quit;

    // prepend "RE:" or "FW:" to subject
    if (FAILED(hRes = PrefixSzProp(pCtx -> m_pMsg,
                                   PR_NORMALIZED_SUBJECT,
                                   CTX_FORWARD == pCtx -> m_lContext ? 
                                        FW_PREFIX : 
                                        RE_PREFIX,
                                   &spvNew[SUBJ].Value.lpszA)))
        goto Quit;

    // prepend "-----\n" to body
    if (FAILED(hRes = PrefixSzProp(pCtx -> m_pMsg,
                                   PR_BODY,
                                   REPLY_BODY_PREFIX,
                                   &spvNew[BODY].Value.lpszA)))
        goto Quit;

    if (FAILED(hRes = pNewMsg -> SetProps(3,spvNew,NULL)))
        goto Quit;

    SetDlgItemText(hDlg,IDC_RECIP,szToLine);
    EnableWindow(GetDlgItem(hDlg,IDC_TO),pCtx -> m_lContext == CTX_FORWARD);
    EnableWindow(GetDlgItem(hDlg,IDC_SEND),pCtx -> m_lContext != CTX_FORWARD);

    if (CTX_FORWARD == pCtx -> m_lContext)
    {
        EnableWindow(GetDlgItem(hDlg,IDC_REPLY),FALSE);
        EnableWindow(GetDlgItem(hDlg,IDC_REPLY_ALL),FALSE);
    }

    if (SUCCEEDED(hRes = PopulateNote(hDlg,pNewMsg)))
        *ppReplyMsg = pNewMsg;

Quit:
    if (FAILED(hRes))
        if (pNewMsg)
            pNewMsg -> Release();

    MAPIFreeBuffer((LPVOID) szToLine);
    MAPIFreeBuffer((LPVOID) spvNew[SUBJ].Value.lpszA);
    MAPIFreeBuffer((LPVOID) spvNew[BODY].Value.lpszA);

    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Gets message sender or recipients and makes them the recipients of the
//  new reply message. The user is excluded on replies.
//
STDMETHODIMP CSession :: SetReplyRecips(LPMESSAGE pReplyMsg,const LPMESSAGE pOrigMsg,ULONG ulTo,LPSTR * pszToLine)
{
    HRESULT      hRes       = S_OK;
    LPMAPITABLE  pTbl       = NULL;
    LPSPropValue pspvSndr   = NULL,
                 pspvRecdBy = NULL;
    LPADRLIST    palTbl     = NULL,
                 palTo      = NULL;
    BOOL         fAddSndr   = TRUE;
    ULONG        ulCnt,
                 cRecips;
    int          i, j;
    
    // get all sender properties we need for a resolved name
    if (HR_FAILED(pOrigMsg -> GetProps((LPSPropTagArray) &g_sptSender,
                                       0,
                                       &ulCnt,
                                       &pspvSndr)))
    {
        hRes = MAPI_E_NOT_FOUND;
        goto Quit;
    }

    // convert the PR_SENT_REPRESENTING_XX tags to PR_XX tags
    for (i = 0; i < (int) ulCnt; i++)
    {
        if (pspvSndr[i].ulPropTag != g_sptSender.aulPropTag[i])
        {
            hRes = MAPI_E_NOT_FOUND;
            goto Quit;
        }
        pspvSndr[i].ulPropTag = g_sptRecips.aulPropTag[i];
    }

    pspvSndr[RECIP].Value.l = MAPI_TO;

    if (FAILED(hRes = HrGetOneProp(pOrigMsg,
                                   PR_RECEIVED_BY_SEARCH_KEY,
                                   &pspvRecdBy)))
        goto Quit;

    if (CTX_REPLY == ulTo)
        cRecips = 1;            // always reply to sender, even if us
    else
    {
        // turn inbound recips to reply recips.can't restrict for this
        if (FAILED(hRes = pOrigMsg -> GetRecipientTable(0, &pTbl)))
            goto Quit;

        // Use rows we got from table as ADRLIST. they're same structs
        if (FAILED(hRes = HrQueryAllRows(pTbl,
                                         (LPSPropTagArray) &g_sptRecips,
                                         NULL,
                                         NULL,
                                         0, 
                                         (LPSRowSet *)&palTbl))) 
            goto Quit;
        
        cRecips = palTbl -> cEntries;
        
        for (i = 0; i < (int) palTbl-> cEntries; i++)
            if (MAPI_BCC == palTbl -> aEntries[i].rgPropVals[RECIP].Value.l ||
                palTbl -> aEntries[i].rgPropVals[SKEY].Value.bin == pspvSndr[SKEY].Value.bin ||
                palTbl -> aEntries[i].rgPropVals[SKEY].Value.bin == pspvRecdBy[0].Value.bin)
            {
                cRecips--;
                palTbl -> aEntries[i].cValues = 0;
                MAPIFreeBuffer((LPVOID) palTbl -> aEntries[i].rgPropVals);
                palTbl -> aEntries[i].rgPropVals = NULL;
            }            

        if (cRecips)
            fAddSndr = FALSE;
        else
            cRecips = 1;
    }   

    // create merged adrlist of filtered tbl entries + (possibly) sender
    if (FAILED(hRes = MAPIAllocateBuffer(CbNewSRowSet(cRecips),(LPVOID*) &palTo)))
        goto Quit;

    palTo -> cEntries = 0;

    if (fAddSndr)
    {
        palTo -> aEntries[0].cValues = ulCnt;       // add to adrlist
        palTo -> aEntries[0].rgPropVals = pspvSndr;
        pspvSndr = NULL;        // so we don't have to check when freeing
        palTo -> cEntries++;
        cRecips--;
    }
    
    // if there are any more from the tbl, add them
    if (cRecips)
        for (i = 0, j = palTbl -> cEntries; i < j; i++)
        {
            if (palTbl -> aEntries[i].cValues)      // hasn't been filtered out
                palTo -> aEntries[palTo -> cEntries++] = palTbl -> aEntries[i];

            palTbl -> cEntries--;                   // when done, it should be empty
        }         

    // keep CC recips as MAPI_CC, but display them all on OUR TO line
    *pszToLine = ConcatRecips((LPADRLIST)palTo);
    hRes = pReplyMsg -> ModifyRecipients(0,palTo);

Quit:
    FreePadrlist(palTo);     
    FreePadrlist(palTbl);
    MAPIFreeBuffer((LPVOID) pspvSndr);
    MAPIFreeBuffer((LPVOID) pspvRecdBy);
    
    if (pTbl)
        pTbl -> Release();

    return hRes;
}
  
///////////////////////////////////////////////////////////////////////////////
//
//  Extracts display names from an ADRLIST and concats with ';' between each
//
LPSTR WINAPI ConcatRecips(LPADRLIST pAdrList)
{
    ULONG   cBytes   = 0;
    int     cRecips  = 0;
    LPSTR   szRecips = NULL,
          * szNameArray;
    
    if (0 == pAdrList -> cEntries)
        return NULL;

    szNameArray = new LPSTR[pAdrList -> cEntries];

    if (!szNameArray)
        return NULL;
    
    for (int i = 0; i < (int)pAdrList -> cEntries; i++)
    {
        LPSPropValue pProp = PpropFindProp(pAdrList -> aEntries[i].rgPropVals,
                                           pAdrList -> aEntries[i].cValues,
                                           PR_DISPLAY_NAME);
        
         if (pProp)
         {
            szNameArray[cRecips] = pProp -> Value.lpszA;
            cBytes += lstrlen(szNameArray[cRecips++]);
         }
    }

    cBytes += cRecips; // for the ';' delimiters and NULL

    if (FAILED(MAPIAllocateBuffer(sizeof TCHAR * (cBytes + 1),
                                  (LPVOID *) & szRecips)))
        return NULL;
    
    szRecips[0] = '\0';
        
    for (i = 0; i < cRecips; i++)
    {
        lstrcat(szRecips,szNameArray[i]);
        lstrcat(szRecips,";");
    }
    
    szRecips[cBytes - 1] = '\0';    // overwrites trailing ";" if only 1 recip
    delete [] szNameArray;
    return szRecips;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Prepends a prefix (e.g "FW: ") to a string property and returns new string.
//  
STDMETHODIMP CSession :: PrefixSzProp(LPMESSAGE   pMsg,
                                      ULONG       ulPropTag,
                                      LPTSTR      szPrefix,
                                      LPTSTR *    pszProp)
{
    HRESULT       hRes   = S_OK;
    LPSTR         szTemp = NULL;
    LPSPropValue  pspv   = NULL;
    ULONG         cb     = lstrlen(szPrefix);

    *pszProp = NULL;

    // get string to prefix
    if (FAILED(hRes = HrGetOneProp(pMsg,ulPropTag, &pspv)))
    {
        // that property not set, return prefix
        hRes = MAPIAllocateBuffer(sizeof TCHAR * (cb + 1),
                                  (LPVOID *) pszProp);
        
        if (FAILED(hRes))
            return hRes;
        lstrcpy(*pszProp,szPrefix);
        return hRes;
    }        
    
    cb = lstrlen(szPrefix) + lstrlen(pspv->Value.lpszA) + 1;
    hRes = MAPIAllocateBuffer(sizeof TCHAR * cb,(LPVOID *) & szTemp);

    if (FAILED(hRes))
        goto Quit;

    wsprintf(szTemp,"%s%s",szPrefix,pspv->Value.lpszA);    

Quit:
    MAPIFreeBuffer((LPVOID)pspv);    
    *pszProp = szTemp;
    return hRes;
}