#ifndef _HELLO_MAPI_H_
#define _HELLO_MAPI_H_
#include "resource.h"

#define APPNAME                 "Hello MAPI"
#define RE_PREFIX               "RE: "
#define FW_PREFIX               "FW: "
#define REPLY_BODY_PREFIX       "\r\n----------------\r\n\r\n"
#define MAX_SNDR                20
#define MAX_SUBJ                40
// format string for inbox listbox strings, assumes fixed font
#define SZ_FRMT                 "%-20s  %-s"
    
#define NOT_RECEIVED            (MSGFLAG_SUBMIT | MSGFLAG_UNSENT)
#define REQUIRED_FOLDER_MASK    (FOLDER_IPM_OUTBOX_VALID      |\
                                 FOLDER_IPM_SENTMAIL_VALID    |\
                                 FOLDER_IPM_WASTEBASKET_VALID |\
                                 FOLDER_IPM_SUBTREE_VALID     |\
                                 FOLDER_IPM_INBOX_VALID)

#define FLUSH_FLAGS              FLUSH_UPLOAD | FLUSH_DOWNLOAD | FLUSH_FORCE

#define LOGON_FLAGS              MAPI_ALLOW_OTHERS     | MAPI_NEW_SESSION |\
                                 MAPI_EXPLICIT_PROFILE | MAPI_LOGON_UI

#define PR_SENT_REPRESENTING_RECIP_TYPE    PR_NULL // saves spot in sender proplist for recip type

extern char g_szAppName[];
extern char g_szTitle[];

enum _MSG_PROPS_{SUBJ,BODY,FLAG,SNDR,ATT,NUM_MSG_PROPS}; 
enum _RECIPS_   {NAME,EID,ADDR,RECIP,EMAIL,SKEY,NUM_RECIP_PROPS};

enum _CTX_ID_   {CTX_INIT,CTX_INBOX,CTX_COMPOSE,CTX_READNOTE,CTX_SAVED,CTX_REPLY,CTX_REPLY_ALL,CTX_FORWARD};
enum _FOLDERS_  {INBOX,OUTBOX,WASTE,SENT,IPM_ROOT,NUM_FOLDERS};
enum _WHO_TO_   {SENDER,ALL};
enum _OB_PROPS_ {OBSUBJ,OBBODY,OBFLAG,OBSENT,NUM_OUTBOUND_PROPS};


typedef struct
{
    ULONG       ulFlags,
                ulPad;
    SBinary     sbEID;
} SMsgID, *PSMsgID;

typedef class CSession * PCSession;
typedef class CContext * PContext;

class CSession
{
    friend class CContext;
    public:        
        CSession(LPMAPISESSION pSess, HINSTANCE hInst)
        {
            m_pSess    = pSess;    
            m_hInst    = hInst;

            for (int i = 0; i < NUM_FOLDERS; i++)
                m_pFolders[i] = NULL;

            m_pMDB         = NULL;
            m_pAddrBook    = NULL;
            m_psbFolderEID = NULL;
            m_pSpoolerStat = NULL;
        };

        ~CSession()
        {
            if (m_pSpoolerStat)
                m_pSpoolerStat -> Release();

            for (int i = 0; i < NUM_FOLDERS; i++)
                if (m_pFolders[i])
                    m_pFolders[i] -> Release();

            if (m_pAddrBook)
                m_pAddrBook -> Release();

            if (m_pMDB)
                m_pMDB -> Release();
                            
            if (m_pSess)
                m_pSess -> Release();

            MAPIFreeBuffer((LPVOID) m_psbFolderEID);
            MAPIUninitialize();
        };

        HINSTANCE       m_hInst;
        LPSBinary       m_psbFolderEID;
        LPMAPIFOLDER    m_pFolders[NUM_FOLDERS];
        LPMAPISESSION   m_pSess;
        LPADRBOOK       m_pAddrBook;

        STDMETHODIMP    Init();
        STDMETHODIMP    OpenDefStore(HWND hWnd);
        STDMETHODIMP    OpenIPMSubtree(HWND hWnd);
        STDMETHODIMP    GetSpoolerStatus();

        STDMETHODIMP    AddRecips(HWND hDlg,LPSTR szTitle,PCSession pSess,LPMESSAGE pMsg);
        STDMETHODIMP    SendNote(HWND hDlg,LPMESSAGE pMsg);
        STDMETHODIMP    SenderDetails(HWND * phDlg,LPMESSAGE pMsg);
        STDMETHODIMP    CreateReplyNote(HWND        hDlg, 
                                        PContext    pCtx, 
                                        LPMESSAGE * ppReplyMsg);
        STDMETHODIMP    ShowDialog(HWND      hWnd,
                                   ULONG     ulCtx, 
                                   ULONG     cbEID, 
                                   LPENTRYID pEID,
                                   LPSTR     szTitle = "",
                                   LPMESSAGE pMsg = NULL);

        STDMETHODIMP    PopulateReadNote(HWND hDlg, PContext pCtx, LPMESSAGE * ppNewMsg);
        STDMETHODIMP    Save(HWND hDlg, LPMESSAGE pMsg);
        STDMETHODIMP    AttachFile(HWND hWnd,LPMESSAGE pMsg);
        STDMETHODIMP    DeleteMsg(HWND hDlg);
        STDMETHODIMP    FlushQs(HWND hWnd);

    private:
        TCHAR           m_szTitle[MAX_PATH];
        LPMDB           m_pMDB;
        LPMAPISTATUS    m_pSpoolerStat;

        STDMETHODIMP    PrefixSzProp(LPMESSAGE   pMsg,
                                     ULONG       ulPropTag,
                                     LPSTR       szPrefix,
                                     LPSTR *     pszProp);
        STDMETHODIMP    SetOutgoingProps(LPMESSAGE pMsg, LPSPropValue pspvOut);
        STDMETHODIMP    SetReplyRecips(LPMESSAGE pReplyMsg,const LPMESSAGE pOrigMsg,ULONG ulTo,LPSTR * pszToLine);
        STDMETHODIMP    ReplyTo(ULONG ulTo,LPMESSAGE pReplyMsg,const LPMESSAGE pOrigMsg);
};

   
class CContext
{
    public:
        CContext(ULONG lContext, ULONG cbEID, LPENTRYID pEID,PCSession pSess,LPSTR szTitle = "",LPMESSAGE pMsg = NULL)
        {
            m_lContext = lContext;
            m_cbEID = cbEID;
            m_pParentEID = pEID;
            m_pSess = pSess;    
            m_pMsg = pMsg;
            m_szTitle = szTitle;
        };

        ~CContext(){};

        LONG        m_lContext;
        ULONG       m_cbEID;
        LPENTRYID   m_pParentEID;
        PCSession   m_pSess;
        LPMESSAGE   m_pMsg;
        LPSTR       m_szTitle;
};


extern "C"
{
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK ViewInbox(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK ComposeNote(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK ReadNote(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK ReplyNote(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
}

BOOL WINAPI  InitApplication(HINSTANCE);
BOOL WINAPI  InitInstance(HINSTANCE hInstance, int nCmdShow);

void WINAPI  InitDialog(HWND hDlg,LPARAM lParam,PContext & pContext);
LONG WINAPI  GetLBSelection(HWND hDlg,WORD wCtlID,LPVOID * ppItemData);
STDMETHODIMP CopySBinary(LPSBinary psbDest,
                         const LPSBinary psbSrc,
                         LPVOID pParent);
STDMETHODIMP InitSession(HWND hWnd,
                         HINSTANCE hInst,
                         PCSession * ppCSess);
void WINAPI  EmptyListBox(HWND hDlg,WORD wCtlID);
STDMETHODIMP GetEditText(HWND hDlg, 
                         WORD wCtl,
                         LPVOID pParent,
                         LPSTR * pszText);
STDMETHODIMP GetUIData(HWND hDlg, LPSPropValue pspvOut);
STDMETHODIMP LoadListBox(HWND hDlgParent,LPARAM lParam,ULONG ulCtlID);
STDMETHODIMP PopulateNote(HWND hDlg,LPMESSAGE pMsg);
LPSTR WINAPI ConcatRecips(LPADRLIST pAdrList);
BOOL         operator == (const SBinary & lhs, const SBinary & rhs);

// Displays a message box and posts a quit message on a MAPI 
// warning or error. Note that use of this macro is intended
// for the duration of a single (Windows) message. If code follows this
// that depends on the result of an earlier call in the same message,
// continued execution may not be possible.

#define OnError(hWnd, hRes)                                         \
{                                                                   \
    HRESULT hr = (hRes);                                            \
    if (S_OK != hr)                                                 \
    {                                                               \
        char szMsg[80];                                             \
        wsprintf(szMsg,"Encountered MAPI error 0x%08X.",hr);        \
        if (IDOK == MessageBox((hWnd),                              \
                                szMsg,                              \
                                "Hit Cancel to Quit, OK to Debug",  \
                                MB_OKCANCEL))                       \
            DebugBreak();                                           \
        else                                                        \
            PostMessage((hWnd),WM_COMMAND,ID_ERROR_EXIT,0);         \
    }                                                               \
} 

BOOL WINAPI BreakToDebug(HRESULT hr, LPMAPIERROR pErr,HWND hWnd);

#define ShowLastError(hRes, hWnd, _pIF_)    \
{                                           \
    HRESULT     hr   = (hRes);              \
    LPMAPIERROR pErr = NULL;                \
    if (hr)                                 \
    {                                       \
        (_pIF_) -> GetLastError(hr,0,&pErr);\
        if (BreakToDebug(hr,pErr,hWnd))     \
            DebugBreak();                   \
        MAPIFreeBuffer((LPVOID)pErr);       \
    }                                       \
}
    
#endif
