///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      BECOMMON.CPP
//
//  Description
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Corporation
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//
#include "BECOMMON.H"

///////////////////////////////////////////////////////////////////////////////
//    GetHierarchyTableProps()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI GetHierarchyTableProps (CPropList *     pList,
                                       LPSPropValue *  ppProps,
                                       PRIVATE_ENTRYID FolderEID,
                                       PRIVATE_ENTRYID ParentFolderEID,
                                       long            lAccess,
                                       LPSBinary       pStoreEID,
                                       ULONG           ulFolderType,
                                       DWORD &         dwLastAllocated,
                                       LPTABLEDATA     pTable,
                                       MAPIUID &       uidData)
{
    DWORD dwPropCount = MIN_HIERARCHY_TABLE_PROPS;
    if (dwLastAllocated < dwPropCount)
    {
        PrivFree ((LPVOID *)ppProps);
        dwLastAllocated = 0;
        HRESULT hResult = PrivAllocate (sizeof(SPropValue) * dwPropCount, (LPVOID *)ppProps, "Properties in Hierarchy Table");
        if (hResult)
        {
            return hResult;
        }
        dwLastAllocated = dwPropCount;
    }
    ULONG i = 0;
    DWORD key = FolderEID.dwObjID;
    (*ppProps)[i].ulPropTag          = PR_INSTANCE_KEY;
    (*ppProps)[i].Value.bin.cb       = sizeof(DWORD);
    (*ppProps)[i++].Value.bin.lpb    = (LPBYTE)&key;
    
    (*ppProps)[i].ulPropTag          = PR_ACCESS;
    (*ppProps)[i++].Value.l          = lAccess;
    
    (*ppProps)[i].ulPropTag          = PR_ENTRYID;
    (*ppProps)[i].Value.bin.cb       = CB_PRIVATE_EID;
    (*ppProps)[i++].Value.bin.lpb    = (LPBYTE)&FolderEID;

    (*ppProps)[i].ulPropTag          = PR_PARENT_ENTRYID;
    (*ppProps)[i].Value.bin.cb       = CB_PRIVATE_EID;
    (*ppProps)[i++].Value.bin.lpb    = (LPBYTE)&ParentFolderEID;
    
    (*ppProps)[i].ulPropTag          = PR_STORE_ENTRYID;
    (*ppProps)[i].Value.bin.cb       = pStoreEID->cb;
    (*ppProps)[i++].Value.bin.lpb    = pStoreEID->lpb;
    
    (*ppProps)[i].ulPropTag          = PR_STORE_RECORD_KEY;
    (*ppProps)[i].Value.bin.cb       = sizeof(MAPIUID);
    (*ppProps)[i++].Value.bin.lpb    = (LPBYTE)&uidData;

    (*ppProps)[i].ulPropTag          = PR_RECORD_KEY;
    (*ppProps)[i].Value.bin.cb       = sizeof(MAPIUID);
    (*ppProps)[i++].Value.bin.lpb    = (LPBYTE)&uidData;

    (*ppProps)[i].ulPropTag          = PR_MAPPING_SIGNATURE;
    (*ppProps)[i].Value.bin.cb       = sizeof(MAPIUID);
    (*ppProps)[i++].Value.bin.lpb    = (LPBYTE)&uidData;

    (*ppProps)[i].ulPropTag          = PR_OBJECT_TYPE;
    (*ppProps)[i++].Value.bin.cb     = MAPI_FOLDER;

    (*ppProps)[i].ulPropTag          = PR_DEPTH;
    (*ppProps)[i++].Value.l          = 1;

    (*ppProps)[i].ulPropTag          = PR_FOLDER_TYPE;
    (*ppProps)[i++].Value.ul         = ulFolderType;

    (*ppProps)[i].ulPropTag          = PR_DISPLAY_TYPE;
    (*ppProps)[i++].Value.l          = DT_FOLDER;

    pList->Find (PR_STATUS, &(*ppProps)[i++]);
    pList->Find (PR_CONTENT_COUNT, &(*ppProps)[i++]);
    pList->Find (PR_CONTENT_UNREAD, &(*ppProps)[i++]);
    pList->Find (PR_ASSOC_CONTENT_COUNT, &(*ppProps)[i++]);
    pList->Find (PR_SUBFOLDERS, &(*ppProps)[i++]);
    pList->Find (PR_CREATION_TIME, &(*ppProps)[i++]);
    pList->Find (PR_LAST_MODIFICATION_TIME, &(*ppProps)[i++]);
    pList->Find (PR_DISPLAY_NAME, &(*ppProps)[i++]);
    pList->Find (PR_COMMENT, &(*ppProps)[i++]);
    pList->Find (PR_DEFAULT_VIEW_ENTRYID, &(*ppProps)[i++]);
    
    ASSERT (i == dwPropCount);
    SRow srRow = { 0 };
    srRow.cValues = i;
    srRow.lpProps = *ppProps;
    return pTable->HrModifyRow (&srRow);
}

///////////////////////////////////////////////////////////////////////////////
//    SetPropsInList()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI SetPropsInList (ULONG        cValues,
                               LPSPropValue pProps,
                               CPropList ** ppList)
{
    HRESULT hResult = S_OK;
    *ppList = new CPropList;
    if (*ppList)
    {
        for (UINT i=0; i<cValues; i++)
        {
            (*ppList)->Insert (&pProps[i]);
        }
    }
    else
    {
        hResult = E_OUTOFMEMORY;
    }
    TraceResult ("SetPropsInList", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    InitFolderProps()
//
//    Parameters
//
//    Purpose
//      The Default properties in our folders are:
//          PR_CONTENT_COUNT
//          PR_CONTENT_UNREAD
//          PR_ASSOC_CONTENT_COUNT
//          PR_SUBFOLDERS
//          PR_CREATION_TIME
//          PR_LAST_MODIFICATION_TIME
//          PR_DISPLAY_NAME
//          PR_COMMENT
//          PR_PARENT_ENTRYID
//          PR_STATUS
//      Other properties are also part of the default set but they are
//      computed on the fly or implicit (i.e. PR_ENTRYID)
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI InitFolderProps (LPTSTR          pszName,
                                LPTSTR          pszComment,
                                PRIVATE_ENTRYID ParentEID,
                                CPropList **    ppList)
{
    SYSTEMTIME st;
    FILETIME ft;
    GetSystemTime (&st);
    SystemTimeToFileTime (&st, &ft);
    ULONG i = 0;
    SPropValue spvProps[10] = { 0 };
    spvProps[i].ulPropTag       = PR_CONTENT_COUNT;
    spvProps[i++].Value.l       = 0;
    spvProps[i].ulPropTag       = PR_CONTENT_UNREAD;
    spvProps[i++].Value.l       = 0;
    spvProps[i].ulPropTag       = PR_ASSOC_CONTENT_COUNT;
    spvProps[i++].Value.l       = 0;
    spvProps[i].ulPropTag       = PR_SUBFOLDERS;
    spvProps[i++].Value.b       = FALSE;
    spvProps[i].ulPropTag       = PR_CREATION_TIME;
    spvProps[i++].Value.ft      = ft;
    spvProps[i].ulPropTag       = PR_LAST_MODIFICATION_TIME;
    spvProps[i++].Value.ft      = ft;
    spvProps[i].ulPropTag       = PR_DISPLAY_NAME;
    spvProps[i++].Value.LPSZ    = pszName;
    spvProps[i].ulPropTag       = PR_COMMENT;
    spvProps[i++].Value.LPSZ    = pszComment;
    spvProps[i].ulPropTag       = PR_PARENT_ENTRYID;
    spvProps[i].Value.bin.cb    = CB_PRIVATE_EID;
    spvProps[i++].Value.bin.lpb = (LPBYTE)&ParentEID;
    spvProps[i].ulPropTag       = PR_STATUS;
    spvProps[i++].Value.l       = 0;
    ASSERT (i == (sizeof(spvProps)/sizeof(SPropValue)));
    return SetPropsInList (i, spvProps, ppList);
}

///////////////////////////////////////////////////////////////////////////////
//    InitMessageProps()
//
//    Parameters
//
//    Purpose
//      The Default properties in our messages are:
//          PR_MESSAGE_FLAGS
//          PR_CREATION_TIME
//          PR_LAST_MODIFICATION_TIME
//          PR_MESSAGE_SIZE
//          PR_PRIORITY
//          PR_SEARCH_KEY
//          PR_MESSAGE_CLASS
//          PR_PARENT_ENTRYID
//      Here we also add PR_MSG_STATUS so that it gets stored in the
//      database, but this property is not part of the object's property list.
//      This is set here to initialize it to a value.
//
//      Other properties are also part of the default set but they are
//      computed on the fly or implicit (i.e. PR_ENTRYID)
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI InitMessageProps (BOOL            fCreateAssociated,
                                 PRIVATE_ENTRYID ParentEID,
                                 LPBYTE          pbSearchKey,
                                 CPropList **    ppList)
{
    SYSTEMTIME st;
    FILETIME ft;
    GetSystemTime (&st);
    SystemTimeToFileTime (&st, &ft);
    ULONG i = 0;
    SPropValue spvProps[9] = { 0 };
    spvProps[i].ulPropTag       = PR_MESSAGE_FLAGS;
    spvProps[i++].Value.l       = MSGFLAG_UNSENT | MSGFLAG_READ | (fCreateAssociated ? MSGFLAG_ASSOCIATED : 0);
    spvProps[i].ulPropTag       = PR_CREATION_TIME;
    spvProps[i++].Value.ft      = ft;
    spvProps[i].ulPropTag       = PR_LAST_MODIFICATION_TIME;
    spvProps[i++].Value.ft      = ft;
    spvProps[i].ulPropTag       = PR_MESSAGE_SIZE;
    spvProps[i++].Value.l       = 0; // Because there is no PR_BODY/PR_RTF_COMPRESSED yet
    spvProps[i].ulPropTag       = PR_PRIORITY;
    spvProps[i++].Value.l       = PRIO_NORMAL;
    spvProps[i].ulPropTag       = PR_SEARCH_KEY;
    spvProps[i].Value.bin.cb    = sizeof(MAPIUID);
    spvProps[i++].Value.bin.lpb = pbSearchKey;
    spvProps[i].ulPropTag       = PR_MESSAGE_CLASS;
    spvProps[i++].Value.LPSZ    = TEXT("IPM");
    spvProps[i].ulPropTag       = PR_PARENT_ENTRYID;
    spvProps[i].Value.bin.cb    = CB_PRIVATE_EID;
    spvProps[i++].Value.bin.lpb = (LPBYTE)&ParentEID;
    spvProps[i].ulPropTag       = PR_MSG_STATUS;
    spvProps[i++].Value.l       = 0;
    ASSERT (i == (sizeof(spvProps)/sizeof(SPropValue)));
    return SetPropsInList (i, spvProps, ppList);
}

///////////////////////////////////////////////////////////////////////////////
//    InitAttachProps()
//
//    Parameters
//
//    Purpose
//      The Default properties in our attachment implementation are:
//          PR_ATTACH_METHOD
//          PR_ATTACH_SIZE
//          PR_CREATION_TIME
//          PR_LAST_MODIFICATION_TIME
//          PR_DISPLAY_NAME
//      Other properties are also part of the default set but they are
//      computed on the fly or implicit (i.e. PR_ENTRYID)
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI InitAttachProps (CPropList ** ppList)
{
    SYSTEMTIME st;
    FILETIME ft;
    GetSystemTime (&st);
    SystemTimeToFileTime (&st, &ft);
    ULONG i = 0;
    SPropValue spvProps[5] = { 0 };
    spvProps[i].ulPropTag    = PR_ATTACH_METHOD;
    spvProps[i++].Value.l    = NO_ATTACHMENT;
    spvProps[i].ulPropTag    = PR_ATTACH_SIZE;
    spvProps[i++].Value.l    = 0;
    spvProps[i].ulPropTag    = PR_CREATION_TIME;
    spvProps[i++].Value.ft   = ft;
    spvProps[i].ulPropTag    = PR_LAST_MODIFICATION_TIME;
    spvProps[i++].Value.ft   = ft;
    spvProps[i].ulPropTag    = PR_DISPLAY_NAME;
    spvProps[i++].Value.LPSZ = TEXT("");
    ASSERT (i == (sizeof(spvProps)/sizeof(SPropValue)));
    return SetPropsInList (i, spvProps, ppList);
}

// End of file for BECOMMON.CPP