///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      CDBFILE.CPP
//              
//  Description
//
//  Author
//      Irving De la Cruz
//
//  Revision: 0.99
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#pragma warning (disable : 4100)
#pragma warning (disable : 4201)
#pragma warning (disable : 4514)

#include "CDBFILE.H"

// Remark this line to turn verbose tracing OFF
//#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)                TraceMessage(a)
#define InfoTraceString1(a,b)       TraceString1(a, b)
#define InfoTraceString2(a,b,c)     TraceString2(a, b, c)
#else
#define InfoTrace(a)
#define InfoTraceString1(a,b)
#define InfoTraceString2(a,b,c)
#endif // DO_INFO_TRACES

#ifndef USE_VM_ALLOCS
HANDLE ghDatabaseObjs = NULL;
#endif // USE_VM_ALLOCS

BOOL fTerminateNow = FALSE;

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::CDataBase()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
CDataBase::CDataBase (MAPIUID UIDinFile, HANDLE hFile)
{
    InfoTraceString1 ("CDataBase: Constructor called on thread ID %d", GetCurrentThreadId());
    m_UID = UIDinFile;
    m_hFile = hFile;
    m_cRef = 1;
    m_fGotTreeNodes = FALSE;
    m_fNodesChanged = FALSE;
    m_fFreeNodePoolChanged = FALSE;
    m_fGotFreeNodePool = FALSE;
    
    fTerminateNow = FALSE;

    InitializeCriticalSection (&m_csObj);
    
    HRESULT hResult = S_OK;
    DWORD dwThreadID;

    m_hReleaseTreeEvent = CreateEvent (NULL, TRUE, FALSE, TEXT("CDB TREE RELEASE"));
    if (NULL == m_hReleaseTreeEvent)
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
        goto ErrorExit;
    }
    m_hTreeReleasedEvent = CreateEvent (NULL, TRUE, FALSE, TEXT("CDB RELEASE DONE"));
    if (NULL == m_hTreeReleasedEvent)
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
        goto ErrorExit;
    }

    m_hSaveEvent = CreateEvent (NULL, TRUE, FALSE, NULL);
    if (NULL == m_hSaveEvent)
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
        goto ErrorExit;
    }
    
    m_hListenThread = CreateThread (NULL,
                                    0,
                                    (LPTHREAD_START_ROUTINE)ListenThreadProc,
                                    (LPVOID)this,
                                    0,
                                    &dwThreadID);
    if (!m_hListenThread)
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
        goto ErrorExit;
    }
    
    m_hSaveThread = CreateThread (NULL,
                                  0,
                                  (LPTHREAD_START_ROUTINE)BackgroundSaveThreadProc,
                                  (LPVOID)this,
                                  CREATE_SUSPENDED,
                                  &dwThreadID);
    if (!m_hSaveThread)
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
        goto ErrorExit;
    }
    SetThreadPriority (m_hSaveThread, THREAD_PRIORITY_LOWEST);

ErrorExit:
    if (hResult)
    {
        TraceResult ("CDataBase constructor", hResult);
        DeleteCriticalSection (&m_csObj);
        CloseHandle (m_hReleaseTreeEvent);
        CloseHandle (m_hTreeReleasedEvent);
        CloseHandle (m_hSaveEvent);
        CloseHandle (m_hListenThread);
        CloseHandle (m_hSaveThread);
        throw CException (hResult);
    }
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::~CDataBase()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
CDataBase::~CDataBase()
{
    InfoTrace ("CDataBase: Destructor called");
    fTerminateNow = TRUE;
    EnterCriticalSection (&m_csObj);
    SaveIndexNodes();
    SaveFreeNodesPool();
    LeaveCriticalSection (&m_csObj);
    PulseEvent (m_hSaveEvent);
    CloseHandle (m_hSaveEvent);
    
    SetThreadPriority (m_hSaveThread, THREAD_PRIORITY_HIGHEST);
    PulseEvent (m_hReleaseTreeEvent);
    CloseHandle (m_hReleaseTreeEvent);
    CloseHandle (m_hTreeReleasedEvent);
    
    CloseHandle (m_hSaveThread);
    CloseHandle (m_hListenThread);
    CloseHandle (m_hFile);
    DeleteCriticalSection (&m_csObj);
    
    m_btNodes.DeleteAll(); 
    
    #ifndef USE_VM_ALLOCS
    HeapDestroy (ghDatabaseObjs);
    ghDatabaseObjs = NULL;
    #endif // USE_VM_ALLOCS
    InfoTrace ("CDataBase: Destructor is finished");
}

///////////////////////////////////////////////////////////////////////////////
//    ListenThreadProc()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
DWORD WINAPI ListenThreadProc (CDataBase * pObj)
{
    InfoTraceString1 ("ListenThreadProc: Thread started. Thread ID %d", GetCurrentThreadId());
    DWORD dwProcessID = GetCurrentProcessId();
    InfoTraceString1 ("ListenThreadProc: Process %d is listening ...", dwProcessID);
    BOOL fContinue = TRUE;
    while (fContinue)
    {
        __try
        {
            WaitForSingleObject (pObj->m_hReleaseTreeEvent, INFINITE);
            if (fTerminateNow)
            {
                InfoTraceString1 ("ListenThreadProc: Process %d is going away", dwProcessID);
                fContinue = FALSE;
            }
            else
            {
                EnterCriticalSection (&(pObj->m_csObj));
                if (pObj->m_fGotTreeNodes)
                {
                    pObj->SaveIndexNodes();
                    pObj->m_fGotTreeNodes = FALSE;
                    pObj->m_btNodes.DeleteAll();

                    pObj->SaveFreeNodesPool();
                    pObj->m_fGotFreeNodePool = FALSE;
            
                    SuspendThread (pObj->m_hSaveThread);
                    InfoTraceString1 ("ListenThreadProc: Background save thread SUSPENDED in process %d", dwProcessID);

                    pObj->UnlockRecord (OFFSET_FIRST_INDEX_RECORD, sizeof(DATA_NODE));
            
                    InfoTraceString1 ("ListenThreadProc: Process %d will release the index", dwProcessID);
                    PulseEvent (pObj->m_hTreeReleasedEvent);
                }
                LeaveCriticalSection (&(pObj->m_csObj));
            }
        }
        __except (EXCEPTION_EXECUTE_HANDLER)
        {
            TraceString1 ("ListenThreadProc: An exception occured. Exception code: %#08x", GetExceptionCode());
            fContinue = FALSE;
        }
    }
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
//    BackgroundSaveThreadProc()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
DWORD WINAPI BackgroundSaveThreadProc (CDataBase * pObj)
{
    InfoTraceString1 ("BackgroundSaveThreadProc: Thread started. Thread ID %d", GetCurrentThreadId());
    DWORD dwProcessID = GetCurrentProcessId();
    BOOL fContinue = TRUE;
    while (fContinue)
    {
        __try
        {
            WaitForSingleObject (pObj->m_hSaveEvent, 5000);
            if (fTerminateNow)
            {
                InfoTraceString1 ("BackgroundSaveThreadProc: Process %d is going away", dwProcessID);
                fContinue = FALSE;
            }
            else
            {
                EnterCriticalSection (&(pObj->m_csObj));
                if (pObj->m_fGotTreeNodes)
                {
                    if (pObj->m_fNodesChanged)
                    {
                        InfoTraceString1 ("BackgroundSaveThreadProc: Process %d is saving changed nodes", dwProcessID);
                        pObj->SaveIndexNodes();
                    }
                }
                if (pObj->m_fGotFreeNodePool)
                {
                    if (pObj->m_fFreeNodePoolChanged)
                    {
                        InfoTraceString1 ("BackgroundSaveThreadProc: Process %d is saving the free nodes pool", dwProcessID);
                        pObj->SaveFreeNodesPool();
                    }
                }
                LeaveCriticalSection (&(pObj->m_csObj));
            }
        }
        __except (EXCEPTION_EXECUTE_HANDLER)
        {
            TraceString1 ("BackgroundSaveThreadProc: An exception occured. Exception code: %#08x", GetExceptionCode());
            fContinue = FALSE;
        }
    }
    return 0;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::ReadWriteFreeNodeHeader()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::ReadWriteFreeNodeHeader (DWORD   dwAction,
                                                 DWORD * pdwNewRecordID,
                                                 DWORD * pdwNewOffSet)
{
    FREENODES_HEADER fnhInfo = { 0 };
    DWORD dwBytes;
    HRESULT hResult = LockRecord (OFFSET_FREENODES_HEADER, sizeof(FREENODES_HEADER));
    if (!hResult)
    {
        SetFilePointer (m_hFile, OFFSET_FREENODES_HEADER, NULL, FILE_BEGIN);
        ReadFile (m_hFile, &fnhInfo, sizeof(FREENODES_HEADER), &dwBytes, NULL);
        ASSERT (sizeof(FREENODES_HEADER) == dwBytes);
        ASSERT (fnhInfo.dwNextID);
        if (FNH_GET_FREE_BLOCK & dwAction)
        {
            if (GetFreeNodeFromPool (*pdwNewOffSet))
            {
                hResult = GetNewEmptyNode (fnhInfo, *pdwNewOffSet);
            }
        }
        if (!hResult && (FNH_GET_NEXT_RECORD_ID & dwAction))
        {
            *pdwNewRecordID = fnhInfo.dwNextID;
            fnhInfo.dwNextID++;
            if (0 == fnhInfo.dwNextID)
            {
                fnhInfo.dwNextID++;
            }
        }
        if (!hResult)
        {
            SetFilePointer (m_hFile, OFFSET_FREENODES_HEADER, NULL, FILE_BEGIN);
            WriteFile (m_hFile, &fnhInfo, sizeof(FREENODES_HEADER), &dwBytes, NULL);
            ASSERT (sizeof(FREENODES_HEADER) == dwBytes);
        }
        UnlockRecord (OFFSET_FREENODES_HEADER, sizeof(FREENODES_HEADER));
    }
    TraceResult ("CDataBase::ReadWriteFreeNodeHeader", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::AccessData()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::AccessData (ACCESS_DATA_ACTION Action,
                                    DATA_NODE &        dnData,
                                    BOOL               fJustReadHeader)
{
    DWORD dwBytes, dwBytesToRead;
    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    if (0xFFFFFFFF == SetFilePointer (m_hFile, dnData.dwOffSetNode, NULL, FILE_BEGIN))
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
    }
    else
    {
        if (WRITE == Action)
        {
            if (FALSE == WriteFile (m_hFile, &dnData, sizeof(DATA_NODE), &dwBytes, NULL))
            {
                hResult = HRESULT_FROM_WIN32 (GetLastError());
            }
        }
        else
        {
            if (fJustReadHeader)
            {
                dwBytesToRead = DATA_NODE_HEADER_SIZE;
            }
            else
            {
                dwBytesToRead = sizeof(DATA_NODE);
            }
            if (FALSE == ReadFile (m_hFile, &dnData, dwBytesToRead, &dwBytes, NULL))
            {
                hResult = HRESULT_FROM_WIN32 (GetLastError());
            }
        }
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CDataBase::ReadWriteFreeNodeHeader", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::LockRecord()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::LockRecord (DWORD dwRecordOffSet,
                                    DWORD dwRecordSize,
                                    BOOL  fWaitUntilUnlock)
{
    HRESULT hResult;
    DWORD dwProcessID = GetCurrentProcessId(), dwWaitedTime = 0;
RestartLock:
    if (LockFile (m_hFile, dwRecordOffSet, 0, dwRecordSize, 0))
    {
        hResult = S_OK;
    }
    else
    {
        hResult = GetLastError();
        if (fWaitUntilUnlock && ERROR_LOCK_VIOLATION == hResult)
        {
            Sleep (250);
            dwWaitedTime += 250;
            InfoTraceString2 ("LockRecord: Process %d could not lock record... Waited time %d milliseconds... Restarting again... ", dwProcessID, dwWaitedTime);
            if (dwWaitedTime < 15000) // Wait 15 seconds, and then give up
            {
                goto RestartLock;
            }
            hResult = MAPI_E_BUSY;
        }
        else
        {
            hResult = HRESULT_FROM_WIN32 (hResult);
        }
    }
#ifdef _DEBUG
    if (hResult && (HRESULT_FROM_WIN32(ERROR_LOCK_VIOLATION) != hResult))
    {
        TraceResult ("CDataBase::LockRecord", hResult);
    }
#endif // _DEBUG
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::UnlockRecord()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::UnlockRecord (DWORD dwRecordOffSet, DWORD dwRecordSize)
{
    HRESULT hResult = S_OK;
    if (!UnlockFile (m_hFile, dwRecordOffSet, 0, dwRecordSize, 0))
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
    }
    TraceResult ("CDataBase::UnlockRecord", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::CreateEmptyRecord()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::CreateEmptyRecord (DWORD * pdwRecordID)
{
    InfoTrace ("CDataBase::CreateEmptyRecord method called");
    HRESULT hResult;
    EnterCriticalSection (&m_csObj);
    __try
    {
GetNewIDAgain :
        hResult = ReadWriteFreeNodeHeader (FNH_GET_NEXT_RECORD_ID, pdwRecordID, NULL);
        if (!hResult && m_btNodes.Find (*pdwRecordID))
        {
            goto GetNewIDAgain;
        }
    }
    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        TraceString1 ("CDataBase::CreateEmptyRecord: An exception occured. Exception code: %#08x", GetExceptionCode());
        hResult = E_FAIL;
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CDataBase::CreateEmptyRecord", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::CreateRecord()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::CreateRecord (DWORD * pdwRecordID, LPSTREAM * ppStream)
{
    InfoTrace ("CDataBase::CreateRecord method called");
    #ifdef USE_MEMORY_STREAM
    CMemoryStream * pNewStream;
    #else
    CStream * pNewStream;
    #endif 
    INDEX_NODE NewNode = { 0 }, *pInserted;
    DWORD dwFlags;
    EnterCriticalSection (&m_csObj);
    HRESULT hResult = LoadIndexNodes();
    if (!hResult)
    {
        if (0 == *pdwRecordID)
        {
            dwFlags = FNH_GET_NEXT_RECORD_ID | FNH_GET_FREE_BLOCK;
        }
        else
        {
            NewNode.dwNodeID = *pdwRecordID;
            dwFlags = FNH_GET_FREE_BLOCK;
            if (m_btNodes.Find (NewNode.dwNodeID))
            {
                hResult = E_INVALIDARG;
            }
        }
        if (!hResult)
        {
GetNewIDAgain :
            hResult = ReadWriteFreeNodeHeader (dwFlags, &NewNode.dwNodeID, &NewNode.dwPropsStartOffSet);
            if (!hResult)
            {
                if (m_btNodes.Find (NewNode.dwNodeID))
                {
                    dwFlags = FNH_GET_NEXT_RECORD_ID;
                    goto GetNewIDAgain;
                }
            }
            if (!hResult)
            {
                try
                {
                    pInserted = m_btNodes.Insert (NewNode);
                    *pdwRecordID = NewNode.dwNodeID;
                }
                catch (CException & Exception)
                {
                    hResult = Exception.GetError();
                }
                if (!hResult)
                {
                    ASSERT (pInserted);
                    ASSERT (pInserted->dwNodeID == NewNode.dwNodeID);
                    ASSERT (*pdwRecordID == NewNode.dwNodeID);
                    ASSERT (pInserted->dwPropsStartOffSet);
                    hResult = LockRecord (pInserted->dwPropsStartOffSet, sizeof(DATA_NODE));
                    if (!hResult)
                    {
                        try
                        {
                            #ifdef USE_MEMORY_STREAM
                            pNewStream = new CMemoryStream (this, pInserted->dwPropsStartOffSet, &(pInserted->dwPropStreamSize));
                            #else
                            pNewStream = new CStream (this, pInserted->dwPropsStartOffSet, &(pInserted->dwPropStreamSize));
                            #endif 
                            if (pNewStream)
                            {
                                *ppStream = (LPSTREAM)pNewStream;
                            }
                            else
                            {
                                hResult = E_OUTOFMEMORY;
                            }
                        }
                        catch (CException & Exception)
                        {
                            hResult = Exception.GetError();
                        }
                        if (hResult)
                        {
                            UnlockRecord (NewNode.dwPropsStartOffSet, sizeof(DATA_NODE));
                        }
                    }
                }
                if (hResult && (0 == *pdwRecordID))
                {
                    m_btNodes.Delete (NewNode.dwNodeID);
                }
                else
                {
                    NodeChanged();
                }
            }
        }
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CDataBase::CreateRecord", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::OpenRecord()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::OpenRecord (DWORD dwRecordID, LPSTREAM * ppStream)
{
    InfoTrace ("CDataBase::OpenRecord method called");
    HRESULT hResult = GetNodeStream (dwRecordID, RECFLD_PROPERTIES, ppStream, FALSE);
    TraceResult ("CDataBase::OpenRecord", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::OpenRecordField()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::OpenRecordField (DWORD         dwRecordID,
                                         BOOL          fCreateIfNotFound,
                                         RECORD_FIELDS rfStream,
                                         LPSTREAM *    ppStream)
{
    InfoTrace ("CDataBase::OpenRecordField method called");
    HRESULT hResult = GetNodeStream (dwRecordID, rfStream, ppStream, fCreateIfNotFound);
#ifdef _DEBUG
    if (MAPI_E_NOT_FOUND != hResult)
    {
        TraceResult ("CDataBase::OpenRecordField", hResult);
    }
#endif // _DEBUG
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::DeleteRecord()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::DeleteRecord (DWORD dwRecordID)
{
    InfoTrace ("CDataBase::DeleteRecord method called");
    HRESULT hResult = DeleteField (dwRecordID, RECFLD_ALL);
    TraceResult ("CDataBase::DeleteRecord", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::DeleteRecordField()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::DeleteRecordField (DWORD         dwRecordID,
                                           RECORD_FIELDS rfStream)
{
    InfoTrace ("CDataBase::DeleteRecordField method called");
    HRESULT hResult = DeleteField (dwRecordID, rfStream);
    TraceResult ("CDataBase::DeleteRecordField", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::DeleteField()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::DeleteField (DWORD dwRecordID, RECORD_FIELDS rfStream)
{
    HRESULT hResult = S_OK;
    INDEX_NODE * pNode;
    DWORD *pdwFieldOffSet, *pdwSize;
    RECORD_FIELDS tmpAll = (RECFLD_ALL == rfStream ? RECFLD_PROPERTIES : rfStream);
    BOOL fDeleteNodeFromTree = (RECFLD_ALL == rfStream ? TRUE : FALSE);
    EnterCriticalSection (&m_csObj);
    __try
    {
        hResult = LoadIndexNodes();
        if (!hResult)
        {
            pNode = m_btNodes.Find (dwRecordID);
NextRecordField :
            if (pNode)
            {
                switch (tmpAll)
                {
                    case RECFLD_PROPERTIES :
                        pdwFieldOffSet = &pNode->dwPropsStartOffSet;
                        pdwSize = &pNode->dwPropStreamSize;
                        break;
                    case RECFLD_CHILDREN :
                        pdwFieldOffSet = &pNode->dwChildIDsStartOffSet;
                        pdwSize = &pNode->dwChildStreamSize;
                        break;
                    case RECFLD_PARENTS :
                        pdwFieldOffSet = &pNode->dwParentIDsStartOffSet;
                        pdwSize = &pNode->dwParentStreamSize;
                        break;
                    case RECFLD_NAMES :
                        pdwFieldOffSet = &pNode->dwNamesStartOffSet;
                        pdwSize = &pNode->dwNamesStreamSize;
                        break;
                    case RECFLD_ASSOCIATED :
                        pdwFieldOffSet = &pNode->dwAssociatedStartOffSet;
                        pdwSize = &pNode->dwAssociatedStreamSize;
                        break;
                    default :
                        pNode = NULL;
                        goto NextRecordField;
                        break;
                }
                if (0 != *pdwFieldOffSet)
                {
                    hResult = LockRecord (*pdwFieldOffSet, sizeof(DATA_NODE), FALSE);
                    if (hResult)
                    {
                        hResult = MAPI_E_BUSY;
                    }
                    else
                    {
                        UnlockRecord (*pdwFieldOffSet, sizeof(DATA_NODE));
                        hResult = DeleteStreamRecords (*pdwFieldOffSet);
                        if (!hResult)
                        {
                            *pdwFieldOffSet = 0;
                            *pdwSize = 0;
                        }
                    }
                }
                if (0 == pNode->dwPropsStartOffSet && 0 == pNode->dwChildIDsStartOffSet && 0 == pNode->dwParentIDsStartOffSet)
                {
                    fDeleteNodeFromTree = TRUE;
                }
                if (!hResult && (RECFLD_ALL == rfStream))
                {
                    ++tmpAll;
                    if (RECFLD_ALL != tmpAll)
                    {
                        goto NextRecordField;
                    }
                }
                if (!hResult && fDeleteNodeFromTree)
                {
                    m_btNodes.Delete (pNode->dwNodeID);
                    NodeChanged();
                }
            }
            else
            {
                hResult = MAPI_E_NOT_FOUND;
            }
        }
    }
    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        TraceString1 ("CDataBase::DeleteField: An exception occured. Exception code: %#08x", GetExceptionCode());
        hResult = E_FAIL;
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CDataBase::DeleteField", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::DeleteStreamRecords()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::DeleteStreamRecords (DWORD dwStartOffSet)
{
    DWORD dwIndex;
    FREENODES_HEADER fnhInfo = { 0 };
    DATA_NODE dnData = { 0 };
    HRESULT hResult = LockRecord (OFFSET_FREENODES_HEADER, sizeof(FREENODES_HEADER));
    if (!hResult)
    {
        if (0xFFFFFFFF == SetFilePointer (m_hFile, OFFSET_FREENODES_HEADER, NULL, FILE_BEGIN))
        {
            hResult = HRESULT_FROM_WIN32 (GetLastError());
        }
        else
        {
            if (FALSE == ReadFile (m_hFile, &fnhInfo, sizeof(FREENODES_HEADER), &dwIndex, NULL))
            {
                hResult = HRESULT_FROM_WIN32 (GetLastError());
            }
        }
        if (!hResult)
        {
            dnData.dwOffSetNode = dwStartOffSet;
            while (dnData.dwOffSetNode && !hResult)
            {
                if (EMPTY_OFFSETS_PER_NODE == m_FreeNodes.wCount)
                {
                    if (GetFreeNodeFromPool (m_FreeNodes.dwOffSetNextNode))
                    {
                        hResult = GetNewEmptyNode (fnhInfo, m_FreeNodes.dwOffSetNextNode);
                    }
                    if (!hResult)
                    {
                        AccessData (WRITE, m_FreeNodes);
                        dwIndex = m_FreeNodes.dwOffSetNextNode;
                        ZeroMemory (&m_FreeNodes, sizeof(DATA_NODE));
                        m_FreeNodes.dwOffSetNode = dwIndex;
                        m_FreeNodes.wType = DATA_NODE_FN_POOL;
                    }
                }
                if (!hResult)
                {
                    hResult = AccessData (READ, dnData, TRUE);
                    if (!hResult)
                    {
                        dwIndex = (m_FreeNodes.wCount * sizeof(DWORD));
                        m_FreeNodes.wCount++;
                        ASSERT (dwIndex <= DATA_SIZE);
                        CopyMemory (&m_FreeNodes.bRawData[dwIndex], &dnData.dwOffSetNode, sizeof(DWORD));
                        dwIndex = dnData.dwOffSetNextNode;
                        dnData.wType = DATA_NODE_EMPTY_NODE;
                        dnData.wSize = 0;
                        dnData.dwOffSetNextNode = 0;
                        AccessData (WRITE, dnData);
                        dnData.dwOffSetNode = dwIndex;
                    }
                }
            }
            m_fFreeNodePoolChanged = TRUE;
            hResult = S_OK;
            SetFilePointer (m_hFile, OFFSET_FREENODES_HEADER, NULL, FILE_BEGIN);
            WriteFile (m_hFile, &fnhInfo, sizeof(FREENODES_HEADER), &dwIndex, NULL);
        }
        UnlockRecord (OFFSET_FREENODES_HEADER, sizeof(FREENODES_HEADER));
    }
    TraceResult ("CDataBase::DeleteStreamRecords", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::GetNewEmptyNode()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::GetNewEmptyNode (FREENODES_HEADER & fnhInfo, DWORD & dwNewOffSet)
{
    dwNewOffSet = fnhInfo.dwOffSetNextFreeNode;
    fnhInfo.dwOffSetNextFreeNode += sizeof(DATA_NODE);
    HRESULT hResult = S_OK;
    if (0xFFFFFFFF == SetFilePointer (m_hFile, fnhInfo.dwOffSetNextFreeNode, NULL, FILE_BEGIN))
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
    }
    else
    {
        SetEndOfFile (m_hFile);
    }
    TraceResult ("CDataBase::GetNewEmptyNode", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::LoadFreeNodesPool()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::LoadFreeNodesPool()
{
    HRESULT hResult = S_OK;
    if (FALSE == m_fGotFreeNodePool)
    {
        ZeroMemory (&m_FreeNodes, sizeof(DATA_NODE));
        m_FreeNodes.dwOffSetNode = m_FreeNodes.dwOffSetNextNode = OFFSET_FIRST_FREE_NODES_POOL;
        while (!hResult && m_FreeNodes.dwOffSetNextNode)
        {
            hResult = AccessData (READ, m_FreeNodes, FALSE);
            if (m_FreeNodes.dwOffSetNextNode)
            {
                m_FreeNodes.dwOffSetNode = m_FreeNodes.dwOffSetNextNode;
            }
        }
        if (!hResult)
        {
            m_fGotFreeNodePool = TRUE;
        }
        TraceResult ("CDataBase::LoadFreeNodesPool", hResult);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::GetFreeNodeFromPool()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::GetFreeNodeFromPool (DWORD & dwNewOffSet)
{
    HRESULT hResult = S_OK;
    WORD wIndex;
    if (0 == m_FreeNodes.wCount && (OFFSET_FIRST_FREE_NODES_POOL != m_FreeNodes.dwOffSetNode))
    {
        if (m_fFreeNodePoolChanged)
        {
            hResult = AccessData (WRITE, m_FreeNodes);
            if (!hResult)
            {
                m_fFreeNodePoolChanged = FALSE;
            }
        }
        if (!hResult)
        {
            m_fGotFreeNodePool = FALSE;
        }
    }
    if (!hResult)
    {
        hResult = LoadFreeNodesPool();
    }
    if (!hResult)
    {
        if (m_FreeNodes.wCount)
        {
            m_FreeNodes.wCount--;
            #pragma warning (disable : 4244)
            wIndex = m_FreeNodes.wCount*sizeof(DWORD);
            #pragma warning (default : 4100)
            CopyMemory (&dwNewOffSet, &m_FreeNodes.bRawData[wIndex], sizeof(DWORD));
            ZeroMemory (&m_FreeNodes.bRawData[wIndex], sizeof(DWORD));
            ASSERT (dwNewOffSet);
            if (!dwNewOffSet)
            {
                hResult = MAPI_E_CORRUPT_DATA;
            }
            m_fFreeNodePoolChanged = TRUE;
        }
        else
        {
            hResult = MAPI_E_NOT_FOUND;
        }
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::GetNodeStream()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::GetNodeStream (DWORD         dwRecordID,
                                       RECORD_FIELDS rfStream,
                                       LPSTREAM *    ppStream,
                                       BOOL          fCreateIfNotFound)
{
    #ifdef USE_MEMORY_STREAM
    CMemoryStream * pNewStream;
    #else
    CStream * pNewStream;
    #endif 
    INDEX_NODE * pNode;
    DWORD * pdwFieldOffSet, *pdwSize;
    EnterCriticalSection (&m_csObj);
    HRESULT hResult = LoadIndexNodes();
    if (!hResult)
    {
        pNode = m_btNodes.Find (dwRecordID);
        if (!pNode)
        {
            if (fCreateIfNotFound)
            {
                INDEX_NODE NewNode = { 0 };
                NewNode.dwNodeID = dwRecordID;
                try
                {
                    pNode = m_btNodes.Insert (NewNode);
                }
                catch (CException & Exception)
                {
                    hResult = Exception.GetError();
                    pNode = NULL;
                }
            }
            else
            {
                hResult = MAPI_E_NOT_FOUND;
            }
        }
        if (!hResult)
        {
            switch (rfStream)
            {
                case RECFLD_PROPERTIES :
                    pdwFieldOffSet = &pNode->dwPropsStartOffSet;
                    pdwSize = &pNode->dwPropStreamSize;
                    break;
                case RECFLD_CHILDREN :
                    pdwFieldOffSet = &pNode->dwChildIDsStartOffSet;
                    pdwSize = &pNode->dwChildStreamSize;
                    break;
                case RECFLD_PARENTS :
                    pdwFieldOffSet = &pNode->dwParentIDsStartOffSet;
                    pdwSize = &pNode->dwParentStreamSize;
                    break;
                case RECFLD_NAMES :
                    pdwFieldOffSet = &pNode->dwNamesStartOffSet;
                    pdwSize = &pNode->dwNamesStreamSize;
                    break;
                case RECFLD_ASSOCIATED :
                    pdwFieldOffSet = &pNode->dwAssociatedStartOffSet;
                    pdwSize = &pNode->dwAssociatedStreamSize;
                    break;
                default :
                    hResult = E_INVALIDARG;
            }
            if (!hResult)
            {
                if (0 == *pdwFieldOffSet)
                {
                    if (fCreateIfNotFound)
                    {
                        hResult = ReadWriteFreeNodeHeader (FNH_GET_FREE_BLOCK, NULL, pdwFieldOffSet);
                    }
                    else
                    {
                        hResult = MAPI_E_NOT_FOUND;
                    }
                }
            }
            if (!hResult)
            {
                hResult = LockRecord (*pdwFieldOffSet, sizeof(DATA_NODE));
                if (!hResult)
                {
                    try
                    {
                        #ifdef USE_MEMORY_STREAM
                        pNewStream = new CMemoryStream (this, *pdwFieldOffSet, pdwSize);
                        #else
                        pNewStream = new CStream (this, *pdwFieldOffSet, pdwSize);
                        #endif
                        if (pNewStream)
                        {
                            *ppStream = (LPSTREAM)pNewStream;
                        }
                        else
                        {
                            hResult = E_OUTOFMEMORY;
                        }
                    }
                    catch (CException & Exception)
                    {
                        hResult = Exception.GetError();
                    }
                    if (hResult)
                    {
                        UnlockRecord (*pdwFieldOffSet, sizeof(DATA_NODE));
                    }
                }
            }
        }
    }
    LeaveCriticalSection (&m_csObj);
#ifdef _DEBUG
    if (hResult && MAPI_E_NOT_FOUND != hResult)
    {
        TraceResult ("CDataBase::GetNodeStream", hResult);
    }
#endif // _DEBUG
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::LoadIndexNodes()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::LoadIndexNodes()
{
    HRESULT hResult = S_OK;
    if (m_fGotTreeNodes)
    {
        return hResult;
    }
    DATA_NODE dnData = { 0 };
    INDEX_NODE Node = { 0 };
    DWORD dwIndex = 0;
RestartLock:
    hResult = LockRecord (OFFSET_FIRST_INDEX_RECORD, sizeof(DATA_NODE), FALSE);
    if (hResult)
    {
        if (HRESULT_FROM_WIN32(ERROR_LOCK_VIOLATION) == hResult)
        {
            InfoTraceString1 ("Process %d does not have the index. Setting global event...", GetCurrentProcessId());
            PulseEvent (m_hReleaseTreeEvent);
            WaitForSingleObject (m_hTreeReleasedEvent, 1000);
            goto RestartLock;
        }
    }
    else
    {
        hResult = LoadFreeNodesPool();
        if (hResult)
        {
            goto ErrorExit;
        }
        if (0xFFFFFFFF == SetFilePointer (m_hFile, OFFSET_FIRST_INDEX_RECORD, NULL, FILE_BEGIN))
        {
            hResult = HRESULT_FROM_WIN32 (GetLastError());
        }
        else
        {
            dnData.dwOffSetNode = OFFSET_FIRST_INDEX_RECORD;
            do
            {
                hResult = AccessData (READ, dnData, FALSE);
                if (!hResult)
                {
                    ASSERT (DATA_NODE_INDEX == dnData.wType);
                    dwIndex = 0;
                    while (dnData.wCount)
                    {
                        CopyMemory (&Node, &dnData.bRawData[dwIndex], sizeof(INDEX_NODE));
                        dwIndex += sizeof(INDEX_NODE);
                        ASSERT (dwIndex <= DATA_SIZE);
                        try
                        {
                            m_btNodes.Insert(Node);
                        }
                        catch (CException & Exception)
                        {
                            hResult = Exception.GetError();
                        }
                        dnData.wCount--;
                    }
                    dwIndex = dnData.dwOffSetNextNode;
                    ZeroMemory (&dnData, sizeof(DATA_NODE));
                    dnData.dwOffSetNode = dwIndex;
                }
            } while (!hResult && dnData.dwOffSetNode);
            if (hResult)
            {
                m_btNodes.DeleteAll();
            }
        }
ErrorExit:
        if (hResult)
        {
            m_fGotFreeNodePool = FALSE;
            UnlockRecord (OFFSET_FIRST_INDEX_RECORD, sizeof(DWORD));
        }
        else
        {
            m_fGotTreeNodes = TRUE;
            m_fGotFreeNodePool = TRUE;
            ResumeThread (m_hSaveThread);
            InfoTrace ("LoadIndexNodes: Background save thread RESUMED");
        }
    }
    TraceResult ("CDataBase::LoadIndexNodes", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::SaveIndexNodes()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::SaveIndexNodes()
{
    InfoTraceString1 ("CDataBase::SaveIndexNodes method called on thread ID %d", GetCurrentThreadId());
    HRESULT hResult = S_OK;
    if (TRUE == m_fGotTreeNodes && TRUE == m_fNodesChanged)
    {
        if (0xFFFFFFFF == SetFilePointer (m_hFile, OFFSET_FIRST_INDEX_RECORD, NULL, FILE_BEGIN))
        {
            hResult = HRESULT_FROM_WIN32 (GetLastError());
        }
        else
        {
            DATA_NODE dnData = { 0 };
            INDEX_NODE * pNode = (INDEX_NODE * )TRUE; // To get into the WHILE () loop;
            DWORD dwIndex;
            BOOL fReadNextNode = TRUE, fNodeFlushed = FALSE;
            dnData.dwOffSetNode = OFFSET_FIRST_INDEX_RECORD;
            m_btNodes.Walk (TRUE);
            while (pNode && !hResult)
            {
                dwIndex = 0;
                fNodeFlushed = FALSE;
                pNode = m_btNodes.Walk();
                if (fReadNextNode)
                {
                    hResult = AccessData (READ, dnData, FALSE);
                }
                if (!hResult)
                {
                    dnData.wCount = 0;
                    ZeroMemory (dnData.bRawData, DATA_SIZE);
                    while (pNode && (MAX_INDEX_PER_DATA_NODE > dnData.wCount))
                    {
                        CopyMemory (&dnData.bRawData[dwIndex], pNode, sizeof(INDEX_NODE));
                        dwIndex += sizeof(INDEX_NODE);
                        ASSERT (dwIndex <= DATA_SIZE);
                        dnData.wCount++;
                        ASSERT (dnData.wCount <= MAX_INDEX_PER_DATA_NODE);
                        pNode = m_btNodes.Walk();
                        if (MAX_INDEX_PER_DATA_NODE == dnData.wCount)
                        {
                            if (pNode && (0 == dnData.dwOffSetNextNode))
                            {
                                hResult = ReadWriteFreeNodeHeader (FNH_GET_FREE_BLOCK, NULL, &dnData.dwOffSetNextNode);
                                fReadNextNode = FALSE;
                            }
                            hResult = AccessData (WRITE, dnData);
                            dnData.dwOffSetNode = dnData.dwOffSetNextNode;
                            dnData.dwOffSetNextNode = 0;
                            fNodeFlushed = TRUE;
                        }
                    }
                }
            }
            if (!hResult && !fNodeFlushed)
            {
                hResult = AccessData (WRITE, dnData, FALSE);
            }
        }
        if (!hResult)
        {
            m_fNodesChanged = FALSE;
        }
        TraceResult ("CDataBase::SaveIndexNodes", hResult);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::SaveFreeNodesPool()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::SaveFreeNodesPool()
{
    HRESULT hResult = S_OK;
    if (TRUE == m_fGotTreeNodes && TRUE == m_fFreeNodePoolChanged)
    {
        hResult = AccessData (WRITE, m_FreeNodes);
        if (!hResult)
        {
            m_fFreeNodePoolChanged = FALSE;
        }
        TraceResult ("CDataBase::SaveIndexNodes", hResult);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CommonOpenFile()
//
//    Parameters
//       //!! LT pszName and pszComment can be NULL if opening an existing file
//       //!! i.e. fCreateIt == FALSE. This way you can make pszName,Comment IN or OUT
//       and not have to worry about allocating buffers big enough.
//    Purpose
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CommonOpenFile (LPTSTR       pszFilePath,
                             LPTSTR       pszName,
                             LPTSTR       pszComment,
                             MAPIUID &    UID,
                             CDataBase ** ppDBobj,
                             BOOL         fCreateIt)
{
    FILE_HEADER      fhInfo = { 0 };
    FREENODES_HEADER fnhInfo = { 0 };
    DATA_NODE        DataNode = { 0 };
    DWORD dwBytes;
    HRESULT hResult = S_OK;

    if ((!pszName || !pszComment) && fCreateIt)
    {
        TraceMessage ("CommonOpenFile: NULL name and comment pointer for a NEW file");
        return E_INVALIDARG;
    }

    #ifndef USE_VM_ALLOCS
    if (!ghDatabaseObjs)
    {
        ghDatabaseObjs = HeapCreate (0, 8192, 0);
    }
    #endif // USE_VM_ALLOCS

    HANDLE hFile = CreateFile (pszFilePath,
                               GENERIC_WRITE | GENERIC_READ,
                               FILE_SHARE_READ | FILE_SHARE_WRITE,
                               NULL,
                               fCreateIt ? CREATE_NEW : OPEN_EXISTING,
                               FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH | FILE_FLAG_RANDOM_ACCESS,
                               NULL);
    if (INVALID_HANDLE_VALUE == hFile)
    {
        hResult = HRESULT_FROM_WIN32 (GetLastError());
    }
    else
    {
        if (fCreateIt)
        {
            // Write the database file header information
            fhInfo.dwVersion = FILE_HEADER_VERSION;
            fhInfo.dwFileFlags = fMapiUnicode ? FILE_HEADER_UNICODE : 0;
            fhInfo.FileUID = UID;
            lstrcpy ((LPSTR)fhInfo.abDataBaseName, pszName);
            lstrcpy ((LPSTR)fhInfo.abDataBaseComment, pszComment);
            SetFilePointer (hFile, OFFSET_FILE_HEADER, NULL, FILE_BEGIN);
            WriteFile (hFile, &fhInfo, sizeof(FILE_HEADER), &dwBytes, NULL);
            ASSERT (sizeof(FILE_HEADER) == dwBytes);
            
            // Write where the next free data record can be created
            fnhInfo.dwNextID = 1;
            fnhInfo.dwOffSetNextFreeNode = OFFSET_FIRST_FREE_DATA_RECORD;
            fnhInfo.dwOffSetDeletedNodes = OFFSET_FIRST_FREE_NODES_POOL;
            SetFilePointer (hFile, OFFSET_FREENODES_HEADER, NULL, FILE_BEGIN);
            WriteFile (hFile, &fnhInfo, sizeof(FREENODES_HEADER), &dwBytes, NULL);
            ASSERT (sizeof(FREENODES_HEADER) == dwBytes);

            // Write the first index record
            DataNode.wType = DATA_NODE_INDEX;
            DataNode.dwOffSetNode = OFFSET_FIRST_INDEX_RECORD;
            SetFilePointer (hFile, OFFSET_FIRST_INDEX_RECORD, NULL, FILE_BEGIN);
            WriteFile (hFile, &DataNode, sizeof(DATA_NODE), &dwBytes, NULL);
            ASSERT (sizeof(DATA_NODE) == dwBytes);

            // Write the first free nodes pool
            DataNode.wType = DATA_NODE_FN_POOL;
            DataNode.dwOffSetNode = OFFSET_FIRST_FREE_NODES_POOL;
            SetFilePointer (hFile, OFFSET_FIRST_FREE_NODES_POOL, NULL, FILE_BEGIN);
            WriteFile (hFile, &DataNode, sizeof(DATA_NODE), &dwBytes, NULL);
            ASSERT (sizeof(DATA_NODE) == dwBytes);
        }
        else
        {
            // Read the database file header information
            ReadFile (hFile, &fhInfo, sizeof(FILE_HEADER), &dwBytes, NULL);
            ASSERT (sizeof(FILE_HEADER) == dwBytes);
            if (FILE_HEADER_VERSION != fhInfo.dwVersion)
            {
                hResult = MAPI_E_VERSION;
            }
            else
            {
                CopyMemory (&UID, &fhInfo.FileUID, sizeof(MAPIUID));
                if (pszName)
                {
                    lstrcpy (pszName, (LPTSTR)fhInfo.abDataBaseName);
                }
                if (pszComment)
                {
                    lstrcpy (pszComment, (LPTSTR)fhInfo.abDataBaseComment);
                }
            }
        }
        if (!hResult)
        {
            try
            {
                *ppDBobj = new CDataBase (UID, hFile);
                if (NULL == *ppDBobj)
                {
                    hResult = E_OUTOFMEMORY;
                }
            }
            catch (CException & Exception)
            {
                hResult = Exception.GetError();
            }
        }
    }
    if (hResult)
    {
        CloseHandle (hFile);
        if (fCreateIt)
        {
            DeleteFile (pszFilePath);
        }
        TraceResult ("CommonOpenFile", hResult);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CDataBase::SetDataBaseStrings()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
STDMETHODIMP CDataBase::SetDataBaseStrings (LPSTR szName, LPSTR szComment)
{
    DWORD dwBytes;
    FILE_HEADER fhInfo = { 0 };
    EnterCriticalSection (&m_csObj);
    HRESULT hResult = LockRecord (OFFSET_FILE_HEADER, sizeof(FILE_HEADER));
    if (!hResult)
    {
        if (0xFFFFFFFF == SetFilePointer (m_hFile, OFFSET_FILE_HEADER, NULL, FILE_BEGIN))
        {
            hResult = HRESULT_FROM_WIN32 (GetLastError());
        }
        else
        {
            if (ReadFile (m_hFile, &fhInfo, sizeof(FILE_HEADER), &dwBytes, NULL))
            {
                if (szName)
                {
                    lstrcpy ((LPSTR)fhInfo.abDataBaseName, szName);
                }
                if (szComment)
                {
                    lstrcpy ((LPSTR)fhInfo.abDataBaseComment, szComment);
                }
                if (0xFFFFFFFF == SetFilePointer (m_hFile, OFFSET_FILE_HEADER, NULL, FILE_BEGIN))
                {
                    hResult = HRESULT_FROM_WIN32 (GetLastError());
                }
                else
                {
                    if (!WriteFile (m_hFile, &fhInfo, sizeof(FILE_HEADER), &dwBytes, NULL))
                    {
                        hResult = HRESULT_FROM_WIN32 (GetLastError());
                    }
                }
            }
            else
            {
                hResult = HRESULT_FROM_WIN32 (GetLastError());
            }
        }
        UnlockRecord (OFFSET_FILE_HEADER, sizeof(FILE_HEADER));
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CDataBase::SetDataBaseStrings", hResult);
    return hResult;
}

#ifdef USE_MEMORY_STREAM

///////////////////////////////////////////////////////////////////////////////
//    CMemoryStream::CMemoryStream()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
CMemoryStream::CMemoryStream (CDataBase * pDBobj, DWORD dwStartOffSet, DWORD * pdwSize)
{
    InfoTrace ("CMemoryStream: Constructor called");

    HRESULT hResult = CreateStreamOnHGlobal (NULL, TRUE, &m_pImpStream);
    if (hResult)
    {
        TraceResult ("CMemoryStream: Failed to create memory stream", hResult);
        throw CException(hResult);
    }

    m_fChanged = FALSE;
    m_cRef = 1;
    m_dwStartOffSet = dwStartOffSet;
    m_pDBobj = pDBobj;
    m_pDBobj->AddRef();
    m_pdwSize = pdwSize;

    // We save the offsets of the nodes after the first link so that if
    // we have to commit, we would use the same nodes for writing without
    // having to walk the node again to find out the next link on the chain.
    DWORD iCache = 0;
    ZeroMemory (m_rgdwCachedOffSets, sizeof(m_rgdwCachedOffSets));
    m_dwDeleteAfter = 0;
    
    DWORD dwSize = 0;
    DATA_NODE dnData = { 0 };
    dnData.dwOffSetNode = dwStartOffSet;
    do
    {
        hResult = m_pDBobj->AccessData (READ, dnData);
        if (!hResult)
        {
            dnData.dwOffSetNode = dnData.dwOffSetNextNode;
            if (iCache < MEMSTRM_CACHE_NODE_OFFSETS)
            {
                m_rgdwCachedOffSets[iCache] = dnData.dwOffSetNextNode;
                if (m_rgdwCachedOffSets[iCache])
                {
                    iCache++;
                }
            }
            else
            {
                if (!m_dwDeleteAfter)
                {
                    m_dwDeleteAfter = dnData.dwOffSetNextNode;
                }
            }
            if (dnData.wSize)
            {
                hResult = m_pImpStream->Write (dnData.bRawData, dnData.wSize, NULL);
                dwSize += dnData.wSize;
            }
        }
    }
    while (!hResult && dnData.dwOffSetNode);
    LARGE_INTEGER liZero = { 0 };
    m_pImpStream->Seek (liZero, STREAM_SEEK_SET, NULL);
    if (hResult)
    {
        m_pImpStream->Release();
        m_pImpStream = NULL;
        throw CException(hResult);
    }
    InitializeCriticalSection (&m_csObj);
}

///////////////////////////////////////////////////////////////////////////////
//    CMemoryStream::~CMemoryStream()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
CMemoryStream::~CMemoryStream()
{
    InfoTrace ("CMemoryStream: Destructor called");
    if (m_pImpStream)
    {
        if (m_fChanged)
        {
            CommitStreamToDatabase();
        }
        m_pImpStream->Release();
        m_pImpStream = NULL;
    }
    UnlockFile (m_pDBobj->GetFileHandle(), m_dwStartOffSet , 0, sizeof(DATA_NODE), 0);
    m_pDBobj->Release();
    m_pDBobj = NULL;
    DeleteCriticalSection (&m_csObj);
    m_dwStartOffSet = 0;
    m_pdwSize = NULL;
}

///////////////////////////////////////////////////////////////////////////////
//    CMemoryStream::QueryInterface()
//
//    Parameters
//      { Refer to the OLE documentation for parameter description }
//
//    Purpose
//      Returns a pointer to a interface requested if the interface is 
//      supported and implemented by this object. If it is not supported, it
//      returns NULL
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMemoryStream::QueryInterface (REFIID riid, LPVOID * ppvObj)
{
    // OLE requires NULLing parameter
    *ppvObj = NULL;
    // If the interface requested is supported by this object, return a pointer
    // to ourselves, with the reference count incremented by one.
    if (riid == IID_IStream || riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    // This object does not support the interface requested
    return E_NOINTERFACE;
}

///////////////////////////////////////////////////////////////////////////////
//    CMemoryStream::CommitStreamToDatabase()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMemoryStream::CommitStreamToDatabase()
{
    LARGE_INTEGER liZero = { 0 };
    STATSTG stat = { 0 };
    HRESULT hResult = m_pImpStream->Stat (&stat, STATFLAG_NONAME);
    if (!hResult)
    {
        hResult = m_pImpStream->Seek (liZero, STREAM_SEEK_SET, NULL);
    }
    if (hResult)
    {
        TraceResult ("CMemoryStream::CommitStreamToDatabase: Stat & Seek", hResult);
        return hResult;
    }
    ULONG ulBytesRead, ulBytesLeft = stat.cbSize.LowPart, ulTotalRead = 0;
    DATA_NODE dnData;
    DWORD dwNextOffset = m_dwStartOffSet, iCache = 0;
    m_pDBobj->LockObject();
    do
    {
        ulBytesRead = 0;
        ZeroMemory (&dnData, sizeof(dnData));
        hResult = m_pImpStream->Read (dnData.bRawData, DATA_SIZE, &ulBytesRead);
        if (hResult)
        {
            goto ExitNow;
        }
        ASSERT (ulBytesRead <= DATA_SIZE);
        ulBytesLeft -= ulBytesRead;
        ulTotalRead += ulBytesRead;

        dnData.wType = DATA_NODE_DATA;
        dnData.wSize = (WORD)ulBytesRead;
        dnData.dwOffSetNode = dwNextOffset;

        if (ulBytesLeft)
        {
            if (iCache < MEMSTRM_CACHE_NODE_OFFSETS && m_rgdwCachedOffSets[iCache])
            {
                dwNextOffset = m_rgdwCachedOffSets[iCache];
                iCache++;
            }
            else
            {
                hResult = m_pDBobj->ReadWriteFreeNodeHeader (FNH_GET_FREE_BLOCK, NULL, &dwNextOffset);
                
            }
            dnData.dwOffSetNextNode = dwNextOffset;
            if (hResult)
            {
                goto ExitNow;
            }
        }
        else
        {
            if (iCache < MEMSTRM_CACHE_NODE_OFFSETS && m_rgdwCachedOffSets[iCache])
            {
                m_pDBobj->DeleteStreamRecords (m_rgdwCachedOffSets[iCache]);
            }
        }
        hResult = m_pDBobj->AccessData (WRITE, dnData);
    } while (!hResult && ulBytesLeft);
    if (!hResult)
    {
        *m_pdwSize = ulTotalRead;
        m_pDBobj->NodeChanged();
    }
    if (m_dwDeleteAfter)
    {
        m_pDBobj->DeleteStreamRecords (m_dwDeleteAfter);
    }

ExitNow:
    m_pDBobj->UnlockObject();
    TraceResult ("CMemoryStream::CommitStreamToDatabase", hResult);
    return hResult;
}

#else // USE_MEMORY_STREAM

///////////////////////////////////////////////////////////////////////////////
//    CStream::CStream()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
CStream::CStream (CDataBase * pDBobj, DWORD dwStartOffSet, DWORD * pdwSize)
{
    InfoTrace ("CStream: Constructor called");
    m_cRef = 1;
    m_dwStartOffSet = dwStartOffSet;
    m_pDBobj = pDBobj;
    m_pDBobj->AddRef();
    m_pdwSize = pdwSize;
    m_dwCurPos = 0;

    DATA_NODE dnData = { 0 };
    dnData.dwOffSetNode = dwStartOffSet;
    HRESULT hResult = pDBobj->AccessData (READ, dnData);
    if (!hResult)
    {
        dnData.wType = DATA_NODE_DATA;
        dnData.dwOffSetNode = dwStartOffSet;
        hResult = pDBobj->AccessData (WRITE, dnData);
    }
    if (hResult)
    {
        throw CException(hResult);
    }
    InitializeCriticalSection (&m_csObj);
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::~CStream()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
CStream::~CStream()
{
    InfoTrace ("CStream: Destructor called");
    UnlockFile (m_pDBobj->GetFileHandle(), m_dwStartOffSet , 0, sizeof(DATA_NODE), 0);
    m_pDBobj->Release();
    m_pDBobj = NULL;
    DeleteCriticalSection (&m_csObj);
    m_dwStartOffSet = 0;
    m_dwCurPos = 0;
    m_pdwSize = NULL;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::QueryInterface()
//
//    Parameters
//      { Refer to the OLE documentation for parameter description }
//
//    Purpose
//      Returns a pointer to a interface requested if the interface is 
//      supported and implemented by this object. If it is not supported, it
//      returns NULL
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CStream::QueryInterface (REFIID riid, LPVOID * ppvObj)
{
    // OLE requires NULLing parameter
    *ppvObj = NULL;
    // If the interface requested is supported by this object, return a pointer
    // to ourselves, with the reference count incremented by one.
    if (riid == IID_IStream || riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    // This object does not support the interface requested
    return E_NOINTERFACE;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::LoadCurrentBlock()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
STDMETHODIMP CStream::LoadCurrentBlock (DATA_NODE & dnData)
{
    HRESULT hResult;
    dnData.dwOffSetNode = m_dwStartOffSet;
    DWORD dwBlock = (m_dwCurPos / DATA_SIZE) + 1;
    do
    {
        hResult = m_pDBobj->AccessData (READ, dnData, (1 == dwBlock ? FALSE : TRUE));
        if (dwBlock != 1 && dnData.dwOffSetNextNode)
        {
            dnData.dwOffSetNode = dnData.dwOffSetNextNode;
        }
        dwBlock--;
    } while (dwBlock && !hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::Write()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
STDMETHODIMP CStream::Write (const void * pv,
                             ULONG        cb,
                             ULONG *      pcbWritten)
{
    InfoTrace ("CStream::Write method called");
    HRESULT hResult = S_OK;
    DWORD cbSpaceLeft, dwCopy, dwIndex, dwOldSize;
    DWORD dwCopied = 0, dwAbsoluteOffSet;
    WORD wBlockSize = 0;
    LPBYTE pBytes = (LPBYTE)pv;
    DATA_NODE dnData = { 0 };
    BOOL fDirty = FALSE, fReadNextBlock;
    EnterCriticalSection (&m_csObj);
    m_pDBobj->LockObject();
    __try
    {
        hResult = LoadCurrentBlock (dnData);
        dwAbsoluteOffSet = (m_dwCurPos / DATA_SIZE) * DATA_SIZE;
        while (cb && !hResult)
        {
            dwIndex = m_dwCurPos - dwAbsoluteOffSet;
            cbSpaceLeft = DATA_SIZE - dwIndex;
            if (DATA_SIZE == dnData.wSize && (m_dwCurPos == dwAbsoluteOffSet))
            {
                cbSpaceLeft = 0;
            }
            dwCopy = min (cbSpaceLeft, cb);
            cb -= dwCopy;
            if (0 == cbSpaceLeft)
            {
                // Flush ...
                if (cb)
                {
                    // If the node that we are writting does not have a link to a
                    // next block, we need to get a link to the next FREE block BEFORE
                    // writting the current node to the disk.
                    // If the node has an a NEXT BLOCK offset address, it means that
                    // we are writing across the boundaries of two nodes
                    // in the data base, so we need to read the next block to overwrite
                    // what was there.
                    if (0 == dnData.dwOffSetNextNode)
                    {
                        fReadNextBlock = FALSE;
                        hResult = m_pDBobj->ReadWriteFreeNodeHeader (FNH_GET_FREE_BLOCK, NULL, &dnData.dwOffSetNextNode);
                    }
                    else
                    {
                        fReadNextBlock = TRUE;
                    }
                }
                if (!hResult)
                {
                    ASSERT (dnData.wSize <= DATA_SIZE);
                    hResult = m_pDBobj->AccessData (WRITE, dnData);
                    if (!hResult)
                    {
                        if (fReadNextBlock)
                        {
                            dnData.dwOffSetNode = dnData.dwOffSetNextNode;
                            hResult = m_pDBobj->AccessData (READ, dnData);
                            wBlockSize = dnData.wSize;
                        }
                        else
                        {
                            dnData.dwOffSetNode = dnData.dwOffSetNextNode;
                            dnData.dwOffSetNextNode = 0;
                            wBlockSize = dnData.wSize = 0;
                            ZeroMemory (dnData.bRawData, DATA_SIZE);
                        }
                        fDirty = FALSE;
                        dwAbsoluteOffSet = (m_dwCurPos / DATA_SIZE) * DATA_SIZE;
                    }
                }
            }
            else
            {
                ASSERT (dwCopy <= DATA_SIZE);
                CopyMemory (&dnData.bRawData[dwIndex], &pBytes[dwCopied], dwCopy);
                dwOldSize = dnData.wSize;
                dnData.wSize = (WORD)max ((dwIndex + dwCopy), dnData.wSize);
                if (dwOldSize != dnData.wSize)
                {
                    *m_pdwSize += dwCopy;
                    // Fix the size one time per block when writing data in
                    // between blocks that have data already
                    if (wBlockSize)
                    {
                        *m_pdwSize -= wBlockSize;
                        wBlockSize = 0;
                    }
                }
                m_dwCurPos += dwCopy;
                dwCopied += dwCopy;
                fDirty = TRUE;
            }
        }
        if (!hResult && fDirty)
        {
            // Flush ...
            hResult = m_pDBobj->AccessData (WRITE, dnData);
        }
        if (!hResult)
        {
            if (pcbWritten)
            {
                *pcbWritten = dwCopied;
            }
            if (dwCopied)
            {
                m_pDBobj->NodeChanged();
            }
        }
    }
    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        TraceString1 ("CStream::Write: An exception occured. Exception code: %#08x", GetExceptionCode());
        hResult = E_FAIL;
    }
    m_pDBobj->UnlockObject();
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CStream::Write", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::Read()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
STDMETHODIMP CStream::Read (void * pv, ULONG cb, ULONG * pcbRead)
{
    InfoTrace ("CStream::Read method called");
    HRESULT hResult = S_OK;
    DWORD dwCopy, dwIndex, dwLeftAfterIndex, dwAbsoluteOffSet, dwCopied = 0;
    LPBYTE pBytes = (LPBYTE)pv;
    DATA_NODE dnData = { 0 };
    EnterCriticalSection (&m_csObj);
    __try
    {
        hResult = LoadCurrentBlock (dnData);
        if (!hResult)
        {
            dwAbsoluteOffSet = (m_dwCurPos / DATA_SIZE) * DATA_SIZE;
            while (cb && !hResult)
            {
                dwIndex = m_dwCurPos - dwAbsoluteOffSet;
                ASSERT (dwIndex <= DATA_SIZE);
                dwLeftAfterIndex = dnData.wSize - dwIndex;
                if (0 == dwLeftAfterIndex)
                {
                    break;
                }
                dwCopy = min (dwLeftAfterIndex, cb);
                ASSERT (dwCopy <= DATA_SIZE);
                CopyMemory (&pBytes[dwCopied], &dnData.bRawData[dwIndex], dwCopy);
                cb -= dwCopy;
                m_dwCurPos += dwCopy;
                dwCopied += dwCopy;
                if (cb)
                {
                    dnData.dwOffSetNode = dnData.dwOffSetNextNode;
                    if (0 == dnData.dwOffSetNode)
                    {
                        cb = 0;
                    }
                    else
                    {
                        hResult = m_pDBobj->AccessData (READ, dnData);
                        dwAbsoluteOffSet = (m_dwCurPos / DATA_SIZE) * DATA_SIZE;
                    }
                }
            }
        }
        if (!hResult && pcbRead)
        {
            *pcbRead = dwCopied;
        }
    }
    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        TraceString1 ("CStream::Read: An exception occured. Exception code: %#08x", GetExceptionCode());
        hResult = E_FAIL;
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CStream::Read", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::Seek()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
STDMETHODIMP CStream::Seek (LARGE_INTEGER    dlibMove,
                            DWORD            dwOrigin,
                            ULARGE_INTEGER * plibNewPosition)
{
    InfoTrace ("CStream::Seek method called");
    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    __try
    {
        ASSERT (0 == dlibMove.HighPart);
        switch (dwOrigin)
        {
            case STREAM_SEEK_SET :
                m_dwCurPos = (DWORD)dlibMove.LowPart;
                break;
            case STREAM_SEEK_CUR :
                m_dwCurPos += dlibMove.LowPart;
                break;
            case STREAM_SEEK_END :
                if ((DWORD)dlibMove.LowPart > (*m_pdwSize))
                {
                    m_dwCurPos = 0;
                }
                else
                {
                    m_dwCurPos = (DWORD)((*m_pdwSize) - dlibMove.LowPart);
                }
                break;
            default :
                hResult = STG_E_INVALIDFUNCTION;
                break;
        }
        if (m_dwCurPos > (*m_pdwSize))
        {
            m_dwCurPos = *m_pdwSize;
        }
        if (!hResult && plibNewPosition)
        {
            plibNewPosition->LowPart = m_dwCurPos;
            plibNewPosition->HighPart = 0;
        }
    }
    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        TraceString1 ("CStream::Seek: An exception occured. Exception code: %#08x", GetExceptionCode());
        hResult = STG_E_INVALIDPOINTER;
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CStream::Seek", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::SetSize()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
STDMETHODIMP CStream::SetSize (ULARGE_INTEGER uliNewSize)
{
    InfoTrace ("CStream::SetSize method called");
    if (uliNewSize.HighPart)
    {
        TraceMessage ("CStream::SetSize: The high part is not supported, returning E_INVALIDARG");
        return E_INVALIDARG;
    }
    if (*m_pdwSize == uliNewSize.LowPart)
    {
        return S_OK;
    }
    DATA_NODE dnData = { 0 };
    dnData.dwOffSetNode = m_dwStartOffSet;
    DWORD cbSpaceLeft, dwBytesToAdd, dwBytes = 0, dwOffSetNode, dwBytesToExtend;
    BOOL fDirty = FALSE;
    HRESULT hResult = S_OK;
    BOOL fTrucate = FALSE, fExtend = FALSE;
    EnterCriticalSection (&m_csObj);
    while (dnData.dwOffSetNode && !hResult && dwBytes != uliNewSize.LowPart)
    {
        hResult = m_pDBobj->AccessData (READ, dnData);
        if (!hResult)
        {
            if ((dwBytes + dnData.wSize) > uliNewSize.LowPart)
            {
                fTrucate = TRUE;
                break;
            }
            else
            {
                dwBytes += dnData.wSize;
                if (dnData.dwOffSetNextNode)
                {
                    dnData.dwOffSetNode = dnData.dwOffSetNextNode;
                }
                else
                {
                    fExtend = TRUE;
                    break;
                }
            }
        }
    }
    if (!hResult && fTrucate)
    {
        dwOffSetNode = dnData.dwOffSetNextNode;
        dnData.dwOffSetNextNode = 0;
        dnData.wSize = (WORD)(uliNewSize.LowPart - dwBytes);
        ZeroMemory (&dnData.bRawData[dnData.wSize], DATA_SIZE - dnData.wSize);
        hResult = m_pDBobj->AccessData (WRITE, dnData);
        if (!hResult)
        {
            m_pDBobj->LockObject();
            if (dwOffSetNode)
            {
                m_pDBobj->DeleteStreamRecords (dwOffSetNode);
            }
            *m_pdwSize = uliNewSize.LowPart;
            if (m_dwCurPos > (*m_pdwSize))
            {
                m_dwCurPos = *m_pdwSize;
            }
            m_pDBobj->NodeChanged();
            m_pDBobj->UnlockObject();
        }
    }
    if (!hResult && fExtend)
    {
        dwBytesToExtend = uliNewSize.LowPart - dwBytes;
        m_pDBobj->LockObject();
        while (!hResult && dwBytesToExtend)
        {
            cbSpaceLeft = DATA_SIZE - dnData.wSize;
            dwBytesToAdd = min (cbSpaceLeft, dwBytesToExtend);
            ASSERT (dwBytesToAdd <= DATA_SIZE);
            #pragma warning (disable : 4244)
            dnData.wSize += (WORD)dwBytesToAdd;
            #pragma warning (default : 4244)
            dwBytesToExtend -= dwBytesToAdd;
            if (0 == cbSpaceLeft)
            {
                if (dwBytesToExtend)
                {
                    hResult = m_pDBobj->ReadWriteFreeNodeHeader (FNH_GET_FREE_BLOCK, NULL, &dnData.dwOffSetNextNode);
                }
                if (!hResult)
                {
                    hResult = m_pDBobj->AccessData (WRITE, dnData);
                    if (!hResult)
                    {
                        dnData.dwOffSetNode = dnData.dwOffSetNextNode;
                        dnData.dwOffSetNextNode = 0;
                        dnData.wSize = 0;
                        ZeroMemory (dnData.bRawData, DATA_SIZE);
                        fDirty = FALSE;
                    }
                }
            }
            else
            {
                fDirty = TRUE;
            }
        }
        if (!hResult && fDirty)
        {
            // Flush ...
            hResult = m_pDBobj->AccessData (WRITE, dnData);
        }
        if (!hResult)
        {
            *m_pdwSize = uliNewSize.LowPart;
            m_pDBobj->NodeChanged();
        }
        m_pDBobj->UnlockObject();
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CStream::SetSize", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::CopyTo()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
STDMETHODIMP CStream::CopyTo (IStream *        pstm,
                              ULARGE_INTEGER   cb,
                              ULARGE_INTEGER * pcbRead,
                              ULARGE_INTEGER * pcbWritten)
{
    InfoTrace ("CStream::CopyTo method called");
    HRESULT hResult = S_OK;
    LPBYTE pBytes;
    ULONG ulRead, ulWritten, ulBytesToRead, ulReadTotal = 0, ulWrittenTotal = 0, ulTotalBytes;
    ulTotalBytes = cb.LowPart;
    #define BUFFER_SIZE 4096
    EnterCriticalSection (&m_csObj);
    __try
    {
        pBytes = new BYTE[BUFFER_SIZE];
	    if (NULL == pBytes)
        {
            hResult = E_OUTOFMEMORY;
        }
        if (!hResult)
        {
	        while (!hResult && ulTotalBytes)
        	{
                ulBytesToRead = min (ulTotalBytes, BUFFER_SIZE);
                hResult = Read (pBytes, ulBytesToRead, &ulRead);
                if (!hResult)
                {
        		    hResult = pstm->Write (pBytes, ulRead, &ulWritten);
        		    if (!hResult)
                    {
        		        ulReadTotal += ulRead;
        		        ulWrittenTotal += ulWritten;
                        ulTotalBytes -= ulBytesToRead;
                        if (ulBytesToRead != ulRead)
                        {
			                ulTotalBytes = 0;
                        }
                    }
                }
        	}
        	if (!hResult)
        	{
        		if (pcbRead)
        		{
        			pcbRead->LowPart = ulReadTotal;
        			pcbRead->HighPart = 0;
        		}
        		if (pcbWritten)
        		{
        			pcbWritten->LowPart = ulWrittenTotal;
        			pcbWritten->HighPart = 0;
        		}
        	}
            delete [] pBytes;
        }
    }
    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        TraceString1 ("CStream::CopyTo: An exception occured. Exception code: %#08x", GetExceptionCode());
        hResult = STG_E_INVALIDPOINTER;
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CStream::CopyTo", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::Stat()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
STDMETHODIMP CStream::Stat (STATSTG * pstatstg, DWORD grfStatFlag)
{
    InfoTrace ("CStream::Stat method called");
    if (STATFLAG_NONAME != grfStatFlag)
    {
        TraceMessage ("CStream::Stat: This implementation does not have a NAME, returning E_INVALIDARG");
        return E_INVALIDARG;
    }
    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    __try
    {
        ZeroMemory (pstatstg, sizeof(STATSTG));
        pstatstg->type = STGTY_STREAM;
        pstatstg->cbSize.LowPart = *m_pdwSize;
    }
    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        TraceString1 ("CStream::Stat: An exception occured. Exception code: %#08x", GetExceptionCode());
        hResult = STG_E_INVALIDPOINTER;
    }
    LeaveCriticalSection (&m_csObj);
    TraceResult ("CStream::Stat", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::operator new()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
void * CStream::operator new (size_t size)
{
    #ifdef USE_VM_ALLOCS
    LPVOID pVoid = VirtualAlloc (NULL, (DWORD)size, MEM_COMMIT, PAGE_READWRITE);
    if (pVoid)
    {
        ZeroMemory (pVoid, (DWORD)size);
    }
    return pVoid;
    #else
    return HeapAlloc (ghDatabaseObjs, HEAP_ZERO_MEMORY, (DWORD)size);
    #endif // USE_VM_ALLOCS
}

///////////////////////////////////////////////////////////////////////////////
//    CStream::operator delete()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
void CStream::operator delete (void * pVoid)
{
    #ifdef USE_VM_ALLOCS
    VirtualFree (pVoid, 0, MEM_RELEASE);
    #else
    HeapFree (ghDatabaseObjs, 0, pVoid);
    #endif // USE_VM_ALLOCS
}

#endif // USE_MEMORY_STREAM

///////////////////////////////////////////////////////////////////////////////
//    CBTree::Walk()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
INDEX_NODE * WINAPI CBTree::Walk (BOOL fStart)
{
    INDEX_NODE * pInfo = NULL;
    if (fStart)
    {
        m_pWalk = m_pHead;
    }
    else
    {
        if (m_pWalk)
        {
            pInfo = &(m_pWalk->Info);
            m_pWalk = m_pWalk->pNext;
        }
    }
    return pInfo;
}

///////////////////////////////////////////////////////////////////////////////
//    CBTree::Delete()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
BOOL WINAPI CBTree::Delete (DWORD dwKey)
{
    LINKED_LIST_NODE * pNode = m_pHead;
    while (pNode)
    {
        if (pNode->Info.dwNodeID == dwKey)
        {
            if (pNode == m_pHead)
            {
                m_pHead = m_pHead->pNext;
            }
            if (pNode == m_pCurrent)
            {
                m_pCurrent = m_pHead;
            }
            if (pNode == m_pTail)
            {
                m_pTail = m_pTail->pPrev;
            }
            if (pNode->pPrev)
            {
                pNode->pPrev->pNext = pNode->pNext;
            }
            if (pNode->pNext)
            {
                pNode->pNext->pPrev = pNode->pPrev;
            }
            HeapFree (ghDatabaseObjs, 0, pNode);
            return TRUE;
        }
        pNode = pNode->pNext;
    }
    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//    CBTree::DeleteAll()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
void WINAPI CBTree::DeleteAll()
{
    LINKED_LIST_NODE * pNext;
    while (m_pHead)
    {
        pNext = m_pHead->pNext;
        HeapFree (ghDatabaseObjs, 0, m_pHead);
        m_pHead = pNext;
    }
    m_pHead = m_pCurrent = m_pTail = NULL;
}

///////////////////////////////////////////////////////////////////////////////
//    CBTree::Insert()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
INDEX_NODE * WINAPI CBTree::Insert (const INDEX_NODE & item)
{
    LINKED_LIST_NODE * pNewNode = (LINKED_LIST_NODE *)HeapAlloc (ghDatabaseObjs, HEAP_ZERO_MEMORY, sizeof (LINKED_LIST_NODE));
    if (!pNewNode)
    {
        throw CException (E_OUTOFMEMORY);
    }
    pNewNode->Info = item;
    if (m_pHead)
    {
        m_pTail->pNext = pNewNode;
        pNewNode->pPrev = m_pTail;
        m_pTail = pNewNode;
    }
    else
    {
        m_pHead = m_pCurrent = m_pTail = pNewNode;
    }
    return &(pNewNode->Info);
}

///////////////////////////////////////////////////////////////////////////////
//    CBTree::Find()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
INDEX_NODE * WINAPI CBTree::Find (DWORD dwKey)
{
    if (!m_pHead)
    {
        return NULL;
    }
    ASSERT (m_pCurrent);
    LINKED_LIST_NODE * pNode = m_pCurrent;
    while (pNode)
    {
        if (pNode->Info.dwNodeID == dwKey)
        {
            // For locality
            m_pCurrent = pNode;
            return &(pNode->Info);
        }
        pNode = pNode->pNext;
    }
    pNode = m_pCurrent->pPrev;
    while (pNode)
    {
        if (pNode->Info.dwNodeID == dwKey)
        {
            // For locality
            m_pCurrent = pNode;
            return &(pNode->Info);
        }
        pNode = pNode->pPrev;
    }
    return NULL;
}

// End of file for CDBFILE.CPP
