///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      CDBFILE.H
//              
//  Description
//
//  Author
//      Irving De la Cruz
//
//  Revision: 0.99
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _CDBFILE_H
#define _CDBFILE_H

#include <WINDOWS.H>

#ifdef _DEBUG
    #ifndef     ENABLE_DEBUG_OUTPUT
    #define     ENABLE_DEBUG_OUTPUT     1
    #endif
#endif // _DEBUG

#include "TRACES.H"

#define USE_MEMORY_STREAM
//#define USE_VM_ALLOCS

#ifndef USE_VM_ALLOCS
extern HANDLE ghDatabaseObjs;
#endif // USE_VM_ALLOCS

#ifndef MAPIDEFS_H
// START stuff borrowed from MAPIDEFS.H and MAPICODE.H
#define MAPI_UNICODE            ((ULONG) 0x80000000)

#ifdef UNICODE
#define fMapiUnicode            MAPI_UNICODE
#else
#define fMapiUnicode            0
#endif

typedef struct _MAPIUID
{
    BYTE ab[16];
} MAPIUID, *LPMAPIUID;

#define MAPI_E_BUSY                          0x8004010B
#define MAPI_E_NOT_FOUND                     0x8004010F
#define MAPI_E_VERSION                       0x80040110
#define MAPI_E_CORRUPT_DATA                  0x8004011B
// END stuff borrowed from MAPIDEFS.H and MAPICODE.H
#endif // MAPIDEFS_H

enum
{
    DATA_NODE_UNINITIALIZED,
    DATA_NODE_INDEX,
    DATA_NODE_FN_POOL,
    DATA_NODE_DATA,
    DATA_NODE_EMPTY_NODE
};

#define DATA_SIZE               2048
#define DATA_NODE_HEADER_SIZE   16

typedef struct _DATA_NODE
{
    // 16 bytes
    WORD            wType;
    WORD            wSize;
    WORD            wCount;
    WORD            wReserved;
    DWORD           dwOffSetNode;
    DWORD           dwOffSetNextNode;
    // Blob of data
    BYTE            bRawData[DATA_SIZE];
} DATA_NODE;

typedef enum _RECORD_FIELDS
{
    // Field 0
    RECFLD_PROPERTIES   = 0,    // For All objects is MS and AB providers
    RECFLD_RAWDATA      = 0,    // For OPENED/SAVED properties on IMAPIProp objects
    RECFLD_NAMED_PROPS  = 0,    // For Named Properties on our IMAPIProp implementation
    // Field 1
    RECFLD_CONTENTS     = 1,    // For Message Stores (Folders)
    RECFLD_CHILDREN     = 1,    // For Address Books (Distribution Lists and Containers)
    RECFLD_ATTACHMENTS  = 1,    // For Message Stores (Messages)
    // Field 2
    RECFLD_HIERARCHY    = 2,    // For Message Stores (Folders)
    RECFLD_PARENTS      = 2,    // For Address Books (Mail Users)
    RECFLD_RECIPIENTS   = 2,    // For Message Stores (Messages)
    // Field 3
    RECFLD_ASSOCIATED   = 3,    // For Message Stores (Folders)
    // Field 4
    RECFLD_NAMES        = 4,    // For Message Stores (Folders)
    

    RECFLD_ALL          = 5
} RECORD_FIELDS;

inline void operator++ (RECORD_FIELDS& Field)
{
    Field = (RECORD_FIELDS)((DWORD)Field + 1);
}

#define MAX_INDEX_PER_DATA_NODE     46
typedef struct _INDEX_NODE
{
    DWORD           dwNodeID;
    DWORD           dwPropsStartOffSet;
    DWORD           dwPropStreamSize;
    DWORD           dwChildIDsStartOffSet;
    DWORD           dwChildStreamSize;
    DWORD           dwParentIDsStartOffSet;
    DWORD           dwParentStreamSize;
    DWORD           dwAssociatedStartOffSet;
    DWORD           dwAssociatedStreamSize;
    DWORD           dwNamesStartOffSet;
    DWORD           dwNamesStreamSize;
} INDEX_NODE;

inline BOOL operator== (const INDEX_NODE& Node1, const INDEX_NODE& Node2)
{
    return Node1.dwNodeID == Node2.dwNodeID;
}

inline BOOL operator!= (const INDEX_NODE& Node1, const INDEX_NODE& Node2)
{
    return Node1.dwNodeID != Node2.dwNodeID;
}

inline BOOL operator> (const INDEX_NODE& Node1, const INDEX_NODE& Node2)
{
    return Node1.dwNodeID > Node2.dwNodeID;
}

inline BOOL operator< (const INDEX_NODE& Node1, const INDEX_NODE& Node2)
{
    return Node1.dwNodeID < Node2.dwNodeID;
}

enum
{
    FILE_HEADER_VERSION   = 0x0001,
    FILE_HEADER_UNICODE   = 0x0001
};

typedef struct _FILE_HEADER
{
    DWORD           dwVersion;
    DWORD           dwFileFlags;
    MAPIUID         FileUID;
    BYTE            abDataBaseName[32];
    BYTE            abUnicodePad1[32];
    BYTE            abDataBaseComment[32];
    BYTE            abUnicodePad2[32];
} FILE_HEADER;

#define EMPTY_OFFSETS_PER_NODE  512
typedef struct _FREENODES_HEADER
{
    DWORD           dwNextID;
    DWORD           dwOffSetNextFreeNode;
    DWORD           dwOffSetDeletedNodes;
} FREENODES_HEADER;

enum
{
    OFFSET_FILE_HEADER              = 0,
    OFFSET_FREENODES_HEADER         = 256,
    OFFSET_FIRST_INDEX_RECORD       = 512,
    OFFSET_FIRST_FREE_NODES_POOL    = 2576,
    OFFSET_FIRST_FREE_DATA_RECORD   = 4640,
};

enum
{
    FNH_GET_FREE_BLOCK      = 0x0001,
    FNH_GET_NEXT_RECORD_ID  = 0x0002
};

typedef enum _ACCESS_DATA_ACTION
{
    READ,
    WRITE
} ACCESS_DATA_ACTION;

class CException
{
public:
    CException ()
    {
        m_hResult = E_FAIL;
    }
    CException (HRESULT hResult)
    {
        if (0 == hResult)
        {
            hResult = E_FAIL;
        }
        m_hResult = hResult;
    }
    HRESULT WINAPI GetError()
    {
        return m_hResult;
    }
private:
    HRESULT m_hResult;
};

typedef struct _LINKED_LIST_NODE
{
    struct _LINKED_LIST_NODE *  pNext;
    struct _LINKED_LIST_NODE *  pPrev;
    INDEX_NODE                  Info;
} LINKED_LIST_NODE;

// TODO: This is not a B-Tree class yet but I will get around it in
//       the next revision (if we still use this). The perfomance for
//       this right now is going to suck big time. We'll still be OK
//       because on small DB's
//          IrvingD 4/22/96  
class CBTree
{
///////////////////////////////////////////////////////////////////////////////
// Public members
//
public:
    INDEX_NODE * WINAPI Insert
                    (const INDEX_NODE &         item);
    INDEX_NODE * WINAPI Find
                    (DWORD                      dwKey);
    BOOL WINAPI Delete
                    (DWORD                      dwKey);
    void WINAPI DeleteAll
                    ();
    INDEX_NODE * WINAPI Walk
                    (BOOL                       fStart = FALSE);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public:
    CBTree() { m_pHead = m_pCurrent = m_pTail = m_pWalk = NULL; };
    ~CBTree() { };

///////////////////////////////////////////////////////////////////////////////
// Data members
    LINKED_LIST_NODE *      m_pHead;
    LINKED_LIST_NODE *      m_pTail;
    LINKED_LIST_NODE *      m_pCurrent;
    LINKED_LIST_NODE *      m_pWalk;
};

// Forward declarations
#ifdef USE_MEMORY_STREAM
class CMemoryStream;
#else
class CStream;
#endif
class CDataBase;

DWORD WINAPI ListenThreadProc (CDataBase * pObj);
DWORD WINAPI BackgroundSaveThreadProc (CDataBase * pObj);

class CDataBase
{

#ifdef USE_MEMORY_STREAM
friend class CMemoryStream;
#else
friend class CStream;
#endif

friend DWORD WINAPI ListenThreadProc (CDataBase * pObj);
friend DWORD WINAPI BackgroundSaveThreadProc (CDataBase * pObj);

///////////////////////////////////////////////////////////////////////////////
// Members of this class
//
public :
    STDMETHODIMP SetDataBaseStrings
                    (LPSTR                      szName,
                     LPSTR                      szComment);
    STDMETHODIMP CreateRecord
                    (DWORD *                    pdwRecordID,
                     LPSTREAM *                 ppStream);
    STDMETHODIMP CreateEmptyRecord
                    (DWORD *                    pdwRecordID);
    STDMETHODIMP OpenRecord
                    (DWORD                      dwRecordID,
                     LPSTREAM *                 ppStream);
    STDMETHODIMP OpenRecordField
                    (DWORD                      dwRecordID,
                     BOOL                       fCreateIfNotFound,
                     RECORD_FIELDS              rfStream,
                     LPSTREAM *                 ppStream);
    STDMETHODIMP DeleteRecord
                    (DWORD                      dwRecord);
    STDMETHODIMP DeleteRecordField
                    (DWORD                      dwRecord,
                     RECORD_FIELDS              rfStream);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}

///////////////////////////////////////////////////////////////////////////////
// Other members of this class
public:
    inline HANDLE WINAPI GetFileHandle
                    () { return m_hFile; }
    STDMETHODIMP ReadWriteFreeNodeHeader
                    (DWORD                      dwAction,
                     DWORD *                    pdwNewRecordID,
                     DWORD *                    pdwNewOffSet);
    STDMETHODIMP AccessData
                    (ACCESS_DATA_ACTION         Action,
                     DATA_NODE &                dnData,
                     BOOL                       fReadJustHeader = FALSE);
    STDMETHODIMP LockRecord
                    (DWORD                      dwRecordOffSet,
                     DWORD                      dwRecordSize,
                     BOOL                       fWaitUntilUnlock = TRUE);
    STDMETHODIMP UnlockRecord
                    (DWORD                      dwRecordOffSet,
                     DWORD                      dwRecordSize);
    STDMETHODIMP LoadIndexNodes
                    ();
    STDMETHODIMP SaveIndexNodes
                    ();
    STDMETHODIMP GetNodeStream
                    (DWORD                      dwRecordID,
                     RECORD_FIELDS              rfStream,
                     LPSTREAM *                 ppStream,
                     BOOL                       fCreateIfNotFound);
    STDMETHODIMP DeleteField
                    (DWORD                      dwRecordID,
                     RECORD_FIELDS              rfStream);
    STDMETHODIMP DeleteStreamRecords
                    (DWORD                      dwStartOffSet);
    STDMETHODIMP GetNewEmptyNode
                    (FREENODES_HEADER &         fnhInfo,
                     DWORD &                    dwNewOffSet);
    STDMETHODIMP GetFreeNodeFromPool
                    (DWORD &                    dwNewOffSet);
    STDMETHODIMP LoadFreeNodesPool
                    ();
    STDMETHODIMP SaveFreeNodesPool
                    ();
    void WINAPI NodeChanged
                    () { m_fNodesChanged = TRUE; }
    inline void WINAPI LockObject
                    () { EnterCriticalSection (&m_csObj); }
    inline void WINAPI UnlockObject
                    () { LeaveCriticalSection (&m_csObj); }

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CDataBase       (MAPIUID                    UIDinFile,
                     HANDLE                     hFile);
    ~CDataBase      ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    ULONG                       m_cRef;
    MAPIUID                     m_UID;
    HANDLE                      m_hFile;
    HANDLE                      m_hReleaseTreeEvent;
    HANDLE                      m_hTreeReleasedEvent;
    HANDLE                      m_hSaveEvent;
    HANDLE                      m_hSaveThread;
    HANDLE                      m_hListenThread;
    CRITICAL_SECTION            m_csObj;
    DATA_NODE                   m_FreeNodes;
    BOOL                        m_fNodesChanged;
    BOOL                        m_fFreeNodePoolChanged;
    BOOL                        m_fGotTreeNodes;
    BOOL                        m_fGotFreeNodePool;
    CBTree                      m_btNodes;
};

#ifdef USE_MEMORY_STREAM

class CMemoryStream : public IStream
{
public:
///////////////////////////////////////////////////////////////////////////////
// IUnknown virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
///////////////////////////////////////////////////////////////////////////////
// IStream virtual member functions
//
    inline STDMETHODIMP Read
                    (void *                     pv,
                     ULONG                      cb,
                     ULONG *                    pcbRead)
                     { return m_pImpStream->Read (pv, cb, pcbRead); };
    inline STDMETHODIMP Write
                    (const void *               pv,
                     ULONG                      cb,
                     ULONG *                    pcbWritten)
                     { EnterCriticalSection (&m_csObj);
                       HRESULT hResult = m_pImpStream->Write (pv, cb, pcbWritten);
                       if (!hResult) { m_fChanged = TRUE; } 
                       LeaveCriticalSection (&m_csObj);
                       return hResult; };
    inline STDMETHODIMP Seek
                    (LARGE_INTEGER              dlibMove,
                     DWORD                      dwOrigin,
                     ULARGE_INTEGER *           plibNewPosition)
                     { return m_pImpStream->Seek (dlibMove, dwOrigin, plibNewPosition); };
    inline STDMETHODIMP SetSize
                    (ULARGE_INTEGER             libNewSize)
                     { return m_pImpStream->SetSize (libNewSize); };
    inline STDMETHODIMP CopyTo
                    (IStream *                  pstm,
                     ULARGE_INTEGER cb,
                     ULARGE_INTEGER *           pcbRead,
                     ULARGE_INTEGER *           pcbWritten)
                     { return m_pImpStream->CopyTo (pstm, cb, pcbRead, pcbWritten); };
    inline STDMETHODIMP Commit
                    (DWORD                      grfCommitFlags)
                     { return m_pImpStream->Commit (grfCommitFlags); };
    inline STDMETHODIMP Revert
                    ()
                     { return m_pImpStream->Revert(); };
    inline STDMETHODIMP LockRegion
                    (ULARGE_INTEGER             libOffset,
                     ULARGE_INTEGER             cb,
                     DWORD                      dwLockType)
                     { return m_pImpStream->LockRegion (libOffset, cb, dwLockType); };
    inline STDMETHODIMP UnlockRegion
                    (ULARGE_INTEGER             libOffset,
                     ULARGE_INTEGER             cb,
                     DWORD                      dwLockType)
                     { return m_pImpStream->UnlockRegion (libOffset, cb, dwLockType); };
    inline STDMETHODIMP Stat
                    (STATSTG *                  pstatstg,
                     DWORD                      grfStatFlag)
                     { return m_pImpStream->Stat (pstatstg, grfStatFlag); };
    inline STDMETHODIMP Clone
                    (IStream **                 ppstm)
                     { return m_pImpStream->Clone (ppstm); };

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
private :
    STDMETHODIMP CommitStreamToDatabase
                    ();

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CMemoryStream
                    (CDataBase *                pDBobj,
                     DWORD                      dwStartOffSet,
                     DWORD *                    pdwSize);
    ~CMemoryStream
                    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
public :
    ULONG               m_cRef;
    CDataBase *         m_pDBobj;
    CRITICAL_SECTION    m_csObj;
    LPSTREAM            m_pImpStream;
    DWORD               m_dwStartOffSet;
    DWORD *             m_pdwSize;
    BOOL                m_fChanged;

#define MEMSTRM_CACHE_NODE_OFFSETS  40
    DWORD               m_rgdwCachedOffSets[MEMSTRM_CACHE_NODE_OFFSETS];
    DWORD               m_dwDeleteAfter;
};

#else // USE_MEMORY_STREAM

class CStream : public IStream
{
public:
///////////////////////////////////////////////////////////////////////////////
// IUnknown virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release
                    () { ULONG ulCount = --m_cRef;
                         if (!ulCount) { delete this; }
                         return ulCount;};
///////////////////////////////////////////////////////////////////////////////
// IStorage virtual member functions
//
    STDMETHODIMP Read
                     (void *                    pv,
                      ULONG                     cb,
                      ULONG *                   pcbRead);
    STDMETHODIMP Write
                     (const void *              pv,
                      ULONG                     cb,
                      ULONG *                   pcbWritten);
    STDMETHODIMP Seek
                     (LARGE_INTEGER             dlibMove,
                      DWORD                     dwOrigin,
                      ULARGE_INTEGER *          plibNewPosition);
    STDMETHODIMP SetSize
                    (ULARGE_INTEGER             libNewSize);
    STDMETHODIMP CopyTo
                    (IStream *                  pstm,
                     ULARGE_INTEGER             cb,
                     ULARGE_INTEGER *           pcbRead,
                     ULARGE_INTEGER *           pcbWritten);
    STDMETHODIMP Stat
                    (STATSTG *                  pstatstg,
                     DWORD                      grfStatFlag);
    inline STDMETHODIMP Commit
                    (DWORD                      grfCommitFlags)
                     { return S_OK; };
    inline STDMETHODIMP Revert
                    ()
                     { return STG_E_UNIMPLEMENTEDFUNCTION; };
    inline STDMETHODIMP LockRegion
                    (ULARGE_INTEGER             libOffset,
                     ULARGE_INTEGER             cb,
                     DWORD                      dwLockType)
                     { return STG_E_UNIMPLEMENTEDFUNCTION; };
    inline STDMETHODIMP UnlockRegion
                    (ULARGE_INTEGER             libOffset,
                     ULARGE_INTEGER             cb,
                     DWORD                      dwLockType)
                     { return STG_E_UNIMPLEMENTEDFUNCTION; };
    inline STDMETHODIMP Clone
                    (IStream **                 ppstm)
                     { return STG_E_UNIMPLEMENTEDFUNCTION; };

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    STDMETHODIMP LoadCurrentBlock
                    (DATA_NODE &                dnData);

///////////////////////////////////////////////////////////////////////////////
// Overidden Operators
public :
    static void * operator new
                    (size_t                     size);
    static void operator delete
                    (void *                     pVoid);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CStream     (CDataBase *            pDBobj,
                 DWORD                  dwStartOffSet,
                 DWORD *                pdwSize);
    ~CStream    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
public :
    ULONG               m_cRef;
    CDataBase *         m_pDBobj;
    CRITICAL_SECTION    m_csObj;
    DWORD               m_dwStartOffSet;
    DWORD *             m_pdwSize;
    DWORD               m_dwCurPos;
};

#endif // USE_MEMORY_STREAM

STDMETHODIMP CommonOpenFile (LPTSTR         pszFilePath,
                             LPTSTR         pszName,
                             LPTSTR         pszComment,
                             MAPIUID &      UID,
                             CDataBase **   ppDBobj,
                             BOOL           fCreateIt);

///////////////////////////////////////////////////////////////////////////////
//    OpenDataBaseFile()
//
//    Parameters                        
//
//    Purpose
//      
//    Return Value
//
inline STDMETHODIMP OpenDataBaseFile (LPTSTR       pszFilePath,
                                      LPTSTR       pszName,
                                      LPTSTR       pszComment,
                                      MAPIUID &    UID,
                                      CDataBase ** ppDBobj)
{
    return CommonOpenFile (pszFilePath, pszName, pszComment, UID, ppDBobj, FALSE);
}

///////////////////////////////////////////////////////////////////////////////
//    CreateDataBaseFile()
//
//    Parameters                        
//
//    Purpose
//      
//    Return Value
//
inline STDMETHODIMP CreateDataBaseFile (LPTSTR       pszFilePath,
                                        LPTSTR       pszName,
                                        LPTSTR       pszComment,
                                        MAPIUID      UID,
                                        CDataBase ** ppDBobj)
{
    return CommonOpenFile (pszFilePath, pszName, pszComment, UID, ppDBobj, TRUE);
}

#endif // _CDBFILE_H

// End of file for CDBFILE.H
