///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      CFILESTG.H
//              
//  Description
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Corporation
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//
#ifndef _CFILESTG_H
#define _CFILESTG_H

#include "CDBFILE.H"
#include "BECOMMON.H"

#ifdef UNICODE
#error MSLMS: We don't support UNICODE yet... IrvingD
#endif // UNICODE

// Fixed-ID records used in the data base file
enum
{
    ROOT_FOLDER_ID                  = 0xFFFFFFFF,
    RECEIVE_FOLDER_SETTINGS         = 0xFFFFFFFE,
    OUTGOING_SUBMITTED_MESSAGES     = 0xFFFFFFFD,
    NAMED_PROPERTIES_MAPPING        = 0xFFFFFFFC
};

#define FLDNAME_ADD         0x0001
#define FLDNAME_CHECK       0x0002
#define FLDNAME_REPLACE     0x0004
enum
{
    IPM_SUBTREE_ID          = 1,
    IPM_WASTEBASKET_ID      = 2,
    IPM_INBOX_ID            = 3,
    IPM_OUTBOX_ID           = 4,
    IPM_SENTMAIL_ID         = 5,
    COMMON_VIEWS_FOLDER_ID  = 6,
    VIEWS_FOLDER_ID         = 7,
    FINDER_FOLDER_ID        = 8
};

// Forward declaration
class CFileBackEnd;

extern "C"
{
    HRESULT WINAPI OpenMsgStoreDataBase
                    (LPMAPISUP              pSupObj,
                     LPTSTR                 pszFilePath,
                     LPTSTR                 pszStoreName,
                     LPTSTR                 pszStoreComment,
                     MAPIUID *              pUID,
                     BOOL                   fCreateIfNotFound,
                     CBackEnd **            ppBackEnd);
};

typedef struct _OBJECT_ID
{
    struct _OBJECT_ID * pNext;
    DWORD               dwObjID;
} OBJECT_ID, *POBJECT_ID;

class CIDList
{
public:
    STDMETHODIMP Insert
                    (DWORD                      dwObjID);
    STDMETHODIMP Delete
                    (DWORD                      dwObjID);
    STDMETHODIMP SaveToStream
                    (LPSTREAM                   pStream);
    STDMETHODIMP LoadFromStream
                    (LPSTREAM                   pStream);
    DWORD WINAPI Walk
                    (WALKING_ACTION             waAction);
    BOOL WINAPI IsInList
                    (DWORD                      dwObjID);
    inline BOOL WINAPI IsEmpty
                    () { return (m_dwNodeCount ? FALSE : TRUE); }
    inline DWORD WINAPI GetNodeCount
                    () { return m_dwNodeCount; }
private :
    inline void WINAPI Reset
                    () { m_dwNodeCount = 0; m_pList = m_pWalkNext = NULL; }
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CIDList         () { Reset(); }
    CIDList         (LPSTREAM pStream) { Reset(); LoadFromStream (pStream); }
    ~CIDList        ()
                    { if (m_pList) { POBJECT_ID pTmp = m_pList;
                                     while (pTmp) { pTmp = m_pList->pNext;
                                                    PrivFree ((LPVOID *)&m_pList);
                                                    m_pList = pTmp; } }
                      Reset();}

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    DWORD           m_dwNodeCount;
    POBJECT_ID      m_pList;
    POBJECT_ID      m_pWalkNext;
};

class CFileBackEnd : CBackEnd
{
///////////////////////////////////////////////////////////////////////////////
// Friend functions and classes
//
friend HRESULT WINAPI OpenMsgStoreDataBase
                    (LPMAPISUP              pSupObj,
                     LPTSTR                 pszFilePath,
                     LPTSTR                 pszStoreName,
                     LPTSTR                 pszStoreComment,
                     MAPIUID *              pUID,
                     BOOL                   fCreateIfNotFound,
                     CBackEnd **            ppBackEnd);
public:
///////////////////////////////////////////////////////////////////////////////
// CBackEnd members implemented here
//
    inline STDMETHODIMP_(ULONG) AddRef
                    () { ++m_cRef; return m_cRef; }
    inline STDMETHODIMP_(ULONG) Release
                    () { ULONG ulCount = --m_cRef; if (!ulCount) { delete this; } return ulCount;}
    STDMETHODIMP CloseResources
                    () { return S_OK; };
    STDMETHODIMP CreateObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID *          pNewObjEID,
                     CPropList **               ppList,
                     ULONG                      ulObjType,
                     LPVOID                     pData1,
                     LPVOID                     pData2);
    STDMETHODIMP OpenObject
                    (PRIVATE_ENTRYID            ObjectEID,
                     CPropList **               ppList);
    STDMETHODIMP SaveObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID            ObjectEID,
                     CPropList *                pList,
                     COpenPropList *            pOpenList,
                     LPTABLEDATA                pTable);
    STDMETHODIMP OpenPropertyInMemory
                    (PRIVATE_ENTRYID            ObjectEID,
                     ULONG                      ulMaxBytes,
                     LPVOID                     pMemBlock,
                     LPSPropValue               pProp);
    STDMETHODIMP OpenProperty
                    (PRIVATE_ENTRYID            ObjectEID,
                     LPSPropValue               pProp,
                     LPUNKNOWN *                ppUnk);
    STDMETHODIMP CopyObjects
                    (PRIVATE_ENTRYID            SrcParentContainerEID,
                     PRIVATE_ENTRYID            SrcContainerEID,
                     PRIVATE_ENTRYID            DstParentContainerEID,
                     PRIVATE_ENTRYID            DstContainerEID,
                     DWORD                      dwCopyFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData);
    STDMETHODIMP DeleteOneObject
                    (PRIVATE_ENTRYID            ObjectEID,
                     PRIVATE_ENTRYID            ParentEID);
    STDMETHODIMP DeleteObjects
                    (PRIVATE_ENTRYID            ContainerEID,
                     ULONG                      ulDeleteFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData);
    STDMETHODIMP GetTable
                    (const TABLE_TYPE           tblType,
                     PRIVATE_ENTRYID            ContainerEID,
                     long                       lAccess,
                     LPVOID                     pData1,
                     LPVOID                     pData2,
                     LPTABLEDATA *              ppTable);
    STDMETHODIMP GetTableProps
                    (const TABLE_TYPE           tblType,
                     long                       lAccess,
                     PRIVATE_ENTRYID *          ParentEID,
                     PRIVATE_ENTRYID *          ObjectEID,
                     CPropList *                pList,
                     LPSPropValue *             ppProps,
                     DWORD &                    dwLastAllocated,
                     LPTABLEDATA                pTable,
                     LPVOID                     pData1,
                     LPVOID                     pData2);
    STDMETHODIMP UpdateContentCount
                    (PRIVATE_ENTRYID            ContainerEID,
                     PRIVATE_ENTRYID            ObjectEID,
                     BOOL                       fAssociated,
                     BOOL                       fUnread);
    STDMETHODIMP SetObjNameInContainer
                    (LPTSTR                     pszNewFolderName,
                     PRIVATE_ENTRYID            ParentFolderEID,
                     PRIVATE_ENTRYID            FolderEID);
    STDMETHODIMP SetRFSEntryID
                    (LPTSTR                     pszMsgClass,
                     PRIVATE_ENTRYID            FolderEID,
                     LPTABLEDATA                pTable);
    STDMETHODIMP GetRFSEntryID
                    (LPTSTR                     pszMsgClass,
                     PRIVATE_ENTRYID *          pFolderEID,
                     LPTSTR                     pszExplicitClass);
    STDMETHODIMP GetNamedPropertiesStream
                    (LPSTREAM *                 ppStream);

    inline STDMETHODIMP SetBackEndStrings
                    (LPTSTR                     szName,
                     LPTSTR                     szComment)
                    { return m_pDBobj->SetDataBaseStrings (szName, szComment); }

///////////////////////////////////////////////////////////////////////////////
// CFileBackEnd members
//
private:
    // Functions to create objects in the database
    STDMETHODIMP CreateFolderObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID            NewFolderEID,
                     LPTSTR                     pszNewFolderName,
                     LPTSTR                     pszNewFolderComment,
                     CPropList **               ppList);
    
    // Functions to delete objects in the database
    STDMETHODIMP DeleteLargePropsStreams
                    (DWORD                      dwObjID);
    STDMETHODIMP DeleteMessageAttachmentsStreams
                    (DWORD                      dwMsgID);
    STDMETHODIMP DeleteMessageRecipientsStreams
                    (DWORD                      dwMsgID);
    STDMETHODIMP DeleteMessageAttachment
                    (DWORD                      dwMsgObjID,
                     DWORD                      dwAttachmentID);
    STDMETHODIMP DeleteFolderContents
                    (PRIVATE_ENTRYID            FolderEID,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData);
    STDMETHODIMP DeleteFolderStorage
                    (DWORD                      dwParentFldObjID,
                     DWORD                      dwFldObjID);
    STDMETHODIMP DeleteFolderNameInParent
                    (DWORD                      dwParentFolderID,
                     DWORD                      dwFolderID);
    // Functions to copy objects in the database
    STDMETHODIMP CopyMessagesInFolder
                    (PRIVATE_ENTRYID            SrcFolderEID,
                     PRIVATE_ENTRYID            DstFolderEID,
                     DWORD                      dwCopyFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData);
    STDMETHODIMP CopyMessageToMessage
                    (DWORD                      dwSrcMsgID,
                     DWORD                      dwDstMsgID);
    STDMETHODIMP CopyObjectProps
                    (DWORD                      dwSrcMsgID,
                     DWORD                      dwDstMsgID);

    STDMETHODIMP GetContentsTableProps
                    (CPropList *                pList,
                     LPSPropValue *             ppProps,
                     PRIVATE_ENTRYID            MessageEID,
                     long                       lAccessLevel,
                     LPSBinary                  pStoreEID,
                     DWORD &                    dwLastAllocated,
                     LPTABLEDATA                pTable);
    STDMETHODIMP GetAttachmentTableProps
                    (CPropList *                pList,
                     LPSPropValue *             ppProps,
                     ULONG                      ulAttachNum,
                     long                       lAccessLevel,
                     LPSBinary                  pStoreEID,
                     DWORD &                    dwLastAllocated,
                     LPTABLEDATA                pTable);
    STDMETHODIMP GetRFSTableData
                    (LPTABLEDATA                pTable);
    STDMETHODIMP GetStream
                    (DWORD                      dwRecordID,
                     RECORD_FIELDS              rfStream,
                     LPSTREAM *                 ppStream,
                     BOOL                       fCreateIfNotFound = TRUE,
                     BOOL                       fWriteZeroCount = TRUE);
    STDMETHODIMP CreateSpecialFolders
                    ();
    STDMETHODIMP UpdateFolderCount
                    (PRIVATE_ENTRYID            FolderEID,
                     PRIVATE_ENTRYID            NewFolderEID);
    STDMETHODIMP UpdateAttachmentsCount
                    (PRIVATE_ENTRYID            MessageEID,
                     PRIVATE_ENTRYID            AttachEID,
                     BOOL                       fRemove);
    STDMETHODIMP SaveLargeProps
                    (COpenPropList *            pOpenList,
                     CPropList *                pList);
    STDMETHODIMP CalculateSize
                    (PRIVATE_ENTRYID            ObjectEID,
                     CPropList *                pList,
                     long &                     lSize);
    STDMETHODIMP LoadStream
                    (CStorage *                 pParentStg,
                     PRIVATE_ENTRYID            ObjectEID,
                     LPSPropValue               pProp,
                     LPVOID                     pMemBlock,
                     DWORD                      dwStreamID,
                     DWORD                      dwMaxBytes);
    STDMETHODIMP CheckFldNameCollision
                    (DWORD                      dwFolderID,
                     LPTSTR                     pszNewFolderName,
                     DWORD *                    pdwExistingFldID,
                     DWORD                      dwFlags);
    STDMETHODIMP FillHierarchyTable
                    (PRIVATE_ENTRYID            FolderEID,
                     LPTABLEDATA                pTable,
                     LPSBinary                  pStoreEID,
                     long                       lFolderAccess);
    STDMETHODIMP FillContentsTable
                    (PRIVATE_ENTRYID            ParentFolderEID,
                     LPTABLEDATA                pTable,
                     LPSBinary                  pStoreEID,
                     long                       lMessageAccess,
                     BOOL                       fGetAsscContTable);
    STDMETHODIMP FillRecipientTable
                    (PRIVATE_ENTRYID            MessageEID,
                     LPTABLEDATA                pTable,
                     DWORD *                    pdwRecipCount);
    STDMETHODIMP FillAttachmentsTable
                    (PRIVATE_ENTRYID            MessageEID,
                     LPTABLEDATA                pTable,
                     LPSBinary                  pStoreEID,
                     long                       lAttachmentAccess);
    BOOL WINAPI IsContentTableProp
                    (ULONG                      ulPropTag);
    BOOL WINAPI IsAttachmentTableProp
                    (ULONG                      ulPropTag);
    STDMETHODIMP LoadProperties
                    (LPSTREAM                   pStream,
                     CPropList **               ppList);
    STDMETHODIMP SaveRecipTable
                    (DWORD                      dwRecordID,
                     LPTABLEDATA                pRecipTable);
    inline void WINAPI LockDataBase
                    () { EnterCriticalSection (&m_csObj); };
    inline void WINAPI UnlockDataBase
                    () { LeaveCriticalSection (&m_csObj); };
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CFileBackEnd       (CDataBase *                pDBobj,
                        MAPIUID *                  pUIDinFile,
                        LPMAPISUP                  pSupObj);
    ~CFileBackEnd      ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    ULONG               m_cRef;
    CDataBase *         m_pDBobj;
    LPMAPISUP           m_pSupObj;
};

// Minimum set of properties available in a folder contents table
#define MIN_CONTENTS_TABLE_PROPS    18
const static SizedSPropTagArray (MIN_CONTENTS_TABLE_PROPS, sptFolderContProps) =
{
    MIN_CONTENTS_TABLE_PROPS,
    {
        PR_INSTANCE_KEY,
        PR_ACCESS_LEVEL,
        PR_ENTRYID,
        PR_PARENT_ENTRYID,
        PR_STORE_ENTRYID,
        PR_STORE_RECORD_KEY,
        PR_RECORD_KEY,
        PR_MAPPING_SIGNATURE,
        PR_OBJECT_TYPE,
        PR_HASATTACH,
        PR_MESSAGE_FLAGS,
        PR_CREATION_TIME,
        PR_LAST_MODIFICATION_TIME,
        PR_MESSAGE_SIZE,
        PR_PRIORITY,
        PR_SEARCH_KEY,
        PR_MESSAGE_CLASS,
        PR_MSG_STATUS
    }
};

enum
{
    RFS_INST_KEY,
    RFS_ENTRYID,
    RFS_RECORD_KEY,
    RFS_MSG_CLASS,
    NUM_RFS_TABLE_PROPS
};

// Minimum set of properties available in the Receive Folder Setting table
const static SizedSPropTagArray(NUM_RFS_TABLE_PROPS, sptRFSprops) =
{
    NUM_RFS_TABLE_PROPS,
    {
        PR_INSTANCE_KEY,
        PR_ENTRYID,
        PR_RECORD_KEY,
		PR_MESSAGE_CLASS
    }
};

// Minimum set of properties available in the message recipients table
#define MIN_RECIPIENT_TABLE_PROPS    10
const static SizedSPropTagArray(MIN_RECIPIENT_TABLE_PROPS, sptRecipTableProps) =
{
    MIN_RECIPIENT_TABLE_PROPS,
    {
        PR_ROWID,
        PR_SEARCH_KEY,
        PR_INSTANCE_KEY,
        PR_ENTRYID,
        PR_DISPLAY_NAME,
        PR_RECIPIENT_TYPE,
        PR_ADDRTYPE,
        PR_EMAIL_ADDRESS,
        PR_OBJECT_TYPE,
        PR_DISPLAY_TYPE
    }
};

// Minimum set of properties available in an attachment table
#define MIN_ATTACHMENT_TABLE_PROPS      12
const static SizedSPropTagArray(MIN_ATTACHMENT_TABLE_PROPS, sptAttTableProps) =
{
    MIN_ATTACHMENT_TABLE_PROPS,
    {
        PR_INSTANCE_KEY,
        PR_STORE_ENTRYID,
        PR_STORE_RECORD_KEY,
        PR_RECORD_KEY,
        PR_MAPPING_SIGNATURE,
        PR_OBJECT_TYPE,
        PR_ATTACH_NUM,
        PR_ATTACH_METHOD,
        PR_CREATION_TIME,
        PR_LAST_MODIFICATION_TIME,
        PR_ATTACH_SIZE,
        PR_DISPLAY_NAME
    }
};

#endif // _CFILESTG_H

// End of file for CFILESTG.H
