//////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      GENPROP.H
//
//  Description
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _GENPROP_H
#define _GENPROP_H
#ifdef _DEBUG
#define     ENABLE_DEBUG_OUTPUT     1
#endif // _DEBUG

#include "PROPLIST.H"
#include "NAMEPROP.H"

// Header file with the Config Page Dialog Control IDs Message String ID's
#include "RESOURCE.H"

typedef enum _ERROR_CONTEXT
{
    NONE_ERROR              = 0,
    ERROR_CNTX_DBFILE       = 1,
    ERROR_CNTX_CLIENT       = 2,
    ERROR_CNTX_INTERNAL     = 3,
    ERROR_CNTX_PROP_CACHE   = 4
} ERROR_CONTEXT;

typedef enum _TABLE_TYPE
{
    TABLE_HIERARCHY,
    TABLE_CONTENTS,
    TABLE_ATTACHMENTS,
    TABLE_RECIPIENTS,
    TABLE_RECEIVE_FOLDERS
} TABLE_TYPE;

enum
{
    COPYFLAG_COPY           = 0x0001,
    COPYFLAG_MOVE           = 0x0002,
    CONTAINER_OBJECTS       = 0x0010,
    EMPTY_CONTAINER         = 0x0020,
    ATTACHMENT_OBJECT       = 0x0040,
    SUB_CONTAINERS          = 0x1000, 
    SUB_OBJECTS             = 0x2000, 
    SUB_ASSOC_OBJECTS       = 0x4000
};

enum
{
    OBJTYPE_ASSOCIATED          = 0x0001,
    OBJTYPE_NORMAL              = 0x0002,
    OBJTYPE_MOVED               = 0x0100,
    OBJTYPE_UNREAD              = 0x1000
};

enum
{
    OBJECT_ASSOCIATED           = 0x00000001,
    OBJECT_MESSAGE_IN_MESSAGE   = 0x00000002,
    OBJECT_UNREAD_MESSAGE       = 0x00000010,
    OBJECT_HAS_ATTACHMENT       = 0x00000020,
    OBJECT_DEFER_ERRORS         = 0x00000100,
    OBJECT_NEVER_SAVED          = 0x00001000,
    OBJECT_JUST_CREATED         = 0x00002000,
    OBJECT_DELETED_IN_CONTAINER = 0x00004000,
    OBJECT_RUNNING_IN_SPOOLER   = 0x00010000,
    OBJECT_MAY_HAVE_RECIPIENTS  = 0x01000000,
    OBJECT_MAY_HAVE_CONTAINERS  = 0x02000000,
    OBJECT_IS_ONE_OFF           = 0x10000000,
    OBJECT_IS_SEARCH_CONTAINER  = 0x20000000
};

enum
{
    EID_OBJTYPE_DL_TEMPLATE         = 0x00A0,
    EID_OBJTYPE_MAILUSER_TEMPLATE   = 0x00B0
};

// This interfaces are used to optimize CopyTo, CopyFolder, and etc, 
// operations on folders, messages, etc, of our own implementation.
// IID_SAMPLE_IMAPIFolder GUID:    040282B0-735F-11CE-B4E2-00AA00510E3B
// IID_SAMPLE_IMessage GUID:       A56393F0-735F-11CE-B4E2-00AA00510E3B
// IID_SAMPLE_IAttach GUID:        5A62FCA2-AC68-11CE-A1C3-00AA00510E3B
// IID_SAMPLE_IMAPIProp GUID:      5A62FCA1-AC68-11CE-A1C3-00AA00510E3B
DEFINE_GUID(IID_SAMPLE_IMAPIFolder, 0x040282B0, 0x735F, 0x11CE, 0xB4, 0xE2, 0x00, 0xAA, 0x00, 0x51, 0xE, 0x3B);
DEFINE_GUID(IID_SAMPLE_IMessage,    0xA56393F0, 0x735F, 0x11CE, 0xB4, 0xE2, 0x00, 0xAA, 0x00, 0x51, 0xE, 0x3B);
DEFINE_GUID(IID_SAMPLE_IAttach,     0x5A62fCA2, 0xAC68, 0x11CE, 0xA1, 0xC3, 0x00, 0xAA, 0x00, 0x51, 0xE, 0x3B);
DEFINE_GUID(IID_SAMPLE_IMAPIProp,   0x5A62fCA1, 0xAC68, 0x11CE, 0xA1, 0xC3, 0x00, 0xAA, 0x00, 0x51, 0xE, 0x3B);

typedef struct _PRIVATE_ENTRYID
{
    // MAPI-required fields
	BYTE        abFlags[4];     // 4 bytes          // MAPI Flags
	MAPIUID     uidGlobal;      // 16 bytes         // UID Unique for the backend (ie. at the database file level)  (This must be here)
    // Provider-defined fields
	BYTE        bVersion;       // 1 bytes          // Version of the entry ID schema
    BYTE        bObject;        // 1 bytes          // Object type
    BYTE        bPad[2];        // 2 bytes          // Pad to align the structure to 4-bytes boundaries
    DWORD       dwObjID;        // 4 bytes          // Object ID (container or object)
                                // 28 Bytes Total
} PRIVATE_ENTRYID;
#define     ENTRYID_VERSION     1       // Schema version
#define     CB_PRIVATE_EID      28      // Size of the structure above

typedef struct _SUB_ENTRYID
{
    BYTE        bObject;        // 1 bytes          // Object type
    BYTE        bAssociated;    // 1 bytes          // 1 if the object is assocated, 0 if not.
    BYTE        bPad[2];        // 3 bytes          // Pad to align the structure to 4-bytes boundaries
    DWORD       dwObjID;        // 4 bytes          // Object ID (container or object)
} SUB_ENTRYID;

class COpenObjectList;

typedef struct _SUBFOLDERS_DELETED_INFO
{
    DWORD               dwFldObjID;
    DWORD               dwCount;
    SUB_ENTRYID *       pSubEIDList;
} SUBFOLDERS_DELETED_INFO;


#define OPENOBJDATA_USE_MSGEIDLIST       0x0001
#define OPENOBJDATA_SKIP_ASSOCIATED      0x0002
#define OPENOBJDATA_SUB_CONTAINER        0x0004
typedef struct _OPENOBJECTS_DATA
{
    LPENTRYLIST                 pSrcEIDList;
    SUB_ENTRYID *               pSubEIDList;
    DWORD                       dwCount;
    SUBFOLDERS_DELETED_INFO *   pSubFldInfo;
    DWORD                       dwSubFldCount;
    LPSBinary                   pStoreEID;
    long                        lDstAccessLevel;
    COpenObjectList *           pOpenObjList;
    LPUNKNOWN                   pSrcUnk;
    LPUNKNOWN                   pDstUnk;
    DWORD                       dwFlags;
} OPENOBJECTS_DATA, *POPENOBJECTS_DATA;

class CBackEnd
{
///////////////////////////////////////////////////////////////////////////////
// Pure-Virtual member functions
//
public :
    virtual ULONG STDMETHODCALLTYPE AddRef
                    () = 0;
    virtual ULONG STDMETHODCALLTYPE Release
                    () = 0;
    virtual HRESULT STDMETHODCALLTYPE CloseResources
                    () = 0;
    virtual HRESULT STDMETHODCALLTYPE CreateObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID *          pNewObjEID,
                     CPropList **               ppList,
                     ULONG                      ulObjType,
                     LPVOID                     pData1,
                     LPVOID                     pData2) = 0;
    virtual HRESULT STDMETHODCALLTYPE OpenObject
                    (PRIVATE_ENTRYID            ObjectEID,
                     CPropList **               ppList) = 0;
    virtual HRESULT STDMETHODCALLTYPE SaveObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID            ObjectEID,
                     CPropList *                pList,
                     COpenPropList *            pOpenList,
                     LPTABLEDATA                pTable) = 0;
    virtual HRESULT STDMETHODCALLTYPE OpenPropertyInMemory
                    (PRIVATE_ENTRYID            ObjectEID,
                     ULONG                      ulMaxBytes,
                     LPVOID                     pMemBlock,
                     LPSPropValue               pProp) = 0;
    virtual HRESULT STDMETHODCALLTYPE OpenProperty
                    (PRIVATE_ENTRYID            ObjectEID,
                     LPSPropValue               pProp,
                     LPUNKNOWN *                ppUnk) = 0;
    virtual HRESULT STDMETHODCALLTYPE CopyObjects
                    (PRIVATE_ENTRYID            SrcParentContainerEID,
                     PRIVATE_ENTRYID            SrcContainerEID,
                     PRIVATE_ENTRYID            DstParentContainerEID,
                     PRIVATE_ENTRYID            DstContainerEID,
                     DWORD                      dwCopyFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData) = 0;
    virtual HRESULT STDMETHODCALLTYPE DeleteOneObject
                    (PRIVATE_ENTRYID            ObjectEID,
                     PRIVATE_ENTRYID            ParentEID) = 0;
    virtual HRESULT STDMETHODCALLTYPE DeleteObjects
                    (PRIVATE_ENTRYID            ContainerEID,
                     ULONG                      ulDeleteFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData) = 0;
    virtual HRESULT STDMETHODCALLTYPE UpdateContentCount
                    (PRIVATE_ENTRYID            ContainerEID,
                     PRIVATE_ENTRYID            ObjectEID,
                     BOOL                       fAssociated,
                     BOOL                       fUnread) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetTable
                    (const TABLE_TYPE           tblType,
                     PRIVATE_ENTRYID            ContainerEID,
                     long                       lAccess,
                     LPVOID                     pData1,
                     LPVOID                     pData2,
                     LPTABLEDATA *              ppTable) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetTableProps
                    (const TABLE_TYPE           tblType,
                     long                       lAccess,
                     PRIVATE_ENTRYID *          ParentEID,
                     PRIVATE_ENTRYID *          ObjectEID,
                     CPropList *                pList,
                     LPSPropValue *             ppProps,
                     DWORD &                    dwLastAllocated,
                     LPTABLEDATA                pTable,
                     LPVOID                     pData1,
                     LPVOID                     pData2) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetObjNameInContainer
                    (LPTSTR                     pszNewFolderName,
                     PRIVATE_ENTRYID            ParentFolderEID,
                     PRIVATE_ENTRYID            FolderEID) = 0;
    virtual HRESULT STDMETHODCALLTYPE SetRFSEntryID
                    (LPTSTR                     pszMsgClass,
                     PRIVATE_ENTRYID            FolderEID,
                     LPTABLEDATA                pTable) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetRFSEntryID
                    (LPTSTR                     pszMsgClass,
                     PRIVATE_ENTRYID *          pFolderEID,
                     LPTSTR                     pszExplicitClass) = 0;
    virtual HRESULT STDMETHODCALLTYPE GetNamedPropertiesStream
                    (LPSTREAM *                 ppStream) = 0;
                    
    virtual HRESULT STDMETHODCALLTYPE SetBackEndStrings
                    (LPTSTR                     szName,
                     LPTSTR                     szComment) = 0;

///////////////////////////////////////////////////////////////////////////////
// Implemented member functions
//
    inline BOOL WINAPI IsValidEntryID
                    (PRIVATE_ENTRYID *          pEntryID);
    inline MAPIUID WINAPI GetBackEndUID
                    () { return m_uidBackEnd; }
    inline BOOL WINAPI IsSameBackEnd
                    (MAPIUID                    uidOther)
                    { return (memcmp (&m_uidBackEnd, &uidOther, sizeof(MAPIUID)) ? FALSE  : TRUE); }
    inline void WINAPI FreeDeleteObjectsData
                    (OPENOBJECTS_DATA *         pData);
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CBackEnd        (MAPIUID             uidBackEnd)
                    { m_uidBackEnd = uidBackEnd;
                      InitializeCriticalSection (&m_csObj); }
    ~CBackEnd       () { DeleteCriticalSection (&m_csObj); }

///////////////////////////////////////////////////////////////////////////////
// Data members
public :
    MAPIUID             m_uidBackEnd;
    CRITICAL_SECTION    m_csObj;
};

///////////////////////////////////////////////////////////////////////////////
//    CBackEnd::IsValidEntryID()
//
//    Parameters
//      [IN]    pEntryID            Pointer to the entry ID structure
//                                  we will verify.
//      [IN[    fCheckObjectExists  Flag to request (TRUE) the database to
//                                  check the object indicated by the entry
//                                  ID currently exists in the database.
//
//    Purpose
//      Given a pointer to a private entry ID structure, determines if its
//      contents would be something in this database.
//
//    Return Value
//      TRUE if the entry is not null and the UID matches the UID of this
//      database. FALSE otherwise.
//
inline BOOL WINAPI CBackEnd::IsValidEntryID (PRIVATE_ENTRYID * pEntryID)
{
    // Check the members of the structure against our known values.
    BOOL bValid = TRUE;
    __try
    {
        if (!pEntryID ||
            memcmp (&pEntryID->uidGlobal, &m_uidBackEnd, sizeof(MAPIUID)) ||
            (pEntryID->bVersion != ENTRYID_VERSION))
        {
            bValid = FALSE;
        }
        else
        {
            switch (pEntryID->bObject)
            {
                case MAPI_FOLDER :
                case MAPI_MESSAGE :
                case MAPI_ATTACH :
                case MAPI_ABCONT :
                case MAPI_MAILUSER :
                case MAPI_DISTLIST :
                    break;
                default :
                    TraceMessage ("CBackEnd::IsValidEntryID: Invalid object type in entry ID structure");
                    bValid = FALSE;
                    break;
            }
        }
    }
    __except (EXCEPTION_EXECUTE_HANDLER)
    {
        TraceString1 ("CBackEnd::IsValidEntryID: An exception occured. Exception code: %#08x", GetExceptionCode());
        bValid = FALSE;
    }
    return bValid;
}

///////////////////////////////////////////////////////////////////////////////
//    CBackEnd::FreeDeleteObjectsData()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
inline void WINAPI CBackEnd::FreeDeleteObjectsData (OPENOBJECTS_DATA * pData)
{
    while (pData->dwSubFldCount)
    {
        pData->dwSubFldCount--;
        PrivFree ((LPVOID *)&(pData->pSubFldInfo[pData->dwSubFldCount].pSubEIDList));
    }
    if (pData->pSubFldInfo)
    {
        HeapFree (GetProcessHeap(), 0, pData->pSubFldInfo);
    }
    if (pData->pSubEIDList)
    {
        PrivFree ((LPVOID *)&(pData->pSubEIDList));
    }
}


typedef struct _OPEN_OBJECTS
{
    struct _OPEN_OBJECTS *  pNext;
    PRIVATE_ENTRYID *       pEID;
    LPUNKNOWN               pUnkObj;
} OPEN_OBJECTS, *POPEN_OBJECTS;

class COpenObjectList
{
///////////////////////////////////////////////////////////////////////////////
// Members of this class
//
public:
    STDMETHODIMP Insert
                    (LPUNKNOWN                  pUnkObj,
                     PRIVATE_ENTRYID *          pObjectEID);
    void WINAPI Delete
                    (LPUNKNOWN                  pUnkObj,
                     PRIVATE_ENTRYID *          pObjectEID);
    LPUNKNOWN WINAPI Walk
                    (SUB_ENTRYID                ObjectEID,
                     WALKING_ACTION             waAction,
                     LPUNKNOWN                  pPreviousUnk);
    LPUNKNOWN WINAPI Walk
                    (PRIVATE_ENTRYID            ObjectEID,
                     WALKING_ACTION             waAction,
                     LPUNKNOWN                  pPreviousUnk);
    inline BOOL WINAPI IsEmpty
                    () { return m_pHead ? FALSE : TRUE; }

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    COpenObjectList     () { InitializeCriticalSection (&m_csObj); m_pHead = NULL; };
    ~COpenObjectList    () { DeleteCriticalSection (&m_csObj); };

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    CRITICAL_SECTION    m_csObj;
    POPEN_OBJECTS       m_pHead;
};

class CGeneric
{
public :
    inline void WINAPI SetLastError
                    (HRESULT                    hError,
                     ERROR_CONTEXT              ctxError = ERROR_CNTX_INTERNAL)
                     { if (S_OK == hError){ return; }
                       m_hLastError = hError;
                       m_ctxLastError = ctxError; };
    STDMETHODIMP GetLastError
                    (HRESULT                    hResult,
                     ULONG                      ulFlags,
                     LPMAPIERROR *              ppMAPIError,
                     HINSTANCE                  hInstance,
                     LPTSTR                     pszComponent);
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CGeneric        ();
    ~CGeneric       ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
protected :
    ULONG               m_cRef;
    CRITICAL_SECTION    m_csObj;
private :
    HRESULT             m_hLastError;
    ERROR_CONTEXT       m_ctxLastError;
};

class CGenericProp : public CGeneric
{
public:
    // Common implementation for the IMAPIProp interface
    STDMETHODIMP SaveChanges
                    (ULONG                      ulFlags,
                     LPTABLEDATA                pTable);
    STDMETHODIMP GetPropList
                    (const ULONG                ulFlags,
                     const LPSPropTagArray      pObjectTags,
                     LPSPropTagArray *          ppAllTags);
    STDMETHODIMP OpenProperty
                    (ULONG                      ulPropTag,
                     LPCIID                     piid,
                     ULONG                      ulInterfaceOptions,
                     ULONG                      ulFlags,
                     LPUNKNOWN *                ppUnk);
    STDMETHODIMP GetProps
                    (LPSPropTagArray            pPropTagArray,
                     ULONG                      ulFlags,
                     ULONG *                    pcValues,
                     LPSPropValue *             ppPropArray,
                     LPSPropTagArray            pObjectTags);
    STDMETHODIMP GetProps
                    (LPSPropValue               pProp,
                     LPVOID                     pMemBlock);
    STDMETHODIMP GetProps
                    (HRESULT                    hError,
                     ULONG                      cbSize,
                     LPVOID                     pSrc,
                     LPSPropValue               pProp,
                     LPVOID                     pMemBlock);
    STDMETHODIMP DeleteProps
                    (LPSPropTagArray            pPropTagArray,
                     LPSPropProblemArray *      ppProblems);
    STDMETHODIMP GetNamesFromIDs
                    (LPSPropTagArray *          ppPropTags,
                     LPGUID                     pPropSetGuid,
                     ULONG                      ulFlags,
                     ULONG *                    pcPropNames,
                     LPMAPINAMEID **            pppPropNames,
                     LPSPropTagArray            pTags);
    STDMETHODIMP GetIDsFromNames
                    (ULONG                      cPropNames,
                     LPMAPINAMEID *             ppPropNames,
                     ULONG                      ulFlags,
                     LPSPropTagArray *          ppPropTags,
                     LPSPropTagArray            pTags,
                     LPSTREAM                   pStream);
    // Public helper function
    STDMETHODIMP AddPropToList
                    (const LPSPropValue         pProp,
                     BOOL                       fForceIt = TRUE);
    STDMETHODIMP GetPropAccess 
                    (ULONG                      ulPropTag);
    STDMETHODIMP AddTagToProbArray
                    (ULONG                      cNumberOfTags,
                     LPSPropProblemArray *      ppProbArray,
                     UINT                       ulPropTagIndex,
                     ULONG                      ulPropTag,
                     HRESULT                    hError);
    STDMETHODIMP GetObjectData
                    (LPVOID *                   ppVoid);
                         
private:
    // Internal Helper function
    STDMETHODIMP CombineTags
                    (LPSPropTagArray            pObjectTags ,
                     LPSPropTagArray            pListTags,
                     LPSPropTagArray *          ppAllTags,
                     BOOL                       fUseMAPIAlloc = FALSE);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CGenericProp    (CBackEnd *                 pBEobj,
                     PRIVATE_ENTRYID *          pParentEID,
                     PRIVATE_ENTRYID *          pObjectEID,
                     CPropList *                pList,
                     BOOL                       fDeferErrors,
                     long                       lAccess);
    ~CGenericProp   ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
public :
    ULONG               m_ulObjFlags;
    long                m_lAccess;
    CBackEnd *          m_pBEobj;
    CPropList *         m_pList;
    COpenPropList *     m_pOpenPropList;
    CNamePropMgr *      m_pNPList;
    PRIVATE_ENTRYID     m_ObjectEID;
    PRIVATE_ENTRYID     m_ParentEID;
};

#endif // _GENPROP_H

// End of file for GENPROP.H
