//////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      MSLMS.H
//
//  Description
//      Interface declaraction file for CMSLogon, CMSProvider,
//      CMAPIFolder, CMessage, and CMsgStore
//      Property tag definitions.
//      Constant definitions
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Corporation
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//
#ifndef _MSLMS_H
#define _MSLMS_H

#pragma warning (disable : 4100)
#pragma warning (disable : 4127)
#pragma warning (disable : 4201)
#pragma warning (disable : 4245)
#pragma warning (disable : 4514)

#define STRICT
#define INC_RPC
#define _INC_OLE
#include <WINDOWS.H>
#include <CTL3D.H>
#include <MAPIWIN.H>
#include <MAPISPI.H>
#include <MAPIUTIL.H>
#include <MAPIVAL.H>

#ifdef _DEBUG
#define     ENABLE_DEBUG_OUTPUT     1
#endif // _DEBUG

// Header file for the trace and logging functions.
#include "TRACES.H"

#include "SUPPORT.H"

#include "GENPROP.H"

#define BASE_PROVIDER_ID    0x6600  // Look at the comments in MAPITAGS.H
#define PR_SMP_STORE_PATH       PROP_TAG (PT_TSTRING, (BASE_PROVIDER_ID + 0x0001))
#define PR_SMP_PASSWORD         PROP_TAG (PT_TSTRING, (BASE_PROVIDER_ID + 0x0002))
#define PR_SMP_REMEMBER_PWD     PROP_TAG (PT_BOOLEAN, (BASE_PROVIDER_ID + 0x0003))

// The order for this enumarations must match the order of the properties
// in the tag arrays. These arrays are used to access the 
// LPSPropValue arrays returned from MAPI interfaces. The use of enumerations
// hides differences when the number of properties in the tag array increases
// or decreases. It also helps maintain error-free indexing in the use of 
// the prop arrays
enum
{
    STORE_PATH,             // Arrays Indexes
    STORE_NAME,
    STORE_COMMENT,
    STORE_FILE_UID,
    STORE_PASSWORD,
    STORE_REMEM_PWD,
    NUM_LOGON_PROPERTIES    // Array Size
};

// ID array of property tags for the store logon
const static SizedSPropTagArray(NUM_LOGON_PROPERTIES, sptLogonArray) =
{
    NUM_LOGON_PROPERTIES,
    {
        PR_SMP_STORE_PATH,
        PR_DISPLAY_NAME,
        PR_COMMENT,
        PR_STORE_RECORD_KEY,
        PR_SMP_PASSWORD,
        PR_SMP_REMEMBER_PWD
    }
};

enum
{
    EVENT_UNDEFINED                 = 0,
    EVENT_SUBMIT_MESSAGE_START,
    EVENT_SUBMIT_MESSAGE_FINISHED,
    EVENT_NEW_RECEIVE_FOLDER_SET
};

#define SUPPORT_MASK    (STORE_ENTRYID_UNIQUE   | \
                         STORE_MODIFY_OK        | \
                         STORE_MV_PROPS_OK      | \
                         STORE_CREATE_OK        | \
                         STORE_ATTACH_OK        | \
                         STORE_OLE_OK           | \
                         STORE_SUBMIT_OK        | \
                         STORE_NOTIFY_OK        | \
                         STORE_RESTRICTION_OK   | \
                         STORE_SORT_OK)

// PR_MDB_PROVIDER is the GUID that represent this Message Store Provider.
// This GUID is available as a property in the stores table and on
// the message store object. The UID is hard coded.
// This number is generated only once per provider during the developement
// of it using the standard tools (GUIDGEN or UUIDGEN) in VC++ or the Win32 SDK.
// With this number you could identify your provider across multiple platforms.
// GUIDGEN: 0x7be3c380, 0x5b0a, 0x11ce, 0xb4, 0xe1, 0x0, 0xaa, 0x0, 0x51, 0xe, 0x3b
#define MS_STORE_UID_PROVIDER { 0x7b, 0xe3, 0xc3, 0x80, \
                                0x5b, 0x0a, 0x11, 0xce, \
                                0xb4, 0xe1, 0x0,  0xaa, \
                                0x0,  0x51, 0xe,  0x3b }

// Version of this store provider returned in PR_CREATE_VERSION and PR_CURRENT_VERSION
#define MSG_STORE_VERSION           1

// Display name for the provider used in PR_PROVIDER_NAME
#define PROVIDER_DISPLAY_NAME_STRING    "Local Message Store"

// The target OS should not be included in the DLL name. MAPI will handle that.
// For Win32 platforms (Windows NT and Windows 95), the full name will be MSLMS32.DLL
#define MS_DLL_NAME_STRING              "MSLMS.DLL"

// Structure passed to the configuration dialog.
typedef struct _CFGDLG
{
    HINSTANCE           hInst;          // Instance of DLL
    HWND                hWnd;           // Handle of parent window  
    LPSPropValue *      ppPropArray;    // Property array pointer       
    LPSPropTagArray     pTags;          // List of property tags        
    LPMAPISUP           pSupObj;        // IMAPISupport object  
    ULONG               ulFlags;        // 0 or UI_READONLY             
} CFGDLG, *PCFGDLG;

// CTL3D Dynamic Loading Support
void WINAPI PrivInitialize3DCtl
                    (HINSTANCE                  hInstance);
void WINAPI PrivUninitialize3DCtl
                    (HINSTANCE                  hInstance);

// Stand-alone helper functions
extern "C"
{
    CALLERRELEASE FolderTablesViewRelease;
    CALLERRELEASE MessageTablesViewRelease;
    CALLERRELEASE StoreTablesViewRelease;
    NOTIFCALLBACK ContextNotify;
    BOOL WINAPI BrowseFileDialog
                    (HINSTANCE                  hInstance,
                     HWND                       hOwnerWnd,
                     LPTSTR                     pszFileNameBuffer);
    HRESULT WINAPI CreateStoreEntryID
                    (MAPIUID *                  pStoreUID,
                     LPSBinary                  pStoreEID);
    HRESULT WINAPI DoLogonDlg
                    (PCFGDLG);
    extern MAPIUID guidMSProvider;
    extern char szBlank[];
};

// Forward class declarations
class CMSProvider;
class CMsgStore;
class CMSLogon;
class CMAPIFolder;
class CAttach;

typedef enum _UPDATE_TABLE_ACTION
{
    UPD_TABLE_REFRESH,
    UPD_TABLE_REFRESH_ADD_UNREAD,
    UPD_TABLE_REFRESH_DEL_UNREAD,
    UPD_TABLE_ADD_ROW,
    UPD_TABLE_DELETE_ROW
} UPDATE_TABLE_ACTION;

// Node structure ofr the linked list of notification set by clients on objects in our store
typedef struct _NOTIF_OBJS
{
    struct _NOTIF_OBJS *    pNext;
    PRIVATE_ENTRYID         eidObject;
    ULONG                   ulConnection;
} NOTIF_OBJS, *PNOTIF_OBJS;

// Type of actions deferred in transacted object (Messages and Attachments only)
typedef enum _TRANSACTED_ACTIONS
{
    TRANS_OBJ_ACTION_UNDEFINED = 0,
    TRANS_OBJ_ACTION_DELETE,
    TRANS_OBJ_ACTION_SAVE,
    TRANS_OBJ_ACTION_DELETE_OBJ_PROP
} TRANSACTED_ACTIONS;

// Entry for a queued action on a transacted object.
typedef struct _TRANSACTED_OBJECTS
{
    struct _TRANSACTED_OBJECTS * pNext;
    TRANSACTED_ACTIONS      Action;
    PRIVATE_ENTRYID         ObjEID;
    LPUNKNOWN               pUnkObj;
    SPropValue              Prop;
} TRANSACTED_OBJECTS;

// This structure is used for inter process communications
// (IPC) between the client process and the spooler process
typedef struct _OUTQUEUE_NOTIF_DATA
{
    PRIVATE_ENTRYID     MsgEID;
    PRIVATE_ENTRYID     ParentEID;
    FILETIME            ftSubmitTime;
} OUTQUEUE_NOTIF_DATA, *POUTQUEUE_NOTIF_DATA;

// This structure is use to notif the different message store contexts
// that a new Receive Folder has been set and they need to update
// their Receive Folder Settings table.
typedef struct _NEW_RFS_NOTIF_DATA
{
    char                szMsgClass[256];    // 256
    PRIVATE_ENTRYID     FolderEID;          // 28
    CMsgStore *         pStore;             // 4
    DWORD               dwProcessID;        // 4
    BYTE                abPad[4];           // 4
                                     // Total: 296
} NEW_RFS_NOTIF_DATA;

class CMsgStore : public CGenericProp, public IMsgStore
{
friend CMSProvider;
friend HRESULT STDAPICALLTYPE ContextNotify (LPVOID, ULONG, LPNOTIFICATION);
friend void STDAPICALLTYPE StoreTablesViewRelease (ULONG, LPTABLEDATA, LPMAPITABLE);
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMSGSTORE_METHODS(IMPL);
    
///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class or overides
//
    STDMETHODIMP UpdateSubscriptionList
                    (PRIVATE_ENTRYID *          pEID,
                     ULONG                      ulConnectionNum,
                     BOOL                       fAddToList);
    STDMETHODIMP NotifyClients
                    (PRIVATE_ENTRYID *          pEID,
                     ULONG                      ulNotifsCount,
                     LPNOTIFICATION             pNotifs);
    BOOL WINAPI IsObjectInNotifList
                    (PRIVATE_ENTRYID *          pEID);
    BOOL WINAPI IsValidMsgClass
                    (LPTSTR                     pszMsgClass);
    STDMETHODIMP SubmitToSpoolerContextStore
                    (FILETIME                   ftSubmitTime,
                     PRIVATE_ENTRYID            SubmittedMsgEID);
    STDMETHODIMP AddMsgToOutGoingQueue
                    (FILETIME                   ftSubmitTime,
                     PRIVATE_ENTRYID            SubmittedMsgEID);
    STDMETHODIMP UpdateFolder
                    (BOOL                       fUpdateOutgoing,
                     PRIVATE_ENTRYID *          pParentEID, 
                     PRIVATE_ENTRYID *          pMsgEID);
    STDMETHODIMP AddNewRFSEntryToTable
                    (const NEW_RFS_NOTIF_DATA & info);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMsgStore       (CMSProvider *              pMSProviderObj,
                     LPMAPISUP                  pSupObj,
                     LPTSTR                     pszFilePath,
                     LPTSTR                     pszStoreName,
                     LPTSTR                     pszComment,
                     SBinary                    StoreEID,
                     CBackEnd *                 pBEobj,
                     BOOL                       fWriteAllowed,
                     long                       lAccess,
                     MAPIUID                    uidStore,
                     BOOL                       fSpoolerContext);
    ~CMsgStore      ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    CMSProvider *       m_pProvObj;
    ULONG               m_ulNotifAllObjects;
    PNOTIF_OBJS         m_pNotifList;
    LPTABLEDATA         m_pRFSTbl;
    LPTABLEDATA         m_pOutQueueTbl;
    DWORD               m_dwRFSTableViews;
    DWORD               m_dwOQTableViews;
    char                m_szFile[_MAX_PATH];
    char                m_szIPCNotifKey[64];
    ULONG               m_ulCachedStoreLogOffFlags;
    
public :
    COpenObjectList     m_OpenObjList;
    ULONG               m_ulIntConnection;
    HINSTANCE           m_hInstance;
    LPMAPISUP           m_pSupObj;
    MAPIUID		        m_StoreUID;
    SBinary             m_StoreEID;
    CMsgStore *         m_pNext;
};

class CMSLogon : public CGeneric, public IMSLogon
{
friend class CMSProvider;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMSLOGON_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    STDMETHODIMP InitializeStatusRow
                    ();

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CMSLogon        (CMsgStore *                pMsgStoreObj);
    ~CMSLogon       ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    CMsgStore *         m_pStore;
    CMSLogon *          m_pNext;
};

class CMSProvider : public IMSProvider
{
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMSPROVIDER_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    void WINAPI AddStoreObjectToList
                    (CMsgStore *                pMsgStoreObj);
    void WINAPI RemoveOpenStoreObject
                    (CMsgStore *                pStore);
    STDMETHODIMP GetProfileProps
                    (LPMAPISUP                  pSupObj,
                     LPSPropValue *             ppProps,
	                 HWND                       hOwnerWnd);
    BOOL WINAPI IsStoreAlreadyOpen
                    (LPTSTR                     pszStorePath,
                     CMsgStore **               ppStoreObj);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMSProvider     (HINSTANCE                  hInst);
    ~CMSProvider    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG               m_cRef; 
    CMsgStore *         m_pMsgStoreList;
    CRITICAL_SECTION    m_csObj;

public :
    HINSTANCE           m_hInstance;
};

class CMAPIFolder : public CGenericProp, public IMAPIFolder
{
friend void STDAPICALLTYPE FolderTablesViewRelease (ULONG, LPTABLEDATA, LPMAPITABLE);
friend class CFileBackEnd;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMAPICONTAINER_METHODS(IMPL);
    MAPI_IMAPIFOLDER_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
private :
    STDMETHODIMP UpdatePropertyCache
                    (CPropList *                pSourceList);
    STDMETHODIMP UpdatePropCacheInReplicas
                    ();
    STDMETHODIMP DeleteFolderContents
                    (PRIVATE_ENTRYID            ParentFolderEID,
                     PRIVATE_ENTRYID            FolderEID,
                     ULONG                      ulUIParam,
                     LPMAPIPROGRESS             pProgress,
                     ULONG                      ulFlags);
public:
    STDMETHODIMP UpdateParentTable
                    ();
    STDMETHODIMP UpdateContentTables
                    (UPDATE_TABLE_ACTION        Action,
                     BOOL                       fAssociatedCount,
                     BOOL                       fUnReadCount,
                     PRIVATE_ENTRYID *          pMessageEID,
                     CPropList *                pMsgProps,
                     BOOL                       bRecursing = FALSE);
    inline STDMETHODIMP CommitChanges
                    () { return CGenericProp::SaveChanges (KEEP_OPEN_READWRITE, NULL); }

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMAPIFolder     (CMsgStore *                pStore,
                     PRIVATE_ENTRYID *          pParentEID,
                     PRIVATE_ENTRYID *          pFolderEID,
                     long                       lFolderType,
                     CPropList *                pList,
                     BOOL                       fDeferErrors,
                     long                       lAccess,
                     CNamePropMgr *             pNPList);
    ~CMAPIFolder    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private:
    long                m_lFolderType;
    CMsgStore *         m_pStore;
    DWORD               m_dwHierTableViews;
    DWORD               m_dwContTableViews;
    DWORD               m_dwAsscContTableViews;
public:
    LPTABLEDATA         m_pHierTable;
    LPTABLEDATA         m_pContTable;
    LPTABLEDATA         m_pAsscContTable;
};

class CMessage : public CGenericProp, public IMessage
{
friend class CAttach;
friend void STDAPICALLTYPE MessageTablesViewRelease (ULONG, LPTABLEDATA, LPMAPITABLE);
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMESSAGE_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
public :
    STDMETHODIMP UpdateAttachmentTable
                    (CPropList *                pList,
                     PRIVATE_ENTRYID            AttachEID,
                     UPDATE_TABLE_ACTION        utaChange);
    STDMETHODIMP InternalSetReadFlag
                    (ULONG                      ulFlags,
                     BOOL                       fSentFolderNotification);
    STDMETHODIMP SaveChangesNow
                    (ULONG                      ulFlags);
    void WINAPI LoadObjectTables
                    ();

private :
    STDMETHODIMP GenerateDisplayNames
                    (LPMAPITABLE                pTable);
    STDMETHODIMP AddNameToString
                    (LPTSTR                     pszNewName,
                     LPTSTR *                   ppszNameList);
    STDMETHODIMP SetNormalizedSubject
                    (LPTSTR                     pszSubject,
                     LPTSTR *                   ppNormSubject);
    STDMETHODIMP PrepareRecipsForSubmission
                    ();
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMessage        (CMsgStore *                pStore,
                     PRIVATE_ENTRYID *          pParentEID,
                     PRIVATE_ENTRYID *          pMsgEID,
                     CPropList *                pList,
                     BOOL                       fDeferErrors,
                     long                       lAccess,
                     CAttach *                  pParentAttObj,
                     CNamePropMgr *             pNPList);
    ~CMessage       ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    CMsgStore *             m_pStore;
    CAttach *               m_pParentAttObj;
    LPTABLEDATA             m_pAttTable;
    LPTABLEDATA             m_pRecipTable;
    DWORD                   m_dwRecipTableViews;
    DWORD                   m_dwAttTableViews;
    DWORD                   m_dwRecipRowID;
    BOOL                    m_fRecipTableDirty;
    TRANSACTED_OBJECTS *    m_pActionList;
};

class CAttach : public CGenericProp, public IAttach
{
friend class CMessage;
public :
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
public :
    STDMETHODIMP SaveChangesNow
                    (ULONG                      ulFlags);
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CAttach         (CMsgStore *                pStore,
                     PRIVATE_ENTRYID *          pParentEID,
                     PRIVATE_ENTRYID *          pAttachEID,
                     CPropList *                pList,
                     BOOL                       fDeferErrors,
                     long                       lAccess,
                     CMessage *                 pMsgObj,
                     CNamePropMgr *             pNPList);
    ~CAttach        ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    CMsgStore *             m_pStore;
    CMessage *              m_pParentMsgObj;
    TRANSACTED_OBJECTS *    m_pActionList;
};

#endif // _MSLMS_H

// End of file for MSLMS.H
