///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      MSSRVCE.CPP
//
//  Description
//      This file implements some of the entry point (MSProviderInit and 
//      ServiceEntry) for this provider DLL.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#include "MSLMS.H"
#include "CFILESTG.H"

// Remark this line to turn verbose tracing OFF
//#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)   TraceMessage(a)
#else
#define InfoTrace(a)   
#endif // DO_INFO_TRACES

extern "C"
{
    HRESULT WINAPI InitializeStoreFile
                    (HINSTANCE                  hInstance,
                     HWND                       hOwnerWnd,
                     LPTSTR                     pszFileName,
                     MAPIUID *                  pUIDFile,
                     LPSPropValue               pProps,
                     LPMAPISUP                  pSupObj,
                     LPPROFSECT                 pProfObj);
    HRESULT WINAPI OpenProviderProfileSection
                    (LPPROVIDERADMIN,
                     LPPROFSECT *);
    MSGSERVICEENTRY ServiceEntry;
    STDINITMETHODIMP MSProviderInit
                    (HINSTANCE,
                     LPMALLOC,
                     LPALLOCATEBUFFER,
                     LPALLOCATEMORE,
                     LPFREEBUFFER,
                     ULONG,
                     ULONG,
                     ULONG *,
                     LPMSPROVIDER *);
};

// Static data for the message store provider configuration property sheets
char szFilter[] = "*"; // "*" to allow any character

// Control description structures
DTBLLABEL DtLabel = {sizeof(DTBLLABEL), fMapiUnicode};
DTBLPAGE DtPage  = {sizeof(DTBLPAGE), fMapiUnicode};
DTBLEDIT DtCtlStorePath    = {sizeof(DTBLEDIT), fMapiUnicode, _MAX_PATH, PR_SMP_STORE_PATH};
DTBLEDIT DtCtlStoreName    = {sizeof(DTBLEDIT), fMapiUnicode, 31, PR_DISPLAY_NAME};
DTBLEDIT DtCtlStoreComment = {sizeof(DTBLEDIT), fMapiUnicode, 63, PR_COMMENT};

// Description table for the controls in the first page
#define NUMBER_OF_CONTROLS  7
DTCTL ConfigPage[] =
{
    {DTCT_PAGE,    0,           NULL, 0, NULL,     0,                &DtPage           },
    {DTCT_EDIT,    DT_REQUIRED, NULL, 0, szFilter, IDC_STOREPATH,    &DtCtlStorePath   },
    {DTCT_EDIT,    DT_REQUIRED, NULL, 0, szFilter, IDC_STORENAME,    &DtCtlStoreName   },
    {DTCT_EDIT,    0,           NULL, 0, szFilter, IDC_STORECOMMENT, &DtCtlStoreComment},
    {DTCT_LABEL,   0,           NULL, 0, NULL,     IDC_STATIC1,      &DtLabel          },
    {DTCT_LABEL,   0,           NULL, 0, NULL,     IDC_STATIC2,      &DtLabel          },
    {DTCT_LABEL,   0,           NULL, 0, NULL,     IDC_STATIC3,      &DtLabel          }
};

DTPAGE DtPropPage;

///////////////////////////////////////////////////////////////////////////////
//    ServiceEntry()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by the profile setup adminitration tool to display the provider
//      configuration properties for this message provider
//
//    Return Value
//      An HRESULT
//
HRESULT STDAPICALLTYPE ServiceEntry (HINSTANCE       hInstance,
                                     LPMALLOC        pMallocObj,
                                     LPMAPISUP       pSupObj,
                                     ULONG           ulUIParam,
                                     ULONG           ulFlags,
                                     ULONG           ulContext,
                                     ULONG           ulCfgPropCount,
                                     LPSPropValue    pCfgProps,
                                     LPPROVIDERADMIN pAdminProvObj,
                                     LPMAPIERROR *   ppMAPIError)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    // Send output only to the COM1 port
    //InitTraces (0);
    // Send output to a console window BUT NOT to the COM1
    InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);
    // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);

    InfoTrace ("ServiceEntry function called");
    HRESULT hResult = S_OK;
    BOOL fDeInitMemoryHeap = FALSE;
    CGeneric obj;
    
    // What context where were we called in?
    if (MSG_SERVICE_UNINSTALL       == ulContext ||     // We don't support these contexts
        MSG_SERVICE_INSTALL         == ulContext ||
        MSG_SERVICE_PROVIDER_CREATE == ulContext ||
        MSG_SERVICE_PROVIDER_DELETE == ulContext ||
        MSG_SERVICE_DELETE          == ulContext)
    {
        // We don't handle any of these contexts
        TraceString1 ("ServiceEntry: Immediate return. Context: %d", ulContext);
        return hResult;
    }
    TraceString1 ("ServiceEntry: Context for call: %d", ulContext);
        
    // If we got called to create the initial files WITHOUT allowing UI,
    // then the providers in being added through the profile wizard. We
    // don't do anything now. After the wizard has collected all the
    // properties we need, we will be called again with MSG_SERVICE_CONFIGURE
    // and a property array in pCfgProps
    if (MSG_SERVICE_CREATE == ulContext && !(SERVICE_UI_ALWAYS & ulFlags))
    {
        return hResult;
    }

    hResult = InitMemoryHeap (pSupObj);
    if (FAILED(hResult)) // It may return a warning
    {
        goto ErrorExit;
    }
    if (S_OK == hResult)
    {
        fDeInitMemoryHeap = TRUE;
    }
    
    // Open the Profile Section for this message store provider
    LPPROFSECT pProfileObj;
    hResult = OpenProviderProfileSection (pAdminProvObj, &pProfileObj);
    if (hResult)
    {
        goto ErrorExit;
    }
    
    // Profile is open, get the properties out of it
    LPSPropValue pProps;
    pProps = NULL;
    ULONG ulPropCount;
    hResult = pProfileObj->GetProps ((LPSPropTagArray)&sptLogonArray,
                                     fMapiUnicode,
                                     &ulPropCount,
                                     &pProps);
    if (FAILED(hResult))
    {
        TraceResult ("ServiceEntry: Failed to get the properties from the profile", hResult);
        goto ServiceEntryFinished;
    }
    ASSERT (NUM_LOGON_PROPERTIES == ulPropCount);

    char szFileName[_MAX_PATH], szDefaultName[32+1], szDefaultComment[32+1];
    MAPIUID uidStore;
    if (MSG_SERVICE_CREATE == ulContext || (MSG_SERVICE_CONFIGURE == ulContext && pCfgProps && ulCfgPropCount))
    {
        lstrcpy (szDefaultName, "Sample Mailbox");
        ZeroMemory (szDefaultComment, sizeof(szDefaultComment));
        ZeroMemory (&uidStore, sizeof(MAPIUID));
        if (pCfgProps)
        {
            ASSERT (pCfgProps->ulPropTag == PR_SMP_STORE_PATH);
            CharUpper (pCfgProps->Value.lpszA);
            pProps[STORE_PATH].ulPropTag = PR_SMP_STORE_PATH;
            pProps[STORE_PATH].Value.lpszA = pCfgProps->Value.lpszA;
        }

        pProps[STORE_NAME].ulPropTag = PR_DISPLAY_NAME;
        pProps[STORE_NAME].Value.lpszA = szDefaultName;

        pProps[STORE_COMMENT].ulPropTag = PR_COMMENT;
        pProps[STORE_COMMENT].Value.lpszA = szDefaultComment;

        pProps[STORE_PASSWORD].ulPropTag = PR_SMP_PASSWORD;
        pProps[STORE_PASSWORD].Value.lpszA = szBlank;

        pProps[STORE_REMEM_PWD].ulPropTag = PR_SMP_REMEMBER_PWD;
        pProps[STORE_REMEM_PWD].Value.b = TRUE;
        hResult = InitializeStoreFile (hInstance,
                                       (HWND)ulUIParam,
                                       szFileName,
                                       &uidStore,
                                       pProps,
                                       pSupObj,
                                       pProfileObj);
        if (hResult)
        {
            goto ServiceEntryFinished;
        }
        pProps[STORE_FILE_UID].ulPropTag = PR_STORE_RECORD_KEY;
        pProps[STORE_FILE_UID].Value.bin.cb = sizeof(MAPIUID);
        pProps[STORE_FILE_UID].Value.bin.lpb = (LPBYTE)&uidStore;
    }

    // If we can, display the UI...
    if (SERVICE_UI_ALWAYS & ulFlags || SERVICE_UI_ALLOWED & ulFlags)
    {
        PrivInitialize3DCtl (hInstance);
        CFGDLG CfgDialog = { 0 };
        CfgDialog.hInst       = hInstance;
        CfgDialog.hWnd        = (HWND)ulUIParam;
        CfgDialog.ppPropArray = &pProps;
        CfgDialog.pTags       = (LPSPropTagArray)&sptLogonArray;
        CfgDialog.pSupObj     = pSupObj;
        CfgDialog.ulFlags     = MSG_SERVICE_UI_READ_ONLY & ulFlags ? UI_READONLY : 0;
        hResult = DoLogonDlg (&CfgDialog);
        if (hResult)
        {
            goto ServiceEntryFinished;
        }
    }

    hResult = pProfileObj->SetProps (ulPropCount, pProps, NULL);
    TraceResult ("ServiceEntry: Failed to save the properties in the profile", hResult);
    
ServiceEntryFinished:
    pProfileObj->Release();
    PrivMAPIFree ((LPVOID *)&pProps);

ErrorExit:  
    // In case the DLL was loaded, unload it.
    PrivUninitialize3DCtl (hInstance);
      
    // Close down the traces
    UnInitTraces();
    if (hResult)
    {
        obj.SetLastError (hResult);
        obj.GetLastError (hResult, fMapiUnicode, ppMAPIError, hInstance, PROVIDER_DISPLAY_NAME_STRING);
    }
    if (fDeInitMemoryHeap)
    {
        CloseMemoryHeap();
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    InitializeStoreFile()
//
//    Parameters
//      [IN]        hInstance       Instance that owns any UI 
//      [IN]        hOwnerWnd       Windows that owns any UI
//      [IN]        pszFileName     String with the name of the file
//      [IN,OUT]    pUIDStore       Where we return the UID of the database
//                                  file. If we are creating a new file, a new
//                                  UID is allocated, otherwise we read the UID
//                                  from the existing file.
//      [IN]        pProps          Array of properties from the configuration
//      [IN]        pSupObj         Support object used to allocate a UID
//      [IN]        pProfObj        Pointer to the profile object
//
//    Purpose
//      This function initializes the message store database file and
//      registers standard MAPI properties into the profile section which are
//      needed for message store providers.
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI InitializeStoreFile (HINSTANCE    hInstance,
                                    HWND         hOwnerWnd,
                                    LPTSTR       pszFileName,
                                    MAPIUID *    pUIDStore,
                                    LPSPropValue pProps,
                                    LPMAPISUP    pSupObj,
                                    LPPROFSECT   pProfObj)
{
    if (pProps[STORE_PATH].ulPropTag != PR_SMP_STORE_PATH)
    {
        // Display the dialog box now
        if (!BrowseFileDialog (hInstance, hOwnerWnd, pszFileName))
        {
            // If the user cancelled the dialog or we failed for some reason, then the
            // service has not been configured yet and we must tell this to MAPI.
            return MAPI_E_UNCONFIGURED;
        }
        CharUpper (pszFileName);
        pProps[STORE_PATH].ulPropTag = PR_SMP_STORE_PATH;
        pProps[STORE_PATH].Value.lpszA = pszFileName;
    }

    HRESULT hResult;
    HANDLE hFile; 
    // If the file already exist we assume is one of our files, so we won't
    // generate a new UID. UID will be read from the message file instead.
    hFile = CreateFile (pszFileName,
                        GENERIC_READ,
                        FILE_SHARE_READ | FILE_SHARE_WRITE,
                        NULL,
                        OPEN_EXISTING,
                        FILE_ATTRIBUTE_NORMAL,
                        NULL);
    if (INVALID_HANDLE_VALUE == hFile)
    {
        // Get a new UID for this message store. This is used as the
        // as the UID contained in EntryIDs of objects in this database and others
        // unique long term entry ID's (i.e. PR_STORE_ENTRYID)
        hResult = pSupObj->NewUID (pUIDStore);
        if (hResult)
        {
            TraceResult ("InitializeStoreFile: Failed to get new UID", hResult);
            return hResult;
        }
    }
    else
    {
        CloseHandle (hFile);
    }

    CBackEnd * pObj;
    hResult = OpenMsgStoreDataBase (pSupObj,
                                    pProps[STORE_PATH].Value.lpszA,
                                    pProps[STORE_NAME].Value.lpszA,
                                    pProps[STORE_COMMENT].Value.lpszA,
                                    pUIDStore,
                                    TRUE,
                                    &pObj);
    if (hResult)
    {
        return hResult;
    }
    pObj->Release();

    // Set the profile properties for a our message store.
    // These properties are required.
    SPropValue rgProps[7];

    rgProps[0].ulPropTag = PR_PROVIDER_DISPLAY;
    rgProps[0].Value.lpszA = PROVIDER_DISPLAY_NAME_STRING;

    rgProps[1].ulPropTag = PR_DISPLAY_NAME;
    rgProps[1].Value.lpszA = pProps[STORE_NAME].Value.lpszA;
    
    // Use our helper function to create a new store entry ID to
    // register with the system. A store entry ID is composed of
    // our native entry ID information plus some more information
    // MAPI wraps around it.
    SBinary StoreEID;
    hResult = CreateStoreEntryID (pUIDStore, &StoreEID);
    if (hResult)
    {
        return hResult;
    }
    rgProps[2].ulPropTag = PR_ENTRYID;
    rgProps[2].Value.bin.cb = StoreEID.cb;
    rgProps[2].Value.bin.lpb = (LPBYTE)StoreEID.lpb;

    rgProps[3].ulPropTag = PR_STORE_ENTRYID;
    rgProps[3].Value.bin.cb = StoreEID.cb;
    rgProps[3].Value.bin.lpb = (LPBYTE)StoreEID.lpb;

    rgProps[4].ulPropTag = PR_SMP_STORE_PATH;
    rgProps[4].Value.lpszA = pProps[STORE_PATH].Value.lpszA;

    rgProps[5].ulPropTag = PR_MDB_PROVIDER;
    rgProps[5].Value.bin.cb = sizeof(MAPIUID);
    rgProps[5].Value.bin.lpb = (LPBYTE)&guidMSProvider;

    rgProps[6].ulPropTag = PR_RECORD_KEY;
    rgProps[6].Value.bin.cb = sizeof(MAPIUID);
    rgProps[6].Value.bin.lpb = (LPBYTE)pUIDStore;

    hResult = pProfObj->SetProps (sizeof(rgProps)/sizeof(SPropValue), rgProps, NULL);
    PrivMAPIFree ((LPVOID *)&StoreEID.lpb);
    TraceResult ("InitializeStoreFile: Failed to set the properties into the profile", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    OpenProviderProfileSection()
//
//    Parameters
//      [IN]    pAdminProvObj   Pointer to an IProviderAdmin object
//      [OUT]   ppProfSectObj   Pointer to a pointer where this function
//                              returns an IProfSect object open through the
//                              pAdmingProvObj object
//
//    Purpose
//      This function opens the profile section where this provider stores its
//      settings for the current profile
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI OpenProviderProfileSection (LPPROVIDERADMIN pAdminProvObj,
                                           LPPROFSECT *    ppProfSectObj)
{
    // Get the provider table where the settings are stored
    LPMAPITABLE pTableObj;
    HRESULT hResult = pAdminProvObj->GetProviderTable (0, &pTableObj);
    if (hResult)
    {
        TraceResult ("OpenProviderProfileSection: GetProviderTable failed", hResult);
        return hResult;
    }

    // Since we are going only looking for one property, limit the view
    // of the columns to what we are interested in
    SPropTagArray sptProvider = { 1, { PR_PROVIDER_UID } };
    hResult = pTableObj->SetColumns (&sptProvider, 0);
    if (SUCCEEDED(hResult))
    {
        // Query only one row, the user's row with the settings in it
        LPSRowSet pRows;
        hResult = pTableObj->QueryRows (1, 0, &pRows);
        if (SUCCEEDED(hResult))
        {
            ASSERT (PR_PROVIDER_UID == pRows->aRow->lpProps->ulPropTag);
            hResult = pAdminProvObj->OpenProfileSection ((LPMAPIUID)pRows->aRow->lpProps->Value.bin.lpb,
                                                         NULL,
                                                         MAPI_MODIFY,
                                                         ppProfSectObj);
            FreeProws (pRows);
            if (hResult)
            {
                // Report failure, if any
                TraceResult ("OpenProviderProfileSection: OpenProfileSection failed", hResult);
                hResult = MAPI_E_NOT_FOUND;
            }
        }
        else
        {
            TraceResult ("OpenProviderProfileSection: Failed to query the rows", hResult);
        }
    }
    else
    {
        TraceResult ("OpenProviderProfileSection: Failed to set the columns", hResult);
    }
    pTableObj->Release();
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    DoLogonDlg()
//
//    Parameters
//      [IN]    pDialog     Pointer to a CFGDLG structure.
//
//    Purpose
//      This functions gets called to display the configuration properties for
//      this provider. When invoked, it will parse the message store logon 
//      property array and initialize the fields in the property sheet with it.
//      The function could also get invoked to display the settings in 
//      READ-ONLY mode
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI DoLogonDlg (PCFGDLG pDialog)
{
    // Take ownership of the property value array, so the calling
    // code will not attempt to free this block
    LPSPropValue pProps = *pDialog->ppPropArray;
    *pDialog->ppPropArray = NULL;
    
    // Retrieve the dialog title string
    char szDialogTitle[128];
    LoadString (pDialog->hInst, IDS_MSG_CONFIG_TITLE, szDialogTitle, 128);
    
    // Now make a temporary property interface and feed it the data
    LPPROPDATA pPropDataObj = NULL;
    LPMAPITABLE pTableObj   = NULL;
    HRESULT hResult = CreateIProp ((LPIID)&IID_IMAPIPropData,
                                   GetMAPIAllocBuffer(),
                                   GetMAPIAllocMore(),
                                   GetMAPIFreeBuffer(),
                                   NULL,
                                   &pPropDataObj);
    if (hResult)
    {   
        TraceResult ("DoLogonDlg: CreateIProp failed to create IPropData object", hResult);
        goto DoLogonDlgFinished;
    }
    
    // Set the properties in the property array. The properties MUST exist in
    // the array before invoking the property page with MAPI methods, 
    // otherwise the call fails
    hResult = pPropDataObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);    
    if (hResult)
    {
        TraceResult ("DoLogonDlg: The logon propertiest SetProps failed", hResult);
        goto DoLogonDlgFinished;
    }

    DtPropPage.cctl = NUMBER_OF_CONTROLS; // Count of controls in the ConfigPage array,
    DtPropPage.lpszComponent = szBlank;
    DtPropPage.lpszResourceName = MAKEINTRESOURCE (IDD_CONFIGPAGE);
    DtPropPage.lpctl = ConfigPage;

    // If the UI_READONLY flag is set, then we are being called from the 
    // SettingDialog method in Read Only mode. Disable all of the controls
    // on the property sheet, else Enable them
    if (pDialog->ulFlags & UI_READONLY)
    {
        // Disable the controls (make them read only)
        DtPropPage.lpctl[2].ulCtlFlags &= ~DT_EDITABLE;
        DtPropPage.lpctl[3].ulCtlFlags &= ~DT_EDITABLE;
    }
    else
    {
        // Enable the controls (make them read/write)
        DtPropPage.lpctl[2].ulCtlFlags |= DT_EDITABLE;
        DtPropPage.lpctl[3].ulCtlFlags |= DT_EDITABLE;
    }
        
    // Create the display table for the logon dialog. It is based on dialog
    // resources plus the static information at the head of this module.       
    hResult = BuildDisplayTable (GetMAPIAllocBuffer(),
                                 GetMAPIAllocMore(),
                                 GetMAPIFreeBuffer(),
                                 NULL,
                                 pDialog->hInst,
                                 1,
                                 &DtPropPage,
                                 fMapiUnicode,
                                 &pTableObj,
                                 NULL);        
    if (hResult)
    {
        TraceResult ("DoLogonDlg: BuildDisplayTable failed", hResult);
        goto DoLogonDlgFinished;
    }

    // Display the dialog/property sheets
    hResult = pDialog->pSupObj->DoConfigPropsheet ((ULONG)pDialog->hWnd,
                                                   0,
                                                   szDialogTitle,
                                                   pTableObj,
                                                   (LPMAPIPROP)pPropDataObj,
                                                   0);     // Default index for the top property sheet
    if (hResult)
    {
        if (MAPI_E_USER_CANCEL != hResult)
        {
            TraceResult ("DoLogonDlg: DoConfigPropsheet failed", hResult);
        }
        goto DoLogonDlgFinished;
    }

    // Retrieve the altered data
    PrivMAPIFree ((LPVOID *)&pProps);
    pProps = NULL;
    ULONG ulPropCount;
    hResult = pPropDataObj->GetProps (pDialog->pTags,
                                      fMapiUnicode,
                                      &ulPropCount,
                                      &pProps);
    if (FAILED(hResult))
    {
        TraceResult ("DoLogonDlg: GetProps failed", hResult);
        goto DoLogonDlgFinished;
    }
        
    // Return the new property array
    *(pDialog->ppPropArray) = pProps;
    pProps = NULL;  // Do not free this in cleanup code below
    
DoLogonDlgFinished:
    // Release objects
    if (pTableObj)
    {
        pTableObj->Release();
    }
    if (pPropDataObj)
    {
        pPropDataObj->Release();
    }    
    PrivMAPIFree ((LPVOID *)&pProps);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    MSProviderInit()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Entry pointe called by MAPI when a profile uses this 
//      provider. MAPI calls this method and expects a pointer to an
//      implementation of the IMSProvider interface. MAPI uses the returned
//      IMSProvider interface pointer to logon onto the message store provider
//
//    Return Value
//      An HRESULT
//
STDINITMETHODIMP MSProviderInit (HINSTANCE        hInstance, 
                                 LPMALLOC         pMallocObj,
                                 LPALLOCATEBUFFER pfnAllocateBuffer,
                                 LPALLOCATEMORE   pfnAllocateMore,
                                 LPFREEBUFFER     pfnFreeBuffer,
                                 ULONG            ulFlags,
                                 ULONG            ulMAPIVer,
                                 ULONG *          pulProviderVer,
                                 LPMSPROVIDER *   ppMSProviderObj)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    // Send output only to the COM1 port
    //InitTraces (0);
    // Send output to a console window BUT NOT to the COM1
    InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);
    // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);

    InfoTrace ("MSProviderInit function called");
    *pulProviderVer = CURRENT_SPI_VERSION;
    if (ulMAPIVer < CURRENT_SPI_VERSION)
    {
        TraceMessage ("MSProviderInit: The version of the subsystem cannot handle this version of the provider");
        return MAPI_E_VERSION;
    }
    HRESULT hResult = InitMemoryHeap (pfnAllocateBuffer, pfnAllocateMore, pfnFreeBuffer);
    if (FAILED(hResult))
    {
        return hResult;
    }
    hResult = S_OK;

    // Allocate space for the MSProvider object, the constructor will be called to initialize member variables
    CMSProvider * pMSProvider = new CMSProvider (hInstance);
    if (NULL == pMSProvider)
    {
        TraceMessage ("MSProviderInit: Failed to allocate new CMSProvider object");
        hResult = E_OUTOFMEMORY;
    }
    // Copy pointer to the allocated object back into the return IMSProvider object pointer
    *ppMSProviderObj = (LPMSPROVIDER)pMSProvider;
    if (hResult)
    {
        CloseMemoryHeap();
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CreateStoreEntryID()
//
//    Parameters
//      [IN]    pStoreUID   UID of the database file for this message store.
//      [OUT]   pStoreEID   Pointer to a SBinary structure allocated by the
//                          caller, where we returned the entry ID of the store
//                          as returned by MAPI. The memory in the lpb member,
//                          should be released using the MAPI allocation
//                          function.
//
//    Purpose
//      Creates an entry ID that will be returned to the MAPI subsystem which
//      identifies our store in this session and profile.
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI CreateStoreEntryID (MAPIUID * pStoreUID, LPSBinary pStoreEID)
{
    PRIVATE_ENTRYID eid = { 0 }; // Initialize the contents to 0
    eid.uidGlobal = *pStoreUID;
	eid.bVersion = ENTRYID_VERSION;
    eid.bObject = MAPI_STORE;
    HRESULT hResult = WrapStoreEntryID (0,
                                        MS_DLL_NAME_STRING,
                                        CB_PRIVATE_EID,
                                        (LPENTRYID)&eid,
                                        &pStoreEID->cb,
                                        (LPENTRYID *)&pStoreEID->lpb);
    // If any, trace out the error.
    TraceResult ("CreateStoreEntryID: Failed to wrap the store entry ID", hResult);
    return hResult;
}

// End of file for MSSRVCE.CPP
