///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      MSSRVBE.H
//              
//  Description
//      This file implements a class derived from the abstract class CBackEnd
//      used by the sample providers to access the provider's data storage.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _MSSRVBE_H
#define _MSSRVBE_H

#include "BECOMMON.H"

#ifdef UNICODE
#error MSWDS: We don't support UNICODE yet... IrvingD
#endif // UNICODE

extern "C"
{
    HRESULT WINAPI OpenRemoteDataBase
                    (LPTSTR                 pszServer,
                     MAPIUID *              pUID,
                     CBackEnd **            ppBackEnd);
};

class CPFServerBackEnd : public CBackEnd
{
public:
///////////////////////////////////////////////////////////////////////////////
// CBackEnd members implemented here
//
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    STDMETHODIMP CloseResources
                    () { return S_OK; };
    STDMETHODIMP CreateObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID *          pNewObjEID,
                     CPropList **               ppList,
                     ULONG                      ulObjType,
                     LPVOID                     pData1,
                     LPVOID                     pData2);
    STDMETHODIMP OpenObject
                    (PRIVATE_ENTRYID            ObjectEID,
                     CPropList **               ppList);
    STDMETHODIMP SaveObject
                    (PRIVATE_ENTRYID            ParentEID,
                     PRIVATE_ENTRYID            ObjectEID,
                     CPropList *                pList,
                     COpenPropList *            pOpenList,
                     LPTABLEDATA                pTable);
    STDMETHODIMP OpenPropertyInMemory
                    (PRIVATE_ENTRYID            ObjectEID,
                     ULONG                      ulMaxBytes,
                     LPVOID                     pMemBlock,
                     LPSPropValue               pProp)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP OpenProperty
                    (PRIVATE_ENTRYID            ObjectEID,
                     LPSPropValue               pProp,
                     LPUNKNOWN *                ppUnk)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP CopyObjects
                    (PRIVATE_ENTRYID            SrcParentContainerEID,
                     PRIVATE_ENTRYID            SrcContainerEID,
                     PRIVATE_ENTRYID            DstParentContainerEID,
                     PRIVATE_ENTRYID            DstContainerEID,
                     DWORD                      dwCopyFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP DeleteOneObject
                    (PRIVATE_ENTRYID            ObjectEID,
                     PRIVATE_ENTRYID            ParentEID);
    STDMETHODIMP DeleteObjects
                    (PRIVATE_ENTRYID            ContainerEID,
                     ULONG                      ulDeleteFlags,
                     LPMAPIPROGRESS             pProgress,
                     POPENOBJECTS_DATA          pData);
    STDMETHODIMP GetTable
                    (const TABLE_TYPE           tblType,
                     PRIVATE_ENTRYID            ContainerEID,
                     long                       lAccess,
                     LPVOID                     pData1,
                     LPVOID                     pData2,
                     LPTABLEDATA *              ppTable);
    STDMETHODIMP GetTableProps
                    (const TABLE_TYPE           tblType,
                     long                       lAccess,
                     PRIVATE_ENTRYID *          ParentEID,
                     PRIVATE_ENTRYID *          ObjectEID,
                     CPropList *                pList,
                     LPSPropValue *             ppProps,
                     DWORD &                    dwLastAllocated,
                     LPTABLEDATA                pTable,
                     LPVOID                     pData1,
                     LPVOID                     pData2);
    STDMETHODIMP UpdateContentCount
                    (PRIVATE_ENTRYID            ContainerEID,
                     PRIVATE_ENTRYID            ObjectEID,
                     BOOL                       fAssociated,
                     BOOL                       fUnread)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP SetRFSEntryID
                    (LPTSTR                     pszMsgClass,
                     PRIVATE_ENTRYID            FolderEID,
                     LPTABLEDATA                pTable)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP GetRFSEntryID
                    (LPTSTR                     pszMsgClass,
                     PRIVATE_ENTRYID *          pFolderEID,
                     LPTSTR                     pszExplicitClass)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP SetObjNameInContainer
                    (LPTSTR                     pszNewFolderName,
                     PRIVATE_ENTRYID            ParentFolderEID,
                     PRIVATE_ENTRYID            FolderEID)
                    { return MAPI_E_NO_SUPPORT; }
    STDMETHODIMP GetNamedPropertiesStream
                    (LPSTREAM *                 ppStream);
    STDMETHODIMP SetBackEndStrings
                    (LPTSTR                     szName,
                     LPTSTR                     szComment)
                    { return MAPI_E_NO_SUPPORT; }

///////////////////////////////////////////////////////////////////////////////
// CPFServerBackEnd members
//
private:
    STDMETHODIMP FillHierarchyTable
                    (PRIVATE_ENTRYID            FolderEID,
                     LPTABLEDATA                pTable,
                     LPSBinary                  pStoreEID,
                     long                       lFolderAccess);
    STDMETHODIMP FillContentsTable
                    (PRIVATE_ENTRYID            FolderEID,
                     LPTABLEDATA                pTable,
                     LPSBinary                  pStoreEID,
                     long                       lFolderAccess);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CPFServerBackEnd 
                    (MAPIUID *                  pUIDinFile,
                     LPTSTR                     pszRemoteServer);
    ~CPFServerBackEnd
                    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    ULONG               m_cRef;
    TCHAR               m_szServer[32];
};

// Minimum set of properties available in a folder contents table
#define MIN_CONTENTS_TABLE_PROPS    18
const static SizedSPropTagArray (MIN_CONTENTS_TABLE_PROPS, sptFolderContProps) =
{
    MIN_CONTENTS_TABLE_PROPS,
    {
        PR_INSTANCE_KEY,
        PR_ACCESS_LEVEL,
        PR_ENTRYID,
        PR_PARENT_ENTRYID,
        PR_STORE_ENTRYID,
        PR_STORE_RECORD_KEY,
        PR_RECORD_KEY,
        PR_MAPPING_SIGNATURE,
        PR_OBJECT_TYPE,
        PR_HASATTACH,
        PR_MESSAGE_FLAGS,
        PR_CREATION_TIME,
        PR_LAST_MODIFICATION_TIME,
        PR_MESSAGE_SIZE,
        PR_PRIORITY,
        PR_SEARCH_KEY,
        PR_MESSAGE_CLASS,
        PR_MSG_STATUS
    }
};

#endif // _MSSRVBE_H

// End of file for MSSRVBE.H
