///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      MSSRVCE.CPP
//
//  Description
//      This file implements the entry points, MSProviderInit and 
//      ServiceEntry for this provider's DLL.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#include "MSWDS.H"

// Remark this line to turn verbose tracing OFF
//#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)                TraceMessage(a)
#define InfoTraceString1(a, b)      TraceString1(a, b)
#else
#define InfoTrace(a)   
#define InfoTraceString1(a, b)
#endif // DO_INFO_TRACES

extern "C"
{
    char gszProviderName[] = "WINDS Public Folders";
    MAPIUID guidProvider = MS_STORE_UID_PROVIDER;
};

HRESULT WINAPI InitStoreProps (LPPROFSECT pProfObj, LPPROFSECT pMSProfObj);
HRESULT WINAPI OpenProviderProfileSection (LPPROVIDERADMIN, LPPROFSECT *);

///////////////////////////////////////////////////////////////////////////////
//    ServiceEntry()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by the profile setup adminitration tool to display the provider
//      configuration properties for this message provider
//
//    Return Value
//      An HRESULT
//
HRESULT STDAPICALLTYPE ServiceEntry (HINSTANCE          hInstance,
                                     LPMALLOC           pMallocObj,
                                     LPMAPISUP          pSupObj,
                                     ULONG              ulUIParam,
                                     ULONG              ulFlags,
                                     ULONG              ulContext,
                                     ULONG              ulCfgPropCount,
                                     LPSPropValue       pCfgProps,
                                     LPPROVIDERADMIN    pAdminProvObj,
                                     LPMAPIERROR *      ppMAPIError)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    // Send output only to the COM1 port
    //InitTraces (0);
    // Send output to a console window BUT NOT to the COM1
    InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);
    // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);

    InfoTrace ("ServiceEntry function called");
    CGeneric obj;
    HRESULT hResult = S_OK;
    LPPROFSECT pProfileObj = NULL, pMSProfileObj = NULL;
    ULONG ulPropCount;
    LPSPropValue pProps = NULL;
    HANDLE hUIMutex = NULL;
    // This is for our internal memory heap management
    BOOL fDeInitMemoryHeap = FALSE;
    
    // What context where were we called in?
    if (MSG_SERVICE_INSTALL         == ulContext ||
        MSG_SERVICE_UNINSTALL       == ulContext ||
        MSG_SERVICE_PROVIDER_CREATE == ulContext ||
        MSG_SERVICE_DELETE          == ulContext ||
        MSG_SERVICE_PROVIDER_DELETE == ulContext)
    {
        // We don't handle any of these contexts
        InfoTraceString1 ("ServiceEntry: Immediate return. Context: %d", ulContext);
        goto ErrorExit;
    }
    ghInstance = hInstance;
    InfoTraceString1 ("ServiceEntry: Context for call: %d", ulContext);
    hResult = InitMemoryHeap (pSupObj);
    if (FAILED(hResult)) // It may return a warning
    {
        return hResult;
    }
    if (S_OK == hResult)
    {
        fDeInitMemoryHeap = TRUE;
    }
        
    // Open the Profile Section for this service
    hResult = pAdminProvObj->OpenProfileSection (NULL,
                                                 NULL,
                                                 MAPI_MODIFY,
                                                 &pProfileObj);
    if (hResult)
    {
        goto ErrorExit;
    }
    // Open the Profile Section for this message store provider
    hResult = OpenProviderProfileSection (pAdminProvObj, &pMSProfileObj);
    if (hResult)
    {
        goto ErrorExit;
    }

    if (MSG_SERVICE_CREATE == ulContext)
    {
        ULONG i = 0;
        SPropValue spvProps[2] = { 0 };
        spvProps[i].ulPropTag = PR_COMMENT;
        spvProps[i++].Value.lpszA = szBlank;
        spvProps[i].ulPropTag = PR_SMP_CONNECTION_TYPE;
        spvProps[i++].Value.l = LAN_CONNECT;
        hResult = pProfileObj->SetProps (i, spvProps, NULL);
        TraceResult ("ServiceEntry: Failed to save the properties into the profile", hResult);
    }

    // Profile is open, get the properties out of it
    hResult = pProfileObj->GetProps ((LPSPropTagArray)&sptLogonProps,
                                     fMapiUnicode,
                                     &ulPropCount,
                                     &pProps);
    if (FAILED(hResult))
    {
        goto ErrorExit;
    }
    
    // After the setup wizard has been called the ServiceEntry functions gets called
    // with a set of properties obtained from the user for configuring this provider.
    // Merge or replace those properties onto the one we obtained from the profile section.
    if (pCfgProps && ulCfgPropCount)
    {
        MergeConfigProps (ulCfgPropCount, pCfgProps, pProps);
    }

    if (MSG_SERVICE_CREATE == ulContext || (MSG_SERVICE_CONFIGURE == ulContext && pCfgProps && ulCfgPropCount))
    {
        hResult = InitStoreProps (pProfileObj, pMSProfileObj);
        if (hResult)
        {
            goto ErrorExit;
        }
    }

    // If we can, display the UI...
    if (SERVICE_UI_ALWAYS & ulFlags || SERVICE_UI_ALLOWED & ulFlags)
    {
        CFGDLG CfgDialog = { 0 };
        // Get the mutex to gain exclusive access to the UI dialog
        // This prevent multiple processes and/or threads from displaying
        // the configuration dialog for this service
        hUIMutex = CreateMutex (NULL, FALSE, CONFIG_UI_MUTEX);
        if (NULL == hUIMutex)
        {
            TraceResult ("ServiceEntry: Failed to create UI mutext", HRESULT_FROM_WIN32(GetLastError()));
        }
        PrivInitialize3DCtl (hInstance);
        if (WAIT_TIMEOUT == WaitForSingleObject (hUIMutex, 250)) // Wait for the MUTEX for 1/4 second
        {
            PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE, (HWND)ulUIParam);
            hResult = MAPI_E_BUSY;
            goto ErrorExit;
        }

        // Fill in the logon UI structure
        CfgDialog.hInst    = hInstance;
        CfgDialog.hWnd     = (HWND)ulUIParam;
        CfgDialog.ppProps  = &pProps;
        CfgDialog.pSupObj  = pSupObj;
        CfgDialog.ulFlags  = MSG_SERVICE_UI_READ_ONLY & ulFlags ? UI_READONLY : 0;
        CfgDialog.hUIMutex = NULL; // So that the DoLogonDlg() won't have to wait.
ShowPropsAgain:
        hResult = DoLogonDlg (&CfgDialog);
        if (hResult)
        {
            goto ErrorExit;
        }
        if (!PingRemoteServer ((HWND)ulUIParam, pProps))
        {
            goto ShowPropsAgain;
        }
    }

    hResult = pProfileObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);
    TraceResult ("ServiceEntry: Failed to save the properties back into the profile", hResult);

ErrorExit:
    if (pProfileObj)
    {
        pProfileObj->Release();
    }
    if (pMSProfileObj)
    {
        pMSProfileObj->Release();
    }
    if (NULL != GetMAPIFreeBuffer())
    {
        PrivMAPIFree ((LPVOID *)&pProps);
    }
    if (hUIMutex)
    {
        ReleaseMutex (hUIMutex);
        CloseHandle (hUIMutex);
    }
    // In case the DLL was loaded, unload it.
    // This DLL gets loaded and initialized in the call to DoLogonDlg()
    PrivUninitialize3DCtl (hInstance);

    // If we have remote connections going on, close them now
    TerminateRemoteConnections();

    // Close down the traces
    UnInitTraces();
    if (hResult)
    {
        TraceResult ("ServiceEntry", hResult);
        obj.GetLastError (hResult, fMapiUnicode, ppMAPIError, hInstance, PROVIDER_DISPLAY_NAME_STRING);
    }
    if (fDeInitMemoryHeap)
    {
        CloseMemoryHeap();
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    DoLogonDlg()
//
//    Parameters
//      pCfgDialog   Pointer to an CFGDLG structure.
//
//    Purpose
//      This functions gets called to display the configuration properties for
//      this transport. When invoked, it will parse the transport logon 
//      property array and initialize the fields in the property sheet with it.
//      The function could also get invoked to display the settings in 
//      READ-ONLY mode in which case we disable writing to the editable fields.
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI DoLogonDlg (PCFGDLG pCfgDialog)
{
    DWORD dwWait = WaitForSingleObject (pCfgDialog->hUIMutex, 1000); // Wait for the MUTEX for 1 second
    PrivInitialize3DCtl (pCfgDialog->hInst);
    if (WAIT_TIMEOUT == dwWait)
    {
        PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE, pCfgDialog->hWnd);
        return MAPI_E_BUSY;
    }

    LPMAPITABLE pTableObj= NULL;
    ULONG ulPropCount;
    
    // Take ownership of the property value array, so calling code
    // will not leak memory
    LPSPropValue pProps = *pCfgDialog->ppProps;
    *pCfgDialog->ppProps = NULL;
    
    // Retrieve the dialog title string
    TCHAR szDialogTitle[64];
    LoadString (pCfgDialog->hInst, IDS_SERVICE_PROPERTY_SHEETS_TITLE, szDialogTitle, 64);
    CUIMAPIProp * pPropObj = NULL;
    HRESULT hResult = S_OK;
    try
    {
        pPropObj = new CUIMAPIProp (pCfgDialog->hInst,
                                    GetMAPIAllocBuffer(),
                                    GetMAPIAllocMore(),
                                    GetMAPIFreeBuffer(),
                                    pCfgDialog->ulFlags & UI_READONLY);
        if (!pPropObj)
        {
            hResult = E_OUTOFMEMORY;
        }
    }
    catch (CException & Exception)
    {
        hResult = Exception.GetError();
    }
    if (hResult)
    {
        goto ErrorExit;
    }

    hResult = pPropObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);
    if (hResult)
    {
        TraceResult ("DoLogonDlg: Failed to set the properties", hResult);
        goto ErrorExit;
    }
    DTPAGE DtPropPages[1];
    DtPropPages[0].cctl = NUM_CFG_USERINFO_PAGE_CTLS;   // Count of controls in the ConfigPage array
    DtPropPages[0].lpszComponent = szBlank;
    DtPropPages[0].lpszResourceName = MAKEINTRESOURCE (IDD_CFG_USERINFO);
    DtPropPages[0].lpctl = UserConfigPage;

    // If the UI_READONLY flag is set, then we are being called from the 
    // SettingDialog method in Read Only mode. Disable all of the controls
    // on the property sheets, else Enable them
    if (pCfgDialog->ulFlags & UI_READONLY)
    {
        // Disable the controls (make them read-only)
        DtPropPages[0].lpctl[1].ulCtlFlags &= ~(DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[0].lpctl[6].ulCtlFlags &= ~(DT_EDITABLE | DT_SET_IMMEDIATE);
    }
    else
    {
        // Enable the controls (make them read/write)
        DtPropPages[0].lpctl[1].ulCtlFlags |= (DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[0].lpctl[6].ulCtlFlags |= (DT_EDITABLE | DT_SET_IMMEDIATE);
    }

    // Create the display table for the logon dialog. It is based on dialog
    // resources plus the static information at the head of this module.
    LPTABLEDATA pTableData;
    hResult = BuildDisplayTable (GetMAPIAllocBuffer(),
                                 GetMAPIAllocMore(),
                                 GetMAPIFreeBuffer(),
                                 NULL,
                                 pCfgDialog->hInst,
                                 sizeof(DtPropPages)/sizeof(DTPAGE),
                                 DtPropPages,
                                 fMapiUnicode,
                                 &pTableObj,
                                 &pTableData);
    if (!hResult)
    {
        pPropObj->SetTableData (pTableData);
        pTableData->Release();

        // Display the dialog/property sheets
DisplayPropsAgain:
        hResult = pCfgDialog->pSupObj->DoConfigPropsheet ((ULONG)pCfgDialog->hWnd,
                                                          0,
                                                          szDialogTitle,
                                                          pTableObj,
                                                          pPropObj,
                                                          0);     // Default index for the top property sheet
        if (!hResult)
        {
            // Retrieve the altered data
            PrivMAPIFree ((LPVOID *)&pProps);
            pProps = NULL;
            hResult = pPropObj->GetProps ((LPSPropTagArray)&sptLogonProps, fMapiUnicode, &ulPropCount, &pProps);
            if (FAILED(hResult))
            {
                goto ErrorExit;
            }
            hResult = S_OK;
            // Check the Server name
            if (!IsValidServerName (pProps[SERVER_NAME].Value.lpszA))
            {
                PrivMAPIFree ((LPVOID *)&pProps);
                pProps = NULL;
                PrivateMessageBox (IDS_MSG_INVALID_SERVER_NAME, pCfgDialog->hWnd);
                goto DisplayPropsAgain;
            }

            // Return the new property array
            *pCfgDialog->ppProps = pProps;
            pProps = NULL;      // do not free this in cleanup code below
        }
    }

ErrorExit:
    // Release objects
    if (pTableObj)
    {
        pTableObj->Release();
    }
    if (pPropObj)
    {
        pPropObj->Release();
    }    
    PrivMAPIFree ((LPVOID *)&pProps);
    PrivUninitialize3DCtl (pCfgDialog->hInst);
    ReleaseMutex (pCfgDialog->hUIMutex);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    MSProviderInit()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Entry pointe called by MAPI when a profile uses this 
//      provider. The MAPI calls this method and expects a pointer to an
//      implementation of the IMSProvider interface. MAPI uses the returned
//      IMSProvider interface pointer to logon on the message store provider
//
//    Return Value
//      An HRESULT
//
STDINITMETHODIMP MSProviderInit (HINSTANCE          hInstance, 
                                 LPMALLOC           pMallocObj,
                                 LPALLOCATEBUFFER   pfnAllocateBuffer,
                                 LPALLOCATEMORE     pfnAllocateMore,
                                 LPFREEBUFFER       pfnFreeBuffer,
                                 ULONG              ulFlags,
                                 ULONG              ulMAPIVer,
                                 ULONG *            pulProviderVer,
                                 LPMSPROVIDER *     ppMSProviderObj)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    // Send output only to the COM1 port
    //InitTraces (0);
    // Send output to a console window BUT NOT to the COM1
    InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);
    // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);

    InfoTrace ("MSProviderInit function called");
    *pulProviderVer = CURRENT_SPI_VERSION;
    if (ulMAPIVer < CURRENT_SPI_VERSION)
    {
        TraceMessage ("MSProviderInit: The version of the subsystem cannot handle this version of the provider");
        return MAPI_E_VERSION;
    }
    HRESULT hResult = InitMemoryHeap (pfnAllocateBuffer, pfnAllocateMore, pfnFreeBuffer);
    if (FAILED(hResult))
    {
        return hResult;
    }
    hResult = S_OK;

    // Allocate space for the MSProvider object, the constructor will be called to initialize member variables
    CMSProvider * pMSProvider = new CMSProvider (hInstance);
    if (NULL == pMSProvider)
    {
        TraceMessage ("MSProviderInit: Failed to allocate new CMSProvider object");
        hResult = E_OUTOFMEMORY;
    }
    // Copy pointer to the allocated object back into the return IMSProvider object pointer
    *ppMSProviderObj = (LPMSPROVIDER)pMSProvider;
    if (hResult)
    {
        CloseMemoryHeap();
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CreateStoreEntryID()
//
//    Parameters
//      [IN]    pStoreUID   UID of the database file for this message store.
//      [OUT]   pStoreEID   Pointer to a SBinary structure allocated by the
//                          caller, where we returned the entry ID of the store
//                          as returned by MAPI. The memory in the lpb member,
//                          should be released using the MAPI allocation
//                          function.
//
//    Purpose
//      Creates an entry ID that will be returned to the MAPI subsystem which
//      identifies our store in this session and profile.
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI CreateStoreEntryID (MAPIUID * pStoreUID, LPSBinary pStoreEID)
{
    PRIVATE_ENTRYID eid = { 0 }; // Initialize the contents to 0
    eid.uidGlobal = *pStoreUID;
	eid.bVersion = ENTRYID_VERSION;
    eid.bObject = MAPI_STORE;
    HRESULT hResult = WrapStoreEntryID (0,
                                        MS_DLL_NAME_STRING,
                                        CB_PRIVATE_EID,
                                        (LPENTRYID)&eid,
                                        &pStoreEID->cb,
                                        (LPENTRYID *)&pStoreEID->lpb);
    // If any, trace out the error.
    TraceResult ("CreateStoreEntryID: Failed to wrap the store entry ID", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    InitStoreProps()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI InitStoreProps (LPPROFSECT pProfObj, LPPROFSECT pMSProfObj)
{
    ULONG i = 0;
    SPropValue rgProps[6] = { 0 };

    rgProps[i].ulPropTag = PR_PROVIDER_DISPLAY;
    rgProps[i++].Value.lpszA = PROVIDER_DISPLAY_NAME_STRING;

    rgProps[i].ulPropTag = PR_DISPLAY_NAME;
    rgProps[i++].Value.lpszA = WINDS_STORE_NAME;
    
    MAPIUID uidResource = WINDS_PF_MSG_ENTRIES;
    SBinary StoreEID;
    HRESULT hResult = CreateStoreEntryID (&uidResource, &StoreEID);
    if (!hResult)
    {
        rgProps[i].ulPropTag = PR_ENTRYID;
        rgProps[i].Value.bin.cb = StoreEID.cb;
        rgProps[i++].Value.bin.lpb = (LPBYTE)StoreEID.lpb;

        rgProps[i].ulPropTag = PR_STORE_ENTRYID;
        rgProps[i].Value.bin.cb = StoreEID.cb;
        rgProps[i++].Value.bin.lpb = (LPBYTE)StoreEID.lpb;

        rgProps[i].ulPropTag = PR_MDB_PROVIDER;
        rgProps[i].Value.bin.cb = sizeof(MAPIUID);
        rgProps[i++].Value.bin.lpb = (LPBYTE)&guidProvider;

        rgProps[i].ulPropTag = PR_RECORD_KEY;
        rgProps[i].Value.bin.cb = sizeof(MAPIUID);
        rgProps[i++].Value.bin.lpb = (LPBYTE)&uidResource;
        ASSERT (i <= (sizeof(rgProps)/sizeof(SPropValue)));
        hResult = pProfObj->SetProps (i, rgProps, NULL);
        pMSProfObj->SetProps (i, rgProps, NULL);
        PrivMAPIFree ((LPVOID *)&StoreEID.lpb);
    }
    TraceResult ("InitStoreProps", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    OpenProviderProfileSection()
//
//    Parameters
//      [IN]    pAdminProvObj   Pointer to an IProviderAdmin object
//      [OUT]   ppProfSectObj   Pointer to a pointer where this function
//                              returns an IProfSect object open through the
//                              pAdmingProvObj object
//
//    Purpose
//      This function opens the profile section where this provider stores its
//      settings for the current profile
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI OpenProviderProfileSection (LPPROVIDERADMIN pAdminProvObj,
                                           LPPROFSECT *    ppProfSectObj)
{
    // Get the provider table where the settings are stored
    LPMAPITABLE pTableObj;
    HRESULT hResult = pAdminProvObj->GetProviderTable (0, &pTableObj);
    if (hResult)
    {
        TraceResult ("OpenProviderProfileSection: GetProviderTable failed", hResult);
        return hResult;
    }

    // Since we are going only looking for one property, limit the view
    // of the columns to what we are interested in
    SPropTagArray sptProvider = { 1, { PR_PROVIDER_UID } };
    hResult = pTableObj->SetColumns (&sptProvider, 0);
    if (SUCCEEDED(hResult))
    {
        // Query only one row, the user's row with the settings in it
        LPSRowSet pRows;
        hResult = pTableObj->QueryRows (1, 0, &pRows);
        if (SUCCEEDED(hResult))
        {
            ASSERT (PR_PROVIDER_UID == pRows->aRow->lpProps->ulPropTag);
            hResult = pAdminProvObj->OpenProfileSection ((LPMAPIUID)pRows->aRow->lpProps->Value.bin.lpb,
                                                         NULL,
                                                         MAPI_MODIFY,
                                                         ppProfSectObj);
            FreeProws (pRows);
            if (hResult)
            {
                // Report failure, if any
                TraceResult ("OpenProviderProfileSection: OpenProfileSection failed", hResult);
                hResult = MAPI_E_NOT_FOUND;
            }
        }
        else
        {
            TraceResult ("OpenProviderProfileSection: Failed to query the rows", hResult);
        }
    }
    else
    {
        TraceResult ("OpenProviderProfileSection: Failed to set the columns", hResult);
    }
    pTableObj->Release();
    return hResult;
}

// End of file for MSSRVCE.CPP
