//////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      MSWDS.H
//
//  Description
//      Interface declaraction file for CMSLogon, CMSProvider,
//      CMAPIFolder, CMessage, and CMsgStore
//      Property tag definitions.
//      Constant definitions
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _MSWDS_H
#define _MSWDS_H

#pragma warning (disable : 4100)
#pragma warning (disable : 4127)
#pragma warning (disable : 4201)
#pragma warning (disable : 4245)
#pragma warning (disable : 4514)

#define STRICT
#include <WINDOWS.H>
#include <CTL3D.H>
#include <MAPIWIN.H>
#include <MAPISPI.H>
#include <MAPIUTIL.H>
#include <MAPIVAL.H>

// Header file for the trace and logging functions.
#include "TRACES.H"

#include "SUPPORT.H"

// Include our base implementation of IMAPIProp
#include "GENPROP.H"

// Common defs used by providers to communicate to the WINDS Sample Messaging Host
#include "COMWINDS.H"

#define SUPPORT_MASK    (STORE_ENTRYID_UNIQUE   | \
                         STORE_SEARCH_OK        | \
                         STORE_MODIFY_OK        | \
                         STORE_MV_PROPS_OK      | \
                         STORE_CREATE_OK        | \
                         STORE_ATTACH_OK        | \
                         STORE_OLE_OK           | \
                         STORE_NOTIFY_OK        | \
                         STORE_RESTRICTION_OK   | \
                         STORE_SORT_OK          | \
                         STORE_PUBLIC_FOLDERS)

// PR_MDB_PROVIDER is the GUID that represent this Message Store Provider.
// This GUID is available as a property in the stores table and on
// the message store object. The UID is hard coded.
// This number is generated only once per provider during the developement
// of it using the standard tools (GUIDGEN or UUIDGEN) in VC++ or the Win32 SDK.
// With this number you could identify your provider across multiple platforms.
// GUIDGEN: 87DB8840-F845-11ce-B2F9-00AA00510E3B
#define MS_STORE_UID_PROVIDER { 0x87, 0xdb, 0x88, 0x40,  \
                                0xf8, 0x45, 0x11, 0xce,  \
                                0xb2, 0xf9, 0x0,  0xaa,  \
                                0x0,  0x51, 0xe,  0x3b }

// This is the UID in the entry ID's the a WINDS transport uses to open entries in
// the WINDS address book, through the support object.
// This UID is hard coded because we only allow one instance of our provider
// to be installed in a single profile at any time.
// GUIDGEN: 0x174a32b0, 0xf84a, 0x11ce, 0xb2, 0xf9, 0x0, 0xaa, 0x0, 0x51, 0xe, 0x3b
// { 174A32B0-F84A-11ce-B2F9-00AA00510E3B }
#define WINDS_PF_MSG_ENTRIES { 0x17, 0x4a, 0x32, 0xb0,  \
                               0xf8, 0x4a, 0x11, 0xce,  \
                               0xb2, 0xf9, 0x0,  0xaa,  \
                               0x0,  0x51, 0xe,  0x3b }


// The order for this enumarations must match the order of the properties
// in the tag arrays. These arrays are used to access the 
// LPSPropValue arrays returned from MAPI interfaces. The use of enumerations
// hides differences when the number of properties in the tag array increases
// or decreases. It also helps maintain error-free indexing in the use of 
// the prop arrays
enum
{
    //SERVER_NAME           = 0,        // Array Indexes
    //MAILBOX_NAME          = 1,
    //MAILBOX_ID            = 2,
    //USER_NAME             = 3,
    //PASSWORD              = 4,
    //NET_CON               = 5,
    NUM_LOGON_PROPERTIES    = 6         // Array Size
};

// ID array of property tags for properties stored in the profile section.
// These properties are retrieved at logon time to get the necessary
// information to connect to the server.
const static SizedSPropTagArray(NUM_LOGON_PROPERTIES, sptLogonProps) =
{
    NUM_LOGON_PROPERTIES,
    {
        PR_SMP_REMOTE_SERVER,           // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 0)
        PR_SMP_MAILBOX_NAME,            // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 1)
        PR_SMP_MAILBOX_ID,              // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 2)
        PR_SMP_USER_NAME,               // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 3)
        PR_SMP_MAILBOX_PASSWORD,        // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 4)
        PR_SMP_CONNECTION_TYPE,         // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 5)
    }
};

// Version of this provider returned in PR_CREATION_VERSION and PR_CURRENT_VERSION
#define MSG_STORE_VERSION_MAJOR         1
#define MSG_STORE_VERSION_MINOR         8

#define ROOT_FOLDER_ID                  0

// Display name for the provider used in PR_PROVIDER_NAME
#define PROVIDER_DISPLAY_NAME_STRING    "WINDS Public Folders"

#define WINDS_STORE_NAME                "WINDS Public Folders"

// The target OS should not be included in the DLL name. MAPI will handle that.
// For Win32 platforms (Windows NT and Windows 95), the full name will be MSWDS32.DLL
#define MS_DLL_NAME_STRING              "MSWDS.DLL"

// Stand-alone helper functions
extern "C"
{
    CALLERRELEASE FolderTablesViewRelease;
    CALLERRELEASE MessageTablesViewRelease;
    
    HRESULT WINAPI CreateStoreEntryID
                    (MAPIUID *                  pStoreUID,
                     LPSBinary                  pStoreEID);
    HRESULT WINAPI DoLogonDlg
                    (PCFGDLG                    pCfgDialog);
    extern MAPIUID guidProvider;
};

// Forward class declarations
class CMSProvider;
class CMsgStore;
class CMSLogon;
class CMAPIFolder;
class CAttach;

typedef enum _UPDATE_TABLE_ACTION
{
    UPD_TABLE_REFRESH,
    UPD_TABLE_REFRESH_ADD_UNREAD,
    UPD_TABLE_REFRESH_DEL_UNREAD,
    UPD_TABLE_ADD_ROW,
    UPD_TABLE_DELETE_ROW
} UPDATE_TABLE_ACTION;

// Node structure ofr the linked list of notification set by clients on objects in our store
typedef struct _NOTIF_OBJS
{
    struct _NOTIF_OBJS *    pNext;
    PRIVATE_ENTRYID         eidObject;
    ULONG                   ulConnection;
} NOTIF_OBJS, *PNOTIF_OBJS;

// Type of actions deferred in transacted object (Messages and Attachments only)
typedef enum _TRANSACTED_ACTIONS
{
    TRANS_OBJ_ACTION_UNDEFINED = 0,
    TRANS_OBJ_ACTION_DELETE,
    TRANS_OBJ_ACTION_SAVE,
    TRANS_OBJ_ACTION_DELETE_OBJ_PROP
} TRANSACTED_ACTIONS;

// Entry for a queued action on a transacted object.
typedef struct _TRANSACTED_OBJECTS
{
    struct _TRANSACTED_OBJECTS * pNext;
    TRANSACTED_ACTIONS      Action;
    PRIVATE_ENTRYID         ObjEID;
    LPUNKNOWN               pUnkObj;
    SPropValue              Prop;
} TRANSACTED_OBJECTS;

class CMsgStore : public CGenericProp, public IMsgStore
{
friend CMSProvider;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMSGSTORE_METHODS(IMPL);
    
///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class or overides
//
    STDMETHODIMP UpdateSubscriptionList
                    (PRIVATE_ENTRYID *          pEID,
                     ULONG                      ulConnectionNum,
                     BOOL                       fAddToList);
    STDMETHODIMP NotifyClients
                    (PRIVATE_ENTRYID *          pEID,
                     ULONG                      ulNotifsCount,
                     LPNOTIFICATION             pNotifs);
    BOOL WINAPI IsObjectInNotifList
                    (PRIVATE_ENTRYID *          pEID);
    STDMETHODIMP UpdateFolder
                    (BOOL                       fUpdateOutgoing,
                     PRIVATE_ENTRYID *          pParentEID, 
                     PRIVATE_ENTRYID *          pMsgEID);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMsgStore       (CMSProvider *              pMSProviderObj,
                     LPMAPISUP                  pSupObj,
                     LPSTR                      szServer,
                     SBinary                    StoreEID,
                     CBackEnd *                 pBEobj,
                     BOOL                       fWriteAllowed,
                     long                       lAccess,
                     MAPIUID                    uidStore);
    ~CMsgStore      ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    CMSProvider *       m_pProvObj;
    ULONG               m_ulNotifAllObjects;
    PNOTIF_OBJS         m_pNotifList;
    char                m_szServer[_MAX_PATH];
    ULONG               m_ulCachedStoreLogOffFlags;
    
public :
    COpenObjectList     m_OpenObjList;
    HINSTANCE           m_hInstance;
    LPMAPISUP           m_pSupObj;
    MAPIUID		        m_StoreUID;
    SBinary             m_StoreEID;
    CMsgStore *         m_pNext;
};

class CMSLogon : public CGeneric, public IMSLogon
{
friend class CMSProvider;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMSLOGON_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    STDMETHODIMP InitializeStatusRow
                    ();

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CMSLogon        (CMsgStore *                pMsgStoreObj);
    ~CMSLogon       ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    CMsgStore *         m_pStore;
    CMSLogon *          m_pNext;
};

class CMSProvider : public IMSProvider
{
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMSPROVIDER_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    void WINAPI AddStoreObjectToList
                    (CMsgStore *                pMsgStoreObj);
    void WINAPI RemoveOpenStoreObject
                    (CMsgStore *                pStore);
    STDMETHODIMP GetProfileProps
                    (LPMAPISUP                  pSupObj,
                     LPSPropValue *             ppProps,
	                 HWND                       hOwnerWnd);
    BOOL WINAPI IsStoreAlreadyOpen
                    (LPSTR                      szServer,
                     CMsgStore **               ppStoreObj);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMSProvider     (HINSTANCE                  hInst);
    ~CMSProvider    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG               m_cRef; 
    CMsgStore *         m_pMsgStoreList;
    CRITICAL_SECTION    m_csObj;

public :
    HINSTANCE           m_hInstance;
};

class CMAPIFolder : public CGenericProp, public IMAPIFolder
{
friend void STDAPICALLTYPE FolderTablesViewRelease (ULONG, LPTABLEDATA, LPMAPITABLE);
friend class CFileBackEnd;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMAPICONTAINER_METHODS(IMPL);
    MAPI_IMAPIFOLDER_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
private :
    STDMETHODIMP UpdatePropertyCache
                    (CPropList *                pSourceList);
    STDMETHODIMP UpdatePropCacheInReplicas
                    ();
    STDMETHODIMP DeleteFolderContents
                    (PRIVATE_ENTRYID            ParentFolderEID,
                     PRIVATE_ENTRYID            FolderEID,
                     ULONG                      ulUIParam,
                     LPMAPIPROGRESS             pProgress,
                     ULONG                      ulFlags);
public:
    STDMETHODIMP UpdateParentTable
                    ();
    STDMETHODIMP UpdateContentTables
                    (UPDATE_TABLE_ACTION        Action,
                     BOOL                       fAssociatedCount,
                     BOOL                       fUnReadCount,
                     PRIVATE_ENTRYID *          pMessageEID,
                     CPropList *                pMsgProps,
                     BOOL                       bRecursing = FALSE);
    inline STDMETHODIMP CommitChanges
                    () { return CGenericProp::SaveChanges (KEEP_OPEN_READWRITE, NULL); }

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMAPIFolder     (CMsgStore *                pStore,
                     PRIVATE_ENTRYID *          pParentEID,
                     PRIVATE_ENTRYID *          pFolderEID,
                     long                       lFolderType,
                     CPropList *                pList,
                     BOOL                       fDeferErrors,
                     long                       lAccess,
                     CNamePropMgr *             pNPList);
    ~CMAPIFolder    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private:
    long                m_lFolderType;
    CMsgStore *         m_pStore;
    DWORD               m_dwHierTableViews;
    DWORD               m_dwContTableViews;
    DWORD               m_dwAsscContTableViews;
public:
    LPTABLEDATA         m_pHierTable;
    LPTABLEDATA         m_pContTable;
    LPTABLEDATA         m_pAsscContTable;
};

class CMessage : public CGenericProp, public IMessage
{
friend class CAttach;
friend void STDAPICALLTYPE MessageTablesViewRelease (ULONG, LPTABLEDATA, LPMAPITABLE);
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMESSAGE_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
public :
    STDMETHODIMP UpdateAttachmentTable
                    (CPropList *                pList,
                     PRIVATE_ENTRYID            AttachEID,
                     UPDATE_TABLE_ACTION        utaChange);
    STDMETHODIMP InternalSetReadFlag
                    (ULONG                      ulFlags,
                     BOOL                       fSentFolderNotification);
    STDMETHODIMP SaveChangesNow
                    (ULONG                      ulFlags);
    void WINAPI LoadObjectTables
                    ();

private :
    STDMETHODIMP GenerateDisplayNames
                    (LPMAPITABLE                pTable);
    STDMETHODIMP AddNameToString
                    (LPTSTR                     pszNewName,
                     LPTSTR *                   ppszNameList);
    STDMETHODIMP SetNormalizedSubject
                    (LPTSTR                     pszSubject,
                     LPTSTR *                   ppNormSubject);
    STDMETHODIMP PrepareRecipsForSubmission
                    ();
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CMessage        (CMsgStore *                pStore,
                     PRIVATE_ENTRYID *          pParentEID,
                     PRIVATE_ENTRYID *          pMsgEID,
                     CPropList *                pList,
                     BOOL                       fDeferErrors,
                     long                       lAccess,
                     CAttach *                  pParentAttObj,
                     CNamePropMgr *             pNPList);
    ~CMessage       ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    CMsgStore *             m_pStore;
    CAttach *               m_pParentAttObj;
    LPTABLEDATA             m_pAttTable;
    LPTABLEDATA             m_pRecipTable;
    DWORD                   m_dwRecipTableViews;
    DWORD                   m_dwAttTableViews;
    DWORD                   m_dwRecipRowID;
    BOOL                    m_fRecipTableDirty;
    TRANSACTED_OBJECTS *    m_pActionList;
};

class CAttach : public CGenericProp, public IAttach
{
friend class CMessage;
public :
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IMAPIPROP_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
public :
    STDMETHODIMP SaveChangesNow
                    (ULONG                      ulFlags);
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CAttach         (CMsgStore *                pStore,
                     PRIVATE_ENTRYID *          pParentEID,
                     PRIVATE_ENTRYID *          pAttachEID,
                     CPropList *                pList,
                     BOOL                       fDeferErrors,
                     long                       lAccess,
                     CMessage *                 pMsgObj,
                     CNamePropMgr *             pNPList);
    ~CAttach        ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    CMsgStore *             m_pStore;
    CMessage *              m_pParentMsgObj;
    TRANSACTED_OBJECTS *    m_pActionList;
};

#endif // _MSWDS_H

// End of file for MSWDS.H
