///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      PROPLIST.H
//
//  Description
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _PROPLIST_H
#define _PROPLIST_H

#pragma warning (disable : 4100)
#pragma warning (disable : 4201)
#pragma warning (disable : 4514)
#pragma warning (disable : 4702)

#define STRICT
#include <WINDOWS.H>
#include <MAPIDEFS.H>
#include <MAPIUTIL.H>
#include <MAPICODE.H>
#include <MAPISPI.H>

#ifdef _DEBUG
#define     ENABLE_DEBUG_OUTPUT     1
#endif // _DEBUG

#include "TRACES.H"     // Include functions prototypes for the Trace and logging


// Copied from MAPIWIN.H
#ifndef Cbtszsize
#define Cbtszsize(_a)   ((lstrlen(_a)+1)*sizeof(TCHAR))
#endif

HRESULT WINAPI InitMemoryHeap
                    (LPMAPISUP                  pSupObj);
HRESULT WINAPI InitMemoryHeap
                    (LPALLOCATEBUFFER           pfnAllocateBuffer,
                     LPALLOCATEMORE             pfnAllocateMore,
                     LPFREEBUFFER               pfnFreeBuffer);
HRESULT WINAPI CloseMemoryHeap
                    ();

#ifdef _DEBUG
#define PrivAllocate(a, b, c)   _PrivAllocate(a,b,c)
#else
#define PrivAllocate(a, b, c)   _PrivAllocate(a,b,NULL)
#endif // DEBUG
HRESULT WINAPI _PrivAllocate
                    (DWORD                      cbSize,
                     LPVOID *                   ppVoid,
                     LPTSTR                     pszBlockName);
void WINAPI PrivFree
                    (LPVOID *                   ppVoid);
HRESULT WINAPI PrivMAPIAllocate
                    (DWORD                      cbSize,
                     LPVOID *                   ppVoid,
                     LPVOID                     pParentMemBlock = NULL);
void WINAPI PrivMAPIFree
                    (LPVOID *                   ppVoid);

typedef struct _GENERIC_NODE
{
    struct _GENERIC_NODE *      pNext;
    BYTE                        data[1];
} GENERIC_NODE, *PGENERIC_NODE;

// Forward declarations
class COpenPropList;
class CPropStream;
class CStorage;

#define InsertInList(a, b)  _Insert ((PGENERIC_NODE *)a, (PGENERIC_NODE)b)
void WINAPI _Insert
                    (PGENERIC_NODE *            ppHead,
                     PGENERIC_NODE              pNewNode);

#define DeleteList(a)  _DeleteList ((PGENERIC_NODE *)a)
void WINAPI _DeleteList
                    (PGENERIC_NODE *            ppHead);

LPALLOCATEBUFFER GetMAPIAllocBuffer
                    ();
LPALLOCATEMORE GetMAPIAllocMore
                    ();
LPFREEBUFFER GetMAPIFreeBuffer
                    ();

BOOL WINAPI IncludePropInCopy
                    (ULONG                      ulPropTag,
                     LPSPropTagArray            pTags);
BOOL WINAPI IsPropInRow
                    (LPSRow                     pRow,
                     ULONG                      ulPropTag,
                     LPSPropValue *             ppProp = NULL);
HRESULT WINAPI AllocateNotifKey
                    (DWORD                      cbSize,
                     LPBYTE                     pKeyData,
                     LPNOTIFKEY *               ppKey);
BOOL WINAPI GetTempFullPathFileName
                    (LPSTR                      szFileName);

// Structure to hold pointers to our wrappers of IStream or IStorage interfaces
typedef union _IO_INTERFACE
{
    CPropStream *   pStream;
    CStorage *      pStorage;
} IO_INTERFACE;

// Type for large properties opened with IStream or IStorage interfaces
#define PT_STREAM_STORAGE   0x8008

#define PROP_STREAM         0x0001      // The interface used is IStream
#define PROP_STORAGE        0x0002      // The interface used is IStorage
#define PROP_MESSAGE        0x0004      // The interface used is IMessage
#define PROP_READ_ONLY      0x0100
#define PROP_READ_WRITE     0x0200
#define PROP_DELETE_THIS    0x1000      // The property is marked for deletion
#define PROP_NEVER_SAVED    0x8000      // The property has never been written to disk.

typedef struct _OPEN_PROPS
{
    struct _OPEN_PROPS *    pNext;
    ULONG                   ulPropTag;  // Original tag of the property the stream/storage is open on.
    DWORD                   ulFlags;    // The PROP_xxx bits above
    DWORD                   dwID;       // ID of the stream or storage on the container object storage.
    IO_INTERFACE            iface;      // Union with the interface pointers available.
} OPEN_PROPS, *POPEN_PROPS;

typedef struct _FlatSPropValue
{
    ULONG       cbSize;             // Total number of bytes including fixed length
    ULONG       ulPropTag;
    union _PV   Value;
} FlatSPropValue, *PFlatSPropValue;

typedef struct _FlatPropNode
{
    struct _FlatPropNode    * pNext;
    FlatSPropValue          Prop;
} FlatPropNode, *PFlatPropNode;

typedef enum _WALKING_ACTION
{
    WALK_START,
    WALK_CONTINUE
} WALKING_ACTION;

class CPropList
{
///////////////////////////////////////////////////////////////////////////////
// Member functions of this class
//
public:
    STDMETHODIMP Insert
                    (LPSPropValue               pProp);
    inline STDMETHODIMP Insert
                    (PFlatSPropValue            pProp);
    inline STDMETHODIMP InsertOpenProp
                    (ULONG                      ulPropTag,
                     DWORD                      dwID,
                     WORD                       wInterfaceOnProp);
    STDMETHODIMP Find
                    (ULONG                      ulPropTag,
                     LPSPropValue               pProp,
                     BOOL                       fMatchExact = TRUE,
                     LPVOID                     pParentBlock = NULL);
    STDMETHODIMP Delete
                    (ULONG                      ulPropTag,
                     LPSPropValue               pProp = NULL);
    STDMETHODIMP GetTags
                    (LPSPropTagArray *          ppTags);
    STDMETHODIMP SaveToStream
                    (LPSTREAM                   pStream);
    STDMETHODIMP LoadFromStream
                    (LPSTREAM                   pStream);
    STDMETHODIMP CopyToStream
                    (ULONG                      ulPropTag,
                     LPSTREAM                   pStream);
    STDMETHODIMP CopyToList
                    (CPropList *                pDestList);
    STDMETHODIMP GetAllProps
                    (LPSPropValue *             ppProps,
                     ULONG *                    pulCount);
    inline BOOL WINAPI IsDirty
                    () { return m_fDirty; };
    inline ULONG WINAPI GetNodeCount
                    () { return m_ulNodes; };
    PFlatSPropValue WINAPI Walk
                    (WALKING_ACTION             waAction);
    STDMETHODIMP CopySmallStreams
                    (COpenPropList *            pLargeProps);
    inline void WINAPI DeleteAllProps
                    () { DeleteList (&m_pHead); Reset(); }
private:
    STDMETHODIMP Flatten
                    (LPSPropValue               pProp,
                     PFlatPropNode *            ppNode);
    STDMETHODIMP Copy
                    (PFlatPropNode              pNode,
                     LPSPropValue               pProp,
                     LPVOID                     pVoid);
    BOOL WINAPI InternalNodePropReplaced
                    (LPSPropValue               pProp);
    void WINAPI FixUpPointers
                    (PFlatPropNode              pNode,
                     LPBYTE                     pByte);
    inline void WINAPI Reset
                    () { m_fDirty = FALSE;
                         m_ulNodes = 0;
                         m_pWalkNext = m_pTail = m_pHead = NULL; };
    STDMETHODIMP GetSize
                    (ULONG                      ulPropTag,
                     union _PV                  Value,
                     ULONG &                    cbSize,
                     LPVOID *                   ppVoid);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CPropList      () { Reset(); };
    CPropList      (LPSTREAM pStream) { Reset(); LoadFromStream (pStream); };
    ~CPropList     () { DeleteAllProps(); };

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    BOOL            m_fDirty;
    ULONG           m_ulNodes;
    PFlatPropNode   m_pTail;
    PFlatPropNode   m_pHead;
    PFlatPropNode   m_pWalkNext;
};

///////////////////////////////////////////////////////////////////////////////
//    CPropList::Insert()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
inline STDMETHODIMP CPropList::Insert (PFlatSPropValue pProp)
{
    SPropValue spvProp;
    spvProp.ulPropTag = pProp->ulPropTag;
    spvProp.Value = pProp->Value;
    return Insert (&spvProp);
}

///////////////////////////////////////////////////////////////////////////////
//    CPropList::InsertOpenProp()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//      None
//
inline STDMETHODIMP CPropList::InsertOpenProp (ULONG ulPropTag,
                                               DWORD dwID,
                                               WORD  wInterfaceOnProp)
{
    SPropValue spvProp;
    spvProp.ulPropTag = PROP_TAG (PT_STREAM_STORAGE, PROP_ID(ulPropTag));
    spvProp.Value.li.LowPart = (long)MAKELPARAM (PROP_TYPE(ulPropTag), wInterfaceOnProp);
    spvProp.Value.li.HighPart = (long)dwID;
    return Insert (&spvProp);
}

class COpenPropList
{
///////////////////////////////////////////////////////////////////////////////
// Member functions of this class
//
public:
    STDMETHODIMP Insert
                    (LPUNKNOWN                  pUnk,
                     ULONG                      ulFlags,
                     DWORD                      dwStreamID,
                     ULONG                      ulPropTag);
    STDMETHODIMP Find
                    (ULONG                      ulPropTag,
                     LPSPropValue               pProp,
                     LPVOID                     pMemBlock,
                     ULONG                      ulMaxBytes);
    STDMETHODIMP Delete
                    (ULONG                      ulPropTag);
    POPEN_PROPS WINAPI Walk
                    (WALKING_ACTION             waAction);
    POPEN_PROPS WINAPI Find
                    (ULONG                      ulPropTag);
    void WINAPI FreeList
                    (BOOL                       fForce);
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    COpenPropList   () { m_pHead = NULL; m_pWalkNext = NULL;};
    ~COpenPropList  () { FreeList (TRUE);
                         m_pHead = NULL;
                         m_pWalkNext = NULL;};

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    POPEN_PROPS m_pHead;
    POPEN_PROPS m_pWalkNext;
};

class CStorage : public IStorage
{
public:
///////////////////////////////////////////////////////////////////////////////
// IUnknown virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
///////////////////////////////////////////////////////////////////////////////
// IStorage virtual member functions
//
    inline STDMETHODIMP CreateStream
                    (LPCOLESTR                  pwcsName,
                     DWORD                      grfMode,
                     DWORD                      reserved1,
                     DWORD                      reserved2,
                     LPSTREAM *                 ppstm)
                     { HRESULT hResult = m_pImpStorage->CreateStream (pwcsName, grfMode, reserved1, reserved2, ppstm);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP OpenStream
                    (LPCOLESTR                  pwcsName,
                     void *                     reserved1,
                     DWORD                      grfMode,
                     DWORD                      reserved2,
                     LPSTREAM *                 ppstm)
                     { HRESULT hResult = m_pImpStorage->OpenStream (pwcsName, reserved1, grfMode, reserved2, ppstm);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP CreateStorage
                    (LPCOLESTR                  pwcsName,
                     DWORD                      grfMode,
                     DWORD                      dwStgFmt,
                     DWORD                      reserved2,
                     LPSTORAGE *                ppstg)
                     { HRESULT hResult = m_pImpStorage->CreateStorage (pwcsName, grfMode, dwStgFmt, reserved2, ppstg);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP OpenStorage
                    (LPCOLESTR                  pwcsName,
                     LPSTORAGE                  pstgPriority,
                     DWORD                      grfMode,
                     SNB                        snbExclude,
                     DWORD                      reserved,
                     LPSTORAGE *                ppstg)
                     { HRESULT hResult = m_pImpStorage->OpenStorage (pwcsName, pstgPriority, grfMode, snbExclude, reserved, ppstg);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP CopyTo
                    (DWORD                      ciidExclude,
                     const IID *                rgiidExclude,
                     SNB                        snbExclude,
                     LPSTORAGE                  pstgDest)
                     { return m_pImpStorage->CopyTo (ciidExclude, rgiidExclude, snbExclude, pstgDest); };
    inline STDMETHODIMP MoveElementTo
                    (LPCOLESTR                  pwcsName,
                     LPSTORAGE                  pstgDest,
                     LPCOLESTR                  pwcsNewName,
                     DWORD                      grfFlags)
                     { HRESULT hResult = m_pImpStorage->MoveElementTo (pwcsName, pstgDest, pwcsNewName, grfFlags);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP Commit
                    (DWORD                      grfCommitFlags)
                     { return m_pImpStorage->Commit (grfCommitFlags); };
    inline STDMETHODIMP Revert
                    ()
                     { return m_pImpStorage->Revert(); };
    inline STDMETHODIMP EnumElements
                    (DWORD                      reserved1,
                     void *                     reserved2,
                     DWORD                      reserved3,
                     LPENUMSTATSTG *            ppenum)
                     { return m_pImpStorage->EnumElements (reserved1, reserved2, reserved3, ppenum); };
    inline STDMETHODIMP DestroyElement
                    (LPCOLESTR                  pwcsName)
                     { HRESULT hResult = m_pImpStorage->DestroyElement (pwcsName);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP RenameElement
                    (LPCOLESTR                  pwcsOldName,
                     LPCOLESTR                  pwcsNewName)
                     { HRESULT hResult = m_pImpStorage->RenameElement (pwcsOldName, pwcsNewName);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP SetElementTimes
                    (LPCOLESTR                  pwcsName,
                     const FILETIME *           pctime,
                     const FILETIME *           patime,
                     const FILETIME *           pmtime)
                     { HRESULT hResult = m_pImpStorage->SetElementTimes (pwcsName, pctime, patime, pmtime);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP SetClass
                    (REFCLSID                   clsid)
                     { HRESULT hResult = m_pImpStorage->SetClass (clsid);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP SetStateBits
                    (DWORD                      grfStateBits,
                     DWORD                      grfMask)
                     { HRESULT hResult = m_pImpStorage->SetStateBits (grfStateBits, grfMask);
                       if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP Stat
                    (STATSTG *                  pstatstg,
                     DWORD                      grfStatFlag)
                     { return m_pImpStorage->Stat (pstatstg, grfStatFlag); };

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    STDMETHODIMP CreateStream
                    (LPSTR                      pszName,
                     DWORD                      grfMode,
                     DWORD                      reserved1,
                     DWORD                      reserved2,
                     LPSTREAM *                 ppstm);
    STDMETHODIMP OpenStream
                    (LPSTR                      pszName,
                     void *                     reserved1,
                     DWORD                      grfMode,
                     DWORD                      reserved2,
                     LPSTREAM *                 ppstm);
    STDMETHODIMP CreateStorage
                    (LPSTR                      pszName,
                     DWORD                      grfMode,
                     DWORD                      dwStgFmt,
                     DWORD                      reserved2,
                     CStorage **                ppstg);
    STDMETHODIMP OpenStorage
                    (LPSTR                      pszName,
                     LPSTORAGE                  pstgPriority,
                     DWORD                      grfMode,
                     SNB                        snbExclude,
                     DWORD                      reserved,
                     CStorage **                ppstg);
    STDMETHODIMP MoveElementTo
                    (LPSTR                      pszName,
                     LPSTORAGE                  pstgDest,
                     LPSTR                      pszNewName,
                     DWORD                      grfFlags);
    STDMETHODIMP DestroyElement
                    (LPSTR                      pszName);
    STDMETHODIMP RenameElement
                    (LPSTR                      pszOldName,
                     LPSTR                      pszNewName);
    STDMETHODIMP SetElementTimes
                    (LPSTR                      pszName,
                     const FILETIME *           pctime,
                     const FILETIME *           patime,
                     const FILETIME *           pmtime);
///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    inline BOOL WINAPI IsStorageInUse
                    () { EnterCriticalSection (&m_csObj);
                         BOOL fInUse = (m_cRef > 1 ? TRUE : FALSE);
                         LeaveCriticalSection (&m_csObj);
                         return fInUse; };
    inline BOOL WINAPI IsDirty
                    () { EnterCriticalSection (&m_csObj);
                         BOOL fState = m_fDirty;
                         LeaveCriticalSection (&m_csObj);
                         return fState; } ;
    inline void WINAPI SetDirty
                    (BOOL fDirty)
                    { EnterCriticalSection (&m_csObj);
                      m_fDirty = fDirty;
                      LeaveCriticalSection (&m_csObj); };
    inline LPSTREAM WINAPI GetRidingStream
                    () { return m_pImpStream; }
    inline BOOL WINAPI IsWriteable
                    () { return m_fWriteOk; }

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CStorage        (LPSTORAGE                  pImpStorage,
                     LPSTREAM                   pImpStream,
                     BOOL                       fWriteOk)
                    { m_cRef = 1;
                      m_pImpStorage = pImpStorage;
                      m_pImpStorage->AddRef();
                      m_pImpStream = pImpStream;
                      m_pImpStream->AddRef();
                      m_fDirty = FALSE;
                      m_fWriteOk = fWriteOk;
                      InitializeCriticalSection (&m_csObj); }
    ~CStorage       ()
                    {
                      m_pImpStream->Release();
                      m_pImpStream = NULL;
                      m_pImpStorage->Release();
                      m_pImpStorage = NULL;
                      m_fDirty = FALSE;
                      m_fWriteOk = FALSE;
                      DeleteCriticalSection (&m_csObj);
                    };

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    ULONG               m_cRef;
    BOOL                m_fDirty;
    BOOL                m_fWriteOk;
    LPSTREAM            m_pImpStream;
    LPSTORAGE           m_pImpStorage;
    CRITICAL_SECTION    m_csObj;
};

typedef CStorage **PPCSTORAGE;
HRESULT WINAPI CreatePrivateIStorage
                    (PPCSTORAGE                 ppStorage,
                     BOOL                       fWriteAccess,
                     LPSTREAM                   pStream = NULL);

class CPropStream : public IStream
{
public:
///////////////////////////////////////////////////////////////////////////////
// IUnknown virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
///////////////////////////////////////////////////////////////////////////////
// IStorage virtual member functions
//
    inline STDMETHODIMP Read
                    (void *                     pv,
                     ULONG                      cb,
                     ULONG *                    pcbRead)
                     { return m_pImpStream->Read (pv, cb, pcbRead); };
    inline STDMETHODIMP Write
                    (const void *               pv,
                     ULONG                      cb,
                     ULONG *                    pcbWritten)
                     { HRESULT hResult = m_pImpStream->Write (pv, cb, pcbWritten);
                      if (!hResult) { SetDirty (TRUE); } return hResult; };
    inline STDMETHODIMP Seek
                    (LARGE_INTEGER              dlibMove,
                     DWORD                      dwOrigin,
                     ULARGE_INTEGER *           plibNewPosition)
                     { return m_pImpStream->Seek (dlibMove, dwOrigin, plibNewPosition); };
    inline STDMETHODIMP SetSize
                    (ULARGE_INTEGER             libNewSize)
                     { return m_pImpStream->SetSize (libNewSize); };
    inline STDMETHODIMP CopyTo
                    (IStream *                  pstm,
                     ULARGE_INTEGER cb,
                     ULARGE_INTEGER *           pcbRead,
                     ULARGE_INTEGER *           pcbWritten)
                     { return m_pImpStream->CopyTo (pstm, cb, pcbRead, pcbWritten); };
    inline STDMETHODIMP Commit
                    (DWORD                      grfCommitFlags)
                     { return m_pImpStream->Commit (grfCommitFlags); };
    inline STDMETHODIMP Revert
                    ()
                     { return m_pImpStream->Revert(); };
    inline STDMETHODIMP LockRegion
                    (ULARGE_INTEGER             libOffset,
                     ULARGE_INTEGER             cb,
                     DWORD                      dwLockType)
                     { return m_pImpStream->LockRegion (libOffset, cb, dwLockType); };
    inline STDMETHODIMP UnlockRegion
                    (ULARGE_INTEGER             libOffset,
                     ULARGE_INTEGER             cb,
                     DWORD                      dwLockType)
                     { return m_pImpStream->UnlockRegion (libOffset, cb, dwLockType); };
    inline STDMETHODIMP Stat
                    (STATSTG *                  pstatstg,
                     DWORD                      grfStatFlag)
                     { return m_pImpStream->Stat (pstatstg, grfStatFlag); };
    inline STDMETHODIMP Clone
                    (IStream **                 ppstm)
                     { return m_pImpStream->Clone (ppstm); };

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    STDMETHODIMP SaveToStream
                    (LPSTREAM                   pStream);
    STDMETHODIMP SaveToBuffer
                    (LPVOID                     pVoid);
    STDMETHODIMP GetSize
                    (DWORD *                    pdwSize);
    inline BOOL WINAPI IsStreamInUse
                    () { EnterCriticalSection (&m_csObj);
                         BOOL fInUse = (m_cRef > 1 ? TRUE : FALSE);
                         LeaveCriticalSection (&m_csObj);
                         return fInUse; };
    inline BOOL WINAPI IsDirty
                    () { EnterCriticalSection (&m_csObj);
                         BOOL fState = m_fDirty;
                         LeaveCriticalSection (&m_csObj);
                         return fState; } ;
    inline void WINAPI SetDirty
                    (BOOL fDirty)
                    { EnterCriticalSection (&m_csObj);
                      m_fDirty = fDirty;
                      LeaveCriticalSection (&m_csObj); };
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CPropStream     (LPSTREAM                   pImpStream,
                     BOOL                       fWriteOk)
                    { m_cRef = 1; m_fDirty = FALSE;
                      m_pImpStream = pImpStream; m_pImpStream->AddRef();
                      m_fWriteOk = fWriteOk;
                      InitializeCriticalSection (&m_csObj); }
    ~CPropStream    ()
                    { m_pImpStream->Release(); m_pImpStream = NULL; m_fWriteOk = FALSE;
                      DeleteCriticalSection (&m_csObj);
                    }

///////////////////////////////////////////////////////////////////////////////
// Data members
public :
    ULONG               m_cRef;
    BOOL                m_fDirty;
    BOOL                m_fWriteOk;
    LPSTREAM            m_pImpStream;
    CRITICAL_SECTION    m_csObj;
};

#endif // _PROPLIST_H

// End of file for PROPLIST.H
