///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      mapisrvc.CPP
//
//  Description
//      Build this as a console application.
//      In the starup parameters for the service use
//      <profile> <mailuser> <directory>
//      MyService Nishad C:\Temp
//      This will send a mail to Nishad using MyService MAPI profile 
//      whenever C:\Temp is modified.
//
//  Author(s)
//      Nishad Mulye
//
//  Revision: 1.0
//
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#define STRICT

#include <windows.h>
#include <process.h>
#include <stdio.h>

#include <mapix.h>
#include <mapiutil.h>

#include "msgs.h"
#include "mapisrvc.h"
#include "ErrorLog.h"

// constants
const g_nHandles				= 2;

// globals
HANDLE					g_aHandles[g_nHandles];
HANDLE					&g_hSrvDone		= g_aHandles[0];
HANDLE					&g_hWorkUnit	= g_aHandles[1];
HANDLE					g_hEventSrc		= 0;

SERVICE_STATUS_HANDLE	g_hSrvStatus = 0;

// routines
void WINAPI ServiceMain(DWORD dwArgc, 
												char **ppszArgv);
void WINAPI ServiceCtrlHandler(DWORD dwCtrlCode);

BOOL WINAPI InitializeService(	char*,
																char*,
																LPMAPISESSION*,
																LPMAPIFOLDER*,
																LPADRLIST*);
void WINAPI DeinitializeService(LPMAPISESSION,
																LPMAPIFOLDER,
																LPADRLIST);
BOOL WINAPI SendMAPIMessage(		LPMAPISESSION,
																LPMAPIFOLDER,
																LPADRLIST,
																char*,
																char*);
void WINAPI SrcReportEvent(			HANDLE hEventSrc,
																DWORD dwEventId,
																DWORD dwParam = 0);

///////////////////////////////////////////////////////////////////////////////
//    main()
//
//    Parameters
//		refer to C/C++ language documentation
//
//    Purpose
//		refer to C/C++ language documentation
//      
//    Return Value
//		refer to C/C++ language documentation
//      
int main(int argc, char **argv)
{
	int nRet = 0;

	SERVICE_TABLE_ENTRY aSTE[] =
		{
				{ (char*)ServiceName, (LPSERVICE_MAIN_FUNCTION)ServiceMain },
				{ 0, 0 }
		};

	if (!StartServiceCtrlDispatcher(aSTE))
	{
		nRet = 1;
	}
	return nRet;
}

///////////////////////////////////////////////////////////////////////////////
//    ServiceMain()
//
//    Parameters
//		refer to Win32/WinNT documentation
//
//    Purpose
//		refer to Win32/WinNT documentation
//      
//    Return Value
//		refer to Win32/WinNT documentation
//      
void WINAPI ServiceMain(DWORD dwArgc, char **ppszArgv)
{
	ErrorLog			el("C:\\ErrorLog.txt");
	LPMAPISESSION	pSession	= 0;
	LPMAPIFOLDER	pFolder		= 0;
	LPADRLIST			pAdrList	= 0;

	SERVICE_STATUS ssStatus;
	ssStatus.dwServiceType							= SERVICE_WIN32_OWN_PROCESS;
	ssStatus.dwServiceSpecificExitCode	= 0;
	ssStatus.dwControlsAccepted					= SERVICE_ACCEPT_STOP;
	ssStatus.dwCurrentState							= SERVICE_START_PENDING;
	ssStatus.dwWin32ExitCode						= 0;
	ssStatus.dwCheckPoint								= 0;
	ssStatus.dwWaitHint									= 3000;

	g_hSrvStatus = RegisterServiceCtrlHandler(ServiceName, 
																						ServiceCtrlHandler);
	if (g_hSrvStatus)
	{
		if (SetServiceStatus(g_hSrvStatus, &ssStatus))
		{
			g_hSrvDone = CreateEvent(	0,
																TRUE,
																FALSE,
																0);
			if (g_hSrvDone)
			{
				ssStatus.dwCheckPoint	= 1;
				ssStatus.dwWaitHint		= 7000;
				if (SetServiceStatus(g_hSrvStatus, &ssStatus))
				{
					if (dwArgc == 4)
					{
						if (InitializeService(ppszArgv[1],
																	ppszArgv[2],
																	&pSession,
																	&pFolder,
																	&pAdrList))
						{
							ssStatus.dwCheckPoint	= 2;
							ssStatus.dwWaitHint		= 3000;
							if (SetServiceStatus(g_hSrvStatus, &ssStatus))
							{
								g_hWorkUnit = FindFirstChangeNotification(
															ppszArgv[3],
															TRUE,
															FILE_NOTIFY_CHANGE_FILE_NAME	|
															FILE_NOTIFY_CHANGE_DIR_NAME		|
															FILE_NOTIFY_CHANGE_ATTRIBUTES	|
															FILE_NOTIFY_CHANGE_SIZE			|
															FILE_NOTIFY_CHANGE_LAST_WRITE	|
															FILE_NOTIFY_CHANGE_SECURITY);
								if (g_hWorkUnit)
								{
									ssStatus.dwCheckPoint	= 0;
									ssStatus.dwCurrentState	= SERVICE_RUNNING;
									if (SetServiceStatus(g_hSrvStatus, &ssStatus))
									{
										while (WaitForMultipleObjects(	2,
																		g_aHandles,
																		FALSE,
																		INFINITE)
																		!= WAIT_OBJECT_0)
										{
											if (SendMAPIMessage(pSession,
																					pFolder,
																					pAdrList,
																					"MAPIService Notification",
																					"Directory has been changed.\n"))
											{
												if (!FindNextChangeNotification(g_hWorkUnit))
												{
													el.Log("FindNextChangeNotification() failed.");
													break;
												}
											}
											else
											{
												el.Log("SendMAPIMessage() failed.");
												break;
											}
										}
									}
									else
									{
										el.Log("SetServiceStatus() failed.");
									}
									FindCloseChangeNotification(g_hWorkUnit);
								}
								else
								{
									el.Log("FindFirstChangeNotification() failed.");
								}
							}
							else
							{
								el.Log("SetServiceStatus() failed.");
							}
						}
						else
						{
							el.Log("InitializeService() failed.");
						}
						DeinitializeService(pSession,
																pFolder,
																pAdrList);
					}
					else
					{
						el.Log("Invalid command-line arguments.");
					}
					CloseHandle(g_hSrvDone);
				}
				else
				{
					el.Log("SetServiceStatus() failed.");
				}
			}
			else
			{
				el.Log("CreateEvent() failed.");
			}
		}
		else
		{
			el.Log("SetServiceStatus() failed.");
		}
		ssStatus.dwCurrentState	= SERVICE_STOPPED;
		SetServiceStatus(g_hSrvStatus, &ssStatus);
	}
	else
	{
		el.Log("RegisterServiceCtrlHandler() failed.");
	}
}

///////////////////////////////////////////////////////////////////////////////
//    ServiceCtrlHandler()
//
//    Parameters
//		dwCtrlCode	control code (recd. from Service Manager)
//
//    Purpose
//		respond to the control code from Service Manager
//      
//    Return Value
//		none
//      
void WINAPI ServiceCtrlHandler(DWORD dwCtrlCode)
{
	SERVICE_STATUS ssStatus;

	ssStatus.dwCurrentState	= SERVICE_RUNNING;
	switch (dwCtrlCode)
	{
	case SERVICE_CONTROL_STOP:	
		ssStatus.dwCurrentState	= SERVICE_STOP_PENDING;
		SetEvent(g_hSrvDone);
		break;
	case SERVICE_CONTROL_PAUSE:
		ssStatus.dwCurrentState	= SERVICE_PAUSED;
		break;
	case SERVICE_CONTROL_CONTINUE:
		break;
	case SERVICE_CONTROL_INTERROGATE:
		break;
	case SERVICE_CONTROL_SHUTDOWN:
		break;
	}

	ssStatus.dwServiceType							= SERVICE_WIN32_OWN_PROCESS;
	ssStatus.dwServiceSpecificExitCode	= 0;
	ssStatus.dwControlsAccepted					= SERVICE_ACCEPT_STOP;
	ssStatus.dwWin32ExitCode						= 0;
	ssStatus.dwCheckPoint								= 0;
	ssStatus.dwWaitHint									= 0;

	if (!SetServiceStatus(g_hSrvStatus, &ssStatus))
	{
		SetEvent(g_hSrvDone);
	}
}

///////////////////////////////////////////////////////////////////////////////
//    InitializeService()
//
//    Parameters
//    pszProfile		profile name used for MAPILogonEx()
//    pszMsilUser		display name or alias of the recipient
//    ppSession			pointer to handle of the MAPI session
//		ppFolder			pointer to handle of the Outbox
//		ppAdrList			pointer to handle of the resolved address list
//
//    Purpose
//		1. initialize MAPI and logon
//		2. get the handle of the Outbox
//		3. cache the resolved address list
//      
//    Return Value
//    TRUE = SUCCESS, FALSE otherwise
//      
BOOL WINAPI InitializeService(char* pszProfile,
															char* pszMailUser,
															LPMAPISESSION* ppSession,
															LPMAPIFOLDER* ppFolder,
															LPADRLIST* ppAdrList)
{
	BOOL bRet = FALSE;
	MAPIINIT_0 mapiInit = {0, MAPI_INIT_VERSION | MAPI_NT_SERVICE};
	HRESULT hr = S_OK;

	// initialize MAPI
	hr = MAPIInitialize(&mapiInit);
	if (SUCCEEDED(hr))
	{
		// logon with default parameters
		hr = MAPILogonEx(	0,
											pszProfile,
											"",
											MAPI_NO_MAIL | MAPI_NT_SERVICE | fMapiUnicode,
											ppSession);
		if (SUCCEEDED(hr))
		{
			LPMAPITABLE pTableStores = 0;

			// obtain the default message store from the set of message stores
			// and open the outbox.
			hr = (*ppSession)->GetMsgStoresTable(	fMapiUnicode,
																				&pTableStores);
			if (SUCCEEDED(hr))
			{
				SizedSPropTagArray(2, tagStores) = 
				{
					2, PR_DEFAULT_STORE, PR_ENTRYID
				};
				LPSRowSet pRowSetStores = 0;
				SRestriction sRes;
				SPropValue propStore;

				propStore.ulPropTag		= PR_DEFAULT_STORE;
				propStore.Value.b			= TRUE;
				propStore.dwAlignPad	= 0;

				sRes.rt = RES_PROPERTY;
				sRes.res.resProperty.relop			= RELOP_EQ;
				sRes.res.resProperty.ulPropTag	= PR_DEFAULT_STORE;
				sRes.res.resProperty.lpProp			= &propStore;

				// this call should be used with caution because it can potentially
				// return a very large number of rows. It is okay in this case 
				// because there is only one default message store.
				hr = HrQueryAllRows(pTableStores,
														(LPSPropTagArray)&tagStores,
														&sRes,
														0,
														0,
														&pRowSetStores);
				if (SUCCEEDED(hr))
				{
					if (pRowSetStores->cRows > 0)
					{
						LPSPropValue pPropStore = pRowSetStores->aRow[0].lpProps;
						LPMDB pMDBDefault = 0;
						hr = (*ppSession)->OpenMsgStore(0,
																				pPropStore[1].Value.bin.cb,
																				(LPENTRYID)pPropStore[1].Value.bin.lpb,
																				0,
																				MDB_WRITE,
																				&pMDBDefault);
						if (SUCCEEDED(hr))
						{
							// open the OutBox Folder
							const SizedSPropTagArray(1, tagOutbox) =
							{
								1, PR_IPM_OUTBOX_ENTRYID
							};
							LPSPropValue pPropFolder = 0;
							unsigned long ulVal = 0;
							hr = pMDBDefault->GetProps(	(LPSPropTagArray)&tagOutbox,
																					fMapiUnicode, 
																					&ulVal, 
																					&pPropFolder);
							if (SUCCEEDED(hr))
							{
								unsigned long ulObjectType = 0;
								hr = pMDBDefault->OpenEntry(pPropFolder[0].Value.bin.cb, 
																						(LPENTRYID)pPropFolder[0].Value.bin.lpb,
																						0,
																						MAPI_MODIFY, 
																						&ulObjectType, 
																						(LPUNKNOWN*)ppFolder);
								if (SUCCEEDED(hr))
								{
									LPADRBOOK pAdrBook = 0;
									// open the address book
									hr = (*ppSession)->OpenAddressBook(	0,
																									0,
																									AB_NO_DIALOG,
																									&pAdrBook);
									if (SUCCEEDED(hr))
									{
										// compose the address list
										// mail recipient count
										const unsigned long nUsers = 1;

										hr = MAPIAllocateBuffer(CbNewADRLIST(nUsers), (void**)ppAdrList);
										if (SUCCEEDED(hr))
										{
											LPSPropValue pValue = 0;

											hr = MAPIAllocateBuffer(sizeof(SPropValue)*2, (void**)&pValue);
											if (SUCCEEDED(hr))
											{
												// use PR_DISPLAY_NAME because not all providers
												// support PR_ACCOUNT
												pValue[0].ulPropTag	  = PR_DISPLAY_NAME;
												pValue[0].dwAlignPad	= 0;
												pValue[0].Value.LPSZ	= pszMailUser;
												pValue[1].ulPropTag	  = PR_RECIPIENT_TYPE;
												pValue[1].dwAlignPad	= 0;
												pValue[1].Value.l	    = MAPI_TO;
												(*ppAdrList)->cEntries	  = nUsers;
												(*ppAdrList)->aEntries[0].ulReserved1	= 0;
												(*ppAdrList)->aEntries[0].cValues			= 2;
												(*ppAdrList)->aEntries[0].rgPropVals	= pValue;

												// resolve names
												hr = pAdrBook->ResolveName(	0,
																										0,
																										0,
																										*ppAdrList);
												if (SUCCEEDED(hr))
												{
													bRet = TRUE;
												}
											}
										}
										pAdrBook->Release();
									}
								}
								MAPIFreeBuffer(pPropFolder);
							}
							pMDBDefault->Release();
						}
					}
					FreeProws(pRowSetStores);
				}
				pTableStores->Release();
			}
		}
	}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    DeinitializeService()
//
//    Parameters
//    pSession			handle of the MAPI session
//		pFolder				handle of the Outbox
//		pAdrList			handle of the resolved address list
//
//    Purpose
//		cleanup, logoff, deinitialize MAPI 
//      
//    Return Value
//		none
//      
void WINAPI DeinitializeService(LPMAPISESSION pSession,
													LPMAPIFOLDER pFolder,
													LPADRLIST pAdrList)
{
	FreePadrlist(pAdrList);
	if (pFolder)
	{
		pFolder ->Release();
	}
	if (pSession)
	{
		pSession->Logoff(	0,
											0,
											0);
		pSession->Release();
	}
	MAPIUninitialize();
}

///////////////////////////////////////////////////////////////////////////////
//    SendMAPIMessage()
//
//    Parameters
//    pSession			handle of the MAPI session
//		pFolder				handle of the Outbox
//		pAdrList			handle of the resolved address list
//
//    Purpose
//		compose, set properties and submit a message 
//      
//    Return Value
//		none
//      
BOOL WINAPI SendMAPIMessage(LPMAPISESSION pSession,
														LPMAPIFOLDER pFolder,
														LPADRLIST pAdrList,
														char* pszSubject,
														char* pszBody)
{
	BOOL bRet = FALSE;
	LPMESSAGE pMessage = 0;
	HRESULT hr = 0;
	// create a message and set its properties
	hr = pFolder->CreateMessage(0,
															0,
															&pMessage);
	if (SUCCEEDED(hr))
	{
		const unsigned long cPropsMesg = 6;
		SPropValue	aPropsMesg[cPropsMesg];
		SYSTEMTIME	systemTime;
		FILETIME	messageTime;
		LPSPropProblemArray	lpPropProblems = 0;

		aPropsMesg[0].dwAlignPad	= 0;
		aPropsMesg[0].ulPropTag		= PR_MESSAGE_CLASS;
		aPropsMesg[0].Value.LPSZ	= "IPM.NOTE";
		aPropsMesg[1].dwAlignPad	= 0;
		aPropsMesg[1].ulPropTag		= PR_PRIORITY;
		aPropsMesg[1].Value.l  		= 0;
		aPropsMesg[2].dwAlignPad	= 0;
		aPropsMesg[2].ulPropTag		= PR_SUBJECT;
		aPropsMesg[2].Value.LPSZ	= pszSubject;
		aPropsMesg[3].dwAlignPad	= 0;
		aPropsMesg[3].ulPropTag		= PR_BODY;
		aPropsMesg[3].Value.LPSZ	= pszBody;

		GetSystemTime(&systemTime);
		SystemTimeToFileTime(&systemTime, &messageTime);
		
		aPropsMesg[4].dwAlignPad	= 0;
		aPropsMesg[4].ulPropTag		= PR_CLIENT_SUBMIT_TIME;
		aPropsMesg[4].Value.ft		= messageTime;

		aPropsMesg[5].ulPropTag		= PR_DELETE_AFTER_SUBMIT;
		aPropsMesg[5].dwAlignPad	= 0;
		aPropsMesg[5].Value.b			= TRUE;

		hr = pMessage->SetProps(cPropsMesg, aPropsMesg, &lpPropProblems);
		if (SUCCEEDED(hr))
		{
			hr = pMessage->ModifyRecipients(0,
																			pAdrList);
			if (SUCCEEDED(hr))
			{
				hr = pMessage->SubmitMessage(FORCE_SUBMIT);
				if (SUCCEEDED(hr))
				{
					bRet = TRUE;
				}
			}
		}
		pMessage->Release();
	}
	return bRet;
}
