///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      setup.CPP
//
//  Description
//      The purpose of this sample is to demonstrate a way of installing or 
//      uninstalling an NT Service which is a MAPI client. This sample is 
//      specifically meant for NT service MAPI client. For a generic NT service
//      refer to additional Microsoft documentation
//      Build this as a console application.
//      Usage:
//          To Install the Service
//          setup -i <Full Path to the binary> <NT account> <NT account password>
//          or
//          To Uninstall the Service
//          setup -u <Full Path to the binary>
//
//  Author(s)
//      Nishad Mulye
//
//  Revision: 1.0
//
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#define STRICT

#include <windows.h>
#include <process.h>
#include <stdio.h>

#include "mapisrvc.h"

int main(int argc, char **argv)
{
	int nRet = 1;

	if (argc >= 2)
	{
		if (argv[1][0] == '-' || argv[1][0] == '/')
		{
			SC_HANDLE scMgr = OpenSCManager(0,
																			0,
																			SC_MANAGER_ALL_ACCESS);
			if (scMgr)
			{
				SC_HANDLE scSrvc = 0;
				if ((argv[1][1] == 'i' || argv[1][1] == 'I') && argc == 5)
				{
					scSrvc = CreateService(	scMgr,
																	ServiceName,
																	ServiceName,
																	SERVICE_ALL_ACCESS,
																	SERVICE_WIN32_OWN_PROCESS,
																	SERVICE_DEMAND_START,
																	SERVICE_ERROR_NORMAL,
																	argv[2],
																	0,
																	0,
																	0,
																	argv[3],
																	argv[4]);
					if (scSrvc)
					{
						HKEY hKey = 0;
						DWORD dwDisposition = 0;

						if (RegCreateKeyEx(	HKEY_LOCAL_MACHINE,
																RegistryKey,
																0,
																0,
																REG_OPTION_NON_VOLATILE,
																KEY_ALL_ACCESS,
																0,
																&hKey,
																&dwDisposition) == ERROR_SUCCESS)
						{
							if (RegSetValueEx(hKey,
																"EventMessageFile",
																0,
																REG_EXPAND_SZ,
																(BYTE*)argv[2],
																strlen(argv[2])+1) == ERROR_SUCCESS)
							{
								DWORD dwValue = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE;
								if (RegSetValueEx(hKey,
																	"TypesSupported",
																	0,
																	REG_DWORD,
																	(BYTE*)&dwValue,
																	sizeof(DWORD)) == ERROR_SUCCESS)
								{
									printf("\nService created.\n");
									nRet = 0;
								}
							}
						}
						RegCloseKey(hKey);
						if (nRet != 0)
						{
							RegDeleteKey(HKEY_LOCAL_MACHINE, RegistryKey);
						}
					}
					if (nRet != 0)
					{
						DeleteService(scSrvc);
					}
					CloseServiceHandle(scSrvc);
				}
				else if (argv[1][1] == 'u' || argv[1][1] == 'U')
				{
					scSrvc = OpenService(	scMgr,
																ServiceName,
																DELETE);
					if (scSrvc)
					{
						DeleteService(scSrvc);
						RegDeleteKey(HKEY_LOCAL_MACHINE, RegistryKey);
						printf("\nService deleted!\n");
						nRet = 0;
					}
					CloseServiceHandle(scSrvc);
				}
				CloseServiceHandle(scMgr);
			}
		}
	}
	if (nRet != 0)
	{
		fprintf(stderr, 
				"usage:\n%s -i <Full Path> <NT Account> <NT Account Password>\n",
				argv[0]);
		fprintf(stderr, 
				"%s -u <Full Path>\n",
				argv[0]);
	}
	return nRet;
}
