#include <windows.h>  
#include <mapicode.h>
#include <mapidefs.h>
#include <mapitags.h>
#include <mapix.h>
#include "otto.h"

char g_szAppName[] = APPNAME;  // The name of this application
char g_szTitle[]   = APPNAME;  // The title bar text


//////////////////////////////////////////////////////////////////////////////
//
//  Application entry point.  
//  
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	MSG         msg;
	HANDLE      hAccelTable;
    HRESULT     hRes = S_OK;

	if (!hPrevInstance)
    	// Perform instance initialization:
		if (!InitApplication(hInstance)) 
			return (FALSE);

	// Perform application initialization:
	if (!InitInstance(hInstance, nCmdShow)) 
		return (FALSE);

	hAccelTable = LoadAccelerators (hInstance, g_szAppName);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
		if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

	return (msg.wParam);

	lpCmdLine; // This will prevent 'unused formal parameter' warnings
}

//////////////////////////////////////////////////////////////////////////////
//
//  InitApplication
//
//  Purpose: 
//      Initializes window data and registers window class 
//      In this function, we initialize a window class by filling out a data
//      structure of type WNDCLASS and calling either RegisterClass or 
//      the internal MyRegisterClass.
//
BOOL WINAPI InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe
    // the main window.
    wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = (WNDPROC)WndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 8;
    wc.hInstance     = hInstance;
    wc.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICONMBOX));
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszMenuName  = MAKEINTRESOURCE(IDM_MAIN);
    wc.lpszClassName = g_szAppName;

    // Register the window class and return success/failure code.
	return RegisterClass(&wc);
}

//////////////////////////////////////////////////////////////////////////////
//
//   InitInstance
//
//   Purpose
//      Saves instance handle and creates main window 
//      In this function, we create and display the main program window.
//
BOOL WINAPI InitInstance(HINSTANCE hInstance,int nCmdShow)
{
    HWND    hWnd;

	hWnd = CreateWindow(g_szAppName, 
	                    g_szTitle,
	                    WS_ICONIC,
		                0,
		                0,
		                300,
		                200,
		                NULL,
		                NULL, 
		                hInstance, 
		                NULL);

	if (!hWnd) 
		return (FALSE);
	
	ShowWindow(hWnd, SW_SHOWMINIMIZED);
	UpdateWindow(hWnd);

	return (TRUE);
}

///////////////////////////////////////////////////////////////////////////////
//    CopySBinary()
//
//    Parameters
//      
//    Purpose
//      Allocates a new SBinary and copies psbSrc into it
//      
STDMETHODIMP CopySBinary(LPSBinary psbDest,const LPSBinary psbSrc, LPVOID pParent)
{
    HRESULT     hRes = S_OK;

    psbDest -> cb = psbSrc -> cb;

    if (psbSrc -> cb)
    {
        if (pParent)
            hRes = MAPIAllocateMore(psbSrc -> cb, 
                                    pParent, (LPVOID *) & psbDest->lpb);
        else
            hRes = MAPIAllocateBuffer(psbSrc -> cb, 
                                      (LPVOID *) & psbDest->lpb);

        CopyMemory(psbDest->lpb,psbSrc->lpb,psbSrc -> cb);
    }
            
    return hRes;
}

///////////////////////////////////////////////////////////////////////////////
//
//  Compares 2 SBinary structs for equality
//  
BOOL operator == (const SBinary & lhs, const SBinary & rhs)
{
    if (lhs.cb != rhs.cb)
        return FALSE;

    return (memcmp(lhs.lpb,rhs.lpb,lhs.cb) ? FALSE : TRUE);
 
}


///////////////////////////////////////////////////////////////////////////////
//
//  About box dialog procedure.
//  
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{

	switch (message) {
        case WM_INITDIALOG:
			ShowWindow (hDlg, SW_HIDE);
			return (TRUE);

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
				EndDialog(hDlg, TRUE);
				return (TRUE);
			}
			break;
	}

    return FALSE;
}


///////////////////////////////////////////////////////////////////////////////
//
//  Displays a message box with error info from GetLastError
//  and lets user break into the debugger.
//
BOOL WINAPI BreakToDebug(HRESULT hr, LPMAPIERROR pErr, HWND hWnd)
{
    TCHAR   szMsg[MAX_PATH + 80];
    LPTSTR  szFormat = "%s\r\n\r\n> Break into debugger? <";

    if (pErr)
        wsprintf(szMsg,szFormat,pErr -> lpszError);
    else
        wsprintf(szMsg,szFormat,"No error information available.");
 
    return (IDYES == MessageBox(hWnd,szMsg,"Critical error in HelloMAPI",MB_YESNO));
}