

///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      CAdviseSink.cpp
//
//  Description
//      implement the OnNotify() method of class CAdviseSink. The other
//      methods are implemented in the header file CAdviseSink.h
//
//  Author
//      Nishad Mulye
//
//  Revision: 1.00
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//

#define STRICT

// windows include
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>

// resource ids.
#include "resource.h"

// MAPI includes
#include "Advise.h"

// global variables
extern BOOL g_bOkToValidateXP;
extern HWND g_hWndRmtConn;

///////////////////////////////////////////////////////////////////////////////
//    CAdviseSink::OnNotify()
//
//    Parameters
//      cNotif,          number of notifications
//      pNotifications,  array of notifications
//
//    Purpose
//      handle notification sent by MAPI
//      
//    Return Value
//      S_OK if successful
//      
STDMETHODIMP_(ULONG) CAdviseSink::OnNotify(ULONG cNotif, LPNOTIFICATION pNotifications)
{
	for (unsigned long i = 0;i < cNotif;i++)
	{
		if (pNotifications[i].ulEventType == fnevTableModified)
		{
			if (pNotifications[i].info.tab.ulTableEvent == TABLE_ROW_MODIFIED)
			{
				LPSPropValue pProps = pNotifications[i].info.tab.row.lpProps;
				if (pProps[4].ulPropTag == PR_REMOTE_VALIDATE_OK)
				{
					g_bOkToValidateXP = pProps[4].Value.b;
					if (g_hWndRmtConn && g_bOkToValidateXP)
					{
						SendMessage(g_hWndRmtConn,
												WM_COMMAND,
												MAKELONG(IDC_STATUS_NOTIFY_MSG, 0),
												0);
					}
				}
				if (pProps[5].ulPropTag == PR_REMOTE_PROGRESS)
				{
					if (pProps[5].Value.l == 100 || pProps[5].Value.l == -1)
					{
						SendMessage(g_hWndRmtConn,
												WM_COMMAND,
												MAKELONG(IDC_REMOTE_PROGRESS_MSG, 0),
												100);
					}
					else
					{
						SendMessage(g_hWndRmtConn,
												WM_COMMAND,
												MAKELONG(IDC_REMOTE_PROGRESS_MSG, 0),
												pProps[5].Value.l);
					}
				}
			}
		}
	}
	return S_OK;
}
