
///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      WinMain.cpp
//
//  Description
//      The purpose of this program is to demonstrate Remote Mail handling
//      using Extended MAPI. This program displays a profile dialog
//      and logs on using the profile the user selected. It then displays
//      another dialog to and lets the user choose a transport provider that
//      supports Remote Mail access. The program then displays the contents
//      of the Remote Mail Folder or the (Remote Header Cache). The user can
//      then perform a variety of operations like, refreshing the Remote Mail
//      Folder, marking the entries in the folder for Copy, Move and Delete,
//      and performing the actual Copy, Move and Delete operations.
//      
//
//  Author
//      Nishad Mulye
//
//  Revision: 1.00
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//

#define STRICT

// windows include
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <stdio.h>

#define USES_IID_IMAPIFolder
#define USES_IID_IMAPIStatus
#define USES_IID_IMAPIAdviseSink

// is defined only once in the project
#define INITGUID		
#include <initguid.h>

// MAPI includes
#include <mapi.h>
#include <mapiutil.h>

// resource ids.
#include "resource.h"

#include "Advise.h"

// data types
typedef struct
{
	unsigned	xpl_uCnt;
	TCHAR**		xpl_pszaNames;
	unsigned  xpl_uRet;
} XPList;
typedef int (*lpfn)(void*);
typedef int (*lpfn2)();

const static SizedSPropTagArray (10, ptaRemFldr) =
{
	10,
	{
		PR_ENTRYID,
		PR_SENT_REPRESENTING_NAME,
		PR_SUBJECT,
		PR_MESSAGE_DELIVERY_TIME,
		PR_MESSAGE_SIZE,
		PR_HASATTACH,
		PR_PRIORITY,
		PR_MESSAGE_FLAGS,
		PR_MSG_STATUS,
		PR_INSTANCE_KEY,
	}
};

enum LVCOLUMNS
{
	LVC_First  = 0,
	LVC_Action = 0,
	LVC_Sender,
	LVC_Subject,
	LVC_Date,
	LVC_Size,
	LVC_Attach,
	LVC_Priority,
	LVC_Extra,
	LVC_EnumSize
};

enum DisplayCmd
{
	DC_Begin = 0,
	DC_End,
	DC_PageUp,
	DC_PageDown,
};

// global variables
BOOL (*g_lpfnCtl3dRegister)(HANDLE) = 0;
BOOL (*g_lpfnCtl3dAutoSubclass)(HANDLE) = 0;
BOOL (*g_lpfnCtl3dUnregister)(HANDLE)  = 0;
BOOL (*g_lpfnCtl3dColorChange)() = 0;

HWND			g_hWndMain = 0,
					g_hWndPgUp = 0,
					g_hWndPgDn = 0,
					g_hWndRmtConn = 0,
					g_hWndListView = 0;
HINSTANCE	g_hInstAppln = 0;
BOOL			g_bCtl3d = FALSE;
ATOM			g_Atom;

const TCHAR* g_pszAppTitle = TEXT("Remote Mail Viewer");

SBinary				g_SBRemoteFolder = {0, 0};
LPMAPISESSION	g_pSession = 0;
LPMAPIFOLDER	g_pFolderRemote = 0;
LPMAPITABLE		g_pTableRmtFldr = 0;
LPMAPISTATUS	g_pStatusRemote = 0;
LPMAPITABLE		g_pTableStatus = 0;
unsigned long	g_ulLVMsgPos = 0;
LPMDB					g_pMDBDef = 0;
BOOL					g_bOkToValidateXP = FALSE;
CAdviseSink*	g_pAdvise = 0;
unsigned long	g_ulConnection = 0;

// function decl.
void SetActionForMessage	(unsigned long ulAction);
void CenterWindow					(HWND hWnd, 
													 HWND hWndParent);
void UninitMapi						();
void UninitInstance				(ATOM& atom);

BOOL InitInstance					(	HINSTANCE hInstance,
														HINSTANCE hPrevInstance, 
														LPSTR lpszCmdLine, 
														int nCmdShow,
														ATOM& atom);
BOOL InitMapi							();
BOOL SelectRemoteFolderUI	(	LPMAPISESSION pSession);
BOOL DisplayMsgHeaders		(	DisplayCmd cmd);

LRESULT InitFileMenu						(HWND hWnd, 
																 UINT uMsgID, 
																 WPARAM wParam, 
																 LPARAM lParam);
LRESULT InitMessageMenu					(HWND hWnd, 
																 UINT uMsgID, 
																 WPARAM wParam, 
																 LPARAM lParam);
LRESULT InitRemoteMenu					(HWND hWnd, 
																 UINT uMsgID, 
																 WPARAM wParam, 
																 LPARAM lParam);
LRESULT InitWndClientArea				(HWND hWnd, 
																 UINT uMsgID, 
																 WPARAM wParam, 
																 LPARAM lParam);
LRESULT RemoteMailClientCmdHndlr(HWND hWnd, 
																 UINT uMsgID, 
																 WPARAM wParam, 
																 LPARAM lParam);

LRESULT CALLBACK RemoteMailClientWndProc	(HWND hWnd, 
																					UINT uMsgID, 
																					WPARAM wParam, 
																					LPARAM lParam);
BOOL CALLBACK RemoteXPDlgProc							(HWND  hwndDlg, 
																					 UINT  uMsg, 
																					 WPARAM  wParam, 
																					 LPARAM  lParam);
BOOL CALLBACK RemoteConnectionDlgProc			(HWND hwndDlg, 
																					 UINT  uMsg, 
																					 WPARAM  wParam, 
																					 LPARAM  lParam);
BOOL CALLBACK AboutDlgProc								(HWND  hwndDlg, 
																					 UINT  uMsg, 
																					 WPARAM  wParam, 
																					 LPARAM  lParam);

int DisplayMapiMessage										(HWND hWnd, 
																					 MAPIERROR *pErr, 
																					 unsigned uFlags = MB_OK | MB_ICONEXCLAMATION);

///////////////////////////////////////////////////////////////////////////////
//    WinMain()
//
//    Parameters
//      { Refer to Win32 SDK documentation }
//      
//    Purpose
//      This is the entry function. The application will start executing here.
//      
//    Return Value
//      Return code of the application to the system.
//      
int WINAPI WinMain (HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPSTR     lpszCmdLine,
					int       nCmdShow)
{
	int	nRet = 1;
	
	// 3D user interface
  HINSTANCE hInstanceCtl3d = LoadLibrary(TEXT("ctl3d32.dll"));
  if (hInstanceCtl3d)
  {
      g_lpfnCtl3dRegister     = (lpfn)GetProcAddress(hInstanceCtl3d, TEXT("Ctl3dRegister"));
      g_lpfnCtl3dAutoSubclass = (lpfn)GetProcAddress(hInstanceCtl3d, TEXT("Ctl3dAutoSubclass"));
      g_lpfnCtl3dUnregister   = (lpfn)GetProcAddress(hInstanceCtl3d, TEXT("Ctl3dUnregister"));
      g_lpfnCtl3dColorChange  = (lpfn2)GetProcAddress(hInstanceCtl3d, TEXT("Ctl3dColorChange"));
      g_bCtl3d = TRUE;
			g_lpfnCtl3dRegister(hInstance);
			g_lpfnCtl3dAutoSubclass(hInstance);
  }
	
	// common controls
	HINSTANCE hLibComCtrl = LoadLibrary("comctl32.dll");
	if (hLibComCtrl)
	{
		FARPROC pfnInit = GetProcAddress(hLibComCtrl, TEXT("InitCommonControls"));
		pfnInit();
	}
	
	// initialize MAPI subsystem
	if (InitMapi())
	{
		// initialize application instance
		if (InitInstance(	hInstance,
							hPrevInstance,
							lpszCmdLine,
							nCmdShow,
							g_Atom))
		{
			MSG			msg;

			while (GetMessage (&msg, NULL, 0, 0))
			{
					TranslateMessage (&msg);
					DispatchMessage (&msg);
			}
			nRet = msg.wParam;
			UninitInstance(g_Atom);
		}
		UninitMapi();
	}

	// free common control library
	if (hLibComCtrl)
	{
		FreeLibrary(hLibComCtrl);
	}

	// free 3d control library
	if (g_bCtl3d)
  {
      g_lpfnCtl3dUnregister(hInstance);
      FreeLibrary(hInstanceCtl3d);
  }

	return nRet;
}

///////////////////////////////////////////////////////////////////////////////
//    RemoteMailClientCmdHndlr()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      depends on the message being processed, 0 is normal return
//      
LRESULT RemoteMailClientCmdHndlr(	HWND hWnd, 
																	UINT uMsgID, 
																	WPARAM wParam, 
																	LPARAM lParam)
{
	LRESULT lRet = 0;   //Return: 0 is normal return
	switch (LOWORD(wParam))
	{
	case ID_PAGE_UP_PB:
		if (!DisplayMsgHeaders(DC_PageUp))
		{
			MessageBox(	hWnd, 
						TEXT("Could not display message headers!"), 
						g_pszAppTitle, 
						MB_OK | MB_ICONEXCLAMATION);
		}
		break;
	case ID_PAGE_DOWN_PB:
		if (!DisplayMsgHeaders(DC_PageDown))
		{
			MessageBox(	hWnd, 
						TEXT("Could not display message headers!"), 
						g_pszAppTitle, 
						MB_OK | MB_ICONEXCLAMATION);
		}
		break;
	case ID_FILE_XP:
		if (SelectRemoteFolderUI(g_pSession))
		{
			if (!DisplayMsgHeaders(DC_Begin))
			{
				MessageBox(	hWnd, 
							TEXT("Could not display message headers!"), 
							g_pszAppTitle, 
							MB_OK | MB_ICONEXCLAMATION);
			}
		}
		break;
	case ID_FILE_EXIT:
		PostMessage(hWnd, WM_CLOSE, 0, 0);
		break;
	case ID_MESSAGE_COPY:
		SetActionForMessage(MSGSTATUS_REMOTE_DOWNLOAD);
		break;
	case ID_MESSAGE_DELETE:
		SetActionForMessage(MSGSTATUS_REMOTE_DELETE);
		break;
	case ID_MESSAGE_MOVE:
		SetActionForMessage(MSGSTATUS_REMOTE_DOWNLOAD | MSGSTATUS_REMOTE_DELETE);
		break;
	case ID_MESSAGE_UNMARK:
		SetActionForMessage(0);
		break;
	case ID_REMOTE_DOWNLOAD:
		{
			int nRet = 0;
			nRet = DialogBoxParam(g_hInstAppln,
														MAKEINTRESOURCE(IDD_REMOTE_CONN_DLG),
														hWnd,
														RemoteConnectionDlgProc,
														(LPARAM)REFRESH_XP_HEADER_CACHE);
			if (nRet != IDCANCEL)
			{
				if (!DisplayMsgHeaders(DC_Begin))
				{
					MessageBox(	hWnd, 
								TEXT("Could not display message headers!"), 
								g_pszAppTitle, 
								MB_OK | MB_ICONEXCLAMATION);
				}
			}
		}
		break;
	case ID_REMOTE_PROCESSMESSAGES:
		{
			int nRet = 0;
			nRet = DialogBoxParam(g_hInstAppln,
														MAKEINTRESOURCE(IDD_REMOTE_CONN_DLG),
														hWnd,
														RemoteConnectionDlgProc,
														(LPARAM)PROCESS_XP_HEADER_CACHE);
			if (nRet != IDCANCEL)
			{
				if (!DisplayMsgHeaders(DC_Begin))
				{
					MessageBox(	hWnd, 
								TEXT("Could not display message headers!"), 
								g_pszAppTitle, 
								MB_OK | MB_ICONEXCLAMATION);
				}
			}
		}
		break;
	case ID_HELP_ABOUT:
		DialogBoxParam(	g_hInstAppln,
										MAKEINTRESOURCE(IDD_ABOUT_DLG),
										hWnd,
										AboutDlgProc,
										(LPARAM)0);
		break;
	default:
		break;
	}
	return lRet;
}

///////////////////////////////////////////////////////////////////////////////
//    RemoteMailClientWndProc()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      depends on the message being processed, 0 is normal return
//      
LRESULT CALLBACK RemoteMailClientWndProc(	HWND hWnd, 
																					UINT uMsgID, 
																					WPARAM wParam, 
																					LPARAM lParam)
{
	LRESULT lRet = 0;   //Return: 0 is normal return

	switch (uMsgID)
	{
	case WM_CREATE:
		lRet = InitWndClientArea(hWnd, uMsgID, wParam, lParam);
		SendMessage(hWnd, WM_COMMAND, MAKELONG(ID_FILE_XP, 0), 0);
		break;
	case WM_GETMINMAXINFO:
		{
			LPMINMAXINFO pMinMaxInfo = (LPMINMAXINFO)lParam;
			pMinMaxInfo->ptMinTrackSize.x = 480;
			pMinMaxInfo->ptMinTrackSize.y = 240;
			break;
		}
	case WM_SIZE:
		{
			unsigned nWidth = LOWORD(lParam);  // width of client area               
			unsigned nHeight = HIWORD(lParam); // height of client area              
			MoveWindow(g_hWndListView, 0, 0, nWidth, nHeight - 45, TRUE);
			RECT r;
			GetWindowRect(g_hWndPgUp, &r);
			SetWindowPos(	g_hWndPgUp, 0, 
										(nWidth / 4) - (r.right - r.left) / 2, 
										nHeight - 35, 
										0, 0, 
										SWP_NOZORDER | SWP_NOSIZE);
			GetWindowRect(g_hWndPgUp, &r);
			SetWindowPos(	g_hWndPgDn, 0, 
										(3 * nWidth / 4) - (r.right - r.left) / 2, 
										nHeight - 35, 
										0, 0, 
										SWP_NOZORDER | SWP_NOSIZE);
			break;
		}
	case WM_DESTROY:
		if (g_pFolderRemote)
		{
			g_pFolderRemote->Release();
			g_pFolderRemote = 0;
		}
		if (g_pTableRmtFldr)
		{
			g_pTableRmtFldr->Release();
			g_pTableRmtFldr = 0;
		}
		if (g_pStatusRemote)
		{
			g_pStatusRemote->Release();
		}
		g_pStatusRemote = 0;
		PostQuitMessage (0);
		break;
	case WM_INITMENUPOPUP:
		{
			switch (LOWORD(lParam))
			{
			case 0:
				lRet = InitFileMenu(hWnd, uMsgID, wParam, lParam);
				break;
			case 1:
				lRet = InitMessageMenu(hWnd, uMsgID, wParam, lParam);
				break;
			case 2:
				lRet = InitRemoteMenu(hWnd, uMsgID, wParam, lParam);
				break;
			default:
				break;
			}
		}
		break;
	case WM_COMMAND:
		lRet = RemoteMailClientCmdHndlr(hWnd, uMsgID, wParam, lParam);
		break;
	default:
		lRet = DefWindowProc (hWnd, uMsgID, wParam, lParam);
		break;
	}
	return lRet;
}

///////////////////////////////////////////////////////////////////////////////
//    InitMapi()
//
//    Parameters
//      None
//
//    Purpose
//      Initialize MAPI, logon using a profile, open the default message store,
//      get the status table which is restricted to transports that provide
//      Remote Mail access and set up an advise to get notified on any changes
//      that occur to the entries of the table.
//      
//    Return Value
//      TRUE if successful, FALSE otherwise
//      
BOOL InitMapi()
{
	MAPIINIT_0 mapiInit = {0, 0};

	HRESULT hr = MAPIInitialize(&mapiInit);
	if (SUCCEEDED(hr))
	{
		hr = MAPILogonEx(	0,
							0,
							0,
							fMapiUnicode | MAPI_LOGON_UI | MAPI_NEW_SESSION,
							&g_pSession);
		if (SUCCEEDED(hr))
		{
			LPMAPITABLE pTable = 0;
			hr = g_pSession->GetMsgStoresTable(fMapiUnicode, &pTable);
			if (SUCCEEDED(hr))
			{
				const SizedSPropTagArray(2, tagArray) =
				{
					2,
					PR_DEFAULT_STORE,
					PR_ENTRYID
				};
				SRestriction res;
				SPropValue propVal;
				LPSRowSet pRow = 0;

				propVal.ulPropTag = PR_DEFAULT_STORE;
				propVal.Value.b   = TRUE;

				res.rt = RES_PROPERTY;
				res.res.resProperty.relop     = RELOP_EQ;
				res.res.resProperty.ulPropTag = PR_DEFAULT_STORE;
				res.res.resProperty.lpProp    = &propVal;

				hr = HrQueryAllRows(pTable,
														(LPSPropTagArray)&tagArray,
														&res,
														0,
														0,
														&pRow);
				if (SUCCEEDED(hr))
				{
					if (pRow->cRows > 0)
					{
						hr = g_pSession->OpenMsgStore(0,
															pRow->aRow[0].lpProps[1].Value.bin.cb,
															(LPENTRYID)(pRow->aRow[0].lpProps[1].Value.bin.lpb),
															0,
															MAPI_BEST_ACCESS,
															&g_pMDBDef);
						if (SUCCEEDED(hr))
						{
							hr = g_pSession->GetStatusTable(fMapiUnicode, &g_pTableStatus);
							if (SUCCEEDED(hr))
							{
								const SizedSPropTagArray(7, tagArray) =
								{
									7,
									PR_DISPLAY_NAME,
									PR_ENTRYID,
									PR_RESOURCE_TYPE,
									PR_STATUS_CODE,
									PR_REMOTE_VALIDATE_OK,
									PR_REMOTE_PROGRESS,
									PR_REMOTE_PROGRESS_TEXT,
									// PR_PROVIDER_DISPLAY
								};
								SRestriction aSR[2];
								SPropValue spvTransport = { 0 };
								spvTransport.ulPropTag = PR_RESOURCE_TYPE;
								spvTransport.Value.l = MAPI_TRANSPORT_PROVIDER;

								SRestriction sr = { 0 };
								sr.rt               = RES_AND;
								sr.res.resAnd.cRes  = 2;
								sr.res.resAnd.lpRes = aSR;

								aSR[0].rt                        = RES_PROPERTY;
								aSR[0].res.resProperty.relop     = RELOP_EQ;
								aSR[0].res.resProperty.ulPropTag = PR_RESOURCE_TYPE;
								aSR[0].res.resProperty.lpProp    = &spvTransport;

								aSR[1].rt                        = RES_BITMASK;
								aSR[1].res.resBitMask.relBMR     = BMR_NEZ;
								aSR[1].res.resBitMask.ulPropTag  = PR_STATUS_CODE;
								aSR[1].res.resBitMask.ulMask     = STATUS_REMOTE_ACCESS;

								hr = g_pTableStatus->SetColumns((LPSPropTagArray)&tagArray, 0);
								if (SUCCEEDED(hr))
								{
									hr = g_pTableStatus->Restrict(&sr, 0);
									if (SUCCEEDED(hr))
									{
										g_pAdvise = new CAdviseSink();
										if (g_pAdvise)
										{
											hr = g_pTableStatus->Advise(fnevTableModified,
																									g_pAdvise,
																									&g_ulConnection);
											if (SUCCEEDED(hr))
											{
												return TRUE;
											}
										}
									}
								}
							}
						}
					}
					FreeProws(pRow);
				}
				pTable->Release();
			}
		}
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//    UninitMapi()
//
//    Parameters
//      None
//
//    Purpose
//      Undo the operation performed in InitMapi()
//      
//    Return Value
//      None
//      
void UninitMapi()
{
	if (g_pTableStatus)
	{
		g_pTableStatus->Unadvise(g_ulConnection);
		g_pTableStatus->Release();
		g_pTableStatus = 0;
	}
	// should not be deleted, because Unadvise()
	// performs a release which deletes the object
	// delete g_pAdvise;
	if (g_pMDBDef)
	{
		g_pMDBDef->Release();
		g_pMDBDef = 0;
	}
	if (g_pSession)
	{
		g_pSession->Logoff(0, 0, 0);
		g_pSession->Release();
		g_pSession = 0;
		MAPIUninitialize();
	}
}

///////////////////////////////////////////////////////////////////////////////
//    InitInstance()
//
//    Parameters
//      hInstance,			handle of the application instance
//			hPrevInstance,	handle of the application's previous instance
//			lpszCmdLine,		command line parameters
//			nCmdShow,				show/hide/minimize main window
//      atom						global atom
//
//    Purpose
//      initialize the application, register window class etc.
//      
//    Return Value
//      TRUE if successful, FALSE otherwise
BOOL InitInstance(	HINSTANCE hInstance,
										HINSTANCE hPrevInstance,
										LPSTR     lpszCmdLine,
										int       nCmdShow,
										ATOM&			atom)
{
	WNDCLASS	wc;

	ZeroMemory (&wc, sizeof(wc));
	ZeroMemory (&g_Atom, sizeof(g_Atom));

	g_hInstAppln = hInstance;

	wc.lpszClassName = TEXT("RemoteMailClient");
	wc.hInstance     = hInstance;
	wc.lpfnWndProc   = RemoteMailClientWndProc;
	wc.hCursor       = LoadCursor (0, IDC_ARROW);
	wc.hIcon         = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_RMC_ICON));
	wc.lpszMenuName  = MAKEINTRESOURCE(IDR_RMC_MENU);
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW +1);
	wc.style         = 0;
	wc.cbClsExtra    = 0;
	wc.cbWndExtra    = 0;

	atom = RegisterClass (&wc);
	if (0 != atom)
	{
		g_hWndMain = CreateWindow (	(LPCSTR)atom,
									g_pszAppTitle,
									WS_OVERLAPPEDWINDOW,
									CW_USEDEFAULT,
									0,
									CW_USEDEFAULT,
									0,
									NULL,
									NULL,
									hInstance,
									NULL);
		CenterWindow(g_hWndMain, 0);
		ShowWindow (g_hWndMain, nCmdShow);
		return TRUE;
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//    UninitInstance()
//
//    Parameters
//     None
//
//    Purpose
//      undo the operations performed in InitInstance()
//      
//    Return Value
//      None
void UninitInstance(ATOM& atom)
{
	UnregisterClass((LPCSTR)atom,
									g_hInstAppln);
}

///////////////////////////////////////////////////////////////////////////////
//    SelectRemoteFolderUI()
//
//    Parameters
//      pSession,			handle of the MAPI session
//
//    Purpose
//      display a dialog to select one of the possibly many transports that
//      support Remote Mail access. Then open the Remote Mail folder.
//      
//    Return Value
//      TRUE if successful, FALSE otherwise
BOOL SelectRemoteFolderUI(LPMAPISESSION pSession)
{
	BOOL bRet = FALSE;
	if (g_pTableStatus)
	{
		unsigned long ulRows = 0;
		HRESULT hr = g_pTableStatus->GetRowCount(0, &ulRows);
		if (SUCCEEDED(hr))
		{
			LPSRowSet psRowSet = 0;
			
			hr = g_pTableStatus->QueryRows(ulRows, TBL_NOADVANCE, &psRowSet);
			if (SUCCEEDED(hr))
			{
				if (psRowSet->cRows)
				{
					LPSPropValue pspVal = 0;
					XPList xpList = {0, 0};
					xpList.xpl_uCnt = psRowSet->cRows;
					if (xpList.xpl_uCnt)
					{
						xpList.xpl_pszaNames = (TCHAR**) new unsigned long[xpList.xpl_uCnt];
						for (unsigned int i = 0;i < xpList.xpl_uCnt;i++)
						{
							pspVal = psRowSet->aRow[i].lpProps;
							xpList.xpl_pszaNames[i] = pspVal[0].Value.LPSZ;
						}
						xpList.xpl_uRet = (unsigned)-1;
						int nSel = DialogBoxParam(	g_hInstAppln,
																				MAKEINTRESOURCE(IDD_RMTXP_DLG),
																				g_hWndMain,
																				RemoteXPDlgProc,
																				(LPARAM)&xpList);
						delete []xpList.xpl_pszaNames;
						xpList.xpl_pszaNames = 0;
						xpList.xpl_uCnt = 0;

						if (nSel != -1)
						{
							pspVal = psRowSet->aRow[nSel].lpProps;
							ULONG ulObjType;
							LPMAPISTATUS pStatus;
							// Get the status object of the specific transport
							hr = pSession->OpenEntry(	pspVal[1].Value.bin.cb,
														(LPENTRYID)pspVal[1].Value.bin.lpb,
														&IID_IMAPIStatus,
														0,
														&ulObjType,
														(LPUNKNOWN*)&pStatus);
							if (SUCCEEDED(hr))
							{
								// Make sure the object we got is of the type it should be
								if (MAPI_STATUS == ulObjType)
								{
									LPMAPIFOLDER	pFolder = 0;
									// On transport that implement the remote interface, 
									// should support returning the remote folder mailbag 
									// through the QueryInterface() of the status object.
									// This folder has the list of message in the remote 
									// host which the client will instruct the transport 
									// to act upon.
									hr = pStatus->QueryInterface(IID_IMAPIFolder, 
																(LPVOID *)&pFolder);
									if (SUCCEEDED(hr))
									{
										LPMAPITABLE pTableRmt = 0;
										hr = pFolder->GetContentsTable(fMapiUnicode, 
																				&pTableRmt);
										if (SUCCEEDED(hr))
										{
											bRet = TRUE;
											if (g_pStatusRemote)
											{
												g_pStatusRemote->Release();
											}
											g_pStatusRemote = pStatus;
											if (g_pFolderRemote)
											{
												g_pFolderRemote->Release();
											}
											g_pFolderRemote = pFolder;
											if (g_pTableRmtFldr)
											{
												g_pTableRmtFldr->Release();
											}
											g_pTableRmtFldr = pTableRmt;
											
											g_bOkToValidateXP = pspVal[4].Value.b;
										}
										else
										{
											pFolder->Release();
											pFolder = 0;
											pTableRmt->Release();
										}
									}
								}
							}
						}
					}
				}
				FreeProws(psRowSet);
			}
		}
	}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    InitFileMenu()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//      enable/disable menu items
//
//    Purpose
//      
//    Return Value
//      depends on the message being processed, 0 is normal return
//      
LRESULT InitFileMenu(HWND hWnd, UINT uMsgID, WPARAM wParam, LPARAM lParam)
{
	LRESULT lRet = 0;
	HMENU hMenu = (HMENU)wParam;
	unsigned uEnable = MF_GRAYED | MF_BYCOMMAND;
	if (g_pSession)
	{
		uEnable = MF_ENABLED | MF_BYCOMMAND;
	}
	EnableMenuItem(hMenu, ID_FILE_XP, uEnable);
	return lRet;
}

///////////////////////////////////////////////////////////////////////////////
//    InitMessageMenu()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      enable/disable menu items
//      
//    Return Value
//      depends on the message being processed, 0 is normal return
//      
LRESULT InitMessageMenu(HWND hWnd, UINT uMsgID, WPARAM wParam, LPARAM lParam)
{
	LRESULT lRet = 0;
	HMENU hMenu = (HMENU)wParam;
	unsigned uEnable = MF_GRAYED | MF_BYCOMMAND;
	if (ListView_GetSelectedCount(g_hWndListView))
	{
		uEnable = MF_ENABLED | MF_BYCOMMAND;
	}
	EnableMenuItem(hMenu, ID_MESSAGE_COPY, uEnable);
	EnableMenuItem(hMenu, ID_MESSAGE_DELETE, uEnable);
	EnableMenuItem(hMenu, ID_MESSAGE_MOVE, uEnable);
	EnableMenuItem(hMenu, ID_MESSAGE_UNMARK, uEnable);
	return lRet;
}

///////////////////////////////////////////////////////////////////////////////
//    InitRemoteMenu()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      enable/disable menu items
//      
//    Return Value
//      depends on the message being processed, 0 is normal return
//      
LRESULT InitRemoteMenu(HWND hWnd, UINT uMsgID, WPARAM wParam, LPARAM lParam)
{
	LRESULT lRet = 0;
	HMENU hMenu = (HMENU)wParam;
	unsigned uEnable = MF_GRAYED | MF_BYCOMMAND;
	if (g_pFolderRemote && g_pStatusRemote && g_bOkToValidateXP)
	{
		uEnable = MF_ENABLED | MF_BYCOMMAND;
	}
	EnableMenuItem(hMenu, ID_REMOTE_DOWNLOAD, uEnable);
	EnableMenuItem(hMenu, ID_REMOTE_PROCESSMESSAGES, uEnable);

	return lRet;
}

///////////////////////////////////////////////////////////////////////////////
//    InitWndClientArea()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      create, initialize and display ListView control in the client area.
//      
//    Return Value
//      depends on the message being processed, 0 is normal return
//      
LRESULT InitWndClientArea(HWND hWnd, UINT uMsgID, WPARAM wParam, LPARAM lParam)
{
	LRESULT lRet = 0;
	g_hWndPgUp = CreateWindow(TEXT("BUTTON"), 
														TEXT("Up"), 
														WS_CHILD | WS_VISIBLE | WS_BORDER | BS_PUSHBUTTON | BS_NOTIFY,
														400, 400, 100, 25, 
														hWnd, 
														(HMENU)ID_PAGE_UP_PB, 
														g_hInstAppln, 
														0);
	g_hWndPgDn = CreateWindow(TEXT("BUTTON"), 
														TEXT("Down"), 
														WS_CHILD | WS_VISIBLE | WS_BORDER | BS_PUSHBUTTON | BS_NOTIFY,
														400, 400, 100, 25, 
														hWnd, 
														(HMENU)ID_PAGE_DOWN_PB, 
														g_hInstAppln, 
														0);
	g_hWndListView = CreateWindow (	WC_LISTVIEW,
									TEXT(""),
									LVS_REPORT | LVS_SINGLESEL | WS_VISIBLE | WS_CHILD | WS_BORDER,
									0,
									0,
									100,
									100,
									hWnd,
									NULL,
									g_hInstAppln,
									NULL);
	if (g_hWndListView)
	{
		RECT rect;
		unsigned uWidth = 0;

		GetClientRect(hWnd, &rect);
		MoveWindow(	g_hWndListView, 
					0, 
					0, 
					rect.right - rect.left, 
					rect.bottom - rect.top, 
					TRUE);
		LV_COLUMN lvC;
		lvC.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
		lvC.fmt = LVCFMT_LEFT;
		TCHAR *lpszHeading[LVC_EnumSize] =
		{
			TEXT("Action"),
			TEXT("Sender"),
			TEXT("Subject"),
			TEXT("Date"),
			TEXT("Size"),
			TEXT("Attachment"),
			TEXT("Priority"),
			TEXT(""),
		};
		unsigned lpColSize[LVC_EnumSize] =
		{
			5, 10, 50, 20, 5, 5, 5
		};
		
		GetWindowRect(g_hWndListView, &rect);
		uWidth = rect.right - rect.left;
		// Add the columns.
		for (int i = LVC_First;i < LVC_EnumSize;i++)
		{
			lvC.cx = lpColSize[i] * uWidth / 100;
			lvC.iSubItem = i;
			lvC.pszText = lpszHeading[i];
			ListView_InsertColumn(g_hWndListView, i, &lvC);
		}
	}
	return lRet;
}

///////////////////////////////////////////////////////////////////////////////
//    DisplayMsgHeaders()
//
//    Parameters
//      ulStartPos,     starting position of the header
//      ulEndPos,       ending position of the header
//
//    Purpose
//      display the message headers in the ListView Ctrl.
//      
//    Return Value
//      TRUE if successful, FALSE otherwise
//      
BOOL DisplayMsgHeaders(DisplayCmd cmd)
{
	BOOL bRet = FALSE;
	HRESULT hr = 0;
	long ulRowsSought = 0;
	unsigned uLVPageCnt = ListView_GetCountPerPage(g_hWndListView);
	unsigned long	ulNumerator = 0,
								ulDenominator = 0,
								ulRowPos = 0;

	ListView_DeleteAllItems(g_hWndListView);

	hr = g_pTableRmtFldr->SetColumns(	(LPSPropTagArray)&ptaRemFldr, 
								TBL_BATCH);
	if (SUCCEEDED(hr))
	{
		g_pTableRmtFldr->QueryPosition(	&ulRowPos,
																		&ulNumerator,
																		&ulDenominator);
		if (SUCCEEDED(hr))
		{
			if (cmd == DC_Begin)
			{
				ulNumerator = 0;
			}
			else if (cmd == DC_End)
			{
				ulNumerator = ulDenominator - uLVPageCnt;
			}
			else if (cmd == DC_PageUp)
			{
				if ((long)(ulNumerator - uLVPageCnt) < 0)
				{
					ulNumerator = 0;
				}
				else
				{
					ulNumerator = ulNumerator - uLVPageCnt;
				}
			}
			else if (cmd == DC_PageDown && (ulDenominator - ulNumerator) > uLVPageCnt)
			{
				if ((ulNumerator + uLVPageCnt) < ulDenominator)
				{
					ulNumerator += uLVPageCnt;
				}
			}
			else
			{
					return TRUE;
			}
			if (ulDenominator != 0)
			{
				hr = g_pTableRmtFldr->SeekRowApprox(ulNumerator,
																						ulDenominator);
				if (SUCCEEDED(hr))
				{
					LPSRowSet pSRowSet = 0;
					hr = g_pTableRmtFldr->QueryRows(uLVPageCnt, TBL_NOADVANCE, &pSRowSet);
					if (SUCCEEDED(hr))
					{
						LPSPropValue pspVal = 0;
						// List view item structure
						LV_ITEM lvI;
						lvI.mask = LVIF_PARAM;
						lvI.iSubItem = 0;
						for (unsigned long i = 0;i < pSRowSet->cRows;i++)
						{
							pspVal = pSRowSet->aRow[i].lpProps;

							lvI.iItem = i;
							SBinary *pSB = (SBinary*) new BYTE[sizeof(SBinary) + pspVal[0].Value.bin.cb];
							pSB->lpb = ((BYTE*)pSB) + sizeof(SBinary);
							pSB->cb = pspVal[0].Value.bin.cb;
							CopyMemory(pSB->lpb, pspVal[0].Value.bin.lpb, pSB->cb);
							lvI.lParam = (LPARAM) pSB;
							// insert a row in the ListView
							ListView_InsertItem(g_hWndListView, &lvI);

							// display sender, subject
							ListView_SetItemText(	g_hWndListView,
																		i, 
																		LVC_Sender, 
																		pspVal[LVC_Sender].Value.LPSZ);
							ListView_SetItemText(	g_hWndListView,
																		i, 
																		LVC_Subject,
																		pspVal[LVC_Subject].Value.LPSZ);

							// date & time
							TCHAR lpTmp[128];
							SYSTEMTIME st;
							FileTimeToSystemTime(&pspVal[LVC_Date].Value.ft, &st);
							sprintf(lpTmp, TEXT("%02u/%02u/%02u %02u:%02u"), st.wMonth, st.wDay, st.wYear,
								st.wHour, st.wMinute);
							ListView_SetItemText(g_hWndListView, i, LVC_Date, lpTmp);

							// size
							sprintf(lpTmp, TEXT("%0.2f"), ((float)pspVal[LVC_Size].Value.ul) / 1024.00);
							ListView_SetItemText(g_hWndListView, i, LVC_Size, lpTmp);

							// attachment
							lpTmp[0] = pspVal[LVC_Attach].Value.b ? TCHAR('A') : TCHAR(' ');
							lpTmp[1] = 0;
							ListView_SetItemText(g_hWndListView, i, LVC_Attach, lpTmp);
							
							// priority
							if (pspVal[LVC_Priority].Value.l == PRIO_NORMAL)
							{
								lpTmp[0] = TCHAR('N');
							}
							else if (pspVal[LVC_Priority].Value.l == PRIO_URGENT)
							{
								lpTmp[0] = TCHAR('H');
							}
							else
							{
								lpTmp[0] = TCHAR('L');
							}
							ListView_SetItemText(g_hWndListView, i, LVC_Priority, lpTmp);
						}
						bRet = TRUE;
						FreeProws(pSRowSet);
					}
				}
			}
			else
			{
				bRet = TRUE;
			}
		}
	}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    RemoteXPDlgProc()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      TRUE if message was handled, FALSE if we don't handle the message
//      
BOOL CALLBACK RemoteXPDlgProc(HWND  hwndDlg, UINT  uMsg, WPARAM  wParam, LPARAM  lParam)
{
	BOOL bRet = FALSE;

	switch (uMsg)
		{
		case WM_INITDIALOG:
			{
				CenterWindow(hwndDlg, 0);
				XPList& xpl = *(XPList*)lParam;
				for (unsigned i = 0;i < xpl.xpl_uCnt;i++)
				{
					SendDlgItemMessage(	hwndDlg, 
															IDC_XP_LB, 
															LB_ADDSTRING, 
															0, 
															(LPARAM)(LPCTSTR)xpl.xpl_pszaNames[i]);
				}
				if (xpl.xpl_uCnt)
				{
					SendDlgItemMessage(	hwndDlg, 
															IDC_XP_LB, 
															LB_SETCURSEL, 
															0, 
															0);
				}
			}
			break;
		case WM_COMMAND:
			if (HIWORD(wParam) == BN_CLICKED)
			{
				if (LOWORD(wParam) == IDOK)
				{
					EndDialog(hwndDlg,
										SendDlgItemMessage(	hwndDlg, 
																				IDC_XP_LB, 
																				LB_GETCURSEL, 
																				0, 
																				0));
				}
				else if (LOWORD(wParam) == IDCANCEL)
				{
					EndDialog(hwndDlg, -1);
				}
			}
			break;
    case WM_SYSCOLORCHANGE:
      if (g_bCtl3d)
      {
        g_lpfnCtl3dColorChange();
      }
      break;
		default:
			break;
		}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    DisplayMapiMessage()
//
//    Parameters
//      hWnd,           handle of the parent window
//      pError,         pointer to MAPI error
//      uFlags,         MB_OK, etc.    
//
//    Purpose
//      display MAPI error messages.
//      
//    Return Value
//      return from MessageBox();
//      
int DisplayMapiMessage(HWND hWnd, MAPIERROR* pError, unsigned uFlags)
{
		int bRet = MessageBox(	hWnd, 
														pError->lpszError, 
														pError->lpszComponent, 
														uFlags);
		MAPIFreeBuffer(pError);
		return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    SetActionForMessage()
//
//    Parameters
//      ulAction,    action to be performed on the message
//
//    Purpose
//      mark a message for Copy, Move, Delete and display it 
//      on the ListView Ctrl.
//      
//    Return Value
//      None
//      
void SetActionForMessage(unsigned long ulAction)
{
	int i = ListView_GetNextItem(g_hWndListView, -1, LVNI_SELECTED);
	LV_ITEM item;

	if (i != -1)
	{
		ZeroMemory(&item, sizeof(item));
		item.mask = LVIF_PARAM;
		item.iItem = i;
		ListView_GetItem(g_hWndListView, &item);
		SBinary* pSB = (SBinary*)item.lParam;
		unsigned long ulOldAction = 0;
		HRESULT hr = 
		g_pFolderRemote->SetMessageStatus(pSB->cb,
																			(LPENTRYID)pSB->lpb,
																			0,
																			MSGSTATUS_REMOTE_DOWNLOAD | MSGSTATUS_REMOTE_DELETE,
																			&ulOldAction);
		hr = g_pFolderRemote->SetMessageStatus(pSB->cb,
																			(LPENTRYID)pSB->lpb,
																			ulAction,
																			MSGSTATUS_REMOTE_DOWNLOAD | MSGSTATUS_REMOTE_DELETE,
																			&ulOldAction);
		TCHAR *pszTmp = TEXT("");
		if ((ulAction & MSGSTATUS_REMOTE_DOWNLOAD) && 
				(ulAction & MSGSTATUS_REMOTE_DELETE))
		{
			pszTmp = TEXT("Move");
		}
		else if (ulAction & MSGSTATUS_REMOTE_DOWNLOAD)
		{
			pszTmp = TEXT("Copy");
		}
		else if (ulAction & MSGSTATUS_REMOTE_DELETE)
		{
			pszTmp = TEXT("Delete");
		}
		ListView_SetItemText(g_hWndListView, i, LVC_Action, pszTmp);
	}
}

///////////////////////////////////////////////////////////////////////////////
//    RemoteConnectionDlgProc()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      TRUE if message was handled, FALSE if we don't handle the message
//      
BOOL CALLBACK RemoteConnectionDlgProc(HWND  hwndDlg, UINT  uMsg, WPARAM  wParam, LPARAM  lParam)
{
	BOOL bRet = FALSE;
	HRESULT hr = 0;
	const int nTimer = 0;
	const unsigned uTimeOut = 2000;
	static unsigned long ulAction = 0;

	switch (uMsg)
		{
		case WM_INITDIALOG:
			{
				ulAction = lParam;
				SendDlgItemMessage(hwndDlg, IDC_XP_PR, PBM_SETRANGE, 0, MAKELPARAM(0, 100));
				g_hWndRmtConn = hwndDlg;
				if (g_bOkToValidateXP)
				{
					hr = g_pStatusRemote->ValidateState((unsigned long)hwndDlg,
														ulAction | // FORCE_XP_CONNECT | 
														PROCESS_XP_HEADER_CACHE | REFRESH_XP_HEADER_CACHE);
					if (SUCCEEDED(hr))
					{
						hr = g_pStatusRemote->ValidateState((unsigned long)hwndDlg, 
																							FORCE_XP_DISCONNECT);
						EndDialog(hwndDlg, IDOK);
						g_bOkToValidateXP = TRUE;
					}
					else if (hr != MAPI_E_BUSY)
					{
						MAPIERROR *pErr = 0;
						g_pStatusRemote->GetLastError(hr, fMapiUnicode, &pErr);
						DisplayMapiMessage(hwndDlg, pErr);
						EndDialog(hwndDlg, IDCANCEL);
					}
				}
				else
				{
					MessageBox(hwndDlg, TEXT("MAPI is busy!"), TEXT(""), MB_OK);
					EndDialog(hwndDlg, IDCANCEL);
				}
				CenterWindow(hwndDlg, 0);
				SetTimer(hwndDlg, nTimer, uTimeOut, 0);
			}
			break;
		case WM_COMMAND:
			if (LOWORD(wParam) == IDCANCEL && HIWORD(wParam) == BN_CLICKED)
			{
				hr = g_pStatusRemote->ValidateState((unsigned long)hwndDlg,
																				ABORT_XP_HEADER_OPERATION);
				if (SUCCEEDED(hr))
				{
					hr = g_pStatusRemote->ValidateState((unsigned long)hwndDlg, 
																							FORCE_XP_DISCONNECT);
					KillTimer(hwndDlg, nTimer);
					EndDialog(hwndDlg, IDCANCEL);
				}
			}
			else if (LOWORD(wParam) == IDC_STATUS_NOTIFY_MSG || g_bOkToValidateXP)
			{
				KillTimer(hwndDlg, nTimer);
				SetTimer(hwndDlg, nTimer, uTimeOut, 0);
				hr = g_pStatusRemote->ValidateState((unsigned long)hwndDlg,
															ulAction | FORCE_XP_CONNECT | 
															PROCESS_XP_HEADER_CACHE | REFRESH_XP_HEADER_CACHE);
				if (SUCCEEDED(hr))
				{
					hr = g_pStatusRemote->ValidateState((unsigned long)hwndDlg, 
																							FORCE_XP_DISCONNECT);
					EndDialog(hwndDlg, IDOK);
				}
			}
			else if (LOWORD(wParam) == IDC_REMOTE_PROGRESS_MSG)
			{
				KillTimer(hwndDlg, nTimer);
				SetTimer(hwndDlg, nTimer, uTimeOut, 0);
				SendDlgItemMessage(hwndDlg, IDC_XP_PR, PBM_SETPOS, lParam, 0);
			}
			break;
		case WM_TIMER:
			if (wParam == 0 && !g_bOkToValidateXP)
			{
					hr = g_pStatusRemote->ValidateState((unsigned long)hwndDlg,
															ulAction | FORCE_XP_CONNECT);
					if (SUCCEEDED(hr))
					{
						KillTimer(hwndDlg, nTimer);
						EndDialog(hwndDlg, IDOK);
					}
			}
			break;
    case WM_SYSCOLORCHANGE:
      if (g_bCtl3d)
      {
        g_lpfnCtl3dColorChange();
      }
      break;
		case WM_DESTROY:
			KillTimer(hwndDlg, nTimer);
			g_hWndRmtConn = 0;
		default:
			break;
		}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    AboutDlgProc()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      TRUE if message was handled, FALSE if we don't handle the message
//      
BOOL CALLBACK AboutDlgProc(HWND  hwndDlg, UINT  uMsg, WPARAM  wParam, LPARAM  lParam)
{
	BOOL bRet = FALSE;
	if (uMsg == WM_COMMAND && LOWORD(wParam) == IDOK && HIWORD(wParam) == BN_CLICKED)
	{
		EndDialog(hwndDlg, IDOK);
		bRet = TRUE;
	}
	else if (uMsg == WM_SYSCOLORCHANGE && g_bCtl3d)
  {
    g_lpfnCtl3dColorChange();
		bRet = TRUE;
  }
	else if (uMsg == WM_INITDIALOG)
	{
		CenterWindow(hwndDlg, 0);
	}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    CenterWindow()
//
//    Parameters
//      handle of the window to center
//
//    Purpose
//      to center a windowrelative to the screen dimensions
//      
//    Return Value
//      None
//      
void CenterWindow(HWND hWnd, HWND hWndParent)
{
	RECT rect;
	unsigned uXExt;
	unsigned uYExt;
	if (hWndParent != 0)
	{
		GetWindowRect(hWndParent, &rect);
		uXExt = GetSystemMetrics(SM_CXSCREEN);
		uYExt = GetSystemMetrics(SM_CYSCREEN);
	}
	else
	{
		GetWindowRect(hWnd, &rect);
		uXExt = GetSystemMetrics(SM_CXSCREEN);
		uYExt = GetSystemMetrics(SM_CYSCREEN);
	}
	uXExt -= (rect.right - rect.left);
	uYExt -= (rect.bottom - rect.top);
	SetWindowPos(hWnd, 0, uXExt / 2, uYExt /2, 0, 0, SWP_NOSIZE);
}
