///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      RMSRPC.CPP
//
//  Description
//      This file implements all the remote functions available to client
//      WINDS public folder store providers. 
//      
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#include "_WINDS.H"
#include <RPC.H>
#include "WINDS.H"     // Header file generated by the MIDL compiler

extern "C"
{
    HRESULT WINAPI PF_GetPFTable
						(DWORD						dwFolderID,
						 BOOL						fHierTable,
						 long *						pPipeNumber);
	HRESULT WINAPI PF_EnumSubFolders
                        (LPSTORAGE                  pRootStg,
                         LPSTORAGE                  pFldStg,
                         HANDLE                     hPipe);
    HRESULT WINAPI PF_EnumFldMessages
                        (LPSTORAGE                  pFldStg,
                         HANDLE                     hPipe,
                         LPMALLOC                   pMalloc);
    HRESULT WINAPI PF_GetNextObjID
                        (LPSTORAGE                  pStorage,
                         DWORD *                    pdwNextID,
                         BOOL                       fNewFolderID);
};

///////////////////////////////////////////////////////////////////////////////
//    RemotePFGetNamePropStream()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
unsigned long RemotePFGetNamePropStream (unsigned long ulFlags,
                                         long *        pPipeNumber)
{
    long lResult = GetServiceState();
    if (lResult)
    {
        return lResult;
    }
    *pPipeNumber = GetNextPipeID();
    
    SECURITY_ATTRIBUTES sa;
    SECURITY_DESCRIPTOR sd;

    // Initialize the new security descriptor.
    InitializeSecurityDescriptor (&sd, SECURITY_DESCRIPTOR_REVISION);

    // Add a NULL descriptor ACL to the security descriptor.
    SetSecurityDescriptorDacl (&sd, TRUE, (PACL)NULL, FALSE);

    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
    sa.lpSecurityDescriptor = &sd;
    sa.bInheritHandle = TRUE;

    TCHAR szPipeName[64];
    wsprintf (szPipeName, SERVER_PIPE_NAME_FORMAT, *pPipeNumber);
    HANDLE hPipe = CreateNamedPipe (szPipeName,
                                    PIPE_ACCESS_OUTBOUND,
                                    PIPE_WAIT | PIPE_READMODE_BYTE | PIPE_TYPE_BYTE,
                                    1,
                                    IO_BUFFERSIZE,
                                    IO_BUFFERSIZE,
                                    0,
                                    &sa);
    if (INVALID_HANDLE_VALUE == hPipe || ERROR_INVALID_PARAMETER == (DWORD)hPipe)
    {
        lResult = HRESULT_FROM_WIN32(GetLastError());
        TraceResult ("RemotePFGetNamePropStream: Failed to create pipe", lResult);
    }
    else
    {
        EnterCriticalSection (&g_csIOInfo);
        SetEvent (g_IOInfo.hResumeEvent);
        g_IOInfo.Action           = IO_GET_NAMED_PROP_STREAM;
        g_IOInfo.hActionCompleted = NULL;
        g_IOInfo.hTmpFile         = hPipe;
        g_IOInfo.pdwData          = NULL;
        g_IOInfo.phLastError      = NULL;
        LeaveCriticalSection (&g_csIOInfo);
    }    
    return lResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_HandleRequest()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_HandleRequest (DWORD                  dwObjectID,
                                 PUBLIC_FOLDER_INFO_A * pFldProps,
                                 ACTION                 Request)
{
    HRESULT hResult = GetServiceState();
    if (hResult)
    {
        return hResult;
    }
    HANDLE hWaitEvent = CreateEvent (NULL, FALSE, FALSE, NULL);
    if (!hWaitEvent)
    {
        hResult = HRESULT_FROM_WIN32(GetLastError());
        TraceResult ("PF_HandleRequest: Failed to create I/O event", hResult);
        return hResult;
    }
    EnterCriticalSection (&g_csIOInfo);
    SetEvent (g_IOInfo.hResumeEvent);
    g_IOInfo.Action           = Request;
    g_IOInfo.hActionCompleted = hWaitEvent;
    g_IOInfo.phLastError      = &hResult;
    g_IOInfo.dwObjID          = dwObjectID;
    g_IOInfo.pPFInfo          = pFldProps;
    LeaveCriticalSection (&g_csIOInfo);
    WaitForSingleObject (hWaitEvent, GENERAL_TIME_OUT);
    CloseHandle (hWaitEvent);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    RemotePFCreateFolder()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
unsigned long RemotePFCreateFolder (PUBLIC_FOLDER_INFO_A * pFldProps)
{
    return PF_HandleRequest (0, pFldProps, IO_CREATE_PUBLIC_FOLDER);
}

///////////////////////////////////////////////////////////////////////////////
//    RemotePFDeleteFolder()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
unsigned long RemotePFDeleteFolder (unsigned long ulFolderID)
{
    return PF_HandleRequest (ulFolderID, NULL, IO_DELETE_PUBLIC_FOLDER);
}

///////////////////////////////////////////////////////////////////////////////
//    RemotePFGetFolderProps()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
unsigned long RemotePFGetFolderProps (unsigned long          ulFolderID,
                                      PUBLIC_FOLDER_INFO_A * pFldProps)
{
    return PF_HandleRequest (ulFolderID, pFldProps, IO_GET_PF_PROPERTIES);
}

///////////////////////////////////////////////////////////////////////////////
//    RemotePFSetFolderProps()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
unsigned long RemotePFSetFolderProps (unsigned long          ulFolderID,
                                      PUBLIC_FOLDER_INFO_A * pFldProps)
{
    return PF_HandleRequest (ulFolderID, pFldProps, IO_SET_PF_PROPERTIES);
}

///////////////////////////////////////////////////////////////////////////////
//    RemotePFGetHierarchyTable()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
unsigned long RemotePFGetHierarchyTable (unsigned long ulFolderID,
										 long *		   pPipeNumber)
{
    return PF_GetPFTable (ulFolderID, TRUE, pPipeNumber);
}

///////////////////////////////////////////////////////////////////////////////
//    RemotePFGetContentsTable()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
unsigned long RemotePFGetContentsTable (unsigned long ulFolderID,
									    long *		  pPipeNumber)
{
    return PF_GetPFTable (ulFolderID, FALSE, pPipeNumber);
}

///////////////////////////////////////////////////////////////////////////////
//    PF_GetPFTable()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_GetPFTable (DWORD dwFolderID, BOOL fHierTable, long * pPipeNumber)
{
	long lResult = GetServiceState();
    if (lResult)
    {
        return lResult;
    }
	*pPipeNumber = GetNextPipeID();
    
    SECURITY_ATTRIBUTES sa;
    SECURITY_DESCRIPTOR sd;

    // Initialize the new security descriptor.
    InitializeSecurityDescriptor (&sd, SECURITY_DESCRIPTOR_REVISION);

    // Add a NULL descriptor ACL to the security descriptor.
    SetSecurityDescriptorDacl (&sd, TRUE, (PACL)NULL, FALSE);

    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
    sa.lpSecurityDescriptor = &sd;
    sa.bInheritHandle = TRUE;

    TCHAR szPipeName[64];
    wsprintf (szPipeName, SERVER_PIPE_NAME_FORMAT, *pPipeNumber);
    HANDLE hPipe = CreateNamedPipe (szPipeName,
                                    PIPE_ACCESS_OUTBOUND,
                                    PIPE_WAIT | PIPE_READMODE_BYTE | PIPE_TYPE_BYTE,
                                    1,
                                    IO_BUFFERSIZE,
                                    IO_BUFFERSIZE,
                                    0,
                                    &sa);
    if (INVALID_HANDLE_VALUE == hPipe || ERROR_INVALID_PARAMETER == (DWORD)hPipe)
    {
        lResult = HRESULT_FROM_WIN32(GetLastError());
        TraceResult ("PF_GetPFTable: Failed to create pipe", lResult);
    }
    else
    {
        EnterCriticalSection (&g_csIOInfo);
        SetEvent (g_IOInfo.hResumeEvent);
        g_IOInfo.Action           = (fHierTable ? IO_GET_PF_HIERARCHY_TABLE : IO_GET_PF_CONTENTS_TABLE);
        g_IOInfo.hActionCompleted = NULL;
        g_IOInfo.hTmpFile         = hPipe;
		g_IOInfo.dwObjID          = dwFolderID;
        g_IOInfo.pdwData          = NULL;
        g_IOInfo.phLastError      = NULL;
        LeaveCriticalSection (&g_csIOInfo);
    }    
    return lResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_GetNextObjID()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_GetNextObjID (LPSTORAGE pStorage, DWORD * pdwNextID, BOOL fNewFolderID)
{
    LPSTREAM pStream;
    LARGE_INTEGER liZero = { 0 };
    HRESULT hResult = pStorage->OpenStream ((fNewFolderID ? NEXT_FLD_ID_STREAM : NEXT_OBJ_ID_STREAM),
                                            NULL,
                                            OPEN_FLAGS,
                                            0,
                                            &pStream);
    if (!hResult)
    {
        hResult = pStream->Read (pdwNextID, sizeof(DWORD), NULL);
        if (!hResult)
        {
            hResult = pStream->Seek (liZero, STREAM_SEEK_SET, NULL);
            if (!hResult)
            {
                (*pdwNextID)++;
                hResult = pStream->Write (pdwNextID, sizeof(DWORD), NULL);
                (*pdwNextID)--;
            }
        }
        pStream->Release();
    }
    TraceResult ("PF_GetNextObjID", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_CreateRootFolder()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_CreateRootFolder (LPSTORAGE pStorage)
{
    PUBLIC_FOLDER_INFO_A FldProps = { 0 };
    LPSTREAM pStream;
    DWORD dwNextID = 1;
    HRESULT hResult = pStorage->CreateStream (NAMED_PROPS_STREAM, CREATE_FLAGS, 0, 0, &pStream);
    if (!hResult)
    {
        pStream->Release();
        hResult = pStorage->CreateStream (NEXT_FLD_ID_STREAM, CREATE_FLAGS, 0, 0, &pStream);
        if (!hResult)
        {
            hResult = pStream->Write (&dwNextID, sizeof(DWORD), NULL);
            pStream->Release();
            if (!hResult)
            {
                hResult = pStorage->CreateStream (NEXT_OBJ_ID_STREAM, CREATE_FLAGS, 0, 0, &pStream);
                if (!hResult)
                {
                    hResult = pStream->Write (&dwNextID, sizeof(DWORD), NULL);
                    pStream->Release();
                    if (!hResult)
                    {
                        hResult = pStorage->CreateStream (SUB_FLD_STREAM, CREATE_FLAGS, 0, 0, &pStream);
                        if (!hResult)
                        {
                            pStream->Release();
                            GetSystemTimeAsFileTime ((FILETIME *)&FldProps.ftCreateTime);
                            FldProps.ftLastModTime = FldProps.ftCreateTime;
                            lstrcpy ((LPSTR)FldProps.szFldName, "Root Folder");
                            hResult = pStorage->CreateStream (PROPERTIES_STREAM, CREATE_FLAGS, 0, 0, &pStream);
                            if (!hResult)
                            {
                                hResult = pStream->Write (&FldProps, sizeof(PUBLIC_FOLDER_INFO_A), NULL);
                                pStream->Release();
                                if (!hResult)
                                {
                                    lstrcpy ((LPSTR)FldProps.szFldName, "WINDS Public Folders");
		                            hResult = hResult = PF_CreateFolder (pStorage, &FldProps);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    TraceResult ("PF_CreateRootFolder", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_GetNamedPropStream()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_GetNamedPropStream (LPSTORAGE pStorage, HANDLE hPipe)
{
    DWORD dwRead, dwWritten;
    char szBuffer[IO_BUFFERSIZE];
    LPSTREAM pStream;
    HRESULT hResult = pStorage->OpenStream (NAMED_PROPS_STREAM, NULL, OPEN_FLAGS, 0, &pStream);
    if (!hResult)
    {
        do
        {
            hResult = pStream->Read (szBuffer, IO_BUFFERSIZE, &dwRead);
            if (!hResult && dwRead)
            {
                WriteFile (hPipe, szBuffer, dwRead, &dwWritten, NULL);
            }
        } while (!hResult && dwRead);
        pStream->Release();
    }
    TraceResult ("PF_GetNamedPropStream", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_AccessFolderProps()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_AccessFolderProps (LPSTORAGE              pRootStg,
                                     DWORD                  dwFolderID,
                                     PUBLIC_FOLDER_INFO_A * pFldInfo,
                                     BOOL                   fSetProps,
                                     LPSTORAGE              pActualFldStg)
{
    WCHAR wcsName[32];
    LPSTORAGE pFldStg;
    LPSTREAM pStream;
    HRESULT hResult = S_OK;
    if (pActualFldStg)
    {
        pFldStg = pActualFldStg;
        pFldStg->AddRef();
    }
    else
    {
        if (dwFolderID)
        {
            wsprintfW (wcsName, PF_FOLDER, dwFolderID);
            hResult = pRootStg->OpenStorage (wcsName,
                                             NULL,
                                             OPEN_FLAGS,
                                             NULL,
                                             0,
                                             &pFldStg);
        }
        else
        {
            pFldStg = pRootStg;
            pFldStg->AddRef();
        }
    }
    if (!hResult)
    {
        hResult = pFldStg->OpenStream (PROPERTIES_STREAM,
                                       NULL,
                                       OPEN_FLAGS,
                                       0,
                                       &pStream);
        if (!hResult)
        {
            if (fSetProps)
            {
                hResult = pStream->Write (pFldInfo, sizeof(PUBLIC_FOLDER_INFO_A), NULL);
            }
            else
            {
                hResult = pStream->Read (pFldInfo, sizeof(PUBLIC_FOLDER_INFO_A), NULL);
            }
            pStream->Release();
        }
        pFldStg->Release();
    }
    TraceResult ("PF_AccessFolderProps", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_GetPFFolderTable()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_GetPFFolderTable (LPSTORAGE pRootStg,
									DWORD     dwFolderID,
                                    HANDLE    hPipe,
                                    LPMALLOC  pMalloc,
									BOOL      fHierTable)
{
    LPSTORAGE pFldStg = NULL;
    WCHAR wcsName[32];
    HRESULT hResult = S_OK;
    if (dwFolderID)
    {
        wsprintfW (wcsName, PF_FOLDER, dwFolderID);
        hResult = pRootStg->OpenStorage (wcsName,
                                         NULL,
                                         OPEN_FLAGS,
                                         NULL,
                                         0,
                                         &pFldStg);
    }
    else
    {
        pFldStg = pRootStg;
        pFldStg->AddRef();
    }
    // Hierarchy tables
    if (fHierTable && !hResult)
    {
        hResult = PF_EnumSubFolders (pRootStg, pFldStg, hPipe);
    }
    // Contents tables
    if (!fHierTable && !hResult)
    {
        hResult = PF_EnumFldMessages (pFldStg, hPipe, pMalloc);
    }
    if (pFldStg)
    {
        pFldStg->Release();
    }
    TraceResult ("PF_GetPFFolderTable", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_EnumSubFolders()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_EnumSubFolders (LPSTORAGE pRootStg, LPSTORAGE pFldStg, HANDLE hPipe)
{
    LPSTORAGE pSubFldStg;
    LPSTREAM pSubFldStream, pPropsStream;
    DWORD dwRead, dwWritten;
    PUBLIC_FOLDER_INFO_A FldProps;
    CHILD_FOLDER_INFO SubFldInfo;
    HRESULT hResult = pFldStg->OpenStream (SUB_FLD_STREAM,
                                           NULL,
                                           OPEN_FLAGS,
                                           0,
                                           &pSubFldStream);
    if (!hResult)
    {
        do
        {
            hResult = pSubFldStream->Read (&SubFldInfo, sizeof(CHILD_FOLDER_INFO), &dwRead);
            if (!hResult && dwRead)
            {
                hResult = pRootStg->OpenStorage (SubFldInfo.wcsStgName,
                                                 NULL,
                                                 OPEN_FLAGS,
                                                 NULL,
                                                 0,
                                                 &pSubFldStg);
                if (!hResult)
                {
                    hResult = pSubFldStg->OpenStream (PROPERTIES_STREAM,
                                                      NULL,
                                                      OPEN_FLAGS,
                                                      0,
                                                      &pPropsStream);
                    if (!hResult)
                    {
                        hResult = pPropsStream->Read (&FldProps, sizeof(PUBLIC_FOLDER_INFO_A), NULL);
                        if (!hResult)
                        {
                            WriteFile (hPipe, &FldProps, sizeof(PUBLIC_FOLDER_INFO_A), &dwWritten, NULL);
                        }
                        pPropsStream->Release();
                    }
                    pSubFldStg->Release();
                }
                TraceResult ("PF_EnumSubFolders: Opening a folder sub-storage", hResult);
                hResult = S_OK;
            }
        } while (!hResult && dwRead);
        pSubFldStream->Release();
    }
    TraceResult ("PF_EnumSubFolders", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_EnumFldMessages()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_EnumFldMessages (LPSTORAGE pFldStg,
                                   HANDLE    hPipe,
                                   LPMALLOC  pMalloc)
{
    LPSTORAGE pSubStg;
    LPSTREAM pStream;
    LPENUMSTATSTG pEnum;
    STATSTG aStat[ELEMENTS_TO_FETCH], statProps;
    DWORD dwWritten, dwLastAlloc = 0;
    LPBYTE pBuffer = NULL;
    ULONG ulFound, i;
    HRESULT hResult = pFldStg->EnumElements (0, NULL, 0, &pEnum);
    TraceResult ("PF_EnumFldMessages: Failed to get ENUM interface", hResult);
    if (!hResult)
    {
        do
        {
            hResult = pEnum->Next (ELEMENTS_TO_FETCH, aStat, &ulFound);
            if (FAILED(hResult)) // We might get S_FALSE
            {
                TraceResult ("PF_EnumFldMessages: Failed to ENUM streams", hResult);
            }
            else
            {
                for (i=0; i<ulFound; i++)
                {
                    if (STGTY_STORAGE == aStat[i].type && (L'M' == aStat[i].pwcsName[0]))
                    {
                        if (!pFldStg->OpenStorage (aStat[i].pwcsName,
                                                   NULL,
                                                   OPEN_FLAGS,
                                                   NULL,
                                                   0,
                                                   &pSubStg))
                        {
                            if (!pSubStg->OpenStream (PROPERTIES_STREAM,
                                                      NULL,
                                                      OPEN_FLAGS,
                                                      0,
                                                      &pStream))
                            {
                                if (!pStream->Stat (&statProps, STATFLAG_NONAME))
                                {
                                    WriteFile (hPipe,
                                               &statProps.cbSize.LowPart,
                                               sizeof(DWORD),
                                               &dwWritten,
                                               NULL);
                                    if (statProps.cbSize.LowPart > dwLastAlloc)
                                    {
                                        delete [] pBuffer;
                                        pBuffer = new BYTE[statProps.cbSize.LowPart];
                                        if (pBuffer)
                                        {
                                            dwLastAlloc = statProps.cbSize.LowPart;
                                        }
                                    }
                                    if (pBuffer)
                                    {
                                        if (!pStream->Read (pBuffer, statProps.cbSize.LowPart, NULL))
                                        {
                                            WriteFile (hPipe, pBuffer, statProps.cbSize.LowPart, &dwWritten, NULL);
                                        }
                                    }
                                }
                                pStream->Release();
                            }
                            pSubStg->Release();
                        }
                    }
                    pMalloc->Free (aStat[i].pwcsName);
                }
            }
        } while (!hResult && ulFound);
        pEnum->Release();
    }
    if (S_FALSE == hResult)
    {
        hResult = S_OK;
    }
    delete [] pBuffer;
    TraceResult ("PF_EnumFldMessages", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_SearchForNameCollision()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_SearchForNameCollision (LPSTORAGE pFldStg, PUBLIC_FOLDER_INFO_A NewFldInfo)
{
    LPSTREAM pStream;
    DWORD dwRead;
    CHILD_FOLDER_INFO SubFldInfo;
    HRESULT hResult = pFldStg->OpenStream (SUB_FLD_STREAM,
                                           NULL,
                                           OPEN_FLAGS,
                                           0,
                                           &pStream);
    if (!hResult)
    {
        do
        {
            hResult = pStream->Read (&SubFldInfo, sizeof(CHILD_FOLDER_INFO), &dwRead);
            if (!hResult && dwRead)
            {
                // Check for folder name collision
                if (SubFldInfo.fUsed && !lstrcmpi ((LPSTR)SubFldInfo.szFolderName, (LPSTR)NewFldInfo.szFldName))
                {
                    hResult = hResult = HRESULT_FROM_WIN32(ERROR_ALREADY_EXISTS);
                }
            }
        } while (!hResult && dwRead);
        pStream->Release();
    }
    TraceResult ("PF_SearchForNameCollision", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_AddNewFolderName()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_AddNewFolderName (LPSTORAGE pFldStg, PUBLIC_FOLDER_INFO_A NewFldInfo)
{
    LPSTREAM pStream;
    DWORD dwRead;
    LARGE_INTEGER liSize = { 0 };
    CHILD_FOLDER_INFO SubFldInfo;
    HRESULT hResult = pFldStg->OpenStream (SUB_FLD_STREAM,
                                           NULL,
                                           OPEN_FLAGS,
                                           0,
                                           &pStream);
    if (!hResult)
    {
        do
        {
            hResult = pStream->Read (&SubFldInfo, sizeof(CHILD_FOLDER_INFO), &dwRead);
            if (!hResult && dwRead)
            {
                if (!SubFldInfo.fUsed)
                {
                    hResult = pStream->Seek (liSize, STREAM_SEEK_SET, NULL);
                    dwRead = 0; // To get out of the DO-WHILE() loop
                }
                liSize.LowPart += dwRead;
            }
        } while (!hResult && dwRead);
        if (!hResult)
        {
            ZeroMemory (&SubFldInfo, sizeof (CHILD_FOLDER_INFO));
            SubFldInfo.fUsed = TRUE;
            SubFldInfo.dwFolderID = NewFldInfo.dwObjID;
            lstrcpy ((LPSTR)SubFldInfo.szFolderName, (LPSTR)NewFldInfo.szFldName);
            wsprintfW (SubFldInfo.wcsStgName, PF_FOLDER, NewFldInfo.dwObjID);
            hResult = pStream->Write (&SubFldInfo, sizeof(CHILD_FOLDER_INFO), NULL);
        }
        pStream->Release();
    }
    TraceResult ("PF_AddNewFolderName", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_RemoveFolderName()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_RemoveFolderName (LPSTORAGE pFldStg, DWORD dwFolderID)
{
    LPSTREAM pStream;
    DWORD dwRead;
    LARGE_INTEGER liSize = { 0 };
    CHILD_FOLDER_INFO SubFldInfo;
    HRESULT hResult = pFldStg->OpenStream (SUB_FLD_STREAM,
                                           NULL,
                                           OPEN_FLAGS,
                                           0,
                                           &pStream);
    if (!hResult)
    {
        do
        {
            hResult = pStream->Read (&SubFldInfo, sizeof(CHILD_FOLDER_INFO), &dwRead);
            if (!hResult && dwRead)
            {
                if (SubFldInfo.fUsed && (SubFldInfo.dwFolderID == dwFolderID))
                {
                    ZeroMemory (&SubFldInfo, sizeof (CHILD_FOLDER_INFO));
                    hResult = pStream->Seek (liSize, STREAM_SEEK_SET, NULL);
                    if (!hResult)
                    {
                        hResult = pStream->Write (&SubFldInfo, sizeof(CHILD_FOLDER_INFO), NULL);
                    }
                    dwRead = 0; // To get out of the DO-WHILE() loop
                }
                liSize.LowPart += dwRead;
            }
        } while (!hResult && dwRead);
        pStream->Release();
    }
    TraceResult ("PF_RemoveFolderName", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_CreateFolder()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_CreateFolder (LPSTORAGE              pRootStg,
                                PUBLIC_FOLDER_INFO_A * pFldInfo)
{
    LPSTREAM pStream;
    PUBLIC_FOLDER_INFO_A Info = { 0 };
    LPSTORAGE pFldStg, pNewFldStg;
    WCHAR wcsName[32];
    wsprintfW (wcsName, PF_FOLDER, pFldInfo->dwParentID);
    HRESULT hResult = S_OK;
    if (pFldInfo->dwParentID)
    {
        hResult = pRootStg->OpenStorage (wcsName,
                                         NULL,
                                         OPEN_FLAGS,
                                         NULL,
                                         0,
                                         &pFldStg);
    }
    else
    {
        pFldStg = pRootStg;
        pFldStg->AddRef();
    }
    if (!hResult)
    {
        hResult = PF_SearchForNameCollision (pFldStg, *pFldInfo);
        if (!hResult)
        {
            hResult = PF_GetNextObjID (pRootStg, &(pFldInfo->dwObjID), TRUE);
            if (!hResult)
            {
                wsprintfW (wcsName, PF_FOLDER, pFldInfo->dwObjID);
                hResult = pRootStg->CreateStorage (wcsName, CREATE_FLAGS, 0, 0, &pNewFldStg);
                if (!hResult)
                {
                    hResult = pNewFldStg->CreateStream (PROPERTIES_STREAM, CREATE_FLAGS, 0, 0, &pStream);
                    if (!hResult)
                    {
                        hResult = pStream->Write (pFldInfo, sizeof(PUBLIC_FOLDER_INFO_A), NULL);
                        pStream->Release();
                        if (!hResult)
                        {
                            hResult = pNewFldStg->CreateStream (SUB_FLD_STREAM, CREATE_FLAGS, 0, 0, &pStream);
                            if (!hResult)
                            {
                                pStream->Release();
                                hResult = PF_AccessFolderProps (pRootStg,
                                                                pFldInfo->dwParentID,
                                                                &Info,
                                                                FALSE,
                                                                pFldStg);
                                if (!hResult)
                                {
                                    if (FALSE == Info.fHasSubFolders)
                                    {
                                        Info.fHasSubFolders = TRUE;
                                        hResult = PF_AccessFolderProps (pRootStg,
                                                                        pFldInfo->dwParentID,
                                                                        &Info,
                                                                        TRUE, 
                                                                        pFldStg);
                                    }
                                }
                            }
                            if (!hResult)
                            {
                                hResult = PF_AddNewFolderName (pFldStg, *pFldInfo);
                            }
                            // TODO: Broadcast notification
                        }
                    }
                    pNewFldStg->Release();
                }
                if (hResult)
                {
                    pFldStg->DestroyElement (wcsName);
                }
            }
        }
        pFldStg->Release();
    }
    TraceResult ("PF_CreateFolder", hResult);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_DeleteFolder()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_DeleteFolder (LPSTORAGE pRootStg, DWORD dwFolderID, BOOL fRecursing)
{
    BOOL fPartialSuccess = FALSE;
    LPSTREAM pStream;
    LPSTORAGE pFldStg;
    DWORD dwRead, dwParentID;
    CHILD_FOLDER_INFO SubFldInfo;
    WCHAR wcsName[32];
    wsprintfW (wcsName, PF_FOLDER, dwFolderID);
    HRESULT hResult = pRootStg->OpenStorage (wcsName,
                                             NULL,
                                             OPEN_FLAGS,
                                             NULL,
                                             0,
                                             &pFldStg);
    if (!hResult)
    {
        hResult = pFldStg->OpenStream (SUB_FLD_STREAM,
                                       NULL,
                                       OPEN_FLAGS,
                                       0,
                                       &pStream);
        if (!hResult)
        {
            do
            {
                dwRead= 0;
                hResult = pStream->Read (&SubFldInfo, sizeof(CHILD_FOLDER_INFO), &dwRead);
                if (!hResult && dwRead)
                {
                    if (SubFldInfo.fUsed)
                    {
                        hResult = PF_DeleteFolder (pRootStg, SubFldInfo.dwFolderID, TRUE);
                    }
                }
                if (hResult)
                {
                    fPartialSuccess = TRUE;
                }
            } while (dwRead);
            pStream->Release();
        }
        hResult = S_OK;
        if (!fRecursing)
        {
            PUBLIC_FOLDER_INFO_A Props = { 0 };
            PF_AccessFolderProps (pRootStg, dwFolderID, &Props, FALSE, pFldStg);
            dwParentID = Props.dwParentID;
        }
        pFldStg->AddRef();
        hResult = pRootStg->DestroyElement (wcsName);
        if (!hResult)
        {
            if (!fRecursing)
            {
                wsprintfW (wcsName, PF_FOLDER, dwParentID);
                if (!pRootStg->OpenStorage (wcsName,
                                            NULL,
                                            OPEN_FLAGS,
                                            NULL,
                                            0,
                                            &pFldStg))
                {
                    PF_RemoveFolderName (pFldStg, dwFolderID);
                    pFldStg->Release();
                }
            }
            // TODO: Broadcast notification
        }
    }
    TraceResult ("PF_DeleteFolder", hResult);
    if (fPartialSuccess)
    {
        hResult = S_FALSE;
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    PF_ModifyFolderProps()
//
//    Parameters
//
//    Purpose
//
//    Return Value
//
HRESULT WINAPI PF_ModifyFolderProps (LPSTORAGE            pRootStg,
                                     DWORD                dwFolderID,
                                     PUBLIC_FOLDER_INFO_A NewProps)
{
    BOOL fSaveProps = TRUE;
    PUBLIC_FOLDER_INFO_A Info;
    WCHAR wcsName[32];
    wsprintfW (wcsName, PF_FOLDER, dwFolderID);
    LPSTORAGE pFldStg;
    HRESULT hResult = S_OK;
    if (dwFolderID)
    {
        hResult = pRootStg->OpenStorage (wcsName,
                                         NULL,
                                         OPEN_FLAGS,
                                         NULL,
                                         0,
                                         &pFldStg);
    }
    else
    {
        pFldStg = pRootStg;
        pFldStg->AddRef();
    }
    if (!hResult)
    {
        hResult = PF_AccessFolderProps (pRootStg, dwFolderID, &Info, FALSE, pFldStg);
        if (!hResult)
        {
            if (!lstrcmpi ((LPSTR)Info.szFldName, (LPSTR)NewProps.szFldName))
            {
                hResult = PF_SearchForNameCollision (pFldStg, NewProps);
                if (!hResult)
                {
                    lstrcpy ((LPSTR)Info.szFldName, (LPSTR)NewProps.szFldName);
                    fSaveProps = TRUE;
                    PF_RemoveFolderName (pFldStg, dwFolderID);
                    PF_AddNewFolderName (pFldStg, Info);
                }
            }
            if (!lstrcmpi ((LPSTR)Info.szFldComment, (LPSTR)NewProps.szFldComment))
            {
                lstrcpy ((LPSTR)Info.szFldComment, (LPSTR)NewProps.szFldComment);
                fSaveProps = TRUE;
            }
            if (!hResult && fSaveProps)
            {
                hResult = PF_AccessFolderProps (pRootStg, dwFolderID, &Info, TRUE, pFldStg);
                if (!hResult)
                {
                    // TODO: Broadcast notification
                }
            }
        }
        pFldStg->Release();
    }
    TraceResult ("PF_ModifyFolderProps", hResult);
    return hResult;
}

// End of file for RMSRPC.CPP
