///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      SETUP.CPP
//
//  Description
//      The purpose of this sample is to demonstrate one way of installing
//      a new service provider and register it with MAPI. This sample installs
//      one or more service providers and optionally adds them to the default 
//      user profile. This sample requires a one provider inf file for each
//      provider that is being installed. This file is merged with the 
//      MAPISVC.INF file. In addtion, the setup sample requires a SETUP.INF
//      that provides additional information about the providers. The format
//      of SETUP.INF is similar to MAPISVC.INF. Please read the sample
//      SETUP.INF and README.WRI for more details.
//      This sample displays a dialog box providing the user with the choice
//      of service providers to install. After a choice is made, the sample
//      creates installation directories, copies service provider files,
//      merges INF files, installs service providers and optionally adds the
//      services to the default profile and configures them.
//
//  Authors
//      Nishad Mulye
//      Irving De la Cruz
//
//  Revision: 1.00
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1995 Microsoft Corporation. All rights reserved.
//
#include "SETUP.H"

const unsigned  uPBRange = 100;         // range for Progress Bar Ctrl
unsigned        uPBStep  = 20;          // step size for Progress Bar Ctrl
const unsigned  uTimerInterval = 750;

unsigned	g_uPBCount = 0;
HANDLE		g_hThread = NULL;           // install thread handle
DWORD	  	g_dwThreadId = 0;           // install thread Id
HWND        g_hWndSetup = NULL;         // setup Window handle (main dialog window)
BOOL        g_bAddToProfile = FALSE;    // flag indicating user selection
unsigned	g_nMaxServices = 0;         // max number of services to install
unsigned	g_nSelServices = 0;         // number of selected user services
                                        // to install
unsigned	*g_lpSelServices = 0;       // array of indices of user selected
                                        // services

#define MAINDLGCLASS    TEXT("PssMapiSamplesSetup")

const TCHAR *lptszMsg[] =               // messages from the installer thread to the
                                        // main dialog window
{
    TEXT("Creating directory ..."),
	TEXT("Copying files ..."),
	TEXT("Merging files ..."),
	TEXT("Adding service(s) ..."),
	TEXT("Configuring service(s) ..."),
};

///////////////////////////////////////////////////////////////////////////////
//    PrivateDlgClassProc()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      
LRESULT CALLBACK PrivateDlgClassProc (HWND    hDlg, 
                                      UINT    message, 
                                      WPARAM  wParam, 
                                      LPARAM  lParam)
{
    return DefDlgProc (hDlg, message, wParam, lParam);
}

///////////////////////////////////////////////////////////////////////////////
//    WinMain()
//
//    Parameters
//      { Refer to Win32 SDK documentation }
//      
//    Purpose
//      This is the entry function. The application will start executing here.
//      
//    Return Value
//      Return code of the application to the system.
//      
int WINAPI WinMain (HINSTANCE   hInstance,
                    HINSTANCE   hPrevInstance,
                    LPSTR       lpszCmdLine,
                    int         nCmdShow)
{
    // Check for other instances of this application running. If one is found,
    // bring it to the foreground
    HWND hOtherInstance = FindWindow (MAINDLGCLASS, NULL);
    if (NULL != hOtherInstance)
    {
        HWND hChildWnd = GetLastActivePopup (hOtherInstance);
        if (hChildWnd != hOtherInstance)
        {
            BringWindowToTop (hOtherInstance);
        }
        else
        {
            SetForegroundWindow (hOtherInstance);
        }
        return 0;
    }
    WNDCLASS wc = { 0 };
    // Register the class for this application
    wc.hInstance     = hInstance;
    wc.hCursor       = LoadCursor (NULL, IDC_ARROW);
    wc.lpfnWndProc   = PrivateDlgClassProc;
    wc.cbWndExtra    = DLGWINDOWEXTRA;
    wc.hIcon         = LoadIcon (hInstance, MAKEINTRESOURCE (IDI_SETUP));
    wc.lpszClassName = MAINDLGCLASS;
    RegisterClass (&wc);
  
	g_hInstance = hInstance;
	Ctl3dRegister(g_hInstance);
	Ctl3dAutoSubclass(g_hInstance);

	InitCommonControls();
	
	int nDlgErr = DialogBox (g_hInstance, 
                             MAKEINTRESOURCE(IDD_SETUP_DIALOG), 
                             0, 
                             SetupDialogProcedure);

	TCHAR* pzMsg = 0;

	switch (nDlgErr)
	{
		case IDOK:
			pzMsg = TEXT("Setup is complete!");
			break;
		case IDC_E_COPY_FILE:
			pzMsg = TEXT("Setup could not copy file(s)!");
			break;
		case IDC_E_CREATE_DIR:
			pzMsg = TEXT("Setup could not create directory(s)!");
			break;
		case IDC_E_CREATE_SERVICE:
			pzMsg = TEXT("Setup could not create service(s)!");
			break;
		case IDC_E_MERGE_INF:
			pzMsg = TEXT("Setup could not merge INF file(s)!");
			break;
		case -1:
			pzMsg = TEXT("Setup was unable to display the user interface, please try again!");
			break;
		case IDCANCEL:
		default:
			pzMsg = TEXT("Setup encountered a problem, could not complete!");
			break;
	}
	if (pzMsg)
	{
		MessageBox (NULL,
		            pzMsg,
		            TEXT("Setup"),
		            MB_APPLMODAL | MB_OK | MB_ICONEXCLAMATION);
	}
	Ctl3dUnregister(g_hInstance);
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//    SetupDialogProcedure()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      TRUE if message was handled, FALSE if we don't handle the message
//      
BOOL CALLBACK SetupDialogProcedure (HWND    hWnd, 
                                    UINT    message, 
                                    WPARAM  wParam, 
                                    LPARAM  lParam)
{
	BOOL bRet = FALSE;  
	switch (message)
	{
		case WM_INITDIALOG:
		{
            g_hWndSetup = hWnd;
			CenterWindow(hWnd);
            // fill up the list boxes with name of service providers to install
			if (PopulateListBox(hWnd, IDC_INSTALL_LB) == 0)
			{
				MessageBox(hWnd, TEXT("Error in SETUP.INF!"), TEXT("Setup"), MB_ICONSTOP | MB_OK);
				EndDialog(hWnd, IDCANCEL);
			}
			break;
		}
		case WM_COMMAND:
			bRet = SetupDlgCommandHandler(hWnd, message, wParam, lParam);
			break;
        case WM_SYSCOLORCHANGE:
            Ctl3dColorChange();
            break;
		case WM_DESTROY:
			break;
	}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    CenterWindow()
//
//    Parameters
//      handle of the window to center
//
//    Purpose
//      to center a windowrelative to the screen dimensions
//      
//    Return Value
//      None
//      
void WINAPI CenterWindow (HWND hWnd)
{
	unsigned uXExt = GetSystemMetrics(SM_CXSCREEN);
	unsigned uYExt = GetSystemMetrics(SM_CYSCREEN);
	RECT rect;
	GetWindowRect(hWnd, &rect);
	uXExt -= (rect.right - rect.left);
	uYExt -= (rect.bottom - rect.top);
	SetWindowPos(hWnd, 0, uXExt / 2, uYExt /2, 0, 0, SWP_NOSIZE);
}

///////////////////////////////////////////////////////////////////////////////
//    PopulateListBox()
//
//    Parameters
//      handle of the setup(main) dialog window, resourceId of the listbox
//
//    Purpose
//      read the SETUP.INF and display the names of service providers
//      that can be installed
//      
//    Return Value
//      
int WINAPI PopulateListBox (HWND hWnd, int nDlgItem)
{
    PSECTION    lpSection;      // the section of SETUP.INF to read
    unsigned    uSeed = 0;      // seed used to read section entries
	TCHAR       *lptszRHS = 0,  // value of the variable in the section
	            *lptszLHS = 0;  // variable in the section
	int         nRet = 1;       // return value of this routine

	LPTSTR lptszSetupInf = GetSetupInfFileName();
    if (lptszSetupInf)
	{
        // open the Services section of the SETUP.INF
		OpenSection (lptszSetupInf, TEXT("Services"), &lpSection);
		delete [] lptszSetupInf;
        // read every variable in the section which is of the form,
        // Service Name=Display Name
        while (ReadSection (lpSection, &uSeed, &lptszLHS, &lptszRHS))
		{
			SendDlgItemMessage(hWnd, nDlgItem, LB_ADDSTRING, 0, (LPARAM)lptszRHS);
			delete [] lptszLHS;
			delete [] lptszRHS;
			g_nMaxServices++;
		}
		CloseSection (&lpSection);
		if (g_nMaxServices > 0)
		{
			SendDlgItemMessage (hWnd,
                                nDlgItem,
                                LB_SELITEMRANGE,
                                TRUE,
                                MAKELPARAM(0, g_nMaxServices-1));
		}
		SetFocus(hWnd);
		nRet = g_nMaxServices;
	}
	return nRet;
}

///////////////////////////////////////////////////////////////////////////////
//    SetupDlgCommandHandler()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      handle WM_COMMAND messages
//      
//    Return Value
//      TRUE if message was handled, FALSE if we don't handle the message
//      
BOOL WINAPI SetupDlgCommandHandler(HWND hWnd, UINT uMsgID, WPARAM wParam, LPARAM lParam)
{
	WORD wID = LOWORD(wParam);
	BOOL bRet = TRUE;

	switch (wID)
	{
		case IDCANCEL:
            // Exit pushbutton clicked
			if (MessageBox (hWnd,
			                TEXT("Do you really want to exit?"),
			                TEXT("Setup"), 
				            MB_ICONSTOP | MB_YESNO) == IDYES)
			{
				EndDialog(hWnd, IDCANCEL);
			}
			break;
		case IDC_INSTALL:
			{
				// install button clicked
                // get the number of selections user made
				g_nSelServices = SendDlgItemMessage(hWnd, IDC_INSTALL_LB, LB_GETSELCOUNT, 0, 0);
				if (g_nSelServices > 0)
				{
                    // allocate memory for the user selection indices
					g_lpSelServices = new unsigned[g_nSelServices];
					if (g_lpSelServices)
					{
                        // get the user selection indices
						if (SendDlgItemMessage(	hWnd, 
												IDC_INSTALL_LB, 
												LB_GETSELITEMS,
												g_nSelServices, 
												(LPARAM)g_lpSelServices) != -1)
						{
                            // get the state of the Add To Profile check box
			                g_bAddToProfile = (SendDlgItemMessage(  hWnd,
                                               IDC_ADD_XB,
                                               BM_GETSTATE,
                                               0,
                                               0) & 0x0003) == 0x0001 ? TRUE : FALSE;
                            // if services are added to the profile there are
                            // 4 steps involved, otherwise there are 3 steps
                            if (g_bAddToProfile)
                            {
                                uPBStep = uPBRange / 4;
                            }
                            else
                            {
                                uPBStep = uPBRange / 3;
                            }
							// disable this window
							ShowWindow(hWnd, SW_HIDE);
                            // display the installation progress dialog
							int nDlgErr = DialogBox (g_hInstance,
							                         MAKEINTRESOURCE(IDD_PROGRESS_DLG),
							                         0,
  								                     ProgressDialogProcedure);
							EndDialog(hWnd, nDlgErr);
						}
					}
				}
			}
			break;
		default:
			// because we did not handle the message
			bRet = FALSE;
			break;
	}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    ProgressDialogProcedure()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      TRUE if message was handled, FALSE if we don't handle the message
//      
BOOL CALLBACK ProgressDialogProcedure(HWND hWnd, UINT uMsgID,WPARAM wParam, LPARAM lParam)
{
	BOOL bRet = FALSE;
	switch (uMsgID)
	{
		case WM_COMMAND:
		{
			WORD wID = LOWORD(wParam);
			switch (wID)
			{
				case IDCANCEL:
                    // user is trying to abort the installation;
                    // suspend the installation thread until we can confirm.
					SuspendThread(g_hThread);
					if (MessageBox(hWnd, TEXT("Do you really want to exit?"), TEXT("Setup"), 
						MB_ICONSTOP | MB_YESNO) == IDYES)
					{
						EndDialog(hWnd, IDCANCEL);
					}
					else
					{
						ResumeThread(g_hThread);
					}
					bRet = TRUE;
					break;
				case IDC_THREAD:
					// handle messages from the thread
					bRet = ThreadMsgCommandHandler(hWnd, uMsgID, wParam, lParam);
					break;
				default:
					break;
			}
			break;
		}
		case WM_INITDIALOG:
			// initialize the progress bar
			SendDlgItemMessage(hWnd, IDC_PROGRESS, PBM_SETRANGE, uPBRange, 0);
			SendDlgItemMessage(hWnd, IDC_PROGRESS, PBM_SETSTEP, uPBStep, 0);
			CenterWindow(hWnd);
            // create the installation thread
			g_hThread = CreateThread(0, 0, SetupThreadRoutine, hWnd, 0, &g_dwThreadId);
			if (g_hThread == 0)
			{
				MessageBox(hWnd, TEXT("Could not create thread!"), TEXT("Setup"), MB_ICONSTOP | MB_OK);
				EndDialog(hWnd, IDCANCEL);
			}
			break;
		case WM_DESTROY:
			if (g_hThread != 0)
			{
				TerminateThread(g_hThread, 0);
			}
			break;
		default:
			break;
	}
	return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    ThreadMsgCommandHandler()
//
//    Parameters
//      { Refer to Win32 API documentation on dialog procedures }
//
//    Purpose
//      
//    Return Value
//      TRUE if message was handled, FALSE if we don't handle the message
//      
BOOL WINAPI ThreadMsgCommandHandler(HWND hWnd, UINT uMsgID,WPARAM wParam, LPARAM lParam)
{
	BOOL bRet = TRUE;
	switch (HIWORD(wParam))
	{
		// advance the progress bar
		case TM_STEPIT:
			SendDlgItemMessage(hWnd, IDC_PROGRESS, PBM_STEPIT, 0, 0);
			break;
		// thread completed successfully
		case TM_DONE:
			g_hThread = 0;
			EndDialog(hWnd, lParam);
			break;
		// display thread message in the status field
		case TM_DISPLAY:
			SetWindowText(GetDlgItem(hWnd, IDC_STATUS), (LPCTSTR)lParam);
			break;
		default:
			bRet = FALSE;
			break;
	}
	return bRet;
}

//////////////////////////////////////////////////////////////////////////////
//    ParseString()
//
//    Parameters
//      lpstzSrc,   the string to parse
//      lpuSeed,    the seed initialized to 0 for the first call
//      delimiter,  delimiting character
//      lplptszDest, parsed token
//
//    Purpose
//      to parse a string of the form "this;is;Apple" into tokens,
//      "this", "is", "Apple"
//      
//    Return Value
//      TRUE if successful, FALSE   if failure
//      parsed token, garbage collection is the responsibility of
//      the caller.
//      
BOOL WINAPI ParseString (LPTSTR lpstzSrc, unsigned *lpuSeed, TCHAR delimiter, LPTSTR* lplptszDest)
{
	unsigned uStrLen = lstrlen(lpstzSrc);
	for (unsigned i = *lpuSeed, j = i;i < uStrLen;i++)
	{
		if (lpstzSrc[i]	== delimiter)
		{
			*lplptszDest = new TCHAR[i-j+1];
			if (*lplptszDest)
			{
				ZeroMemory(*lplptszDest, sizeof(TCHAR) * (i-j+1));
				if (*lplptszDest)
				{
					CopyMemory (*lplptszDest, &lpstzSrc[j], sizeof(TCHAR) * (i-j));
					*lpuSeed = i + 1;
					return TRUE;
				}
			}
		}
	}
	if (i != j)
	{
		*lplptszDest = new TCHAR[i-j+1];
		if (*lplptszDest)
		{
			ZeroMemory(*lplptszDest, sizeof(TCHAR) * (i-j+1));
			if (*lplptszDest)
			{
				CopyMemory (*lplptszDest, &lpstzSrc[j], sizeof(TCHAR) * (i-j));
				*lpuSeed = i + 1;
				return TRUE;
			}
		}
	}
	*lplptszDest = 0;
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//    OpenSection()
//
//    Parameters
//      lptszFilename,  file name in which the section is located
//      lptszSection,   section name
//      ppSection,      handle of the section that was opened
//
//    Purpose
//      open a profile section for reading
//      
//    Return Value
//      handle of the section
//      
void WINAPI OpenSection (LPTSTR lptszFilename, LPTSTR lptszSection, PSECTION* ppSection)
{
	unsigned long ulnBytes = 2048, ulnActualBytes = 0;
	*ppSection = new SECTION;
	if (*ppSection)
	{
		(*ppSection)->lptszData = new TCHAR[ulnBytes];
		if ((*ppSection)->lptszData)
		{
			while ((ulnActualBytes = GetPrivateProfileSection (lptszSection,
                                                 (*ppSection)->lptszData, 
	                                               ulnBytes,
	                                               lptszFilename)) == (ulnBytes - 2))
			{
				// if the buffer was insufficient, delete it, allocate bigger buffer
				// and try again
				delete [] (*ppSection)->lptszData;
				ulnBytes *= 2;
				(*ppSection)->lptszData = new TCHAR[ulnBytes];
				if (!(*ppSection))
				{
					break;
				}
			}
			if ((*ppSection)->lptszData)
			{
				(*ppSection)->ulSize = ulnActualBytes;
			}
		}
		else
		{
			delete (*ppSection);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
//    ReadSection()
//
//    Parameters
//      pSection,   section handle
//      seed,       seed
//      lptszLHS,   variable name
//      lptszRHS,   variable value
//
//    Purpose
//      read a single variable and its value
//      
//    Return Value
//      TRUE if message was handled, FALSE if we don't handle the message
//      
BOOL WINAPI ReadSection (PSECTION pSection, unsigned* seed, LPTSTR* lptszLHS, LPTSTR* lptszRHS)
{
	unsigned i = *seed, j = *seed;
	for (;i < pSection->ulSize;i++)
	{
		if (pSection->lptszData[i] == TCHAR('='))
		{
			// service name
			*lptszLHS = new TCHAR[i - j + 1];
			if (*lptszLHS)
			{
				CopyMemory (*lptszLHS, &pSection->lptszData[j], (i - j) * sizeof(TCHAR)); 
				(*lptszLHS)[i - j] = 0;
			}
			j = i + 1;
		}
		else
		{
		    if (pSection->lptszData[i] == 0)
    		{
    			// display name
    			*lptszRHS = new TCHAR[i - j + 1];
    			if (*lptszRHS)
    			{
    				CopyMemory (*lptszRHS, &pSection->lptszData[j], (i - j) * sizeof(TCHAR)); 
    				(*lptszRHS)[i - j] = 0;
    			}
    			*seed = i + 1;
    			return TRUE;
    		}
    }
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
//    CloseSection()
//
//    Parameters
//      ppSection,  section that was previously opened
//
//    Purpose
//      close a section that was previously opened
//      
//    Return Value
//      
void WINAPI CloseSection (PSECTION* ppSection)
{
	PSECTION lpSection = *ppSection;
	delete [] lpSection->lptszData;
	delete lpSection;
	*ppSection = NULL;
}

///////////////////////////////////////////////////////////////////////////////
//    GetSetupInfDir()
//
//    Parameters
//
//    Purpose
//      get the directory in which SETUP.INF is located
//      
//    Return Value
//      fully qualified path to the directory in which SETUP.INF is located.
//      includes the lagging '\'. garbage collection is the callers responsibility
//      
LPTSTR WINAPI GetSetupInfDir()
{
	TCHAR *pzSetup = 0;
	TCHAR pzFilename[_MAX_PATH];
	unsigned uPathSize = GetModuleFileName(g_hInstance, pzFilename, _MAX_PATH);
	TCHAR* pzChar = strrchr(pzFilename, TCHAR('\\'));
	if (pzChar)
	{
		pzChar++;
		*pzChar = 0;
		pzSetup = new TCHAR[lstrlen(pzFilename)+1];
		if (pzSetup)
		{
			lstrcpy(pzSetup, pzFilename);
		}
	}
	return pzSetup;
}

///////////////////////////////////////////////////////////////////////////////
//    GetSetupInfFileName()
//
//    Parameters
//      get the full path and the file name of SETUP.INF e.g. c:\Windows\SETUP.INF
//
//    Purpose
//      
//    Return Value
//      fully qualified filename of  SETUP.INF. garbage collection is the 
//      callers responsibility
//      
LPTSTR WINAPI GetSetupInfFileName()
{
	TCHAR *pzSetup = 0;
	TCHAR lptcszFile[] = TEXT("SETUP.INF");
	TCHAR pzFilename[_MAX_PATH];
	unsigned uPathSize = GetModuleFileName(g_hInstance, pzFilename, _MAX_PATH);
	TCHAR* pzChar = strrchr(pzFilename, TCHAR('\\'));
	if (pzChar)
	{
		pzChar++;
		*pzChar = 0;
		pzSetup = new TCHAR[lstrlen(pzFilename)+1+sizeof(lptcszFile)];
		if (pzSetup)
		{
			lstrcpy(pzSetup, pzFilename);
			lstrcat(pzSetup, lptcszFile);
		}
	}
	return pzSetup;
}

///////////////////////////////////////////////////////////////////////////////
//    GetMapiInfFileName()
//
//    Parameters
//
//    Purpose
//      get the full path and the file name of MAPISVC.INF e.g. c:\Windows\MAPISVC.INF
//      
//    Return Value
//      fully qualified filename of  MAPISVC.INF. garbage collection is the 
//      callers responsibility
//      
LPTSTR WINAPI GetMapiInfFileName()
{
	TCHAR lptcszPath[] = TEXT("\\SYSTEM32\\MAPISVC.INF");
	TCHAR lptcszEnv[] = TEXT("SystemRoot");
	DWORD dwSystemRoot = GetEnvironmentVariable(lptcszEnv, 0, 0) + sizeof(lptcszPath) + 2;
	TCHAR *lptszSystemRoot = new TCHAR[dwSystemRoot];
  if (lptszSystemRoot)
	{
    GetEnvironmentVariable (lptcszEnv, lptszSystemRoot, dwSystemRoot);
	  lstrcat(lptszSystemRoot, lptcszPath);
  }
	return lptszSystemRoot;
}

///////////////////////////////////////////////////////////////////////////////
//    GetMapiInfDir()
//
//    Parameters
//      get the directory in which MAPISVC.INF is located
//
//    Purpose
//      
//    Return Value
//      fully qualified path to the directory in which MAPISVC.INF is located.
//      includes the lagging '\'. garbage collection is the callers responsibility
//      
LPTSTR WINAPI GetMapiInfDir()
{
	TCHAR lptcszPath[] = TEXT("\\SYSTEM32\\");
	TCHAR lptcszEnv[] = TEXT("SystemRoot");
	DWORD dwSystemRoot = GetEnvironmentVariable(lptcszEnv, 0, 0) + sizeof(lptcszPath) + 2;
	TCHAR *lptszSystemRoot = new TCHAR[dwSystemRoot];
    if (lptszSystemRoot)
	{
        GetEnvironmentVariable (lptcszEnv, lptszSystemRoot, dwSystemRoot);
	    lstrcat(lptszSystemRoot, lptcszPath);
    }
	return lptszSystemRoot;
}

///////////////////////////////////////////////////////////////////////////////
//    AppendPath()
//
//    Parameters
//      pzDirectory     Pointer to a fully qualified directory
//
//    Purpose
//      Add the directory in szDirectory to the system path.
//      
//    Return Value
//      None
//      
int WINAPI AppendPath(TCHAR* pzDir)
{
    int nRet = 0;
    TCHAR *pzDirectory = new TCHAR[lstrlen(pzDir)+1];
    if (pzDirectory)
    {
        lstrcpy(pzDirectory, pzDir);
        if (pzDirectory[lstrlen(pzDirectory)-1] == TCHAR('\\'))
        {
            pzDirectory[lstrlen(pzDirectory)-1] = 0;
        }
        HKEY    hKeyPath;
        if (RegOpenKeyEx(   HKEY_LOCAL_MACHINE,
                            "System\\CurrentControlSet\\Control\\Session Manager\\Environment",
                            0,
                            KEY_ALL_ACCESS,
                            &hKeyPath) == ERROR_SUCCESS)
        {
            DWORD   dwBytes = 0, dwType = 0;

            if (RegQueryValueEx(hKeyPath, "path", 0, &dwType, 0, &dwBytes) == ERROR_SUCCESS)
            {
                BYTE    *pbValue = new BYTE[dwBytes + (lstrlen(pzDirectory) + 2) * sizeof(TCHAR)];
                if (pbValue)
                {
                    if (RegQueryValueEx(    hKeyPath, 
                                            "path", 
                                            0, 
                                            &dwType, 
                                            pbValue, 
                                            &dwBytes) == ERROR_SUCCESS)
                    {
                        BOOL    bAddPath = TRUE;
                        unsigned nDirLen = 0;
                        TCHAR* pzStr = ((TCHAR*)pbValue) - 1;

                        while (pzStr++)
                        {
                            nDirLen = (unsigned)(strchr(pzStr, TCHAR(';')) - pzStr);
                            if (0 == _strnicmp(pzStr, pzDirectory, nDirLen))
                            {
                                bAddPath = FALSE;
                                break;
                            }
                            pzStr = strchr(pzStr, TCHAR(';'));
                        }
                        if (bAddPath)
                        {
                            TCHAR* pzPath = (TCHAR*)pbValue;
                            lstrcat(pzPath, TEXT(";"));
                            lstrcat(pzPath, pzDirectory);
                            dwBytes = lstrlen(pzPath) * sizeof(TCHAR);
                            if (RegSetValueEx(  hKeyPath, 
                                                "path", 
                                                0, 
                                                dwType, 
                                                pbValue, 
                                                dwBytes) == ERROR_SUCCESS)
                            {
                                SendMessage(    HWND_BROADCAST,
                                                WM_WININICHANGE, 
                                                0, 
                                                (LPARAM)TEXT("Environment"));
                                nRet = 1;
                            }
                        }
                    }
                }
            }
            RegCloseKey(hKeyPath);
        }
        delete [] pzDirectory;
    }
    return nRet;
}

///////////////////////////////////////////////////////////////////////////////
//    SetupThreadRoutine()
//
//    Parameters
//      lpParam,    hWnd, the handle of the main window (setup dialog)
//
//    Purpose
//      perform the actual installation
//      
//    Return Value
//      thread return value
//      
DWORD WINAPI SetupThreadRoutine (void* lpParam)
{
    BOOL    bRet = 0;                   // return value
    TCHAR   **ppzSrvcNames      = 0;    // names of services to install
    TCHAR   **ppzDisplayNames   = 0;    // display names of services to install
	LPARAM	lParam      = IDOK;
	DWORD   dwStepIt    = MAKELONG(IDC_THREAD, TM_STEPIT);
	DWORD   dwDisplay   = MAKELONG(IDC_THREAD, TM_DISPLAY);
	DWORD   dwDone      = MAKELONG(IDC_THREAD, TM_DONE);

    ppzSrvcNames        = (TCHAR**)new unsigned long[g_nSelServices];
    ppzDisplayNames     = (TCHAR**)new unsigned long[g_nSelServices];
    if (ppzSrvcNames && ppzDisplayNames)
    {
	    ZeroMemory(ppzSrvcNames, sizeof(TCHAR*) * g_nSelServices);
		ZeroMemory(ppzDisplayNames, sizeof(TCHAR*) * g_nSelServices);
        TCHAR*      pzSetupInf = GetSetupInfFileName();
        if (pzSetupInf)
        {
            PSECTION    pSection = 0;
            OpenSection(pzSetupInf, TEXT("Services"), &pSection);
            if (pSection)
            {
                unsigned    uSeed = 0;
                TCHAR       *pzLHS = 0, 
                            *pzRHS = 0;
                unsigned    i = 0, j = 0;
            
                // for every Service listed in the SETUP.INF
                while (ReadSection(pSection, &uSeed, &pzLHS, &pzRHS))
                {
                    if (j == g_lpSelServices[i])
                    {
                        // populate the array of selected Services
                        ppzSrvcNames[i] = pzLHS;
                        ppzDisplayNames[i] = pzRHS;
                        i++;
                    }
                    else
                    {
                        delete []pzLHS;
                        delete []pzRHS;
                    }
                    j++;
                }
                CloseSection(&pSection);
            }
            // for every Service selected
            for (unsigned i = 0;i < g_nSelServices;i++)
            {
                TCHAR pzSectName[128];
        
                // open [Provider1] section
                SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwDisplay, (long)lptszMsg[0]);
                lstrcpy(pzSectName, ppzSrvcNames[i]);
                OpenSection(pzSetupInf, pzSectName, &pSection);
                if (pSection)
                {
                    TCHAR      *pzDestDir = GetMapiInfDir();
                    unsigned   uSeed = 0, j = 0;
                    TCHAR      *pzLHS = 0, *pzRHS = 0;

                    // establish the destination directory, i.e.
                    // the default dir. or the specified dir.
			        // Directory=C:\MyProviders
                    while (ReadSection(pSection, &uSeed, &pzLHS, &pzRHS) && (j < 4))
                    {
                        if (!lstrcmpi(pzLHS, TEXT("Directory")))
                        {
                            delete []pzDestDir;
                            pzDestDir = pzRHS;
							unsigned uDestDirSize = lstrlen(pzDestDir);

							// if dir. does not have a trailing \\ append one
							if (pzDestDir[uDestDirSize-1] != TCHAR('\\'))
							{
								pzRHS = new TCHAR[uDestDirSize + 2];
								if (pzRHS)
								{
									lstrcpy(pzRHS, pzDestDir);
									lstrcat(pzRHS, TEXT("\\"));
									delete []pzDestDir;
									pzDestDir = pzRHS;
								}
								else
								{
									lParam = IDC_E_CREATE_DIR;
								}
							}
							if (lParam == IDOK)
							{
								if (CreateDirectory(pzRHS, 0))
								{
									AppendPath(pzDestDir);
								}
							}
                            delete []pzLHS;
                            break;
                        }
                        delete []pzLHS;
                        delete []pzRHS;
                        j++;
                    }

					if (lParam == IDOK)
					{
						// copy files to the destination directory
						SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwStepIt, 0);
						SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwDisplay, (long)lptszMsg[1]);
						j = uSeed = 0;
						while (ReadSection(pSection, &uSeed, &pzLHS, &pzRHS) && j < 4)
						{
							if (!lstrcmpi(pzLHS, TEXT("Files")))
							{
								unsigned uSeedParse = 0;
								TCHAR* pzFilename = 0;
								TCHAR* pzSrcDir = GetSetupInfDir();
								while (ParseString(pzRHS, &uSeedParse, TCHAR(';'), &pzFilename))
								{
									TCHAR *pzDestFile = new TCHAR[lstrlen(pzDestDir)+lstrlen(pzFilename)+1];
									TCHAR *pzSrcFile = new TCHAR[lstrlen(pzSrcDir)+lstrlen(pzFilename)+1];
                
									if (pzDestFile && pzSrcFile)
									{
										lstrcpy(pzDestFile, pzDestDir);
										lstrcat(pzDestFile, pzFilename);
										lstrcpy(pzSrcFile, pzSrcDir);
										lstrcat(pzSrcFile, pzFilename);
										if (!CopyFile(pzSrcFile, pzDestFile, FALSE))
										{
											lParam = IDC_E_COPY_FILE;
										}
									}
									delete []pzFilename;
									delete []pzDestFile;
									delete []pzSrcFile;
								}
								delete []pzSrcDir;
								delete []pzLHS;
								delete []pzRHS;
								break;
							}
							delete []pzLHS;
							delete []pzRHS;
							j++;
						}
					}

                    // merge the PROVIDER.INF entries with MAPISVC.INF for all
                    // subsections of the service
					if (lParam == IDOK)
					{
						SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwStepIt, 0);
						SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwDisplay, (long)lptszMsg[2]);
						j = uSeed = 0;
						TCHAR* pzProviderInf = 0;
						// INF_File=PROVIDER1.INF
						while (ReadSection(pSection, &uSeed, &pzLHS, &pzRHS) && j < 4)
						{
							if (!lstrcmpi(pzLHS, TEXT("INF_File")))
							{
								TCHAR* pzTmp = GetSetupInfDir();
								pzProviderInf = new TCHAR[lstrlen(pzTmp) + lstrlen(pzRHS) + 2];

								if (pzProviderInf)
								{
									lstrcpy(pzProviderInf, pzTmp);
									lstrcat(pzProviderInf, pzRHS);
								}
								delete []pzTmp;
								delete []pzLHS;
								delete []pzRHS;
								break;
							}
							delete []pzLHS;
 							delete []pzRHS;
						 j++;
						}
						if (pzProviderInf)
						{
							j = uSeed = 0;
							// Sections=Provider1;SubSectionP1
							while (ReadSection(pSection, &uSeed, &pzLHS, &pzRHS) && j < 4)
							{
								if (!lstrcmpi(pzLHS, TEXT("Sections")))
								{
									CopySections(pzRHS, pzProviderInf);
									delete []pzLHS;
									delete []pzRHS;
									break;
								}
								delete []pzLHS;
								delete []pzRHS;
								j++;
							}
							delete []pzProviderInf;
						}
						else
						{
							lParam = IDC_E_MERGE_INF;
						}
					}

                    delete []pzDestDir;
                    CloseSection(&pSection);
                }
            }
            delete []pzSetupInf;
        }
        // Add services to the default profile
        if (g_bAddToProfile && lParam == IDOK)
        {
    	    SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwDisplay, (long)lptszMsg[3]);
            MAPIINIT_0  init = { 0 };
            init.ulVersion = MAPI_INIT_VERSION;
            init.ulFlags   = fMapiUnicode;
            HRESULT hr = MAPIInitialize(&init);
            if (SUCCEEDED(hr))
            {
                LPMAPISESSION lpSession = 0;
                hr = MAPILogonEx (0, 0, 0, 
                                  MAPI_NEW_SESSION | MAPI_NO_MAIL | MAPI_USE_DEFAULT, 
                                  &lpSession);
                if (SUCCEEDED(hr))
                {
                    LPSERVICEADMIN lpSrvcAdmin = NULL;
                    hr = lpSession->AdminServices(0, &lpSrvcAdmin);
                    if (SUCCEEDED(hr))
                    {
                        bRet = TRUE;
                        for (unsigned i = 0;i < g_nSelServices;i++)
                        {
                            SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwStepIt, 0);
                            hr = lpSrvcAdmin->CreateMsgService( ppzSrvcNames[i], 
                                                              ppzDisplayNames[i],
                                                              (unsigned long)lpParam,
                                                              fMapiUnicode | SERVICE_UI_ALLOWED);
                            if (FAILED(hr))
                            {
                                lParam = IDC_E_CREATE_SERVICE;
                            }
                        }
                        lpSrvcAdmin->Release();
                    }
                    lpSession->Logoff(0, 0, 0);
                    lpSession->Release();
                }
                MAPIUninitialize();
            }
        }
        for (unsigned i = 0;i < g_nSelServices;i++)
        {
            delete ppzSrvcNames[i];
        }
        for (i = 0;i < g_nSelServices;i++)
        {
            delete ppzDisplayNames[i];
        }
    }
    delete []ppzSrvcNames;
    delete []ppzDisplayNames;
	SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwStepIt, 0);
	SendNotifyMessage((HWND)lpParam, WM_COMMAND, dwDone, lParam);
    return bRet;
}

///////////////////////////////////////////////////////////////////////////////
//    CopySections()
//
//    Parameters
//
//    Purpose
//      
//    Return Value
//      None
//      
void WINAPI CopySections (LPTSTR pzSections, LPTSTR pzProvInf)
{
	TCHAR* pzSect = 0;
	unsigned uSeed = 0;
	TCHAR* pzMapiSvc = GetMapiInfFileName();
	if (pzMapiSvc)
	{
		while (ParseString(pzSections, &uSeed, TCHAR(';'), &pzSect))
		{
			PSECTION pSect = 0;
			OpenSection(pzProvInf, pzSect, &pSect);
			if (pSect)
			{
				TCHAR *pzLHS = 0, *pzRHS = 0;
				unsigned uSeedSctn = 0;
				while (ReadSection(pSect, &uSeedSctn, &pzLHS, &pzRHS))
				{
						WritePrivateProfileString(pzSect, pzLHS, pzRHS, pzMapiSvc);
						delete []pzLHS;
						delete []pzRHS;
				}
				CloseSection(&pSect);
			}
			delete []pzSect;
		}
		delete []pzMapiSvc;
	}
}

// End of file for SETUP.CPP
