///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      ABMLUSER.CPP
//
//  Description
//      This file implements the IMailUser interface with the methods 
//      specified in the MAPI SPI 1.0.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#include "ABWDS.H"

// Remark this line to turn verbose tracing OFF
//#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)        TraceInfoMessage(a)
#define InfoTraceRaw(a)     TraceRaw(a)
#define InfoTraceProp(a)    TraceProp(a)
#else
#define InfoTrace(a)        0
#define InfoTraceRaw(a)     0
#define InfoTraceProp(a)    0
#endif // DO_INFO_TRACES


#define NUM_MAILUSER_PROPS        23
// Properties available in our implementation of IMailUser
SizedSPropTagArray(NUM_MAILUSER_PROPS, sptMailUserObj) =
{
    NUM_MAILUSER_PROPS,
    {
        PR_ACCESS,
        PR_ACCESS_LEVEL,
        PR_RECORD_KEY,
        PR_ENTRYID,
        PR_SEARCH_KEY,
        PR_MAPPING_SIGNATURE,
        PR_OBJECT_TYPE,
        PR_CREATION_VERSION,
        PR_CURRENT_VERSION,
        PR_DISPLAY_TYPE,
        PR_DISPLAY_NAME,
        PR_TRANSMITABLE_DISPLAY_NAME,
        PR_ACCOUNT,
        PR_TITLE,
        PR_OFFICE_LOCATION,
        PR_OFFICE_TELEPHONE_NUMBER,
        PR_OTHER_TELEPHONE_NUMBER,
        PR_PRIMARY_FAX_NUMBER,
        PR_COMMENT,
        PR_ADDRTYPE,
        PR_EMAIL_ADDRESS,
        PR_SEND_RICH_INFO,
        PR_INITIAL_DETAILS_PANE
    }
};

// Control description structures
DTBLEDIT DtObjAlias       = { sizeof(DTBLEDIT), fMapiUnicode, MAX_ALIAS_SIZE,  PR_ACCOUNT};
DTBLEDIT DtObjFullName    = { sizeof(DTBLEDIT), fMapiUnicode, MAX_STRING_SIZE, PR_DISPLAY_NAME};
DTBLEDIT DtMBUserTitle    = { sizeof(DTBLEDIT), fMapiUnicode, MAX_STRING_SIZE, PR_TITLE};
DTBLEDIT DtMBUserOffice   = { sizeof(DTBLEDIT), fMapiUnicode, MAX_STRING_SIZE, PR_OFFICE_LOCATION};
DTBLEDIT DtMBUserPhone    = { sizeof(DTBLEDIT), fMapiUnicode, MAX_PHONE_SIZE,  PR_OFFICE_TELEPHONE_NUMBER};
DTBLEDIT DtMBUserAltPhone = { sizeof(DTBLEDIT), fMapiUnicode, MAX_PHONE_SIZE,  PR_OTHER_TELEPHONE_NUMBER};
DTBLEDIT DtMBUserFax      = { sizeof(DTBLEDIT), fMapiUnicode, MAX_PHONE_SIZE,  PR_PRIMARY_FAX_NUMBER};
DTBLLBX  DtManager        = { MAPI_NO_VBAR, PR_NULL, PR_SMP_MANAGER_NAME_TBL };
DTBLEDIT DtMBUserCompany  = { sizeof(DTBLEDIT), fMapiUnicode, MAX_STRING_SIZE, PR_COMPANY_NAME };
DTBLEDIT DtMBUserDept     = { sizeof(DTBLEDIT), fMapiUnicode, MAX_STRING_SIZE, PR_DEPARTMENT_NAME };

// Description table for the controls in the first page
DTCTL MailboxPropsPage1[] =
{
    { DTCT_PAGE,     0, NULL, 0, NULL,     0,                 &DtPage         },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_ALIAS,    &DtObjAlias     },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_FULLNAME, &DtObjFullName  },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_TITLE,    &DtMBUserTitle  },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_OFFICE,   &DtMBUserOffice },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_PHONE,    &DtMBUserPhone  },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC1,       &DtLabel        },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC2,       &DtLabel        },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC3,       &DtLabel        },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC4,       &DtLabel        },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC5,       &DtLabel        },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC6,       &DtGroupBox     },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC7,       &DtGroupBox     }
};

// Description table for the controls in the second page
DTCTL MailboxPropsPage2[] =
{
    { DTCT_PAGE,     0, NULL, 0, NULL,     0,                 &DtPage           },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_PHONE,    &DtMBUserPhone    },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_ALTPHONE, &DtMBUserAltPhone },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_USER_FAX,      &DtMBUserFax      },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_COMMENTS,      &DtComment        },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC1,       &DtLabel          },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC2,       &DtLabel          },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC3,       &DtLabel          },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC4,       &DtGroupBox       },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC5,       &DtGroupBox       }
};

// Description table for the controls in the third page
DTCTL MailboxPropsPage3[] =
{
    { DTCT_PAGE,     0, NULL, 0, NULL,     0,                 &DtPage          },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_COMPANY,       &DtMBUserCompany },
    { DTCT_EDIT,     0, NULL, 0, szFilter, IDC_DEPARTMENT,    &DtMBUserDept    },
    { DTCT_LBX,      0, NULL, 0, szFilter, IDC_USER_MANAGER,  &DtManager       },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC1,       &DtLabel         },
    { DTCT_LABEL,    0, NULL, 0, NULL,     IDC_STATIC2,       &DtLabel         },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC3,       &DtGroupBox      },
    { DTCT_GROUPBOX, 0, NULL, 0, NULL,     IDC_STATIC4,       &DtGroupBox      }
};


///////////////////////////////////////////////////////////////////////////////
//    CMailUser::CMailUser()
//
//    Parameters
//      pLogon          Parent AB logon object to which this object is
//                      linked to.
//      pObjectEID      Entry ID of this object
//      pList           Property cache list with most of the properties
//                      of this object
//      pNPList         Pointer to the named property map list
//      lAccess         Access level of this object (Read-only,
//                      read-write, etc)
//
//    Purpose
//      Constructor of the object. Parameters are passed to initialize the
//      data members with the appropiate values.
//
//    Return Value
//      None
//
CMailUser::CMailUser (CABLogon *        pLogon,
                      PRIVATE_ENTRYID * pObjectEID,
                      CPropList *       pList,
                      CNamePropMgr *    pNPList,
                      long              lAccess) : CGenericProp (pLogon->m_pBEobj,
                                                                 NULL,
                                                                 pObjectEID,
                                                                 pList,
                                                                 FALSE,
                                                                 lAccess)       // Call the base class constructor
{
    InfoTrace ("CMailUser: Constructor called");
    m_pNPList = pNPList;
    // We must save a reference to the parent logon object
    m_pLogon = pLogon;
    m_pLogon->AddRef();
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::~CMailUser()
//
//    Parameters
//      None
//
//    Purpose
//      Destructor of CMailUser.
//
//    Return Value
//      None
//
CMailUser::~CMailUser()
{
    InfoTrace ("CMailUser: Destructor called");
    m_pLogon->Release();
    m_pLogon = NULL;
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::QueryInterface()
//
//    Parameters
//      { Refer to the OLE documentation for parameter description }
//
//    Purpose
//      Returns a pointer to a interface requested if the interface is 
//      supported and implemented by this object. If it is not supported, it
//      returns NULL.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::QueryInterface (REFIID riid, LPVOID * ppvObj)
{
    Validate_IUnknown_QueryInterface (this, riid, ppvObj);
    // OLE requires NULLing parameter
    *ppvObj = NULL;
    // If the interface requested is supported by this object, return a pointer
    // to ourselves, with the reference count incremented by one.
    if (riid == IID_IMailUser || riid == IID_IMAPIProp || riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    // This object does not support the interface requested
    return E_NOINTERFACE;
}

///////////////////////////////////////////////////////////////////////////////
// IMAPIProp virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::GetLastError()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Return a MAPIERROR structure filled up with meaningful descriptions
//      about an error that occurred in our provider. The caller is responsible
//      for freeing the memory returned.
//      In our implementation we delegate the call up to the base class which
//      implements this call.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::GetLastError (HRESULT        hError,
                                      ULONG          ulFlags,
                                      LPMAPIERROR *  ppMAPIError)
{
    InfoTrace ("CMailUser::GetLastError method called");
    Validate_IMAPIProp_GetLastError (this, hError, ulFlags, ppMAPIError);
    return CGeneric::GetLastError (hError,
                                   ulFlags,
                                   ppMAPIError,
                                   m_pLogon->m_hInstance,
                                   PROVIDER_DISPLAY_NAME_STRING);
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::SaveChanges()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. Our implementation is read-only so there are no changes
//      to commit. We return success all the time to be polite to the caller.
//
//    Return Value
//      S_OK always
//
STDMETHODIMP CMailUser::SaveChanges (ULONG ulFlags)
{
    InfoTrace ("CMailUser::SaveChanges method called");
    Validate_IMAPIProp_SaveChanges (this, ulFlags);
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::GetProps()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Return the value of the properties requested in the list of tags
//      passed in. The returned property array is in the same order as
//      the tag array passed.
//      The memory for returned properties is allocated using the MAPI
//      allocation functions. The caller is responsible for free it with
//      MAPIFreeBuffer() or the equivalent.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::GetProps (LPSPropTagArray     pPropTagArray,
                                  ULONG               ulFlags,
                                  ULONG *             pcValues,
                                  LPSPropValue *      ppPropArray)
{
    InfoTrace ("CMailUser::GetProps method called");
    Validate_IMAPIProp_GetProps (this, pPropTagArray, ulFlags, pcValues, ppPropArray);
    #ifdef DO_INFO_TRACES
    if (!pPropTagArray)
    {
        TraceInfoMessage ("CMailUser::GetProps: All properties requested");
    }
    else
    {
        TraceMessage ("CMailUser::GetProps: Properties requested...");
        for (ULONG j=0; j<pPropTagArray->cValues; j++) { TraceProp (pPropTagArray->aulPropTag[j]); }
    }
    #endif // DO_INFO_TRACES
    
    EnterCriticalSection (&m_csObj);
    BOOL fGotErrors = FALSE;
    // Ask the base class to do partial work for this call:
    //      1. Collect all the property tags of the object including the ones in the property cache
    //      2. Allocate a new SPropValue array for the returned properties
    //      3. Replace the tags of PT_UNSPECIFIED type in the tag passed in
    //         with the actual type of the store property (if it exists in this object)
    HRESULT hResult = CGenericProp::GetProps (pPropTagArray,
                                              ulFlags,
                                              pcValues,
                                              ppPropArray,
                                              (LPSPropTagArray)&sptMailUserObj);
    if (hResult)
    {
        goto ErrorExit;
    }
    
    LPVOID pSrc;
    ULONG cbSize, i;
    // Get the values for all the properties requested
    for (cbSize=0, i=0; i<*pcValues; i++)
    {
        pSrc = NULL;
        hResult = S_OK;
        switch ((*ppPropArray)[i].ulPropTag)
        {
            // Get the PT_OBJECT/PR_NULL properties
            case PR_NULL :
                (*ppPropArray)[i].Value.err = S_OK;
                break;
            // Get the PT_LONG properties
            case PR_ACCESS :
                (*ppPropArray)[i].Value.l = m_lAccess;
                break;
            case PR_ACCESS_LEVEL :
                (*ppPropArray)[i].Value.l = 0;
                break;
            case PR_OBJECT_TYPE :
                (*ppPropArray)[i].Value.l = MAPI_MAILUSER;
                break;
            case PR_INITIAL_DETAILS_PANE :
                (*ppPropArray)[i].Value.l = 0;
                break;
            // Get the PT_BOOLEAN properties
            case PR_SEND_RICH_INFO :
                (*ppPropArray)[i].Value.b = TRUE;
                break;
            // Get the PT_TSTRING properties
            case PR_ADDRTYPE :
                pSrc = (LPVOID)WINDS_ADDRESS_TYPE;
                break;
            // Get the PT_I8 properties
            case PR_CREATION_VERSION :
                (*ppPropArray)[i].Value.li.LowPart = ADR_BOOK_VERSION_MAJOR;
                (*ppPropArray)[i].Value.li.HighPart = ADR_BOOK_VERSION_MINOR;
                break;
            case PR_CURRENT_VERSION :
                (*ppPropArray)[i].Value.li.LowPart = ADR_BOOK_VERSION_MAJOR;
                (*ppPropArray)[i].Value.li.HighPart = ADR_BOOK_VERSION_MINOR;
                break;
            // Get the PT_BINARY properties
            case PR_MAPPING_SIGNATURE :
                cbSize = sizeof(MAPIUID);
                pSrc = (LPVOID)&m_pBEobj->GetBackEndUID();
                break;
            case PR_RECORD_KEY :
            case PR_ENTRYID :
                cbSize = CB_PRIVATE_EID;
                pSrc = (LPVOID)&m_ObjectEID;
                break;
            // If the property was not one of the above, try the property cache.
            default :
                // If the property was not one of the above, ask the base class implementation
                // to search for it in the property cache of the object. If the property was in the
                // property cache, the base class copies it to the LPSPropValue passed in, linking
                // its memory to the parent memory block passed in.
                hResult = CGenericProp::GetProps (&(*ppPropArray)[i], *ppPropArray);
                #ifdef DO_INFO_TRACES
                if (MAPI_E_NOT_FOUND == hResult)
                {
                    TraceRaw ("CMailUser::GetProps: Not Found - "); TraceProp ((*ppPropArray)[i].ulPropTag);
                }
                #endif // #DO_INFO_TRACES
                break;
        }
        // Now that I have the value of the property in pSrc or the error in hResult,
        // copy the property or set the property tag and error if not found.
        if (S_OK != CGenericProp::GetProps (hResult,
                                            cbSize,
                                            pSrc,
                                            &(*ppPropArray)[i],
                                            *ppPropArray))
        {
            fGotErrors = TRUE;
        }
    }
ErrorExit :
    LeaveCriticalSection (&m_csObj);
    if (fGotErrors)
	{
        // I got at least one error. Return partial success.
		hResult = MAPI_W_ERRORS_RETURNED;
	}
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::GetPropList()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Duplicate the tag array with ALL the available properties
//      on this object. In our implementation we delegate the call down to
//      the base class.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::GetPropList (ULONG ulFlags, LPSPropTagArray * ppAllTags)
{
    InfoTrace ("CMailUser::GetPropList method called");
    Validate_IMAPIProp_GetPropList (this, ulFlags, ppAllTags);
    return CGenericProp::GetPropList (ulFlags, (LPSPropTagArray)&sptMailUserObj, ppAllTags);
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::OpenProperty()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      This call is supported only to return an IMAPITable on
//      PR_DETAILS_TABLE with an IID_IMAPITable interface. Any other
//      property return MAPI_E_NO_SUPPORT. And any other interface
//      returns E_NOINTERFACE.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::OpenProperty (ULONG           ulPropTag,
                                      LPCIID          piid,
                                      ULONG           ulInterfaceOptions,
                                      ULONG           ulFlags,
                                      LPUNKNOWN *     ppUnk)
{
    InfoTrace ("CMailUser::OpenProperty method called");
    Validate_IMAPIProp_OpenProperty (this,
                                     ulPropTag, 
                                     piid,
                                     ulInterfaceOptions,
                                     ulFlags,
                                     ppUnk);
    InfoTraceRaw ("Property to open: "); InfoTraceProp (ulPropTag);
    HRESULT hResult = S_OK;
    EnterCriticalSection (&m_csObj);
    // The only property that we allow callers to open is the details table of the user properties.
    if (PR_DETAILS_TABLE == ulPropTag)
    {
        // Make sure the user is requesting a table interface.
        if (IID_IMAPITable != *piid)
        {
            hResult = E_NOINTERFACE;
        }
        else
        {
            // Initialize the DTPAGE structure with the static members defined in
            // the head of this file
            DTPAGE DtMBUserPages[3] = { 0 };
            DtMBUserPages[0].cctl = sizeof(MailboxPropsPage1)/sizeof(DTCTL);
            DtMBUserPages[0].lpszComponent = szBlank;
            DtMBUserPages[0].lpszResourceName = MAKEINTRESOURCE (IDD_DETAILS_USERGENERAL);
            DtMBUserPages[0].lpctl = MailboxPropsPage1;

            DtMBUserPages[1].cctl = sizeof(MailboxPropsPage2)/sizeof(DTCTL);
            DtMBUserPages[1].lpszComponent = szBlank;
            DtMBUserPages[1].lpszResourceName = MAKEINTRESOURCE (IDD_DETAILS_USERPHONES);
            DtMBUserPages[1].lpctl = MailboxPropsPage2;

            // If the IMailUser is a ONE-OFF entry being created in someone else's address book,
            // then the fields in the property pages are writable so that new information can be entered.
            if (OBJECT_IS_ONE_OFF & m_ulObjFlags)
            {
                DtMBUserPages[0].lpctl[1].ulCtlFlags |= (DT_EDITABLE | DT_REQUIRED);
                DtMBUserPages[0].lpctl[2].ulCtlFlags |= DT_EDITABLE;
                DtMBUserPages[0].lpctl[3].ulCtlFlags |= DT_EDITABLE;
                DtMBUserPages[0].lpctl[4].ulCtlFlags |= DT_EDITABLE;
                DtMBUserPages[0].lpctl[5].ulCtlFlags |= DT_EDITABLE;

                DtMBUserPages[1].lpctl[1].ulCtlFlags |= DT_EDITABLE;
                DtMBUserPages[1].lpctl[2].ulCtlFlags |= DT_EDITABLE;
                DtMBUserPages[1].lpctl[3].ulCtlFlags |= DT_EDITABLE;
                DtMBUserPages[1].lpctl[4].ulCtlFlags |= DT_EDITABLE;
                
                DtMBUserPages[2].lpctl[1].ulCtlFlags |= DT_EDITABLE;
                DtMBUserPages[2].lpctl[2].ulCtlFlags |= DT_EDITABLE;
            }
            else
            {
                DtMBUserPages[2].cctl = sizeof(MailboxPropsPage3)/sizeof(DTCTL);
                DtMBUserPages[2].lpszComponent = szBlank;
                DtMBUserPages[2].lpszResourceName = MAKEINTRESOURCE (IDD_DETAILS_USER_ORG);
                DtMBUserPages[2].lpctl = MailboxPropsPage3;

                // .. otherwise, the fields are READ-ONLY because our address book is server based and read-only.
                DtMBUserPages[0].lpctl[1].ulCtlFlags = 0;
                DtMBUserPages[0].lpctl[2].ulCtlFlags = 0;
                DtMBUserPages[0].lpctl[3].ulCtlFlags = 0;
                DtMBUserPages[0].lpctl[4].ulCtlFlags = 0;
                DtMBUserPages[0].lpctl[5].ulCtlFlags = 0;

                DtMBUserPages[1].lpctl[1].ulCtlFlags = 0;
                DtMBUserPages[1].lpctl[2].ulCtlFlags = 0;
                DtMBUserPages[1].lpctl[3].ulCtlFlags = 0;
                DtMBUserPages[1].lpctl[4].ulCtlFlags = 0;

                DtMBUserPages[2].lpctl[1].ulCtlFlags = 0;
                DtMBUserPages[2].lpctl[2].ulCtlFlags = 0;
                DtMBUserPages[2].lpctl[3].ulCtlFlags = 0;
            }
            // Use MAPI's helper function to build the display table.
            hResult = BuildDisplayTable (GetMAPIAllocBuffer(),
                                         GetMAPIAllocMore(),
                                         GetMAPIFreeBuffer(),
                                         NULL,
                                         m_pLogon->m_hInstance,
                                         sizeof (DtMBUserPages)/sizeof(DTPAGE),
                                         DtMBUserPages,
                                         fMapiUnicode,
                                         (LPMAPITABLE *)ppUnk,
                                         NULL);
        }
        TraceResult ("CMailUser::OpenProperty", hResult);
    }
    else
    {
        if (PR_SMP_MANAGER_NAME_TBL == ulPropTag)
        {
            if (IID_IMAPITable != *piid)
            {
                hResult = E_NOINTERFACE;
            }
            else
            {
                LPTABLEDATA pTableData;
                LPMAPITABLE pTableView;
                DWORD dwInstKey = 1;
                CPropList * pPropList = NULL;
                SPropValue spvProps[NUM_DIST_LIST_MEMBERS_DTLBX] = { 0 };
                SRow row = { 0 };
                row.cValues = NUM_DIST_LIST_MEMBERS_DTLBX;
                row.lpProps = spvProps;
                PRIVATE_ENTRYID ManagerEID = { 0 };
                hResult = m_pList->Find (PR_WINDS_AB_MANAGER_ENTRYID, &spvProps[LBX_EID], TRUE);
                if (!hResult)
                {
                    hResult = CreateTable (&IID_IMAPITableData,
                                           GetMAPIAllocBuffer(),
                                           GetMAPIAllocMore(),
                                           GetMAPIFreeBuffer(),
                                           NULL,
                                           TBLTYPE_DYNAMIC,
                                           PR_INSTANCE_KEY,
                                           (LPSPropTagArray)&sptDLMembersLbx,
                                           &pTableData);
                    if (!hResult)
                    {
                        for (ULONG i=0; i<NUM_DIST_LIST_MEMBERS_DTLBX; i++)
                        {
                            spvProps[i].ulPropTag = sptDLMembersLbx.aulPropTag[i];
                        }
                        spvProps[LBX_INSTKEY].Value.bin.cb = sizeof(DWORD);
                        spvProps[LBX_INSTKEY].Value.bin.lpb = (LPBYTE)&dwInstKey;
                        spvProps[LBX_OBJTYPE].Value.l = MAPI_MAILUSER;
                        spvProps[LBX_DISPTYPE].Value.l = DT_MAILUSER;
                        
                        CopyMemory (&ManagerEID, spvProps[LBX_EID].Value.bin.lpb, CB_PRIVATE_EID);
                        hResult = m_pBEobj->OpenObject (ManagerEID, &pPropList);
                        if (!hResult)
                        {
                            pPropList->Find (PR_DISPLAY_NAME, &spvProps[LBX_DISPNAME], TRUE);
                            pPropList->Find (PR_OFFICE_TELEPHONE_NUMBER, &spvProps[LBX_PHONE], TRUE);
                            pPropList->Find (PR_OFFICE_LOCATION, &spvProps[LBX_OFFICE], TRUE);
                            pPropList->Find (PR_TITLE, &spvProps[LBX_TITLE], TRUE);
                            pPropList->Find (PR_DEPARTMENT_NAME, &spvProps[LBX_DEPT], TRUE);
                            pPropList->Find (PR_COMPANY_NAME, &spvProps[LBX_COMPANY], TRUE);
                            pPropList->Find (PR_ACCOUNT, &spvProps[LBX_ACCT], TRUE);
                            hResult = pTableData->HrModifyRow (&row);
                            if (!hResult)
                            {
                                hResult = pTableData->HrGetView (NULL, NULL, 0, &pTableView);
                                if (!hResult)
                                {
                                    *ppUnk = (LPUNKNOWN)pTableView;
                                }
                            }
                            delete pPropList;
                        }
                    }
                    pTableData->Release(); // The view maintains a reference to the table data
                }
            }
        }
        else
        {
            hResult = MAPI_E_NO_SUPPORT;
        }
    }
    LeaveCriticalSection (&m_csObj);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::SetProps()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. Our IMAPIProp implementation is read-only.
//
//    Return Value
//      E_ACCESSDENIED always
//
STDMETHODIMP CMailUser::SetProps (ULONG                   cValues,
                                  LPSPropValue            pPropArray,
                                  LPSPropProblemArray *   ppProblems)
{
    InfoTrace ("CMailUser::SetProps method called");
    Validate_IMAPIProp_SetProps (this, cValues, pPropArray, ppProblems);
    return E_ACCESSDENIED;
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::DeleteProps()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Stub method. Our IMAPIProp implementation is read-only.
//
//    Return Value
//      E_ACCESSDENIED always
//
STDMETHODIMP CMailUser::DeleteProps (LPSPropTagArray          pPropTagArray,
                                     LPSPropProblemArray *    ppProblems)
{
    InfoTrace ("CMailUser::DeleteProps method called");
    Validate_IMAPIProp_DeleteProps (this, pPropTagArray, ppProblems);
    return E_ACCESSDENIED;
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::CopyTo()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Copy an object onto another. In our implementation we delegate the
//      call to MAPI to let it do the work for us.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::CopyTo (ULONG                   ciidExclude,
                                LPCIID                  rgiidExclude,
                                LPSPropTagArray         pExcludeProps,
                                ULONG                   ulUIParam,
                                LPMAPIPROGRESS          pProgress,
                                LPCIID                  pInterface,
                                LPVOID                  pDestObj,
                                ULONG                   ulFlags,
                                LPSPropProblemArray *   ppProblems)
{
    InfoTrace ("CMailUser::CopyTo method called");
    Validate_IMAPIProp_CopyTo (this,
                               ciidExclude,
                               rgiidExclude,
                               pExcludeProps,
                               ulUIParam,
                               pProgress,
                               pInterface,
                               pDestObj,
                               ulFlags,
                               ppProblems);
    return m_pLogon->m_pSupObj->DoCopyTo (pInterface,
                                          (LPVOID)this,
                                          ciidExclude,
                                          rgiidExclude,
                                          pExcludeProps,
                                          ulUIParam,
                                          pProgress,
                                          pInterface,
                                          pDestObj,
                                          ulFlags,
                                          ppProblems);
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::CopyProps()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Copy object properties onto another. In our implementation we
//      delegate the call to MAPI to let it do the work for us.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::CopyProps (LPSPropTagArray            pIncludeProps,
                                   ULONG                      ulUIParam,
                                   LPMAPIPROGRESS             pProgress,
                                   LPCIID                     pInterface,
                                   LPVOID                     pDestObj,
                                   ULONG                      ulFlags,
                                   LPSPropProblemArray *      ppProblems)
{
    InfoTrace ("CMailUser::CopyProps method called");
    Validate_IMAPIProp_CopyProps (this,
                                  pIncludeProps,
                                  ulUIParam,
                                  pProgress,
                                  pInterface,
                                  pDestObj,
                                  ulFlags,
                                  ppProblems);
    return m_pLogon->m_pSupObj->DoCopyProps (pInterface,
                                             (LPVOID)this,
                                             pIncludeProps,
                                             ulUIParam,
                                             pProgress,
                                             pInterface,
                                             pDestObj,
                                             ulFlags,
                                             ppProblems);
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::GetNamesFromIDs()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Returns the names of the named properties of this object. We defer
//      down to the common implementation to retrieve the names for the
//      caller.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::GetNamesFromIDs (LPSPropTagArray *    ppPropTags,
                                         LPGUID               pPropSetGuid,
                                         ULONG                ulFlags,
                                         ULONG *              pcPropNames,
                                         LPMAPINAMEID **      pppPropNames)
{
    InfoTrace ("CMailUser::GetNamesFromIDs method called");
    Validate_IMAPIProp_GetNamesFromIDs (this,
                                        ppPropTags,
                                        pPropSetGuid,
                                        ulFlags,
                                        pcPropNames,
                                        pppPropNames);
    LPSPropTagArray pTags;
    HRESULT hResult = CGenericProp::GetPropList (0, (LPSPropTagArray)&sptMailUserObj, &pTags);
    if (!hResult)
    {
        hResult = CGenericProp::GetNamesFromIDs (ppPropTags,
                                                 pPropSetGuid,
                                                 ulFlags,
                                                 pcPropNames,
                                                 pppPropNames,
                                                 pTags);
        PrivMAPIFree ((LPVOID *)&pTags);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CMailUser::GetIDsFromNames()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Return the IDs from our named properties mapping for a list of names.
//      We defer down to the common implementation to retrieve the IDs for the
//      names the caller requested.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMailUser::GetIDsFromNames (ULONG                cPropNames,
                                         LPMAPINAMEID *       ppPropNames,
                                         ULONG                ulFlags,
                                         LPSPropTagArray *    ppPropTags)
{
    InfoTrace ("CMailUser::GetIDsFromNames method called");
    Validate_IMAPIProp_GetIDsFromNames (this,
                                        cPropNames,
                                        ppPropNames,
                                        ulFlags,
                                        ppPropTags);
    LPSPropTagArray pTags;
    HRESULT hResult = CGenericProp::GetPropList (0, (LPSPropTagArray)&sptMailUserObj, &pTags);
    if (!hResult)
    {
        hResult = CGenericProp::GetIDsFromNames (cPropNames,
                                                 ppPropNames,
                                                 ulFlags,
                                                 ppPropTags,
                                                 pTags,
                                                 NULL);
        PrivMAPIFree ((LPVOID *)&pTags);
    }
    return hResult;
}

// End of file for ABMLUSER.CPP
