//////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      TABLE.H
//
//  Description
//      Class declaraction file for CTable. This file implements a wrapper
//      for the IMAPITable interface. The class intercepts calls to
//      IMAPITable::Restrict() to support PR_ANR restriction for ambiguous
//      name resolution in the contents table of the address book container.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _TABLE_H
#define _TABLE_H

#define STRICT
#include <WINDOWS.H>
#include <MAPIDEFS.H>
#include <MAPITAGS.H>

#include "TRACES.H"     // Include functions prototypes for the Trace and logging
                        // functions. This file must be included AFTER Windows.H

class  CTable: public IMAPITable
{
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    inline STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj)
                    { return m_pTableView->QueryInterface (riid, ppvObj); };
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    inline STDMETHODIMP GetLastError
                    (HRESULT                    hResult,
                     ULONG                      ulFlags,
                     LPMAPIERROR *              ppMAPIError)
                    { return m_pTableView->GetLastError (hResult, ulFlags, ppMAPIError); }
    inline STDMETHODIMP Advise
                    (ULONG                      ulEventMask,
                     LPMAPIADVISESINK           pAdviseSink,
                     ULONG *                    pulConnection)
                    { return m_pTableView->Advise (ulEventMask, pAdviseSink, pulConnection); }
    inline STDMETHODIMP Unadvise
                    (ULONG                      ulConnection)
                    { return m_pTableView->Unadvise (ulConnection); }
    inline STDMETHODIMP GetStatus
                    (ULONG *                    pulTableStatus,
                     ULONG *                    pulTableType)
                    { return m_pTableView->GetStatus (pulTableStatus, pulTableType); }
    inline STDMETHODIMP SetColumns
                    (LPSPropTagArray            pPropTagArray,
                     ULONG                      ulFlags)
                    { return m_pTableView->SetColumns (pPropTagArray, ulFlags); }
    inline STDMETHODIMP QueryColumns
                    (ULONG                      ulFlags,
                     LPSPropTagArray *          pPropTagArray)
                    { return m_pTableView->QueryColumns (ulFlags, pPropTagArray); }
    inline STDMETHODIMP GetRowCount
                    (ULONG                      ulFlags,
                     ULONG *                    pulCount)
                    { return m_pTableView->GetRowCount (ulFlags, pulCount); }
    inline STDMETHODIMP SeekRow 
                    (BOOKMARK                   bkOrigin,
                     LONG                       RowCount,
                     LONG *                     plRowsSought)
                    { return m_pTableView->SeekRow (bkOrigin, RowCount, plRowsSought); }
    inline STDMETHODIMP SeekRowApprox   
                    (ULONG                      ulNumerator,
                     ULONG                      ulDenominator)
                    { return m_pTableView->SeekRowApprox (ulNumerator, ulDenominator); }
    inline STDMETHODIMP QueryPosition
                    (ULONG *                    pulRow,
                     ULONG *                    pulNumerator,
                     ULONG *                    pulDenominator)
                    { return m_pTableView->QueryPosition (pulRow, pulNumerator, pulDenominator); }
    inline STDMETHODIMP FindRow
                    (LPSRestriction             pRestriction,
                     BOOKMARK                   bkOrigin,
                     ULONG                      ulFlags)
                    { return m_pTableView->FindRow (pRestriction, bkOrigin, ulFlags); }
    STDMETHODIMP Restrict
                    (LPSRestriction             pRestriction,
                     ULONG                      ulFlags);
    inline STDMETHODIMP CreateBookmark  
                    (BOOKMARK *                 pbkPosition)
                    { return m_pTableView->CreateBookmark (pbkPosition); }
    inline STDMETHODIMP FreeBookmark (BOOKMARK bkPosition)
                    { return m_pTableView->FreeBookmark (bkPosition); }
    inline STDMETHODIMP SortTable
                    (LPSSortOrderSet            pSortCriteria,
                     ULONG                      ulFlags)
                    { return m_pTableView->SortTable (pSortCriteria, ulFlags); }
    inline STDMETHODIMP QuerySortOrder
                    (LPSSortOrderSet *          ppSortCriteria)
                    { return m_pTableView->QuerySortOrder (ppSortCriteria); }
    inline STDMETHODIMP QueryRows
                    (LONG                       RowCount,
                     ULONG                      ulFlags,
                     LPSRowSet *                ppRows)
                    { return m_pTableView->QueryRows (RowCount, ulFlags, ppRows); }
    inline STDMETHODIMP Abort()
                    { return m_pTableView->Abort(); }
    inline STDMETHODIMP ExpandRow
                    (ULONG                      cbInstanceKey,
                     LPBYTE                     pbInstanceKey,
                     ULONG                      ulRowCount,
                     ULONG                      ulFlags,
                     LPSRowSet *                ppRows,
                     ULONG *                    pulMoreRows)
                    { return m_pTableView->ExpandRow (cbInstanceKey,
                                                      pbInstanceKey,
                                                      ulRowCount,
                                                      ulFlags,
                                                      ppRows,
                                                      pulMoreRows); }
    inline STDMETHODIMP CollapseRow
                    (ULONG                      cbInstanceKey,
                     LPBYTE                     pbInstanceKey,
                     ULONG                      ulFlags,
                     ULONG *                    pulRowCount)
                    { return m_pTableView->CollapseRow (cbInstanceKey, pbInstanceKey, ulFlags, pulRowCount); }
    inline STDMETHODIMP WaitForCompletion
                    (ULONG                      ulFlags,
                     ULONG                      ulTimeout,
                     ULONG *                    pulTableStatus)
                    { return m_pTableView->WaitForCompletion (ulFlags, ulTimeout, pulTableStatus); }
    inline STDMETHODIMP GetCollapseState
                    (ULONG                      ulFlags,
                     ULONG                      cbInstanceKey,
                     LPBYTE                     pbInstanceKey,
                     ULONG *                    pcbCollapseState,
                     LPBYTE *                   ppbCollapseState)
                    { return m_pTableView->GetCollapseState (ulFlags,
                                                             cbInstanceKey,
                                                             pbInstanceKey,
                                                             pcbCollapseState,
                                                             ppbCollapseState); }
    inline STDMETHODIMP SetCollapseState
                    (ULONG                      ulFlags,
                     ULONG                      cbCollapseState,
                     LPBYTE                     pbCollapseState,
                     BOOKMARK *                 pbkLocation)
                    { return m_pTableView->SetCollapseState (ulFlags, cbCollapseState, pbCollapseState, pbkLocation); }

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CTable          (LPMAPITABLE         pTableView)
                    { m_cRef = 1; m_pTableView = pTableView; m_pTableView->AddRef();
                      InitializeCriticalSection (&m_csObj); }
    ~CTable         ()
                    { if (m_pTableView) { m_pTableView->Release(); }
                      DeleteCriticalSection (&m_csObj); }

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG               m_cRef;
    LPMAPITABLE         m_pTableView;
    CRITICAL_SECTION    m_csObj;
};

///////////////////////////////////////////////////////////////////////////////
//    CTable::Restrict()
//
//    Parameters
//      { Refer to the MAPI documentation for parameter description }
//
//    Purpose
//      Intercep restrictions on PR_ANR and handle them in a special way to
//      support Ambiguous Name Resolutions on the parent AB container. If
//      the restriction is not for PR_ANR, we delegate the call down to the
//      contained IMAPITable implementation.
//
//    Return Value
//      An HRESULT 
//
STDMETHODIMP CTable::Restrict (LPSRestriction   pRestriction,
                               ULONG            ulFlags)
{
    HRESULT hResult;
    LPSPropValue pProp;
    SPropValue spvString = { 0 };
    SRestriction SRes = { 0 };
    if (pRestriction && RES_PROPERTY == pRestriction->rt && PR_ANR == pRestriction->res.resProperty.ulPropTag)
    {
        pProp = pRestriction->res.resProperty.lpProp;
        spvString.Value.LPSZ = pProp->Value.LPSZ;
        SRes.rt = RES_CONTENT;
        SRes.res.resContent.lpProp = &spvString;
        SRes.res.resContent.ulPropTag = spvString.ulPropTag = PR_DISPLAY_NAME;
        SRes.res.resContent.ulFuzzyLevel = FL_SUBSTRING | FL_IGNORECASE;
        hResult = m_pTableView->Restrict (&SRes, ulFlags);
    }
    else
    {
        hResult = m_pTableView->Restrict (pRestriction, ulFlags);
    }
    return hResult;
}

#endif // _TABLE_H

// End of file for TABLE.H
