///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      XPWDS.H
//
//  Description
//      Interface declaraction file for CXPLogon, CXPProvider, and
//      CXPMAPIStatus.
//      Property tag definitions.
//      Constant definitions.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#ifndef _XPWDS_H
#define _XPWDS_H

#define STRICT
#include <WINDOWS.H>
#include <WINDOWSX.H>
#include <MAPIWIN.H>
#include <MAPISPI.H>
#include <MAPIUTIL.H>
#include <MAPIVAL.H>
#include <TNEF.H>

// Header file for the trace and logging functions.
#include "TRACES.H"

// Common defs used by providers to communicate to the WINDS Sample Messaging Host
#include "COMWINDS.H"

// Config Page Dialog Control IDs Message String ID's
#include "RESOURCE.H"

// Cached stream for the TNEF object
#include "XPSTREAM.H"

#include "XPCOMMON.H"

/// Names returned in the status object
#define TRANSPORT_DISPLAY_NAME_STRING       TEXT("WINDS Transport")
#define TRANSPORT_DLL_NAME_STRING           TEXT("XPWDS.DLL")

// Internal provider version
#define TRANSPORT_VERSION_MAJOR             1
#define TRANSPORT_VERSION_MINOR             8

// Name of the stream where the message properties get encoded using TNEF encapsulation
#define TNEF_FILE_NAME                      TEXT("WINMAIL.DAT")

// This is copied from GENPROP.H in other MAPI sample providers
typedef struct _PRIVATE_ENTRYID
{
    // MAPI-required fields
	BYTE        abFlags[4];     // 4 bytes          // MAPI Flags
	MAPIUID     uidGlobal;      // 16 bytes         // UID Unique for the backend (ie. at the database file level)  (This must be here)
    // Provider-defined fields
	BYTE        bVersion;       // 1 bytes          // Version of the entry ID schema
    BYTE        bObject;        // 1 bytes          // Object type
    BYTE        bPad[2];        // 2 bytes          // Pad to align the structure to 4-bytes boundaries
    DWORD       dwObjID;        // 4 bytes          // Object ID (container or object)
                                // 28 Bytes Total
} PRIVATE_ENTRYID;
#define     ENTRYID_VERSION     1       // Schema version
#define     CB_PRIVATE_EID      28      // Size of the structure above


// The order for this enumarations must match the order of the properties
// in the tag arrays. These arrays are used to access the 
// LPSPropValue arrays returned from MAPI interfaces. The use of enumerations
// hides differences when the number of properties in the tag array increases
// or decreases. It also helps maintain error-free indexing in the use of 
// the prop arrays
enum
{
    //SERVER_NAME           = 0,        // Array Indexes
    //MAILBOX_NAME          = 1,
    //MAILBOX_ID            = 2,
    //USER_NAME             = 3,
    //PASSWORD              = 4,
    //NET_CON               = 5,
    NUM_LOGON_PROPERTIES    = 6         // Array size
};

// ID array of property tags for the trannsport
const static SizedSPropTagArray(NUM_LOGON_PROPERTIES, sptLogonProps) =
{
    NUM_LOGON_PROPERTIES,
    {
        PR_SMP_REMOTE_SERVER,           // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 0)
        PR_SMP_MAILBOX_NAME,            // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 1)
        PR_SMP_MAILBOX_ID,              // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 2)
        PR_SMP_USER_NAME,               // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 3)
        PR_SMP_MAILBOX_PASSWORD,        // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 4)
        PR_SMP_CONNECTION_TYPE,         // DON'T CHANGE THE ORDER OF THIS PROPERTY  (position 5)
    }
};

// Indexes of the identity property array
enum
{
    XPID_NAME,              // Array Indexes
    XPID_EID,
    XPID_SEARCH_KEY,
    NUM_IDENTITY_PROPS      // Array size
};

#define OUT_MSG_PROPS 2
static const SizedSPropTagArray(OUT_MSG_PROPS, sptOutMsgProps) =
{
    OUT_MSG_PROPS,
    {
        PR_SENDER_ENTRYID,
        PR_SENT_REPRESENTING_NAME
    }
};

// Used to generate one-off's for the sender of newly received messages
enum
{
    NEW_SENDER_NAME,
    NEW_SENDER_EMAIL,
    NEW_SENT_NAME,
    NEW_SENT_EMAIL,
    NEW_MSG_FLAGS,
    NUM_NEW_MSG_PROPS
};
static const SizedSPropTagArray(NUM_NEW_MSG_PROPS, sptNewMsgProps) =
{
    NUM_NEW_MSG_PROPS,
    {
        PR_SENDER_NAME,
        PR_SENDER_EMAIL_ADDRESS,
        PR_SENT_REPRESENTING_NAME,
        PR_SENT_REPRESENTING_EMAIL_ADDRESS,
        PR_MESSAGE_FLAGS
    }
};

// Used to generate transport-computed properties on received mail messages
#define MSG_RECIP_PROPS     2
static const SizedSPropTagArray(MSG_RECIP_PROPS, sptMsgRecipProps) =
{
    MSG_RECIP_PROPS,
    {
        PR_EMAIL_ADDRESS,
        PR_RECIPIENT_TYPE
    }
};
// Indexes to order the recipient's table of messages being submitted
enum
{
    RECIP_ROWID,
    RECIP_NAME,
    RECIP_EMAIL_ADR,
    RECIP_TYPE,
    RECIP_RESPONSIBILITY,
    RECIP_DELIVER_TIME,
    RECIP_REPORT_TEXT,
    RECIP_REPORT_TIME,
    TABLE_RECIP_PROPS
};
const static SizedSPropTagArray (TABLE_RECIP_PROPS, sptRecipTable) =
{
    TABLE_RECIP_PROPS,
    {
        PR_ROWID,
        PR_DISPLAY_NAME,
        PR_EMAIL_ADDRESS,
        PR_RECIPIENT_TYPE,
        PR_RESPONSIBILITY,
        PR_DELIVER_TIME,
        PR_REPORT_TIME,
        PR_REPORT_TEXT
    }
};

// Available properties in the status object of this provider
#define NUM_STATUS_OBJECT_PROPS    13
const static SizedSPropTagArray (NUM_STATUS_OBJECT_PROPS, sptStatusObj) =
{
    NUM_STATUS_OBJECT_PROPS,
    {
        PR_OBJECT_TYPE,
        PR_STATUS_CODE,
        PR_PROVIDER_DISPLAY,
        PR_PROVIDER_DLL_NAME,
        PR_RESOURCE_METHODS,
        PR_RESOURCE_FLAGS,
        PR_RESOURCE_TYPE,
        PR_STATUS_STRING,
        PR_DISPLAY_NAME,
        PR_IDENTITY_DISPLAY,
        PR_IDENTITY_ENTRYID,
        PR_IDENTITY_SEARCH_KEY,
        PR_CURRENT_VERSION
    }
};

// Stand-alone helper functions (C style)
extern "C"
{
    HRESULT WINAPI DoLogonDlg
                    (PCFGDLG                    pCfgDialog);
    MSGSERVICEENTRY ServiceEntry;
    HRESULT WINAPI GetMAPIError
                    (LPMAPIERROR *              ppMAPIError,
                     ULONG                      ulFlags,
                     HRESULT                    hError,
                     HINSTANCE                  hInstance);
    extern LPALLOCATEBUFFER    gpfnAllocateBuffer;  // MAPIAllocateBuffer function
    extern LPALLOCATEMORE      gpfnAllocateMore;    // MAPIAllocateMore function    
    extern LPFREEBUFFER        gpfnFreeBuffer;      // MAPIFreeBuffer function      
}

// Forward class declarations
class CXPLogon;

DWORD WINAPI XPMailslotListenThreadProc (CXPLogon * pLogon);

class CXPMAPIStatus : public IMAPIStatus
{
friend CXPLogon;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}

    MAPI_IMAPIPROP_METHODS(IMPL);
    MAPI_IMAPISTATUS_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this transport
//
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CXPMAPIStatus     (CXPLogon *                 pLogon,
                     LPPROFSECT                 pProfSectObj);
    ~CXPMAPIStatus    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG               m_cRef;
    CXPLogon *          m_pLogon;
    LPPROFSECT          m_pProfSectObj;
    CRITICAL_SECTION    m_csObj;
};

class CXPLogon : public IXPLogon
{
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}

    MAPI_IXPLOGON_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this transport
//
public :
    STDMETHODIMP InitializeStatusRow
                    (ULONG                      ulFlags = 0);
    STDMETHODIMP SetIdentityProps
                    ();
    STDMETHODIMP SendMailMessage
                    (LPMESSAGE                  pMsgObj,
                     LPSRowSet                  pRecipRows);
    STDMETHODIMP SetIncomingProps
                    (LPMESSAGE                  pMsgObj);
    BOOL WINAPI GetMsgTempFileName
                    (LPTSTR                     pszFileName);
    BOOL WINAPI LoadStatusString
                    (LPTSTR                     pString,
                     UINT                       uStringSize);
    void WINAPI InitializeTransportStatusFlags
                    (ULONG                      ulFlags);
    void WINAPI UpdateStatus
                    (BOOL                       fAddValidate = FALSE,
                     BOOL                       fValidateOkState = FALSE);
    void WINAPI SetSessionFlags
                    (ULONG *                    pulFlags);
    void WINAPI SetOutgoingProps
                    (LPMESSAGE                  pMsgObj,
                     FILETIME                   ft);
    inline HANDLE WINAPI GetListenMailslot
                    () { return m_hMailslot; }
    inline void WINAPI SetListenMailslot
                    (HANDLE hNewMailSlot)
                    { CloseHandle (m_hMailslot);
                      m_hMailslot = hNewMailSlot; }
    inline DWORD WINAPI GetConnectionID
                    () { return m_dwServerConnectionID; }
    inline CONNECTION_TYPE WINAPI GetConnectionType
                    () { EnterCriticalSection (&m_csObj);
                         CONNECTION_TYPE Type = m_ConnectionType;
                         LeaveCriticalSection (&m_csObj);
                         return Type; }
    inline void WINAPI SetConnectionType
                    (CONNECTION_TYPE            NewState)
                    { EnterCriticalSection (&m_csObj);
                      m_ConnectionType = NewState;
                      LeaveCriticalSection (&m_csObj); }
    inline TRANSPORT_STATE WINAPI GetTransportState
                    () { return m_TransportState; }
    inline void WINAPI SetTransportState
                    (TRANSPORT_STATE NewState)
                    { EnterCriticalSection (&m_csObj);
                      m_TransportState = NewState;
                      LeaveCriticalSection (&m_csObj);}
    inline void WINAPI ResetServerNotifications
                    () { EnterCriticalSection (&m_csObj);
                         ::ResetServerNotifications (m_szServer,
                                                     m_UserInfo.szMailboxName,
                                                     WINDS_NOTIF_ON_XP | WINDS_NOTIF_ON_USER,
                                                     &m_dwServerConnectionID);
                         LeaveCriticalSection (&m_csObj); }
    inline void WINAPI AddStatusBits
                    (DWORD dwNewBits)
                    { EnterCriticalSection (&m_csObj);
                      m_ulTransportStatus |= dwNewBits;
                      LeaveCriticalSection (&m_csObj); }
    inline void WINAPI RemoveStatusBits
                    (DWORD dwOldBits)
                    { EnterCriticalSection (&m_csObj);
                      m_ulTransportStatus &= ~dwOldBits;
                      LeaveCriticalSection (&m_csObj); }
    inline DWORD WINAPI GetTransportStatusCode
                    ()
                    { EnterCriticalSection (&m_csObj); 
                      ULONG ulStatus = m_ulTransportStatus;
                      LeaveCriticalSection (&m_csObj);
                      return ulStatus; }
    inline HINSTANCE WINAPI GetInstance
                    () { return m_hInstance; }
    inline LPTSTR WINAPI GetServerName
                    () { return m_szServer; }
    inline void WINAPI SetServerName
                    (LPTSTR                     szNewServer)
                    { lstrcpy (m_szServer, szNewServer); }
    inline LPTSTR WINAPI GetAccountName
                    () { return m_UserInfo.szMailboxName; }
    inline void WINAPI SetAccountProps
                    (LPTSTR                     szAccountName,
                     DWORD                      dwAccountID,
                     LPTSTR                     szUserName,
                     LPTSTR                     szPassword)
                    { lstrcpy (m_UserInfo.szMailboxName, szAccountName);
                      lstrcpy (m_UserInfo.szFullName, szUserName);
                      lstrcpy (m_UserInfo.szPassword, szPassword);
                      m_UserEID.dwObjID = dwAccountID;
                      LPSPropValue pOldProps = m_pIdentityProps;
                      if (S_OK == SetIdentityProps())
                      {
                          gpfnFreeBuffer (pOldProps);
                      }
                      wsprintf (m_szAddress, TEXT("%s\\%s"), m_szServer, m_UserInfo.szMailboxName);}

private:
    void WINAPI CheckSpoolerYield
                    (BOOL                       fReset = FALSE);

///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
public :
    CXPLogon        (HINSTANCE                  hInstance,
                     LPMAPISUP                  pSupObj,
                     PMAILBOX_INFO              pUserMailboxInfo,
                     DWORD                      dwMailboxID,
                     LPTSTR                     pszRemoteServer,
                     HANDLE                     hUIMutex,
                     DWORD                      dwConnectID,
                     CONNECTION_TYPE            ConnectionType);
    ~CXPLogon       ();

///////////////////////////////////////////////////////////////////////////////
// Data members
private :
    // Object implementation internal data
    ULONG               m_cRef;
    HINSTANCE           m_hInstance;
    BOOL                m_fABWDSInstalled;
    HANDLE              m_hMailslot;
    CRITICAL_SECTION    m_csObj;
    DWORD               m_dwServerConnectionID;
    CONNECTION_TYPE     m_ConnectionType;

public :
    LPMAPISUP           m_pSupObj;
    HANDLE              m_hUIMutex;
    CXPMAPIStatus *       m_pStatusObj;

private :
    // Transport activity state
    ULONG               m_ulTransportStatus;
    TRANSPORT_STATE     m_TransportState;
    
    // Server information
    TCHAR               m_szServer[MAX_STRING_SIZE+1];
    
    // User information and identity on the WINDS messaging server
    MAILBOX_INFO        m_UserInfo;
    PRIVATE_ENTRYID     m_UserEID;
    TCHAR               m_szAddress[64];
public :
    LPSPropValue        m_pIdentityProps;
};

class CXPProvider : public IXPProvider
{
friend CXPLogon;
friend CXPMAPIStatus;
public:
///////////////////////////////////////////////////////////////////////////////
// Interface virtual member functions
//
    STDMETHODIMP QueryInterface
                    (REFIID                     riid,
                     LPVOID *                   ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef
                    () { EnterCriticalSection (&m_csObj);
                         ++m_cRef; ULONG ulCount = m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         return ulCount; }
    inline STDMETHODIMP_(ULONG) Release
                    () { EnterCriticalSection (&m_csObj);
                         ULONG ulCount = --m_cRef;
                         LeaveCriticalSection (&m_csObj);
                         if (!ulCount) { delete this; }
                         return ulCount;}
    MAPI_IXPPROVIDER_METHODS(IMPL);

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//
    
///////////////////////////////////////////////////////////////////////////////
// Constructors and destructors
//
public :
    CXPProvider     (HINSTANCE                  hInst);
    ~CXPProvider    ();

///////////////////////////////////////////////////////////////////////////////
// Data members
//
private :
    ULONG               m_cRef;
    CRITICAL_SECTION    m_csObj;
    HINSTANCE           m_hInstance;
};

#endif // _XPWDS_H

// End of file for XPWDS.H
