///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      XPSRVCE.CPP
//
//  Description
//      This file implements the entry points, XPProviderInit and 
//      ServiceEntry for this provider's DLL.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.8
//
// Written for Microsoft Corporation
// Copyright (c) 1995-1996 Microsoft Corporation. All rights reserved.
//
#include "XPWDSR.H"

// Remark this line to turn verbose tracing OFF
//#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)   TraceInfoMessage(a)
#else
#define InfoTrace(a)   
#endif // DO_INFO_TRACES

DTBLEDIT     DtUploadTime = { sizeof(DTBLEDIT), fMapiUnicode, 5, PR_TMP_UPLOAD_TIME };
DTBLCHECKBOX DtGetHeaders = { sizeof(DTBLCHECKBOX), fMapiUnicode, PR_SMP_GET_HEADERS };

DTCTL XPConfigPage[] =
{
    { DTCT_PAGE,     0,           NULL, 0, NULL,     0,                  &DtPage       },
    { DTCT_EDIT,     DT_REQUIRED, NULL, 0, szFilter, IDC_UPLOAD_TIME,    &DtUploadTime },
    { DTCT_CHECKBOX, 0,           NULL, 0, NULL,     IDC_UPDATE_HEADERS, &DtGetHeaders },
    { DTCT_LABEL,    0,           NULL, 0, NULL,     IDC_STATIC1,        &DtLabel      },
    { DTCT_GROUPBOX, 0,           NULL, 0, NULL,     IDC_STATIC2,        &DtGroupBox   },
    { DTCT_GROUPBOX, 0,           NULL, 0, NULL,     IDC_STATIC3,        &DtGroupBox   }
};

char szDefaultTime[] = "23:59";

// Global pointer to the MAPI memory allocation functions
LPALLOCATEBUFFER    gpfnAllocateBuffer;  // MAPIAllocateBuffer function
LPALLOCATEMORE      gpfnAllocateMore;    // MAPIAllocateMore function    
LPFREEBUFFER        gpfnFreeBuffer;      // MAPIFreeBuffer function      

extern "C"
{
    MAPIUID guidABEntries = AB_UID_PROVIDER;
    MAPIUID guidXPABEntries = XP_WINDS_AB_ENTRIES;
    MAPIUID * puidWINDSEntries;
    TCHAR gszProviderName[] = "WINDS Transport Service";
    LPTSTR gszWINDSAddressType = WINDS_ADDRESS_TYPE;
    LPTSTR * gpszXPAddressTypes;
};

///////////////////////////////////////////////////////////////////////////////
//    ServiceEntry()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by the profile setup API to display the provider
//      configuration properties for this transport provider
//
//    Return Value
//      An HRESULT
//
HRESULT STDAPICALLTYPE ServiceEntry (HINSTANCE          hInstance,
                                     LPMALLOC           pMallocObj,
                                     LPMAPISUP          pSupObj,
                                     ULONG              ulUIParam,
                                     ULONG              ulFlags,
                                     ULONG              ulContext,
                                     ULONG              ulCfgPropCount,
                                     LPSPropValue       pCfgProps,
                                     LPPROVIDERADMIN    pAdminProvObj,
                                     LPMAPIERROR *      ppMAPIError)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    //InitTraces (0);                                       // Send output only to the COM1 port
    InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);     // Send output to a console window BUT NOT to the COM1
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);        // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT

    InfoTrace ("ServiceEntry function called");
    HRESULT hResult = S_OK;
    LPPROFSECT pProfileObj = NULL;
    ULONG ulPropCount;
    LPSPropValue pProps = NULL;
    HANDLE hUIMutex = NULL;

    // Temporary storage for previous server and account name
    char szOldAccount[64] = { 0 }, szOldServer[64] = { 0 };

    // What context where were we called in?
    if (MSG_SERVICE_INSTALL         == ulContext ||
        MSG_SERVICE_UNINSTALL       == ulContext ||
        MSG_SERVICE_PROVIDER_CREATE == ulContext ||
        MSG_SERVICE_PROVIDER_DELETE == ulContext)
    {
        // We don't handle any of these contexts
        TraceString1 ("ServiceEntry: Immediate return. Context: %d", ulContext);
        goto ErrorExit;
    }
    TraceString1 ("ServiceEntry: Context for call: %d", ulContext);
    ghInstance = hInstance;
    
    // Get the MAPI memory allocation routines we'll be needing
    hResult = pSupObj->GetMemAllocRoutines (&gpfnAllocateBuffer, &gpfnAllocateMore, &gpfnFreeBuffer);
    if (hResult)
    {
        TraceResult ("ServiceEntry: Failed to get the memory allocation functions", hResult);
        goto ErrorExit;
    }

    // Open the Profile Section for this service
    hResult = pAdminProvObj->OpenProfileSection (NULL,
                                                 NULL,
                                                 MAPI_MODIFY,
                                                 &pProfileObj);
    if (hResult)
    {
        goto ErrorExit;
    }
    if (MSG_SERVICE_CREATE == ulContext)
    {
        // If ServiceEntry is called because it the service is being created, we create a temporary
        // file where we save the headers text file when the user updates them from
        // the remote host. The headers should be persistent from session to
        // session (using the same profile, of course) but unique per session.
        char szTmpDir[_MAX_PATH], szFile[_MAX_PATH];
        GetTempPath (_MAX_PATH, szTmpDir);
        lstrcat (szTmpDir, WINDS_DATA_DIRECTORY);
        CreateDirectory (szTmpDir, NULL);
        GetTempFileName (szTmpDir,  // Create the file in the system temp directory
                         "XRH",     // Our transport fixed prefix for temp files
                         0,         // Use the time to create a pseudo-random number
                         szFile);   // Destination buffer
        CharUpper (szFile);
        DeleteFile (szFile);
        SPropValue spvProps[4] = { 0 };
        spvProps[0].ulPropTag = PR_SMP_HEADERS_FILENAME;
        spvProps[0].Value.lpszA = szFile;
        SYSTEMTIME st;
        GetLocalTime (&st);
        st.wHour = 23; st.wMinute = 59; st.wSecond = 0;
        spvProps[1].ulPropTag = PR_SMP_UPLOAD_TIME;
        SystemTimeToFileTime (&st, &spvProps[1].Value.ft);
        spvProps[2].ulPropTag = PR_SMP_CONNECTION_TYPE;
        spvProps[2].Value.l = OFFLINE_CONNECT;
        spvProps[3].ulPropTag = PR_SMP_GET_HEADERS;
        spvProps[3].Value.b = TRUE;
        hResult = pProfileObj->SetProps (sizeof(spvProps)/sizeof(SPropValue), spvProps, NULL);
        TraceResult ("ServiceEntry: Failed to save the headers file name back into the profile", hResult);
    }
    // The profile section is open, get the properties out of it
    hResult = pProfileObj->GetProps ((LPSPropTagArray)&sptLogonProps,
                                     fMapiUnicode,
                                     &ulPropCount,
                                     &pProps);
    if (FAILED(hResult))
    {
        TraceResult ("ServiceEntry: Failed to get profile section properties", hResult);
        goto ErrorExit;
    }
    ASSERT (NUM_LOGON_PROPERTIES == ulPropCount);

    if (MSG_SERVICE_DELETE == ulContext)
    {
        // If the service is being removed from a profile, we must cleanup
        // any temporary file (i.e. the headers text file)
        if (PR_SMP_HEADERS_FILENAME == pProps[HEADER_FILE].ulPropTag)
        {
            DeleteFile (pProps[HEADER_FILE].Value.lpszA);
        }
        char szTmpDir[_MAX_PATH], szDownloadDir[_MAX_PATH];
        GetTempPath (_MAX_PATH, szTmpDir);
        lstrcat (szTmpDir, WINDS_DATA_DIRECTORY);
        if (PR_SMP_MAILBOX_NAME == pProps[MAILBOX_NAME].ulPropTag)
        {
            wsprintf (szDownloadDir,
                      WINDS_DOWNLOAD_DIR_NAME_FORMAT,
                      szTmpDir,
                      &(pProps[SERVER_NAME].Value.lpszA[2]), // Bypass the "\\" prefix
                      pProps[MAILBOX_NAME].Value.lpszA);
            RemoveDirectory (szDownloadDir);
        }
        RemoveDirectory (szTmpDir);
        // We are done
        hResult = S_OK;
        goto ErrorExit;
    }

    // Save the current server and account names
    if (PR_SMP_MAILBOX_NAME == pProps[MAILBOX_NAME].ulPropTag &&
        PR_SMP_REMOTE_SERVER == pProps[SERVER_NAME].ulPropTag)
    {
        lstrcpy (szOldAccount, pProps[MAILBOX_NAME].Value.lpszA);
        lstrcpy (szOldServer, pProps[SERVER_NAME].Value.lpszA);
    }

    // After the setup wizard has been called the ServiceEntry functions gets called
    // with a set of properties obtained from the user for configuring this provider.
    // Merge or replace those properties onto the one we obtained from the profile section.
    if (pCfgProps && ulCfgPropCount)
    {
        hResult = MergeConfigProps (ulCfgPropCount, pCfgProps, pProps);
        if (hResult)
        {
            goto ErrorExit;
        }
    }
    // If we can, display the UI...
    if (SERVICE_UI_ALWAYS & ulFlags || SERVICE_UI_ALLOWED & ulFlags)
    {
        CFGDLG CfgDialog = { 0 };
        hUIMutex = CreateMutex (NULL, FALSE, CONFIG_UI_MUTEX);
        if (NULL == hUIMutex)
        {
            TraceResult ("ServiceEntry: Failed to create UI mutext", GetLastError());
        }
        PrivInitialize3DCtl (hInstance);
        if (WAIT_TIMEOUT == WaitForSingleObject (hUIMutex, 250)) // Wait for the MUTEX for 1/4 second
        {
            PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE, (HWND)ulUIParam);
            hResult = MAPI_E_BUSY;
            goto ErrorExit;
        }

        // Fill in the logon UI structure
        CfgDialog.hInst    = hInstance;
        CfgDialog.hWnd     = (HWND)ulUIParam;
        CfgDialog.ppProps  = &pProps;
        CfgDialog.pSupObj  = pSupObj;
        CfgDialog.ulFlags  = MSG_SERVICE_UI_READ_ONLY & ulFlags ? UI_READONLY : 0;
        CfgDialog.hUIMutex = NULL; // So that the DoLogonDlg() won't have to wait.
ShowPropsAgain:
        hResult = DoLogonDlg (&CfgDialog);
        if (hResult)
        {
            goto ErrorExit;
        }
        if (!PingRemoteServer ((HWND)ulUIParam, pProps))
        {
            goto ShowPropsAgain;
        }
    }
    if (PR_SMP_MAILBOX_NAME == pProps[MAILBOX_NAME].ulPropTag)
    {
        char szTmpDir[512], szDownloadDir[512];
        GetTempPath (512, szTmpDir);
        lstrcat (szTmpDir, WINDS_DATA_DIRECTORY);

        // If the account name change, then we must delete the download directory for the old account
        // and delete the remote headers cache file.
        if (lstrlen (szOldAccount) && lstrcmpi (szOldAccount, pProps[MAILBOX_NAME].Value.lpszA))
        {
            wsprintf (szDownloadDir,
                      WINDS_DOWNLOAD_DIR_NAME_FORMAT,
                      szTmpDir,
                      &szOldServer[2], // Bypass the "\\" prefix
                      szOldAccount);
            RemoveDirectory (szDownloadDir);
            if (PR_SMP_HEADERS_FILENAME == pProps[HEADER_FILE].ulPropTag)
            {
                DeleteFile (pProps[HEADER_FILE].Value.lpszA);
            }
        }
        wsprintf (szDownloadDir,
                  WINDS_DOWNLOAD_DIR_NAME_FORMAT,
                  szTmpDir,
                  &(pProps[SERVER_NAME].Value.lpszA[2]), // Bypass the "\\" prefix
                  pProps[MAILBOX_NAME].Value.lpszA);
        CreateDirectory (szDownloadDir, NULL);
    }

    hResult = pProfileObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);
    TraceResult ("ServiceEntry: Failed to save the properties back into the profile", hResult);

ErrorExit:
    // Clean up
    if (pProfileObj)
    {
        pProfileObj->Release();
    }
    if (gpfnFreeBuffer)
    {
        gpfnFreeBuffer (pProps);
    }
    if (hUIMutex)
    {
        ReleaseMutex (hUIMutex);
        CloseHandle (hUIMutex);
    }
    // In case the DLL was loaded, unload it.
    // This DLL gets loaded and initialized in the call to DoLogonDlg()
    PrivUninitialize3DCtl (hInstance);
    
    // If we have remote connections going on, close them now
    TerminateRemoteConnections();

    // Close down the traces
    UnInitTraces();
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    DoLogonDlg()
//
//    Parameters
//      pCfgDialog   Pointer to an CFGDLG structure.
//
//    Purpose
//      This functions gets called to display the configuration properties for
//      this transport. When invoked, it will parse the transport logon 
//      property array and initialize the fields in the property sheet with it.
//      The function could also get invoked to display the settings in 
//      READ-ONLY mode in which case we disable writing to the editable fields.
//
//    Return Value
//      An HRESULT
//
HRESULT WINAPI DoLogonDlg (PCFGDLG pCfgDialog)
{
    DWORD dwWait = WaitForSingleObject (pCfgDialog->hUIMutex, 1000); // Wait for the MUTEX for 1 second
    PrivInitialize3DCtl (pCfgDialog->hInst);
    if (WAIT_TIMEOUT == dwWait)
    {
        PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE, pCfgDialog->hWnd);
        return MAPI_E_BUSY;
    }

    LPMAPITABLE pTableObj= NULL;
    ULONG ulPropCount;
    SizedSPropTagArray(NUM_LOGON_PROPERTIES, sptCfgProps);
    CopyMemory (&sptCfgProps, &sptLogonProps, sizeof(sptLogonProps));
    sptCfgProps.aulPropTag[UPLOAD_TIME] = PR_TMP_UPLOAD_TIME;
    
    // Take ownership of the property value array, so calling code
    // will not leak memory
    LPSPropValue pProps = *pCfgDialog->ppProps;
    *pCfgDialog->ppProps = NULL;
    
    // Retrieve the dialog title string
    char szDialogTitle[64], szTimeString[32];
    LoadString (pCfgDialog->hInst, IDS_SERVICE_PROPERTY_SHEETS_TITLE, szDialogTitle, 64);
    CUIMAPIProp * pPropObj = NULL;
    HRESULT hResult = S_OK;
    try
    {
        pPropObj = new CUIMAPIProp (pCfgDialog->hInst,
                                    gpfnAllocateBuffer,
                                    gpfnAllocateMore,
                                    gpfnFreeBuffer,
                                    pCfgDialog->ulFlags & UI_READONLY);
        if (!pPropObj)
        {
            hResult = E_OUTOFMEMORY;
        }
    }
    catch (CException & Exception)
    {
        hResult = Exception.GetError();
    }
    if (hResult)
    {
        goto ErrorExit;
    }

    ASSERT (PR_SMP_UPLOAD_TIME == pProps[UPLOAD_TIME].ulPropTag);
    GetTimeString (szTimeString, pProps[UPLOAD_TIME].Value.ft);
    pProps[UPLOAD_TIME].ulPropTag = PR_TMP_UPLOAD_TIME;
    pProps[UPLOAD_TIME].Value.lpszA = szTimeString;

    hResult = pPropObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);
    if (hResult)
    {
        TraceResult ("DoLogonDlg: Failed to set the properties", hResult);
        goto ErrorExit;
    }
    DTPAGE DtPropPages[2];
    DtPropPages[0].cctl = NUM_CFG_USERINFO_PAGE_CTLS;   // Count of controls in the ConfigPage array
    DtPropPages[0].lpszComponent = szBlank;
    DtPropPages[0].lpszResourceName = MAKEINTRESOURCE (IDD_CFG_USERINFO);
    DtPropPages[0].lpctl = UserConfigPage;

    DtPropPages[1].cctl = sizeof(XPConfigPage)/sizeof(DTCTL);   // Count of controls in the ConfigPage array
    DtPropPages[1].lpszComponent = szBlank;
    DtPropPages[1].lpszResourceName = MAKEINTRESOURCE (IDD_CFG_XPCONNECT);
    DtPropPages[1].lpctl = XPConfigPage;

    // If the UI_READONLY flag is set, then we are being called from the 
    // SettingDialog method in Read Only mode. Disable all of the controls
    // on the property sheets, else Enable them
    if (pCfgDialog->ulFlags & UI_READONLY)
    {
        // Disable the controls (make them read-only)
        DtPropPages[0].lpctl[1].ulCtlFlags &= ~(DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[0].lpctl[7].ulCtlFlags &= ~(DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[1].lpctl[1].ulCtlFlags &= ~DT_EDITABLE;
        DtPropPages[1].lpctl[2].ulCtlFlags &= ~DT_EDITABLE;
    }
    else
    {
        // Enable the controls (make them read/write)
        DtPropPages[0].lpctl[1].ulCtlFlags |= (DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[0].lpctl[7].ulCtlFlags |= (DT_EDITABLE | DT_SET_IMMEDIATE);
        DtPropPages[1].lpctl[1].ulCtlFlags |= DT_EDITABLE;
        DtPropPages[1].lpctl[2].ulCtlFlags |= DT_EDITABLE;
    }

    // Create the display table for the logon dialog. It is based on dialog
    // resources plus the static information at the head of this module.
    LPTABLEDATA pTableData;
    hResult = BuildDisplayTable (gpfnAllocateBuffer,
                                 gpfnAllocateMore,
                                 gpfnFreeBuffer,
                                 NULL,
                                 pCfgDialog->hInst,
                                 sizeof(DtPropPages)/sizeof(DTPAGE),
                                 DtPropPages,
                                 fMapiUnicode,
                                 &pTableObj,
                                 &pTableData);
    if (!hResult)
    {
        pPropObj->SetTableData (pTableData);
        pTableData->Release();

        // Display the dialog/property sheets
DisplayPropsAgain:
        hResult = pCfgDialog->pSupObj->DoConfigPropsheet ((ULONG)pCfgDialog->hWnd,
                                                          0,
                                                          szDialogTitle,
                                                          pTableObj,
                                                          pPropObj,
                                                          0);     // Default index for the top property sheet
        if (!hResult)
        {
            // Retrieve the altered data
            gpfnFreeBuffer (pProps);
            pProps = NULL;
            hResult = pPropObj->GetProps ((LPSPropTagArray)&sptCfgProps, fMapiUnicode, &ulPropCount, &pProps);
            if (hResult)
            {
                if (MAPI_W_ERRORS_RETURNED == hResult)
                {
                    if (PR_TMP_UPLOAD_TIME != pProps[UPLOAD_TIME].ulPropTag)
                    {
                        pProps[UPLOAD_TIME].ulPropTag = PR_TMP_UPLOAD_TIME;
                        pProps[UPLOAD_TIME].Value.lpszA = szDefaultTime;
                    }
                    hResult = S_OK;
                }
                else
                {
                    TraceResult ("DoLogonDlg: GetProps failed", hResult);
                    goto ErrorExit;
                }
            }
            // Check the Server name
            if (!IsValidServerName (pProps[SERVER_NAME].Value.lpszA))
            {
                gpfnFreeBuffer (pProps);
                pProps = NULL;
                PrivateMessageBox (IDS_MSG_INVALID_SERVER_NAME, pCfgDialog->hWnd);
                goto DisplayPropsAgain;
            }

            ASSERT (PR_TMP_UPLOAD_TIME == pProps[UPLOAD_TIME].ulPropTag);
            SYSTEMTIME st;
            GetLocalTime (&st);
            if (ParseTime (pProps[UPLOAD_TIME].Value.lpszA, &st.wHour, &st.wMinute))
            {
                pProps[UPLOAD_TIME].ulPropTag = PR_SMP_UPLOAD_TIME;
                SystemTimeToFileTime (&st, &pProps[UPLOAD_TIME].Value.ft);
            }
            else
            {
                gpfnFreeBuffer (pProps);
                pProps = NULL;
                PrivateMessageBox (IDS_MSG_BAD_TIME_FORMAT, pCfgDialog->hWnd);
                goto DisplayPropsAgain;
            }

            // Return the new property array
            *pCfgDialog->ppProps = pProps;
            pProps = NULL;      // do not free this in cleanup code below
        }
    }

ErrorExit:
    // Release objects
    if (pTableObj)
    {
        pTableObj->Release();
    }
    if (pPropObj)
    {
        pPropObj->Release();
    }    
    gpfnFreeBuffer (pProps);
    PrivUninitialize3DCtl (pCfgDialog->hInst);
    ReleaseMutex (pCfgDialog->hUIMutex);
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    XPProviderInit()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Entry point called by the MAPI spooler when a profile uses this 
//      transport. The spooler calls this method and expects a pointer to an
//      implementation of the IXPProvider interface. MAPI uses the returned
//      IXPProvider interface pointer to logon on the transport provider
//
//    Return Value
//      An HRESULT
//
STDINITMETHODIMP XPProviderInit (HINSTANCE          hInstance, 
                                 LPMALLOC           pMallocObj,
                                 LPALLOCATEBUFFER   pfnAllocateBuffer,
                                 LPALLOCATEMORE     pfnAllocateMore,
                                 LPFREEBUFFER       pfnFreeBuffer,
                                 ULONG              ulFlags,
                                 ULONG              ulMAPIVer,
                                 ULONG *            pulProviderVer,
                                 LPXPPROVIDER *     ppXPProviderObj)
{
    // Look at TRACES.H and TRACES.CPP for more options to the InitTraces() function
    //InitTraces (0);                                       // Send output only to the COM1 port
    InitTraces (TRACES_CONSOLE | TRACES_NO_COM_OUTPUT);     // Send output to a console window BUT NOT to the COM1
    //InitTraces (TRACES_CONSOLE | TRACES_LOG_FILE);        // Send output to COM1 port AND a console window AND to a log file in C:\MAPILOG.TXT
    InfoTrace ("XPProviderInit function called");
    *pulProviderVer = CURRENT_SPI_VERSION;
    if (ulMAPIVer < CURRENT_SPI_VERSION)
    {
        TraceMessage ("XPProviderInit: The version of the subsystem cannot handle this version of the provider");
        return MAPI_E_VERSION;
    }
    // Save the pointer to the allocation routines in global variables
    gpfnAllocateBuffer = pfnAllocateBuffer;
    gpfnAllocateMore   = pfnAllocateMore;
    gpfnFreeBuffer     = pfnFreeBuffer;
    ghInstance         = hInstance;
    // Allocate a new XPProvider object, the constructor will initialize the data member
    CXPProvider * pXPProvider = new CXPProvider (hInstance);
    if (!pXPProvider)
    {
        TraceMessage ("XPProviderInit: Failed to allocate a new CXPProvider object");
        return E_OUTOFMEMORY;
    }
    // Copy pointer to the allocated object back into the return IXPProvider object pointer
    *ppXPProviderObj = (LPXPPROVIDER)pXPProvider;        
    return S_OK;
}

// End of file for XPSRVCE.CPP
